<?php

/**
 * mysqli database class
 * Database Object for communicating with a MySQL Database Version > 4.0
 * For more information contact  support@alenet.com
 *
 * @author       $Author: alenetdev $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2019-01-04 12:11:05 -0400 (vi., 04 ene. 2019) $
 * @version    	$Rev: 10438 $
 * @package		Clientix.Classes.Database
 * @abstract		MYSQLI - Database Abstraction Layer
 */
 
class MysqliDatabase extends ClxDatabase {

    /**
     * Constructor
     */
    function __construct($url, $user, $pass, $name) {
        $this->databaseURL = $url;
        $this->databaseUsername = $user;
        $this->databasePassword = $pass;
        $this->databaseName = $name;
        $this->debug = false;
        $this->debugCF=false;
    }

    /**
     * Database Error Number
     *
     * @param
     *        	: SQL Query
     */
    function errorNum() {
        return $this->dblink->errno;
    }

    /**
     * Database Error Message
     *
     * @param
     *        	: SQL Query
     */
    function errorMsg() {
        return $this->dblink->error;
    }

    /**
     * Returns string with information about the most recent query.
     *
     * @param
     *        	: SQL Query
     */
    function info() {
        return mysqli_info($this->dblink);
    }

    /**
     * Open a connection to a Database Server
     */
    function connectToDatabase() {
        $this->dblink = mysqli_connect('p:' . $this->databaseURL, $this->databaseUsername, $this->databasePassword) or $this->reportError('mysqli_connect...', CLX_NUM_ZERO, '<center><h2>Clientix</h2><br><p>ERR1 - Connection to database fail. Please contact System Administrator. Thank You.</p></center>');
    }

    /**
     * Selects the default database for queries
     */
    function selectDatabase() {
        $this->dblink->select_db($this->databaseName) or $this->reportError('mysqli_connect...', CLX_NUM_ZERO, '<center><h2>Clientix</h2><br><p>ERR2 - Connection to database fail. Please contact System Administrator. Thank You.</p></center>');
        // setting character set
        $this->dblink->set_charset($this->charset);
    }

    /**
     * Close database connection
     */
    function closeConnection() {
        $this->dblink->close();
    }

    function freeResult($resource) {
        $resource->free();
    }

// end method
    /**
     * Executes SQL Query and returns the correspondant result resource
     * or error on failure.
     *
     * @param
     *        	: SQL Query
     */
    function executeQuery($sql) { 
	
		/*if($_SERVER["REMOTE_ADDR"]=='186.94.197.73'){
			$archivo = "C:\inetpub\wwwroot\CLX70IVAN/temp/queries.txt";
		$old_content = file_get_contents($archivo);
		$old_content.= $sql.";\n";
		$te=debug_backtrace();
		for ($t=0;$t<18;$t++) {
			if (isset($te[$t]["file"]))
				$old_content.= substr($te[$t]["file"].str_repeat(" ",80),0,80).chr(9).$te[$t]["line"].chr(9).$te[$t]["function"].chr(13).chr(10);
			
		}
		$old_content.="\n";
		$old_content.="\n";
		$old_content.="\n";
			file_put_contents($archivo, $old_content);
		}*/
		
		
		
		IF (ISSET($GLOBALS['ADDRDEBUGMYSQL']) && $GLOBALS['ADDRDEBUGMYSQL']==$_SERVER['REMOTE_ADDR'])
			$this->debug=true;	
	
		if ( !empty($_SESSION["_dEBUG"]) and (((strlen($sql)>65 or (strlen($sql)<=65 and stripos($sql, "IDCODE")<1)) and stripos($sql, "CLX_DICTIONARY")<1 and stripos($sql, "dictionary")<1 and stripos($sql, "CLX_DICTIONARY_HEAP")<1     /*and stripos($sql, "SER_SESSIONS")<1  */ /* and stripos($sql, "events")<1 */ and stripos($sql, "CLX_AC_GROUP_MEMBERS")<1 and stripos($sql, "CLX_UI_FORMS")<1 /* and stripos($sql, "CLX_DB_TABLE")<1 */		   and stripos($sql, "CLX_OBJECTS")<1 and stripos($sql, "CLX_DB_DYNRELATIONS")<1 and stripos($sql, "CLX_AC_USERS")<1 /*and stripos($sql, "CLX_SYS_PARAMETERS")<1 */ and stripos($sql, "CLX_CODELIB")<1 and stripos($sql, "ALRT_ALERT_SCHEDU")<1 and stripos($sql, "CLX_TOOLS")<1 and stripos($sql, "CLX_UI_LISTINGS")<1 and stripos($sql, "CLX_UI_FORM_PANEL")<1 and stripos($sql, "CLX_DB_LISTS")<1 and stripos($sql, "CLX_DB_FIELDS")<1 and stripos($sql, "CLX_UI_VIEW_PREFS")<1) or ( $_SESSION["_dEBUG"]==99 and stripos($sql, "dictionary")<1                 ))){ $mc1=microtime(true);list($h1,$m1,$s1)=explode(':', date('h:i:s'));$sidebug=1; if (isset($_SESSION["maxdebug"]) && filesize(kPathToWebSite."/".kPathToTemp."archivo.txt")>$_SESSION["maxdebug"]) {$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","r"); $conte=fread($tta,filesize(kPathToWebSite."/".kPathToTemp."archivo.txt")); fclose($tta);$arreglos=explode("GET-",$conte);			for($t=0;$t<count($arreglos)-6;$t++) {unset($arreglos[$t]);	}$conte=implode("GET-", (is_Array($conte))?$conte:array());	$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","w+");	fwrite($tta,substr($conte,-$_SESSION["maxdebug"]*80/100));}else {$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");}	fwrite($tta,chr(13).chr(10)."1- hora:".date("h:i:s d/m/Y").chr(13).chr(10));fwrite($tta,				"   remote:".(isset($_SERVER["REMOTE_ADDR"]) ? $_SERVER["REMOTE_ADDR"]:"").chr(13).chr(10));	fwrite($tta,chr(13).chr(10).beautiful_sql($sql,chr(13).chr(10),"    ",false).chr(13).chr(10));$te=debug_backtrace();for ($t=0;$t<18;$t++) { if (isset($te[$t]["file"]))	fwrite($tta,substr($te[$t]["file"].str_repeat(" ",80),0,80).chr(9).$te[$t]["line"].chr(9).$te[$t]["function"].chr(13).chr(10));	}fwrite($tta,chr(13).chr(10));
		fclose($tta);}
			
		
		if (!empty($this->debug)) {
			$te=debug_backtrace();$mc1=microtime(true);	
		  echoLog($sql . ';<br>');
			echoLog('<nobr>1-'.str_repeat(' &nbsp;',50).  substr((isset($te[1]["file"])?$te[1]["file"]:'').str_repeat(" ",80),0,80).chr(9).(isset($te[1]["line"])?$te[1]["line"]:'').chr(9).$te[1]["function"] . ';</nobr><br>');
			echoLog('<nobr>2-'.str_repeat(' &nbsp;',50).  substr((isset($te[2]["file"])?$te[2]["file"]:'').str_repeat(" ",80),0,80).chr(9).(isset($te[2]["line"])?$te[2]["line"]:'').chr(9).$te[2]["function"] . ';<nobr><br>');
			echoLog('<nobr>3-'.str_repeat(' &nbsp;',50).  substr((isset($te[3]["file"])?$te[3]["file"]:'').str_repeat(" ",80),0,80).chr(9).(isset($te[3]["line"])?$te[3]["line"]:'').chr(9).$te[3]["function"] . ';<nobr><br>');
        }//pre($this->dblink);
        $this->logStart();
		$this->dumpScript($sql);
        $result = $this->dblink->query($sql);
		if (empty($this->dblink->errno)) {
			$this->affected_rows=$this->dblink->affected_rows;
		}
		else {
			$this->affected_rows=0;
		}	

        $this->logEnd();
        $this->writeTimeLog($sql); // Writing log
	    if (empty($result)) {
		  $this->reportError($sql, $this->errorNum(), $this->errorMsg());
        }
		if ( !empty($_SESSION["_dEBUG"]) and $_SESSION["_dEBUG"]==99 and isset($sidebug) ) { $tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");	fwrite($tta,"1- fin-hora:".date("h:i:s d/m/Y").chr(13).chr(10));$mc2=microtime(true);list($h2,$m2,$s2)=explode(':', date('h:i:s'));	fwrite($tta,"total segundos:". (($mc2-$mc1)).chr(13).chr(10));fclose($tta);}
        if (!empty($this->debug)) {$mc2=microtime(true);
			$mcresta=$mc2-$mc1;
			$mxt1=''; $mxt2='';
			if ($mcresta>1) {$mxt1='<strong style="color:red;background-color: yellow">'; $mxt2='</strong>'; }
			echoLog(str_repeat(' &nbsp;',50). "total segundos:".$mxt1. (SPRINTF("%7.4f",($mcresta)))  .$mxt2.'<br><br>');

		}
		
        return $result;
    }

    function executeQueryVoid($sql) {
        ClxFreeReturn($this->executeQuery($sql));
    }

    /**
     * Equivalent to executeQuery with the exception of no error reported on
     * failure
     *
     * @param
     *        	: SQL Query
     */
    function executeQuerySilent($sql) {
	/*	if($_SERVER["REMOTE_ADDR"]=='186.94.197.73'){
			$archivo = "C:\inetpub\wwwroot\CLX70IVAN/temp/queries.txt";
		$old_content = file_get_contents($archivo);
		$old_content.= $sql.";\n";
		$te=debug_backtrace();
		for ($t=0;$t<18;$t++) {
			if (isset($te[$t]["file"]))
				$old_content.= substr($te[$t]["file"].str_repeat(" ",80),0,80).chr(9).$te[$t]["line"].chr(9).$te[$t]["function"].chr(13).chr(10);
			
		}
		$old_content.="\n";
		$old_content.="\n";
		$old_content.="\n";
			file_put_contents($archivo, $old_content);
		}*/
				
		
		
		if ( !empty($_SESSION["_dEBUG"]) and (((strlen($sql)>65 or (strlen($sql)<=65 and stripos($sql, "IDCODE")<1)) and stripos($sql, "CLX_DICTIONARY")<1 and stripos($sql, "dictionary")<1  and stripos($sql, "CLX_DICTIONARY_HEAP")<1 /*and stripos($sql, "SER_SESSIONS")<1  */ /* and stripos($sql, "events")<1 */ and stripos($sql, "CLX_AC_GROUP_MEMBERS")<1 and stripos($sql, "CLX_UI_FORMS")<1 /* and stripos($sql, "CLX_DB_TABLE")<1 */		   and stripos($sql, "CLX_OBJECTS")<1 and stripos($sql, "CLX_DB_DYNRELATIONS")<1 and stripos($sql, "CLX_AC_USERS")<1 /*and stripos($sql, "CLX_SYS_PARAMETERS")<1 */ and stripos($sql, "CLX_CODELIB")<1 and stripos($sql, "ALRT_ALERT_SCHEDU")<1 and stripos($sql, "CLX_TOOLS")<1 and stripos($sql, "CLX_UI_LISTINGS")<1 and stripos($sql, "CLX_UI_FORM_PANEL")<1 and stripos($sql, "CLX_DB_LISTS")<1 and stripos($sql, "CLX_DB_FIELDS")<1 and stripos($sql, "CLX_UI_VIEW_PREFS")<1) or $_SESSION["_dEBUG"]==99)) {list($h1,$m1,$s1)=explode(':', date('h:i:s')); $sidebug=1; if (isset($_SESSION["maxdebug"]) && filesize(kPathToWebSite."/".kPathToTemp."archivo.txt")>$_SESSION["maxdebug"]) {$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","r"); $conte=fread($tta,filesize(kPathToWebSite."/".kPathToTemp."archivo.txt")); fclose($tta);$arreglos=explode("GET-",$conte);			for($t=0;$t<count($arreglos)-6;$t++) {unset($arreglos[$t]);	}/*$conte=implode("GET-",$conte);*/	$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","w+"); if(empty($_SESSION["maxdebug"])) $_SESSION["maxdebug"]=50000;	fwrite($tta,substr($conte,-$_SESSION["maxdebug"]*80/100));}else {$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");}	fwrite($tta,chr(13).chr(10)."1- hora:".date("h:i:s d/m/Y").chr(13).chr(10));fwrite($tta,				"   remote:".(isset($_SERVER["REMOTE_ADDR"]) ? $_SERVER["REMOTE_ADDR"]:"").chr(13).chr(10));	fwrite($tta,chr(13).chr(10).beautiful_sql($sql,chr(13).chr(10),"    ",false).chr(13).chr(10));$te=debug_backtrace();for ($t=0;$t<18;$t++) { if (isset($te[$t]["file"]))	fwrite($tta,substr($te[$t]["file"].str_repeat(" ",80),0,80).chr(9).$te[$t]["line"].chr(9).$te[$t]["function"].chr(13).chr(10));	}fwrite($tta,chr(13).chr(10));
		fclose($tta);}
        if (!empty($this->debug)) {
			$te=debug_backtrace();$mc1=microtime(true);	
		  echoLog($sql . ';<br>');
			for ($tx=1;$tx<5;$tx++) 
				if (isset($te[$tx]))
					echoLog('<nobr>'.str_repeat(' &nbsp;',50).  substr((isset($te[$tx]["file"])?$te[1]["file"]:'').str_repeat(" ",80),0,80).chr(9).(isset($te[$tx]["line"])?$te[$tx]["line"]:'').chr(9).$te[$tx]["function"] . ';</nobr><br>');
			
		}//pre($this->dblink);
        $this->logStart();
        $this->dumpScript($sql);
        $result = $this->dblink->query($sql);
		if (empty($this->dblink->errno)) {
			$this->affected_rows=$this->dblink->affected_rows;
		}
		else {
			$this->affected_rows=0;
		}	
        $this->logEnd();
        $this->writeTimeLog($sql); // Writing log
		if ( !empty($_SESSION["_dEBUG"]) and $_SESSION["_dEBUG"]==99 and isset($sidebug)) { $tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");	fwrite($tta,"1- fin-hora:".date("h:i:s d/m/Y").chr(13).chr(10));list($h2,$m2,$s2)=explode(':', date('h:i:s'));	fwrite($tta,"total segundos:". (($h2-$h1)*3600+($m2-$m1)*60+($s2-$s1)).chr(13).chr(10));fclose($tta);}
        if (!empty($this->debug)) {
			$mc2=microtime(true);
			$mcresta=$mc2-$mc1;
			$mxt1=''; $mxt2='';
			if ($mcresta>1) {$mxt1='<strong style="color:red;background-color: yellow">'; $mxt2='</strong>'; }
			echoLog(str_repeat(' &nbsp;',50). "total segundos:".$mxt1. (SPRINTF("%7.4f",($mcresta)))  .$mxt2.'<br><br>');
		}

        return $result;
    }

    function isSelect($statement) {
        return isset($statement->type);
    }

    /**
     * Return true if table exists or false if not found on database
     *
     * @param String tablename
     */
    function tableExists($tablename) {if (ClxCommon::$db->debugCF==true) {echo"==", __FILE__ ."(". __LINE__ .")==>";ECHO $tablename;}
		$fetched=false;
		if(kApcTtl_23>0) $xreturn = apc_fetchFct($findVar=kSpaceName . '_tablename_exist_'.$tablename, $fetched);
		if (empty($fetched)) {
        $records = $this->executeQuery('SHOW TABLES LIKE ' . singleQuote($tablename));
			$xreturn=$this->numRows($records) > CLX_NUM_ZERO;
        	if (kApcTtl_23>0 ) {
				apc_addFct($findVar,$xreturn,kApcTtl_23);
			}
		}
		if (ClxCommon::$db->debugCF==true) {echo"==", __FILE__ ."(". __LINE__ .")==>";ECHO $xreturn;}		
	    return $xreturn; 
    }

	function tableExists_bycode($tablecode) {
		$sql = 'SELECT COUNT(T.TABLE_NAME) FROM CLX_DB_TABLES X 
				JOIN information_schema.tables T 
				ON CONCAT(X.MODULECODE,'.singlequote('_').', X.IDNAME)= T.TABLE_NAME 
				WHERE X.IDCODE='.singlequote(UPPERCASE($tablecode)).
				' AND T.table_type = "base table" AND  T.table_schema=DATABASE()';
		$qty = $this->getOne($sql);
		return ($qty > CLX_NUM_ZERO) ? true : false;
	}
	

    /**
     * Creates physical table on database.
     *
     * @param string $tablename   	Name of table
     * @param string $tabletype
     *        	Type of table VIEW, TEMP, ETC
     * @param array $fields
     *        	List of field and its caracteristicts
     * @param bool $casesensitive
     *        	Additiona parameter to define case sensitivity. only for mysql
     */
    function createTable($tablename, $tabletype, $fields, $casesensitive = false) {
        if ($tabletype === 'VIEW') {
		  return;
        }
        $indexes = array();
        $descriptions = array();
        reset($fields);
        foreach ($fields as $name => $info) {
		  if (!array_key_exists('null', $info)) {
			 $info ['null'] = 'YES';
		  }
		  $type = $info ['type'];
		  $null = ($info ['null'] === 'NO') ? 'NOT null' : ' null';
		  $properties = '';
		  // Handling Defaults
		  if (array_key_exists('default', $info) && strlen('' . $info ['default']) > CLX_NUM_ZERO) {
			 if ($info ['null'] === 'NO' && $info ['default'] === 'null') {
			     // validar en caso de que sea not null que colocar como null
			     $properties .= ' ';
			 } else {
                    if (UPPERCASE(TRIM($info ['default']))=='TODAY' or UPPERCASE(TRIM($info ['default']))=='HOY') {
                        switch (kDatabaseType) {
                            case 'oci': // Oracle
                                $info['default']='CURRENT_DATE';
                                break;
                            default: // MYSQL   NO TIENE FUNCIONES PARA DATE
                                If($info['type']=='DATE') {
                                    $info['default']='';
                                }
                                break;
                        }

                    }
                    if ($info ['default']!=''){
			     $properties .= ' DEFAULT ' . $info ['default'] . ' ';
			 }
		  }
            }


		  if (isset($info ['properties'])) {
			 $properties .= $info ['properties'];
		  }

		  if (!empty($info ['collate'])) {
			 $properties .= ' COLLATE '.$info ['collate'];
		  }

		  $d = '`' . $name . '` ' . $type . ' ' . $null . ' ' . $properties;
		  $descriptions [] = $d;
		  if (!array_key_exists('index', $info))
			 $info ['index'] = '';
		  $info ['name'] = $name;
		  switch ($info ['index']) {
			 case 'PRI' :
			     $indexes [] = 'PRIMARY KEY(' . $name . ')';
			     break;
			 case 'MUL' :
			     $indexes [] = 'INDEX(' . $name . ')';
			     break;
			 case 'UNI' :
			     $indexes [] = 'UNIQUE INDEX(' . $name . ')';
			     break;
			 default:
			     break;
		  }
        }
        if ($tabletype === 'TEMP') {
		  $temporary = ' TEMPORARY ';
		  $type = '';
        } else {
		  $temporary = '';
		  $type = ((kMysqlVersion >= '5.0') ? 'ENGINE=' . singleQuote($tabletype) : 'TYPE=' . singleQuote($tabletype) );
        }
        $sql = 'CREATE ' . $temporary . ' TABLE IF NOT EXISTS ' . $tablename . ' (' . implode(',', $descriptions);
        if (count($indexes) > CLX_NUM_ZERO) {
		  $sql .= ',' . implode(',', $indexes);
        }
        $sql .= ') ' . $type;
        if (!empty($casesensitive)) {
		  $sql .= ' CHARSET=' . $this->charset . ' COLLATE=' . $this->charsetCcollateBin;
        }
        $this->executeQueryVoid($sql);
		$findVar=kSpaceName . '_tablename_exist_'.$tablename;
		apc_deleteFct($findVar);
    }

    /**
     * Return physical table information of empty array otherwise
     *
     * @param string $tablename
     *        	Name of table
     * @return array
     */
    function getTableInfo($tablename) {
        $sql = 'SHOW TABLE STATUS LIKE ' . singleQuote($tablename);
        $records = $this->executeQuery($sql);
        $info = $this->getNextRecord($records, ASSOCIATIVE);
		if ($info['Name']<>uppercase($info['Name'])) {
			$this->changeToUpperTableName($info['Name']);
			$info['Name']=uppercase($info['Name']);
		}
        if (!empty($info)) {
		  $info = array('name' => $info ['Name'], 'type' => ((array_key_exists('Engine', $info)) ? $info ['Engine'] : $info ['Type']), 'fields' => array(), 'indexes' => array(), 'case_sensitive' => (($info ['Collation'] == $this->charsetCcollateBin) ? true : false), 'raw' => $info);
        } else {
		  $info = array();
        }
        return $info;
    }
	function changeToUpperTableName($tablename) {
		$tempName=$tablename.'_'.rand(10,100000000);
		$this->executeQuery('RENAME TABLE '.$tablename. ' TO '.$tempName);	

		$this->executeQuery('RENAME TABLE '.$tempName. ' TO '.uppercase($tablename));
	}	
    /**
     * Changed table type
     *
     * @param string $tablename
     *        	Name of table
     * @param string $newtype
     *        	Type of table ex: VIEW, TEMP, ETC
     */
    function changeTableType($tablename, $newtype) {
        $sql = 'ALTER TABLE ' . $tablename . ((kMysqlVersion >= '5.0') ? 'ENGINE=' . singleQuote($newtype) : 'TYPE=' . singleQuote($newtype) );
        $this->executeQueryVoid($sql);
    }

    /**
     * Sets the table collection for case sensitivity
     *
     * @param string $tablename
     *        	Name of table to set case sensitive
     * @param boolen $active
     *        	Status of active or inactive
     */
    function changeTableCaseSensitive($tablename, $active) {
        $charsetCollate = (!empty($active)) ? $this->charsetCcollateBin : $this->charsetCollateCi;
        $sql = 'ALTER TABLE ' . $tablename . ' CONVERT TO CHARACTER SET ' . $this->charset . ' COLLATE ' . $charsetCollate;
        $this->executeQueryVoid($sql);
    }

// end method
    /**
     * Changes table name
     *
     * @param string $oldtablename
     * @param string $newtablename
     */
    function renameTable($oldtablename, $newtablename) {
        $sql = 'ALTER TABLE ' . $oldtablename . ' RENAME ' . $newtablename;
        $this->executeQueryVoid($sql);
    }

// end method
    /**
     * Drop Table from database
     */
    function dropTable($tablename) {
		$SQL='SELECT X.ID FROM CLX_DB_TABLE_HIST X, CLX_DB_TABLES Y WHERE X.TABLE_CODE_HIST=Y.IDCODE AND X.TABLE_CODE=Y.IDCODE AND CONCAT(Y.MODULECODE,'.singlequote('_').',Y.IDNAME)='.singlequote($tablename);
		$records = $this->executeQuery($SQL);
        while (($record = $this->getNextRecord($records, NUMERIC))) {
			$sql='DELETE FROM CLX_DB_TABLE_HIST WHERE ID='.$record['ID'];
		if ($this->debugCF==true) {pre('after drop table');pre($sql);}		
		$this->executequery($sql);
		}
        $sql = 'DROP TABLE IF EXISTS ' . $tablename;
		if ($this->debugCF==true) {pre ('drop sql');pre($sql);}	
        $this->executeQueryVoid($sql);
    }

    /**
     * Returns array containing tables names
     */
    function tables() {
        $result = array();
        $records = $this->executeQuery('SHOW TABLES');
        while (($record = $this->getNextRecord($records, NUMERIC))) {
		  $result [] = $record [CLX_NUM_ZERO];
        }
        return $result;
    }

    function tableLock($tablename) {
        $sql = 'LOCK TABLES ' . $tablename . ' WRITE';
        return $this->executeQuery($sql);
    }

    function tableUnlock() {
        $sql = 'UNLOCK TABLES';
        return $this->executeQuery($sql);
    }

    function createdId() {
        return $this->dblink->insert_id;
    }

    function getNextRecord(&$id, $type = ASSOCIATIVE) {
		if ($id != NULL){
        return mysqli_fetch_array($id, ($type == ASSOCIATIVE) ? MYSQLI_ASSOC : MYSQLI_NUM);
		}else {
			return NULL;
		}	
    }

    function moveTo($cursor, $position) {
        mysqli_data_seek($cursor, $position);
    }

    function numFields($cursor) {
        return $cursor->field_count;
    }

    function numRows($cursor) {
        return $cursor->num_rows;
    }

    /**
     * Return field name
     *
     * @param
     *        	: resource cursor
     */
    function fieldName($cursor, $field_num) {
        $cursor->field_seek($field_num);
        $finfo = $cursor->fetch_field();
        return $finfo->name;
    }

    // procdures and functions
    /**
     * (non-PHPdoc)
     *
     * @see ClxDatabase::createFunction()
     */
    function createFunction($name, $parameters, $body, $return) {
        $sql = 'CREATE FUNCTION ' . $name . ' (' . $parameters . ')  RETURNS ' . $return . ' DETERMINISTIC READS SQL DATA BEGIN ' . $body . ' END';
        ClxFreeReturn( $this->executeQuerySilent($sql));
        $err = $this->errorMsg();
        if (!empty($err)) {
		  return $err;
        } else {
		  return true;
        }
    }

    /**
     * (non-PHPdoc)
     *
     * @see ClxDatabase::createProcedure()
     */
    function createProcedure($name, $parameters, $body) {
        $sql = 'CREATE PROCEDURE ' . $name . '(' . $parameters . ')	BEGIN ' . $body . '	END';
        ClxFreeReturn( $this->executeQuerySilent($sql));
        $err = $this->errorMsg();
        if (!empty($err)) {
		  return $err;
        } else {
		  return true;
        }
    }

    /**
     * (non-PHPdoc)
     *
     * @see ClxDatabase::existsFunction()
     */
    function existsFunction($name) {
        $sql = 'SHOW FUNCTION STATUS WHERE NAME = ' . singleQuote($name);
        $results = $this->executeQuery($sql);
        if ($this->getNextRecord($results, ASSOCIATIVE)) {
		  return true;
        } else {
		  return false;
        }
    }

    /**
     * (non-PHPdoc)
     *
     * @see ClxDatabase::existsProcedure()
     */
    function existsProcedure($name) {
        $sql = 'SHOW PROCEDURE STATUS WHERE NAME = ' . singleQuote($name);
        $results = $this->executeQuery($sql);
        if ($this->getNextRecord($results, ASSOCIATIVE)) {
		  return true;
        } else {
		  return false;
        }
    }

    /**
     * (non-PHPdoc)
     *
     * @see ClxDatabase::dropProcedure()
     */
    function dropProcedure($name) {
        $sql = 'DROP PROCEDURE IF EXISTS ' . $name;
        $this->executeQueryVoid($sql);
        return true;
    }


    /**
     * (non-PHPdoc)
     *
     * @see ClxDatabase::dropFunction()
     */
    function dropFunction($name) {
        $sql = 'DROP FUNCTION IF EXISTS ' . $name;
        $this->executeQueryVoid($sql);
        return $ret;
    }

    /**
     * To test this has changed on clx database to add indexname
     */
    function hasIndex($tablename, $fieldname, $indexname) {
        $sql = 'SHOW INDEXES FROM  '.$tablename;
		$indexes = $this->executeQuery($sql);
		$ret=false;
		while ($index = $this->getNextRecord($indixes, ASSOCIATIVE)) {
			if ($indexname==index['Key_name']) {
				$ret=true;
				break;
			} 
    }
        return $ret;
    }
	function getTablebyIndexes($tablecode, $type='UNI') {
		$sql='SELECT CONCAT(MODULECODE,"_",IDNAME) FROM CLX_DB_TABLES WHERE IDCODE='.singlequote($tablecode);
		$tablename = $this->getONE($sql);
		$return = array();
		$sql = 'SELECT DISTINCT TABLE_NAME, INDEX_NAME FROM INFORMATION_SCHEMA.STATISTICS WHERE UPPER(TABLE_NAME)='.singlequote(uppercase($tablename)).' AND UPPER(TABLE_SCHEMA) = '.singlequote(uppercase($GLOBALS['kDatabase'][0][3]));
        $tables = $this->executeQuery($sql);
	
	    while ($table = $this->getNextRecord($tables, ASSOCIATIVE)) {
			
			$sql = 'SHOW INDEXES FROM '.$table['TABLE_NAME'] ;
			$indexes = $this->executeQuery($sql); 
			$k=0;
			while ($index = $this->getNextRecord($indexes, ASSOCIATIVE)) {
			//	pre($index);
			
				if ($index['Key_name']=='PRIMARY') {
					$index['INDEX_TYPE']='PRI';
				}
				else if($index['Non_unique']=='0') {
					$index['INDEX_TYPE']='UNI';
				}	
				else {
					$index['INDEX_TYPE']='MUL';
				}
				if ($index==$index['INDEX_TYPE']) {
					if (empty($return[$index['Key_name']]['fieldname'])) {
						$return[$index['Key_name']]['fieldname'] = $index['Column_name'];
					} else {
						if (strpos(",".$return[$index['Key_name']]['fieldname'].",",$index['Column_name'])<1)
							$return[$index['Key_name']]['fieldname'] .= ',' . $index['Column_name'];
					}
					
				}
				$return[$index['Key_name']]['index'] = $index['INDEX_TYPE'];
			}  
		}
		
		return $return;
    }
	/*function getTablebyIndexes($tablename, $type='UNI') {
        $return = array();
		$sql = 'SELECT DISTINCT TABLE_NAME, INDEX_NAME FROM INFORMATION_SCHEMA.STATISTICS WHERE UPPER(TABLE_NAME)='.singlequote(uppercase($tablename)).' AND UPPER(TABLE_SCHEMA) = '.singlequote(uppercase($GLOBALS['kDatabase'][0][3]));
	
		$tables = $this->executeQuery($sql);
	    while ($table = $this->getNextRecord($tables, ASSOCIATIVE)) {
			
			$sql = 'SHOW INDEXES FROM '.$table['TABLE_NAME'] ;
			$indexes = $this->executeQuery($sql); 
			$k=0;
		//	pre($index);
			if ($index['Key_name']=='PRIMARY') {
				$index['INDEX_TYPE']='PRI';
			}
			else if($index['Non_unique']=='0') {
				$index['INDEX_TYPE']='UNI';
			}	
			else {
				$index['INDEX_TYPE']='MUL';
			}
			if ($type==$index['INDEX_TYPE'] ) {
				if (empty($return[$index['INDEX_NAME']]['fieldname'])) {
					$return[$index['INDEX_NAME']]['fieldname'] = $index['COLUMN_NAME'];
				} else {
					$return[$index['INDEX_NAME']]['fieldname'] .= ',' . $index['COLUMN_NAME'];
				}
			}	
			if ($index['Key_name']=='PRIMARY') {
				if ($index['PRIM']==$type) {
					$return[$index['INDEX_NAME']]['index'] = 'PRI';
				}
    }

		}  
		
		return $return;
    }


    /**
     * Returns array containing current table indexes
     * Verify this in oracle
     */
    function getTableIndexes($tablename, $indexname) {
		
		$return = array();
		$sql = 'SELECT DISTINCT TABLE_NAME, INDEX_NAME FROM INFORMATION_SCHEMA.STATISTICS WHERE UPPER(TABLE_NAME)='.singlequote(uppercase($tablename)).' AND UPPER(TABLE_SCHEMA) = DATABASE()';
		////// comentado solo tenia la primera base de datos no todas y no esta en $GLOBALS o $_SESSION.singlequote(uppercase($GLOBALS['kDatabase'][0][3]));
	
		$tables = $this->executeQuery($sql);
	    while ($table = $this->getNextRecord($tables, ASSOCIATIVE)) {
			
			$sql = 'SHOW INDEXES FROM '.$table['TABLE_NAME'] ;
			$indexes = $this->executeQuery($sql); 
			$k=0;
			while ($index = $this->getNextRecord($indexes, ASSOCIATIVE)) {
			//	pre($index);
			
				if ($index['Key_name']=='PRIMARY') {
					$index['INDEX_TYPE']='PRI';
				}
				else if($index['Non_unique']=='0') {
					$index['INDEX_TYPE']='UNI';
				}	
				else {
					$index['INDEX_TYPE']='MUL';
				}
				if ($indexname==$index['Key_name']) {
					if (empty($return[$index['Key_name']]['fieldname'])) {
						$return[$index['Key_name']]['fieldname'] = $index['Column_name'];
					} else {
						if (strpos(",".$return[$index['Key_name']]['fieldname'].",",$index['Column_name'])<1)
							$return[$index['Key_name']]['fieldname'] .= ',' . $index['Column_name'];
					}
					
				}
				$return[$index['Key_name']]['index'] = $index['INDEX_TYPE'];
			}  
		}
		
		return $return;
    }
	function fetch_all($id, $type = ASSOCIATIVE) {
		if ($id != NULL){
			return $id->fetch_all( ($type == ASSOCIATIVE) ? MYSQLI_ASSOC : MYSQLI_NUM);
		}else {
			return NULL;
		}	
	}
	function MulticreateRecord($table, $records, $silent = false) {
        if ( !empty($_SESSION["_dEBUG"]) ) { $tta=fopen(kPathToWebSite.'/'.kPathToTemp."archivo.txt","a+");	fwrite($tta,"1- inicio-hora:".date("h:i:s d/m/Y").chr(13).chr(10));fwrite($tta,"Multicreate:".$table.chr(13).chr(10));fclose($tta);}

		$sql='';
		$cont=0;
		$totrecords=array();
		
		$record_keys=array_keys($records);
		foreach ($record_keys as $field => $value) {
			if (uppercase($value)!=$value) {
				$ant=$record[$value];
				unset($record[$value]);
				$record[uppercase($value)]=$ant;
				
			}
		}	
	
		
		if (!isset($this->numerictype[$table])) {
			$this->numerictype[$table]=$this->getFieldstypeNumeric($table);
		}	
		/* Creating binding variable for id to emulate first insert id used in mysql */
		$existingID=false;
        if ($this->fieldExists($table, 'ID')) {
			$existingID=true;	
		  
			 //$record['ID'] = null; // 2018-07-26 CR : Para que asigne de manera efectiva el proximo en la Secuence
        }
		foreach($records as &$record) {
			if ($existingID===true) {
				$record['ID'] = CLX_INIT_ZERO;
			}	
/*
unset($record["CONSOL_KEY_T24"]);
unset($record["CATEGORIA_MATRIZ_T24"]);
 unset($record["CUENTA_T24"]);
 unset($record["TABLA_ORIGEN_T24"]);
 unset($record["DOC_DATE_T24"]);
 unset($record["PSTNG_DATE_T24"]);
 unset($record["FISC_YEAR_T24"]);
 unset($record["FISC_PERIOD_T24"]);
 unset($record["GL_ACCOUNT_T24"]);
 unset($record["GL_ACCOUNT_NAME_T24"]);
 unset($record["NARRATIVE"]);
 unset($record["CURRENCY_T24"]);
 unset($record["USUARIO_FUNDS_TRANSFER_T24"]);
  unset($record["NOMBRE_USUARIO_FUNDS_T24"]);
 unset($record["AUTORIZADOR_FUNDS_TRANSFER_T24"]);
unset($record[ "USUARIO_TELLER_T24"]);
unset($record[ "NOMBRE_USUARIO_TELLER_T24"]);
unset($record[ "AUTORIZADOR_TELLER_T24"]);
unset($record[ "LR_CCARD_NO_T24"]);
unset($record[ "LR_CHQNO_CONFIRM_T24"]);
unset($record[ "CARD_NUMBER_T24"]);
unset($record[ "AUTH_CODE_T24"]);
unset($record[ "RECORD_STATUS_T24"]);
unset($record[ "BIN_NUMBER_T24"]);
 unset($record["TIGOTRANSACCIONID_T24"]);
unset($record[ "NO_TELEFONO_T24"]);
unset($record[ "LR_ORD_NAME_T24"]);
unset($record[ "LR_ORD_PAR_ID_T24"]);
unset($record[  "TRANS_REFERENCE_T24"]);
unset($record[ "THEIR_REFERENCE_T24"]);
unset($record[ "TXN_CODE_T24"]);
unset($record[ "TXN_DESCRIPTION_T24"]);
unset($record[ "PRODUCT_T24"]);
unset($record[ "DEBIT_ACCT_NO_T24"]);
unset($record[ "CREDIT_ACCT_NO_T24"]);
unset($record[ "LR_NAME_T24"]);
unset($record[ "CHEQUE_NUMBER_T24"]);
unset($record[ "CHEQUE_BANK_CDE_T24"]);
 unset($record["LR_AGENCY_CODE_T24"]);
unset($record[ "NOMBRE_AGENCIA_T24"]);
unset($record[ "PAYMENT_DETAILS_T24"]);

 unset($record["OUR_REFERENCE_T24"]);
unset($record[ "LR_FLINK_CNF_NO1_T24"]);
unset($record[ "LR_TAX_PAY_NAME_T24"]);
unset($record[ "DATE_TIME_T24"]);
*/ 
##	unset($record["AMT_DOC_CUR_T24"]);
##	unset($record["AMT_LOCAL_CCY_T24"]);
			$cont++;
			$fields = '';
			$values = '';
			$comma = '';
			$changetype=array();
			foreach ($record as $field => $value) {
				$this->replaceInvalidValues($record[$field]);
				//$fields .= $comma . $field;
				//$values .= $comma . ':' . $field;
				$fields .= $comma . quote($field);
				
				
				
				if (is_array($value)) { // sample $record['RUNID']=array('ESPECIAL_FIELD'=>' (SELECT MAX(RUNID)+1 FROM RCTA_TRN_MATCH_EXECS) ');
					$values.= $comma .$value['ESPECIAL_FIELD'];
				} else if (isset($this->numerictype[$table][$field]) && $this->numerictype[$table][$field]!='DECN') {
					$values .= $comma . ':' . quote($field.'_'.$cont);
				}
				
				else {
					if (lowercase($value) =='null') {
						$value=0;
					}
					$this->numerictype[$table][$field.'_CHANGE']='CHAR';
					$this->numerictype[$table][$field.'_MASK_'.$cont]='CHAR';
				
					$values .= $comma . 'TO_NUMBER(:' . quote($field.'_'.$cont).',:'.quote($field.'_MASK_'.$cont).')';	
					$totrecords[$field.'_MASK_'.$cont]=preg_replace('/[-0123456789]/','9',$value);
				
				}
				$totrecords[$field.'_'.$cont]=$value;	
				
				$comma = ', ';
			}
			/* Creating binding variable for id to emulate first insert id used in mysql */
			$this->temporary_supress_squote_change = true;
			if (isset($extras) && ($extras['returnid']  || $extras['extracriteria']=='' )){
				$sql .= ' INTO ' . $table . ' (' . $fields . ' ) VALUES (' . $values . ') ' ;
			}
				
		}
		if (!empty($sql)) {
			$sql = 'INSERT ALL '.$sql.' SELECT 1 FROM DUAL';
			if ($silent === true) {
				$results = $this->executeQuerySilent($sql, $totrecords);
			} else {
				$results = $this->executeQuery($sql, $totrecords); //sql, binds
			}
			$this->freeResult($this->stmt);
			unset($results);
		}
        if ( !empty($_SESSION["_dEBUG"]) ) { $tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");	fwrite($tta,"1- fin-hora:".date("h:i:s d/m/Y").chr(13).chr(10));fwrite($tta,"Multicreate:".$table.chr(13).chr(10));fclose($tta);}

	}
	function getFieldstypeNumeric($table) {
        $return = array();
		$sql='SELECT X.FIELDNAME,X.FIELDTYPE FROM CLX_DB_FIELDS X,CLX_DB_TABLEFIELDS Z,CLX_DB_TABLES Y WHERE Z.TABLECODE=Y.IDCODE AND X.FIELDNAME=Z.FIELDNAME AND X.FIELDTYPE IN('.singleQuoteArray(array('DECN','INTN')).') AND CONCAT(CONCAT(Y.MODULECODE,'.singlequote('_').'),Y.IDNAME)='.singleQuote($table);	
        $rs = $this->executeQuery($sql);
	    while ($result = $this->getNextRecord($rs, ASSOCIATIVE)) {
		  $return[$result['FIELDNAME']] = $result['FIELDTYPE'];
			for ($t=1;$t<51;$t++) {
				$return[$result['FIELDNAME'].'_'.$t] = $result['FIELDTYPE'];
			}
        }

        return $return;
	}
	function read_time_tmp() {
		$sql = 'SELECT  SYPRVALUE FROM CLX_SYS_PARAMETERS WHERE SYPRKEY='.singlequote('MAX_TIME_HOUR_DELETE_TMP').' AND TOOLCODE = ' . singleQuote('DBTB');
		$maxtime=$this->getOne($sql);
		if ($maxtime<=1)
			$maxtime=1;
		define('MAX_TIME_HOUR_DELETE_TMP',$maxtime); 
	}
	function review_and_delete_old_tmp($tabletmp='',$tiempo=8640) {
		$this->read_time_tmp();
		$extra='';
		if (!empty($tabletmp))
			$extra=' TABLE_NAME ='.singlequote($tabletmp).' AND ';
		
		if ($this->debug==true) {
			echo "Review Tmp function".chr(10);
		}
		$this->temporary_supress_squote_change = true;
		
		$sql = 'SELECT TABLE_NAME  FROM information_schema.tables WHERE  '.$extra.' table_type = "base table" AND  table_schema=DATABASE() AND TABLE_NAME LIKE '.singlequote('Z__TMP_%');
		$rs = $this->executeQuery($sql);       
	
		$nowPHP=hexdec(uniqid())-200000;
	
		
        while (($result = $this->getNextRecord($rs, ASSOCIATIVE)) ) {
			if ($this->debug==true) {
				echo "Review Tmp function",$result['TABLE_NAME'];
			}
			$cortes=explode('___',$result['TABLE_NAME']);
			
			if (count($cortes)>1 && $nowPHP>intval($cortes[1])) {
				if ($this->debug==true) {
					echo " Delete temp file ".$result['TABLE_NAME'];
				}
				$this->executeQuery('DROP TABLE '.$result['TABLE_NAME']);
			}
			
		}
	}
}
