<?php

/**
 * General Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: imrsnik $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2015-09-17 11:46:16 +0000 (Thu, 17 Sep 2015) $
 * @version        $Rev: 6922 $
 * @package		Clientix.Classes.Calendar
 * @abstract		General Calendar Class*
 *
 * CHANGE LOG
 *  Included label
 */
class clx_calendar {

    /**
     * Additional debug display
     * @var object
     * @access private
     */
    var $debug;

    /**
     * Instance of class implementing calendar engine interface
     * @var object
     * @access private
     */
    var $cE;

    /**
     * Instance of Calendar_Validator (lazy initialized when isValid() or
     * getValidor() is called
     * @var Calendar_Validator
     * @access private
     */
    var $validator;

    /**
     * Year for this calendar object e.g. 2003
     * @access private
     * @var int
     */
    var $year;

    /**
     * Month for this calendar object e.g. 9
     * @access private
     * @var int
     */
    var $month;

    /**
     * Day of month for this calendar object e.g. 23
     * @access private
     * @var int
     */
    var $day;

    /**
     * Hour of day for this calendar object e.g. 13
     * @access private
     * @var int
     */
    var $hour;

    /**
     * Minute of hour this calendar object e.g. 46
     * @access private
     * @var int
     */
    var $minute;

    /**
     * Second of minute this calendar object e.g. 34
     * @access private
     * @var int
     */
    var $second;

    /**
     * Marks this calendar object as selected (e.g. 'today')
     * @access private
     * @var boolean
     */
    var $selected = false;

    /**
     * Collection of child calendar objects created from subclasses
     * of Calendar. Type depends on the object which created them.
     * @access private
     * @var array
     */
    var $children = array();

    /**
     * Constructor
     * @param boolean debug
     * @param int year
     * @param int month
     * @param int day
     * @param int hour
     * @param int minute
     * @param int second
     * @access protected
     */
    function clx_calendar($y = CLX_NUM_2000, $m = CLX_NUM_ONE, $d = CLX_NUM_ONE, $h = CLX_NUM_ZERO, $i = CLX_NUM_ZERO, $s = CLX_NUM_ZERO) {
        $this->cE = new Calendar_Engine_UnixTS();
        $this->year = (int) $y;
        $this->month = (int) $m;
        $this->day = (int) $d;
        $this->hour = (int) $h;
        $this->minute = (int) $i;
        $this->second = (int) $s;
        $this->debug = (bool) false;
    }

// End Method
    /**
     * Returns an instance of the engine
     * @return object instance of a calendar calculation engine
     * @access protected
     */
    function setTimestamp($ts) {
        $this->year = $this->cE->stampToYear($ts);
        $this->month = $this->cE->stampToMonth($ts);
        $this->day = $this->cE->stampToDay($ts);
        $this->hour = $this->cE->stampToHour($ts);
        $this->minute = $this->cE->stampToMinute($ts);
        $this->second = $this->cE->stampToSecond($ts);
    }

    /**
     * Returns a timestamp from the current date / time values. Format of
     * timestamp depends on Calendar_Engine implementation being used
     * @return int|string timestamp
     * @access public
     */
    function getTimestamp() {
        return $this->cE->dateToStamp($this->year, $this->month, $this->day, $this->hour, $this->minute, $this->second);
    }

    /**
     * Defines calendar object as selected (e.g. for today)
     * @param boolean state whether Calendar subclass
     * @return void
     * @access public
     */
    function setSelected($state = true) {
        $this->selected = $state;
    }

    /**
     * true if the calendar subclass object is selected (e.g. today)
     * @return boolean
     * @access public
     */
    function isSelected() {
        return $this->selected;
    }

    /**
     * Adjusts the date (helper method)
     * @return void
     * @access public
     */
    function adjust() {
        $stamp = $this->getTimeStamp();
        $this->year = $this->cE->stampToYear($stamp);
        $this->month = $this->cE->stampToMonth($stamp);
        $this->day = $this->cE->stampToDay($stamp);
        $this->hour = $this->cE->stampToHour($stamp);
        $this->minute = $this->cE->stampToMinute($stamp);
        $this->second = $this->cE->stampToSecond($stamp);
    }

    /**
     * Returns the date as an associative array (helper method)
     * @param mixed timestamp (leave empty for current timestamp)
     * @return array
     * @access public
     */
    function toArray($stamp = null) {
        if (is_null($stamp)) {
		  $stamp = $this->getTimeStamp();
        }
        return array('year' => $this->cE->stampToYear($stamp), 'month' => $this->cE->stampToMonth($stamp), 'day' => $this->cE->stampToDay($stamp), 'hour' => $this->cE->stampToHour($stamp), 'minute' => $this->cE->stampToMinute($stamp), 'second' => $this->cE->stampToSecond($stamp));
    }

    /**
     * Returns the value as an associative array (helper method)
     * @param string type of date object that return value represents
     * @param string $format ['int' | 'array' | 'timestamp' | 'object']
     * @param mixed timestamp (depending on Calendar engine being used)
     * @param int integer default value (i.e. give me the answer quick)
     * @return mixed
     * @access private
     */
    function returnValue($returnType, $format, $stamp, $default) {
        switch (lowercase($format)) {
		  case 'int':
			 $retorno = $default;
			 break;
		  case 'array':
			 $retorno = $this->toArray($stamp);
			 break;
		  case 'object':
			 break;
		  case 'timestamp':
			 if ($returnType == 'year')
			     $retorno = $stamp;
			 else
			     $retorno = $stamp;
			 break;
		  default:
			 $retorno = $stamp;
			 break;
        }
        return $retorno;
    }

    /**
     * Iterator method for fetching child Calendar subclass objects
     * (e.g. a minute from an hour object). On reaching the end of
     * the collection, returns false and resets the collection for
     * further iteratations.
     * @return mixed either an object subclass of Calendar or false
     * @access public
     */
    function fetch() {
        $child = each($this->children);
        if (!empty($child)) {
		  return $child['value'];
        } else {
		  reset($this->children);
		  return false;
        }
    }

    /**
     * Fetches all child from the current collection of children
     * @return array
     * @access public
     */
    function fetchAll() {
        return $this->children;
    }

    /**
     * Get the number Calendar subclass objects stored in the internal
     * collection.
     * @return int
     * @access public
     */
    function size() {
        return count($this->children);
    }

    /**
     * Determine whether this date is valid, with the bounds determined by
     * the Calendar_Engine. The call is passed on to
     * Calendar_Validator::isValid
     * @return boolean
     * @access public
     */
    function isValid() {
        $validator = & $this->getValidator();
        return $validator->isValid();
    }

    /**
     * Returns the value for the previous year
     * @param string return value format ['int' | 'timestamp' | 'object' | 'array']
     * @return int e.g. 2002 or timestamp
     * @access public
     */
    function prevYear($format = 'int') {
        $ts = $this->cE->dateToStamp($this->year - CLX_NUM_ONE, CLX_NUM_ONE, CLX_NUM_ONE, CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO);
        return $this->returnValue('Year', $format, $ts, $this->year - CLX_NUM_ONE);
    }

    /**
     * Returns the value for this year
     * @param string return value format ['int' | 'timestamp' | 'object' | 'array']
     * @return int e.g. 2003 or timestamp
     * @access public
     */
    function thisYear($format = 'int') {
        $ts = $this->cE->dateToStamp($this->year, CLX_NUM_ONE, CLX_NUM_ONE, CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO);
        return $this->returnValue('Year', $format, $ts, $this->year);
    }

    /**
     * Returns the value for next year
     * @param string return value format ['int' | 'timestamp' | 'object' | 'array']
     * @return int e.g. 2004 or timestamp
     * @access public
     */
    function nextYear($format = 'int') {
        $ts = $this->cE->dateToStamp($this->year + CLX_NUM_ONE, CLX_NUM_ONE, CLX_NUM_ONE, CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO);
        return $this->returnValue('Year', $format, $ts, $this->year + CLX_NUM_ONE);
    }

    /**
     * Returns the value for the previous month
     * @param string return value format ['int' | 'timestamp' | 'object' | 'array']
     * @return int e.g. 4 or Unix timestamp
     * @access public
     */
    function prevMonth($format = 'int') {
        $ts = $this->cE->dateToStamp($this->year, $this->month - CLX_NUM_ONE, CLX_NUM_ONE, CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO);
        return $this->returnValue('Month', $format, $ts, $this->cE->stampToMonth($ts));
    }

    /**
     * Returns the value for this month
     * @param string return value format ['int' | 'timestamp' | 'object' | 'array']
     * @return int e.g. 5 or timestamp
     * @access public
     */
    function thisMonth($format = 'int') {
        $ts = $this->cE->dateToStamp($this->year, $this->month, CLX_NUM_ONE, CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO);
        return $this->returnValue('Month', $format, $ts, $this->month);
    }

    /**
     * Returns the value for next month
     * @param string return value format ['int' | 'timestamp' | 'object' | 'array']
     * @return int e.g. 6 or timestamp
     * @access public
     */
    function nextMonth($format = 'int') {
        $ts = $this->cE->dateToStamp($this->year, $this->month + CLX_NUM_ONE, CLX_NUM_ONE, CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO);
        return $this->returnValue('Month', $format, $ts, $this->cE->stampToMonth($ts));
    }

    /**
     * Returns the value for the previous day
     * @param string return value format ['int' | 'timestamp' | 'object' | 'array']
     * @return int e.g. 10 or timestamp
     * @access public
     */
    function prevDay($format = 'int') {
        $ts = $this->cE->dateToStamp($this->year, $this->month, $this->day - CLX_NUM_ONE, CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO);
        return $this->returnValue('Day', $format, $ts, $this->cE->stampToDay($ts));
    }

    /**
     * Returns the value for this day
     * @param string return value format ['int' | 'timestamp' | 'object' | 'array']
     * @return int e.g. 11 or timestamp
     * @access public
     */
    function thisDay($format = 'int') {
        $ts = $this->cE->dateToStamp($this->year, $this->month, $this->day, CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO);
        return $this->returnValue('Day', $format, $ts, $this->day);
    }

    /**
     * Returns the value for the next day
     * @param string return value format ['int' | 'timestamp' | 'object' | 'array']
     * @return int e.g. 12 or timestamp
     * @access public
     */
    function nextDay($format = 'int') {
        $ts = $this->cE->dateToStamp($this->year, $this->month, $this->day + CLX_NUM_ONE, CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO);
        return $this->returnValue('Day', $format, $ts, $this->cE->stampToDay($ts));
    }

    /**
     * Returns the value for the previous hour
     * @param string return value format ['int' | 'timestamp' | 'object' | 'array']
     * @return int e.g. 13 or timestamp
     * @access public
     */
    function prevHour($format = 'int') {
        $ts = $this->cE->dateToStamp($this->year, $this->month, $this->day, $this->hour - CLX_NUM_ONE, CLX_NUM_ZERO, CLX_NUM_ZERO);
        return $this->returnValue('Hour', $format, $ts, $this->cE->stampToHour($ts));
    }

    /**
     * Returns the value for this hour
     * @param string return value format ['int' | 'timestamp' | 'object' | 'array']
     * @return int e.g. 14 or timestamp
     * @access public
     */
    function thisHour($format = 'int') {
        $ts = $this->cE->dateToStamp($this->year, $this->month, $this->day, $this->hour, CLX_NUM_ZERO, CLX_NUM_ZERO);
        return $this->returnValue('Hour', $format, $ts, $this->hour);
    }

    /**
     * Returns the value for the next hour
     * @param string return value format ['int' | 'timestamp' | 'object' | 'array']
     * @return int e.g. 14 or timestamp
     * @access public
     */
    function nextHour($format = 'int') {
        $ts = $this->cE->dateToStamp($this->year, $this->month, $this->day, $this->hour + CLX_NUM_ONE, CLX_NUM_ZERO, CLX_NUM_ZERO);
        return $this->returnValue('Hour', $format, $ts, $this->cE->stampToHour($ts));
    }

    /**
     * Returns the value for the previous minute
     * @param string return value format ['int' | 'timestamp' | 'object' | 'array']
     * @return int e.g. 23 or timestamp
     * @access public
     */
    function prevMinute($format = 'int') {
        $ts = $this->cE->dateToStamp($this->year, $this->month, $this->day, $this->hour, $this->minute - CLX_NUM_ONE, CLX_NUM_ZERO);
        return $this->returnValue('Minute', $format, $ts, $this->cE->stampToMinute($ts));
    }

    /**
     * Returns the value for this minute
     * @param string return value format ['int' | 'timestamp' | 'object' | 'array']
     * @return int e.g. 24 or timestamp
     * @access public
     */
    function thisMinute($format = 'int') {
        $ts = $this->cE->dateToStamp($this->year, $this->month, $this->day, $this->hour, $this->minute, CLX_NUM_ZERO);
        return $this->returnValue('Minute', $format, $ts, $this->minute);
    }

    /**
     * Returns the value for the next minute
     * @param string return value format ['int' | 'timestamp' | 'object' | 'array']
     * @return int e.g. 25 or timestamp
     * @access public
     */
    function nextMinute($format = 'int') {
        $ts = $this->cE->dateToStamp($this->year, $this->month, $this->day, $this->hour, $this->minute + CLX_NUM_ONE, CLX_NUM_ZERO);
        return $this->returnValue('Minute', $format, $ts, $this->cE->stampToMinute($ts));
    }

    /**
     * Returns the value for the previous second
     * @param string return value format ['int' | 'timestamp' | 'object' | 'array']
     * @return int e.g. 43 or timestamp
     * @access public
     */
    function prevSecond($format = 'int') {
        $ts = $this->cE->dateToStamp($this->year, $this->month, $this->day, $this->hour, $this->minute, $this->second - CLX_NUM_ONE);
        return $this->returnValue('Second', $format, $ts, $this->cE->stampToSecond($ts));
    }

    /**
     * Returns the value for this second
     * @param string return value format ['int' | 'timestamp' | 'object' | 'array']
     * @return int e.g. 44 or timestamp
     * @access public
     */
    function thisSecond($format = 'int') {
        $ts = $this->cE->dateToStamp($this->year, $this->month, $this->day, $this->hour, $this->minute, $this->second);
        return $this->returnValue('Second', $format, $ts, $this->second);
    }

    /**
     * Returns the value for the next second
     * @param string return value format ['int' | 'timestamp' | 'object' | 'array']
     * @return int e.g. 45 or timestamp
     * @access public
     */
    function nextSecond($format = 'int') {
        $ts = $this->cE->dateToStamp($this->year, $this->month, $this->day, $this->hour, $this->minute, $this->second + CLX_NUM_ONE);
        return $this->returnValue('Second', $format, $ts, $this->cE->stampToSecond($ts));
    }

    /**
     * Returns the first week day
     * @return current date
     * @access public
     */
    function firstWeekDay() {

    }

}

// End class
class Calendar_Engine_Interface {

    /**
     * Provides a mechansim to make sure parsing of timestamps
     * into human dates is only performed once per timestamp.
     * Typically called 'internally' by methods like stampToYear.
     * Return value can vary, depending on the specific implementation
     * @param int timestamp (depending on implementation)
     * @return mixed
     * @access protected
     */
    function stampCollection($stamp) {
	ClxFreeReturn($stamp);
    }

    /**
     * Returns a numeric year given a timestamp
     * @param int timestamp (depending on implementation)
     * @return int year (e.g. 2003)
     * @access protected
     */
    function stampToYear($stamp) {
	ClxFreeReturn($stamp);
    }

    /**
     * Returns a numeric month given a timestamp
     * @param int timestamp (depending on implementation)
     * @return int month (e.g. 9)
     * @access protected
     */
    function stampToMonth($stamp) {
	ClxFreeReturn($stamp);
    }

    /**
     * Returns a numeric day given a timestamp
     * @param int timestamp (depending on implementation)
     * @return int day (e.g. 15)
     * @access protected
     */
    function stampToDay($stamp) {
	ClxFreeReturn($stamp);
    }

    /**
     * Returns a numeric hour given a timestamp
     * @param int timestamp (depending on implementation)
     * @return int hour (e.g. 13)
     * @access protected
     */
    function stampToHour($stamp) {
	ClxFreeReturn($stamp);
    }

    /**
     * Returns a numeric minute given a timestamp
     * @param int timestamp (depending on implementation)
     * @return int minute (e.g. 34)
     * @access protected
     */
    function stampToMinute($stamp) {
	ClxFreeReturn($stamp);
    }

    /**
     * Returns a numeric second given a timestamp
     * @param int timestamp (depending on implementation)
     * @return int second (e.g. 51)
     * @access protected
     */
    function stampToSecond($stamp) {
	ClxFreeReturn($stamp);
    }

    /**
     * Returns a timestamp. Can be worth 'caching' generated
     * timestamps in a static variable, identified by the
     * params this method accepts, to timestamp will only
     * be calculated once.
     * @param int year (e.g. 2003)
     * @param int month (e.g. 9)
     * @param int day (e.g. 13)
     * @param int hour (e.g. 13)
     * @param int minute (e.g. 34)
     * @param int second (e.g. 53)
     * @return int (depends on implementation)
     * @access protected
     */
    function dateToStamp($y, $m, $d, $h, $i, $s) {
	ClxFreeReturn($y);
	ClxFreeReturn($m);
	ClxFreeReturn($d);
	ClxFreeReturn($h);
	ClxFreeReturn($i);
	ClxFreeReturn($s);
    }

    /**
     * The upper limit on years that the Calendar Engine can work with
     * @return int (e.g. 2037)
     * @access protected
     */
    function getMaxYears() {

    }

    /**
     * The lower limit on years that the Calendar Engine can work with
     * @return int (e.g 1902)
     * @access protected
     */
    function getMinYears() {

    }

    /**
     * Returns the number of months in a year
     * @param int (optional) year to get months for
     * @return int (e.g. 12)
     * @access protected
     */
    function getMonthsInYear($y = null) {
	ClxFreeReturn($y);
    }

    /**
     * Returns the number of days in a month, given year and month
     * @param int year (e.g. 2003)
     * @param int month (e.g. 9)
     * @return int days in month
     * @access protected
     */
    function getDaysInMonth($y, $m) {
	ClxFreeReturn($y);
	ClxFreeReturn($m);
    }

    /**
     * Returns numeric representation of the day of the week in a month,
     * given year and month
     * @param int year (e.g. 2003)
     * @param int month (e.g. 9)
     * @return int
     * @access protected
     */
    function getFirstDayInMonth($y, $m) {
	ClxFreeReturn($y);
	ClxFreeReturn($m);
    }

    /**
     * Returns the number of days in a week
     * @param int year (2003)
     * @param int month (9)
     * @param int day (4)
     * @return int (e.g. 7)
     * @access protected
     */
    function getDaysInWeek($y = null, $m = null, $d = null) {
	ClxFreeReturn($y);
	ClxFreeReturn($m);
	ClxFreeReturn($d);
    }

    /**
     * Returns the number of the week in the year (ISO-8601), given a date
     * @param int year (2003)
     * @param int month (9)
     * @param int day (4)
     * @return int week number
     * @access protected
     */
    function getWeekNInYear($y, $m, $d) {
	ClxFreeReturn($y);
	ClxFreeReturn($m);
	ClxFreeReturn($d);
    }

    /**
     * Returns the number of the week in the month, given a date
     * @param int year (2003)
     * @param int month (9)
     * @param int day (4)
     * @param int first day of the week (default: 1 - monday)
     * @return int week number
     * @access protected
     */
    function getWeekNInMonth($y, $m, $d, $firstDay = CLX_NUM_ONE) {
	ClxFreeReturn($y);
	ClxFreeReturn($m);
	ClxFreeReturn($d);
	ClxFreeReturn($firstday);
    }

    /**
     * Returns the number of weeks in the month
     * @param int year (2003)
     * @param int month (9)
     * @param int first day of the week (default: 1 - monday)
     * @return int weeks number
     * @access protected
     */
    function getWeeksInMonth($y, $m) {
	ClxFreeReturn($y);
	ClxFreeReturn($m);
    }

    /**
     * Returns the number of the day of the week (0=sunday, 1=monday...)
     * @param int year (2003)
     * @param int month (9)
     * @param int day (4)
     * @return int weekday number
     * @access protected
     */
    function getDayOfWeek($y, $m, $d) {
	ClxFreeReturn($y);
	ClxFreeReturn($m);
	ClxFreeReturn($d);
    }

    /**
     * Returns the numeric values of the days of the week.
     * @param int year (2003)
     * @param int month (9)
     * @param int day (4)
     * @return array list of numeric values of days in week, beginning 0
     * @access protected
     */
    function getWeekDays($y = null, $m = null, $d = null) {
	ClxFreeReturn($y);
	ClxFreeReturn($m);
	ClxFreeReturn($d);
    }

    /**
     * Returns the default first day of the week as an integer. Must be a
     * member of the array returned from getWeekDays
     * @param int year (2003)
     * @param int month (9)
     * @param int day (4)
     * @return int (e.g. 1 for Monday)
     * @see getWeekDays
     * @access protected
     */
    function getFirstDayOfWeek($y = null, $m = null, $d = null) {
	ClxFreeReturn($y);
	ClxFreeReturn($m);
	ClxFreeReturn($d);
    }

    /**
     * Returns the number of hours in a day<br>
     * @param int (optional) day to get hours for
     * @return int (e.g. 24)
     * @access protected
     */
    function getHoursInDay($y = null, $m = null, $d = null) {
	ClxFreeReturn($y);
	ClxFreeReturn($m);
	ClxFreeReturn($d);
    }

    /**
     * Returns the number of minutes in an hour
     * @param int (optional) hour to get minutes for
     * @return int
     * @access protected
     */
    function getMinutesInHour($y = null, $m = null, $d = null, $h = null) {
	ClxFreeReturn($y);
	ClxFreeReturn($m);
	ClxFreeReturn($d);
	ClxFreeReturn($h);
    }

    /**
     * Returns the number of seconds in a minutes
     * @param int (optional) minute to get seconds for
     * @return int
     * @access protected
     */
    function getSecondsInMinute($y = null, $m = null, $d = null, $h = null, $i = null) {
	ClxFreeReturn($y);
	ClxFreeReturn($m);
	ClxFreeReturn($d);
	ClxFreeReturn($h);
	ClxFreeReturn($i);
    }

}

class Calendar_Engine_UnixTS /* implements Calendar_Engine_Interface */ {

    /**
     * Makes sure a given timestamp is only ever parsed once
     * <pre>
     * array (
     *  [CLX_NUM_ZERO] => year (e.g 2003),
     *  [1] => month (e.g 9),
     *  [2] => day (e.g 6),
     *  [3] => hour (e.g 14),
     *  [4] => minute (e.g 34),
     *  [5] => second (e.g 45),
     *  [6] => num days in month (e.g. 31),
     *  [7] => week in year (e.g. 50),
     *  [8] => day in week (e.g. 0 for Sunday)
     * )
     * </pre>
     * Uses a static variable to prevent date() being used twice
     * for a date which is already known
     * @param int Unix timestamp
     * @return array
     * @access protected
     */
    function stampCollection($stamp) {
        static $stamps = array();
        if (!isset($stamps[$stamp])) {
		  $date = date('Y n j H i s t W w', $stamp);
		  $stamps[$stamp] = sscanf($date, '%d %d %d %d %d %d %d %d %d');
        }
        return $stamps[$stamp];
    }

    /**
     * Returns a numeric year given a timestamp
     * @param int Unix timestamp
     * @return int year (e.g. 2003)
     * @access protected
     */
    function stampToYear($stamp) {
        $date = Calendar_Engine_UnixTS::stampCollection($stamp);
        return (int) $date[CLX_NUM_ZERO];
    }

    /**
     * Returns a numeric month given a timestamp
     * @param int Unix timestamp
     * @return int month (e.g. 9)
     * @access protected
     */
    function stampToMonth($stamp) {
        $date = Calendar_Engine_UnixTS::stampCollection($stamp);
        return (int) $date[CLX_NUM_ONE];
    }

    /**
     * Returns a numeric day given a timestamp
     * @param int Unix timestamp
     * @return int day (e.g. 15)
     * @access protected
     */
    function stampToDay($stamp) {
        $date = Calendar_Engine_UnixTS::stampCollection($stamp);
        return (int) $date[CLX_NUM_TWO];
    }

    /**
     * Returns a numeric hour given a timestamp
     * @param int Unix timestamp
     * @return int hour (e.g. 13)
     * @access protected
     */
    function stampToHour($stamp) {
        $date = Calendar_Engine_UnixTS::stampCollection($stamp);
        return (int) $date[CLX_NUM_THREE];
    }

    /**
     * Returns a numeric minute given a timestamp
     * @param int Unix timestamp
     * @return int minute (e.g. 34)
     * @access protected
     */
    function stampToMinute($stamp) {
        $date = Calendar_Engine_UnixTS::stampCollection($stamp);
        return (int) $date[CLX_NUM_FOUR];
    }

    /**
     * Returns a numeric second given a timestamp
     * @param int Unix timestamp
     * @return int second (e.g. 51)
     * @access protected
     */
    function stampToSecond($stamp) {
        $date = Calendar_Engine_UnixTS::stampCollection($stamp);
        return (int) $date[CLX_NUM_FIVE];
    }

    /**
     * Returns a timestamp
     * @param int year (2003)
     * @param int month (9)
     * @param int day (13)
     * @param int hour (13)
     * @param int minute (34)
     * @param int second (53)
     * @return int Unix timestamp
     * @access protected
     */
    function dateToStamp($y, $m, $d, $h = CLX_NUM_ZERO, $i = CLX_NUM_ZERO, $s = CLX_NUM_ZERO) {
        static $dates = array();
        if (!isset($dates[$y][$m][$d][$h][$i][$s])) {
		  $dates[$y][$m][$d][$h][$i][$s] = mktime($h, $i, $s, $m, $d, $y);
        }
        return $dates[$y][$m][$d][$h][$i][$s];
    }

    /**
     * The upper limit on years that the Calendar Engine can work with
     * @return int (2037)
     * @access protected
     */
    function getMaxYears() {
        return CLX_NUM_2037;
    }

    /**
     * The lower limit on years that the Calendar Engine can work with
     * @return int (1970 if it's Windows and 1902 for all other OSs)
     * @access protected
     */
    function getMinYears() {
        return  strpos(PHP_OS, 'WIN') === false ? CLX_NUM_1902 : CLX_NUM_1970;
    }

    /**
     * Returns the number of months in a year
     * @return int (12)
     * @access protected
     */
    function getMonthsInYear($y = null) {
        return CLX_NUM_12;
    }

    /**
     * Returns the number of days in a month, given year and month
     * @param int year (2003)
     * @param int month (9)
     * @return int days in month
     * @access protected
     */
    function getDaysInMonth($y, $m) {
        $stamp = Calendar_Engine_UnixTS::dateToStamp($y, $m, CLX_NUM_ONE);
        $date = Calendar_Engine_UnixTS::stampCollection($stamp);
        return $date[CLX_NUM_SIX];
    }

    /**
     * Returns numeric representation of the day of the week in a month,
     * given year and month
     * @param int year (2003)
     * @param int month (9)
     * @return int from 0 to 6
     * @access protected
     */
    function getFirstDayInMonth($y, $m) {
        $stamp = Calendar_Engine_UnixTS::dateToStamp($y, $m, CLX_NUM_ONE);
        $date = Calendar_Engine_UnixTS::stampCollection($stamp);
        return $date[CLX_NUM_8];
    }

    /**
     * Returns the number of days in a week
     * @param int year (2003)
     * @param int month (9)
     * @param int day (4)
     * @return int (7)
     * @access protected
     */
    function getDaysInWeek($y = null, $m = null, $d = null) {
	ClxFreeReturn($y);
	ClxFreeReturn($m);
	ClxFreeReturn($d);
        return CLX_NUM_7;
    }

    /**
     * Returns the number of the week in the year (ISO-8601), given a date
     * @param int year (2003)
     * @param int month (9)
     * @param int day (4)
     * @return int week number
     * @access protected
     */
    function getWeekNInYear($y, $m, $d) {
        $stamp = Calendar_Engine_UnixTS::dateToStamp($y, $m, $d);
        $date = Calendar_Engine_UnixTS::stampCollection($stamp);
        return $date[CLX_NUM_7];
    }

    /**
     * Returns the number of the week in the month, given a date
     * @param int year (2003)
     * @param int month (9)
     * @param int day (4)
     * @param int first day of the week (default: monday)
     * @return int week number
     * @access protected
     */
    function getWeekNInMonth($y, $m, $d, $firstDay = CLX_NUM_ONE) {
        $weekEnd = ($firstDay == CLX_NUM_ZERO) ? $this->getDaysInWeek() - CLX_NUM_ONE : $firstDay - CLX_NUM_ONE;
        $end_of_week = CLX_NUM_ONE;
        while (date('w', mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, $m, $end_of_week, $y)) != $weekEnd) {
		  ++$end_of_week; //find first weekend of the month
        }
        $w = CLX_NUM_ONE;
        while ($d > $end_of_week) {
		  ++$w;
		  $end_of_week += $this->getDaysInWeek();
        }
        return $w;
    }

    /**
     * Returns the number of weeks in the month
     * @param int year (2003)
     * @param int month (9)
     * @param int first day of the week (default: monday)
     * @return int weeks number
     * @access protected
     */
    function getWeeksInMonth($y, $m, $firstDay = CLX_NUM_ONE) {
	$FDOM = $this->getFirstDayInMonth($y, $m);
        if ($FDOM > $firstDay) {
		  $firstWeekDays = $this->getDaysInWeek() - $FDOM + $firstDay;
		  $weeks = CLX_NUM_ONE;
        } else {
		  $firstWeekDays = $firstDay - $FDOM;
		  $weeks = CLX_INIT_ZERO;
        }
        $firstWeekDays %= $this->getDaysInWeek();
        $result = (int) (ceil(($this->getDaysInMonth($y, $m) - $firstWeekDays) /
			         $this->getDaysInWeek()) + $weeks);
        // Hack - 0 as FDOM is a special case
        if ($FDOM != CLX_NUM_ZERO) {
		  return $result;
        } else {
		  return $result + CLX_NUM_ONE;
        }
    }

    /**
     * Returns the number of the day of the week (0=sunday, 1=monday...)
     * @param int year (2003)
     * @param int month (9)
     * @param int day (4)
     * @return int weekday number
     * @access protected
     */
    function getDayOfWeek($y, $m, $d) {
        $stamp = Calendar_Engine_UnixTS::dateToStamp($y, $m, $d);
        $date = Calendar_Engine_UnixTS::stampCollection($stamp);
        return $date[CLX_NUM_8];
    }

    /**
     * Returns a list of integer days of the week beginning 0
     * @param int year (2003)
     * @param int month (9)
     * @param int day (4)
     * @return array (0,1,2,3,4,5,6) 1 = Monday
     * @access protected
     */
    function getWeekDays($y = null, $m = null, $d = null) {
	ClxFreeReturn($y);
	ClxFreeReturn($m);
	ClxFreeReturn($d);

        return array(CLX_NUM_ZERO, CLX_NUM_ONE, CLX_NUM_TWO, CLX_NUM_THREE, CLX_NUM_FOUR, CLX_NUM_FIVE, CLX_NUM_SIX);
    }

    /**
     * Returns the default first day of the week
     * @param int year (2003)
     * @param int month (9)
     * @param int day (4)
     * @return int (default 1 = Monday)
     * @access protected
     */
    function getFirstDayOfWeek($y = null, $m = null, $d = null) {
	ClxFreeReturn($y);
	ClxFreeReturn($m);
	ClxFreeReturn($d);

	return CLX_NUM_ONE;
    }

    /**
     * Returns the number of hours in a day
     * @return int (24)
     * @access protected
     */
    function getHoursInDay($y = null, $m = null, $d = null) {
	ClxFreeReturn($y);
	ClxFreeReturn($m);
	ClxFreeReturn($d);
	
        return CLX_NUM_24;
    }

    /**
     * Returns the number of minutes in an hour
     * @return int (60)
     * @access protected
     */
    function getMinutesInHour($y = null, $m = null, $d = null, $h = null) {
	ClxFreeReturn($y);
	ClxFreeReturn($m);
	ClxFreeReturn($d);
	ClxFreeReturn($h);
        return CLX_NUM_60;
    }

    /**
     * Returns the number of seconds in a minutes
     * @return int (60)
     * @access protected
     */
    function getSecondsInMinute($y = null, $m = null, $d = null, $h = null, $i = null) {
	ClxFreeReturn($y);
	ClxFreeReturn($m);
	ClxFreeReturn($d);
	ClxFreeReturn($h);
	ClxFreeReturn($i);    
	return CLX_NUM_60;
    }

}

class ui_calendar {

    /**
     * Instance of the Calendar object
     * @var object
     * @access private
     */
    var $calendar = false;

    /**
     * Data details for calendar entry
     * @var array
     * @access private
     */
    var $details = array();

    /**
     * Constructor
     */
    function ui_calendar($ts = CLX_NUM_ZERO) {
        $this->calendar = new clx_calendar();
        if (empty($ts))
		  $this->calendar->adjust();
        else
		  $this->calendar->setTimestamp($ts);
    }

// End method
    /**
     * Build the calendar on tabular format for day.
     * @access public
     */
    function buildDay() {
        echo $this->calendar->thisHour('int');
    }

    /**
     * Build the calendar on tabular format for week.
     * @access public
     */
    function buildWeek() {

    }

    /**
     * Build the calendar on tabular format for work week.
     * @access public
     */
    function buildWorkWeek() {

    }

    /**
     * Build the calendar on tabular format for month.
     * @access public
     */
    function buildMonth() {

    }

    /**
     * Add detail to list
     * @access public
     */
    function addDetail() {
        $this->datails = array();
    }

}

// End class
$cal = new ui_calendar();
$cal->buildDay();
