<?php

/**
 * General Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: cramirez $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2018-10-09 11:22:02 -0400 (Tue, 09 Oct 2018) $
 * @version        $Rev: 10222 $
 * @package		Clientix.Classes.Form
 * @abstract		General Class
 * CHANGE LOG
 *  Included label
 */
require_once'clx_form_panels.php';
require_once'clx_form_panel_entries.php';
require_once'clx_field.php';
require_once'clx_validate.php';

class clx_form {

    var $title;    // The Title of the form (optional)
    var $subtitle;   // The subtitle of the form (optional)
    var $description;  // The description of the form (optional)
    var $panels;   // array of clx_form_panels
    var $htmlform;   // HTMLForm
    var $topscripts;  // JavaScripts
    var $btmscripts;
    var $action;
    var $modify;
    var $toolcode;
    var $other_html;
    var $submit_action;
    var $arguments;
    var $button_labels;
    var $cancelButton;
	var $break;

    // CONSTRUCTOR //
    function __construct($toolcode, $data) {
        $this->toolcode = $toolcode;
        $this->button_labels = array('SUBMIT' => dict('GEN_LBL_SAVE'), 'CANCEL' => dict('GEN_LBL_CANCEL'));
        $this->cancelButton = true;
        $this->submit_action = 'tool.php';
        $this->loadArray($data);
        $this->arguments = array();
    }

    function loadArray($data) {
        $this->title = (!empty($data[CLX_CLASS_TITLE])) ? $data[CLX_CLASS_TITLE] : '';
        $this->subtitle = (!empty($data[CLX_CLASS_SUBTITLE])) ? $data[CLX_CLASS_SUBTITLE] : '';
        $this->description = (!empty($data['type'])) ? $data['description'] : '';
        $this->panels = array();
        if (isset($data['button_labels'])) {
		  $this->button_labels = $data['button_labels'];
        }
        if (empty($data['panels']))
		  $data['panels'] = array();
        foreach ($data['panels'] as $paneldata) {
		  $paneltype = 'clx_form_panel_' . $paneldata['type'];
		  $this->panels[] = new $paneltype($this, $paneldata);
        }
    }

    function setArgument($key, $value) {
        $this->arguments[$key] = $value;
    }

    function getArgument($key) {
        return $this->arguments[$key];
    }

    function addOtherHtml($html) {
        $this->other_html = $html;
    }

    function editForm($action, $idlist, $record, $msg) {IF ($GLOBALS['ADDRDEBUGEDITFORM_']==1) { ECHO '__METHOD____FILE____LINE__'. __METHOD__.' '.__FILE__.':'.__LINE__.'<br>'; }
		$this->action = $action;
		if ($this->action === 'MODIFY')
			$this->modify = true;
        $this->topscripts = '';
        $this->btmscripts = '';
        $this->htmlform = new htmlForm(CLX_PROP_ACTION, $this->submit_action);
        $this->htmlform->addArgument(CLX_PROP_TO, $this->toolcode);
        $this->htmlform->addArgument('ac', $action);
        $this->htmlform->addArgument('confirm', '1');
        $this->htmlform->addArgument('idlist', toString($idlist));
        foreach ($this->arguments as $key => $value) {
		  $this->htmlform->addArgument($key, $value);
        }
        $this->htmlform->setProp('enctype', 'multipart/form-data');
        $this->htmlform->setProp('accept-charset', 'utf-8');
        $this->htmlform->addArgument($this->toolcode . '_id', !empty($record[$this->toolcode . '_id']) ? $record[$this->toolcode . '_id'] : CLX_NUM_ZERO);
        if ((strlen($this->title) + strlen($this->subtitle) + strlen($this->description)) > CLX_NUM_ZERO) {
		  $table = new htmlTable(CLX_WIDTH_ONE);
		  $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_WIDTH => '90%'));
		  if (strlen($this->title) > CLX_NUM_ZERO)
			 $table->addCell(dict($this->title), array(CLX_PROP_CLASS => CLX_CLASS_TITLE));
		  if (strlen($this->subtitle) > CLX_NUM_ZERO)
			 $table->addCell(dict($this->subtitle), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE));
		  if (strlen($this->description) > CLX_NUM_ZERO)
			 $table->addCell(dict($this->description), array(CLX_PROP_CLASS => CLX_CLASS_DATA));
		  $this->htmlform->addContents($table->source());
        }
        foreach ($this->panels as $panel) {
		  $this->htmlform->addContents($panel->editForm_($this, $record, $msg));
        }
        $this->htmlform->addContents('<br>');
        if ($this->other_html <> '') {
		  $this->htmlform->addContents($this->other_html);
        }
        $table = new htmlTable(CLX_WIDTH_ONE);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => '90%'));
        $buttons = htmlButton::getButton('submit', $this->button_labels['SUBMIT'], array('action' => $action, 'form' => $this->htmlform->getProp('name')))->source();
        if ($this->cancelButton === true) {
		  $c = new htmlButtonCancel($this->button_labels['CANCEL']);
		  $buttons .= $c->source();
        }
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS));
        $this->htmlform->addContents($table->source());
        return $this->topscripts . $this->htmlform->source() . $this->btmscripts;
    }

    function quickFilterForm($action, $idlist, $record, $msg) {
        $this->action = (empty($action)) ? 'NOT_DEFINED' : $action;
        $this->topscripts = '';
        $this->btmscripts = '';
        $this->htmlform = new htmlForm(CLX_PROP_ACTION, $this->submit_action);
        $this->htmlform->addArgument(CLX_PROP_TO, $this->toolcode);
        $this->htmlform->addArgument('ac', $action);
        $this->htmlform->addArgument('confirm', '1');
        $this->htmlform->addArgument('idlist', toString($idlist));
        foreach ($this->arguments as $key => $value) {
		  $this->htmlform->addArgument($key, $value);
        }
        $this->htmlform->setProp('enctype', 'multipart/form-data');

        $table = new htmlTable(CLX_WIDTH_ONE);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_WIDTH => '90%'));


        $table->addCell(dict('GEN_TTL_QFILTER'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE));
        $this->htmlform->addContents($table->source());
        foreach ($this->panels as $panel) {
		  $this->htmlform->addContents($panel->editForm_($this, $record, $msg));
        }
        $buttons = htmlButton::getButton('submit', dict('GEN_LBL_SET_FILTER'), array('action' => 'QFILTER', 'form' => $this->htmlform->getProp('name')))->source();
        $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_CLEAR_FILTER'), array('action' => 'UNFILTER', 'form' => $this->htmlform->getProp('name')))->source();
        $table = new htmlTable(CLX_WIDTH_ONE);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => '90%'));
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_NUM_ONE));
        $this->htmlform->addContents($table->source());
        return $this->topscripts . $this->htmlform->source() . $this->btmscripts;
    }

    function viewForm_($record) {
        $contents = '';
        if ((strlen($this->title) + strlen($this->subtitle) + strlen($this->description)) > CLX_NUM_ZERO) {
		  $table = new htmlTable(CLX_WIDTH_ONE);
		  $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => '90%'));
		  if (strlen($this->title) > CLX_NUM_ZERO)
			 $table->addCell(dict($this->title), array(CLX_PROP_CLASS => CLX_CLASS_TITLE));
		  if (strlen($this->subtitle) > CLX_NUM_ZERO)
			 $table->addCell(dict($this->subtitle), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE));
		  if (strlen($this->description) > CLX_NUM_ZERO)
			 $table->addCell(dict($this->description), array(CLX_PROP_CLASS => CLX_CLASS_DATA));
		  $contents .= $table->source();
        }
        foreach ($this->panels as $panel) {

		  $contents .= $panel->viewForm_2($this->toolcode, $record);
        }
        return $contents;
    }

    function checkForm1($action, &$record, &$msg) {
        $this->action = $action;
        $this->modify = ($this->action === 'MODIFY') ? '1' : '0';
        foreach ($this->panels as $panel) {
		  $panel->checkForm($this, $record, $msg);
        }
		if (!is_array($msg))$msg=array();
        return count($msg) == CLX_NUM_ZERO;
    }

    function setButtonLabel($action, $label) {
        $this->button_labels[$action] = $label;
    }

}
