<?php

/**
 * General Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: cramirez $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2018-10-09 11:22:02 -0400 (Tue, 09 Oct 2018) $
 * @version        $Rev: 10222 $
 * @package		Clientix.Classes.Listing
 * @abstract		Abstract Class
 * CHANGE LOG
 *  Included label
 */
class clx_listing extends clx_object {

    var $title;    // The Title of the form (optional)
    var $subtitle;   // The subtitle of the form (optional)
    var $fields;   // array of clx_fields
    private $buttons;       // array of buttons
    var $htmlform;   // HTMLForm
    var $topscripts;  // JavaScripts
    var $btmscripts;
    var $toolcode;   // Tool code
    var $data;   // List data
    var $rpp;    //Records per page
    var $tablename = '';    // Table name
    var $selectclause = '*';
    var $whereclause = '';
    var $sortclause = '';
    var $panel_arguments;   //Criteria to build panel
    var $ispanel = false; //Set to true to create colapsable panel.
    /**
     * Number of rows on data.
     * @var integer
     */
    var $row_qty = CLX_INIT_ZERO;
    //Button data
    var $has_modify = true;
    var $has_display = true;
    var $has_create = true;
    var $has_delete = true;
    var $has_duplicate = true;
    var $has_export = true;
    var $has_filter = true;
    var $has_reorder = false;
    var $has_return_button = false;
    var $has_close_button = false;
    var $to = '';

    // CONSTRUCTOR //
    function __construct($toolcode, &$data) {
        //from parent
        $this->setCode('CLXLIST');
        $this->setName('Listing');
        $this->setType('LIST');
        $this->setVersion('1.0.0.0');
        $this->setDescrition('Clx Listing Class');
        $this->toolcode = $toolcode;
        $this->data = $data;
        $this->to = $this->toolcode;
        if (isset($this->data['rpp'])) {
		  $this->rpp = $this->data['rpp'];
        }
        if (isset($this->data['name'])) {
		  $this->tablename = $this->data['name'];
        }
        if (isset($this->data['tablename'])) {
		  $this->tablename = $this->data['tablename'];
        }
        if (isset($this->data['selectclause'])) {
		  $this->selectclause = $this->data['selectclause'];
        }
        if (isset($this->data['whereclause'])) {
		  $this->whereclause = $this->data['whereclause'];
        }
        if (isset($this->data['sortclause'])) {
		  $this->sortclause = $this->data['sortclause'];
        }
        $buttons = array('has_modify', 'has_display', 'has_create', 'has_delete', 'has_duplicate', 'has_export', 'has_filter', 'has_reorder');
        $total = count($buttons);
        for ($i = CLX_INIT_ZERO; $i < $total; $i++) {
		  $name = $buttons[$i];
		  if (isset($data[$name])) {
			 $this->$name = $data[$name];
		  }
        }
    }

    public function getButtons() {
        return $this->buttons;
    }

    function loadArray($data) {
        $this->title = (!empty($data[CLX_CLASS_TITLE])) ? $data[CLX_CLASS_TITLE] : '';
        $this->subtitle = (!empty($data[CLX_CLASS_SUBTITLE])) ? $data[CLX_CLASS_SUBTITLE] : '';
        $this->description = (!empty($data['description'])) ? $data['description'] : '';
        $this->panels = array();
        if (isset($data['button_labels'])) {
		  $this->button_labels = $data['button_labels'];
        }
        if (empty($data['panels'])) {
		  $data['panels'] = array();
        }
        foreach ($data['panels'] as $paneldata) {
		  $paneltype = 'clx_form_panel_' . '<p class=' . singleQuote('alert') . '>' . $paneldata['type'];
		  $this->panels[] = new $paneltype($this, $paneldata);
        }
    }

    function setArgument($key, $value) {
        $this->arguments[$key] = $value;
    }

    function getArgument($key) {
        return $this->arguments[$key];
    }

    /**
     * Bulds listin
     * Enter description here ...
     * @param bool $listing when true will make a list otherwise panel
     */
    function makeList($listing) {
        $contents = '';
        $tool = &getTool($this->toolcode);

        $here = 'tool.php?to=' . $this->toolcode . '&t=' . time();
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        if ($this->ispanel === true) {
		  $form->setProp(CLX_PROP_ID, $this->toolcode . 'Panel');
        } else {
		  $form->setProp(CLX_PROP_ID, $this->toolcode . 'List');
        }
        $form->addArgument(CLX_PROP_TO, $this->to);
        $form->addArgument('ac', 'NOT_DEFINED');
        $selectclause = $this->selectclause;
        $fromclause = $this->tablename;
        $whereclause = $this->whereclause;
        $sortclause = '';
        $comma = '';
        if (empty($listing) && !empty($tool->toolpanelkeys)) {
		  $counter = CLX_INIT_ZERO;
		  foreach ($tool->toolpanelkeys as $key => $val) {
			 if ($val['type'] === 'intn' || $val['type'] === 'bool' || $val['type'] === 'decn') {
			     if (empty($this->panel_arguments[$key]))
			         $this->panel_arguments[$key] = CLX_INIT_ZERO;
			     $whereclause .= $comma . $val['name'] . '=' . $this->panel_arguments[$key];
			 }else {
			     $whereclause .= $comma . $val['name'] . '=' . singleQuote($this->panel_arguments[$key]);
			     if ($counter < count($tool->toolpanelkeys) - CLX_NUM_ONE) {
			         $whereclause .= ' AND ';
			     }
			 }
			 $counter++;
		  }
        }
        $comma = '';
        if (!empty($this->data['sortclause'])) {
		  $sortclause .= $comma . $this->data['sortclause'];
		  $comma = ',';
        } else {
		  if (!empty($tool->sortclause)) {
			 $sortclause .= $comma . $tool->sortclause;
			 $comma = ',';
		  } else {
			 if (!empty($tool->seqnum_field)) {
			     $sortclause .= $comma . $tool->seqnum_field;
			     $comma = ',';
			 }
			 if (!empty($tool->primary_field)) {
			     $sortclause .= $comma . $tool->primary_field;
			     $comma = ',';
			 }
			 if (!empty($tool->secondary_field)) {
			     $sortclause .= $comma . $tool->secondary_field;
			     $comma = ',';
			 }
		  }
        }
        $page = new xPaging();
        if (!empty($this->rpp)) {
		  $page->setProp('rpp', $this->rpp);
        }
        $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
        $page->setProp('singular', dict($tool->record_name_singular));
        $page->setProp('plural', dict($tool->record_name_plural));

        $table = new htmlTable(CLX_WIDTH_ONE);
        $table->setProps(array(CLX_PROP_CLASS => 'gen_title'));

        if (empty($this->ispanel) && !empty($this->data[CLX_CLASS_TITLE])) {
		  $table->addCell(dict($this->data[CLX_CLASS_TITLE]), array(CLX_PROP_CLASS => CLX_CLASS_TITLE));
        }
        if (!empty($this->data[CLX_CLASS_SUBTITLE])) {
		  $table->addCell($this->data[CLX_CLASS_SUBTITLE], array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE));
        }
        if (!empty($this->data['caption'])) {
		  $table->addCell(dict($this->data['caption']), array(CLX_PROP_CLASS => 'caption'));
        }
        $contents .= $table->source();
        $tableWidth = count($this->data['fields']);

        $table = new htmlTable($tableWidth);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));

        $table = new htmlTable($tableWidth);
        $table->setProps(array(CLX_PROP_CLASS => 'gen_title'));

        $table = new htmlTable($tableWidth);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => '90%'));

        $table = new htmlTable(CLX_WIDTH_ONE);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => '90%'));

        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));

        $table = new htmlTable($tableWidth);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));


        $page->header($table);
        $table->addCell(dict('GEN_LBL_SEL'), array(CLX_PROP_CLASS => CLX_CLASS_HEADERS));
        foreach ($this->data['fields'] as $val) {
		  if ($val['name'] === 'ID') {
			 continue;
		  }
		  $table->addCell(dict((isset($val['label'])) ? $val['label'] : 'GEN_LBL_' . $val['name']), array(CLX_PROP_CLASS => CLX_CLASS_HEADERS));
        }
        $i = CLX_INIT_ZERO;
        while ($record = $page->nextRecord()) {
		  $class = ($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT;
		  $recordid = $record['ID'];
		  $select = new htmlCheckbox('selector_' . $recordid, false);
		  $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
		  $table->addCell($select->source(), array(CLX_PROP_CLASS => $class, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
		  foreach ($this->data['fields'] as $field) {
			 if ($field['name'] === 'ID') {
			     continue;
			 }
			 $type = 'clx_field_' . $field['type'];
			 $fl = new $type($form, $field);
			 $table->addCell($fl->viewList($field, $record), array(CLX_PROP_CLASS => $class));
		  }
		  $i++;
        }
        $this->row_qty = $page->getProp('qty');
        $formidprop = $form->getProp(CLX_PROP_ID);
        $buttons = $this->listingButtons($formidprop);
        $form->addContents($page->footers($table, $buttons));
        $contents .= $form->source();
        if ($this->ispanel === true) {
		  $contents = buildPanelInterface(dict($this->data[CLX_CLASS_TITLE]), $tool->code, $contents, CLX_NUM_MINUS_1);
        }
        return $contents;
    }

    /**
     * Construct listing buttons
     *
     * @param string $form_id
     * @param array $access
     * @return string
     */
    function listingButtons(&$form_id) {
        $tool = &getTool($this->toolcode);
        if (!empty($this->debug)) { pre('listingButtons('.$form_id.')'); pre($tool->getAccess()); pre('useMenuSecurity = ' . $tool->useMenuSecurity); }
        //$access = array('canread' => $tool->getAccess()->getCanread(), 'DISPLAY' => $tool->getAccess()->getDisplay(), 'canmodify' => $tool->getAccess()->getCanmodify(), 'candelete' => $tool->getAccess()->getCandelete());
        $access = array('canread' => $tool->getAccess()->getCanread(), 'DISPLAY' => $tool->getAccess()->getDisplay(), 'canmodify' => $tool->getAccess()->getCanmodify(), 'candelete' => $tool->getAccess()->getCandelete(), 'canappend' => $tool->getAccess()->getCanAppend());
        $fixed = array(
		  array('access' => 'canread', 'tool' => 'has_display', 'library' => 'GEN_LBL_DISPLAY', 'name' => 'DISPLAY', 'prop' => 'MULT'),
		  //array('access' => 'DISPLAY', 'tool' => 'has_create', 'library' => 'GEN_LBL_CREATE', 'name' => 'CREATE', 'prop' => 'ON'),
		  array('access' => (empty($tool->useMenuSecurity) ? 'DISPLAY' : 'canappend'), 'tool' => 'has_create', 'library' => 'GEN_LBL_CREATE', 'name' => 'CREATE', 'prop' => 'ON'),
		  array('access' => 'canmodify', 'tool' => 'has_modify', 'library' => 'GEN_LBL_MODIFY', 'name' => 'MODIFY', 'prop' => 'MULT'),
		  array('access' => 'candelete', 'tool' => 'has_delete', 'library' => 'GEN_LBL_DELETE', 'name' => 'DELETE', 'prop' => 'MULT'),
		  //array('access' => 'DISPLAY', 'tool' => 'has_duplicate', 'library' => 'GEN_LBL_DUPLICATE', 'name' => 'DUPLICATE', 'prop' => 'ONE'),
		  array('access' => (empty($tool->useMenuSecurity) ? 'DISPLAY' : 'canappend'), 'tool' => 'has_duplicate', 'library' => 'GEN_LBL_DUPLICATE', 'name' => 'DUPLICATE', 'prop' => 'ONE'),
		  array('access' => 'canread', 'tool' => 'has_export', 'library' => 'GEN_LBL_EXPORT', 'name' => 'EXPORT', 'prop' => 'ON'),
		  array('access' => 'canread', 'tool' => 'has_filter', 'library' => 'GEN_LBL_FILTER', 'name' => 'FILTER', 'prop' => 'ON'),
		  array('access' => 'canmodify', 'tool' => 'has_reorder', 'library' => 'GEN_LBL_REORDER', 'name' => 'REORDER', 'prop' => 'ON', 'minrowqty' => CLX_NUM_TWO)
        );
        if (!empty($this->data['buttons'])) {
		  $indexes = array_merge($fixed, $this->data['buttons']);
        } else {
		  $indexes = $fixed;
        }
        $buttons = '';
        $total = count($indexes);
        for ($i = CLX_INIT_ZERO; $i < $total; $i++) {
		  //not show if qty is less than required
		  if (!empty($indexes[$i]['minrowqty']) && $indexes[$i]['minrowqty'] > $this->row_qty || empty($indexes[$i]['tool']) && $indexes[$i]['tool'] <> '') {
			 continue;
		  }
		  //if (!empty($access[$indexes[$i]['access']])) { // [2016-04-05]
		  if (!empty($this->debug)) { pre('$indexes[$i][access] = ' . $indexes[$i]['access'] . ' | $access[$indexes[$i][access]] Value: ' . $access[$indexes[$i]['access']]); }
		  if ($access[$indexes[$i]['access']] === CLX_NUM_ONE) {
			 if (isset($indexes[$i]['window'])) {
			     switch ($indexes[$i]['window']) {
			         case 'POP':
					   UserRequest::$request['idlist'] = toString(getIdlist());
					   $buttonid = $indexes[$i]['name'];
					   $window = 'pop.php';
					   $url = $window . '?to=' . $tool->code . '&ac=' . $buttonid . '&wind=1';
					   $title = CLX_CHAR_EMPTY_STRING;
					   break;
			         default:
					   break;
			     }

			     $b = new htmlButton(dict($indexes[$i]['library']));
			     $b->setProp('onclick', 'getIdlistOpenWindow(' . singleQuote($title) . ',' . singleQuote($url) . ', 640, 480)');
			     $b->setProp(CLX_PROP_ID, $indexes[$i]['prop']);
                    $buttons .= '<!--'. __FILE__ .'_'. __LINE__ .'-->'.$b->source();

			 } else {
			     $b = new htmlFormSubmit(dict($indexes[$i]['library']), $indexes[$i]['name'], $form_id);
			     $b->setProp(CLX_PROP_ID, $indexes[$i]['prop']);
			     $buttons .= $b->source();
			 }
		  }
        }
        if ($this->has_return_button === true) {
		  $b = new htmlButtonCancel ( );
		  $buttons .= $b->source();
        }
        if ($this->has_close_button === true) {
		  $b = new htmlButtonCloseWindow(dict('GEN_LBL_CLOSE'));
		  $buttons .= $b->source();
        }
        return $buttons;
    }

}
