<?php

/**
 * General Code File
 * For more information contact  support@alenet.com
 *
 * @author       $Author: fsantana $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2015-09-23 08:07:17 -0400 (Wed, 23 Sep 2015) $
 * @version        $Rev: 6960 $
 *
 * CHANGE LOG
 *  Included svn prop
 */
class Crcl {

    function __construct($procParameters = array()) {
        multiTableName($this, array('CRCL', 'PROD', 'PRAT', 'PRLT', 'FICO', 'JEHD', 'JELN', 'PROD', 'GDVW', 'COMP', 'GLCP'));
        $this->batch_id = (!empty($procParameters['BATCH_ID'])) ? $procParameters['BATCH_ID'] : batch_id();
        $this->username = $_SESSION['user_record']['username'];
        $this->now = xFormat::getNow('Y-m-d H:i:s');
        $this->je_source = '';
        $this->debug = false;
    }

    function init_form(&$record) {
        if (UserRequest::$request['ac'] === 'DUPLICATE_EFFTDATE' || UserRequest::$request['ac'] === 'CREATE') {
            $record['CRCL_CLX_EFFTDATE'] = ClxCommon::$db->getField($this->glcpname, 'OPERATION_DATE', 'COMP_CODE = ' . singleQuote($record['CRCL_COMP_CODE']));
            $record['CRCL_CRCL_OP_STATUS'] = '';
            $record['CRCL_MEMO'] = '';
            $record['CRCL_CLX_AUTH_STATUS'] = 'PEND';
        }
    }

    /**
     * Validate record information
     * Executed on table validate
     *
     * @param array $record
     * @param array $msg
     */
    function validate_form(&$record, &$msg) {
        if ($record['CRCL_END_DATE'] <> '' && $record['CRCL_CLX_EFFTDATE'] > $record['CRCL_END_DATE']) {
            $msg['CRCL_END_DATE'] = smsg('MSG-205');
        }
        if ($record['CRCL_END_DATE'] <> '' && $record['CRCL_END_DATE'] <= $record['CRCL_START_DATE']) {
            $msg['CRCL_END_DATE'] = smsg('MSG-206');
        }
        $sql = 'SELECT COUNT(ID) FROM ' . $this->prltname . ' WHERE PRODUCT_CODE = ' . singleQuote($record['CRCL_PRODUCT_CODE']) . ' AND CHART_FIELD01 IS NOT null';
        $qty = ClxCommon::$db->getOne($sql);
        if (empty($record['CRCL_CHART_FIELD01']) && $qty == CLX_NUM_ZERO) {
            $msg['CRCL_CHART_FIELD01'] = smsg('MSG-171');
        }
        $sql = 'SELECT COUNT(ID) FROM ' . $this->prltname . ' WHERE PRODUCT_CODE = ' . singleQuote($record['CRCL_PRODUCT_CODE']) . ' AND CHART_FIELD02 IS NOT null';
        $qty = ClxCommon::$db->getOne($sql);
        if (empty($record['CRCL_CHART_FIELD02']) && $qty == CLX_NUM_ZERO) {
            $msg['CRCL_CHART_FIELD02'] = smsg('MSG-172');
        }
        //Validate contract vs company should match
        $sql = 'SELECT COUNT(COMP_CODE) FROM ' . $this->ficoname . ' WHERE IDCODE = ' . singleQuote($record['CRCL_CHART_FIELD01']) . ' AND COMP_CODE = ' . singleQuote($record['CRCL_COMP_CODE']);
        $qty = ClxCommon::$db->getOne($sql);
        if ($qty == CLX_NUM_ZERO) {
            $sql = 'SELECT COUNT(PRLT.ID) FROM ' . $this->prltname . ' PRLT, ' . $this->ficoname . ' FICO WHERE PRLT.PRODUCT_CODE = ' . singleQuote($record['CRCL_PRODUCT_CODE']) . ' AND PRLT.CHART_FIELD01 IS NOT null AND FICO.IDCODE = PRLT.CHART_FIELD01 AND FICO.COMP_CODE = ' . singleQuote($record['CRCL_COMP_CODE']);
            $qty += ClxCommon::$db->getOne($sql);
        }
        if ($qty == CLX_NUM_ZERO) {
            $msg['CRCL_COMP_CODE'] = smsg('MSG-201', $record['CRCL_COMP_CODE'] . '.' . $record['CRCL_CHART_FIELD01']);
        }
        if (UserRequest::$request['ac'] === 'REQAUTHORIZATION') {
            $sql = 'SELECT OPERATION_DATE FROM ' . $this->glcpname . ' WHERE COMP_CODE = ' . singleQuote($record['CRCL_COMP_CODE']);
            $operation_date = ClxCommon::$db->getOne($sql);
            if ($operation_date <> $record['CRCL_START_DATE']) {
                $msg['CRCL_START_DATE'] = smsg('MSG-209', $operation_date);
            }
        }
        //Validar Crédito sea Estatus Activo
        $sql = 'SELECT STATUS FROM ' . $this->prodname . ' WHERE IDCODE =' . singleQuote($record['CRCL_PRODUCT_CODE_prim']);
        $statusProducto = ClxCommon::$db->getOne($sql);
        if ($statusProducto === 'I') {
            $msg['CRCL_PRODUCT_CODE'] = smsg('MSG-405');
        }
    }

    /**
     * Save form event
     *
     * @param array $record
     */
    function save_form(&$record) {
        if (empty($record['CHART_FIELD01']) || $record['CHART_FIELD01'] === 'null') {
            $sql = 'SELECT CHART_FIELD01 FROM ' . $this->prltname . ' WHERE PRODUCT_CODE = ' . singleQuote($record['PRODUCT_CODE']) . ' AND CHART_FIELD01 IS NOT null';
            $record['CHART_FIELD01'] = ClxCommon::$db->getOne($sql);
        }
        if (empty($record['CHART_FIELD02']) || $record['CHART_FIELD02'] === 'null') {
            $sql = 'SELECT CHART_FIELD02 FROM ' . $this->prltname . ' WHERE PRODUCT_CODE = ' . singleQuote($record['PRODUCT_CODE']) . ' AND CHART_FIELD02 IS NOT null';
            $record['CHART_FIELD02'] = ClxCommon::$db->getOne($sql);
        }
        if ($record['START_DATE'] === '') {
            $record['START_DATE'] = $record['CLX_EFFTDATE'];
        }
        if ($record['CRCL_OP_STATUS'] === 'TER') {
            $record['CR_LINE_STATUS'] = 'CAN';
        }
    }

    /**
     * Post Authorization Method
     *
     * @param unknown_type $record
     */
    function post_autorize(&$record) {
        if (UserRequest::$request['ac'] === 'DELETE')
            return;
        if (array_key_exists('CRCL_PRODUCT_CODE', $record)) {
            $prefix = 'CRCL_';
        } else {
            $prefix = '';
        }
        if ($this->buildAccounting($record)) {
            if (!class_exists('Fipa'))
                includeCodeLib('FIPA');
            $fipa = new Fipa();
            $fipa->build($record[$prefix . 'CHART_FIELD01'], $record[$prefix . 'ACCOUNTCODE'], 'DB');
        }
    }

    /**
     * Buld accounting on creation
     *
     * @param array $crcl_record
     * @return boolean
     */
    function buildAccounting($record) {
        $contenidoHTML = '';
        if (array_key_exists('CRCL_PRODUCT_CODE', $record)) {
            $prefix = 'CRCL_';
        } else {
            $prefix = '';
        }
        $crlncode = $record[$prefix . 'IDCODE'];
        $comp_code = $record[$prefix . 'COMP_CODE'];
        $product_code = $record[$prefix . 'PRODUCT_CODE'];
        $contract_code = $record[$prefix . 'CHART_FIELD01'];
        $participant_id = $record[$prefix . 'ACCOUNTCODE'];
        $currency_code = $record[$prefix . 'CURRENCY_CODE'];
        $debtor_code = $record[$prefix . 'ACCOUNTCODE'];
        $line_amt = $record[$prefix . 'LINE_AMT'];
        $sql = 'SELECT PRAT.ORDER_GL_CODE, PRAT.ORDER2_GL_CODE FROM ' . $this->pratname . ' PRAT WHERE PRAT.PRODUCT_CODE = ' . singleQuote($product_code) . ' AND PRAT.PROD_STATUS = ' . singleQuote('VIG') . '';
        $prat = ClxCommon::$db->getOneRecord($sql);
        if (empty($prat['ORDER_GL_CODE']) || empty($prat['ORDER2_GL_CODE'])) {
            return false;
        }
        $this->je_source = 'CRCL';
        $DBLT = &getTool('DBLT');
        $idname = $DBLT->getListItem('SOTY', $this->je_source);
        if (!class_exists('Fico'))
            includeCodeLib('Fico');
        $fico = new Fico();
        $oper = $fico->fico_comp_operation_parameters($contract_code);

        $efftdate = $oper['OPERATION_DATE'];
        //Sumar monto de cuentas contables para ver si suma a monto de linea
        //Verificar diferencia con monto en linea para ajustar
        $dif_amt = $this->credLineDif($comp_code, $contract_code, $debtor_code);
        if (!empty($this->debug)) {
            $contenidoHTML.= 'Diferencia encontrada :' . $dif_amt . '<Br>';
        }
        if ($dif_amt == CLX_NUM_ZERO) {
            //Already accounted just return
            echo $contenidoHTML;
            return false;
        }
        //Try to search any pending journal for current operation
        $sql = 'SELECT MAX(JEHD.IDCODE) FROM ' . $this->jehdname . ' JEHD, ' . $this->jelnname . ' JELN WHERE JEHD.IDCODE = JELN.JEHDR_CODE AND JEHD.COMP_CODE = ' . singleQuote($comp_code) . ' AND JEHD.JE_SOURCE = ' . singleQuote($this->je_source) . ' AND JEHD.JE_STATUS = ' . singleQuote('PEND') . ' AND JELN.CHART_FIELD01= ' . singleQuote($contract_code) . ' AND JELN.CHART_FIELD02= ' . singleQuote($debtor_code);
        $jehdr_code = (string) ClxCommon::$db->getOne($sql);
        if (!class_exists('Jehd'))
            includeCodeLib('Jehd');
        if (!class_exists('Comp'))
            includeCodeLib('COMP');
        //Get Product Currency

        $comp = new Comp();
        $rate = $comp->comp_current_exchange($oper['IDCODE'], $currency_code, $oper['OPERATION_DATE']);
        $jehd = new Jehd();
        if ($jehdr_code === '') {
            $gl_rec = $jehd->jehd_init_record();
            $gl_rec['IDNAME'] = $idname;
            $gl_rec['JE_SOURCE'] = $this->je_source;
            $gl_rec['LGGROUP_CODE'] = $oper['LGGROUP_CODE'];
            $gl_rec['EFFTDATE'] = $efftdate;
            $gl_rec['REAL_DATE'] = $efftdate;

            $gl_rec['JE_STATUS'] = 'PEND';
            $gl_rec['CLX_AUTH_STATUS'] = 'PEND';
            $gl_rec['COMP_CODE'] = $oper['IDCODE'];
            $gl_rec['FISCAL_PERIOD'] = $oper['FISCAL_PERIOD'];
            $gl_rec['TRANS_CODE'] = 'null';
            $gl_rec['TRANSTBL_CODE'] = '';
            $gl_rec['REQ_CHANNEL'] = 'SYS';
            $gl_rec['BATCH_ID'] = $this->batch_id;
            $jehd->jehd_build($gl_rec);
            $jehdr_code = $gl_rec['IDCODE'];
        } else {
            $gl_rec = array();
            $gl_rec['FISCAL_PERIOD'] = $oper['FISCAL_PERIOD'];
            $gl_rec['EFFTDATE'] = $efftdate;
            $gl_rec['REAL_DATE'] = $efftdate;

            $gl_rec['LGGROUP_CODE'] = $oper['LGGROUP_CODE'];
            $gl_rec['BATCH_ID'] = $this->batch_id;
            ClxCommon::$db->updateRecord($this->jehdname, $gl_rec, 'IDCODE = ' . singleQuote($jehdr_code));
        }
        if (!empty($this->debug)) {
            $contenidoHTML.= 'Modificando Asiento:' . $jehdr_code . '<br>';
        }
        //Search any scale if apply and dump lines to journal
        $line = array('JEHDR_CODE' => $jehdr_code,
            'GLACCOUNTCODE' => $prat['ORDER_GL_CODE'],
            'ITEM_CODE' => 'null',
            'CHART_FIELD01' => $contract_code,
            'CHART_FIELD02' => $participant_id,
            'CHART_FIELD03' => 'null',
            'CHART_FIELD04' => 'null',
            'ITEM_REF_NO' => $crlncode,
            'CURRENCY_CODE' => $rate['FOREIGN_CURRENCY'],
            'EXCHANGE_RT' => $rate['EXCHANGE_RT'],
            'CUR_OPERATOR' => $rate['CUR_OPERATOR'],
            'DEBIT_AMT' => CLX_NUM_ZERO,
            'CREDIT_AMT' => CLX_NUM_ZERO,
            'DEBIT_TRANS_AMT' => CLX_NUM_ZERO,
            'CREDIT_TRANS_AMT' => CLX_NUM_ZERO,
            'SEQNUM' => CLX_NUM_ONE);
        if ($dif_amt > CLX_NUM_ZERO) {
            $line['DEBIT_AMT'] = $dif_amt;
        } elseif ($dif_amt < CLX_NUM_ZERO) {
            $line['CREDIT_AMT'] = bcabs($dif_amt);
        }
        if (!empty($this->debug)) {
            $contenidoHTML.= 'Linea 1: ' . $line['GLACCOUNTCODE'] . ' dr-cr: ' .
                    ($line['DEBIT_AMT'] - $line['CREDIT_AMT']) . '<br>';
        }
        //Conversion
        $line['DEBIT_TRANS_AMT'] = $line['DEBIT_AMT'];
        $line['CREDIT_TRANS_AMT'] = $line['CREDIT_AMT'];
        if ($line['DEBIT_AMT'] > CLX_NUM_ZERO) {
            $line['DEBIT_AMT'] = round($line['DEBIT_AMT'] * pow($line['EXCHANGE_RT'], $line['CUR_OPERATOR']), CLX_NUM_TWO);
        }
        if ($line['CREDIT_AMT'] > CLX_NUM_ZERO) {
            $line['CREDIT_AMT'] = round($line['CREDIT_AMT'] * pow($line['EXCHANGE_RT'], $line['CUR_OPERATOR']), CLX_NUM_TWO);
        }
        $qty = ClxCommon::$db->oneExistsRecords($this->jelnname, 'JEHDR_CODE = ' . singleQuote($jehdr_code) . ' AND SEQNUM=1');
        if ($qty == CLX_NUM_ZERO) {
            ClxCommon::$db->createRecord($this->jelnname, $line, false);
        } else {
            ClxCommon::$db->updateRecord($this->jelnname, $line, 'JEHDR_CODE = ' . singleQuote($jehdr_code) . ' AND SEQNUM=1');
        }
        //contraparte
        $line = array('JEHDR_CODE' => $jehdr_code,
            'GLACCOUNTCODE' => $prat['ORDER2_GL_CODE'],
            'ITEM_CODE' => 'null',
            'CHART_FIELD01' => $contract_code,
            'CHART_FIELD02' => $participant_id,
            'CHART_FIELD03' => 'null',
            'CHART_FIELD04' => 'null',
            'ITEM_REF_NO' => $crlncode,
            'CURRENCY_CODE' => $rate['FOREIGN_CURRENCY'],
            'EXCHANGE_RT' => $rate['EXCHANGE_RT'],
            'CUR_OPERATOR' => $rate['CUR_OPERATOR'],
            'DEBIT_AMT' => CLX_NUM_ZERO,
            'CREDIT_AMT' => CLX_NUM_ZERO,
            'DEBIT_TRANS_AMT' => CLX_NUM_ZERO,
            'CREDIT_TRANS_AMT' => CLX_NUM_ZERO,
            'SEQNUM' => CLX_NUM_TWO);
        if ($dif_amt > CLX_NUM_ZERO) {
            $line['CREDIT_AMT'] = $dif_amt;
        } elseif ($dif_amt < CLX_NUM_ZERO) {
            $line['DEBIT_AMT'] = bcabs($dif_amt);
        }
        if (!empty($this->debug)) {
            $contenidoHTML.= 'Linea 2: ' . $line['GLACCOUNTCODE'] . ' dr-cr: ' .
                    ($line['DEBIT_AMT'] - $line['CREDIT_AMT']) . '<br>';
        }
        //Perform conversion
        $line['DEBIT_TRANS_AMT'] = $line['DEBIT_AMT'];
        $line['CREDIT_TRANS_AMT'] = $line['CREDIT_AMT'];
        if ($line['DEBIT_AMT'] > CLX_NUM_ZERO) {
            $line['DEBIT_AMT'] = round($line['DEBIT_AMT'] * pow($line['EXCHANGE_RT'], $line['CUR_OPERATOR']), CLX_NUM_TWO);
        }
        if ($line['CREDIT_AMT'] > CLX_NUM_ZERO) {
            $line['CREDIT_AMT'] = round($line['CREDIT_AMT'] * pow($line['EXCHANGE_RT'], $line['CUR_OPERATOR']), CLX_NUM_TWO);
        }
        $qty = ClxCommon::$db->oneExistsRecords($this->jelnname, 'JEHDR_CODE = ' . singleQuote($jehdr_code) . ' AND SEQNUM=2');
        if ($qty == CLX_NUM_ZERO) {
            ClxCommon::$db->createRecord($this->jelnname, $line, false);
        } else {
            ClxCommon::$db->updateRecord($this->jelnname, $line, 'JEHDR_CODE = ' . singleQuote($jehdr_code) . ' AND SEQNUM=2');
        }
        if (!empty($this->debug)) {
            $contenidoHTML.= 'Autorizando asiento:' . $jehdr_code . '<br>';
        }
        $jehd->jehd_approve_queue_by_code($jehdr_code);
        unset($comp, $line, $line_amt, $gl_rec, $jehd, $dif_amt, $currency_code, $contract_code, $product_code, $participant_id);
        echo $contenidoHTML;
        return (!empty($jehdr_code)) ? true : false;
    }

    private function credLineDif($comp_code, $contract_code, $debtor_code) {
        $sql = 'SELECT (SUM(CRCL.LINE_AMT)- IFNULL((SELECT SUM(GDVW.DEBIT_TRANS_AMT-GDVW.CREDIT_TRANS_AMT) FROM GL_JOURNAL_DET_VW GDVW WHERE GDVW.GLACCOUNTCODE = PRAT.ORDER_GL_CODE AND GDVW.COMP_CODE = CRCL.COMP_CODE AND GDVW.CHART_FIELD01 = CRCL.CHART_FIELD01 AND GDVW.CHART_FIELD02 = CRCL.ACCOUNTCODE AND GDVW.CURRENCY_CODE = PROD.CURRENCY_CODE AND GDVW.JE_SOURCE = ' . singleQuote('CRCL') . '),0)) BALANCE_AMT FROM ' . $this->crclname . ' CRCL, ' . $this->pratname . ' PRAT, ' . $this->prodname . ' PROD WHERE PRAT.PRODUCT_CODE = CRCL.PRODUCT_CODE AND PRAT.PROD_STATUS = ' . singleQuote('VIG') . ' AND PROD.IDCODE = PRAT.PRODUCT_CODE AND CRCL.COMP_CODE = ' . singleQuote($comp_code) . ' AND CRCL.CHART_FIELD01= ' . singleQuote($contract_code) . ' AND CRCL.ACCOUNTCODE = ' . singleQuote($debtor_code) . ' AND CRCL.CLX_EFFTDATE = (SELECT MAX(X.CLX_EFFTDATE) FROM ' . $this->crclname . ' X WHERE X.IDCODE = CRCL.IDCODE)';
        return ClxCommon::$db->getOne($sql);
    }

}
