<?php

/**
 * General Code File
 * For more information contact  support@alenet.com
 *
 * @author       $Author: fsantana $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2016-10-27 16:01:43 -0400 (Thu, 27 Oct 2016) $
 * @version        $Rev: 8853 $
 *
 * CHANGE LOG
 *  Included svn prop
 */
class Fipa {

    /**
     * Contructor
     *
     * @param array $procParameters
     * @return void
     */
    function __construct($procParameters = array()) {
        multiTableName($this, array('FIPA', 'FICO', 'JELN', 'JEHD', 'COMP', 'GLCP'));
        $this->batch_id = (!empty($procParameters['BATCH_ID'])) ? $procParameters['BATCH_ID'] : batch_id();
        $this->username = $_SESSION['user_record']['username'];
        $this->now = xFormat::getNow('Y-m-d H:i:s');
    }

    /**
     * Initialice fipa form
     * Executed on init event
     */
    function fipa_init_form(&$record) {
        $record['FIPA_CAPITALIZATION_PCT'] = CLX_NUM_HUNDRED;
        if (!class_exists('Fico'))
            includeCodeLib('FICO');
        $fico = new Fico();
        $comprec = $fico->fico_comp_operation_parameters($record['FIPA_CONTRACT_CODE']);
        $ficostart = ClxCommon::$db->getField($this->ficoname, 'START_DATE', 'IDCODE = ' . singleQuote($record['FIPA_CONTRACT_CODE']));
        $record['FIPA_START_DATE'] = ($ficostart > $comprec['OPERATION_DATE']) ? $ficostart : $comprec['OPERATION_DATE'];
        $record['FIPA_END_DATE'] = '';
        $record['FIPA_STATUS'] = 'I';
    }

    /**
     * Validate record information
     * Executed on table validate
     */
    function fipa_validate_form(&$record, &$msg) {
        //prevent delete if accounting exists
        $ficorec = ClxCommon::$db->getFields($this->ficoname, 'START_DATE, CONTRACT_STATUS, CANCEL_DATE, EXPIRATION_DATE, IS_COST_CENTER', 'IDCODE = ' . singleQuote($record['FIPA_CONTRACT_CODE']));
        if (UserRequest::$request['ac'] === 'DELETE') {
            //validar situacion de transaccion pendiente o no
            if ($ficorec['IS_COST_CENTER']==='0' && $record['FIPA_CLX_AUTH_STATUS'] <> 'PEND') {
                $msg['FIPA_CLX_AUTH_STATUS'] = smsg('MSG-53', $record['FIPA_IDCODE']);
                addMessage($msg['FIPA_CLX_AUTH_STATUS']);
                return false;
            }

            $sql = 'SELECT COUNT(JEHD.ID) QTY FROM ' . $this->jehdname . ' JEHD, ' . $this->jelnname . ' JELN WHERE JEHD.IDCODE = JELN.JEHDR_CODE AND JEHD.JE_STATUS<>' . singleQuote('PEND') . ' AND JELN.CHART_FIELD01 = ' . singleQuote($record['FIPA_CONTRACT_CODE']) . ' AND JELN.CHART_FIELD02 = ' . singleQuote($record['FIPA_ACCOUNTCODE']);
            if ($ficorec['IS_COST_CENTER']==='0' && ClxCommon::$db->getOne($sql) > CLX_NUM_ZERO) {
                $msg['FIPA_CLX_AUTH_STATUS'] = smsg('MSG-53', $record['FIPA_IDCODE']);
                addMessage($msg['FIPA_CLX_AUTH_STATUS']);
                return false;
            }
            return true;
        }
        //Porcentaje de capitalizacion mas porcentaje de pago no puede superar el 100%
        if (($record['FIPA_CAPITALIZATION_PCT'] + $record['FIPA_CAPPAYMENT_PCT']) > CLX_NUM_HUNDRED) {
            /**
             * GEN_ERR_CAPPAYMUSTBEHUNDRED
             * Monto de capitalizacion mas pago no puede superar 100%
             * Capitalization amount + payment amount could not be greater than 100%
             */
            $msg['FIPA_CAPPAYMENT_PCT'] = smsg('MSG-54', $record['FIPA_IDCODE']);
            $msg['FIPA_CAPITALIZATION_PCT'] = $msg['FIPA_CAPPAYMENT_PCT'];
        }
        //Fecha de inicio no puede ser menor a la fecha de inicio del contrato
        $start_date = strtotime($record['FIPA_START_DATE']);
        $fico_start_date = strtotime($ficorec['START_DATE']);
        if ($start_date < $fico_start_date) {
            $msg['FIPA_START_DATE'] = smsg('MSG-55', $record['FIPA_IDCODE']);
        }
        if (!empty($record['FIPA_END_DATE'])) {
            $end_date = strtotime($record['FIPA_END_DATE']);
            if ($end_date < $start_date) {
                $msg['FIPA_END_DATE'] = smsg('MSG-56', $record['FIPA_IDCODE']);
            }
            //test that participant end date is not bigger than contract end date
            if (!empty($ficorec['CANCEL_DATE']) && $end_date > strtotime($ficorec['CANCEL_DATE'])) {
                $msg['FIPA_END_DATE'] = smsg('MSG-57', $record['FIPA_IDCODE']);
            }
        }
        if ($ficorec['CONTRACT_STATUS'] <> 'NEG' && $ficorec['CONTRACT_STATUS'] <> 'ACTI') {
            $msg['FIPA_CONTRACT_CODE'] = smsg('MSG-58', $record['FIPA_IDCODE']);
        }
    }

    /**
     * Events running post autorizations
     */
    function fipa_post_autorize(&$record, $reject = false) {
        if (!empty($reject)) {
            return false;
        }
        if (!array_key_exists('IDCODE', $record) && array_key_exists('FIPA_IDCODE', $record)) {
            $record['IDCODE'] = $record['FIPA_IDCODE'];
        }
        if(!empty($record['IDCODE'])){
            $sql = 'UPDATE ' . $this->fipaname . ' FIPA SET FIPA.STATUS = ' . singleQuote('A') . ' WHERE FIPA.IDCODE = ' . singleQuote($record['IDCODE']) . ' AND (FIPA.END_DATE IS NULL OR FIPA.END_DATE >= (SELECT MAX(GLCP.OPERATION_DATE) FROM GL_PARAMETERS_TBL GLCP, FIDU_CONTRACTS FICO WHERE FICO.COMP_CODE = GLCP.COMP_CODE AND FICO.IDCODE = FIPA.CONTRACT_CODE))';
            ClxCommon::$db->executeQuery($sql);
        }

        if (UserRequest::$request['ac'] === 'DELETE'){
            return false;
        }
    }

    /**
     * Update Parent (JEHD) Information one record is saved
     */
    function fipa_save_form(&$record) {

    }

    function fipa_settle_status($contract_code, $participant_id) {
        //test if balance is still allowed
        if ($this->fipa_can_inactivate_participant($contract_code, $participant_id)) {
            $sql = 'SELECT GLCP.OPERATION_DATE FROM FIDU_CONTRACTS FICO, GL_PARAMETERS_TBL GLCP WHERE GLCP.COMP_CODE = FICO.COMP_CODE AND FICO.IDCODE = ' . singleQuote($contract_code);
            $operation_date = ClxCommon::$db->getOne($sql);
            $sql = 'UPDATE ' . $this->fipaname . '  SET STATUS = ' . singleQuote('I') . ', FIPATERM_REASON = ' . singleQuote('CC') . ', END_DATE = ' . singleQuote($operation_date) . ' WHERE CONTRACT_CODE = ' . singleQuote($contract_code) . ' AND ACCOUNTCODE = ' . singleQuote($participant_id);
            ClxCommon::$db->executeQuery($sql);
            //check if contract has balances
            if ($this->fipa_can_inactivate_contract($contract_code)) {
                $sql = 'UPDATE ' . $this->fipaname . '  SET STATUS = ' . singleQuote('I') . ', FIPATERM_REASON = ' . singleQuote('CC') . ', END_DATE = ' . singleQuote($operation_date) . ' WHERE CONTRACT_CODE = ' . singleQuote($contract_code);
                ClxCommon::$db->executeQuery($sql);
                $sql = 'UPDATE ' . $this->ficoname . ' FICO SET FICO.CONTRACT_STATUS = ' . singleQuote('FINQ') . ', FICO.CANCEL_DATE = ' . singleQuote($operation_date) . ' WHERE FICO.IDCODE = ' . singleQuote($contract_code);
                ClxCommon::$db->executeQuery($sql);
            }
        }
    }

    /**
     * Return true if participant can be inactivated
     */
    function fipa_can_inactivate_contract($contract_code) {
        $sql = 'SELECT JELN.GLACCOUNTCODE, SUM(JELN.DEBIT_AMT - JELN.CREDIT_AMT) BALANCE_AMT FROM ' . $this->jehdname . ' JEHD, ' . $this->jelnname . ' JELN WHERE JEHD.IDCODE = JELN.JEHDR_CODE AND JEHD.JE_STATUS = ' . singleQuote('AP') . ' AND JELN.CHART_FIELD01 = ' . singleQuote($contract_code) . ' GROUP BY JELN.GLACCOUNTCODE HAVING SUM(JELN.DEBIT_AMT - JELN.CREDIT_AMT)<>0 LIMIT 1';
        $result = ClxCommon::$db->getOneRecord($sql);
        $status = (empty($result)) ? true : false;
        return $status;
    }

    /**
     * Return true if participant can be inactivated
     */
    function fipa_can_inactivate_participant($contract_code, $participant_id) {
        $sql = 'SELECT JELN.GLACCOUNTCODE, SUM(JELN.DEBIT_AMT - JELN.CREDIT_AMT) BALANCE_AMT FROM ' . $this->jehdname . ' JEHD, ' . $this->jelnname . ' JELN WHERE JEHD.IDCODE = JELN.JEHDR_CODE AND JEHD.JE_STATUS = ' . singleQuote('AP') . ' AND JELN.CHART_FIELD01 = ' . singleQuote($contract_code) . ' AND	JELN.CHART_FIELD02 = ' . singleQuote($participant_id) . ' GROUP BY JELN.GLACCOUNTCODE HAVING SUM(JELN.DEBIT_AMT - JELN.CREDIT_AMT)<>0 LIMIT 1';
        $result = ClxCommon::$db->getOneRecord($sql);
        $status = (empty($result)) ? true : false;
        return $status;
    }

    /**
     * Buils Participant Record
     *
     * @param string $contract_code
     * @param string $accountcode
     * @param string $participant_type
     * @param array $record
     * @return boolean
     */
    function build($contract_code, $accountcode, $participant_type, $record = array()) {
        if (empty($contract_code) || empty($accountcode))
            return false;
        $qty = ClxCommon::$db->oneExistsRecords($this->fipaname, 'CONTRACT_CODE = ' . singleQuote($contract_code) . ' AND ACCOUNTCODE = ' . singleQuote($accountcode));
        if ($qty > CLX_NUM_ZERO)
            return false;
        $rec = array();
        $sql = 'SELECT FIELDNAME, FIELD_DEFAULT FROM CLX_DB_TABLEFIELDS WHERE TABLECODE = ' . singleQuote('FIPA') . ' AND FIELDNAME NOT IN (' . singleQuote('ID') . ') ORDER BY SEQNUM ASC';
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
            if (array_key_exists($r['FIELDNAME'], $record)) {
                $rec[$r['FIELDNAME']] = $record[$r['FIELDNAME']];
            } else {
			if ($r['FIELDNAME'] === 'START_DATE') {
                    $sql = 'SELECT GLCP.OPERATION_DATE FROM ' . $this->glcpname . ' GLCP, ' . $this->ficoname . ' FICO WHERE FICO.COMP_CODE = GLCP.COMP_CODE AND FICO.IDCODE = ' . singleQuote($contract_code);
                    $rec['START_DATE'] = ClxCommon::$db->getOne($sql);
                } elseif ($r['FIELDNAME'] === 'END_DATE') {
                    $rec['END_DATE'] = 'null';
                } else {
                    $rec[$r['FIELDNAME']] = $r['FIELD_DEFAULT'];
                }
            }
        }
		ClxCommon::$db->freeResult($rs);
        $rec['IDCODE'] = 'AUTO';
        $rec['CONTRACT_CODE'] = $contract_code;
        $rec['ACCOUNTCODE'] = $accountcode;
        $rec['PARTICIPANT_TYPE'] = $participant_type;
        $rec['CLX_AUTH_STATUS'] = 'AP';
        $rec['STATUS'] = 'A';
        if (!function_exists('setCreationData'))
            includeCodeLib('GEN_REC_UTIL');
        setCreationData($rec);
        setCreationIdCode($rec, 'FIPA');
        ClxCommon::$db->createRecord($this->fipaname, $rec, false);
        return true;
    }

}
