<?php

/**
 * General Code File
 * For more information contact  support@alenet.com
 *
 * @author       $Author: fsantana $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2017-01-27 08:31:17 -0500 (Fri, 27 Jan 2017) $
 * @version        $Rev: 9065 $
 *
 * CHANGE LOG
 *  Included svn prop
 */
class Inas {

    /**
     * Contructor
     *
     * @param array $procParameters
     * @return void
     */
    function __construct($procParameters = array()) {
        multiTableName($this, array('INAS', 'INPR', 'GLCP'));
        $this->batch_id = (!empty($procParameters['BATCH_ID'])) ? $procParameters['BATCH_ID'] : batch_id();
        $this->username = $_SESSION['user_record']['username'];
        $this->now = xFormat::getNow('Y-m-d H:i:s');
        $this->today = xFormat::getNow('Y-m-d');
    }

    /**
     * Initialice inas form
     * Executed on init event
     */
    function inas_init_form(&$record) {
        if (UserRequest::$request['ac'] === 'CREATE') {
		  if (!function_exists('comp_operation_parameters'))
			 includeCodeLib('COMP');
		  $comp = new Comp();
		  $comprec = $comp->comp_operation_parameters();
		  $record['INAS_START_DATE'] = $comprec['OPERATION_DATE'];
		  unset($comp);
        }
    }

    /**
     * Post database commit
     * @param string $record
     */
    function save_form_post(&$record) {
        //Updating future coupons
        $sql = 'UPDATE ' . $this->inasname . ' SET COUPON_RT = ' . $record['COUPON_RT'] . ' WHERE INVESTPROD_CODE =' . singleQuote($record['INVESTPROD_CODE']) . ' AND EFFTDATE=' . singleQuote($record['EFFTDATE']) . ' AND START_DATE>' . singleQuote($record['START_DATE']);
        ClxCommon::$db->executeQuery($sql);
    }

    /**
     * Validate record information
     * Executed on table validate
     */
    function inas_validate_form(&$record, &$msg) {
        $record['INAS_START_DATE'] = xFormat::date($record['INAS_START_DATE'], 'Y-m-d');
        $sql = 'SELECT INPR.MATURITY_DATE, INPR.ISSUE_DATE, INPR.CALC_INTEREST, INPR.INTEREST_TYPE, INPR.PAY_DAY FROM ' . $this->inprname . ' INPR WHERE INPR.INVESTMENT_CODE =' . singleQuote($record['INAS_INVESTPROD_CODE']);
        $inprrec = ClxCommon::$db->getOneRecord($sql);
        if (empty($inprrec)) {
		  $msg['INAS_INVESTPROD_CODE'] = 'INAS001 ' . dict('CODE_ERR_INVALIDPRODUCTATDATE');
		  return;
        }
        if ($inprrec['MATURITY_DATE'] < $record['INAS_START_DATE']) {
		  //fecha de devengo no puede ser menor a la fecha actual
		  $msg['INAS_START_DATE'] = 'INAS002 ' . dict('CODE_ERR_DATEMUSTBEBETWEEN', $inprrec['MATURITY_DATE'], $inprrec['ISSUE_DATE']);
        }
        if ($inprrec['ISSUE_DATE'] > $record['INAS_START_DATE']) {
		  //fecha de devengo no puede ser menor a la fecha actual
		  $msg['INAS_START_DATE'] = 'INAS003 ' . dict('CODE_ERR_DATEMUSTBEBETWEEN', $inprrec['MATURITY_DATE'], $inprrec['ISSUE_DATE']);
        }
        if ($record['INAS_COUPON_RT'] <= CLX_NUM_ZERO) {
		  //fecha de devengo no puede ser menor a la fecha actual
		  $msg['INAS_COUPON_RT'] = 'INAS005 ' . dict('GEN_LBL_GREATERTHAN') . ' 0.00';
        }
    }

    /**
     * Cuponera
     */
    function build_payment_schedule($investment_code, $efftdate = '') {
        if ($efftdate === '')
		  $efftdate = $this->today;
        $sql = 'SELECT INPR.INVESTMENT_CODE, INPR.CLX_EFFTDATE EFFTDATE, GLCP.OPERATION_DATE, INPR.ISSUE_DATE, INPR.MATURITY_DATE, INPR.COUPON_RT, INPR.INTEREST_CALC, INPR.PAYMENT_FREQ, INPR.INT_PER_TYPE, INPR.PAY_DAY, INPR.FIRST_PYMNT_DATE, INPR.CDATE, INPR.MDATE, INPR.CREATEDBY, INPR.MODIFIEDBY, INPR.INTEREST_TYPE FROM  ' . $this->glcpname . ' GLCP, ' . $this->inprname . ' INPR WHERE  GLCP.COMP_CODE = INPR.COMP_CODE AND INPR.INVESTMENT_CODE = ' . singleQuote($investment_code) . '  AND INPR.ISSUE_DATE IS NOT null AND INPR.MATURITY_DATE IS NOT null AND INPR.FIRST_PYMNT_DATE IS NOT null AND INPR.CALC_INTEREST = 1 AND INPR.CLX_EFFTDATE = (SELECT MAX(INPRX.CLX_EFFTDATE) FROM ' . $this->inprname . ' INPRX WHERE INPRX.INVESTMENT_CODE = INPR.INVESTMENT_CODE AND INPRX.CLX_EFFTDATE<=' . singleQuote($efftdate) . ')';
        $invs = ClxCommon::$db->executeQuery($sql);
        while (($inv = ClxCommon::$db->getNextRecord($invs))) {
		  if ($inv['ISSUE_DATE'] > $inv['MATURITY_DATE'] || $inv['PAYMENT_FREQ'] == CLX_NUM_ZERO) {
			 continue;
		  }
		  //Mark all for update
		  $sql = 'UPDATE ' . $this->inasname . ' SET IS_REMOVE = 1 WHERE INVESTPROD_CODE = ' . singleQuote($investment_code) . ' AND EFFTDATE = ' . singleQuote($inv['EFFTDATE']);
		  ClxCommon::$db->executeQuery($sql);
		  $sdate = $inv['ISSUE_DATE'];
		  $org_sdate = $sdate;
		  $seqnum = CLX_INIT_ZERO;
		  while ($sdate < $inv['MATURITY_DATE']) {
			 //Setting first coupon date
			 if ($seqnum == CLX_NUM_ZERO) {
			     if ($inv['PAY_DAY'] > CLX_NUM_ZERO) {
			         $payment_date = xFormat::date($inv['ISSUE_DATE'], 'Y-m-' . $inv['PAY_DAY']);
			         $sdate = $payment_date;
			     }
			     $from_date = $org_sdate;
			     if ($inv['FIRST_PYMNT_DATE'] <> '') {
			         $payment_date = $inv['FIRST_PYMNT_DATE'];
			         $sdate = $payment_date;
			         $org_sdate = $payment_date;
			     }
			     if (empty($payment_date)) {
			         $payment_date = $this->find_period_end_date($inv['PAYMENT_FREQ'], $org_sdate, CLX_NUM_ONE);
			     }
			 } else {
			     $from_date = $payment_date;
			     $sdate = $this->find_period_end_date($inv['PAYMENT_FREQ'], $org_sdate, $seqnum);
			     $payment_date = $sdate;
			     if (intval(substr($payment_date, -CLX_NUM_TWO)) < intval(substr($org_sdate, -CLX_NUM_TWO))) {
			         $sdate = $payment_date;
			         $payment_date = adodb_prev_month_lastday($payment_date);
			     }
			 }
			 $seqnum++;
			 //colocar pago a fecha de final de vencimiento en caso de aplicar
			 if ($inv['PAYMENT_FREQ'] === '99') {
			     $sdate = $inv['MATURITY_DATE'];
			     $payment_date = $inv['MATURITY_DATE'];
			 }
			 $cupon_rt = $inv['COUPON_RT'];
			 if($inv['INTEREST_TYPE']<>'FIX'){
			     //Testing interest
			     $sql = 'SELECT  INAS.COUPON_RT FROM ' . $this->inasname . ' INAS WHERE INAS.INVESTPROD_CODE =' . singleQuote($inv['INVESTMENT_CODE']) . ' AND INAS.EFFTDATE = ' . singleQuote($inv['EFFTDATE']) . ' AND INAS.START_DATE <= ' . singleQuote($sdate) . ' ORDER BY INAS.START_DATE DESC LIMIT 1';
			     $cupon_rt = ClxCommon::$db->getOne($sql);
			     if (empty($cupon_rt)) {
			         $cupon_rt = $inv['COUPON_RT'];
			     }
			 }
			 $rec = array('INVESTPROD_CODE' => $inv['INVESTMENT_CODE'],
			     'EFFTDATE' => $inv['EFFTDATE'],
			     'START_DATE' => $from_date,
			     'COUPON_RT' => $cupon_rt,
			     'PAYMENT_DATE' => $payment_date,
			     'CREATEDBY' => $inv['CREATEDBY'],
			     'MODIFIEDBY' => $inv['MODIFIEDBY'],
			     'CDATE' => $inv['CDATE'],
			     'MDATE' => $inv['MDATE'],
			     'IS_REMOVE' => CLX_NUM_ZERO);
			 ClxCommon::$db->loadDbRecord($this->inasname, $rec, false, 'INVESTPROD_CODE =' . singleQuote($inv['INVESTMENT_CODE']) . ' AND EFFTDATE = ' . singleQuote($rec['EFFTDATE']) . ' AND START_DATE = ' . singleQuote($rec['START_DATE']));
			 if ($seqnum == CLX_NUM_10000000)
			     break; //just break
		  }
		  //Mark all for update
		  $sql = 'DELETE FROM ' . $this->inasname . ' WHERE IS_REMOVE = 1  AND INVESTPROD_CODE = ' . singleQuote($investment_code);
		  ClxCommon::$db->executeQuery($sql);
        }//end while
    }

    /**
     * Sets period of dates
     * @param string $interval
     * @param string $sdate
     * @param string $seqnum
     */
    private function find_period_end_date($interval, $sdate, $seqnum) {

        switch ($interval) {
		  case '52':
			 $sdate = adodb_period_date($sdate, 'D', CLX_NUM_7 * $seqnum, 'Y-m-d');
			 break;
		  case '360':
			 $sdate = adodb_period_date($sdate, 'D', CLX_NUM_ONE * $seqnum, 'Y-m-d');
			 break;
		  case '12'://Monthly
			 $sdate = adodb_period_date($sdate, 'M', CLX_NUM_ONE * $seqnum, 'Y-m-d');
			 break;
		  case '6'://Bimonthly(6)
			 $sdate = adodb_period_date($sdate, 'M', CLX_NUM_TWO * $seqnum, 'Y-m-d');
			 break;
		  case '4'://Quarterly(4)
			 $sdate = adodb_period_date($sdate, 'M', CLX_NUM_THREE * $seqnum, 'Y-m-d');
			 break;
		  case '3'://Four-monthly 1/3(3)
			 $sdate = adodb_period_date($sdate, 'M', CLX_NUM_FOUR * $seqnum, 'Y-m-d');
			 break;
		  case '2'://Semiannually(2)
			 $sdate = adodb_period_date($sdate, 'M', CLX_NUM_SIX * $seqnum, 'Y-m-d');
			 break;
		  case '1'://Annually(1)
			 $sdate = adodb_period_date($sdate, 'Y', CLX_NUM_ONE * $seqnum, 'Y-m-d');
			 break;
		  default:
			 $sdate = 'null';
			 break;
        }
        return $sdate;
    }

}
