<?php

/**
 * General Code File
 * For more information contact  support@alenet.com
 *
 * @author	   $Author: fsantana $
 * @copyright	Alenet, Inc.
 * @since		$Date: 2017-01-27 08:31:17 -0500 (Fri, 27 Jan 2017) $
 * @version		$Rev: 9065 $
 *
 * CHANGE LOG
 *  Included svn prop
 */
if (!class_exists('Infa')) {

    class Infa {

        var $display = false;

        /**
         * Constructor
         * @param array [$procParameters]
         */
        function __construct($procParameters = array()) {
            // Initial tables
            multiTableName($this, array('ATMP', 'INAT', 'INFA', 'JEHD', 'GLAC', 'GLAG', 'INFU', 'FIAS',
                'FIAC', 'JELN', 'TTPA', 'FIBK', 'GLCP', 'FIGN', 'FTMP', 'FIAT', 'FITB',
                'FITS', 'JEIT', 'OPBV', 'LEDG', 'LEBA', 'INTT', 'INFU', 'FIPA', 'INPR',
                'INTL', 'GLCP', 'JELN', 'FICO', 'COMP', 'TTPA', 'FIBK', 'PTMP', 'JCBV',
                'PRAT', 'PRAT', 'INAS', 'INDX', 'COMP', 'IAT1', 'IAT2', 'IAT3', 'IAT4',
                'INDT', 'CURR', 'INFA', 'INDV', 'PROD', 'STOC', 'INLS', 'INVR'));
            $this->batch_id = (!empty($procParameters ['BATCH_ID'])) ? $procParameters ['BATCH_ID'] : batch_id();
            $this->username = $_SESSION ['user_record'] ['username'];
            $this->now = xFormat::getNow('Y-m-d H:i:s');
            $this->today = xFormat::getNow('Y-m-d');
            $this->initTemporaryTables($this->batch_id);
        }

        /**
         * Builds investment record infa if not exists
         * @param string $position_number
         * @param string $comp_code
         * @param string $portfolio_code
         * @param string $investment_code
         * @param string $contract_code
         * @param string $participant_id
         * @param string $operation_date
         * @param string $sec_broker
         * @param string $sec_custodian
         * @param string $unit_price
         * @param string $unit_price_pct
         */
        function infa_build($position_number, $comp_code, $portfolio_code, $investment_code, $contract_code, $participant_id, $operation_date, $invest_class, $sec_broker, $sec_custodian, $unit_price = CLX_NUM_ZERO, $unit_price_pct = CLX_NUM_ZERO, $yield_rt = CLX_NUM_ZERO, $pd_yield_rt = CLX_NUM_ZERO) {
            $sql = 'SELECT COUNT(ID) FROM ' . $this->infaname . ' WHERE POSITION_NUMBER = ' . singleQuote($position_number);
            $qty = ClxCommon::$db->getOne($sql);
            if ($qty == CLX_NUM_ZERO) {
                $prodinfo = ClxCommon::$db->getOneRecord('SELECT IS_FIXINCOME,PRODUCT_CODE, COUPON_RT FROM ' . $this->inprname . ' WHERE INVESTMENT_CODE = ' . singleQuote($investment_code));
                $prodinfo ['IS_FIXINCOME'] = (!empty($prodinfo ['IS_FIXINCOME'])) ? CLX_NUM_ONE : CLX_NUM_ZERO;
                $prodinfo ['PRODUCT_CODE'] = (!empty($prodinfo ['PRODUCT_CODE'])) ? $prodinfo ['PRODUCT_CODE'] : 'null';
                if ($unit_price === '') {
                    $unit_price = CLX_INIT_ZERO;
                }
                if ($unit_price_pct === '') {
                    $unit_price_pct = CLX_INIT_ZERO;
                }
                if (empty($prodinfo['COUPON_RT'])) {
                    $prodinfo['COUPON_RT'] = CLX_INIT_ZERO;
                }
                $record = array('PORTFOLIO_CODE' => $portfolio_code, 'POSITION_NUMBER' => $position_number, 'CONTRACT_CODE' => $contract_code, 'PARTICIPANT_ID' => $participant_id, 'INVESTMENT_CODE' => $investment_code, 'ACQUISITION_DATE' => $operation_date, 'COMP_CODE' => $comp_code, 'IS_FIXINCOME' => $prodinfo ['IS_FIXINCOME'], 'PRODUCT_CODE' => $prodinfo ['PRODUCT_CODE'], 'CURRENCY_CODE' => ClxCommon::$db->getField($this->infuname, 'CURRENCY_CODE', 'IDCODE =' . singleQuote($portfolio_code)), 'STATUS' => 'A', 'INVEST_AMT' => CLX_NUM_ZERO, 'FACE_VALUE' => CLX_NUM_ZERO, 'SHARES' => CLX_NUM_ZERO, 'AMORTIZED_AMT' => CLX_NUM_ZERO, 'PAID_INT_AMT' => CLX_NUM_ZERO, 'ACCRUED_INT_AMT' => CLX_NUM_ZERO, 'GAIN_AMT' => CLX_NUM_ZERO, 'UNIT_PRICE' => $unit_price, 'UNIT_PRICE_PCT' => $unit_price_pct, 'TAX_AMT' => CLX_NUM_ZERO, 'FEES_AMT' => CLX_NUM_ZERO, 'TAX_AMT' => CLX_NUM_ZERO, 'DIVIDEND_AMT' => CLX_NUM_ZERO, 'COUPON_RT' => $prodinfo['COUPON_RT'], 'INVEST_CLASS' => $invest_class, 'YIELD_RT' => $yield_rt, 'PD_YIELD_RT' => $pd_yield_rt, 'SEC_BROKER' => $sec_broker, 'SEC_CUSTODIAN' => $sec_custodian, 'CREATEDBY' => $this->username, 'MODIFIEDBY' => $this->username, 'MDATE' => $this->now, 'CDATE' => $this->now);
                ClxCommon::$db->createRecord($this->infaname, $record, false);
            }
        }

        /**
         * Runs interest calculation
         */
        function infa_run_interest($pay_process = false) {
            $idlist = getIdlist();
            $sql = 'SELECT COMP_CODE, IDCODE FROM ' . $this->infuname . ' WHERE ID IN (' . toString($idlist) . ')';
            $rs = ClxCommon::$db->executeQuery($sql);
            while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
                if ($pay_process === true) {
                    $this->infa_pay_interest($r ['COMP_CODE'], $r ['IDCODE']);
                } else {
                    $this->infa_earn_interest($r ['COMP_CODE'], $r ['IDCODE']);
                }
            }
			ClxCommon::$db->freeResult($rs);
        }

        /**
         * Run Interest by company
         *
         * @param string $comp_code
         */
        function infa_run_interest_by_comp($comp_code, $pay_process = false) {
            if ($pay_process === true) {
                $this->infa_pay_interest($comp_code);
            } else {
                $this->infa_earn_interest($comp_code);
            }
        }

        /**
         * Proceso para devengar intereses de portafolio
         *
         * @param string $portfolio_code
         */
        private function infa_earn_interest($comp_code, $portfolio_code = '') {
            $this->je_source = 'INIP';
            if (!class_exists('Infu')) {
                includeCodeLib('INFU');
            }
            if (!class_exists('Comp')) {
                includeCodeLib('COMP');
            }
            $infu = new Infu ();
            $comp = new Comp ();
            $op = $comp->comp_operation_parameters($comp_code);
            $operation_date = $op ['OPERATION_DATE'];
            if ($this->display === true) {
                echoLog(' INTEREST EARNINGS. BATCH ID: ' . $this->batch_id);
                echoLog('Efective date: ' . $operation_date);
                echoLog('Portfolio: ' . $portfolio_code);
            }
            $this->clearTemporaryTables($this->batch_id);



            /* NOTA: Revisar actual/actual */
            $sql = 'INSERT INTO ' . $this->atmpname . ' (COMP_CODE, POSITION_NUMBER, INVESTMENT_CODE, PROD_EFFTDATE, PORTFOLIO_CODE, PRODUCT_CODE, INVEST_AMT, COUPON_RT, INDEX_RATE, ACCRUED_INT_AMT, PAID_INT_AMT, BATCH_ID, GL01_CODE, GL02_CODE, CA_GL_CODE, INTEREST_CALC, PAYMENT_FREQ, YEAR_DAYS, COUPON_AMT, ADJ_INDEX, CHART_FIELD01, CHART_FIELD02, CURRENCY_CODE, MATURITY_DATE, ACQUISITION_DATE, NO_DAYS, START_DATE, PAYMENT_DATE, EXCHANGE_RT, CUR_OPERATOR, EFFTDATE, OPERATION_DATE) SELECT INFA.COMP_CODE, INFA.POSITION_NUMBER, INFA.INVESTMENT_CODE, INPR.CLX_EFFTDATE PROD_EFFTDATE, INFA.PORTFOLIO_CODE, INPR.PRODUCT_CODE, INFA.FACE_VALUE INVEST_AMT, INPR.COUPON_RT, 0 INDEX_RATE, INFA.ACCRUED_INT_AMT, INFA.PAID_INT_AMT, ' . singleQuote($this->batch_id) . ' BATCH_ID, null GL01_CODE, null GL02_CODE, null CA_GL_CODE, INPR.INTEREST_CALC, INPR.PAYMENT_FREQ, (case INPR.INTEREST_CALC WHEN ' . singleQuote('') . ' THEN 0 WHEN ' . singleQuote('AC/360') . ' THEN 360 WHEN ' . singleQuote('30/360') . ' THEN 360 WHEN ' . singleQuote('AC/365') . ' THEN 365 WHEN ' . singleQuote('30/365') . ' THEN 365 ELSE 360 END) YEAR_DAYS, 0 COUPON_AMT, INPR.ADJ_INDEX, INFA.CONTRACT_CODE CHART_FIELD01, INFA.PARTICIPANT_ID CHART_FIELD02, INFA.CURRENCY_CODE, INPR.MATURITY_DATE, INFA.ACQUISITION_DATE, 0 NO_DAYS, null START_DATE, INPR.MATURITY_DATE PAYMENT_DATE, 1 EXCHANGE_RT, 1 CUR_OPERATOR, ' . singleQuote($operation_date) . ' EFFTDATE, ' . singleQuote($operation_date) . ' OPERATION_DATE FROM ' . $this->infaname . ' INFA, ' . $this->inprname . ' INPR, ' . $this->infuname . ' INFU WHERE INFA.PORTFOLIO_CODE = INFU.IDCODE AND INPR.CALC_INTEREST = 1 AND INFA.COMP_CODE = ' . singleQuote($comp_code) . ' AND (INPR.MATURITY_DATE IS NULL OR INPR.MATURITY_DATE >' . singleQuote($operation_date) . ') AND INPR.INVESTMENT_CODE = INFA.INVESTMENT_CODE AND INPR.CLX_EFFTDATE = (SELECT MAX(INPRX.CLX_EFFTDATE) FROM ' . $this->inprname . ' INPRX WHERE INPRX.INVESTMENT_CODE = INPR.INVESTMENT_CODE AND INPRX.CLX_EFFTDATE<=' . singleQuote($operation_date) . ') ' . (($portfolio_code <> '') ? 'AND INFA.PORTFOLIO_CODE = ' . singleQuote($portfolio_code) : '') . ' AND INFA.FACE_VALUE<>0';
            //Prevenir que corra 2 veces si ya corrio y no se ha reversado el asiento de devengo
            if ($portfolio_code === '') {
                $sql .= ' AND NOT EXISTS(SELECT JEHD.ID FROM ' . $this->jehdname . ' JEHD WHERE JEHD.COMP_CODE = ' . singleQuote($comp_code) . ' AND JEHD.JE_SOURCE = ' . singleQuote($this->je_source) . ' AND JEHD.EFFTDATE = ' . singleQuote($operation_date) . ' AND JEHD.REAL_DATE = ' . singleQuote($operation_date) . ' AND JEHD.JE_STATUS = ' . singleQuote('AP') . ' AND JEHD.REVERSED = 0) ';
            }
            ClxCommon::$db->executeQuery($sql);

            if (ClxCommon::$db->oneExistsRecords($this->atmpname, 'BATCH_ID = ' . singleQuote($this->batch_id)) == CLX_NUM_ZERO) {
                return;
            }

            /* Defining next coupon or payment date */
            $sql = 'UPDATE  ' . $this->atmpname . ' ATMP, ' . $this->inasname . ' INAS SET ATMP.PAYMENT_DATE = INAS.PAYMENT_DATE, ATMP.START_DATE = INAS.START_DATE WHERE ATMP.INVESTMENT_CODE = INAS.INVESTPROD_CODE AND ATMP.PROD_EFFTDATE = INAS.EFFTDATE AND INAS.START_DATE<=' . singleQuote($operation_date) . ' AND INAS.PAYMENT_DATE>' . singleQuote($operation_date) . ' AND ATMP.BATCH_ID = ' . singleQuote($this->batch_id);
            ClxCommon::$db->executeQuery($sql);

            // Revisando Actual/Actual o el año tiene el numero de dias totales en cupon
            // Defining next coupon or payment date
            $sql = 'UPDATE ' . $this->atmpname . ' SET START_DATE = ADDDATE(STR_TO_DATE(CONCAT(YEAR(OPERATION_DATE),\'-\',MONTH(OPERATION_DATE),\'-\',DAY(PAYMENT_DATE)),\'%Y-%m-%d\'), INTERVAL IF(STR_TO_DATE(CONCAT(YEAR(OPERATION_DATE),\'-\',MONTH(OPERATION_DATE),\'-\',DAY(PAYMENT_DATE)),\'%Y-%m-%d\')>OPERATION_DATE, -1,0) MONTH), PAYMENT_DATE = ADDDATE(STR_TO_DATE(CONCAT(YEAR(OPERATION_DATE),\'-\',MONTH(OPERATION_DATE),\'-\',DAY(PAYMENT_DATE)),\'%Y-%m-%d\'), INTERVAL IF(STR_TO_DATE(CONCAT(YEAR(OPERATION_DATE),\'-\',MONTH(OPERATION_DATE),\'-\',DAY(PAYMENT_DATE)),\'%Y-%m-%d\')>OPERATION_DATE, 0,1) MONTH) WHERE BATCH_ID = ' . singleQuote($this->batch_id) . ' AND INTEREST_CALC NOT IN (\'AC/360\',\'AC/365\',\'AC/AC\') AND PAYMENT_FREQ IN (\'12\',\'6\',\'4\',\'3\',\'2\',\'1\',\'99\')';
            ClxCommon::$db->executeQuery($sql);

            /* Setting base calc on year days */
            $sql = 'UPDATE ' . $this->atmpname . ' SET NO_DAYS = 30 WHERE BATCH_ID = ' . singleQuote($this->batch_id);
            ClxCommon::$db->executeQuery($sql);


            /* Determinando dias a devengar
             * 1)Fecha Operaciones, Inicio cupon y Fin cupon se determinan.
             * 2)se calcula Transcurridos dependiendo si el cupon a sido completo o no. de ser menor se coloca desde el inicio del bono
             * 3)se calculan los dias extras si la proxima fecha es el pago
             * 4)por ultima basado en los dias extras o dias que han transcurridos se calcula el numero de dias a devengar si es cero por tener mas de los dias entonces no devenga
             * de ser requeridos dias adicionales como en febrero entonces se agregan los dias. Por ultimo se devenga solo 1 dia.
             */
            $sql = 'UPDATE ' . $this->atmpname . ' SET NO_DAYS = IFNULL(IF(CAST(DATEDIFF(OPERATION_DATE, PAYMENT_DATE)+CAST(DATEDIFF(PAYMENT_DATE, START_DATE)-CAST(NO_DAYS AS SIGNED) AS SIGNED) AS SIGNED)>=0, 0, IF(OPERATION_DATE != ADDDATE(PAYMENT_DATE, INTERVAL -1 DAY),  1, IF(CAST(DATEDIFF(PAYMENT_DATE, START_DATE)=NO_DAYS AS SIGNED), 1,0))) + IF(OPERATION_DATE = ADDDATE(PAYMENT_DATE,INTERVAL -1 DAY), IF(CAST(DATEDIFF(OPERATION_DATE, PAYMENT_DATE)+CAST(DATEDIFF(PAYMENT_DATE, START_DATE)-CAST(NO_DAYS AS SIGNED) AS SIGNED) AS SIGNED)<-1, -1*CAST(DATEDIFF(OPERATION_DATE, PAYMENT_DATE)+CAST(DATEDIFF(PAYMENT_DATE, START_DATE)-CAST(NO_DAYS AS SIGNED) AS SIGNED) AS SIGNED),0),0),0) WHERE BATCH_ID = ' . singleQuote($this->batch_id);
            ClxCommon::$db->executeQuery($sql);

            /* COLOCANDO 1 SI ES ac */
            $sql = 'UPDATE ' . $this->atmpname . ' SET NO_DAYS = 1 WHERE INTEREST_CALC IN (' . singleQuote('AC/360') . ',' . singleQuote('AC/365') . ',' . singleQuote('AC/AC') . ') AND BATCH_ID = ' . singleQuote($this->batch_id);
            ClxCommon::$db->executeQuery($sql);

            /* Actualizando subcuenta */
            $sql = 'UPDATE ' . $this->atmpname . ' TMP SET TMP.CHART_FIELD03 = (SELECT MAX(FIAS.IDCODE) FROM ' . $this->fiasname . ' FIAS WHERE FIAS.PORTFOLIO_CODE = TMP.PORTFOLIO_CODE AND FIAS.CONTRACT_CODE = TMP.CHART_FIELD01) WHERE TMP.BATCH_ID = ' . singleQuote($this->batch_id);
            ClxCommon::$db->executeQuery($sql);

            //Actualizando tasa si es por scala
            $sql = 'UPDATE ' . $this->atmpname . ' ATMP SET ATMP.COUPON_RT = IFNULL((SELECT INAS.COUPON_RT FROM ' . $this->inasname . ' INAS WHERE INAS.INVESTPROD_CODE = ATMP.INVESTMENT_CODE AND INAS.EFFTDATE = ATMP.PROD_EFFTDATE AND INAS.START_DATE = ( SELECT MAX(INAS1.START_DATE) FROM ' . $this->inasname . ' INAS1 WHERE INAS1.INVESTPROD_CODE = INAS.INVESTPROD_CODE AND INAS1.EFFTDATE = INAS.EFFTDATE AND INAS1.START_DATE<=' . singleQuote($operation_date) . ') LIMIT 1), ATMP.COUPON_RT) WHERE ATMP.BATCH_ID = ' . singleQuote($this->batch_id);
            ClxCommon::$db->executeQuery($sql);

            //actualizando indice si es variable
            $sql = 'UPDATE ' . $this->atmpname . ' ATMP SET ATMP.INDEX_RATE = IFNULL((SELECT INDX.INTEREST_RT FROM ' . $this->indxname . ' INDX WHERE INDX.ADJ_INDEX = ATMP.ADJ_INDEX AND INDX.CLX_EFFTDATE = ( SELECT MAX(INDX1.CLX_EFFTDATE) FROM ' . $this->indxname . ' INDX1 WHERE INDX.ADJ_INDEX = INDX1.ADJ_INDEX AND INDX1.CLX_EFFTDATE<=' . singleQuote($operation_date) . ')),0) WHERE ATMP.BATCH_ID = ' . singleQuote($this->batch_id) . ' AND ATMP.ADJ_INDEX <> ' . singleQuote('N/A');
            ClxCommon::$db->executeQuery($sql);

            //Tasa efectiva cupon rt = cupon_ra+index
            $sql = 'UPDATE ' . $this->atmpname . ' ATMP SET ATMP.COUPON_RT = ATMP.COUPON_RT+ATMP.INDEX_RATE WHERE ATMP.BATCH_ID = ' . singleQuote($this->batch_id);
            ClxCommon::$db->executeQuery($sql);

            $sql = 'UPDATE ' . $this->atmpname . ' ATMP SET ATMP.ACCRUED_INT_AMT = IFNULL(ROUND((ATMP.INVEST_AMT*(ATMP.COUPON_RT/100)/NULLIF(ATMP.YEAR_DAYS,0)* ATMP.NO_DAYS), 5),0) WHERE ATMP.BATCH_ID = ' . singleQuote($this->batch_id);
            ClxCommon::$db->executeQuery($sql);

            //Borrar los casos que no deben devengar
            $sql = 'DELETE FROM ' . $this->atmpname . ' WHERE ACCRUED_INT_AMT = 0 AND BATCH_ID = ' . singleQuote($this->batch_id);
            ClxCommon::$db->executeQuery($sql);

            if (ClxCommon::$db->oneExistsRecords($this->atmpname, 'BATCH_ID = ' . singleQuote($this->batch_id)) == CLX_NUM_ZERO) {
                return;
            }

            //INTEARNED_ACC
            $sql = 'UPDATE ' . $this->infaname . ' INFA, ' . $this->atmpname . ' ATMP SET INFA.INTEARNED_ACC = INFA.INTEARNED_ACC+ATMP.ACCRUED_INT_AMT, INFA.COUPON_RT = ATMP.COUPON_RT WHERE INFA.POSITION_NUMBER = ATMP.POSITION_NUMBER AND INFA.PORTFOLIO_CODE = ATMP.PORTFOLIO_CODE';
            ClxCommon::$db->executeQuery($sql);

            $sql = 'UPDATE ' . $this->infaname . ' INFA, ' . $this->atmpname . ' ATMP SET ATMP.ACCRUED_INT_AMT = ROUND(INFA.INTEARNED_ACC, 2) WHERE INFA.POSITION_NUMBER = ATMP.POSITION_NUMBER AND INFA.PORTFOLIO_CODE = ATMP.PORTFOLIO_CODE';
            ClxCommon::$db->executeQuery($sql);

            //Ready to update infa
            $sql = 'UPDATE ' . $this->infaname . ' INFA, ' . $this->atmpname . ' ATMP SET INFA.ACCRUED_INT_AMT = INFA.ACCRUED_INT_AMT + ATMP.ACCRUED_INT_AMT, INFA.INTEARNED_ACC = INFA.INTEARNED_ACC - ATMP.ACCRUED_INT_AMT, INFA.COUPON_RT = ATMP.COUPON_RT WHERE INFA.POSITION_NUMBER = ATMP.POSITION_NUMBER AND INFA.PORTFOLIO_CODE = ATMP.PORTFOLIO_CODE';
            ClxCommon::$db->executeQuery($sql);

            //Ajustando signo si es debito o credito para activos o pasivos
            //ATMP.ACCRUED_INT_AMT debe ser negativo si crece por el haber
            //ya que la cuenta de debito GL01 estara asociada a
            //Actualizando cuentas contables de efectivo
            $sql = 'UPDATE ' . $this->atmpname . ' ATMP, ' . $this->pratname . ' PRAT SET ATMP.GL01_CODE = PRAT.AR_GL_CODE, ATMP.GL02_CODE = PRAT.INC_GL_CODE WHERE ATMP.PRODUCT_CODE = PRAT.PRODUCT_CODE AND PRAT.PROD_STATUS = ' . singleQuote('VIG') . ' AND ATMP.BATCH_ID = ' . singleQuote($this->batch_id) . ' AND EXISTS(SELECT GLAC.ID FROM ' . $this->glacname . ' GLAC WHERE GLAC.IDCODE = PRAT.AR_GL_CODE AND GLAC.GL_TYPE = ' . singleQuote('AS') . ') ';
            ClxCommon::$db->executeQuery($sql);

            //En caso de ser pasivo ver contrapartida
            $sql = 'UPDATE ' . $this->atmpname . ' ATMP,' . $this->pratname . ' PRAT SET ATMP.GL01_CODE = PRAT.INC_GL_CODE, ATMP.GL02_CODE = PRAT.AR_GL_CODE WHERE ATMP.PRODUCT_CODE = PRAT.PRODUCT_CODE AND PRAT.PROD_STATUS = ' . singleQuote('VIG') . ' AND ATMP.BATCH_ID = ' . singleQuote($this->batch_id) . ' AND EXISTS(SELECT GLAC.ID FROM ' . $this->glacname . ' GLAC WHERE GLAC.IDCODE = PRAT.AR_GL_CODE AND GLAC.GL_TYPE <> ' . singleQuote('AS') . ') ';
            ClxCommon::$db->executeQuery($sql);

            //Setting exchange rate
            $sql = 'SELECT DISTINCT COMP_CODE, CURRENCY_CODE FROM ' . $this->atmpname . ' TMP WHERE TMP.BATCH_ID = ' . singleQuote($this->batch_id);
            $rs = ClxCommon::$db->executeQuery($sql);
            while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
                $rate = $comp->comp_current_exchange($r ['COMP_CODE'], $r ['CURRENCY_CODE'], $operation_date);
                $sql = 'UPDATE ' . $this->atmpname . ' SET CURRENCY_CODE = ' . singleQuote($r ['CURRENCY_CODE']) . ', EXCHANGE_RT = ' . $rate ['EXCHANGE_RT'] . ', CUR_OPERATOR = ' . singleQuote($rate ['CUR_OPERATOR']) . ' WHERE BATCH_ID = ' . singleQuote($this->batch_id) . ' AND CURRENCY_CODE = ' . singleQuote($r ['CURRENCY_CODE']);
                ClxCommon::$db->executeQuery($sql);
            }
			ClxCommon::$db->freeResult($rs);
            $sql = 'DELETE FROM ' . $this->atmpname . ' WHERE ACCRUED_INT_AMT = 0 AND BATCH_ID = ' . singleQuote($this->batch_id);
            ClxCommon::$db->executeQuery($sql);
            if (ClxCommon::$db->oneExistsRecords($this->atmpname, 'BATCH_ID = ' . singleQuote($this->batch_id)) == CLX_NUM_ZERO) {
                return;
            }

            //Accounting from records in temprary table
            $this->build_gl_from_atmp($comp_code, $portfolio_code);

            //Cleanup
            $this->clearTemporaryTables($this->batch_id);

            // Updating Totals
            $infu->infu_update_portfolio_totals($comp_code, $portfolio_code);
        }

        /**
         * Runs interest payment
         * @param string $comp_code
         * @param string $portfolio_code
         */
        private function infa_pay_interest($comp_code, $portfolio_code = '') {
            $DBTB = &getTool('DBTB');
            $DBTB->buildTemporaryTable('INLS');
            ClxCommon::$db->executeQuery('DELETE FROM ' . $this->inlsname . ' WHERE BATCH_ID = ' . singleQuote($this->batch_id));
            $sql = 'INSERT INTO ' . $this->inlsname . ' (POSITION_NUMBER, PORTFOLIO_CODE, INVESTMENT_CODE, PAYMENT_FREQ, INTEREST_CALC, INT_PER_TYPE, PAYMENT_DATE, INVEST_AMT, INTEREST_AMT, CAPITAL_AMT, COUPON_RT, PAYMENT_AMT, BALANCE_AMT, BATCH_ID) SELECT INFA.POSITION_NUMBER, INFA.PORTFOLIO_CODE, INFA.INVESTMENT_CODE, INPR.PAYMENT_FREQ, INPR.INTEREST_CALC, INPR.INT_PER_TYPE, INAS.PAYMENT_DATE, INFA.FACE_VALUE INVEST_AMT, ROUND((INFA.ACCRUED_INT_AMT-INFA.PAID_INT_AMT), 2) INTEREST_AMT, INFA.INVEST_AMT CAPITAL_AMT, INAS.COUPON_RT, ROUND((INFA.ACCRUED_INT_AMT-INFA.PAID_INT_AMT), 2) PAYMENT_AMT, INFA.FACE_VALUE BALANCE_AMT, ' . singleQuote($this->batch_id) . ' BATCH_ID FROM ' . $this->glcpname . ' GLCP, ' . $this->infaname . ' INFA, ' . $this->inasname . ' INAS, ' . $this->inprname . ' INPR WHERE INPR.INVESTMENT_CODE = INFA.INVESTMENT_CODE AND INPR.CLX_EFFTDATE = (SELECT MAX(INPRX.CLX_EFFTDATE) FROM ' . $this->inprname . ' INPRX WHERE INPRX.INVESTMENT_CODE = INPR.INVESTMENT_CODE AND INPRX.CLX_EFFTDATE<=GLCP.OPERATION_DATE) AND INAS.INVESTPROD_CODE = INPR.INVESTMENT_CODE AND INAS.EFFTDATE = INPR.CLX_EFFTDATE AND INFA.COMP_CODE = ' . singleQuote($comp_code) . ' ' . (($portfolio_code <> '') ? ' AND INFA.PORTFOLIO_CODE = ' . singleQuote($portfolio_code) . ' ' : '') . ' AND INAS.PAYMENT_DATE = GLCP.OPERATION_DATE AND INAS.PAYMENT_DATE <> INPR.MATURITY_DATE AND INPR.MATURITY_DATE <> GLCP.OPERATION_DATE AND INPR.COMP_CODE = GLCP.COMP_CODE AND ROUND((INFA.ACCRUED_INT_AMT-INFA.PAID_INT_AMT), 2)<>0 ORDER BY INAS.EFFTDATE ASC';
            ClxCommon::$db->executeQuery($sql);

            //Delete elements that currency have payment operations
            $sql = 'DELETE INLS.* FROM ' . $this->inlsname . ' INLS WHERE EXISTS (SELECT INAT.ID FROM ' . $this->inatname . ' INAT WHERE INAT.POSITION_NUMBER = INLS.POSITION_NUMBER AND INAT.EFFTDATE = INLS.PAYMENT_DATE AND INAT.CLX_AUTH_STATUS = ' . singleQuote('PEND') . ' AND INAT.INVEST_ACTIVITY IN (' . singleQuote('RCA') . ',' . singleQuote('RCA') . ')) ';
            ClxCommon::$db->executeQuery($sql);
            //check if anything to do
            if (ClxCommon::$db->oneExistsRecords($this->inlsname) == CLX_NUM_ZERO) {
                return;
            }
            $this->je_source = 'INIC';
            if (!class_exists('Infu'))
                includeCodeLib('INFU');
            if (!class_exists('Inat'))
                includeCodeLib('INAT');
            if (!class_exists('Comp'))
                includeCodeLib('COMP');
            $inat = new Inat ();
            $comp = new Comp ();
            $op = $comp->comp_operation_parameters($comp_code);
            $operation_date = $op ['OPERATION_DATE'];
            unset($comp);
            $DBTB = &getTool('DBTB');
            $inat_def_rec = $DBTB->tableDefaultRecord('INAT');
            $inat_def_rec['CREATEDBY'] = $this->username;
            $inat_def_rec['MODIFIEDBY'] = $this->username;
            $inat_def_rec['MDATE'] = $this->now;
            $inat_def_rec['CDATE'] = $this->now;
            if ($this->display === true) {
                echoLog('Investment Coupon Payment. BATCH ID: ' . $this->batch_id);
                echoLog('Efective date: ' . $operation_date);
                echoLog('Comp code: ' . $comp_code);
                echoLog('Portfolio: ' . $portfolio_code);
            }
            $sql = 'SELECT INFA.COMP_CODE, INLS.PORTFOLIO_CODE, INLS.INVESTMENT_CODE, INLS.POSITION_NUMBER, ' . singleQuote('PINT') . ' INVEST_ACTIVITY, ' . singleQuote($operation_date) . ' REAL_DATE, ' . singleQuote($operation_date) . ' EFFTDATE, INLS.PAYMENT_AMT ACCRUED_INT_AMT, 0 AMORTIZED_AMT, 0 VALORIZED_AMT, INLS.PAYMENT_AMT CASH_AMT, 0 INVEST_AMT, 0 FACE_VALUE, 0 GAIN_AMT, 0 PREMIUM_AMT, 0 FEES_AMT, 0 TAX_AMT, 0 SHARES, INFA.CONTRACT_CODE, INFA.PARTICIPANT_ID, INLS.BATCH_ID, ' . singleQuote($this->je_source) . ' JE_SOURCE, ' . singleQuote('SYS') . ' REQ_CHANNEL, INFU.FIDU_BANKACT_CODE FROM ' . $this->infuname . ' INFU, ' . $this->infaname . ' INFA, ' . $this->inlsname . ' INLS WHERE INFA.POSITION_NUMBER = INLS.POSITION_NUMBER AND INFA.PORTFOLIO_CODE = INFU.IDCODE AND INLS.PORTFOLIO_CODE = INFU.IDCODE AND INLS.BATCH_ID = ' . singleQuote($this->batch_id);
            $rs = ClxCommon::$db->executeQuery($sql);
            while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
                $inatrec = array_merge($inat_def_rec, $r);
                $inatrec ['REVACTI_CODE'] = 'null';
                $inatrec ['FROM_CONTRACT_CODE'] = 'null';
                $inatrec ['FROM_PARTICIPANT_ID'] = 'null';
                $inatrec ['TO_PORTFOLIO_CODE'] = 'null';
                $inatrec ['INVCRED_RATING'] = 'null';
                $inatrec ['JE_COMPLETED'] = CLX_INIT_ZERO;
                $idcode = $inat->inat_build($inatrec);
                //Building accounting
                $inat->inat_build_accounting_batch($inatrec['COMP_CODE'], $inatrec['EFFTDATE'], array(), $idcode);
            }
			ClxCommon::$db->freeResult($rs);
            //clean up
            ClxCommon::$db->executeQuery('DELETE FROM ' . $this->inlsname . ' WHERE BATCH_ID = ' . singleQuote($this->batch_id));
        }

        // PROCESOS DE VENCIMIENTO
        /**
         * Runs investment maturity by company
         * @param unknown_type $comp_code
         */
        function infa_run_maturity_by_comp($comp_code) {
            $this->infa_maturity($comp_code);
        }

        /**
         * Execute Maturity process
         */
        function infa_run_maturity_btn() {
            $idlist = getIdlist();
            $sql = 'SELECT COMP_CODE, IDCODE FROM ' . $this->infuname . ' WHERE ID IN (' . toString($idlist) . ')';
            $rs = ClxCommon::$db->executeQuery($sql);
            while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
                $this->infa_maturity($r ['COMP_CODE'], $r ['IDCODE']);
            }
			ClxCommon::$db->freeResult($rs);
        }

        /**
         * Builds amortization
         *
         * @param string $portfolio_code
         */
        private function infa_maturity($comp_code, $portfolio_code = '') {
            $this->je_source = 'INMT';
            if (!class_exists('Infu')) {
                includeCodeLib('INFU');
            }
            if (!class_exists('Inat')) {
                includeCodeLib('INAT');
            }
            $inat = new Inat ();
            $DBTB = &getTool('DBTB');
            $inat_def_rec = $DBTB->tableDefaultRecord('INAT');
            $inat_def_rec['CREATEDBY'] = $this->username;
            $inat_def_rec['MODIFIEDBY'] = $this->username;
            $inat_def_rec['MDATE'] = $this->now;
            $inat_def_rec['CDATE'] = $this->now;
            if (!class_exists('Infu'))
                includeCodeLib('INFU');
            if (!class_exists('Comp'))
                includeCodeLib('COMP');
            $comp = new Comp ();
            $op = $comp->comp_operation_parameters($comp_code);
            $operation_date = $op ['OPERATION_DATE'];
            unset($comp);
            if ($this->display === true) {
                echoLog('Investment Maturity. BATCH ID: ' . $this->batch_id);
                echoLog('Efective date: ' . $operation_date);
                echoLog('Comp code: ' . $comp_code);
                echoLog('Portfolio: ' . $portfolio_code);
            }
            /* Seleccionando todas las inversiones que no contengan parametro de renovacion */
            $sql = 'INSERT INTO ' . $this->iat2name . ' (COMP_CODE, PORTFOLIO_CODE, INVESTMENT_CODE, POSITION_NUMBER, INVEST_ACTIVITY, EFFTDATE, ACCRUED_INT_AMT, CASH_AMT, INVEST_AMT, FACE_VALUE, SHARES, AMORTIZED_AMT, VALORIZED_AMT, VALORIZED_GAIN_AMT, VALORIZED_LOSS_AMT, CHART_FIELD01, CHART_FIELD02, BATCH_ID, JE_SOURCE, INVR_ID, PROD_EFFTDATE) SELECT INFA.COMP_CODE, INFA.PORTFOLIO_CODE, INFA.INVESTMENT_CODE, INFA.POSITION_NUMBER, ' . singleQuote('VE') . ' INVEST_ACTIVITY, ' . singleQuote($operation_date) . ' EFFTDATE, (INFA.ACCRUED_INT_AMT-INFA.PAID_INT_AMT) ACCRUED_INT_AMT, (INFA.INVEST_AMT+(INFA.ACCRUED_INT_AMT-INFA.PAID_INT_AMT)) CASH_AMT, INFA.INVEST_AMT INVEST_AMT, INFA.FACE_VALUE FACE_VALUE, INFA.SHARES SHARES, INFA.AMORTIZED_AMT AMORTIZED_AMT, INFA.VALORIZED_AMT VALORIZED_AMT, INFA.VALORIZED_GAIN_AMT, INFA.VALORIZED_LOSS_AMT, INFA.CONTRACT_CODE CHART_FIELD01, INFA.PARTICIPANT_ID CHART_FIELD02, ' . singleQuote($this->batch_id) . ' BATCH_ID, ' . singleQuote($this->je_source) . ' JE_SOURCE, 0 INVR_ID, INPR.CLX_EFFTDATE FROM ' . $this->infaname . ' INFA, ' . $this->inprname . ' INPR WHERE INPR.INVESTMENT_CODE = INFA.INVESTMENT_CODE AND INPR.CLX_EFFTDATE = (SELECT MAX(INPRX.CLX_EFFTDATE) FROM ' . $this->inprname . ' INPRX WHERE INPRX.INVESTMENT_CODE = INPR.INVESTMENT_CODE AND INPRX.CLX_EFFTDATE<=' . singleQuote($operation_date) . ') AND INPR.MATURITY_DATE = ' . singleQuote($operation_date) . ' AND INFA.FACE_VALUE <> 0 AND INFA.COMP_CODE =' . singleQuote($comp_code) . (($portfolio_code <> '') ? ' AND INFA.PORTFOLIO_CODE =' . singleQuote($portfolio_code) : '');
            ClxCommon::$db->executeQuery($sql);
            if (ClxCommon::$db->oneExistsRecords($this->iat2name, 'BATCH_ID = ' . singleQuote($this->batch_id)) == CLX_NUM_ZERO) {
                return;
            }
            //SET PARAMETER FOR RENOVATION IS ID IS FOUND THEN MUST PROCESS RENEWAL
            $sql = 'UPDATE ' . $this->iat2name . ' IAT2 SET IAT2.INVR_ID = IFNULL((SELECT MAX(INVR.ID) FROM ' . $this->invrname . ' INVR WHERE INVR.CLX_AUTH_STATUS = ' . singleQuote('AP') . ' AND INVR.RENEW_QTY > INVR.RENEW_EXEC AND INVR.INVESTMENT_CODE = IAT2.INVESTMENT_CODE AND INVR.CLX_EFFTDATE = (SELECT MAX(INVRX.CLX_EFFTDATE) FROM ' . $this->invrname . ' INVRX WHERE INVRX.INVESTMENT_CODE = INVRX.INVESTMENT_CODE AND INVRX.CLX_EFFTDATE <= CURDATE() AND IFNULL(INVRX.POSITION_NUMBER,' . singleQuote('ALL') . ') = IFNULL(INVR.POSITION_NUMBER,' . singleQuote('ALL') . ')) AND (INVR.POSITION_NUMBER = IAT2.POSITION_NUMBER OR INVR.POSITION_NUMBER IS NULL)),0) WHERE IAT2.BATCH_ID = ' . singleQuote($this->batch_id);
            ClxCommon::$db->executeQuery($sql);

            //NORMAL PROCESS WHERE NOT RENEWAL IS SET INVR_ID = 0
            $sql = 'SELECT IAT2.COMP_CODE, IAT2.PORTFOLIO_CODE, IAT2.INVESTMENT_CODE, IAT2.POSITION_NUMBER, IAT2.INVEST_ACTIVITY, IAT2.EFFTDATE REAL_DATE, IAT2.EFFTDATE, IAT2.ACCRUED_INT_AMT, IAT2.AMORTIZED_AMT, IAT2.VALORIZED_AMT, IAT2.VALORIZED_GAIN_AMT, IAT2.VALORIZED_LOSS_AMT, IAT2.CASH_AMT, IAT2.INVEST_AMT, IAT2.FACE_VALUE, 0 GAIN_AMT, 0 PREMIUM_AMT, 0 FEES_AMT, 0 TAX_AMT, IAT2.SHARES, IAT2.CHART_FIELD01 CONTRACT_CODE, IAT2.CHART_FIELD02 PARTICIPANT_ID, IAT2.BATCH_ID, IAT2.JE_SOURCE, ' . singleQuote('SYS') . ' REQ_CHANNEL, INFU.FIDU_BANKACT_CODE FROM ' . $this->iat2name . ' IAT2, ' . $this->infuname . ' INFU WHERE INFU.IDCODE = IAT2.PORTFOLIO_CODE AND IAT2.INVR_ID = 0 AND IAT2.BATCH_ID = ' . singleQuote($this->batch_id);
            $rs = ClxCommon::$db->executeQuery($sql);
            while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
                $inatrec = array_merge($inat_def_rec, $r);
                $inatrec ['REVACTI_CODE'] = 'null';
                $inatrec ['FROM_CONTRACT_CODE'] = 'null';
                $inatrec ['FROM_PARTICIPANT_ID'] = 'null';
                $inatrec ['TO_PORTFOLIO_CODE'] = 'null';
                $inatrec ['INVCRED_RATING'] = 'null';
                $inatrec ['JE_COMPLETED'] = CLX_INIT_ZERO;
                $inatrec['GAIN_AMT'] = $inatrec['AMORTIZED_AMT'];
                if ($inatrec['FACE_VALUE'] <> CLX_NUM_ZERO) {

                    $inatrec ['AMORTIZED_AMT'] = bcsub($inatrec['INVEST_AMT'], $inatrec['FACE_VALUE']);
                    $inatrec ['CASH_AMT'] = bcadd($inatrec ['ACCRUED_INT_AMT'], $inatrec['FACE_VALUE']);
                    $inatrec ['GAIN_AMT'] = CLX_NUM_ZERO; //VENCE A LA PAR
                    $inatrec ['INVEST_AMT'] = $inatrec['FACE_VALUE'];
                } else {
                    $inatrec ['AMORTIZED_AMT'] = CLX_INIT_ZERO;
                }
                $idcode = $inat->inat_build($inatrec);
                //Building accounting
                $inat->inat_build_accounting_batch($inatrec['COMP_CODE'], $inatrec['EFFTDATE'], array(), $idcode);
            }
			
            // Renewals
            // Updating Totals not really
            $sql = 'DELETE FROM ' . $this->iat2name . ' WHERE INVR_ID = 0 AND BATCH_ID = ' . singleQuote($this->batch_id);
            ClxCommon::$db->executeQuery($sql);
            //Borrando casos con mas de 1 renovacion
            $sql = 'DELETE IAT2.* FROM ' . $this->iat2name . ' IAT2, ' . $this->invrname . ' INVR WHERE IAT2.INVR_ID = INVR.ID AND INVR.RENEW_QTY <= INVR.RENEW_EXEC AND IAT2.BATCH_ID = ' . singleQuote($this->batch_id);
            ClxCommon::$db->executeQuery($sql);
            if (!class_exists('Inas'))
                includeCodeLib('INAS');
            $inas = new Inas();
            //Procesing renewals from this point invr_id >0
            //Copiar registro inpr a nueva fecha efectiva
            $sql = 'SELECT DISTINCT INPR.*, IAT2.INVR_ID INVR_ID_IAT, INVR.NEW_INVEST_CODE INVR_NEW_INVEST_CODE, INVR.INTEREST_RT INVR_INTEREST_RT, INVR.ADJUSTMENT_AMT INVR_ADJUSTMENT_AMT FROM ' . $this->inprname . ' INPR, ' . $this->iat2name . ' IAT2, ' . $this->invrname . ' INVR WHERE INPR.INVESTMENT_CODE = IAT2.INVESTMENT_CODE AND INVR.ID = IAT2.INVR_ID AND INPR.CLX_EFFTDATE = IAT2.PROD_EFFTDATE';
            $rs = ClxCommon::$db->executeQuery($sql);
            while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
                $new_investment_code = $r['INVR_NEW_INVEST_CODE'];
                $new_interest_rt = $r['INVR_INTEREST_RT'];
                $adjustment_amt = $r['INVR_ADJUSTMENT_AMT'];
                $r['INVR_ID'] = $r['INVR_ID_IAT'];
                unset($r['ID'], $r['INVR_ID'], $r['INVR_ID_IAT'], $r['INVR_NEW_INVEST_CODE'], $r['INVR_INTEREST_RT'], $r['INVR_ADJUSTMENT_AMT']);
                $r['CLX_EFFTDATE'] = $operation_date;
                $mat_date = $r['MATURITY_DATE'];
                $days = day_diff($r['MATURITY_DATE'], $r['ISSUE_DATE']);
                $r['ISSUE_DATE'] = $r['MATURITY_DATE'];
                $r['FIRST_PYMNT_DATE'] = $r['ISSUE_DATE'];
                $r['MATURITY_DATE'] = adodb_period_date($mat_date, 'D', $days);
                if ($new_investment_code <> '') {
                    $r['INVESTMENT_CODE'] = $new_investment_code;
                }
                if ($new_interest_rt > CLX_NUM_ZERO) {
                    $r['COUPON_RT'] = $new_interest_rt;
                }
                //Load record
                ClxCommon::$db->loadDbRecord($this->inprname, $r, false, 'INVESTMENT_CODE = ' . singleQuote($r['INVESTMENT_CODE']) . ' AND CLX_EFFTDATE = ' . singleQuote($r['CLX_EFFTDATE']));
                //Build schedule
                $inas->build_payment_schedule($r['INVESTMENT_CODE'], $r['CLX_EFFTDATE']);
            }
			ClxCommon::$db->freeResult($rs);
            //Increase counter
            /* RENEW_QTY RENEW_EXEC */
            $sql = 'UPDATE ' . $this->invrname . ' INVR SET INVR.RENEW_EXEC = INVR.RENEW_EXEC+1 WHERE EXISTS (SELECT IAT2.INVR_ID FROM ' . $this->iat2name . ' IAT2 WHERE IAT2.INVR_ID = INVR.ID)';
            ClxCommon::$db->executeQuery($sql);

            $sql = 'SELECT IAT2.COMP_CODE, IAT2.PORTFOLIO_CODE, IAT2.INVESTMENT_CODE, IAT2.POSITION_NUMBER, IAT2.INVEST_ACTIVITY, IAT2.EFFTDATE REAL_DATE, IAT2.EFFTDATE, IAT2.ACCRUED_INT_AMT, IAT2.AMORTIZED_AMT, IAT2.VALORIZED_AMT, IAT2.CASH_AMT, IAT2.INVEST_AMT, IAT2.FACE_VALUE, 0 GAIN_AMT, 0 PREMIUM_AMT, 0 FEES_AMT, 0 TAX_AMT, IAT2.SHARES, IAT2.CHART_FIELD01 CONTRACT_CODE, IAT2.CHART_FIELD02 PARTICIPANT_ID, IAT2.BATCH_ID, IAT2.JE_SOURCE, ' . singleQuote('SYS') . ' REQ_CHANNEL, INFU.FIDU_BANKACT_CODE, INVR.NEW_INVEST_CODE INVR_NEW_INVEST_CODE, INVR.INTEREST_RT INVR_INTEREST_RT, INVR.ADJUSTMENT_AMT INVR_ADJUSTMENT_AMT, INVR.RENWAL_OPER_TYPE INVR_RENWAL_OPER_TYPE FROM ' . $this->iat2name . ' IAT2, ' . $this->infuname . ' INFU, ' . $this->invrname . ' INVR WHERE INFU.IDCODE = IAT2.PORTFOLIO_CODE AND IAT2.INVR_ID = INVR.ID AND IAT2.BATCH_ID = ' . singleQuote($this->batch_id);
            $rs = ClxCommon::$db->executeQuery($sql);
            while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
                $renew_oper_type = $r['INVR_RENWAL_OPER_TYPE'];
                $new_investment_code = $r['INVR_NEW_INVEST_CODE'];
                $new_interest_rt = $r['INVR_INTEREST_RT'];
                $adjustment_amt = $r['INVR_ADJUSTMENT_AMT'];
                unset($r['INVR_NEW_INVEST_CODE'], $r['INVR_INTEREST_RT'], $r['INVR_ADJUSTMENT_AMT'], $r['INVR_RENWAL_OPER_TYPE']);
                $inatrec = array_merge($inat_def_rec, $r);
                $inatrec ['REVACTI_CODE'] = 'null';
                $inatrec ['FROM_CONTRACT_CODE'] = 'null';
                $inatrec ['FROM_PARTICIPANT_ID'] = 'null';
                $inatrec ['TO_PORTFOLIO_CODE'] = 'null';
                $inatrec ['INVCRED_RATING'] = 'null';
                $inatrec ['JE_COMPLETED'] = CLX_INIT_ZERO;
                $inatrec['GAIN_AMT'] = $inatrec['AMORTIZED_AMT'];
                if ($inatrec['FACE_VALUE'] <> CLX_NUM_ZERO) {
                    $inatrec ['AMORTIZED_AMT'] = ($inatrec['INVEST_AMT'] - $inatrec['FACE_VALUE']) - $inatrec['AMORTIZED_AMT'];
                } else {
                    $inatrec ['AMORTIZED_AMT'] = CLX_INIT_ZERO;
                }
                //Check activity type
                if ($renew_oper_type === 'CAP') {
                    //a. Renovación de capital (pagar intereses y reducir capital si aplica) RCAP
                    //Solo se pagara el capital en la renovacion
                    $inatrec['INVEST_ACTIVITY'] = 'RCAP';
                    $inatrec['CASH_AMT'] = round($inatrec['ACCRUED_INT_AMT'] - $adjustment_amt, CLX_NUM_TWO);
                    $inatrec['INVEST_AMT'] = round(CLX_NUM_ZERO + $adjustment_amt, CLX_NUM_TWO);
                    $inatrec['FACE_VALUE'] = round(CLX_NUM_ZERO + $adjustment_amt, CLX_NUM_TWO);
                    //Ver si hay incremento realizar una compra aparte
                } else {//CAI
                    //b. Renovación de capital + Intereses(Capitalización) RCAI
                    $inatrec['INVEST_ACTIVITY'] = 'RCAI';
                    $inatrec['CASH_AMT'] = round(CLX_NUM_ZERO + $adjustment_amt, CLX_NUM_TWO);
                    $inatrec['INVEST_AMT'] = round($inatrec['ACCRUED_INT_AMT'] + $adjustment_amt, CLX_NUM_TWO);
                    $inatrec['FACE_VALUE'] = round($inatrec['ACCRUED_INT_AMT'] + $adjustment_amt, CLX_NUM_TWO);
                }
                if ($inatrec['CASH_AMT'] == CLX_NUM_ZERO && $inatrec['INVEST_AMT'] == CLX_NUM_ZERO && $inatrec['FACE_VALUE'] == CLX_NUM_ZERO && $inatrec ['AMORTIZED_AMT'] == CLX_NUM_ZERO) {
                    continue;
                }
                //Building activity
                $idcode = $inat->inat_build($inatrec);
                //Building accounting
                $inat->inat_build_accounting_batch($inatrec['COMP_CODE'], $inatrec['EFFTDATE'], array(), $idcode);
            }
			ClxCommon::$db->freeResult($rs);
        }

        // AMORTIZATION
        /**
         * Runs amortization by company
         * @param string $comp_code
         */
        function infa_run_amortization_by_comp($comp_code) {
            //Procesar fecha valor primero
            $this->infa_valorization($comp_code, null, true);
            $this->infa_amortization($comp_code);
            //Valorizar para tomar monto amortizado
            $this->infa_valorization($comp_code, null, false);
            $this->clearTemporaryTables($this->batch_id);
        }

        /**
         * Runs amortization by btn
         */
        function infa_run_amortization_btn() {
            $idlist = getIdlist();
            $sql = 'SELECT COMP_CODE, IDCODE FROM ' . $this->infuname . ' WHERE ID IN (' . toString($idlist) . ')';
            $rs = ClxCommon::$db->executeQuery($sql);
            while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
                //Procesar fecha valor primero
                $this->infa_valorization($r ['COMP_CODE'], $r ['IDCODE'], true);
                $this->infa_amortization($r ['COMP_CODE'], $r ['IDCODE']);
                //Procesar valorizacion por amortizacion
                $this->infa_valorization($r ['COMP_CODE'], $r ['IDCODE'], false);
            }
			ClxCommon::$db->freeResult($rs);
            $this->clearTemporaryTables($this->batch_id);
        }

        /**
         * Builds amortization
         * @param string $comp_code
         * @param string [$portfolio_code]
         */
        private function infa_amortization($comp_code, $portfolio_code = '') {
            $this->je_source = 'INAP';
            if (!class_exists('Infu'))
                includeCodeLib('INFU');
            if (!class_exists('Comp'))
                includeCodeLib('COMP');
            $infu = new Infu ();
            $comp = new Comp ();
            $op = $comp->comp_operation_parameters($comp_code);
            $operation_date = $op ['OPERATION_DATE'];
            if ($this->display === true) {
                echoLog(' AMORTIZATION. BATCH ID: ' . $this->batch_id);
                echoLog('Efective date: ' . $operation_date);
                echoLog('Compcode: ' . $comp_code);
                echoLog('Portfolio: ' . $portfolio_code);
            }
            $this->clearTemporaryTables($this->batch_id);
            // Generando registros temporales con monto a amortizar
            //Amortizacion Lineal es prima/descuento entre nro de dias restantes
            $sql = 'INSERT INTO ' . $this->atmpname . ' (COMP_CODE, POSITION_NUMBER, INVESTMENT_CODE, PORTFOLIO_CODE, PRODUCT_CODE, MATURITY_DATE, INVEST_AMT, FACE_VALUE, PREMIUM_DISC_AMT, AMORTIZED_AMT, BATCH_ID, GL01_CODE, GL02_CODE, IS_FIXINCOME, CHART_FIELD01, CHART_FIELD02, CURRENCY_CODE, EXCHANGE_RT, CUR_OPERATOR, EFFTDATE) '
                    . 'SELECT ' . singleQuote($comp_code) . ' COMP_CODE, INFA.POSITION_NUMBER, INFA.INVESTMENT_CODE, INFA.PORTFOLIO_CODE, INPR.PRODUCT_CODE, INPR.MATURITY_DATE, INFA.INVEST_AMT, INFA.FACE_VALUE, INFA.PREMIUM_DISC_AMT, -1*ROUND((INFA.PREMIUM_DISC_AMT/NULLIF(DATEDIFF(ADDDATE(INPR.MATURITY_DATE, INTERVAL -1 DAY),' . singleQuote($operation_date) . '),0)),2) AMORTIZED_AMT, ' . singleQuote($this->batch_id) . ' BATCH_ID, null GL01_CODE, null GL02_CODE, INPR.IS_FIXINCOME, INFA.CONTRACT_CODE CHART_FIELD01, INFA.PARTICIPANT_ID CHART_FIELD02, INFA.CURRENCY_CODE, 1 EXCHANGE_RT, 1 CUR_OPERATOR, ' . singleQuote($operation_date) . ' EFFTDATE '
                    . 'FROM ' . $this->inprname . ' INPR, ' . $this->infaname . ' INFA WHERE INPR.INVESTMENT_CODE = INFA.INVESTMENT_CODE AND INPR.INV_AMORT_TYPE = ' . singleQuote('L') . ' AND INPR.CLX_EFFTDATE = (SELECT MAX(INPRX.CLX_EFFTDATE) FROM ' . $this->inprname . ' INPRX WHERE INPRX.INVESTMENT_CODE = INPR.INVESTMENT_CODE AND INPRX.CLX_EFFTDATE<=' . singleQuote($operation_date) . ') AND (INPR.MATURITY_DATE IS NULL OR INPR.MATURITY_DATE >' . singleQuote($operation_date) . ') AND INFA.STOP_AMORTIZATION = 0 AND INFA.FACE_VALUE <> 0 AND INPR.IS_FIXINCOME = 1 AND INFA.COMP_CODE = ' . singleQuote($comp_code) . ' AND INFA.PREMIUM_DISC_AMT <>0 ' . (($portfolio_code <> '') ? 'AND INFA.PORTFOLIO_CODE =' . singleQuote($portfolio_code) : '');
            ClxCommon::$db->executeQuery($sql);

            $sql = 'INSERT INTO ' . $this->atmpname . ' (COMP_CODE, POSITION_NUMBER, INVESTMENT_CODE, PORTFOLIO_CODE, PRODUCT_CODE, MATURITY_DATE, INVEST_AMT, FACE_VALUE, PREMIUM_DISC_AMT, COUPON_RT, AMORTIZED_AMT, BATCH_ID, GL01_CODE, GL02_CODE, IS_FIXINCOME, CHART_FIELD01, CHART_FIELD02, CURRENCY_CODE, EXCHANGE_RT, CUR_OPERATOR, EFFTDATE, INV_AMORT_TYPE, ACQUISITION_DATE, PROD_EFFTDATE, INTEREST_CALC, PAYMENT_FREQ, YEAR_DAYS, OPERATION_DATE) '
                    . 'SELECT ' . singleQuote($comp_code) . ' COMP_CODE, INFA.POSITION_NUMBER, INFA.INVESTMENT_CODE, INFA.PORTFOLIO_CODE, INPR.PRODUCT_CODE, INPR.MATURITY_DATE, INFA.INVEST_AMT, INFA.FACE_VALUE, INFA.PREMIUM_DISC_AMT, INFA.PD_YIELD_RT COUPON_RT, 0 AMORTIZED_AMT, ' . singleQuote($this->batch_id) . ' BATCH_ID, null GL01_CODE, null GL02_CODE, INPR.IS_FIXINCOME, INFA.CONTRACT_CODE CHART_FIELD01, INFA.PARTICIPANT_ID CHART_FIELD02, INFA.CURRENCY_CODE, 1 EXCHANGE_RT, 1 CUR_OPERATOR, ' . singleQuote($operation_date) . ' EFFTDATE, INPR.INV_AMORT_TYPE, INFA.ACQUISITION_DATE, INPR.CLX_EFFTDATE PROD_EFFTDATE, INPR.INTEREST_CALC, INPR.PAYMENT_FREQ, (case INPR.INTEREST_CALC WHEN ' . singleQuote(CLX_CHAR_EMPTY_STRING) . ' THEN 0 WHEN ' . singleQuote('AC/360') . ' THEN 360 WHEN ' . singleQuote('30/360') . ' THEN 360 WHEN ' . singleQuote('AC/360') . ' THEN 365 WHEN ' . singleQuote('30/365') . ' THEN 365 ELSE 360 END) YEAR_DAYS, ' . singleQuote($operation_date) . ' OPERATION_DATE FROM ' . $this->inprname . ' INPR, ' . $this->infaname . ' INFA '
                    . 'WHERE INPR.INVESTMENT_CODE = INFA.INVESTMENT_CODE AND INPR.INV_AMORT_TYPE = ' . singleQuote('S') . ' AND INPR.CLX_EFFTDATE = (SELECT MAX(INPRX.CLX_EFFTDATE) FROM ' . $this->inprname . ' INPRX WHERE INPRX.INVESTMENT_CODE = INPR.INVESTMENT_CODE AND INPRX.CLX_EFFTDATE<=' . singleQuote($operation_date) . ') AND (INPR.MATURITY_DATE IS NULL OR INPR.MATURITY_DATE >' . singleQuote($operation_date) . ') AND INFA.STOP_AMORTIZATION = 0 AND INFA.FACE_VALUE <> 0 AND INPR.IS_FIXINCOME = 1 AND INFA.PREMIUM_DISC_AMT <> 0 AND INFA.COMP_CODE = ' . singleQuote($comp_code) . ' ' . (($portfolio_code <> '') ? 'AND INFA.PORTFOLIO_CODE =' . singleQuote($portfolio_code) : '');
            ClxCommon::$db->executeQuery($sql);

            /* Defining next coupon or payment date */
            $sql = 'UPDATE ' . $this->atmpname . ' ATMP, ' . $this->inasname . ' INAS '
                    . 'SET ATMP.PAYMENT_DATE = INAS.PAYMENT_DATE, ATMP.START_DATE = INAS.START_DATE '
                    . 'WHERE ATMP.INVESTMENT_CODE = INAS.INVESTPROD_CODE AND ATMP.PROD_EFFTDATE = INAS.EFFTDATE AND INAS.START_DATE<=' . singleQuote($operation_date) . ' AND INAS.PAYMENT_DATE>' . singleQuote($operation_date) . ' AND ATMP.BATCH_ID = ' . singleQuote($this->batch_id) . ' AND ATMP.INV_AMORT_TYPE = ' . singleQuote('S');
            ClxCommon::$db->executeQuery($sql);

            $sql = 'UPDATE ' . $this->atmpname . ' '
                    . 'SET START_DATE = ADDDATE(STR_TO_DATE(CONCAT(YEAR(OPERATION_DATE),\'-\',MONTH(OPERATION_DATE),\'-\',DAY(PAYMENT_DATE)),\'%Y-%m-%d\'), INTERVAL IF(STR_TO_DATE(CONCAT(YEAR(OPERATION_DATE),\'-\',MONTH(OPERATION_DATE),\'-\',DAY(PAYMENT_DATE)),\'%Y-%m-%d\')>OPERATION_DATE, -1,0) MONTH), PAYMENT_DATE = ADDDATE(STR_TO_DATE(CONCAT(YEAR(OPERATION_DATE),\'-\',MONTH(OPERATION_DATE),\'-\',DAY(PAYMENT_DATE)),\'%Y-%m-%d\'), INTERVAL IF(STR_TO_DATE(CONCAT(YEAR(OPERATION_DATE),\'-\',MONTH(OPERATION_DATE),\'-\',DAY(PAYMENT_DATE)),\'%Y-%m-%d\')>OPERATION_DATE, 0,1) MONTH) '
                    . 'WHERE BATCH_ID = ' . singleQuote($this->batch_id) . ' AND INTEREST_CALC NOT IN (\'AC/360\',\'AC/365\',\'AC/AC\') AND PAYMENT_FREQ IN (\'12\',\'6\',\'4\',\'3\',\'2\',\'1\',\'99\')';
            ClxCommon::$db->executeQuery($sql);

            /* Setting base calc on year days */
            $sql = 'UPDATE ' . $this->atmpname . ' SET NO_DAYS = 30 WHERE BATCH_ID = ' . singleQuote($this->batch_id) . ' AND INV_AMORT_TYPE = ' . singleQuote('S');
            ClxCommon::$db->executeQuery($sql);


            /* Determinando dias a devengar
             * 1)Fecha Operaciones, Inicio cupon y Fin cupon se determinan.
             * 2)se calcula Transcurridos dependiendo si el cupon a sido completo o no. de ser menor se coloca desde el inicio del bono
             * 3)se calculan los dias extras si la proxima fecha es el pago
             * 4)por ultima basado en los dias extras o dias que han transcurridos se calcula el numero de dias a devengar si es cero por tener mas de los dias entonces no devenga
             * de ser requeridos dias adicionales como en febrero entonces se agregan los dias. Por ultimo se devenga solo 1 dia.
             */
            $sql = 'UPDATE ' . $this->atmpname . ' SET NO_DAYS = IFNULL(IF(CAST(DATEDIFF(OPERATION_DATE, PAYMENT_DATE)+CAST(DATEDIFF(PAYMENT_DATE, START_DATE)-CAST(NO_DAYS AS SIGNED) AS SIGNED) AS SIGNED)>=0,0, IF(OPERATION_DATE != ADDDATE(PAYMENT_DATE, INTERVAL -1 DAY),  1, IF(CAST(DATEDIFF(PAYMENT_DATE, START_DATE)=NO_DAYS AS SIGNED), 1,0))) + IF(OPERATION_DATE = ADDDATE(PAYMENT_DATE,INTERVAL -1 DAY), IF(CAST(DATEDIFF(OPERATION_DATE, PAYMENT_DATE)+CAST(DATEDIFF(PAYMENT_DATE, START_DATE)-CAST(NO_DAYS AS SIGNED) AS SIGNED) AS SIGNED)<-1, -1*CAST(DATEDIFF(OPERATION_DATE, PAYMENT_DATE)+CAST(DATEDIFF(PAYMENT_DATE, START_DATE)-CAST(NO_DAYS AS SIGNED) AS SIGNED) AS SIGNED),0),0),0) WHERE BATCH_ID = ' . singleQuote($this->batch_id) . ' AND INV_AMORT_TYPE = ' . singleQuote('S');
            ClxCommon::$db->executeQuery($sql);

            /* COLOCANDO 1 SI ES ac */
            $sql = 'UPDATE ' . $this->atmpname . ' SET NO_DAYS = 1 WHERE INTEREST_CALC IN (' . singleQuote('AC/360') . ',' . singleQuote('AC/365') . ',' . singleQuote('AC/AC') . ') AND BATCH_ID = ' . singleQuote($this->batch_id) . ' AND INV_AMORT_TYPE = ' . singleQuote('S');
            ClxCommon::$db->executeQuery($sql);

            $sql = 'UPDATE ' . $this->atmpname . ' ATMP SET ATMP.AMORTIZED_AMT = IFNULL(ROUND((ATMP.INVEST_AMT*(ATMP.COUPON_RT/100)/nullIF(ATMP.YEAR_DAYS,0)* ATMP.NO_DAYS), 5),0) WHERE ATMP.BATCH_ID = ' . singleQuote($this->batch_id) . ' AND ATMP.INV_AMORT_TYPE = ' . singleQuote('S');
            ClxCommon::$db->executeQuery($sql);

            //Ultima amortizacion validando diferencia casos con mayor amortizacion
            $sql = 'UPDATE  ' . $this->atmpname . ' SET AMORTIZED_AMT = -1*(INVEST_AMT-FACE_VALUE) WHERE COUPON_RT > 0 AND (INVEST_AMT-FACE_VALUE)>0 AND BATCH_ID = ' . singleQuote($this->batch_id) . ' AND INV_AMORT_TYPE = ' . singleQuote('S');
            ClxCommon::$db->executeQuery($sql);

            $sql = 'UPDATE  ' . $this->atmpname . ' SET AMORTIZED_AMT = -1*(INVEST_AMT-FACE_VALUE) WHERE COUPON_RT < 0 AND (INVEST_AMT-FACE_VALUE)<0 AND BATCH_ID = ' . singleQuote($this->batch_id) . ' AND INV_AMORT_TYPE = ' . singleQuote('S');
            ClxCommon::$db->executeQuery($sql);

            //Amortizando diferencia en casos a 1 dia del vencimiento
            $sql = 'UPDATE  ' . $this->atmpname . ' SET AMORTIZED_AMT = -1*(INVEST_AMT-FACE_VALUE) WHERE INVEST_AMT<FACE_VALUE AND MATURITY_DATE = ADDDATE(OPERATION_DATE, INTERVAL 1 DAY) AND BATCH_ID = ' . singleQuote($this->batch_id);
            ClxCommon::$db->executeQuery($sql);

            $sql = 'UPDATE  ' . $this->atmpname . ' SET AMORTIZED_AMT = -1*(INVEST_AMT-FACE_VALUE) WHERE INVEST_AMT>FACE_VALUE AND MATURITY_DATE = ADDDATE(OPERATION_DATE, INTERVAL 1 DAY) AND BATCH_ID = ' . singleQuote($this->batch_id);
            ClxCommon::$db->executeQuery($sql);

            //casos con monto aproximado en ultima iteracion
            $sql = 'UPDATE  ' . $this->atmpname . ' SET AMORTIZED_AMT = -1*(INVEST_AMT-FACE_VALUE) WHERE INVEST_AMT<FACE_VALUE AND (INVEST_AMT-FACE_VALUE)+AMORTIZED_AMT>0 AND BATCH_ID = ' . singleQuote($this->batch_id) . ' AND INV_AMORT_TYPE = ' . singleQuote('S');
            ClxCommon::$db->executeQuery($sql);
            $sql = 'UPDATE  ' . $this->atmpname . ' SET AMORTIZED_AMT = (INVEST_AMT-FACE_VALUE) WHERE INVEST_AMT>FACE_VALUE AND (INVEST_AMT-FACE_VALUE)+AMORTIZED_AMT<0 AND BATCH_ID = ' . singleQuote($this->batch_id) . ' AND INV_AMORT_TYPE = ' . singleQuote('S');
            ClxCommon::$db->executeQuery($sql);



            //AMORTIZED_AMT
            $sql = 'UPDATE ' . $this->infaname . ' INFA, ' . $this->atmpname . ' ATMP SET INFA.AMORTIZATION_ACC = INFA.AMORTIZATION_ACC+ATMP.AMORTIZED_AMT WHERE INFA.POSITION_NUMBER = ATMP.POSITION_NUMBER AND INFA.PORTFOLIO_CODE = ATMP.PORTFOLIO_CODE';
            ClxCommon::$db->executeQuery($sql);
            $sql = 'UPDATE ' . $this->infaname . ' INFA, ' . $this->atmpname . ' ATMP SET ATMP.AMORTIZED_AMT = ROUND(INFA.AMORTIZATION_ACC, 2) WHERE INFA.POSITION_NUMBER = ATMP.POSITION_NUMBER AND INFA.PORTFOLIO_CODE = ATMP.PORTFOLIO_CODE';
            ClxCommon::$db->executeQuery($sql);

            //I.INVEST_AMT-I.FACE_VALUE
            $sql = 'DELETE FROM ' . $this->atmpname . ' WHERE BATCH_ID = ' . singleQuote($this->batch_id) . ' AND AMORTIZED_AMT = 0';
            ClxCommon::$db->executeQuery($sql);
            $sql = 'UPDATE ' . $this->atmpname . ' SET AMORTIZED_AMT = PREMIUM_DISC_AMT WHERE PREMIUM_DISC_AMT < AMORTIZED_AMT AND PREMIUM_DISC_AMT>0';
            ClxCommon::$db->executeQuery($sql);
            $sql = 'UPDATE ' . $this->atmpname . ' SET AMORTIZED_AMT = PREMIUM_DISC_AMT WHERE PREMIUM_DISC_AMT > AMORTIZED_AMT AND PREMIUM_DISC_AMT<0';
            ClxCommon::$db->executeQuery($sql);

            if (ClxCommon::$db->oneExistsRecords($this->atmpname, 'BATCH_ID = ' . singleQuote($this->batch_id)) == CLX_NUM_ZERO) {
                return;
            }
            //Buscando cuentas contables
            $sql = 'UPDATE ' . $this->atmpname . ' ATMP, ' . $this->pratname . ' PRAT SET ATMP.GL01_CODE = PRAT.PYD2D_GL_CODE, ATMP.GL02_CODE = PRAT.DISCOUNT_GL_CODE WHERE ATMP.PRODUCT_CODE = PRAT.PRODUCT_CODE AND PRAT.PROD_STATUS = ' . singleQuote('VIG') . ' AND ATMP.BATCH_ID = ' . singleQuote($this->batch_id) . ' AND ATMP.PREMIUM_DISC_AMT<0';
            ClxCommon::$db->executeQuery($sql);

            //cuenta de credito para activos > 0
            $sql = 'UPDATE ' . $this->atmpname . ' ATMP, ' . $this->pratname . ' PRAT SET ATMP.GL01_CODE = PRAT.PYD2P_GL_CODE, ATMP.GL02_CODE = PRAT.PREMIUM_GL_CODE WHERE ATMP.PRODUCT_CODE = PRAT.PRODUCT_CODE AND PRAT.PROD_STATUS = ' . singleQuote('VIG') . ' AND ATMP.BATCH_ID = ' . singleQuote($this->batch_id) . ' AND ATMP.PREMIUM_DISC_AMT>=0';
            ClxCommon::$db->executeQuery($sql);

            //SEE FIAS.BACK_VALUE FOR AMORTIZATION CHANGE IN BACK VALUE
            $sql = 'UPDATE ' . $this->infaname . ' INFA,  ' . $this->atmpname . ' X SET INFA.AMORTIZED_AMT = INFA.AMORTIZED_AMT+X.AMORTIZED_AMT, INFA.BOOK_VALUE_AMT = INFA.BOOK_VALUE_AMT+X.AMORTIZED_AMT, INFA.INVEST_AMT = INFA.INVEST_AMT+X.AMORTIZED_AMT, INFA.PREMIUM_DISC_AMT = INFA.PREMIUM_DISC_AMT+X.AMORTIZED_AMT, INFA.AMORTIZATION_ACC = INFA.AMORTIZATION_ACC-X.AMORTIZED_AMT WHERE X.POSITION_NUMBER = INFA.POSITION_NUMBER AND X.PORTFOLIO_CODE=INFA.PORTFOLIO_CODE AND X.BATCH_ID = ' . singleQuote($this->batch_id);
            ClxCommon::$db->executeQuery($sql);

            //Ajustando signo de asiento
            $sql = 'UPDATE ' . $this->atmpname . ' X SET X.ACCRUED_INT_AMT = -1*X.AMORTIZED_AMT WHERE X.BATCH_ID = ' . singleQuote($this->batch_id);
            ClxCommon::$db->executeQuery($sql);

            $sql = 'UPDATE ' . $this->atmpname . ' TMP SET TMP.CHART_FIELD03 = (SELECT MAX(FIAS.IDCODE) FROM ' . $this->fiasname . ' FIAS WHERE FIAS.PORTFOLIO_CODE = TMP.PORTFOLIO_CODE AND FIAS.CONTRACT_CODE = TMP.CHART_FIELD01) WHERE TMP.BATCH_ID = ' . singleQuote($this->batch_id);
            ClxCommon::$db->executeQuery($sql);

            //Setting exchange rate
            $sql = 'SELECT DISTINCT ATMP.COMP_CODE, ATMP.CURRENCY_CODE FROM ' . $this->atmpname . ' ATMP WHERE ATMP.BATCH_ID = ' . singleQuote($this->batch_id) . ' AND ATMP.COMP_CODE = ' . singleQuote($comp_code);
            $rs = ClxCommon::$db->executeQuery($sql);
            while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
                $rate = $comp->comp_current_exchange($r ['COMP_CODE'], $r ['CURRENCY_CODE'], $operation_date);
                $sql = 'UPDATE ' . $this->atmpname . ' SET CURRENCY_CODE = ' . singleQuote($r ['CURRENCY_CODE']) . ', EXCHANGE_RT = ' . $rate ['EXCHANGE_RT'] . ', CUR_OPERATOR = ' . singleQuote($rate ['CUR_OPERATOR']) . ' WHERE BATCH_ID = ' . singleQuote($this->batch_id) . ' AND CURRENCY_CODE = ' . singleQuote($r ['CURRENCY_CODE']);
                ClxCommon::$db->executeQuery($sql);
            }
			ClxCommon::$db->freeResult($rs);
            // building accounting
            $this->build_gl_from_atmp($comp_code, $portfolio_code);
            $this->clearTemporaryTables($this->batch_id);

            // Updating Totals
            $infu->infu_update_portfolio_totals($comp_code, $portfolio_code);
        }


        /**
         * Valorization process
         * @param string $comp_code
         * @param unknown_type $portfolio_code
         * @param unknown_type $position_number
         * @param unknown_type $operation_date
         */
        function infa_valorization_backvalue($comp_code, $portfolio_code, $position_number, $operation_date) {
            //$this->display = true;
            /*$this->is_back_value = true;
            return $this->infa_valorization($comp_code, $portfolio_code, $position_number, $operation_date);*/
            return;
        }

        /**
         * Valorization process
         *
         * @param string $portfolio_code
         */
        private function infa_valorization($comp_code, $portfolio_code = '', $backvalue = false) {
            $this->je_source = 'INVA';
            if (!class_exists('Infu')) {
                includeCodeLib('INFU');
            }
            if (!class_exists('Comp')) {
                includeCodeLib('COMP');
            }
            $infu = new Infu ();
            $comp = new Comp ();
            $op = $comp->comp_operation_parameters($comp_code);
            $operation_date = $op ['OPERATION_DATE'];
            if ($this->display === true) {
                echoLog(' VALORIZATION. BATCH ID: ' . $this->batch_id);
                echoLog('Efective date: ' . $operation_date);
                echoLog('Portfolio: ' . $portfolio_code);
            }
            $this->clearTemporaryTables($this->batch_id);
            // Generando registros temporales con monto a amortizar La valorizacion = precio de mercado actual - anterior x nominal, valor libro ya tiene el precio anterior entonces, tomaremos el como monto valorizacion = valorlibro - nominal*precio mercado
            $sql = 'INSERT INTO ' . $this->atmpname . ' (COMP_CODE, POSITION_NUMBER, INVESTMENT_CODE, PORTFOLIO_CODE, PRODUCT_CODE, ACQUISITION_DATE, MATURITY_DATE, INVEST_AMT, FACE_VALUE, SHARES, PREMIUM_DISC_AMT, UNIT_PRICE_PCT, AMORTIZED_AMT, MARKET_PRICE, CURRENT_PRICE, BATCH_ID, GL01_CODE, GL02_CODE, CA_GL_CODE, IS_FIXINCOME, CHART_FIELD01, CHART_FIELD02, CURRENCY_CODE, EXCHANGE_RT, CUR_OPERATOR, EFFTDATE, OPERATION_DATE) SELECT ' . singleQuote($comp_code) . ' COMP_CODE, INFA.POSITION_NUMBER, INFA.INVESTMENT_CODE, INFA.PORTFOLIO_CODE, INPR.PRODUCT_CODE, INFA.ACQUISITION_DATE, INPR.MATURITY_DATE, INFA.BOOK_VALUE_AMT INVEST_AMT, INFA.FACE_VALUE, INFA.SHARES, INFA.PREMIUM_DISC_AMT, ROUND((INFA.INVEST_AMT/NULLIF(INFA.FACE_VALUE,0))*100, 5) UNIT_PRICE_PCT, INFA.VALORIZED_AMT AMORTIZED_AMT, 0 MARKET_PRICE, ROUND((INFA.INVEST_AMT/NULLIF(INFA.FACE_VALUE,0))*100, 5) CURRENT_PRICE, ' . singleQuote($this->batch_id) . ' BATCH_ID, NULL GL01_CODE, NULL GL02_CODE, NULL CA_GL_CODE, INPR.IS_FIXINCOME, INFA.CONTRACT_CODE CHART_FIELD01, INFA.PARTICIPANT_ID CHART_FIELD02, INFA.CURRENCY_CODE, 1 EXCHANGE_RT, 1 CUR_OPERATOR, ' . singleQuote($operation_date) . ' EFFTDATE, ' . singleQuote($operation_date) . ' OPERATION_DATE FROM ' . $this->inprname . ' INPR, ' . $this->infaname . ' INFA WHERE INFA.COMP_CODE = ' . singleQuote($comp_code) . ' AND INFA.STOP_VALORIZATION = 0 AND (INFA.INVEST_CLASS <> ' . singleQuote('H') . ' OR INFA.INVEST_CLASS IS NULL) AND INFA.FACE_VALUE<>0 AND INPR.INVESTMENT_CODE = INFA.INVESTMENT_CODE AND (INPR.MATURITY_DATE IS NULL OR INPR.MATURITY_DATE >' . singleQuote($operation_date) . ') AND INFA.COMP_CODE = ' . singleQuote($comp_code) . ' AND INPR.CLX_EFFTDATE = (SELECT MAX(INPRX.CLX_EFFTDATE) FROM ' . $this->inprname . ' INPRX WHERE INPRX.INVESTMENT_CODE = INPR.INVESTMENT_CODE AND INPRX.CLX_EFFTDATE<=' . singleQuote($operation_date) . ') AND INPR.IS_FIXINCOME = 1 ' . (($portfolio_code <> '') ? 'AND INFA.PORTFOLIO_CODE =' . singleQuote($portfolio_code) : '');
            ClxCommon::$db->executeQuery($sql);

            // handling stocks use shares
            $sql = 'INSERT INTO ' . $this->atmpname . ' (COMP_CODE, POSITION_NUMBER, INVESTMENT_CODE, PORTFOLIO_CODE, PRODUCT_CODE, ACQUISITION_DATE, MATURITY_DATE, INVEST_AMT, FACE_VALUE, SHARES, PREMIUM_DISC_AMT, UNIT_PRICE, AMORTIZED_AMT, MARKET_PRICE, CURRENT_PRICE, BATCH_ID, GL01_CODE, GL02_CODE, CA_GL_CODE, IS_FIXINCOME, CHART_FIELD01, CHART_FIELD02, CURRENCY_CODE, EXCHANGE_RT, CUR_OPERATOR, EFFTDATE, OPERATION_DATE) SELECT ' . singleQuote($comp_code) . ' COMP_CODE, INFA.POSITION_NUMBER, INFA.INVESTMENT_CODE, INFA.PORTFOLIO_CODE, INPR.PRODUCT_CODE, INFA.ACQUISITION_DATE, INPR.MATURITY_DATE, INFA.INVEST_AMT, INFA.FACE_VALUE, INFA.SHARES, INFA.PREMIUM_DISC_AMT, INFA.UNIT_PRICE, INFA.VALORIZED_AMT AMORTIZED_AMT, INFA.UNIT_PRICE MARKET_PRICE, INFA.UNIT_PRICE CURRENT_PRICE, ' . singleQuote($this->batch_id) . ' BATCH_ID, NULL GL01_CODE, NULL GL02_CODE, NULL CA_GL_CODE, INPR.IS_FIXINCOME, INFA.CONTRACT_CODE CHART_FIELD01, INFA.PARTICIPANT_ID CHART_FIELD02, INFA.CURRENCY_CODE, 1 EXCHANGE_RT, 1 CUR_OPERATOR, ' . singleQuote($operation_date) . ' EFFTDATE, ' . singleQuote($operation_date) . ' OPERATION_DATE FROM ' . $this->inprname . ' INPR, ' . $this->infaname . ' INFA WHERE INPR.INVESTMENT_CODE = INFA.INVESTMENT_CODE AND INPR.CLX_EFFTDATE = (SELECT MAX(INPRX.CLX_EFFTDATE) FROM ' . $this->inprname . ' INPRX WHERE INPRX.INVESTMENT_CODE = INPR.INVESTMENT_CODE AND INPRX.CLX_EFFTDATE<=' . singleQuote($operation_date) . ') AND INPR.IS_FIXINCOME = 0 AND INFA.COMP_CODE = ' . singleQuote($comp_code) . ' AND INFA.STOP_VALORIZATION = 0 AND INFA.SHARES<>0 ' . (($portfolio_code <> '') ? 'AND INFA.PORTFOLIO_CODE =' . singleQuote($portfolio_code) : '');
            ClxCommon::$db->executeQuery($sql);

            //setting market price
            $sql = 'UPDATE ' . $this->atmpname . ' ATMP, '
                    . '(SELECT STOC.INVESTPROD_CODE, STOC.CLX_EFFTDATE, ROUND(STOC.PRICE, 5) PRICE FROM ' . $this->stocname . ' STOC WHERE STOC.CLX_EFFTDATE = (SELECT MAX(STOCX.CLX_EFFTDATE) FROM ' . $this->stocname . ' STOCX WHERE STOCX.INVESTPROD_CODE = STOC.INVESTPROD_CODE AND '.((true === $backvalue)?'STOC.CLX_EFFTDATE <= '.singleQuote($operation_date):'STOC.CLX_EFFTDATE <= '.singleQuote($operation_date)).') AND STOC.CLX_AUTH_STATUS = ' . singleQuote('AP') . ') P '
                    . 'SET ATMP.MARKET_PRICE = P.PRICE, ATMP.EFFTDATE = P.CLX_EFFTDATE '
                    . 'WHERE ATMP.INVESTMENT_CODE =  P.INVESTPROD_CODE AND P.CLX_EFFTDATE>=ATMP.ACQUISITION_DATE AND ATMP.BATCH_ID = ' . singleQuote($this->batch_id);
            ClxCommon::$db->executeQuery($sql);


            $sql = 'DELETE FROM ' . $this->atmpname . ' WHERE BATCH_ID = ' . singleQuote($this->batch_id) . ' AND MARKET_PRICE = CURRENT_PRICE OR MARKET_PRICE = 0';
            ClxCommon::$db->executeQuery($sql);

            //Nota: Variacion de valorizacion = monto amortizado - ((Precio compra - Precio mercado)*cantidad). Cantidad puede ser shares o nomto nominal.
            //$sql = 'UPDATE ' . $this->atmpname . ' ATMP SET ATMP.AMORTIZED_AMT = ROUND((ATMP.SHARES*(ATMP.MARKET_PRICE-ATMP.CURRENT_PRICE))-ATMP.AMORTIZED_AMT, 2) WHERE ATMP.BATCH_ID = ' . singleQuote($this->batch_id) . ' AND ATMP.IS_FIXINCOME = 0';
            $sql = 'UPDATE ' . $this->atmpname . ' ATMP SET ATMP.AMORTIZED_AMT = ROUND((ATMP.SHARES*(ATMP.MARKET_PRICE-ATMP.CURRENT_PRICE)), 2) WHERE ATMP.BATCH_ID = ' . singleQuote($this->batch_id) . ' AND ATMP.IS_FIXINCOME = 0';
            ClxCommon::$db->executeQuery($sql);

            $sql = 'UPDATE ' . $this->atmpname . ' ATMP SET ATMP.AMORTIZED_AMT = IFNULL(ROUND(((ATMP.FACE_VALUE*ATMP.MARKET_PRICE/100)-ATMP.INVEST_AMT), 2),0) WHERE ATMP.BATCH_ID = ' . singleQuote($this->batch_id) . ' AND ATMP.IS_FIXINCOME = 1';
            ClxCommon::$db->executeQuery($sql);

            //Se asigna fecha efectiva con fecha de cambio si es menor a fecha de operaciones o fecha de operaciones en caso contrario.
            //$sql = 'UPDATE ' . $this->atmpname . ' ATMP SET ATMP.EFFTDATE = ATMP.OPERATION_DATE WHERE ATMP.BATCH_ID = ' . singleQuote($this->batch_id) . ' AND ATMP.EFFTDATE > ATMP.OPERATION_DATE';
            if (false === $backvalue) {
                $sql = 'UPDATE ' . $this->atmpname . ' ATMP SET ATMP.EFFTDATE = ATMP.OPERATION_DATE WHERE ATMP.BATCH_ID = ' . singleQuote($this->batch_id);
                ClxCommon::$db->executeQuery($sql);
            }

            //Marcar valorizaciones anteriores a la fecha de operaciones
            $sql = 'UPDATE  ' . $this->atmpname . ' ATMP, GL_PARAMETERS_TBL GLCP SET ATMP.AMORTIZED_AMT = 0 WHERE GLCP.COMP_CODE = ATMP.COMP_CODE AND ATMP.EFFTDATE < ADDDATE(GLCP.OPERATION_DATE, INTERVAL GLCP.BCKDATE_ALLWCE DAY) AND ATMP.BATCH_ID = ' . singleQuote($this->batch_id);
            ClxCommon::$db->executeQuery($sql);

            $sql = 'DELETE FROM ' . $this->atmpname . ' WHERE BATCH_ID = ' . singleQuote($this->batch_id) . ' AND AMORTIZED_AMT = 0';
            ClxCommon::$db->executeQuery($sql);

            if (ClxCommon::$db->oneExistsRecords($this->atmpname, 'BATCH_ID = ' . singleQuote($this->batch_id)) == CLX_NUM_ZERO) {
                return;
            }
            //Colocando cuentas contables Incremento
            $sql = 'UPDATE ' . $this->atmpname . ' ATMP, ' . $this->pratname . ' PRAT SET ATMP.GL01_CODE = PRAT.VALOR2_GL_CODE, ATMP.GL02_CODE = PRAT.VALOR_GL_CODE, ATMP.VALORIZED_GAIN_AMT = ATMP.AMORTIZED_AMT, ATMP.VALORIZED_LOSS_AMT = 0 WHERE ATMP.PRODUCT_CODE = PRAT.PRODUCT_CODE AND PRAT.PROD_STATUS = ' . singleQuote('VIG') . ' AND ATMP.BATCH_ID = ' . singleQuote($this->batch_id) . ' AND ATMP.AMORTIZED_AMT>=0';
            ClxCommon::$db->executeQuery($sql);

            //cuenta de contable para decremento
            $sql = 'UPDATE ' . $this->atmpname . ' ATMP, ' . $this->pratname . ' PRAT SET ATMP.GL01_CODE = PRAT.VALOR4_GL_CODE, ATMP.GL02_CODE = PRAT.VALOR3_GL_CODE, ATMP.VALORIZED_GAIN_AMT = 0, ATMP.VALORIZED_LOSS_AMT = ATMP.AMORTIZED_AMT WHERE ATMP.PRODUCT_CODE = PRAT.PRODUCT_CODE AND PRAT.PROD_STATUS = ' . singleQuote('VIG') . ' AND ATMP.BATCH_ID = ' . singleQuote($this->batch_id) . ' AND ATMP.AMORTIZED_AMT<0';
            ClxCommon::$db->executeQuery($sql);

            /* Eliminando valoraciones donde no exista cuenta contable */
            $sql = 'DELETE FROM ' . $this->atmpname . ' WHERE (GL01_CODE = ' . singleQuote(CLX_CHAR_EMPTY_STRING) . ' OR GL02_CODE=' . singleQuote(CLX_CHAR_EMPTY_STRING) . ' OR GL01_CODE IS NULL OR GL02_CODE IS NULL) AND BATCH_ID = ' . singleQuote($this->batch_id);
            ClxCommon::$db->executeQuery($sql);

            //si es back value hay que retomar el monto de inversiones


            //Actualizando posicion amortized amount contiene monto valorizado
            $sql = 'UPDATE ' . $this->infaname . ' INFA, ' . $this->atmpname . ' ATMP SET INFA.VALORIZED_AMT = INFA.VALORIZED_AMT+ ATMP.AMORTIZED_AMT, INFA.VALORIZED_GAIN_AMT = INFA.VALORIZED_GAIN_AMT+ATMP.VALORIZED_GAIN_AMT, INFA.VALORIZED_LOSS_AMT = INFA.VALORIZED_LOSS_AMT+ATMP.VALORIZED_LOSS_AMT, INFA.BOOK_VALUE_AMT = INFA.BOOK_VALUE_AMT + ATMP.AMORTIZED_AMT WHERE INFA.PORTFOLIO_CODE = ATMP.PORTFOLIO_CODE AND INFA.POSITION_NUMBER = ATMP.POSITION_NUMBER AND ATMP.BATCH_ID = ' . singleQuote($this->batch_id) . ' AND ATMP.IS_FIXINCOME = 1';
            ClxCommon::$db->executeQuery($sql);

            //$sql = 'UPDATE ' . $this->infaname . ' INFA, ' . $this->atmpname . ' ATMP SET INFA.VALORIZED_AMT = INFA.VALORIZED_AMT+ATMP.AMORTIZED_AMT, INFA.VALORIZED_GAIN_AMT = INFA.VALORIZED_GAIN_AMT+ATMP.VALORIZED_GAIN_AMT, INFA.VALORIZED_LOSS_AMT = INFA.VALORIZED_LOSS_AMT+ATMP.VALORIZED_LOSS_AMT, INFA.BOOK_VALUE_AMT = INFA.BOOK_VALUE_AMT + ATMP.AMORTIZED_AMT, INFA.INVEST_AMT = INFA.INVEST_AMT + ATMP.AMORTIZED_AMT, INFA.UNIT_PRICE = ATMP.MARKET_PRICE WHERE INFA.PORTFOLIO_CODE = ATMP.PORTFOLIO_CODE AND INFA.POSITION_NUMBER = ATMP.POSITION_NUMBER AND ATMP.BATCH_ID = ' . singleQuote($this->batch_id) . ' AND ATMP.IS_FIXINCOME = 0';
            $sql = 'UPDATE ' . $this->infaname . ' INFA, ' . $this->atmpname . ' ATMP SET INFA.VALORIZED_AMT = INFA.VALORIZED_AMT+ATMP.AMORTIZED_AMT, INFA.VALORIZED_GAIN_AMT = INFA.VALORIZED_GAIN_AMT+ATMP.VALORIZED_GAIN_AMT, INFA.VALORIZED_LOSS_AMT = INFA.VALORIZED_LOSS_AMT+ATMP.VALORIZED_LOSS_AMT, INFA.BOOK_VALUE_AMT = INFA.BOOK_VALUE_AMT + ATMP.AMORTIZED_AMT, INFA.UNIT_PRICE = ATMP.MARKET_PRICE WHERE INFA.PORTFOLIO_CODE = ATMP.PORTFOLIO_CODE AND INFA.POSITION_NUMBER = ATMP.POSITION_NUMBER AND ATMP.BATCH_ID = ' . singleQuote($this->batch_id) . ' AND ATMP.IS_FIXINCOME = 0';
            ClxCommon::$db->executeQuery($sql);

            $sql = 'UPDATE ' . $this->atmpname . ' ATMP SET ATMP.ACCRUED_INT_AMT = -1*ATMP.AMORTIZED_AMT WHERE ATMP.BATCH_ID = ' . singleQuote($this->batch_id);
            ClxCommon::$db->executeQuery($sql);

            $sql = 'UPDATE ' . $this->atmpname . ' TMP SET TMP.CHART_FIELD03 = (SELECT MAX(FIAS.IDCODE) FROM ' . $this->fiasname . ' FIAS WHERE FIAS.PORTFOLIO_CODE = TMP.PORTFOLIO_CODE AND FIAS.CONTRACT_CODE = TMP.CHART_FIELD01) WHERE TMP.BATCH_ID = ' . singleQuote($this->batch_id);
            ClxCommon::$db->executeQuery($sql);

            //Setting exchange rate
            $sql = 'SELECT DISTINCT COMP_CODE, CURRENCY_CODE, EFFTDATE FROM ' . $this->atmpname . ' TMP WHERE TMP.BATCH_ID = ' . singleQuote($this->batch_id) . ' AND TMP.COMP_CODE = ' . singleQuote($comp_code);
            $rs = ClxCommon::$db->executeQuery($sql);
            while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
                $rate = $comp->comp_current_exchange($r ['COMP_CODE'], $r ['CURRENCY_CODE'], $r['EFFTDATE']);
                $sql = 'UPDATE ' . $this->atmpname . ' SET CURRENCY_CODE = ' . singleQuote($r ['CURRENCY_CODE']) . ', EXCHANGE_RT = ' . $rate ['EXCHANGE_RT'] . ', CUR_OPERATOR = ' . singleQuote($rate ['CUR_OPERATOR']) . ' WHERE BATCH_ID = ' . singleQuote($this->batch_id) . ' AND CURRENCY_CODE = ' . singleQuote($r ['CURRENCY_CODE']) . ' AND OPERATION_DATE =' . singleQuote($r['EFFTDATE']);
                ClxCommon::$db->executeQuery($sql);
            }
			ClxCommon::$db->freeResult($rs);

            // building accounting
            $this->build_gl_from_atmp($comp_code, $portfolio_code, true);
            $this->clearTemporaryTables($this->batch_id);

            // Updating Totals
            $infu->infu_update_portfolio_totals($comp_code, $portfolio_code);
        }

        /**
         * Build accounting from temporary table atmp
         * Processes like earnins or amortization will add record to atmp for accounting
         */
        private function build_gl_from_atmp($comp_code, $portfolio_code = '', $bv_disabled = false) {

            $sql = 'DELETE FROM ' . $this->iat3name . ' WHERE BATCH_ID = ' . singleQuote($this->batch_id);
            ClxCommon::$db->executeQuery($sql);

            //Building Lines for debit DR
            $sql = 'INSERT INTO ' . $this->iat3name . ' (BATCH_ID, EFFTDATE, SEQNUM, GLACCOUNTCODE, CHART_FIELD01, CHART_FIELD02, CHART_FIELD03, CHART_FIELD04, ITEM_CODE, ITEM_REF_NO, CURRENCY_CODE, EXCHANGE_RT, CUR_OPERATOR, DEBIT_TRANS_AMT, CREDIT_TRANS_AMT, DEBIT_AMT, CREDIT_AMT, RECONCILIED) SELECT ' . singleQuote($this->batch_id) . ' BATCH_ID, EFFTDATE, 1 SEQNUM, ATMP.GL01_CODE GLACCOUNTCODE, ATMP.CHART_FIELD01, ATMP.CHART_FIELD02, ATMP.CHART_FIELD03, null CHART_FIELD04, ' . singleQuote('INV') . ' ITEM_CODE, ATMP.POSITION_NUMBER ITEM_REF_NO, ATMP.CURRENCY_CODE, ATMP.EXCHANGE_RT, ATMP.CUR_OPERATOR, SUM(ATMP.ACCRUED_INT_AMT) DEBIT_TRANS_AMT, 0 CREDIT_TRANS_AMT, 0 DEBIT_AMT, 0 CREDIT_AMT, 0 RECONCILIED FROM ' . $this->atmpname . ' ATMP WHERE ATMP.BATCH_ID = ' . singleQuote($this->batch_id) . ' AND ATMP.COMP_CODE = ' . singleQuote($comp_code) . ' GROUP BY ATMP.GL01_CODE, ATMP.CHART_FIELD01, ATMP.CHART_FIELD02, ATMP.CHART_FIELD03, /*ATMP.PORTFOLIO_CODE,*/ ATMP.POSITION_NUMBER, ATMP.CURRENCY_CODE, ATMP.EXCHANGE_RT, ATMP.CUR_OPERATOR HAVING ROUND(SUM(ATMP.ACCRUED_INT_AMT), 2) <> 0';
            ClxCommon::$db->executeQuery($sql);
            //contrapartida credits CR
            $sql = 'INSERT INTO ' . $this->iat3name . ' (BATCH_ID, EFFTDATE, SEQNUM, GLACCOUNTCODE, CHART_FIELD01, CHART_FIELD02, CHART_FIELD03, CHART_FIELD04, ITEM_CODE, ITEM_REF_NO, CURRENCY_CODE, EXCHANGE_RT, CUR_OPERATOR, DEBIT_TRANS_AMT, CREDIT_TRANS_AMT, DEBIT_AMT, CREDIT_AMT, RECONCILIED) SELECT ' . singleQuote($this->batch_id) . ' BATCH_ID, EFFTDATE, 2 SEQNUM, ATMP.GL02_CODE GLACCOUNTCODE, ATMP.CHART_FIELD01, ATMP.CHART_FIELD02, ATMP.CHART_FIELD03, null CHART_FIELD04, ' . singleQuote('INV') . ' ITEM_CODE, ATMP.POSITION_NUMBER ITEM_REF_NO, ATMP.CURRENCY_CODE, ATMP.EXCHANGE_RT, ATMP.CUR_OPERATOR, 0 DEBIT_TRANS_AMT, SUM(ATMP.ACCRUED_INT_AMT) CREDIT_TRANS_AMT, 0 DEBIT_AMT, 0 CREDIT_AMT, 0 RECONCILIED FROM ' . $this->atmpname . ' ATMP WHERE ATMP.BATCH_ID = ' . singleQuote($this->batch_id) . ' AND ATMP.COMP_CODE = ' . singleQuote($comp_code) . ' GROUP BY ATMP.GL02_CODE, ATMP.CHART_FIELD01, ATMP.CHART_FIELD02, ATMP.CHART_FIELD03, ATMP.POSITION_NUMBER, ATMP.CURRENCY_CODE, ATMP.EXCHANGE_RT, ATMP.CUR_OPERATOR HAVING ROUND(SUM(ATMP.ACCRUED_INT_AMT), 2) <> 0';
            ClxCommon::$db->executeQuery($sql);
            // Updating debits and credits according signs
            $sql = 'UPDATE ' . $this->iat3name . ' SET CREDIT_TRANS_AMT = -1*DEBIT_TRANS_AMT, DEBIT_TRANS_AMT = 0 WHERE DEBIT_TRANS_AMT < 0 AND BATCH_ID = ' . singleQuote($this->batch_id) . ' AND SEQNUM = 1';
            ClxCommon::$db->executeQuery($sql);
            $sql = 'UPDATE ' . $this->iat3name . ' SET DEBIT_TRANS_AMT = -1*CREDIT_TRANS_AMT, CREDIT_TRANS_AMT = 0 WHERE CREDIT_TRANS_AMT < 0 AND BATCH_ID = ' . singleQuote($this->batch_id) . ' AND SEQNUM = 2';
            ClxCommon::$db->executeQuery($sql);
            //Setting conversions
            $sql = 'UPDATE ' . $this->iat3name . ' SET DEBIT_AMT = DEBIT_TRANS_AMT * POW(EXCHANGE_RT, CUR_OPERATOR) WHERE BATCH_ID = ' . singleQuote($this->batch_id) . ' AND DEBIT_TRANS_AMT<>0';
            ClxCommon::$db->executeQuery($sql);
            $sql = 'UPDATE ' . $this->iat3name . ' SET CREDIT_AMT = CREDIT_TRANS_AMT * POW(EXCHANGE_RT, CUR_OPERATOR) WHERE BATCH_ID = ' . singleQuote($this->batch_id) . ' AND CREDIT_TRANS_AMT<>0';
            ClxCommon::$db->executeQuery($sql);
            $sql = 'UPDATE ' . $this->iat3name . ' IAT3 SET IAT3.CHART_FIELD03 = null WHERE EXISTS (SELECT GLAC.ID FROM ' . $this->glacname . ' GLAC WHERE GLAC.IDCODE = IAT3.GLACCOUNTCODE AND GLAC.GL_TYPE IN (' . singleQuote('IN') . ',' . singleQuote('EX') . ',' . singleQuote('EQ') . '))';
            ClxCommon::$db->executeQuery($sql);
            //SETTING ORDER TO DISPLAY DEBITS FIRST
            $sql = 'UPDATE ' . $this->iat3name . ' IAT3 SET IAT3.SEQNUM = 1 WHERE IAT3.BATCH_ID = ' . singleQuote($this->batch_id) . ' AND IAT3.DEBIT_AMT>0';
            ClxCommon::$db->executeQuery($sql);
            $sql = 'UPDATE ' . $this->iat3name . ' IAT3 SET IAT3.SEQNUM = 2 WHERE IAT3.BATCH_ID = ' . singleQuote($this->batch_id) . ' AND IAT3.CREDIT_AMT>0';
            ClxCommon::$db->executeQuery($sql);
            // Building Journal
            if (!class_exists('Jehd'))
                includeCodeLib('JEHD');
            $jehd = new Jehd ();
            $DBLT = &getTool('DBLT');
            $idname = $DBLT->getListItem('SOTY', $this->je_source, CLX_NUM_TWO);
            $idname = substr($idname, CLX_NUM_ZERO, CLX_NUM_60);
            if ($this->display === true) {
                echoLog($idname . ' starting');
            }
            // Build JEHD
            $sql = 'SELECT DISTINCT ' . singleQuote($idname) . ' IDNAME, ATMP.COMP_CODE, ATMP.EFFTDATE, ' . singleQuote($this->je_source) . ' JE_SOURCE, 0 CLOSING_ENTRY, 0 REVERSED, ATMP.BATCH_ID, ' . singleQuote($this->username) . ' CREATEDBY, ' . singleQuote($this->username) . ' MODIFIEDBY, ' . singleQuote($this->now) . ' CDATE, ' . singleQuote($this->now) . ' MDATE FROM ' . $this->atmpname . ' ATMP, ' . $this->iat3name . ' IAT3 WHERE ATMP.BATCH_ID = IAT3.BATCH_ID AND ATMP.BATCH_ID = ' . singleQuote($this->batch_id) . ' AND ATMP.COMP_CODE = ' . singleQuote($comp_code);
            $rs = ClxCommon::$db->executeQuery($sql);
            while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
                // Generar encabezado para portafolio
                $jehd->jehd_build($r);
                $jehdr_code = $r ['IDCODE'];
                // Load Lines
                $this->build_jeln_from_iat3($jehdr_code, $r['EFFTDATE']);
                //Verificando conversion;
                $jehd->balance_exchange_rounding($jehdr_code);
                // Cambiar estatus de asientos generados por el proceso
                if (true === $bv_disabled)
                    $jehd->back_value_enabled = false;
                $jehd->jehd_approve_by_code($jehdr_code);
                $jehd->back_value_enabled = true;
                if ($this->display === true) {
                    echoLog('Approving jehd: ' . $jehdr_code . '-' . $this->je_source);
                }
            } // end while
			ClxCommon::$db->freeResult($rs);
            $sql = 'DELETE FROM ' . $this->iat3name . ' WHERE BATCH_ID = ' . singleQuote($this->batch_id);
            ClxCommon::$db->executeQuery($sql);
        }

        /**
         * Builds accounting
         *
         * @param string $portfolio_code
         */
        private function build_jeln_from_iat3(&$jehdr_code, $efftdate = '') {
            // Ajustando lineas
            $sql = 'INSERT INTO ' . $this->jelnname . ' (JEHDR_CODE, SEQNUM, GLACCOUNTCODE, CHART_FIELD01, CHART_FIELD02, CHART_FIELD03, CHART_FIELD04, ITEM_CODE, ITEM_REF_NO, CURRENCY_CODE, EXCHANGE_RT, CUR_OPERATOR, DEBIT_AMT, CREDIT_AMT, DEBIT_TRANS_AMT, CREDIT_TRANS_AMT, RECONCILIED) SELECT ' . singleQuote($jehdr_code) . ' JEHDR_CODE, SEQNUM, GLACCOUNTCODE, CHART_FIELD01, CHART_FIELD02, CHART_FIELD03, CHART_FIELD04, ITEM_CODE, ITEM_REF_NO, CURRENCY_CODE, EXCHANGE_RT, CUR_OPERATOR, DEBIT_AMT, CREDIT_AMT, DEBIT_TRANS_AMT, CREDIT_TRANS_AMT, RECONCILIED FROM ' . $this->iat3name . ' WHERE BATCH_ID = ' . singleQuote($this->batch_id);
            if($efftdate<>''){
                $sql .= ' AND EFFTDATE = '.singleQuote($efftdate);
            }
            ClxCommon::$db->executeQuery($sql);
        }

        /**
         * Returns all instruments missing gl configuration
         *
         * @return string
         */
        function has_invalid_gl_for_earnings($comp_code) {
            $return = array();
            $sql = 'SELECT DISTINCT INPR.PRODUCT_CODE FROM ' . $this->infaname . ' INFA, ' . $this->inprname . ' INPR, ' . $this->pratname . ' PRAT, ' . $this->infuname . ' INFU, ' . $this->glcpname . ' GLCP WHERE INPR.INVESTMENT_CODE = INFA.INVESTMENT_CODE AND INPR.CLX_EFFTDATE = (SELECT MAX(INPRX.CLX_EFFTDATE) FROM ' . $this->inprname . ' INPRX WHERE INPRX.INVESTMENT_CODE = INPR.INVESTMENT_CODE AND INPRX.CLX_EFFTDATE<=GLCP.OPERATION_DATE) AND INFU.IDCODE = INFA.PORTFOLIO_CODE AND GLCP.COMP_CODE = INFU.COMP_CODE AND INPR.ISSUE_DATE<=GLCP.OPERATION_DATE AND INPR.MATURITY_DATE>=GLCP.OPERATION_DATE AND PRAT.PRODUCT_CODE = INPR.PRODUCT_CODE AND PRAT.PROD_STATUS = ' . singleQuote('VIG') . ' AND INPR.CALC_INTEREST = 1 AND (PRAT.AR_GL_CODE = ' . singleQuote(CLX_CHAR_EMPTY_STRING) . ' OR PRAT.AR_GL_CODE IS NULL OR PRAT.INC_GL_CODE=' . singleQuote(CLX_CHAR_EMPTY_STRING) . ' OR PRAT.INC_GL_CODE IS NULL) AND INFU.COMP_CODE = ' . singleQuote($comp_code);
            $rs = ClxCommon::$db->executeQuery($sql);
            while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
                $return [] = $r;
            }
			ClxCommon::$db->freeResult($rs);
            return toString($return);
        }

        //TEMPORARY TABLE METHODS
        /**
         * Initialice all temporary tables
         *
         * @param unknown_type $batch_id
         */
        private function initTemporaryTables($batch_id) {
            // Load temporary table - Req on mysql
            $DBTB = &getTool('DBTB');
            $DBTB->buildTemporaryTable('ATMP');
            $DBTB->buildTemporaryTable('PTMP');
            $DBTB->buildTemporaryTable('IAT1');
            $DBTB->buildTemporaryTable('IAT2');
            $DBTB->buildTemporaryTable('IAT3');
            $DBTB->buildTemporaryTable('IAT4');
            $this->clearTemporaryTables($batch_id);
        }

        /**
         * Clear temporary tables
         * @param unknown_type $batch_id
         */
        private function clearTemporaryTables($batch_id) {
            // Clear tmp
            ClxCommon::$db->executeQuery('DELETE FROM ' . $this->atmpname . ' WHERE BATCH_ID = ' . singleQuote($batch_id));
            ClxCommon::$db->executeQuery('DELETE FROM ' . $this->ptmpname . ' WHERE BATCH_ID = ' . singleQuote($batch_id));
            ClxCommon::$db->executeQuery('DELETE FROM ' . $this->iat1name . ' WHERE BATCH_ID = ' . singleQuote($batch_id));
            ClxCommon::$db->executeQuery('DELETE FROM ' . $this->iat2name . ' WHERE BATCH_ID = ' . singleQuote($batch_id));
            ClxCommon::$db->executeQuery('DELETE FROM ' . $this->iat3name . ' WHERE BATCH_ID = ' . singleQuote($batch_id));
            ClxCommon::$db->executeQuery('DELETE FROM ' . $this->iat4name . ' WHERE BATCH_ID = ' . singleQuote($batch_id));
        }

        //EARNING SCHEDULE - CUPONERA
        /**
         * Build schedule in temporary table. used from event in listing
         */
        function earning_schedule_panel() {
            $callingpage = peekHistory();
            if (empty($callingpage['parentrecord']['INFA_POSITION_NUMBER'])) {
                return;
            }
            $this->earning_schedule($callingpage['parentrecord']['INFA_POSITION_NUMBER']);
        }

        /**
         * Buils coupon schedule
         * @param string $position_number
         */
        private function earning_schedule($position_number) {

            $DBTB = &getTool('DBTB');
            $DBTB->buildTemporaryTable('INLS');
            // $sql = 'INSERT INTO ' . $this->inlsname . ' (POSITION_NUMBER, PORTFOLIO_CODE, INVESTMENT_CODE, PAYMENT_FREQ, INTEREST_CALC, COUPON_RT, PAYMENT_DATE, INVEST_AMT, FACE_VALUE, AMORTIZATION_AMT, COUPON_AMT, YIELD_RT, PD_YIELD_RT, NO_DAYS, START_DATE, AMORTIZED_COST, DISCOUNTED_FLOW, BATCH_ID) SELECT INFA.POSITION_NUMBER, INFA.PORTFOLIO_CODE, INFA.INVESTMENT_CODE, INPR.PAYMENT_FREQ, INPR.INTEREST_CALC, INPR.COUPON_RT, INAS.PAYMENT_DATE, INFA.INVEST_AMT, INFA.FACE_VALUE, 0 AMORTIZATION_AMT, ROUND((INFA.FACE_VALUE*(INPR.COUPON_RT/100))* (case INPR.INTEREST_CALC WHEN ' . singleQuote('30/360') . ' THEN (360*1/INPR.PAYMENT_FREQ)/360 WHEN ' . singleQuote('30/365') . ' THEN (360*1/INPR.PAYMENT_FREQ)/365 WHEN ' . singleQuote('AC/360') . ' THEN DATEDIFF(INAS.PAYMENT_DATE, INAS.START_DATE)/360 WHEN ' . singleQuote('AC/360') . ' THEN DATEDIFF(INAS.PAYMENT_DATE, INAS.START_DATE)/365 WHEN ' . singleQuote('AC/AC') . ' THEN DATEDIFF(INAS.PAYMENT_DATE, INAS.START_DATE)/365 ELSE 0 END), 2)  COUPON_AMT, INFA.YIELD_RT, INFA.PD_YIELD_RT, (case INPR.INTEREST_CALC WHEN ' . singleQuote('30/360') . ' THEN 360*1/INPR.PAYMENT_FREQ WHEN ' . singleQuote('30/360') . ' THEN 360*1/INPR.PAYMENT_FREQ WHEN ' . singleQuote('AC/360') . ' THEN DATEDIFF(INAS.PAYMENT_DATE, INAS.START_DATE) WHEN ' . singleQuote('AC/360') . ' THEN DATEDIFF(INAS.PAYMENT_DATE, INAS.START_DATE) WHEN ' . singleQuote('AC/AC') . ' THEN DATEDIFF(INAS.PAYMENT_DATE, INAS.START_DATE) ELSE 0 END) NO_DAYS, INAS.START_DATE, 0 AMORTIZED_COST, ROUND((INFA.INVEST_AMT*(INFA.YIELD_RT*(case INPR.INTEREST_CALC WHEN ' . singleQuote('30/360') . ' THEN 1/INPR.PAYMENT_FREQ WHEN ' . singleQuote('30/36') . ' THEN 1/(INPR.PAYMENT_FREQ*(365/360)) WHEN ' . singleQuote('AC/360') . ' THEN DATEDIFF(INAS.PAYMENT_DATE, INAS.START_DATE)/360 WHEN ' . singleQuote('AC/360') . ' THEN DATEDIFF(INAS.PAYMENT_DATE, INAS.START_DATE)/360 WHEN ' . singleQuote('AC/AC') . ' THEN DATEDIFF(INAS.PAYMENT_DATE, INAS.START_DATE)/360 ELSE 0 END))/100), 2) DISCOUNTED_FLOW, ' . singleQuote($this->batch_id) . ' BATCH_ID FROM ' . $this->glcpname . ' GLCP, ' . $this->infaname . ' INFA, ' . $this->inasname . ' INAS, ' . $this->inprname . ' INPR WHERE INAS.INVESTPROD_CODE = INPR.INVESTMENT_CODE AND INAS.EFFTDATE = INPR.CLX_EFFTDATE AND INPR.CLX_EFFTDATE = (SELECT MAX(INPRX.CLX_EFFTDATE) FROM ' . $this->inprname . ' INPRX WHERE INPRX.INVESTMENT_CODE = INPR.INVESTMENT_CODE AND INPRX.CLX_EFFTDATE<=GLCP.OPERATION_DATE) AND INFA.INVESTMENT_CODE = INPR.INVESTMENT_CODE AND INFA.POSITION_NUMBER = ' . singleQuote($position_number) . ' AND INAS.PAYMENT_DATE>GLCP.OPERATION_DATE AND INPR.COMP_CODE = GLCP.COMP_CODE ORDER BY INAS.START_DATE ASC';

            $sql = 'INSERT INTO ' . $this->inlsname . '
                        (POSITION_NUMBER,
                        PORTFOLIO_CODE,
                        INVESTMENT_CODE,
                        PAYMENT_FREQ,
                        INTEREST_CALC,
                        COUPON_RT,
                        PAYMENT_DATE,
                        INVEST_AMT,
                        FACE_VALUE,
                        AMORTIZATION_AMT,
                        INV_AMORT_TYPE,
                        COUPON_AMT,
                        YIELD_RT,
                        PD_YIELD_RT,
                        NO_DAYS,
                        START_DATE,
                        ACQUISITION_DATE,
                        MATURITY_DATE,
                        AMORTIZED_COST,
                        DISCOUNTED_FLOW,
                        BATCH_ID)
                  SELECT
                        INFA.POSITION_NUMBER,
                        INFA.PORTFOLIO_CODE,
                        INFA.INVESTMENT_CODE,
                        INPR.PAYMENT_FREQ,
                        INPR.INTEREST_CALC,
                        INPR.COUPON_RT,
                        INAS.PAYMENT_DATE,
                        INFA.INVEST_AMT,
                        INFA.FACE_VALUE,
                        0 AMORTIZATION_AMT,
                        INPR.INV_AMORT_TYPE,
                        ROUND((INFA.FACE_VALUE*(INPR.COUPON_RT/100))*
                            (CASE INPR.INTEREST_CALC
                                WHEN \'30/360\' THEN (360*1/INPR.PAYMENT_FREQ)/360
                                WHEN \'30/365\' THEN (360*1/INPR.PAYMENT_FREQ)/365
                                WHEN \'AC/360\' THEN DATEDIFF(INAS.PAYMENT_DATE, INAS.START_DATE)/360
                                WHEN \'AC/365\' THEN DATEDIFF(INAS.PAYMENT_DATE, INAS.START_DATE)/365
                                WHEN \'AC/AC\' THEN DATEDIFF(INAS.PAYMENT_DATE, INAS.START_DATE)/365
                            ELSE 0 END),2)  COUPON_AMT,
                        INFA.YIELD_RT,
                        INFA.PD_YIELD_RT,
                        (CASE INPR.INTEREST_CALC
                            WHEN \'30/360\' THEN 360*1/INPR.PAYMENT_FREQ
                            WHEN \'30/365\' THEN 360*1/INPR.PAYMENT_FREQ
                            WHEN \'AC/360\' THEN DATEDIFF(INAS.PAYMENT_DATE, INAS.START_DATE)
                            WHEN \'AC/365\' THEN DATEDIFF(INAS.PAYMENT_DATE, INAS.START_DATE)
                            WHEN \'AC/AC\' THEN DATEDIFF(INAS.PAYMENT_DATE, INAS.START_DATE)
                        ELSE 0 END) NO_DAYS,
                    INAS.START_DATE,
                    INFA.ACQUISITION_DATE,
                    INPR.MATURITY_DATE,
                    0 AMORTIZED_COST,
                        ROUND((INFA.INVEST_AMT*(INFA.YIELD_RT*(CASE INPR.INTEREST_CALC
                            WHEN \'30/360\' THEN 1/INPR.PAYMENT_FREQ
                            WHEN \'30/365\' THEN 1/(INPR.PAYMENT_FREQ*(365/360))
                            WHEN \'AC/360\' THEN DATEDIFF(INAS.PAYMENT_DATE, INAS.START_DATE)/360
                            WHEN \'AC/365\' THEN DATEDIFF(INAS.PAYMENT_DATE, INAS.START_DATE)/360
                            WHEN \'AC/AC\' THEN DATEDIFF(INAS.PAYMENT_DATE, INAS.START_DATE)/360
                            ELSE 0
                            END))/100),2) DISCOUNTED_FLOW,
                        ' . singleQuote($this->batch_id) . ' BATCH_ID
                FROM
                        ' . $this->glcpname . ' GLCP,
                        ' . $this->infaname . ' INFA,
                        ' . $this->inasname . ' INAS,
                        ' . $this->inprname . ' INPR
                WHERE
                    INAS.INVESTPROD_CODE = INPR.INVESTMENT_CODE
                    AND INAS.EFFTDATE = INPR.CLX_EFFTDATE
                    AND INPR.CLX_EFFTDATE = (SELECT MAX(INPRX.CLX_EFFTDATE)
                                                FROM ' . $this->inprname . ' INPRX
                                                WHERE INPRX.INVESTMENT_CODE = INPR.INVESTMENT_CODE
                                                        AND INPRX.CLX_EFFTDATE<=GLCP.OPERATION_DATE)
                    AND INFA.INVESTMENT_CODE = INPR.INVESTMENT_CODE
                    AND INFA.POSITION_NUMBER = ' . singleQuote($position_number) . '
                    AND INAS.PAYMENT_DATE>GLCP.OPERATION_DATE
                    AND INPR.COMP_CODE = GLCP.COMP_CODE
                ORDER BY INAS.START_DATE ASC';

            ClxCommon::$db->executeQuery($sql);
            $sql = 'SELECT * FROM ' . $this->inlsname . ' WHERE POSITION_NUMBER = ' . singleQuote($position_number) . ' AND BATCH_ID = ' . singleQuote($this->batch_id) . ' ORDER BY PAYMENT_DATE ASC';
            $rs = ClxCommon::$db->executeQuery($sql);
            $invest_amt = CLX_INIT_ZERO;
            $total_days = 0;
            $init_premiundisc = 0;
            $flag = true;
            $qty = CLX_INIT_ZERO;
            $i = CLX_INIT_ZERO;
            while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {



                //INV_AMORT_TYPE
                //INV_AMORT_TYPE
                $isactual = false;
                switch ($r['INTEREST_CALC']) {
                    case '30/365':
                        $base = '365';
                        break;
                    case 'AC/360':
                        $isactual = true;
                        $base = '360';
                        break;
                    case 'AC/AC':
                        $isactual = true;
                        $base = '365';
                        break;
                    case 'AC/365':
                        $isactual = true;
                        $base = '365';
                        break;
                    default:
                        $base = '360';
                        break;
                }
                if ($flag === true) {
                    //Setting initial invest amount
                    $invest_amt = $r['INVEST_AMT'];
                    $qty = ClxCommon::$db->numRows($rs);
                    $flag = false;

                    if ($r['INV_AMORT_TYPE'] === 'L') {
                        if ($isactual === true) {
                            $total_days = day_diff($r['MATURITY_DATE'], $r['ACQUISITION_DATE']);
                        } else {
                            $total_days = date_diff360($r['ACQUISITION_DATE'], $r['MATURITY_DATE']);
                        }

                        $init_premiundisc = round($r['INVEST_AMT'] - $r['FACE_VALUE'], CLX_NUM_TWO);
                    }
                }

                $i++;
                $it = array();
                //Calculando cupon
                if ($r['START_DATE'] < $r['ACQUISITION_DATE']) {
                    if ($isactual === true) {
                        $it['NO_DAYS'] = day_diff($r['ACQUISITION_DATE'], $r['PAYMENT_DATE']) * -CLX_NUM_ONE;
                    } else {
                        $it['NO_DAYS'] = date_diff360($r['ACQUISITION_DATE'], $r['PAYMENT_DATE']);
                    }
                } else {
                    if ($isactual === true) {
                        $it['NO_DAYS'] = day_diff($r['START_DATE'], $r['PAYMENT_DATE']) * -CLX_NUM_ONE;
                    } else {
                        $it['NO_DAYS'] = date_diff360($r['START_DATE'], $r['PAYMENT_DATE']);
                    }
                }
                if ($r['PAYMENT_FREQ'] <> CLX_NUM_ZERO) {
                    if ($r['FACE_VALUE'] == $r['INVEST_AMT']) {
                        $it['DISCOUNTED_FLOW'] = CLX_INIT_ZERO;
                        $it['AMORTIZATION_AMT'] = CLX_INIT_ZERO;
                    } else {
                        $it['AMORTIZATION_AMT'] = round($invest_amt * ($r['PD_YIELD_RT'] * ($it['NO_DAYS'] / $base)) / CLX_NUM_HUNDRED, CLX_NUM_TWO);
                    }
                } else {
                    $it['AMORTIZATION_AMT'] = CLX_INIT_ZERO;
                }




                $it['COUPON_AMT'] = round($r['FACE_VALUE'] * ($r['COUPON_RT'] * ($it['NO_DAYS'] / $base)) / CLX_NUM_HUNDRED, CLX_NUM_TWO);



                if ($it['AMORTIZATION_AMT'] <> CLX_NUM_ZERO) {
                    $it['DISCOUNTED_FLOW'] = round(($it['COUPON_AMT'] - $it['AMORTIZATION_AMT']), CLX_NUM_TWO);
                } else {
                    $it['DISCOUNTED_FLOW'] = CLX_INIT_ZERO;
                }

                if ($r['INV_AMORT_TYPE'] === 'L') {
                    $it['AMORTIZATION_AMT'] = round(((-1 * $init_premiundisc) / $total_days) * $it['NO_DAYS'], CLX_NUM_TWO);
                }

                if ($it['AMORTIZATION_AMT'] < CLX_NUM_ZERO && (-CLX_NUM_ONE * round($invest_amt - $r['FACE_VALUE'], CLX_NUM_TWO)) > $it['AMORTIZATION_AMT']) {
                    $it['AMORTIZATION_AMT'] = $r['FACE_VALUE'] - $invest_amt;
                }
                if ($it['AMORTIZATION_AMT'] > CLX_NUM_ZERO && (-CLX_NUM_ONE * round($invest_amt - $r['FACE_VALUE'], CLX_NUM_TWO)) < $it['AMORTIZATION_AMT']) {
                    $it['AMORTIZATION_AMT'] = $r['FACE_VALUE'] - $invest_amt;
                }

                if ($i == $qty) {
                    $it['AMORTIZATION_AMT'] = $r['FACE_VALUE'] - $invest_amt;
                }
                $it['AMORTIZED_COST'] = round($invest_amt - $r['FACE_VALUE'], CLX_NUM_TWO);
                $it['INVEST_AMT'] = $invest_amt;
                $it['BALANCE_AMT'] = round($it['INVEST_AMT'] + $it['AMORTIZATION_AMT'], CLX_NUM_TWO);
                ClxCommon::$db->updateRecord($this->inlsname, $it, 'ID = ' . $r['ID']);
                $invest_amt += $it['AMORTIZATION_AMT'];
            }
			ClxCommon::$db->freeResult($rs);
        }

    }

}