<?php

/**
 * General Code File
 * For more information contact  support@alenet.com
 *
 * @author       $Author: fsantana $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2017-01-27 08:31:17 -0500 (Fri, 27 Jan 2017) $
 * @version        $Rev: 9065 $
 *
 * CHANGE LOG
 *  Included svn prop
 */
class Inpr {

    /**
     * Contructor
     *
     * @param array $procParameters
     * @return void
     */
    function __construct($procParameters = array()) {
        multiTableName($this, array('INPR', 'ININ', 'INFA', 'GLCP', 'PROD'));
        $this->batch_id = (!empty($procParameters ['BATCH_ID'])) ? $procParameters ['BATCH_ID'] : batch_id();
        $this->username = $_SESSION ['user_record'] ['username'];
        $this->now = xFormat::getNow('Y-m-d H:i:s');
    }

    /**
     * Initialice inpr form
     * Executed on init event
     */
    function inpr_init_form(&$record) {
        if (!function_exists('comp_operation_date'))
		  includeCodeLib('COMP');
        $comp = new Comp ();
        $comprec = $comp->comp_operation_parameters($record ['INPR_COMP_CODE']);
        if (UserRequest::$request['ac'] === 'CREATE') {
		  $record ['INPR_CLX_EFFTDATE'] = $comprec ['OPERATION_DATE'];
        }
        if (!empty($record ['INPR_MATURITY_DATE']) && !empty($record ['INPR_ISSUE_DATE'])) {
		  $record['INPR_ACCRUEDINT_DAYS'] = date_diff360($record ['INPR_ISSUE_DATE'], $record ['INPR_MATURITY_DATE'], $method = false);
		  if ($record ['INPR_ACCRUEDINT_DAYS'] < CLX_NUM_ZERO)
			 $record ['INPR_ACCRUEDINT_DAYS'] = CLX_INIT_ZERO;
        } else {
		  $record ['INPR_ACCRUEDINT_DAYS'] = CLX_INIT_ZERO;
        }
        if (!empty($record ['INPR_MATURITY_DATE']) && !empty($comprec ['OPERATION_DATE']) && !empty($record ['INPR_PAYMENT_FREQ']) && $comprec ['OPERATION_DATE'] <= $record ['INPR_MATURITY_DATE']) {
		  if ($record ['INPR_PAYMENT_FREQ'] === '99') {
			 $record ['INPR_CPN_COUNT'] = CLX_NUM_ONE;
		  } else {
			 if ($record ['INPR_ISSUE_DATE'] <= $comprec ['OPERATION_DATE']) {
			     $record ['INPR_CPN_COUNT'] = adodb_number_of_periods360($comprec ['OPERATION_DATE'], $record ['INPR_MATURITY_DATE'], $record ['INPR_PAYMENT_FREQ']);
			 } else {
			     $record ['INPR_CPN_COUNT'] = adodb_number_of_periods360($record ['INPR_ISSUE_DATE'], $record ['INPR_MATURITY_DATE'], $record ['INPR_PAYMENT_FREQ']);
			 }
		  }
		  if ($record ['INPR_CPN_COUNT'] < CLX_NUM_ZERO)
			 $record ['INPR_CPN_COUNT'] = CLX_INIT_ZERO;
        } else {
		  $record ['INPR_CPN_COUNT'] = CLX_INIT_ZERO;
        }
        unset($comp);
    }

// end function
    /**
     * Validate record information
     * Executed on table validate
     */
    function inpr_validate_form(&$record, &$msg) {
        // de calcular intereses validar que tenga escala, base de calculo o
        // interes
        if ($record ['INPR_CALC_INTEREST'] === true) {
		  if ($record ['INPR_COUPON_RT'] == CLX_NUM_ZERO) {
			 $msg ['INPR_COUPON_RT'] = smsg('MSG-160');
		  }
		  if (strlen($record ['INPR_INTEREST_CALC']) == CLX_NUM_ZERO) {
			 $msg ['INPR_INTEREST_CALC'] = smsg('MSG-161');
		  }
		  //Validacion
		  if ($record ['INPR_INTEREST_CALC'] !== '' && $record ['INPR_INTEREST_CALC'] !== 'AC/360' && $record ['INPR_INTEREST_CALC'] !== 'AC/365' && $record ['INPR_INTEREST_CALC'] !== '30/360' && $record ['INPR_INTEREST_CALC'] !== '30/365' && $record ['INPR_INTEREST_CALC'] !== 'AC/AC') {
			 $msg ['INPR_INTEREST_CALC'] = smsg('MSG-390');
		  }
		  if (strlen($record ['INPR_PAYMENT_FREQ']) === 'NONE') {
			 $msg ['INPR_PAYMENT_FREQ'] = smsg('MSG-162');
		  }
        }
        if (!empty($record['INPR_ISIN']) && !$this->isIsin($record['INPR_ISIN'])) {
		  $msg['INPR_ISIN'] = smsg('MSG-341');
        }
        if (!empty($record['INPR_CUSIP']) && !$this->isCusip($record['INPR_CUSIP'])) {
		  $msg['INPR_CUSIP'] = smsg('MSG-342');
        }
        $prodrec = ClxCommon::$db->getFields($this->prodname, 'HAS_ISSUE_DATE, HAS_MATURITY_DATE, HAS_FRST_COUPON_DT', 'IDCODE = ' . singleQuote($record ['INPR_PRODUCT_CODE']));
        if ($prodrec['HAS_ISSUE_DATE'] == CLX_NUM_ONE && $record ['INPR_ISSUE_DATE'] === '') {
		  $msg ['INPR_ISSUE_DATE'] = smsg('MSG-164', $record ['INPR_INVESTMENT_CODE'], $record ['INPR_ISSUE_DATE']);
        }
        if ($prodrec['HAS_MATURITY_DATE'] == CLX_NUM_ONE && $record ['INPR_MATURITY_DATE'] === '') {
		  $msg ['INPR_MATURITY_DATE'] = smsg('MSG-165', $record ['INPR_INVESTMENT_CODE'], $record ['INPR_MATURITY_DATE']);
        }
        if ($record ['INPR_ISSUE_DATE'] <> '' && $record ['INPR_MATURITY_DATE'] <> '' && ($record ['INPR_MATURITY_DATE'] < $record ['INPR_ISSUE_DATE'])) {
		  $msg ['INPR_ISSUE_DATE'] = smsg('MSG-349', $record ['INPR_INVESTMENT_CODE'], $record ['INPR_MATURITY_DATE'] . '<' . $record ['INPR_ISSUE_DATE']);
        }
        if ($prodrec['HAS_FRST_COUPON_DT'] === CLX_NUM_ONE && $record ['INPR_FIRST_PYMNT_DATE'] === '') {
		  $msg ['INPR_FIRST_PYMNT_DATE'] = smsg('MSG-430', $record ['INAT_IDCODE']);
        }
    }

// end function
    /**
     * Perform save recors post commit
     */
    function save_form_post(&$record) {
        //Updating product code on infa if change occurs
        $sql = 'UPDATE ' . $this->infaname . ' SET PRODUCT_CODE = ' . singleQuote($record ['PRODUCT_CODE']) . ' WHERE INVESTMENT_CODE = ' . singleQuote($record ['INVESTMENT_CODE']);
        ClxCommon::$db->executeQuery($sql);
        // Calculating amortizacion
        if (!class_exists('Inas'))
		  includeCodeLib('INAS');
        $inas = new Inas ();
        $inas->build_payment_schedule($record ['INVESTMENT_CODE']);
    }

// end method
    /**
     * Load payment schedules from investment
     * @param type $comp_code
     */
    function build_inpr_inas($comp_code = '') {
        if (!class_exists('Inas'))
		  includeCodeLib('INAS');
        $inas = new Inas ();
        $sql = 'SELECT DISTINCT INVESTMENT_CODE FROM INVE_PRODUCTS_TBL WHERE IS_FIXINCOME = 1 ';
        if ($comp_code <> '')
		  $sql .= ' AND COMP_CODE = ' . singleQuote($comp_code);
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
		  $inas->build_payment_schedule($r ['INVESTMENT_CODE']);
		  ClxCommon::$db->commit();
        }
		ClxCommon::$db->freeResult($rs);
    }

    /**
     * Verify isin number
     * @param string $isin
     * @return boolean
     */
    private function isIsin($isin) {
        return clx_isin($isin);
    }

    /**
     * Verify cusip digit
     * @param string $cusip
     * @return boolean
     */
    private function isCusip($cusip) {
        return isCusip($cusip);
    }

}
