<?php

/**
 * General Code File
 * For more information contact  support@alenet.com
 *
 * @author	   $Author: cramirez $
 * @copyright	Alenet, Inc.
 * @since		$Date: 2016-12-13 15:37:37 -0500 (Tue, 13 Dec 2016) $
 * @version		$Rev: 8984 $
 *
 * CHANGE LOG
 *  Included svn prop
 */
class Inrm {

    var $pct_round = CLX_NUM_TWO;
    var $debug = false; //set true to test and display formulas
    var $disable_exception = false;

    /**
     * Constructor
     * @param array [$procParameters]
     */
    function __construct($procParameters = array()) {
        multiTableName($this, array('INRM', 'INRD', 'INRA', 'FICO', 'INPR', 'FIAS', 'GLCP', 'GLAC', 'JEHD', 'JELN', 'FIAT', 'INAT', 'JABV', 'JABV', 'JAVW', 'FIBK', 'FIAE', 'PROD', 'FILO', 'INFA', 'GLCT', 'INRE', 'PRAT'));
        $this->batch_id = (!empty($procParameters['BATCH_ID'])) ? $procParameters['BATCH_ID'] : batch_id();
        $this->username = $_SESSION['user_record']['username'];
        $this->now = xFormat::getNow('Y-m-d H:i:s');
        $this->today = xFormat::getNow('Y-m-d');
    }

    /**
     * Initialice form - Executed on init event
     *
     * @param array $record
     */
    function init_form(&$record) {
        if (UserRequest::$request['ac'] === 'CREATE' || UserRequest::$request['ac'] === 'DUPLICATE') {

        }
    }

    /**
     * Events post apas save commit
     * @param array $record
     */
    function post_save_form($record) {

    }

    /**
     * Validate record information
     * Executed on table validate
     *
     * @param array $record
     * @param array $msg
     * @return boolean
     */
    function validate_form(&$record, &$msg) {
        return (count($msg) > CLX_NUM_ZERO) ? false : true;
    }

    //Policy Validation
    function check_jehd_policies(&$jehd_code, &$msg_err) {
        //Verificar que no exista una solicitud de excepcion pendiente.
        $sql = 'SELECT AUTH_STATUS FROM ' . $this->inraname . ' WHERE JEHD_FM_CODE = ' . singleQuote($jehd_code);
        $inra_status = ClxCommon::$db->getOne($sql);
        if (!empty($inra_status)) {
            switch ($inra_status) {
                case 'PEND':
                    $msg_err[] = smsg('MSG-381');
                    $treturn = false;
                    break;
                case 'AP'://aprobada solo regresa ok para que pueda aprobar la transaccion
                    $treturn = true;
                    break;
                case 'REJ':
                    $msg_err[] = smsg('MSG-380');
                    $treturn = false;
                    break;
                default :
                    break;
            }
        }
        if (!isset($treturn)) {
            //Continuar con validacion
            $sql = 'SELECT DISTINCT JEHD.COMP_CODE, JELN.CHART_FIELD01 CONTRACT_CODE FROM ' . $this->jehdname . ' JEHD, ' . $this->jelnname . ' JELN WHERE JEHD.IDCODE = JELN.JEHDR_CODE AND JEHD.IDCODE = ' . singleQuote($jehd_code);
            $rs = ClxCommon::$db->executeQuery($sql);
            while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
                ClxFreeReturn($this->check_all_policies($r['COMP_CODE'], $msg_err, $r['CONTRACT_CODE'], null, $jehd_code));
            }
			ClxCommon::$db->freeResult($rs);
            $treturn = (count($msg_err) > CLX_NUM_ZERO) ? false : true;
        }
        return $treturn;
    }

    /**
     * Runs all policies for criteria
     */
    function check_all_policies(&$comp_code, &$msg_err, $contract_code = '', $portfolio_code = '', $jehd_code = '') {
        $sql = 'SELECT INRM.ID, INRM.ALLOW_EXCEPTION FROM ' . $this->inrmname . ' INRM WHERE INRM.COMP_CODE = ' . singleQuote($comp_code) . ' AND INRM.STATUS = ' . singleQuote('A') . ' AND INRM.CLX_EFFTDATE = (SELECT MAX(INRM.CLX_EFFTDATE) FROM INVE_RISKMGMT_POLICIES INRMX WHERE INRMX.IDCODE = INRM.IDCODE AND INRMX.CLX_EFFTDATE <= NOW()) ';
        if ($contract_code <> '') {
            $sql .= ' AND (INRM.CHART_FIELD01 = ' . singleQuote($contract_code) . ' OR INRM.CHART_FIELD01 IS NULL)';
        }
        if ($portfolio_code <> '') {
            $sql .= ' AND INRM.PORTFOLIO_CODE = ' . singleQuote($portfolio_code);
        }
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
            if (!$this->check_policy($r['ID'], $msg_err, $jehd_code)) {
                //Set exception request
                if ($r['ALLOW_EXCEPTION'] == CLX_NUM_ONE) {
                    $this->request_policy_exception($r['ID'], $msg_err, $jehd_code);
                }
            }
        }
		ClxCommon::$db->freeResult($rs);
        return (count($msg_err) > CLX_NUM_ZERO) ? false : true;
    }

    /**
     * Runs policy by id
     * @param integer $policyid
     */
    private function check_policy($policyid, &$msg_err, $jehd_code = '') {
        $sql = 'SELECT INRM.COMP_CODE, INRM.CHART_FIELD01, INRM.PORTFOLIO_CODE, INRD.* FROM ' . $this->inrmname . ' INRM, ' . $this->inrdname . ' INRD WHERE INRM.IDCODE = INRD.RM_POLICY AND INRM.CLX_EFFTDATE = INRD.CLX_EFFTDATE AND INRM.ID = ' . $policyid . ' AND INRD.STATUS = ' . singleQuote('A') . ' AND INRD.CLX_AUTH_STATUS = ' . singleQuote('AP') . '';
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
            
            $r['INSTRMT_ALOC_TYPE'] = htmlspecialchars_decode($r['INSTRMT_ALOC_TYPE']);
            $r['PRODUCT_ALOC_TYPE'] = htmlspecialchars_decode($r['PRODUCT_ALOC_TYPE']);
            $r['REGION_ALOC_TYPE'] = htmlspecialchars_decode($r['REGION_ALOC_TYPE']);
            $r['INV_SECTOR_ALOC_TYPE'] = htmlspecialchars_decode($r['INV_SECTOR_ALOC_TYPE']);
            $r['INVCRED_ALOC_TYPE'] = htmlspecialchars_decode($r['INVCRED_ALOC_TYPE']);
            $r['COUNTRY_ALOC_TYPE'] = htmlspecialchars_decode($r['COUNTRY_ALOC_TYPE']);
            $r['FIAE_ALOC_TYPE'] = htmlspecialchars_decode($r['FIAE_ALOC_TYPE']);
            $r['SEC_ISSR_ALOC_TYPE'] = htmlspecialchars_decode($r['SEC_ISSR_ALOC_TYPE']);
            $r['TRUST_ITEM_ALOC_TYPE'] = htmlspecialchars_decode($r['TRUST_ITEM_ALOC_TYPE']);
            $r['GLAC_ALLOC_TYPE'] = htmlspecialchars_decode($r['GLAC_ALLOC_TYPE']);
            $r['INRD_OTH_ALLOC_TYPE'] = htmlspecialchars_decode($r['INRD_OTH_ALLOC_TYPE']);

            //Investment instrument or product code
            ClxFreeReturn($this->check_policy_inve_instrmt_code($r, $msg_err, $jehd_code));

            ClxFreeReturn($this->check_policy_product_type($r, $msg_err, $jehd_code));
            ClxFreeReturn($this->check_policy_region($r, $msg_err, $jehd_code));

            ClxFreeReturn($this->check_policy_inv_sector($r, $msg_err, $jehd_code));
            ClxFreeReturn($this->check_policy_country($r, $msg_err, $jehd_code));
            ClxFreeReturn($this->check_policy_sec_issuer($r, $msg_err, $jehd_code));
            ClxFreeReturn($this->check_policy_invcred_rating($r, $msg_err, $jehd_code));

            ClxFreeReturn($this->check_policy_gl_type($r, $msg_err, $jehd_code));
            ClxFreeReturn($this->check_policy_glaccount($r, $msg_err, $jehd_code));
            ClxFreeReturn($this->check_policy_trust_item($r, $msg_err, $jehd_code));
            ClxFreeReturn($this->check_policy_other_formulas($r, $msg_err, $jehd_code));
        }
		ClxCommon::$db->freeResult($rs);
        return (count($msg_err) > CLX_NUM_ZERO) ? false : true;
    }

    /**
     * Check limits by product code
     * @param array $policy_item
     * @param array $msg_err
     * @param string $jehd_code
     */
    private function check_policy_inve_instrmt_code(&$policy_item, &$msg_err, $jehd_code = '') {
        if (empty($policy_item['INVE_INSTRMT_CODE']))
            return true;
        if (empty($policy_item['INSTRMT_ALOC_TYPE']))
            return true;
        if (empty($policy_item['INSTRMT_CODE_PCT']) && empty($policy_item['INSTRMT_CODE_AMT']))
            return true;
        $cc = '';
        if (!empty($policy_item['CHART_FIELD01'])) {
            $cc = ' AND GL.CHART_FIELD01 = ' . singleQuote($policy_item['CHART_FIELD01']);
        }
        if (!empty($policy_item['PORTFOLIO_CODE'])) {
            $cc = ' AND FIAS.PORTFOLIO_CODE = ' . singleQuote($policy_item['PORTFOLIO_CODE']);
        }
        $cc2 = 'AND EXISTS(SELECT PRAT.ID FROM ' . $this->pratname . ' PRAT WHERE PRAT.PRO_GL_CODE = GL.GLACCOUNTCODE AND (PRAT.PRODUCT_CODE = ' . singleQuote($policy_item['INVE_INSTRMT_CODE']) . ' OR EXISTS (SELECT INRE.ID FROM ' . $this->inrename . ' INRE WHERE INRE.RM_POLICY = ' . singleQuote($policy_item['RM_POLICY']) . ' AND INRE.CLX_EFFTDATE = ' . singleQuote($policy_item['CLX_EFFTDATE']) . ' AND INRE.IDNAME = ' . singleQuote($policy_item['IDNAME']) . ' AND INRE.INVE_INSTRMT_CODE = PRAT.PRODUCT_CODE AND INRE.IS_EXCEPTION = 0)))';
        //Monto de operacion JELN
        $sql = 'SELECT IFNULL(SUM(GL.DEBIT_AMT-GL.CREDIT_AMT),0) OP_BAL FROM ' . $this->glcpname . ' GLCP, ' . $this->jehdname . ' JEHD, ' . $this->jelnname . ' GL, ' . $this->glacname . ' GLAC, ' . $this->fiasname . ' FIAS WHERE GLCP.COMP_CODE = JEHD.COMP_CODE AND GLCP.GLGROUP_CODE = GLAC.GLGROUP_CODE AND GL.JEHDR_CODE = JEHD.IDCODE AND GLAC.IDCODE = GL.GLACCOUNTCODE AND GL.CHART_FIELD03 = FIAS.IDCODE AND JEHD.IDCODE = ' . singleQuote($jehd_code) . $cc . ' ' . $cc2;
        $oper_amt = ClxCommon::$db->getOne($sql);
        if ($oper_amt == CLX_NUM_ZERO) {
            return;
        }
        if ($policy_item['INSTRMT_CODE_AMT'] <> CLX_NUM_ZERO) {
            //Saldo actual no incluye la operacion  JABV
            $sql = 'SELECT SUM(GL.BASE_BAL_AMT) BASE_BAL_AMT FROM ' . $this->jabvname . ' GL, ' . $this->fiasname . ' FIAS WHERE GL.CHART_FIELD03 = FIAS.IDCODE AND GL.COMP_CODE = ' . singleQuote($policy_item['COMP_CODE']) . $cc . $cc2;
            $bal_amt = ClxCommon::$db->getOne($sql);
            $final_bal = ($bal_amt + $oper_amt);
            if (!$this->check_policy_operator($final_bal, $policy_item['INSTRMT_ALOC_TYPE'], $policy_item['INSTRMT_CODE_AMT'])) {
                $debug_info = 'Monto Operacion: ' . $oper_amt . ' Saldo:' . $bal_amt . ' Saldo Final: ' . $final_bal . ' ' . $policy_item['INSTRMT_ALOC_TYPE'] . ' ' . $policy_item['INSTRMT_CODE_AMT'];
                $this->add_error_msg($msg_err, $policy_item['RM_POLICY'], $policy_item['IDNAME'], $final_bal, $policy_item['INSTRMT_ALOC_TYPE'], $policy_item['INSTRMT_CODE_AMT'], false, $debug_info);
            }
        }
        //Si es por porcentaje de cuenta se evalua con base en el
        //patrimonio por ejemplo total activos / Total patrimonio *100 > nro de pct JELN
        if ($policy_item['INSTRMT_CODE_PCT'] <> CLX_NUM_ZERO) {
            //JABV
            $sql = 'SELECT IFNULL(SUM(GL.BASE_BAL_AMT),0) BASE_BAL_AMT FROM ' . $this->jabvname . ' GL, ' . $this->fiasname . ' FIAS WHERE GL.CHART_FIELD03 = FIAS.IDCODE AND GL.COMP_CODE = ' . singleQuote($policy_item['COMP_CODE']) . ' ' . $cc;
            $eq_total = ClxCommon::$db->getOne($sql);
            if ($eq_total == CLX_NUM_ZERO) {
                return;
            }
            $pct = rounder(($oper_amt / $eq_total) * CLX_NUM_HUNDRED, $this->pct_round);
            if (!$this->check_policy_operator($pct, $policy_item['INSTRMT_ALOC_TYPE'], $policy_item['INSTRMT_CODE_PCT'])) {
                $debug_info = 'Monto Operacion: ' . $oper_amt . ' Saldo:' . $eq_total . ' Pct: ' . $pct . ' ' . $policy_item['INSTRMT_ALOC_TYPE'] . ' ' . $policy_item['INSTRMT_CODE_PCT'];
                $this->add_error_msg($msg_err, $policy_item['RM_POLICY'], $policy_item['IDNAME'], $pct, $policy_item['INSTRMT_ALOC_TYPE'], $policy_item['INSTRMT_CODE_PCT'], true, $debug_info);
            }
        }
    }

    /**
     * Check limits by product type
     * @param array $policy_item
     * @param array $msg_err
     * @param string $jehd_code
     */
    private function check_policy_product_type(&$policy_item, &$msg_err, $jehd_code = '') {
        if (empty($policy_item['PRODUCT_TYPE']))
            return true;
        if (empty($policy_item['PRODUCT_ALOC_TYPE']))
            return true;
        if (empty($policy_item['PRODUCT_TYPE_PCT']) && empty($policy_item['PRODUCT_TYPE_AMT']))
            return true;
        $cc = '';
        if (!empty($policy_item['CHART_FIELD01'])) {
            $cc = ' AND GL.CHART_FIELD01 = ' . singleQuote($policy_item['CHART_FIELD01']);
        }
        $cc3 = $cc;
        if (!empty($policy_item['PORTFOLIO_CODE'])) {
            $cc = ' AND FIAS.PORTFOLIO_CODE = ' . singleQuote($policy_item['PORTFOLIO_CODE']);
        }
        $cc2 = 'AND EXISTS( /*cta banco*/ SELECT FIBK.ID FROM ' . $this->fibkname . ' FIBK, ' . $this->prodname . ' PROD WHERE FIBK.BANK_ACCOUNT_NO = FIAS.FIDU_BANKACT_CODE AND PROD.IDCODE = FIBK.BANK_PROD_CODE AND (PROD.PRODUCT_TYPE = ' . singleQuote($policy_item['PRODUCT_TYPE']) . '  OR PROD.PRODUCT_TYPE IN (SELECT DISTINCT INRE.PRODUCT_TYPE FROM ' . $this->inrename . ' INRE WHERE INRE.RM_POLICY = ' . singleQuote($policy_item['RM_POLICY']) . ' AND INRE.CLX_EFFTDATE = ' . singleQuote($policy_item['CLX_EFFTDATE']) . ' AND INRE.IDNAME = ' . singleQuote($policy_item['IDNAME']) . ' AND INRE.PRODUCT_TYPE IS NOT null AND INRE.IS_EXCEPTION = 0) ) UNION/*activo*/ SELECT FIAE.ID FROM ' . $this->fiaename . ' FIAE, ' . $this->prodname . ' PROD WHERE FIAE.IDCODE = FIAS.ASSET_CODE AND PROD.IDCODE = FIAE.PRODUCT_CODE AND (PROD.PRODUCT_TYPE = ' . singleQuote($policy_item['PRODUCT_TYPE']) . ' OR PROD.PRODUCT_TYPE IN (SELECT DISTINCT INRE.PRODUCT_TYPE FROM ' . $this->inrename . ' INRE WHERE INRE.RM_POLICY = ' . singleQuote($policy_item['RM_POLICY']) . ' AND INRE.CLX_EFFTDATE = ' . singleQuote($policy_item['CLX_EFFTDATE']) . ' AND INRE.IDNAME = ' . singleQuote($policy_item['IDNAME']) . ' AND INRE.PRODUCT_TYPE IS NOT null AND INRE.IS_EXCEPTION = 0) ) UNION /*credito*/ SELECT FILO.ID FROM ' . $this->filoname . ' FILO, ' . $this->prodname . ' PROD WHERE FILO.IDCODE = FIAS.LOAN_CODE AND PROD.IDCODE = FILO.PRODUCT_CODE AND (PROD.PRODUCT_TYPE = ' . singleQuote($policy_item['PRODUCT_TYPE']) . ' OR PROD.PRODUCT_TYPE IN (SELECT DISTINCT INRE.PRODUCT_TYPE FROM ' . $this->inrename . ' INRE WHERE INRE.RM_POLICY = ' . singleQuote($policy_item['RM_POLICY']) . ' AND INRE.CLX_EFFTDATE = ' . singleQuote($policy_item['CLX_EFFTDATE']) . ' AND INRE.IDNAME = ' . singleQuote($policy_item['IDNAME']) . ' AND INRE.PRODUCT_TYPE IS NOT null AND INRE.IS_EXCEPTION = 0) ))';
        //Monto de operacion JELN
        $sql = 'SELECT IFNULL(SUM(GL.DEBIT_AMT-GL.CREDIT_AMT),0) OP_BAL FROM ' . $this->glcpname . ' GLCP, ' . $this->jehdname . ' JEHD, ' . $this->jelnname . ' GL, ' . $this->glacname . ' GLAC, ' . $this->fiasname . ' FIAS WHERE GLCP.COMP_CODE = JEHD.COMP_CODE AND GLCP.GLGROUP_CODE = GLAC.GLGROUP_CODE AND GL.JEHDR_CODE = JEHD.IDCODE AND GLAC.IDCODE = GL.GLACCOUNTCODE AND GL.CHART_FIELD03 = FIAS.IDCODE AND JEHD.IDCODE = ' . singleQuote($jehd_code) . $cc . ' ' . $cc2;
        $oper_amt = ClxCommon::$db->getOne($sql);
        //Saldo actual no incluye la operacion  JABV
        $sql = 'SELECT SUM(GL.BASE_BAL_AMT) BASE_BAL_AMT FROM ' . $this->jabvname . ' GL, ' . $this->fiasname . ' FIAS WHERE GL.CHART_FIELD03 = FIAS.IDCODE AND GL.COMP_CODE = ' . singleQuote($policy_item['COMP_CODE']) . $cc . ' ' . $cc2;
        $bal_amt = ClxCommon::$db->getOne($sql);
        $final_bal = ($bal_amt + $oper_amt);
        if ($policy_item['PRODUCT_TYPE_AMT'] <> CLX_NUM_ZERO) {
            if (!$this->check_policy_operator($final_bal, $policy_item['PRODUCT_ALOC_TYPE'], $policy_item['PRODUCT_TYPE_AMT'])) {
                $debug_info = 'Monto Operacion: ' . $oper_amt . ' Saldo:' . $bal_amt . ' Saldo Final: ' . $final_bal . ' Condicion por monto: ' . $final_bal . ' ' . $policy_item['PRODUCT_ALOC_TYPE'] . ' ' . $policy_item['PRODUCT_TYPE_AMT'];
                $this->add_error_msg($msg_err, $policy_item['RM_POLICY'], $policy_item['IDNAME'], $final_bal, $policy_item['PRODUCT_ALOC_TYPE'], $policy_item['PRODUCT_TYPE_AMT'], false, $debug_info);
            }
        }
        //Si es por porcentaje de cuenta se evalua con base en el
        //patrimonio por ejemplo total activos / Total patrimonio *100 > nro de pct JELN
        if ($policy_item['PRODUCT_TYPE_PCT'] <> CLX_NUM_ZERO) {

            $sql = 'SELECT IFNULL(SUM(GL.DEBIT_AMT-GL.CREDIT_AMT),0) OP_BAL FROM ' . $this->glcpname . ' GLCP, ' . $this->jehdname . ' JEHD, ' . $this->jelnname . ' GL, ' . $this->glacname . ' GLAC WHERE GLCP.COMP_CODE = JEHD.COMP_CODE AND GLCP.GLGROUP_CODE = GLAC.GLGROUP_CODE AND GL.JEHDR_CODE = JEHD.IDCODE AND GLAC.IDCODE = GL.GLACCOUNTCODE AND JEHD.IDCODE = ' . singleQuote($jehd_code) . ' AND GLAC.GL_TYPE = ' . singleQuote('EQ') . ' ' . $cc3;
            $oper_eq_amt = ClxCommon::$db->getOne($sql);
            //JABV
            $sql = 'SELECT SUM(GL.BASE_BAL_AMT) BASE_BAL_AMT FROM ' . $this->jabvname . ' GL WHERE GL.GL_TYPE = ' . singleQuote('EX') . ' AND GL.COMP_CODE = ' . singleQuote($policy_item['COMP_CODE']) . ' ' . $cc3;
            $eq_total = ClxCommon::$db->getOne($sql);
            $eq_total = -CLX_NUM_ONE * ($eq_total + $oper_eq_amt);
            if ($eq_total == CLX_NUM_ZERO)
                return;
            $pct = round(($final_bal / $eq_total) * CLX_NUM_HUNDRED, $this->pct_round);
            if (!$this->check_policy_operator($pct, $policy_item['PRODUCT_ALOC_TYPE'], $policy_item['PRODUCT_TYPE_PCT'])) {
                $this->add_error_msg($msg_err, $policy_item['RM_POLICY'], $policy_item['IDNAME'], $pct, $policy_item['PRODUCT_ALOC_TYPE'], $policy_item['PRODUCT_TYPE_PCT'], true);
            }
        }
    }

    /**
     * Check limits by region
     * @param array $policy_item
     * @param array $msg_err
     * @param string $jehd_code
     */
    private function check_policy_region(&$policy_item, &$msg_err, $jehd_code = '') {
        if (empty($policy_item['PRODUCT_TYPE']))
            return true;
        if (empty($policy_item['PRODUCT_ALOC_TYPE']))
            return true;
        if (empty($policy_item['PRODUCT_TYPE_PCT']) && empty($policy_item['PRODUCT_TYPE_AMT']))
            return true;
    }

    /**
     * Check limits by investment sector
     * @param array $policy_item
     * @param array $msg_err
     * @param string $jehd_code
     */
    private function check_policy_inv_sector(&$policy_item, &$msg_err, $jehd_code = '') {
        if (empty($policy_item['INV_SECTOR']))
            return true;
        if (empty($policy_item['INV_SECTOR_ALOC_TYPE']))
            return true;
        if (empty($policy_item['INV_SECTOR_PCT']) && empty($policy_item['INV_SECTOR_AMT']))
            return true;
        $comp_code = ClxCommon::$db->getField($this->jehdname, 'COMP_CODE', 'IDCODE = ' . singleQuote($jehd_code));
        $cc = '';
        if (!empty($policy_item['CHART_FIELD01'])) {
            $cc = ' AND INAT.CONTRACT_CODE = ' . singleQuote($policy_item['CHART_FIELD01']);
        }
        if (!empty($policy_item['PORTFOLIO_CODE'])) {
            $cc = ' AND FIAS.PORTFOLIO_CODE = ' . singleQuote($policy_item['PORTFOLIO_CODE']);
        }
        //si es actividad de compra o aporte monto de inversion+ajuste no puede ser mayor del monto colocado.
        $cc2 = 'AND (INPR.INV_SECTOR = ' . singleQuote($policy_item['INV_SECTOR']) . ' OR INPR.INV_SECTOR IN (SELECT DISTINCT INRE.INV_SECTOR FROM ' . $this->inrename . ' INRE WHERE INRE.RM_POLICY = ' . singleQuote($policy_item['RM_POLICY']) . ' AND INRE.CLX_EFFTDATE = ' . singleQuote($policy_item['CLX_EFFTDATE']) . ' AND INRE.IDNAME = ' . singleQuote($policy_item['IDNAME']) . ' AND INRE.INV_SECTOR IS NOT null AND INRE.IS_EXCEPTION = 0))';
        //Monto actualizado en actividad de inversion
        $sql = 'SELECT IFNULL(SUM(INAT.INVEST_AMT),0) OP_BAL FROM ' . $this->jehdname . ' JEHD, ' . $this->inatname . ' INAT, ' . $this->fiasname . ' FIAS, ' . $this->inprname . ' INPR WHERE JEHD.INAT_CODE = INAT.IDCODE AND FIAS.PORTFOLIO_CODE = INAT.PORTFOLIO_CODE AND INPR.INVESTMENT_CODE = INAT.INVESTMENT_CODE AND INAT.INVEST_ACTIVITY IN (' . singleQuote('B') . ',' . singleQuote('APIN') . ') AND JEHD.IDCODE = ' . singleQuote($jehd_code) . ' ' . $cc2 . ' AND INAT.REVACTI_CODE IS NULL AND INPR.CLX_EFFTDATE = (SELECT MAX(INPRX.CLX_EFFTDATE) FROM ' . $this->inprname . ' INPRX WHERE INPRX.INVESTMENT_CODE = INPR.INVESTMENT_CODE AND INPR.CLX_EFFTDATE<=now()) ' . $cc;
        $oper_amt = ClxCommon::$db->getOne($sql);
        if ($oper_amt == CLX_NUM_ZERO)
            return; //si el monto es cero solo retorna
        $sql = 'SELECT SUM(INFA.INVEST_AMT) INVEST_AMT FROM ' . $this->inprname . ' INPR, '.$this->infaname.' INFA WHERE INFA.COMP_CODE =  ' . singleQuote($comp_code) . ' AND INFA.INVESTMENT_CODE = INPR.INVESTMENT_CODE ' . $cc2 . ' AND INPR.CLX_EFFTDATE = (SELECT MAX(INPRX.CLX_EFFTDATE) FROM ' . $this->inprname . ' INPRX WHERE INPRX.INVESTMENT_CODE = INPR.INVESTMENT_CODE AND INPR.CLX_EFFTDATE<=now())';
        if (!empty($policy_item['CHART_FIELD01'])) {
            $sql .=' AND EXISTS (SELECT FIAS.ID FROM ' . $this->fiasname . ' FIAS WHERE FIAS.PORTFOLIO_CODE = INFA.PORTFOLIO_CODE AND FIAS.CONTRACT_CODE = ' . singleQuote($policy_item['CHART_FIELD01']) . ')';
        }
        if (!empty($policy_item['PORTFOLIO_CODE'])) {
            $sql .=' AND INFA.PORTFOLIO_CODE = ' . singleQuote($policy_item['PORTFOLIO_CODE']);
        }
        $bal_amt = ClxCommon::$db->getOne($sql);
        $final_bal = ($bal_amt + $oper_amt);
        if ($policy_item['INV_SECTOR_AMT'] <> CLX_NUM_ZERO) {
            if (!$this->check_policy_operator($final_bal, $policy_item['INV_SECTOR_ALOC_TYPE'], $policy_item['INV_SECTOR_AMT'])) {
                $this->add_error_msg($msg_err, $policy_item['RM_POLICY'], $policy_item['IDNAME'], $final_bal, $policy_item['INV_SECTOR_ALOC_TYPE'], $policy_item['INV_SECTOR_AMT'], false);
            }
        }
        //Si es por porcentaje de cuenta se evalua con base en portafolio
        if ($policy_item['INV_SECTOR_PCT'] <> CLX_NUM_ZERO) {
            //total de cartera
            $sql = 'SELECT SUM(INFA.INVEST_AMT) INVEST_AMT FROM ' . $this->inprname . ' INPR, ' . $this->infaname . ' INFA WHERE INFA.COMP_CODE =  ' . singleQuote($comp_code) . ' AND INFA.INVESTMENT_CODE = INPR.INVESTMENT_CODE AND INPR.CLX_EFFTDATE = (SELECT MAX(INPRX.CLX_EFFTDATE) FROM ' . $this->inprname . ' INPRX WHERE INPRX.INVESTMENT_CODE = INPR.INVESTMENT_CODE AND INPR.CLX_EFFTDATE<=now()) ';
            if (!empty($policy_item['CHART_FIELD01'])) {
                $sql .=' AND EXISTS (SELECT FIAS.ID FROM ' . $this->fiasname . ' FIAS WHERE FIAS.PORTFOLIO_CODE = INFA.PORTFOLIO_CODE AND FIAS.CONTRACT_CODE = ' . singleQuote($policy_item['CHART_FIELD01']) . ')';
            }
            if (!empty($policy_item['PORTFOLIO_CODE'])) {
                $sql .=' AND INFA.PORTFOLIO_CODE = ' . singleQuote($policy_item['PORTFOLIO_CODE']);
            }
            $eq_total = ClxCommon::$db->getOne($sql);
            if ($eq_total == CLX_NUM_ZERO)
                return;
            $pct = round(($final_bal / $eq_total) * CLX_NUM_HUNDRED, $this->pct_round);
            if (!$this->check_policy_operator($pct, $policy_item['INV_SECTOR_ALOC_TYPE'], $policy_item['INV_SECTOR_PCT'])) {
                $this->add_error_msg($msg_err, $policy_item['RM_POLICY'], $policy_item['IDNAME'], $pct, $policy_item['INV_SECTOR_ALOC_TYPE'], $policy_item['INV_SECTOR_PCT'], true);
            }
        }
    }

    /**
     * Check limits by investment rating
     * @param array $policy_item
     * @param array $msg_err
     * @param string $jehd_code
     */
    private function check_policy_invcred_rating(&$policy_item, &$msg_err, $jehd_code = '') {
        if (empty($policy_item['INVCRED_RATING']))
            return true;
        if (empty($policy_item['INVCRED_ALOC_TYPE']))
            return true;
        if (empty($policy_item['INVRATING_PCT']) && empty($policy_item['INVRATING_AMT']))
            return true;
        $comp_code = ClxCommon::$db->getField($this->jehdname, 'COMP_CODE', 'IDCODE = ' . singleQuote($jehd_code));
        $cc = '';
        if (!empty($policy_item['CHART_FIELD01'])) {
            $cc = ' AND INAT.CONTRACT_CODE = ' . singleQuote($policy_item['CHART_FIELD01']);
        }
        if (!empty($policy_item['PORTFOLIO_CODE'])) {
            $cc = ' AND FIAS.PORTFOLIO_CODE = ' . singleQuote($policy_item['PORTFOLIO_CODE']);
        }
        $cc2 = 'AND (INPR.INVCRED_RATING = ' . singleQuote($policy_item['INVCRED_RATING']) . ' OR INPR.INVCRED_RATING IN (SELECT DISTINCT INRE.INVCRED_RATING FROM ' . $this->inrename . ' INRE WHERE INRE.RM_POLICY = ' . singleQuote($policy_item['RM_POLICY']) . ' AND INRE.CLX_EFFTDATE = ' . singleQuote($policy_item['CLX_EFFTDATE']) . ' AND INRE.IDNAME = ' . singleQuote($policy_item['IDNAME']) . ' AND INRE.INVCRED_RATING IS NOT null AND INRE.IS_EXCEPTION = 0))';
        //si es actividad de compra o aporte monto de inversion+ajuste no puede ser mayor del monto colocado.
        //Monto actualizado en actividad de inversion
        $sql = 'SELECT IFNULL(SUM(INAT.INVEST_AMT),0) OP_BAL FROM ' . $this->jehdname . ' JEHD, ' . $this->inatname . ' INAT, ' . $this->fiasname . ' FIAS, ' . $this->inprname . ' INPR WHERE JEHD.INAT_CODE = INAT.IDCODE AND FIAS.PORTFOLIO_CODE = INAT.PORTFOLIO_CODE AND INPR.INVESTMENT_CODE = INAT.INVESTMENT_CODE AND INAT.INVEST_ACTIVITY IN (' . singleQuote('B') . ',' . singleQuote('APIN') . ') AND JEHD.IDCODE = ' . singleQuote($jehd_code) . $cc2 . ' AND INAT.REVACTI_CODE IS NULL AND INPR.CLX_EFFTDATE = (SELECT MAX(INPRX.CLX_EFFTDATE) FROM ' . $this->inprname . ' INPRX WHERE INPRX.INVESTMENT_CODE = INPR.INVESTMENT_CODE AND INPR.CLX_EFFTDATE<=now()) ' . $cc;
        $oper_amt = ClxCommon::$db->getOne($sql);
        if ($oper_amt == CLX_NUM_ZERO)
            return; //si el monto es cero solo retorna
//Buscando el saldo actual total por condicion
        $sql = 'SELECT SUM(INFA.INVEST_AMT) INVEST_AMT FROM ' . $this->inprname . ' INPR, ' . $this->infaname . ' INFA WHERE INFA.COMP_CODE =  ' . singleQuote($comp_code) . ' AND INFA.INVESTMENT_CODE = INPR.INVESTMENT_CODE ' . $cc2 . '  AND INPR.CLX_EFFTDATE = (SELECT MAX(INPRX.CLX_EFFTDATE) FROM ' . $this->inprname . ' INPRX WHERE INPRX.INVESTMENT_CODE = INPR.INVESTMENT_CODE AND INPR.CLX_EFFTDATE<=now()) ';
        if (!empty($policy_item['CHART_FIELD01'])) {
            $sql .=' AND EXISTS (SELECT FIAS.ID FROM ' . $this->fiasname . ' FIAS WHERE FIAS.PORTFOLIO_CODE = INFA.PORTFOLIO_CODE AND FIAS.CONTRACT_CODE = ' . singleQuote($policy_item['CHART_FIELD01']) . ')';
        }
        if (!empty($policy_item['PORTFOLIO_CODE'])) {
            $sql .=' AND INFA.PORTFOLIO_CODE = ' . singleQuote($policy_item['PORTFOLIO_CODE']);
        }
        $bal_amt = ClxCommon::$db->getOne($sql);
        $final_bal = ($bal_amt + $oper_amt);
        if ($policy_item['INVRATING_AMT'] <> CLX_NUM_ZERO) {
            if (!$this->check_policy_operator($final_bal, $policy_item['INVCRED_ALOC_TYPE'], $policy_item['INVRATING_AMT'])) {
                $this->add_error_msg($msg_err, $policy_item['RM_POLICY'], $policy_item['IDNAME'], $final_bal, $policy_item['INVCRED_ALOC_TYPE'], $policy_item['INVRATING_AMT'], false);
            }
        }
        //Si es por porcentaje de cuenta se evalua con base en portafolio
        if ($policy_item['INVRATING_PCT'] <> CLX_NUM_ZERO) {
            //total de cartera
            $sql = 'SELECT SUM(INFA.INVEST_AMT) INVEST_AMT FROM ' . $this->inprname . ' INPR, ' . $this->infaname . ' INFA WHERE INFA.COMP_CODE =  ' . singleQuote($comp_code) . ' AND INFA.INVESTMENT_CODE = INPR.INVESTMENT_CODE AND INPR.CLX_EFFTDATE = (SELECT MAX(INPRX.CLX_EFFTDATE) FROM ' . $this->inprname . ' INPRX WHERE INPRX.INVESTMENT_CODE = INPR.INVESTMENT_CODE AND INPR.CLX_EFFTDATE<=now()) ';
            if (!empty($policy_item['CHART_FIELD01'])) {
                $sql .=' AND EXISTS (SELECT FIAS.ID FROM ' . $this->fiasname . ' FIAS WHERE FIAS.PORTFOLIO_CODE = INFA.PORTFOLIO_CODE AND FIAS.CONTRACT_CODE = ' . singleQuote($policy_item['CHART_FIELD01']) . ')';
            }
            if (!empty($policy_item['PORTFOLIO_CODE'])) {
                $sql .=' AND INFA.PORTFOLIO_CODE = ' . singleQuote($policy_item['PORTFOLIO_CODE']);
            }
            $eq_total = ClxCommon::$db->getOne($sql);
            if ($eq_total == CLX_NUM_ZERO)
                return;
            $pct = round(($final_bal / $eq_total) * CLX_NUM_HUNDRED, $this->pct_round);
            if (!$this->check_policy_operator($pct, $policy_item['INVCRED_ALOC_TYPE'], $policy_item['INVRATING_PCT'])) {
                $this->add_error_msg($msg_err, $policy_item['RM_POLICY'], $policy_item['IDNAME'], $pct, $policy_item['INVCRED_ALOC_TYPE'], $policy_item['INVRATING_PCT'], true);
            }
        }
    }

    /**
     * Check limits by investment rating
     * @param array $policy_item
     * @param array $msg_err
     * @param string $jehd_code
     */
    private function check_policy_country(&$policy_item, &$msg_err, $jehd_code = '') {
        if (empty($policy_item['COUNTRY_ALOC_TYPE']))
            return true;
        if (empty($policy_item['COUNTRY_PCT']) && empty($policy_item['COUNTRY_AMT']))
            return true;
        $comp_code = ClxCommon::$db->getField($this->jehdname, 'COMP_CODE', 'IDCODE = ' . singleQuote($jehd_code));
        $cc = '';
        if (!empty($policy_item['CHART_FIELD01'])) {
            $cc = ' AND INAT.CONTRACT_CODE = ' . singleQuote($policy_item['CHART_FIELD01']);
        }
        if (!empty($policy_item['PORTFOLIO_CODE'])) {
            $cc = ' AND FIAS.PORTFOLIO_CODE = ' . singleQuote($policy_item['PORTFOLIO_CODE']);
        }
        if ($policy_item['COUNTRY'] === '' && $policy_item['COUNTRY_PCT'] == CLX_NUM_HUNDRED) {
            $cc2 = ' AND (INPR.COUNTRY NOT IN (SELECT DISTINCT INRE.INVCRED_RATING FROM ' . $this->inrename . ' INRE WHERE INRE.RM_POLICY = ' . singleQuote($policy_item['RM_POLICY']) . ' AND INRE.CLX_EFFTDATE = ' . singleQuote($policy_item['CLX_EFFTDATE']) . ' AND INRE.IDNAME = ' . singleQuote($policy_item['IDNAME']) . ' AND INRE.COUNTRY IS NOT null AND INRE.IS_EXCEPTION = 0))';
        } else {
            $cc2 = ' AND (INPR.COUNTRY = ' . singleQuote($policy_item['COUNTRY']) . ' OR INPR.COUNTRY IN (SELECT DISTINCT INRE.INVCRED_RATING FROM ' . $this->inrename . ' INRE WHERE INRE.RM_POLICY = ' . singleQuote($policy_item['RM_POLICY']) . ' AND INRE.CLX_EFFTDATE = ' . singleQuote($policy_item['CLX_EFFTDATE']) . ' AND INRE.IDNAME = ' . singleQuote($policy_item['IDNAME']) . ' AND INRE.COUNTRY IS NOT null AND INRE.IS_EXCEPTION = 0))';
        }
        //si es actividad de compra o aporte monto de inversion+ajuste no puede ser mayor del monto colocado.
        //Monto actualizado en actividad de inversion
        $sql = 'SELECT COUNT(JEHD.ID) FROM ' . $this->jehdname . ' JEHD, ' . $this->inatname . ' INAT WHERE JEHD.IDCODE = ' . singleQuote($jehd_code) . ' AND JEHD.INAT_CODE = INAT.IDCODE';
        $has_portfolio = ClxCommon::$db->getOne($sql);
        $sql = 'SELECT IFNULL(SUM(INAT.INVEST_AMT),0) OP_BAL FROM ' . $this->jehdname . ' JEHD, ' . $this->inatname . ' INAT, ' . $this->fiasname . ' FIAS, ' . $this->inprname . ' INPR WHERE JEHD.INAT_CODE = INAT.IDCODE AND FIAS.PORTFOLIO_CODE = INAT.PORTFOLIO_CODE AND INPR.INVESTMENT_CODE = INAT.INVESTMENT_CODE AND INAT.INVEST_ACTIVITY IN (' . singleQuoteArray(array('BY', 'APINV')) . ')  AND JEHD.IDCODE = ' . singleQuote($jehd_code) . ' ' . $cc2 . '  AND INAT.REVACTI_CODE IS NULL AND INPR.CLX_EFFTDATE = (SELECT MAX(INPRX.CLX_EFFTDATE) FROM ' . $this->inprname . ' INPRX WHERE INPRX.INVESTMENT_CODE = INPR.INVESTMENT_CODE AND INPR.CLX_EFFTDATE<=now()) ' . $cc;
        $oper_amt = ClxCommon::$db->getOne($sql);
        if ($policy_item['COUNTRY'] === '' && $policy_item['COUNTRY_PCT'] == CLX_NUM_HUNDRED && $oper_amt == CLX_NUM_ZERO && $has_portfolio > CLX_NUM_ZERO) {
            $pct = CLX_INIT_ZERO;
            $this->add_error_msg($msg_err, $policy_item['RM_POLICY'], $policy_item['IDNAME'], $pct, $policy_item['COUNTRY_ALOC_TYPE'], $policy_item['COUNTRY_PCT'], true);
            return;
        }
        if ($oper_amt == CLX_NUM_ZERO)
            return; //si el monto es cero solo retorna
        $sql = 'SELECT SUM(INFA.INVEST_AMT) INVEST_AMT FROM ' . $this->inprname . ' INPR, ' . $this->infaname . ' INFA WHERE INFA.COMP_CODE =  ' . singleQuote($comp_code) . ' AND INFA.INVESTMENT_CODE = INPR.INVESTMENT_CODE ' . $cc2 . '  AND INPR.CLX_EFFTDATE = (SELECT MAX(INPRX.CLX_EFFTDATE) FROM ' . $this->inprname . ' INPRX WHERE INPRX.INVESTMENT_CODE = INPR.INVESTMENT_CODE AND INPR.CLX_EFFTDATE<=now())';
        if (!empty($policy_item['CHART_FIELD01'])) {
            $sql .=' AND EXISTS (SELECT FIAS.ID FROM ' . $this->fiasname . ' FIAS WHERE FIAS.PORTFOLIO_CODE = INFA.PORTFOLIO_CODE AND FIAS.CONTRACT_CODE = ' . singleQuote($policy_item['CHART_FIELD01']) . ')';
        }
        if (!empty($policy_item['PORTFOLIO_CODE'])) {
            $sql .=' AND INFA.PORTFOLIO_CODE = ' . singleQuote($policy_item['PORTFOLIO_CODE']);
        }
        $bal_amt = ClxCommon::$db->getOne($sql);
        $final_bal = ($bal_amt + $oper_amt);
        if ($policy_item['COUNTRY_AMT'] <> CLX_NUM_ZERO) {
            if (!$this->check_policy_operator($final_bal, $policy_item['COUNTRY_ALOC_TYPE'], $policy_item['COUNTRY_AMT'])) {
                $this->add_error_msg($msg_err, $policy_item['RM_POLICY'], $policy_item['IDNAME'], $final_bal, $policy_item['COUNTRY_ALOC_TYPE'], $policy_item['COUNTRY_AMT'], false);
            }
        }
        //Si es por porcentaje de cuenta se evalua con base en portafolio
        if ($policy_item['COUNTRY_PCT'] <> CLX_NUM_ZERO) {
            //total de cartera
            $sql = 'SELECT SUM(INFA.INVEST_AMT) INVEST_AMT FROM ' . $this->inprname . ' INPR, '.$this->infaname.' INFA WHERE INFA.COMP_CODE =  ' . singleQuote($comp_code) . ' AND INFA.INVESTMENT_CODE = INPR.INVESTMENT_CODE AND INPR.CLX_EFFTDATE = (SELECT MAX(INPRX.CLX_EFFTDATE) FROM ' . $this->inprname . ' INPRX WHERE INPRX.INVESTMENT_CODE = INPR.INVESTMENT_CODE AND INPR.CLX_EFFTDATE<=now())';
            if (!empty($policy_item['CHART_FIELD01'])) {
                $sql .=' AND EXISTS (SELECT FIAS.ID FROM ' . $this->fiasname . ' FIAS WHERE FIAS.PORTFOLIO_CODE = INFA.PORTFOLIO_CODE AND FIAS.CONTRACT_CODE = ' . singleQuote($policy_item['CHART_FIELD01']) . ')';
            }
            if (!empty($policy_item['PORTFOLIO_CODE'])) {
                $sql .=' AND INFA.PORTFOLIO_CODE = ' . singleQuote($policy_item['PORTFOLIO_CODE']);
            }
            $eq_total = ClxCommon::$db->getOne($sql);
            if ($eq_total == CLX_NUM_ZERO)
                return;
            $pct = round(($final_bal / $eq_total) * CLX_NUM_HUNDRED, $this->pct_round);
            if (!$this->check_policy_operator($pct, $policy_item['COUNTRY_ALOC_TYPE'], $policy_item['COUNTRY_PCT'])) {
                $this->add_error_msg($msg_err, $policy_item['RM_POLICY'], $policy_item['IDNAME'], $pct, $policy_item['COUNTRY_ALOC_TYPE'], $policy_item['COUNTRY_PCT'], true);
            }
        }
    }

    /**
     * Check limits by investment rating
     * @param array $policy_item
     * @param array $msg_err
     * @param string $jehd_code
     */
    private function check_policy_sec_issuer(&$policy_item, &$msg_err, $jehd_code = '') {
        if (empty($policy_item['SEC_ISSR_ALOC_TYPE']))
            return true;
        if (empty($policy_item['SEC_ISSR_PCT']) && empty($policy_item['SEC_ISSR_AMT']))
            return true;
        $comp_code = ClxCommon::$db->getField($this->jehdname, 'COMP_CODE', 'IDCODE = ' . singleQuote($jehd_code));
        $cc = '';
        if (!empty($policy_item['CHART_FIELD01'])) {
            $cc = ' AND INAT.CONTRACT_CODE = ' . singleQuote($policy_item['CHART_FIELD01']);
        }
        if (!empty($policy_item['PORTFOLIO_CODE'])) {
            $cc = ' AND FIAS.PORTFOLIO_CODE = ' . singleQuote($policy_item['PORTFOLIO_CODE']);
        }
        $cc2 = '';
        if (!empty($policy_item['SEC_ISSUER'])) {
            $cc2 = ' AND (INPR.SEC_ISSUER = ' . singleQuote($policy_item['SEC_ISSUER']) . ' OR EXISTS (SELECT INRE.ID FROM ' . $this->inrename . ' INRE WHERE INRE.RM_POLICY = ' . singleQuote($policy_item['RM_POLICY']) . ' AND INRE.CLX_EFFTDATE = ' . singleQuote($policy_item['CLX_EFFTDATE']) . ' AND INRE.IDNAME = ' . singleQuote($policy_item['IDNAME']) . ' AND (INRE.SEC_ISSUER = INPR.SEC_ISSUER OR INRE.INVE_INSTRMT_CODE = INPR.PRODUCT_CODE) AND INRE.IS_EXCEPTION = 0))';
        }
        if (!empty($policy_item['INVE_INSTRMT_CODE'])) {
            $cc2 = ' AND INPR.PRODUCT_CODE = ' . singleQuote($policy_item['INVE_INSTRMT_CODE']);
        }
        //si es actividad de compra o aporte monto de inversion+ajuste no puede ser mayor del monto colocado.
        //Monto actualizado en actividad de inversion
        $sql = 'SELECT IFNULL(SUM(INAT.INVEST_AMT),0) OP_BAL FROM ' . $this->jehdname . ' JEHD, ' . $this->inatname . ' INAT, ' . $this->fiasname . ' FIAS, ' . $this->inprname . ' INPR WHERE JEHD.INAT_CODE = INAT.IDCODE AND FIAS.PORTFOLIO_CODE = INAT.PORTFOLIO_CODE AND INPR.INVESTMENT_CODE = INAT.INVESTMENT_CODE AND INAT.INVEST_ACTIVITY IN (' . singleQuote('B') . ',' . singleQuote('APIN') . ', ' . singleQuote('BY') . ') AND JEHD.IDCODE = ' . singleQuote($jehd_code) . ' AND INAT.REVACTI_CODE IS NULL AND NOT EXISTS (SELECT INRE2.ID FROM ' . $this->inrename . ' INRE2 WHERE INRE2.RM_POLICY = ' . singleQuote($policy_item['RM_POLICY']) . ' AND INRE2.CLX_EFFTDATE = ' . singleQuote($policy_item['CLX_EFFTDATE']) . ' AND INRE2.IDNAME = ' . singleQuote($policy_item['IDNAME']) . ' AND (INRE2.SEC_ISSUER = INPR.SEC_ISSUER OR INRE2.INVE_INSTRMT_CODE = INPR.PRODUCT_CODE) AND INRE2.IS_EXCEPTION = 1) AND INPR.CLX_EFFTDATE = (SELECT MAX(INPRX.CLX_EFFTDATE) FROM ' . $this->inprname . ' INPRX WHERE INPRX.INVESTMENT_CODE = INPR.INVESTMENT_CODE AND INPR.CLX_EFFTDATE<=now()) ' . $cc . $cc2;
        $oper_amt = ClxCommon::$db->getOne($sql);
        if ($oper_amt == CLX_NUM_ZERO)
            return; //si el monto es cero solo retorna
        if ($policy_item['SEC_ISSR_AMT'] <> CLX_NUM_ZERO) {
            //Buscando el saldo actual total por condicion
            $sql = 'SELECT SUM(INFA.INVEST_AMT) INVEST_AMT FROM ' . $this->inprname . ' INPR, ' . $this->infaname . ' INFA WHERE INFA.COMP_CODE =  ' . singleQuote($comp_code) . ' AND NOT EXISTS (SELECT INRE2.ID FROM ' . $this->inrename . ' INRE2 WHERE INRE2.RM_POLICY = ' . singleQuote($policy_item['RM_POLICY']) . ' AND INRE2.CLX_EFFTDATE = ' . singleQuote($policy_item['CLX_EFFTDATE']) . ' AND INRE2.IDNAME = ' . singleQuote($policy_item['IDNAME']) . ' AND (INRE2.SEC_ISSUER = INPR.SEC_ISSUER OR INRE2.INVE_INSTRMT_CODE = INPR.PRODUCT_CODE) AND INRE2.IS_EXCEPTION = 1) ' . $cc2 . '  AND INPR.INVESTMENT_CODE = INFA.INVESTMENT_CODE AND INPR.CLX_EFFTDATE = (SELECT MAX(INPRX.CLX_EFFTDATE) FROM ' . $this->inprname . ' INPRX WHERE INPRX.INVESTMENT_CODE = INPR.INVESTMENT_CODE AND INPR.CLX_EFFTDATE<=now()) AND INPR.SEC_ISSUER IN (SELECT INPR2.SEC_ISSUER FROM ' . $this->inatname . ' INAT, ' . $this->jehdname . ' JEHD, ' . $this->inprname . ' INPR2 WHERE JEHD.INAT_CODE = INAT.IDCODE AND INAT.INVESTMENT_CODE = INPR2.INVESTMENT_CODE AND JEHD.IDCODE = ' . singleQuote($jehd_code) . ')';
            if (!empty($policy_item['CHART_FIELD01'])) {
                $sql .= ' AND EXISTS (SELECT FIAS.ID FROM ' . $this->fiasname . ' FIAS WHERE FIAS.PORTFOLIO_CODE = INFA.PORTFOLIO_CODE AND FIAS.CONTRACT_CODE = ' . singleQuote($policy_item['CHART_FIELD01']) . ')';
            }
            if (!empty($policy_item['PORTFOLIO_CODE'])) {
                $sql .= ' AND INFA.PORTFOLIO_CODE = ' . singleQuote($policy_item['PORTFOLIO_CODE']);
            }
            $bal_amt = ClxCommon::$db->getOne($sql);
            $final_bal = ($bal_amt + $oper_amt);
            if (!$this->check_policy_operator($final_bal, $policy_item['SEC_ISSR_ALOC_TYPE'], $policy_item['SEC_ISSR_AMT'])) {
                $debug_info = 'Monto Operacion: ' . $oper_amt . ' Saldo:' . $bal_amt . ' Saldo Final: ' . $final_bal . ' Condicion por monto: ' . $final_bal . ' ' . $policy_item['SEC_ISSR_ALOC_TYPE'] . ' ' . $policy_item['SEC_ISSR_AMT'];
                $this->add_error_msg($msg_err, $policy_item['RM_POLICY'], $policy_item['IDNAME'], $final_bal, $policy_item['SEC_ISSR_ALOC_TYPE'], $policy_item['SEC_ISSR_AMT'], false, $debug_info);
            }
        }
        //Si es por porcentaje de cuenta se evalua con base en portafolio
        if ($policy_item['SEC_ISSR_PCT'] <> CLX_NUM_ZERO) {
            //total de activos
            $sql = 'SELECT SUM(GL.BASE_BAL_AMT) BASE_BAL_AMT FROM ' . $this->jabvname . ' GL WHERE GL.COMP_CODE = ' . singleQuote($policy_item['COMP_CODE']) . ' AND GL.GL_TYPE =' . singleQuote('A') . ' ';
            if (!empty($policy_item['PORTFOLIO_CODE'])) {
                $sql .= ' AND EXISTS (SELECT FIAS.IDCODE FROM ' . $this->fiasname . ' FIAS WHERE FIAS.IDCODE = GL.CHART_FIELD01 AND FIAS.PORTFOLIO_CODE = ' . singleQuote($policy_item['PORTFOLIO_CODE']) . ')';
            }
            if (!empty($policy_item['CHART_FIELD01'])) {
                $sql .= ' AND GL.CHART_FIELD01 = ' . singleQuote($policy_item['CHART_FIELD01']);
            }
            $eq_total = ClxCommon::$db->getOne($sql);
            if ($eq_total == CLX_NUM_ZERO)
                return;
            $pct = rounder(($oper_amt / $eq_total) * CLX_NUM_HUNDRED, $this->pct_round);
            if (!$this->check_policy_operator($pct, $policy_item['SEC_ISSR_ALOC_TYPE'], $policy_item['SEC_ISSR_PCT'])) {
                $debug_info = 'Monto Operacion: ' . $oper_amt . ' Saldo:' . $eq_total . ' Pct: ' . $pct . ' ' . $policy_item['SEC_ISSR_ALOC_TYPE'] . ' ' . $policy_item['SEC_ISSR_PCT'];
                $this->add_error_msg($msg_err, $policy_item['RM_POLICY'], $policy_item['IDNAME'], $pct, $policy_item['SEC_ISSR_ALOC_TYPE'], $policy_item['SEC_ISSR_PCT'], true, $debug_info);
            }
        }
    }

    /**
     * Check limits by trust Item Type
     * @param unknown_type $policy_item
     * @param unknown_type $msg_err
     * @param unknown_type $jehd_code
     */
    private function check_policy_trust_item(&$policy_item, &$msg_err, $jehd_code = '') {
        if (empty($policy_item['TRUST_ITEM_TYPE']))
            return true;
        if (empty($policy_item['TRUST_ITEM_ALOC_TYPE']))
            return true;
        if (empty($policy_item['TRUST_ITEM_PCT']) && empty($policy_item['TRUST_ITEM_AMT']))
            return true;
        $cc = '';
        if (!empty($policy_item['CHART_FIELD01'])) {
            $cc = ' AND GL.CHART_FIELD01 = ' . singleQuote($policy_item['CHART_FIELD01']);
        }
        if (!empty($policy_item['PORTFOLIO_CODE'])) {
            $cc .= ' AND EXISTS (SELECT FIAS.ID FROM ' . $this->fiasname . ' FIAS WHERE FIAS.IDCODE = GL.CHART_FIELD03 AND FIAS.PORTFOLIO_CODE = ' . singleQuote($policy_item['PORTFOLIO_CODE']) . ')';
        }
        $cc2 = 'AND (GLAC.TRUST_ITEM_TYPE = ' . singleQuote($policy_item['TRUST_ITEM_TYPE']) . ' OR GLAC.TRUST_ITEM_TYPE IN (SELECT DISTINCT INRE.TRUST_ITEM_TYPE FROM ' . $this->inrename . ' INRE WHERE INRE.RM_POLICY = ' . singleQuote($policy_item['RM_POLICY']) . ' AND INRE.CLX_EFFTDATE = ' . singleQuote($policy_item['CLX_EFFTDATE']) . ' AND INRE.IDNAME = ' . singleQuote($policy_item['IDNAME']) . ' AND INRE.TRUST_ITEM_TYPE IS NOT null AND INRE.IS_EXCEPTION = 0))';
        //Monto de operacion JELN
        $sql = 'SELECT IFNULL(SUM(GL.DEBIT_AMT-GL.CREDIT_AMT),0) OP_BAL FROM ' . $this->glcpname . ' GLCP, ' . $this->jehdname . ' JEHD, ' . $this->jelnname . ' GL, ' . $this->glacname . ' GLAC WHERE GLCP.COMP_CODE = JEHD.COMP_CODE AND GLCP.GLGROUP_CODE = GLAC.GLGROUP_CODE AND GL.JEHDR_CODE = JEHD.IDCODE AND GLAC.IDCODE = GL.GLACCOUNTCODE AND JEHD.IDCODE = ' . singleQuote($jehd_code) . ' ' . $cc2 . $cc;
        $oper_amt = ClxCommon::$db->getOne($sql);
        //Saldo actual no incluye la operacion  JABV
        $sql = 'SELECT SUM(GL.BASE_BAL_AMT) BASE_BAL_AMT FROM ' . $this->jabvname . ' GL WHERE GL.COMP_CODE = ' . singleQuote($policy_item['COMP_CODE']) . $cc2 . $cc;
        $bal_amt = ClxCommon::$db->getOne($sql);
        $final_bal = ($bal_amt + $oper_amt);
        if ($policy_item['TRUST_ITEM_AMT'] <> CLX_NUM_ZERO) {
            if (!$this->check_policy_operator($final_bal, $policy_item['TRUST_ITEM_ALOC_TYPE'], $policy_item['TRUST_ITEM_AMT'])) {
                $this->add_error_msg($msg_err, $policy_item['RM_POLICY'], $policy_item['IDNAME'], $final_bal, $policy_item['TRUST_ITEM_ALOC_TYPE'], $policy_item['TRUST_ITEM_AMT'], false);
            }
        }
        //Si es por porcentaje de cuenta se evalua con base en el
        //patrimonio por ejemplo total activos / Total patrimonio *100 > nro de pct JELN
        if ($policy_item['TRUST_ITEM_PCT'] <> CLX_NUM_ZERO) {
            $sql = 'SELECT IFNULL(SUM(GL.DEBIT_AMT-GL.CREDIT_AMT),0) OP_BAL FROM ' . $this->glcpname . ' GLCP, ' . $this->jehdname . ' JEHD, ' . $this->jelnname . ' GL, ' . $this->glacname . ' GLAC WHERE GLCP.COMP_CODE = JEHD.COMP_CODE AND GLCP.GLGROUP_CODE = GLAC.GLGROUP_CODE AND GL.JEHDR_CODE = JEHD.IDCODE AND GLAC.IDCODE = GL.GLACCOUNTCODE AND JEHD.IDCODE = ' . singleQuote($jehd_code) . ' AND GLAC.GL_TYPE = ' . singleQuote('EQ') . $cc;
            $oper_eq_amt = ClxCommon::$db->getOne($sql);
            //JABV
            $sql = 'SELECT SUM(GL.BASE_BAL_AMT) BASE_BAL_AMT FROM ' . $this->jabvname . ' GL WHERE GL.GL_TYPE = ' . singleQuote('EQ') . '  AND GL.COMP_CODE = ' . singleQuote($policy_item['COMP_CODE']) . $cc;
            $eq_total = ClxCommon::$db->getOne($sql);
            $eq_total = -CLX_NUM_ONE * ($eq_total + $oper_eq_amt);
            if ($eq_total == CLX_NUM_ZERO)
                return;
            $pct = round(($final_bal / $eq_total) * CLX_NUM_HUNDRED, $this->pct_round);
            if (!$this->check_policy_operator($pct, $policy_item['TRUST_ITEM_ALOC_TYPE'], $policy_item['TRUST_ITEM_PCT'])) {
                $this->add_error_msg($msg_err, $policy_item['RM_POLICY'], $policy_item['IDNAME'], $pct, $policy_item['TRUST_ITEM_ALOC_TYPE'], $policy_item['TRUST_ITEM_PCT'], true);
            }
        }
    }

    /**
     * Check limits by gl type
     * @param array &$policy_item
     * @param array &$msg_err
     * @param string $jehd_code=''
     * @return boolean
     */
    private function check_policy_gl_type(&$policy_item, &$msg_err, $jehd_code = '') {
        if (empty($policy_item['GL_TYPE']))
            return true;
        if (empty($policy_item['GL_TYPE_ALOC_TYPE']))
            return true;
        if (empty($policy_item['GL_TYPE_PCT']) && empty($policy_item['GL_TYPE_AMT']))
            return true;
        $cc = '';
        if (!empty($policy_item['CHART_FIELD01'])) {
            $cc = ' AND GL.CHART_FIELD01 = ' . singleQuote($policy_item['CHART_FIELD01']);
        }
        if (!empty($policy_item['PORTFOLIO_CODE'])) {
            $cc .= ' AND EXISTS (SELECT FIAS.ID FROM ' . $this->fiasname . ' FIAS WHERE FIAS.IDCODE = GL.CHART_FIELD03 AND FIAS.PORTFOLIO_CODE = ' . singleQuote($policy_item['PORTFOLIO_CODE']) . ')';
        }
        $cc2 = 'AND (GLAC.GL_TYPE = ' . singleQuote($policy_item['GL_TYPE']) . ' OR GLAC.GL_TYPE IN (SELECT DISTINCT INRE.GL_TYPE FROM ' . $this->inrename . ' INRE WHERE INRE.RM_POLICY = ' . singleQuote($policy_item['RM_POLICY']) . ' AND INRE.CLX_EFFTDATE = ' . singleQuote($policy_item['CLX_EFFTDATE']) . ' AND INRE.IDNAME = ' . singleQuote($policy_item['IDNAME']) . ' AND INRE.GL_TYPE IS NOT null AND INRE.IS_EXCEPTION = 0))';
        $cc3 = str_replace('GLAC.GL_TYPE', 'GL.GL_TYPE', $cc2);
        //Monto de operacion JELN
        $sql = 'SELECT IFNULL(SUM(GL.DEBIT_AMT-GL.CREDIT_AMT),0) OP_BAL FROM ' . $this->glcpname . ' GLCP, ' . $this->jehdname . ' JEHD, ' . $this->jelnname . ' GL, ' . $this->glacname . ' GLAC WHERE GLCP.COMP_CODE = JEHD.COMP_CODE AND GLCP.GLGROUP_CODE = GLAC.GLGROUP_CODE AND GL.JEHDR_CODE = JEHD.IDCODE AND GLAC.IDCODE = GL.GLACCOUNTCODE AND JEHD.IDCODE = ' . singleQuote($jehd_code) . $cc2 . ' ' . $cc;
        $oper_amt = ClxCommon::$db->getOne($sql);
        if ($policy_item['GL_TYPE_AMT'] <> CLX_NUM_ZERO) {
            //Saldo actual no incluye la operacion  JABV
            $sql = 'SELECT SUM(GL.BASE_BAL_AMT) BASE_BAL_AMT FROM ' . $this->jabvname . ' GL WHERE GL.COMP_CODE = ' . singleQuote($policy_item['COMP_CODE']) . $cc3 . ' ' . $cc;
            $bal_amt = ClxCommon::$db->getOne($sql);
            $final_bal = ($bal_amt + $oper_amt);
            if (!$this->check_policy_operator($final_bal, $policy_item['GL_TYPE_ALOC_TYPE'], $policy_item['GL_TYPE_AMT'])) {
                $this->add_error_msg($msg_err, $policy_item['RM_POLICY'], $policy_item['IDNAME'], $final_bal, $policy_item['GL_TYPE_ALOC_TYPE'], $policy_item['GL_TYPE_AMT'], false);
            }
        }
        //Si es por porcentaje de cuenta se evalua con base en el
        //patrimonio por ejemplo total activos / Total patrimonio *100 > nro de pct JELN
        if ($policy_item['GL_TYPE_PCT'] <> CLX_NUM_ZERO) {
            $sql = 'SELECT IFNULL(SUM(GL.DEBIT_AMT-GL.CREDIT_AMT),0) OP_BAL FROM ' . $this->glcpname . ' GLCP, ' . $this->jehdname . ' JEHD, ' . $this->jelnname . ' GL, ' . $this->glacname . ' GLAC WHERE GLCP.COMP_CODE = JEHD.COMP_CODE AND GLCP.GLGROUP_CODE = GLAC.GLGROUP_CODE AND GL.JEHDR_CODE = JEHD.IDCODE AND GLAC.IDCODE = GL.GLACCOUNTCODE AND JEHD.IDCODE = ' . singleQuote($jehd_code) . ' AND GLAC.GL_TYPE = ' . singleQuote('EQ') . $cc;
            $oper_eq_amt = ClxCommon::$db->getOne($sql);
            //JABV
            $sql = 'SELECT SUM(GL.BASE_BAL_AMT) BASE_BAL_AMT FROM ' . $this->jabvname . ' GL WHERE GL.GL_TYPE = ' . singleQuote('EX') . ' AND GL.COMP_CODE = ' . singleQuote($policy_item['COMP_CODE']) . $cc;
            $eq_total = ClxCommon::$db->getOne($sql);
            $eq_total = -CLX_NUM_ONE * ($eq_total + $oper_eq_amt);
            if ($eq_total == CLX_NUM_ZERO)
                return;
            $pct = round(($final_bal / $eq_total) * CLX_NUM_HUNDRED, $this->pct_round);
            if (!$this->check_policy_operator($pct, $policy_item['GL_TYPE_ALOC_TYPE'], $policy_item['GL_TYPE_PCT'])) {
                $this->add_error_msg($msg_err, $policy_item['RM_POLICY'], $policy_item['IDNAME'], $pct, $policy_item['GL_TYPE_ALOC_TYPE'], $policy_item['GL_TYPE_PCT'], true);
            }
        }
    }

    /**
     * Check limits by gl type
     * @param array &$policy_item
     * @param array &$msg_err
     * @param string $jehd_code=''
     * @return boolean
     */
    private function check_policy_glaccount(&$policy_item, &$msg_err, $jehd_code = '') {
        if (empty($policy_item['GLACCOUNTCODE']))
            return true;
        if (empty($policy_item['GLAC_ALLOC_TYPE']))
            return true;
        if (empty($policy_item['GLAC_ALLOC_PCT']) && empty($policy_item['GLAC_ALLOC_AMT']))
            return true;
        $DBTB = &getTool('DBTB');
        $DBTB->buildTemporaryTable('GLCT');
        // CARGANDO CUENTAS DE MOVIMIENTO POSIBLES. LA IDEA ES QUE SI
        // SELECCIONAN UNA CUENTA PADRE TRAIGA LOS HIJOS
        // SE CARGAN TODAS LAS CUENTAS POSIBLES DE ACUERDO A LA CUENTA PADRE.
        $sql = 'SELECT CAST(REVERSE(CAST(REVERSE(' . singleQuote($policy_item['GLACCOUNTCODE']) . ') AS UNSIGNED INTEGER)) AS CHAR) ACPARENT FROM DUAL UNION SELECT CAST(REVERSE(CAST(REVERSE(INRE.GLACCOUNTCODE) AS UNSIGNED INTEGER)) AS CHAR) ACPARENT FROM ' . $this->inrename . ' INRE WHERE INRE.RM_POLICY = ' . singleQuote($policy_item['RM_POLICY']) . ' AND INRE.CLX_EFFTDATE = ' . singleQuote($policy_item['CLX_EFFTDATE']) . ' AND INRE.IDNAME = ' . singleQuote($policy_item['IDNAME']) . ' AND INRE.GLACCOUNTCODE IS NOT null AND INRE.IS_EXCEPTION = 0';
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
            $sql = 'INSERT INTO	' . $this->glctname . ' (IDCODE, GLGROUP_CODE) SELECT DISTINCT GLAC.IDCODE, GLAC.GLGROUP_CODE FROM ' . $this->glacname . ' GLAC, ' . $this->glcpname . ' GLCP WHERE GLAC.GLGROUP_CODE = GLCP.GLGROUP_CODE AND GLCP.COMP_CODE = ' . singleQuote($policy_item['COMP_CODE']) . ' AND GLAC.ALLOW_ACTIVITY = 1 AND GLAC.IDCODE LIKE ' . singleQuote($r ['ACPARENT'] . '%');
            ClxCommon::$db->executeQuery($sql);
        }
		ClxCommon::$db->freeResult($rs);
        //check if exception of issuer exists. if that exists must go by investment
        $sql = 'SELECT COUNT(INRE.ID) FROM ' . $this->inrename . ' INRE WHERE INRE.RM_POLICY = ' . singleQuote($policy_item['RM_POLICY']) . ' AND INRE.CLX_EFFTDATE = ' . singleQuote($policy_item['CLX_EFFTDATE']) . ' AND INRE.IDNAME = ' . singleQuote($policy_item['IDNAME']) . ' AND INRE.IS_EXCEPTION = 1';
        if (ClxCommon::$db->getOne($sql) > CLX_NUM_ZERO) {
            //go by excption instead
            return $this->check_policy_glaccount_except_issuer($policy_item, $msg_err, $jehd_code);
        }
        $cc = '';
        if (!empty($policy_item['CHART_FIELD01'])) {
            $cc = ' AND GL.CHART_FIELD01 = ' . singleQuote($policy_item['CHART_FIELD01']);
        }
        if (!empty($policy_item['PORTFOLIO_CODE'])) {
            $cc .= ' AND EXISTS (SELECT FIAS.ID FROM ' . $this->fiasname . ' FIAS WHERE FIAS.IDCODE = GL.CHART_FIELD03 AND FIAS.PORTFOLIO_CODE = ' . singleQuote($policy_item['PORTFOLIO_CODE']) . ')';
        }
        //Monto de operacion JELN
        $sql = 'SELECT IFNULL(SUM(GL.DEBIT_AMT-GL.CREDIT_AMT),0) OP_BAL FROM ' . $this->glcpname . ' GLCP, ' . $this->jehdname . ' JEHD, ' . $this->jelnname . ' GL, ' . $this->glacname . ' GLAC, ' . $this->glctname . ' GLCT WHERE JEHD.COMP_CODE = ' . singleQuote($policy_item['COMP_CODE']) . ' AND GLCP.COMP_CODE = JEHD.COMP_CODE AND GLCP.GLGROUP_CODE = GLAC.GLGROUP_CODE AND GL.JEHDR_CODE = JEHD.IDCODE AND GLAC.IDCODE = GL.GLACCOUNTCODE AND JEHD.IDCODE = ' . singleQuote($jehd_code) . ' AND GLAC.IDCODE = GLCT.IDCODE AND GLCP.GLGROUP_CODE = GLCT.GLGROUP_CODE ' . $cc;
        $oper_amt = ClxCommon::$db->getOne($sql);
        //Saldo actual no incluye la operacion  JABV
        $sql = 'SELECT SUM(GL.BASE_BAL_AMT) BASE_BAL_AMT FROM ' . $this->jabvname . ' GL, ' . $this->glctname . ' GLCT WHERE GL.GLACCOUNTCODE = GLCT.IDCODE AND GL.COMP_CODE = ' . singleQuote($policy_item['COMP_CODE']) . ' ' . $cc;
        $bal_amt = ClxCommon::$db->getOne($sql);
        $final_bal = ($bal_amt + $oper_amt);
        if ($policy_item['GLAC_ALLOC_AMT'] <> CLX_NUM_ZERO) {
            if (!$this->check_policy_operator($final_bal, $policy_item['GLAC_ALLOC_TYPE'], $policy_item['GLAC_ALLOC_AMT'])) {
                $this->add_error_msg($msg_err, $policy_item['RM_POLICY'], $policy_item['IDNAME'], $final_bal, $policy_item['GLAC_ALLOC_TYPE'], $policy_item['GLAC_ALLOC_AMT'], false);
            }
        }
        //Si es por porcentaje de cuenta se evalua con base en el
        //patrimonio por ejemplo total activos / Total patrimonio *100 > nro de pct JELN
        if ($policy_item['GLAC_ALLOC_PCT'] <> CLX_NUM_ZERO && $final_bal <> CLX_NUM_ZERO) {
            //JABV
            $sql = 'SELECT SUM(GL.BASE_BAL_AMT) BASE_BAL_AMT FROM ' . $this->jabvname . ' GL WHERE GL.COMP_CODE = ' . singleQuote($policy_item['COMP_CODE']) . ' AND GL.GL_TYPE IN (SELECT DISTINCT GLAC2.GL_TYPE FROM ' . $this->glacname . ' GLAC2 WHERE GLAC2.IDCODE = ' . singleQuote($policy_item['GLACCOUNTCODE']) . ') ' . $cc;
            $eq_total = ClxCommon::$db->getOne($sql);
            //$pct = round(($oper_amt/$eq_total)*100, $this->pct_round);
            if ($eq_total == CLX_NUM_ZERO)
                return;
            $pct = rounder(($final_bal / $eq_total) * CLX_NUM_HUNDRED, $this->pct_round);
            if (!$this->check_policy_operator($pct, $policy_item['GLAC_ALLOC_TYPE'], $policy_item['GLAC_ALLOC_PCT'])) {
                $debug_info = 'Monto Operacion: ' . $oper_amt . ' Saldo:' . $eq_total . ' Pct: ' . $pct . ' ' . $policy_item['GLAC_ALLOC_TYPE'] . ' ' . $policy_item['GLAC_ALLOC_PCT'];
                $this->add_error_msg($msg_err, $policy_item['RM_POLICY'], $policy_item['IDNAME'], $pct, $policy_item['GLAC_ALLOC_TYPE'], $policy_item['GLAC_ALLOC_PCT'], true, $debug_info);
            }
        }
    }

    /**
     * Check limits by gl type
     * @param array &$policy_item
     * @param array &$msg_err
     * @param string $jehd_code=''
     * @return boolean
     */
    private function check_policy_glaccount_except_issuer(&$policy_item, &$msg_err, $jehd_code = '') {
        //Esto trae e llamado desde cuenta contable donde se carga $this->glctname con valor de cuenta
        if (empty($policy_item['GLACCOUNTCODE']))
            return true;
        if (empty($policy_item['GLAC_ALLOC_TYPE']))
            return true;
        if (empty($policy_item['GLAC_ALLOC_PCT']) && empty($policy_item['GLAC_ALLOC_AMT']))
            return true;
        $comp_code = ClxCommon::$db->getField($this->jehdname, 'COMP_CODE', 'IDCODE = ' . singleQuote($jehd_code));
        $cc = '';
        if (!empty($policy_item['CHART_FIELD01'])) {
            $cc = ' AND INAT.CONTRACT_CODE = ' . singleQuote($policy_item['CHART_FIELD01']);
        }
        if (!empty($policy_item['PORTFOLIO_CODE'])) {
            $cc = ' AND FIAS.PORTFOLIO_CODE = ' . singleQuote($policy_item['PORTFOLIO_CODE']);
        }
        $cc2 = ' AND NOT EXISTS (SELECT INRE.ID FROM ' . $this->inrename . ' INRE WHERE INRE.RM_POLICY = ' . singleQuote($policy_item['RM_POLICY']) . ' AND INRE.CLX_EFFTDATE = ' . singleQuote($policy_item['CLX_EFFTDATE']) . ' AND INRE.IDNAME = ' . singleQuote($policy_item['IDNAME']) . ' AND INRE.SEC_ISSUER = INPR.SEC_ISSUER AND INRE.IS_EXCEPTION = 1)';
        //si es actividad de compra o aporte monto de inversion+ajuste no puede ser mayor del monto colocado.
        //Monto actualizado en actividad de inversion
        $sql = 'SELECT IFNULL(SUM(INAT.INVEST_AMT),0) OP_BAL FROM ' . $this->jehdname . ' JEHD, ' . $this->inatname . ' INAT, ' . $this->fiasname . ' FIAS, ' . $this->inprname . ' INPR WHERE JEHD.INAT_CODE = INAT.IDCODE AND FIAS.PORTFOLIO_CODE = INAT.PORTFOLIO_CODE AND INPR.INVESTMENT_CODE = INAT.INVESTMENT_CODE AND INAT.INVEST_ACTIVITY IN (' . singleQuote('B') . ',' . singleQuote('BY') . ',' . singleQuote('APIN') . ') AND EXISTS(SELECT PRAT.ID FROM ' . $this->pratname . ' PRAT, ' . $this->glctname . ' GLCT WHERE GLCT.IDCODE = PRAT.PRO_GL_CODE AND PRAT.PRODUCT_CODE = INPR.PRODUCT_CODE) AND JEHD.IDCODE = ' . singleQuote($jehd_code) . ' AND INAT.REVACTI_CODE IS NULL AND INPR.CLX_EFFTDATE = (SELECT MAX(INPRX.CLX_EFFTDATE) FROM ' . $this->inprname . ' INPRX WHERE INPRX.INVESTMENT_CODE = INPR.INVESTMENT_CODE AND INPR.CLX_EFFTDATE<=now()) ' . $cc . ' ' . $cc2 . ' ';
        $oper_amt = ClxCommon::$db->getOne($sql);
        if ($oper_amt == CLX_NUM_ZERO)
            return; //si el monto es cero solo retorna
//Buscando el saldo actual total por condicion
        $sql = 'SELECT IFNULL(SUM(INFA.INVEST_AMT),0) INVEST_AMT FROM ' . $this->inprname . ' INPR, ' . $this->infaname . ' INFA WHERE INFA.COMP_CODE =  ' . singleQuote($comp_code) . ' AND INFA.INVESTMENT_CODE = INPR.INVESTMENT_CODE ' . $cc2 . ' AND EXISTS(SELECT PRAT.ID FROM ' . $this->pratname . ' PRAT, ' . $this->glctname . ' GLCT WHERE GLCT.IDCODE = PRAT.PRO_GL_CODE AND PRAT.PRODUCT_CODE = INPR.PRODUCT_CODE) AND INPR.CLX_EFFTDATE = (SELECT MAX(INPRX.CLX_EFFTDATE) FROM ' . $this->inprname . ' INPRX WHERE INPRX.INVESTMENT_CODE = INPR.INVESTMENT_CODE AND INPR.CLX_EFFTDATE<=now()) ';
        if (!empty($policy_item['CHART_FIELD01'])) {
            $sql .= ' AND EXISTS (SELECT FIAS.ID FROM ' . $this->fiasname . ' FIAS WHERE FIAS.PORTFOLIO_CODE = INFA.PORTFOLIO_CODE AND FIAS.CONTRACT_CODE = ' . singleQuote($policy_item['CHART_FIELD01']) . ')';
        }
        if (!empty($policy_item['PORTFOLIO_CODE'])) {
            $sql .= ' AND INFA.PORTFOLIO_CODE = ' . singleQuote($policy_item['PORTFOLIO_CODE']);
        }
        $bal_amt = ClxCommon::$db->getOne($sql);
        $final_bal = ($bal_amt + $oper_amt);
        if ($final_bal == CLX_NUM_ZERO)
            return;
        if ($policy_item['GLAC_ALLOC_AMT'] <> CLX_NUM_ZERO) {
            if (!$this->check_policy_operator($final_bal, $policy_item['GLAC_ALLOC_TYPE'], $policy_item['GLAC_ALLOC_AMT'])) {
                $debug_info = 'Monto Operacion: ' . $oper_amt . ' Saldo:' . $bal_amt . ' Saldo Final: ' . $final_bal . ' Condicion por monto: ' . $final_bal . ' ' . $policy_item['GLAC_ALLOC_TYPE'] . ' ' . $policy_item['SECGLAC_ALLOC_AMT_ISSR_AMT'];
                $this->add_error_msg($msg_err, $policy_item['RM_POLICY'], $policy_item['IDNAME'], $final_bal, $policy_item['GLAC_ALLOC_TYPE'], $policy_item['GLAC_ALLOC_AMT'], false, $debug_info);
            }
        }
        //Si es por porcentaje de cuenta se evalua con base en portafolio
        if ($policy_item['GLAC_ALLOC_PCT'] <> CLX_NUM_ZERO) {
            //total de cartera
            $sql = 'SELECT SUM(GL.BASE_BAL_AMT) BASE_BAL_AMT FROM ' . $this->jabvname . ' GL WHERE GL.COMP_CODE = ' . singleQuote($policy_item['COMP_CODE']) . ' AND GL.GL_TYPE IN (SELECT DISTINCT GLAC2.GL_TYPE FROM ' . $this->glacname . ' GLAC2 WHERE GLAC2.IDCODE = ' . singleQuote($policy_item['GLACCOUNTCODE']) . ')  ';
            if (!empty($policy_item['PORTFOLIO_CODE'])) {
                $sql .= ' AND EXISTS (SELECT FIAS.IDCODE FROM ' . $this->fiasname . ' FIAS WHERE FIAS.IDCODE = GL.CHART_FIELD01 AND FIAS.PORTFOLIO_CODE = ' . singleQuote($policy_item['PORTFOLIO_CODE']) . ')';
            }
            if (!empty($policy_item['CHART_FIELD01'])) {
                $sql .= ' AND GL.CHART_FIELD01 = ' . singleQuote($policy_item['CHART_FIELD01']);
            }
            $eq_total = ClxCommon::$db->getOne($sql);
            if ($eq_total == CLX_NUM_ZERO)
                return;
            $pct = rounder(($final_bal / $eq_total) * CLX_NUM_HUNDRED, $this->pct_round);
            if (!$this->check_policy_operator($pct, $policy_item['GLAC_ALLOC_TYPE'], $policy_item['GLAC_ALLOC_PCT'])) {
                $debug_info = 'Monto Operacion: ' . $final_bal . ' Saldo:' . $eq_total . ' Pct: ' . $pct . ' ' . $policy_item['GLAC_ALLOC_TYPE'] . ' ' . $policy_item['GLAC_ALLOC_PCT'];
                $this->add_error_msg($msg_err, $policy_item['RM_POLICY'], $policy_item['IDNAME'], $pct, $policy_item['GLAC_ALLOC_TYPE'], $policy_item['GLAC_ALLOC_PCT'], true, $debug_info);
            }
        }
    }

    /**
     * Check limits by Additional Formulas
     * @param array &$policy_item
     * @param array &$msg_err
     * @param string $jehd_code=''
     * @return boolean
     */
    private function check_policy_other_formulas(&$policy_item, &$msg_err, $jehd_code = '') {
        if (empty($policy_item['INRD_OTHER_TYPES']))
            return true;
        if (empty($policy_item['INRD_OTH_ALLOC_TYPE']))
            return true;
        if (empty($policy_item['INRD_OTH_ALLOC_PCT']) && empty($policy_item['INRD_OTH_ALLOC_AMT']))
            return true;
        switch ($policy_item['INRD_OTHER_TYPES']) {
            case 'F0001';
                $this->check_policy_other_formulas_f0001($policy_item, $msg_err, $jehd_code);
                break;
            default:
                break;
            //do nothing more will come
        }
    }

    /**
     *
     * @param unknown_type $policy_item
     * @param unknown_type $msg_err
     * @param unknown_type $jehd_code
     */
    private function check_policy_other_formulas_f0001(&$policy_item, &$msg_err, $jehd_code = '') {
        $sql = 'SELECT INPR.DENOMINATION_AMT, INAT.INVEST_AMT FROM ' . $this->jehdname . ' JEHD, ' . $this->inatname . ' INAT, ' . $this->inprname . ' INPR WHERE INAT.IDCODE = JEHD.INAT_CODE AND INAT.INVEST_ACTIVITY IN (' . singleQuote('B') . ',' . singleQuote('BY') . ',' . singleQuote('APIN') . ') AND JEHD.IDCODE = ' . singleQuote($jehd_code) . ' AND INPR.INVESTMENT_CODE = INAT.INVESTMENT_CODE AND INPR.CLX_EFFTDATE = (SELECT MAX(INPRX.CLX_EFFTDATE) FROM ' . $this->inprname . ' INPRX WHERE INPRX.INVESTMENT_CODE =INPR.INVESTMENT_CODE AND INPRX.CLX_EFFTDATE<=' . singleQuote($this->now) . ')';
        //Adding product condition
        if ($policy_item['INVE_INSTRMT_CODE'] <> '') {
            $sql .= ' AND (INPR.PRODUCT_CODE = ' . singleQuote($policy_item['INVE_INSTRMT_CODE']) . ' OR INPR.PRODUCT_CODE IN (SELECT DISTINCT INRE.INVE_INSTRMT_CODE FROM ' . $this->inrename . ' INRE WHERE INRE.RM_POLICY = ' . singleQuote($policy_item['RM_POLICY']) . ' AND INRE.CLX_EFFTDATE = ' . singleQuote($policy_item['CLX_EFFTDATE']) . ' AND INRE.IDNAME = ' . singleQuote($policy_item['IDNAME']) . ' AND INRE.INVE_INSTRMT_CODE IS NOT null AND INRE.IS_EXCEPTION = 0))';
        }
        $inatrec = ClxCommon::$db->getOneRecord($sql);
        if (empty($inatrec))
            return;
        if ($inatrec['DENOMINATION_AMT'] == CLX_NUM_ZERO)
            return;
        if ($policy_item['INRD_OTH_ALLOC_AMT'] <> CLX_NUM_ZERO) {
            if (!$this->check_policy_operator($inatrec['DENOMINATION_AMT'], $policy_item['INRD_OTH_ALLOC_TYPE'], $policy_item['INRD_OTH_ALLOC_AMT'])) {
                $this->add_error_msg($msg_err, $policy_item['RM_POLICY'], $policy_item['IDNAME'], $inatrec['DENOMINATION_AMT'], $policy_item['INRD_OTH_ALLOC_TYPE'], $policy_item['INRD_OTH_ALLOC_AMT'], false);
            }
        }
        //Si es por porcentaje de cuenta se evalua con base en el
        //patrimonio por ejemplo total activos / Total patrimonio *100 > nro de pct JELN
        if ($policy_item['INRD_OTH_ALLOC_PCT'] <> CLX_NUM_ZERO) {
            $pct = rounder((($inatrec['INVEST_AMT'] / $inatrec['DENOMINATION_AMT']) * CLX_NUM_HUNDRED), $this->pct_round);
            if (!$this->check_policy_operator($pct, $policy_item['INRD_OTH_ALLOC_TYPE'], $policy_item['INRD_OTH_ALLOC_PCT'])) {
                $debug_info = 'Monto Denom.: ' . $inatrec['DENOMINATION_AMT'] . ' Mto. Inversion:' . $inatrec['INVEST_AMT'] . ' Pct Requerido:' . $policy_item['INRD_OTH_ALLOC_PCT'] . ' Pct: ' . $pct . ' ' . $policy_item['INRD_OTH_ALLOC_TYPE'] . ' ' . rounder((($inatrec['INVEST_AMT'] / $inatrec['DENOMINATION_AMT']) * CLX_NUM_HUNDRED), $this->pct_round);
                $this->add_error_msg($msg_err, $policy_item['RM_POLICY'], $policy_item['IDNAME'], $pct, $policy_item['INRD_OTH_ALLOC_TYPE'], $policy_item['INRD_OTH_ALLOC_PCT'], true, $debug_info);
            }
        }
    }

    //Helper methods
    /**
     * Formas
     * @param unknown_type $msg_err
     * @param unknown_type $policy_code
     * @param unknown_type $policy_name
     * @param unknown_type $operation_amt
     * @param unknown_type $alloc_type
     * @param unknown_type $limit_amt
     * @param unknown_type $is_pct
     */
    private function add_error_msg(&$msg_err, $policy_code, $policy_name, $operation_amt, $alloc_type, $limit_amt, $is_pct = false, $debug_info = '') {
        if (CLX_NUM_ONE == CLX_NUM_TWO && $this->debug) {
            if ($is_pct === true) {
                $msg_err[] = $policy_code . ': ' . $policy_name . '. [' . clx_format_currency($operation_amt) . '% ' . $alloc_type . ' ' . clx_format_currency($limit_amt) . '%] ' . $debug_info;
            } else {
                $msg_err[] = $policy_code . ': ' . $policy_name . '. [' . clx_format_currency($operation_amt) . ' ' . $alloc_type . ' ' . clx_format_currency($limit_amt) . '] ' . $debug_info;
            }
        } else {
            if ($is_pct === true) {
                $msg_err[] = $policy_code . ': ' . $policy_name . '. [' . clx_format_currency($operation_amt) . '% ' . $alloc_type . ' ' . clx_format_currency($limit_amt) . '%]';
            } else {
                $msg_err[] = $policy_code . ': ' . $policy_name . '. [' . clx_format_currency($operation_amt) . ' ' . $alloc_type . ' ' . clx_format_currency($limit_amt) . ']';
            }
        }
    }

    /**
     * Performs logical test
     * @param float $val1
     * @param string $operator
     * @param float $val2
     * @return boolean
     */
    private function check_policy_operator($val1, $operator, $val2) {
        switch ($operator) {
            case '=': //Equal to
                return ($val1 == $val2) ? true : false;
                break;
            case '>=': //Greater or equal to
                return ($val1 >= $val2) ? true : false;
                break;
            case '>': //Greater to
                return ($val1 > $val2) ? true : false;
                break;
            case '<=': //Less or Equal to
                return ($val1 <= $val2) ? true : false;
                break;
            case '<': //Less than
                return ($val1 <= $val2) ? true : false;
                break;
            case '!== ': //Not equal to
                return ($val1 != $val2) ? true : false;
                break;
            default:
                break;
        }
        return true;
    }

    //Solicitud de excepcion de politica y metodos de tabla INRA
    /**
     * Method creates entry in INRA to request exception
     */
    private function request_policy_exception(&$policyid, &$msg_err, $jehd_code) {
        if ($this->disable_exception)
            return;
        if ($jehd_code === '')
            return; //nothing to do
        $indrrec = ClxCommon::$db->getFields($this->inrmname, 'IDCODE, CLX_EFFTDATE, IDNAME, ALLOW_EXCEPTION', 'ID = '.$policyid);
        if (empty($indrrec['ALLOW_EXCEPTION']))
            return;
        $DBTB = &getTool('DBTB');
        $rec = $DBTB->tableDefaultRecord('INRA');
        unset($rec['ID'], $rec['APPROVAL_DATE']);
        $rec['AUTH_STATUS'] = 'PEND';
        $rec['CREATEDBY'] = $this->username;
        $rec['MODIFIEDBY'] = $this->username;
        $rec['IDNAME'] = $indrrec['IDNAME'];
        $rec['JEHD_FM_CODE'] = $jehd_code;
        $rec['MODIFIEDBY'] = $this->username;
        $rec['NOTES'] = toString($msg_err);
        $rec['RM_EFFTDATE'] = $indrrec['CLX_EFFTDATE'];
        $rec['RM_POLICY'] = $indrrec['IDCODE'];
        $criteria = ' RM_EFFTDATE = ' . singleQuote($rec['RM_EFFTDATE']) . ' AND RM_POLICY = ' . singleQuote($rec['RM_POLICY']) . ' AND JEHD_FM_CODE = ' . singleQuote($jehd_code);
        if (ClxCommon::$db->oneExistsRecords($this->inraname, $criteria) == CLX_NUM_ZERO) {
            ClxCommon::$db->createRecord($this->inraname, $rec, false);
        }
    }

}

class Inra {

    function __construct() {
        multiTableName($this, array('INRM', 'INRD', 'INRA', 'FICO', 'FIAS', 'GLCP', 'GLAC', 'JEHD', 'JELN', 'FIAT', 'INAT', 'JABV', 'JABV'));
        $this->batch_id = (!empty($procParameters['BATCH_ID'])) ? $procParameters['BATCH_ID'] : batch_id();
        $this->username = $_SESSION['user_record']['username'];
        $this->now = xFormat::getNow('Y-m-d H:i:s');
        $this->today = xFormat::getNow('Y-m-d');
    }

    /**
     * Validates form
     * @param unknown_type $record
     * @param unknown_type $msg
     * @return boolean
     */
    function validate_form(&$record, &$msg) {

        if (!empty($msg)) {
            return false;
        }
        // validar situacion de transaccion pendiente o no
        if ($record ['INRA_AUTH_STATUS'] !== 'PEND') {
            $msg ['INRA_AUTH_STATUS'] = smsg('MSG-379');
            if (UserRequest::$request['ac'] === 'DELETE')
                addMessage($msg ['INRA_AUTH_STATUS']);
        }
    }

    /**
     * Authorization method
     * @param string $id
     * @param boolean $reject
     */
    function authorize_btn($reject) {
        $idlist = toString(getIdlist());
        $results = ClxCommon::$db->executeQuery('SELECT ID FROM ' . $this->inraname . ' WHERE ID IN (' . $idlist . ') AND AUTH_STATUS = ' . singleQuote('PEND') . '');
        while ($result = ClxCommon::$db->getNextRecord($results, ASSOCIATIVE)) {
            $this->authorize($result ['ID'], $reject);
        }
		ClxCommon::$db->freeResult($results);
    }

    /**
     * Approve or reject record
     * @param unknown_type $id
     * @param unknown_type $reject
     */
    private function authorize($id, $reject) {
        if ($id === '')
            return;
        if (empty($reject)) {
            ClxCommon::$db->updateRecord($this->inraname, array('AUTH_STATUS' => 'AP', 'APPROVEDBY' => $this->username, 'APPROVAL_DATE' => $this->now), 'AUTH_STATUS=' . singleQuote('PEND') . ' AND ID = ' . $id);
        } else {
            ClxCommon::$db->updateRecord($this->inraname, array('AUTH_STATUS' => 'REJ', 'APPROVEDBY' => $this->username, 'APPROVAL_DATE' => $this->now), 'AUTH_STATUS=' . singleQuote('PEND') . ' AND ID = ' . $id);
            $jehd_code = ClxCommon::$db->getField($this->inraname, 'JEHD_FM_CODE', 'ID = ' . $id);
            if (!class_exists('Jehd'))
                includeCodeLib('JEHD');
            $jehd = new Jehd ();
            $jehd->jehd_set_reject_status($jehd_code);
        }
    }

}
