<?php

/**
 * Object Class
 * For more information contact  support@alenet.com
 * @package		Clientix.Tool.Event
 * @abstract		System Events Manager - Mail Event.
 * @author       $Author: jclerma $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2017-11-03 15:09:40 -0400 (Fri, 03 Nov 2017) $
 * @version        $Rev: 9615 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  Included label
 */
if (!class_exists('actn') && !defined('clxInstallProcess')) {
    include_once kPathToObjects . 'actn.php';
}

class actn_mail extends actn {

    /**
     * Constructor
     */
    function __construct() {
        $this->code = 'MAIL';
        $this->name = 'Electronic Mail';
        $this->type = 'ACTN';
        $this->version = '1.1.0.0';
        $this->description = 'Send an electronic mail message.';
    }

    /**
     * Build code object panel on edit event.
     * @param array $msg
     * @param array $actionrecord
     * @return string
     */
    function panel($msg, $actionrecord) {
        if ($actionrecord['EVNT_action'] != $this->code) {
		  $this->defaultPanel($actionrecord);
        }
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_WIDTH => '90%', CLX_PROP_CLASS => CLX_CLASS_FORM));
        $table->addCell(dict('GEN_LBL_MAIL_PROPS') . ':', array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addField($msg, dict('GEN_LBL_SENDER') . ':', 'MAIL_sender', $actionrecord['MAIL_sender'], CLX_NUM_48, CLX_NUM_128);
        $table->addField($msg, dict('GEN_LBL_RECIPIENT') . ':', 'MAIL_recipient', $actionrecord['MAIL_recipient'], CLX_NUM_48, CLX_NUM_128);
        $table->addField($msg, dict('GEN_LBL_SUBJECT') . ':', 'MAIL_subject', $actionrecord['MAIL_subject'], CLX_NUM_64, CLX_NUM_128);
        $table->addCell(dict('GEN_LBL_CONTENTS'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        if (isset($msg['MAIL_contents'])) {
		  $table->addCell($msg['MAIL_contents'], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        }
        $c = new htmlObject('textarea', htmlspecialchars($actionrecord['MAIL_contents']), array('name' => 'MAIL_contents', 'rows' => CLX_NUM_15, 'cols' => CLX_NUM_80));
        $c->forceclose = true;
        $table->addCell($c->source(), array(CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $div = new htmlDiv($this->code, 'visibility:hidden; display:none', $table->source());
        return CLX_CHAR_NL . CLX_CHAR_NL . $div->source() . CLX_CHAR_NL . CLX_CHAR_NL;
    }

    /**
     * Validate Panel User Input before saving record.
     * @param array $msg
     */
    function checkPanel(&$msg) {
        if (UserRequest::$post['EVNT_language'] <> 'PHP' && UserRequest::$post['EVNT_language'] <> 'COMP') {
		  $msg['EVNT_action'] = dict('EVNT_LBL_PHP_ONLY');
        } else {
		  xValidate::mail($msg, 'MAIL_sender', UserRequest::$post['MAIL_sender']);
		  xValidate::mail($msg, 'MAIL_recipient', UserRequest::$post['MAIL_recipient']);
		  if (empty(UserRequest::$post['MAIL_subject'])) {
			 $msg['MAIL_subject'] = dict('GEN_LBL_SUBJECT_MISS');
		  }
		  if (empty(UserRequest::$post['MAIL_contents'])) {
			 $msg['MAIL_contents'] = dict('GEN_LBL_CONTENT_MISS');
		  }
        }
    }

    /**
     * Build object record from user post
     * @param array $actionrecord
     */
    function getPanel(&$actionrecord) {
        $actionrecord['extra1'] = CLX_INIT_ZERO;
        $actionrecord['extra2'] = UserRequest::$post['MAIL_sender'];
        $actionrecord['extra3'] = UserRequest::$post['MAIL_recipient'];
        $actionrecord['extra4'] = UserRequest::$post['MAIL_subject'];
        $actionrecord['extra5'] = '';
        $actionrecord['extra6'] = UserRequest::$post['MAIL_contents'];
    }

    /**
     * Sets edit record required on forms of user interface.
     * @param array $actionrecord
     * @param array $values
     */
    function setPanel(&$actionrecord, $values) {
        if ($values['ACTION'] == $this->code) {
		  $actionrecord['MAIL_sender'] = $values['EXTRA2'];
		  $actionrecord['MAIL_recipient'] = $values['EXTRA3'];
		  $actionrecord['MAIL_subject'] = $values['EXTRA4'];
		  $actionrecord['MAIL_contents'] = $values['EXTRA6'];
        } else {
		  $this->defaultPanel($actionrecord);
        }
    }

    /**
     * Sets the default values for form on create event.
     * @param array $actionrecord
     */
    function defaultPanel(&$actionrecord) {
        $actionrecord['MAIL_sender'] = $_SESSION['user_record']['email'];
        $actionrecord['MAIL_recipient'] = '';
        $actionrecord['MAIL_subject'] = '';
        $actionrecord['MAIL_contents'] = '';
    }

    /**
     * Build panel for display form
     * @param obj $table
     * @param array $record
     */
    function viewForm_2(&$table, $record) {
        $table->addData(dict('GEN_LBL_ACTION') . ':', $this->name);
        $table->addData(dict('GEN_LBL_SENDER') . ':', $record['MAIL_sender']);
        $table->addData(dict('GEN_LBL_RECIPIENT') . ':', $record['MAIL_recipient']);
        $table->addData(dict('GEN_LBL_SUBJECT') . ':', $record['MAIL_subject']);
        $table->addData(dict('GEN_LBL_CONTENTS') . ':', $record['MAIL_subject']);
    }

    /**
     * Builds description about the object on event panel listing.
     * @param array $record
     * @return string
     */
    function describe($record) {
        return dict('GEN_LBL_SENDMAILTO') . ': ' . $record['EXTRA3'];
    }

    /**
     * Execute code according object type.
     * @param string $event
     * @param array $record
     * @param array $msg
     */
    function execute3($event, &$record, &$msg) {IF (ISSET($GLOBALS['ADDRDEBUGEVENTS'])) { echo __METHOD__.' '. __FILE__ .':'.__LINE__; pre($event);pre($record); }
        $from = $event['EXTRA2'];
        $to = $event['EXTRA3'];
        $subject = $event['EXTRA4'];
        $message = $event['EXTRA6'];
        $message = str_replace(CLX_CHAR_NL, '<br>', $message);
        $record_formated = $record;
        $this->replace_load_format($record_formated);
        $this->replace($subject, $record_formated);
        $this->replace($to, $record_formated);
        $this->replace($message, $record_formated);
        $result = clx_mail($from, $to, $message, $subject);
        if (empty($result)) {
		  addMessage(dict('GEN_ERR_EVENTMAILFAIL'));
        }
    }

    /**
     * Parses record fields
     * @param $contents
     * @param $record
     */
    function replace_load_format(&$record) {
        if (empty($record))
		  return;
        $DBFL = &getTool('DBFL');
        foreach ($record as $key => $value) {
		  $field = $DBFL->getField($key);
		  if (!empty($field)) {
			 $fieldobj = getToolObject('DBFT', $field['FIELDTYPE']);
		$record[$key] = trim(strip_tags($fieldobj->formatField($key, $field, $record)));
		  }
        }
    }

    /**
     * Parses record fields
     * @param $contents
     * @param $record
     */
    function replace(&$contents, &$record) {
        if (empty($record))
		  return;
        foreach ($record as $key => $value) {
		  $contents = str_replace('!' . $key, $value, $contents);
        }
    }

}
