<?php

/**
 * Recurrence field object class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: jclerma $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2017-11-03 15:09:40 -0400 (Fri, 03 Nov 2017) $
 * @version    	$Rev: 9615 $
 */
if (!class_exists('dbft') && !defined('clxInstallProcess'))
    include_once kPathToObjects . 'dbft.php';

class dbft_recurrent extends dbft {

    var $debug = false;

    /**
     * CONSTRUCTOR
     */
    function __construct() {
        $this->code = 'RECU';
        $this->name = 'Recurrence Field';
        $this->type = 'DBFT';
        $this->version = '1.0.0.2';
        $this->description = 'Recurrent field type.';
        $this->multifield = true; // Describes it field is a multi field see properties;
        $this->has_lookup = false; // Decribes if field can be set as copy field on lookup
        $this->has_question = false; // Describes if field can be used on queries criteria
        $this->operators = array();
    	IF (ISSET($GLOBALS['ADDRDEBUGEDIT']) && $GLOBALS['ADDRDEBUGEDIT']==$_SERVER['REMOTE_ADDR']) {
			$this->debug=true;
		}

    }

// End method
    function panel($msg, $fieldrecord, &$form) {
        $c = $this->code;
        if ($fieldrecord['DBFL_fieldtype'] != $c) {
		  $this->defaultPanel($fieldrecord);
        }
        $contents = '';
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM));
        $table->addCell(dict('GEN_LBL_RECUFIELDPROP'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $yesno = new htmlSelect($this->code . '_required');
        $yesno->addItem(dict('GEN_LBL_NO'), '0', ($fieldrecord[$c . '_required'] == CLX_NUM_ZERO));
        $yesno->addItem(dict('GEN_LBL_YES'), '1', ($fieldrecord[$c . '_required'] == CLX_NUM_ONE));
        $table->addData(dict('GEN_LBL_REQUIRED') . ':', $yesno->source());
        //Default Options
        $options = array('CLX_DBTF_RECU_none' => 'NONE', 'CLX_DBTF_RECU_day' => 'DAY', 'CLX_DBTF_RECU_week' => 'WEEK', 'CLX_DBTF_RECU_mnth' => 'MNTH', 'CLX_DBTF_RECU_year' => 'YEAR', 'CLX_DBTF_RECU_one' => 'ONE');
        $yesno = new htmlSelect($this->code . '_default');
        foreach ($options as $key => $val) {
		  $yesno->addItem(dict($key), $val, ($fieldrecord[$c . '_default'] == $val));
        }
        $table->addData(dict('GEN_LBL_DEFAULT') . ':', $yesno->source());
        $div = new htmlDiv($c, 'visibility:hidden; display:none', $table->source());
        $contents .= CLX_CHAR_NL . CLX_CHAR_NL . $div->source();
        return $contents . CLX_CHAR_NL . CLX_CHAR_NL;
    }

// End method
    function checkPanel(&$msg) {
        $c = $this->code;
        if (!empty(UserRequest::$post[$c . '_required'])) {
		  xValidate::required($msg, $c . '_default', UserRequest::$post[$c . '_default']);
        }
    }

// End method
    function savePanel(&$fieldrecord) {
        $c = $this->code;
        $fieldrecord['FIELD_SIZE'] = '';
        $fieldrecord['FIELD_REQUIRED'] = UserRequest::$post[$c . '_required'];
        $fieldrecord['FIELD_FORMAT'] = '';
        $fieldrecord['FIELD_ALIGN'] = '0';
        $fieldrecord['FIELD_DEFAULT'] = UserRequest::$post[$c . '_default'];
        $fieldrecord['FIELD_MINVALUE'] = '';
        $fieldrecord['FIELD_MAXVALUE'] = '';
        $fieldrecord['FIELD_DISPLAYSIZE'] = '';
        $fieldrecord['FIELD_EXTRA1'] = '';
        $fieldrecord['FIELD_EXTRA2'] = '';
        $fieldrecord['FIELD_EXTRA3'] = '';
        $fieldrecord['FIELD_EXTRA4'] = '';
        $fieldrecord['FIELD_EXTRA5'] = '';
    }

// End method
    function loadPanel(&$fieldrecord, $values) {
        $c = $this->code;
        if ($values['FIELDTYPE'] == $c) {
		  $fieldrecord[$c . '_required'] = $values['FIELD_REQUIRED'];
		  $fieldrecord[$c . '_default'] = $values['FIELD_DEFAULT'];
        } else {
		  $this->defaultPanel($fieldrecord);
        }
    }

// End method
    function defaultPanel(&$fieldrecord) {
        $c = $this->code;
        $fieldrecord[$c . '_size'] = CLX_NUM_32;
        $fieldrecord[$c . '_required'] = CLX_INIT_ZERO;
        $fieldrecord[$c . '_format'] = '';
        $fieldrecord[$c . '_align'] = CLX_INIT_ZERO;
        $fieldrecord[$c . '_default'] = 'NONE';
        $fieldrecord[$c . '_minvalue'] = '';
        $fieldrecord[$c . '_maxvalue'] = '';
        $fieldrecord[$c . '_displaysize'] = '32';
        $fieldrecord[$c . '_extra1'] = '';
        $fieldrecord[$c . '_extra2'] = '';
        $fieldrecord[$c . '_extra3'] = '';
        $fieldrecord[$c . '_extra4'] = '';
        $fieldrecord[$c . '_extra5'] = '';
    }

// End method
    function viewForm(&$table, $record) {
        $c = $this->code;
        $yesno = array(CLX_NUM_ZERO => dict('GEN_LBL_NO'), CLX_NUM_ONE => dict('GEN_LBL_YES'));
        $table->addData(dict('GEN_LBL_TYPE') . ':', $this->name);
        $table->addData(dict('GEN_LBL_REQUIRED') . ':', $yesno[$record[$c . '_required']]);
    }

    function fieldProperties($fieldrecord) {
        /*
		The following method are customizations for this object type. For other field objects review general DBTF object */
        // DB implementation
        $props = array();
        /**
         * Field Descriptions
         * ['R1'] => Recurrence Type General Type of recurrence
         * ['R2'] => Day Value
         * ['R3'] => Day Type
         * ['R4'] => Every week repetitions
         * ['R5'] => Every Month
         * ['R6'] => Month type
         * ['R7'] => Week days for month
         * ['R8'] => Every Year
         * ['R9'] => End date
         * ['R0'] => No End
         */
        switch (kDatabaseType) {
		  case 'oci':
			 $varchar = 'varchar2';
			 $smallint = 'number';
			 $tinyint = 'number';
			 $date = 'timestamp';
			 break;
		  default:
			 $varchar = 'varchar';
			 $smallint = 'smallint';
			 $tinyint = 'tinyint';
			 $date = 'date';
			 break;
        }
        $props['R1'] = array('type' => $varchar . '(4)', 'properties' => '', 'null' => 'YES', 'default' => 'null');
        $props['R2'] = array('type' => $smallint . '(10)', 'properties' => '', 'null' => 'YES', 'default' => 'null');
        $props['R3'] = array('type' => $tinyint . '(1)', 'properties' => '', 'null' => 'YES', 'default' => 'null');
        $props['R4'] = array('type' => $smallint . '(10)', 'properties' => '', 'null' => 'YES', 'default' => 'null');
        $props['R5'] = array('type' => $smallint . '(10)', 'properties' => '', 'null' => 'YES', 'default' => 'null');
        $props['R6'] = array('type' => $tinyint . '(1)', 'properties' => '', 'null' => 'YES', 'default' => 'null');
        $props['R7'] = array('type' => $varchar . '(4)', 'properties' => '', 'null' => 'YES', 'default' => 'null');
        $props['R8'] = array('type' => $smallint . '(10)', 'properties' => '', 'null' => 'YES', 'default' => 'null');
        $props['R9'] = array('type' => $date, 'properties' => '', 'null' => 'YES', 'default' => 'null');
        $props['R0'] = array('type' => $tinyint . '(1)', 'properties' => '', 'null' => 'YES', 'default' => 'null');
        return $props;
    }

    function validateField(&$msg, $form_fieldname, $fieldrecord, &$record, &$table, &$form) {
        $fn = $form_fieldname;
        //Setting form require pref if any
        if (array_key_exists('FIELD_FORM_REQUIRED', $fieldrecord) && $fieldrecord['FIELD_FORM_REQUIRED'] != CLX_NUM_TWO) {
		  if (UserRequest::$request['ac']!='DELETE')
		  $fieldrecord['FIELD_REQUIRED'] = $fieldrecord['FIELD_FORM_REQUIRED'];
        }
        // Note: !array_key_exists was added to enable lookups to validate
        if (!array_key_exists('FIELD_HIDDEN', $fieldrecord))
		  $fieldrecord['FIELD_HIDDEN'] = false;
        //Setting hidden to override required
        if (empty($record[$fn . '_r0']) && $record[$fn . '_r9'] == '0000-00-00 00:00:00' || $record[$fn . '_r9'] == '0000-00-00') {
		  /*
		   * DBTL_RECU_INVALIDENDDATE
		   * Invalid end date, please verify end date or choose no end.
		   * Fecha Final Invalida, por favor colocar fecha final o marcar opción sin fecha final.
		   */
		  $msg[$fn] = dict('DBTL_RECU_INVALIDENDDATE');
        }
        if (!empty($record[$fn . '_r9']) && !adodb_valid_date($record[$fn . '_r9']) && $record[$fn . '_r9'] <> '0000-00-00 00:00:00' && $record[$fn . '_r9'] <> '0000-00-00') {
		  $msg[$fn] = dict('DBTL_RECU_INVALIDENDDATE');
        }
        if (!empty($record[$fn . '_r0']) && !empty($record[$fn . '_r9']) && $record[$fn . '_r9'] <> '0000-00-00 00:00:00' && $record[$fn . '_r9'] <> '0000-00-00') {
		  /*
		   * DBTL_RECU_ENDATENOTREC
		   * Invalid end date, no end date option selected. please leave blank end date.
		   * Fecha Final Invalida, la opcion sin fecha final se encuentra habilitada. favor dejar fecha final sin valor.
		   */
		  $msg[$fn] = dict('DBTL_RECU_ENDATENOTREC');
        }
        /**
         * DBTL_RECU_INVARECURENCE
         * Recurence Invalid
         * Recurencia Invalida
         */
        if (empty($record[$fn . '_r1']))
		  $record[$fn . '_r1'] = 'NONE';
        switch ($record[$fn . '_r1']) {
		  case 'NONE':
			 break;
		  case 'DAY':
			 if (empty($record[$fn . '_r3']))
			     $msg[$fn] = dict('DBTL_RECU_INVARECURENCE');
			 break;
		  case 'WEEK':
			 if (empty($record[$fn . '_r4']) || empty($record[$fn . '_r7']))
			     $msg[$fn] = dict('DBTL_RECU_INVARECURENCE');
			 break;
		  case 'MNTH':
			 if (empty($record[$fn . '_r5']))
			     $msg[$fn] = dict('DBTL_RECU_INVARECURENCE');
			 break;
		  case 'YEAR':
			 if (empty($record[$fn . '_r8']))
			     $msg[$fn] = dict('DBTL_RECU_INVARECURENCE');
			 break;
		  default:
			 break;
        }
    }


    function validateField_original(&$msg, $form_fieldname, $fieldrecord, &$record, &$table, &$form) {
        $fn = $form_fieldname;
        //Setting form require pref if any
        if (array_key_exists('FIELD_FORM_REQUIRED', $fieldrecord) && $fieldrecord['FIELD_FORM_REQUIRED'] != CLX_NUM_TWO) {
            if (UserRequest::$request['ac']!='DELETE')
            	$fieldrecord['FIELD_REQUIRED'] = $fieldrecord['FIELD_FORM_REQUIRED'];
        }
        // Note: !array_key_exists was added to enable lookups to validate
        if (!array_key_exists('FIELD_HIDDEN', $fieldrecord))
            $fieldrecord['FIELD_HIDDEN'] = false;
        //Setting hidden to override required
        if (empty($record[$fn . '_r0']) && $record[$fn . '_r1']<>'ONE' && ($record[$fn . '_r9'] == '0000-00-00 00:00:00' || $record[$fn . '_r9'] == '0000-00-00' || trim($record[$fn . '_r9']) == '')) {
            /*
             * DBTL_RECU_INVALIDENDDATE
             * Invalid end date, please verify end date or choose no end.
             * Fecha Final Invalida, por favor colocar fecha final o marcar opción sin fecha final.
             */
            $msg[$fn] = dict('DBTL_RECU_INVALIDENDDATE');
        }
        if (!empty($record[$fn . '_r9']) && !adodb_valid_date($record[$fn . '_r9']) && $record[$fn . '_r9'] <> '0000-00-00 00:00:00' && $record[$fn . '_r9'] <> '0000-00-00') {
            $msg[$fn] = dict('DBTL_RECU_INVALIDENDDATE');
        }
        if (!empty($record[$fn . '_r0']) && !empty($record[$fn . '_r9']) && $record[$fn . '_r9'] <> '0000-00-00 00:00:00' && $record[$fn . '_r9'] <> '0000-00-00') {
            /*
             * DBTL_RECU_ENDATENOTREC
             * Invalid end date, no end date option selected. please leave blank end date.
             * Fecha Final Invalida, la opcion sin fecha final se encuentra habilitada. favor dejar fecha final sin valor.
             */
            $msg[$fn] = dict('DBTL_RECU_ENDATENOTREC');
        }
        /**
         * DBTL_RECU_INVARECURENCE
         * Recurence Invalid
         * Recurencia Invalida
         */
        if (empty($record[$fn . '_r1']))
            $record[$fn . '_r1'] = 'NONE';
        switch ($record[$fn . '_r1']) {
            case 'NONE':
                break;
            case 'DAY':
                if (empty($record[$fn . '_r3']))
                    $msg[$fn] = dict('DBTL_RECU_INVARECURENCE');
                break;
            case 'WEEK':
                if (empty($record[$fn . '_r4']) || empty($record[$fn . '_r7']))
                    $msg[$fn] = dict('DBTL_RECU_INVARECURENCE');
                break;
            case 'MNTH':
                if (empty($record[$fn . '_r5']))
                    $msg[$fn] = dict('DBTL_RECU_INVARECURENCE');
                break;
            case 'YEAR':
                if (empty($record[$fn . '_r8']))
                    $msg[$fn] = dict('DBTL_RECU_INVARECURENCE');
                break;
            case 'ONE':
                if (!empty($record[$fn . '_r9'])) {
                    $msg[$fn] = dict('DBTL_RECU_ENDATENOTREC');
                }    
                break;    
            default:
                break;
        }
    }

    function validateImportField(&$msg, $fieldrecord, $fieldvalue) {
        $msg[$fieldrecord['FIELDNAME']] = dict('GEN_LBL_CONTACT_ADMIN');
    }

    /**
     * The following methods are customizations for this object type. For other field objects review general DBTF object
     * @param type $tablename
     * @param type $fieldrecord
     */
    function createPhysicalFieldByType($tablename, $fieldrecord) {
        $props = $this->fieldProperties($fieldrecord);
        reset($props);
        foreach ($props as $key => $prop) {
		  if ($key === 'multiple') {
			 continue;
		  }
		  $fieldname = $fieldrecord['FIELDNAME'] . '_' . $key;
		  ClxCommon::$db->createField($tablename, $fieldname, $prop);
        }
    }

    function deleteField($tablename, $fieldrecord) {
        $props = $this->fieldProperties($fieldrecord);
        reset($props);
        foreach ($props as $key => $prop) {
		  if ($key === 'multiple')
			 continue;
		  $fieldname = $fieldrecord['FIELDNAME'] . '_' . $key;
		  ClxCommon::$db->dropField($tablename, $fieldname);
        }
    }

    function placeEditField($msg, $form_fieldname, $fieldrecord, $record, &$form, $jsevents) {
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_WIDTH => CLX_PROP_100PCT));
        /*
		Setting form field requirement if
		any diferent than table default
         */
        if ($fieldrecord['FIELD_FORM_REQUIRED'] != CLX_NUM_TWO) {
		  $fieldrecord['FIELD_REQUIRED'] = $fieldrecord['FIELD_FORM_REQUIRED'];
        }
        if ($fieldrecord['FIELD_READONLY'] == CLX_NUM_ONE || $fieldrecord['FIELD_HIDDEN'] == CLX_NUM_ONE) {
		  $form->addArgument($form_fieldname, $record[$form_fieldname]);
		  if ($fieldrecord['FIELD_READONLY'] === true && !$fieldrecord['FIELD_HIDDEN'] === true) {
			 $table = $this->placeViewField($form_fieldname, $fieldrecord, $record);
		  }
        } else {
		  // Field Caption
		  $cpt = dict($fieldrecord['CAPTION']);
		  if (strlen($cpt) > CLX_NUM_ZERO) {
			 $table->addCell($cpt, array(CLX_PROP_CLASS => $fieldrecord['CAPTION_CLASS'], CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
		  }
		  // Field message
		  if (isset($msg[$form_fieldname])) {
			 $table->addCell($msg[$form_fieldname], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
		  }
		  $fieldobj = $this->fieldObject($form_fieldname, $fieldrecord, $record);
		  foreach ($jsevents as $key => $value) {
			 $fieldobj->setProp($key, $value);
		  }
		  $pfx = dict($fieldrecord['FIELDPREFIX'],true,false); 
		  $fieldsource = (strlen($pfx) > CLX_NUM_ZERO) ? $pfx : '';
		  $fieldsource .= $fieldobj->source();
		  $sfx = dict($fieldrecord['FIELDSUFIX'],true,false); 
		  $fieldsource .= (strlen($sfx) > CLX_NUM_ZERO) ? $sfx : '';
		  $class = ($fieldrecord['FIELD_REQUIRED']) ? CLX_CLASS_PROMPTREQUIRED : CLX_CLASS_PROMPT;
		  $fieldlabel = dict($fieldrecord['FIELDFORMLABEL']);
		  $table->addCell($fieldlabel, array(CLX_PROP_CLASS => $class));
		  $table->addCell($fieldsource, array(CLX_PROP_CLASS, CLX_CLASS_DATA));
			if ( $this->debug==true ) {$pos=$posx=0;
				$te=debug_backtrace();
				$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo4.txt","a+");
				fwrite($tta,chr(13).chr(10).$pos."_".$te[$posx]["function"]."dbft_recurrent-hora:".date("h:i:s d/m/Y").chr(13).chr(10));
				
				for ($t=0;$t<12;$t++) {
					if (isset($te[$t]["file"]))
						fwrite($tta,substr($te[$t]["file"].str_repeat(" ",80),0,80).chr(9).$te[$t]["line"].chr(9).$te[$t]["function"].chr(13).chr(10));
					
				}

				fwrite($tta,chr(13).chr(10)."placeListFieldText: ".print_r($fieldsource,true));
				fclose($tta);
				if (!isset($GLOBALS['placelistcount']))
					$GLOBALS['placelistcount']=1;
				else
					$GLOBALS['placelistcount']++;				
			}
        }
        return $table;
    }

    function fieldObject(&$form_fieldname, &$fieldrecord, $record) {
        $fieldobj = new htmlTable(CLX_WIDTH_TWO);
        $fieldobj->setProps(array('border' => '0'));
        $fn = $form_fieldname;
        for ($i = CLX_INIT_ZERO; $i <= CLX_NUM_NINE; $i++) {
		  if (!array_key_exists($fn . '_r' . $i, $record))
			 $record[$fn . '_r' . $i] = false;
        }
        unset($i);
        // End Recurrence
        if ($record[$fn . '_r9'] === '0000-00-00') {
		  $record[$fn . '_r9'] = '';
        }
        //For field below
        $datefieldobj = getToolObject('DBFT', 'DATE');
        $msg = array();
        $cell = $datefieldobj->placeEditFieldObj('theForm', $msg, $record[$fn . '_r9'], 'DATE', $fn . '_r9', $fn . '_r9', 'CLX_DBTF_RECU_ENDDATE', CLX_NUM_30, CLX_NUM_30, '', '', '', false, false, false);
        $fieldobj->addCell($cell->source(), array(CLX_PROP_COLSPAN => CLX_WIDTH_TWO, 'nowrap' => 'true'));
        $check = new htmlCheckbox($fn . '_r0', $record[$fn . '_r0']);
        $anchor = new htmlAnchor(getSkinIcon('DBTF_recurrence', '', 'Detail'), 'javascript:donothing();');
        $anchor->setProp('onClick', 'showpanel(' . singleQuote($fn . '_DET') . ')');
        $fieldobj->addcell(dict('CLX_DBTF_RECU_noend') . ':' . $check->source() . $anchor->source(), array(CLX_PROP_CLASS => CLX_CLASS_DATAALT, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $detTable = new htmlTable(CLX_WIDTH_TWO);
        $detTable->setProps(array(CLX_PROP_WIDTH => CLX_PROP_100PCT));
        // Option menus
        $options = array('CLX_DBTF_RECU_none' => 'NONE', 'CLX_DBTF_RECU_day' => 'DAY', 'CLX_DBTF_RECU_week' => 'WEEK', 'CLX_DBTF_RECU_mnth' => 'MNTH', 'CLX_DBTF_RECU_year' => 'YEAR', 'CLX_DBTF_RECU_one' => 'ONE');
        $o = new htmlTable(CLX_WIDTH_TWO);
        foreach ($options as $key => $val) {
		  $radio = new htmlRadioButton($fn . '_r1', (($record[$fn . '_r1'] == $val) ? true : false), $val);
		  $radio->setProp('onClick', 'DBTF_recurence_field(' . singleQuote($fn) . ', this.value);');
		  $o->addcell(dict($key), array());
		  $o->addcell($radio->source(), array());
        }
        $js = setJavaScriptTag(getJavaScript('DBTF_recurrence_field.js', array()));
        $detTable->addcell($js . $o->source(), array(CLX_PROP_CLASS => CLX_CLASS_DATAALT));
        unset($radio, $options, $js, $o);
        $div = new htmlDiv($fn . '_NONE', 'display:' . (($record[$fn . '_r1'] === 'NONE') ? '' : 'none') . ';', CLX_CHAR_HTML_NBSP);
        $divs = $div->source();
        // Days Div
        $o = new htmlTable(CLX_WIDTH_ONE);
        $o->addcell(dict('CLX_DBTF_recurrenceby') . ': ', array());
        $select = new htmlSelect($fn . '_r3');
        $select->addItem(dict('CLX_DBTF_days'), '1', (($record[$fn . '_r3']) ? true : false));
        $select->addItem(dict('CLX_DBTF_weekdays'), '0', (!$record[$fn . '_r3']) ? true : false);
        $o->addcell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_DATAALT));
        $o->addcell(dict('GEN_LBL_every') . ': ', array());
        $htmlfield = new htmlField($fn . '_r2', $record[$fn . '_r2'], '20', '20');
        $o->addcell($htmlfield->source(), array('nowrap' => '', CLX_PROP_CLASS => CLX_CLASS_DATAALT));
        $div = new htmlDiv($fn . '_DAY', 'display:' . (($record[$fn . '_r1'] === 'DAY') ? '' : 'none') . ';', $o->source());
        $divs .= $div->source();
        // Week Div
        $o = new htmlTable(CLX_WIDTH_ONE);
        $o->addcell(dict('GEN_LBL_every') . ': ', array());
        $htmlfield = new htmlField($fn . '_r4', $record[$fn . '_r4'], '20', '20');
        $o->addcell($htmlfield->source() . ' ' . dict('GEN_LBL_WEEK-S'), array('nowrap' => ''));
        $o->addcell(dict('CLX_DBTF_on') . ' ', array());
        $select = new htmlSelect($fn . '_r7');
        $select->addItem($this->weekDays(CLX_NUM_ZERO), CLX_NUM_8, (($record[$fn . '_r7'] == CLX_NUM_8) ? true : false));
        $select->addItem($this->weekDays(CLX_NUM_ONE), CLX_NUM_ONE, (($record[$fn . '_r7'] == CLX_NUM_ONE) ? true : false));
        $select->addItem($this->weekDays(CLX_NUM_TWO), CLX_NUM_TWO, (($record[$fn . '_r7'] == CLX_NUM_TWO) ? true : false));
        $select->addItem($this->weekDays(CLX_NUM_THREE), CLX_NUM_THREE, (($record[$fn . '_r7'] == CLX_NUM_THREE) ? true : false));
        $select->addItem($this->weekDays(CLX_NUM_FOUR), CLX_NUM_FOUR, (($record[$fn . '_r7'] == CLX_NUM_FOUR) ? true : false));
        $select->addItem($this->weekDays(CLX_NUM_FIVE), CLX_NUM_FIVE, (($record[$fn . '_r7'] == CLX_NUM_FIVE) ? true : false));
        $select->addItem($this->weekDays(CLX_NUM_SIX), CLX_NUM_SIX, (($record[$fn . '_r7'] == CLX_NUM_SIX) ? true : false));
        $select->addItem($this->weekDays(CLX_NUM_7), CLX_NUM_7, (($record[$fn . '_r7'] == CLX_NUM_7) ? true : false));
        $o->addcell($select->source(), array());
        $div = new htmlDiv($fn . '_WEEK', 'display:' . (($record[$fn . '_r1'] === 'WEEK') ? '' : 'none') . ';', $o->source());
        $divs .= $div->source();
        // Month Div
        $o = new htmlTable(CLX_WIDTH_ONE);
        $o->addcell(dict('GEN_LBL_every') . ': ', array());
        $htmlfield = new htmlField($fn . '_r5', $record[$fn . '_r5'], '20', '20');
        $o->addcell($htmlfield->source() . ' ' . dict('GEN_LBL_MONTH-S'), array('nowrap' => ''));
        $select = new htmlSelect($fn . '_r6');
        //Ex: The 1st Friday of every month
        $select->addItem(dict('CLX_DBTF_dayofmonth'), CLX_NUM_ZERO, ((!$record[$fn . '_r6']) ? true : false));
        //Ex: The 14th of every month
        $select->addItem(dict('CLX_DBTF_dateinfuture'), CLX_NUM_ONE, (($record[$fn . '_r6']) ? true : false));
        $div = new htmlDiv($fn . '_MNTH', 'display:' . (($record[$fn . '_r1'] === 'MNTH') ? '' : 'none') . ';', $o->source());
        $divs .= $div->source();
        // Year Div
        $o = new htmlTable(CLX_WIDTH_ONE);
        $o->addcell(dict('GEN_LBL_every') . ': ', array());
        $htmlfield = new htmlField($fn . '_r8', $record[$fn . '_r8'], '20', '20');
        $o->addcell($htmlfield->source() . dict('CLX_DBTF_year'), array('nowrap' => ''));
        $div = new htmlDiv($fn . '_YEAR', 'display:' . (($record[$fn . '_r1'] === 'YEAR') ? '' : 'none') . ';', $o->source());
        $divs .= $div->source();
        // One Div
        $div = new htmlDiv($fn . '_ONE', 'display:' . (($record[$fn . '_r1'] === 'ONE') ? '' : 'none') . ';', CLX_CHAR_HTML_NBSP);
        $divs .= $div->source();
        $detTable->addcell($divs, array('valign' => 'middle', CLX_PROP_CLASS => CLX_CLASS_DATAALT));
        $div = new htmlDiv($fn . '_DET', 'display:none;', $detTable->source());
        $fieldobj->addcell($div->source(), array(CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        return $fieldobj;
    }

// End method
    function placeViewField($form_fieldname, $fieldrecord, $record, $msg = array()) {
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_WIDTH => CLX_PROP_100PCT));
        if (empty($fieldrecord['FIELD_HIDDEN'])) {
		  $cpt = dict($fieldrecord['CAPTION']);
		  if (strlen($cpt) > CLX_NUM_ZERO) {
			 $table->addCell($cpt, array(CLX_PROP_CLASS => $fieldrecord['CAPTION_CLASS'], CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
		  }
		  if (isset($msg[$form_fieldname])) {
			 $table->addCell($msg[$form_fieldname], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
		  }
		  $pfx = dict($fieldrecord['FIELDPREFIX'],true,false); 
		  $sfx = dict($fieldrecord['FIELDSUFIX'],true,false); 
		  $cell = $this->formatField($form_fieldname, $fieldrecord, $record);
		  // Field Anchor
		  $cell = $this->placeFieldAnchor($form_fieldname, $fieldrecord, $record, $cell);
		  $table->addCell(dict($fieldrecord['FIELDFORMLABEL']), array(CLX_PROP_CLASS => CLX_CLASS_PROMPT));
		  $table->addCell($pfx . ((strlen($cell) == CLX_NUM_ZERO) ? CLX_CHAR_HTML_NBSP : $cell) . $sfx, array(CLX_PROP_CLASS => CLX_CLASS_DATA));
        }
        return $table;
    }

// End method
    function placeSearchField($form_fieldname, $fieldrecord, $record, &$form) {
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_WIDTH => CLX_PROP_100PCT));
        return $table;
    }

    function placeListField($form_fieldname, $fieldrecord, $record, $class, &$table,$csv=1) {
        $pfx = dict($fieldrecord['FIELDPREFIX'],true,false); 
        $sfx = dict($fieldrecord['FIELDSUFIX'],true,false); 
        $cell = $this->formatField($form_fieldname, $fieldrecord, $record);
        // Field Anchor
        $cell = $this->placeFieldAnchor($form_fieldname, $fieldrecord, $record, $cell);
        $table->addCell($pfx . $cell . $sfx, array(CLX_PROP_CLASS => $class));
		if ( $this->debug==true ) {
			$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");
			fwrite($tta,chr(13).chr(10).$pos."_".$te[$posx]["function"]."dbft_recurrent-hora:".date("h:i:s d/m/Y").chr(13).chr(10));
			$te=debug_backtrace();
			for ($t=0;$t<12;$t++) {
				if (isset($te[$t]["file"]))
					fwrite($tta,substr($te[$t]["file"].str_repeat(" ",80),0,80).chr(9).$te[$t]["line"].chr(9).$te[$t]["function"].chr(13).chr(10));
				
			}

			fwrite($tta,chr(13).chr(10)."placeListFieldText: ".print_r($cell,true));
			fclose($tta);
			if (!isset($GLOBALS['placelistcount']))
				$GLOBALS['placelistcount']=1;
			else
				$GLOBALS['placelistcount']++;			
		}
    }

    function saveField($form_fieldname, $fieldrecord, &$record, $recordid) {
        $fn = $form_fieldname;
        $fr = uppercase($fieldrecord['FIELDNAME']);
        if (empty(UserRequest::$post[$fn . '_r1']) || UserRequest::$post[$fn . '_r1'] === 'NONE')
		  $record[$fr . '_R9'] = '';
        else
		  $record[$fr . '_R9'] = xFormat::date(UserRequest::$post[$fn . '_r9'], 'Y-m-d H:i:s');
        $record[$fr . '_R0'] = (!empty(UserRequest::$post[$fn . '_r0'])) ? CLX_NUM_ONE : CLX_NUM_ZERO;
        $record[$fr . '_R1'] = (array_key_exists($fn . '_r1', UserRequest::$post)) ? UserRequest::$post[$fn . '_r1'] : '';
        $record[$fr . '_R3'] = (array_key_exists($fn . '_r3', UserRequest::$post)) ? UserRequest::$post[$fn . '_r3'] : '';
        $record[$fr . '_R2'] = (array_key_exists($fn . '_r2', UserRequest::$post)) ? UserRequest::$post[$fn . '_r2'] : '';
        $record[$fr . '_R4'] = (array_key_exists($fn . '_r4', UserRequest::$post)) ? UserRequest::$post[$fn . '_r4'] : '';
        $record[$fr . '_R7'] = (array_key_exists($fn . '_r7', UserRequest::$post)) ? UserRequest::$post[$fn . '_r7'] : '';
        $record[$fr . '_R5'] = (array_key_exists($fn . '_r5', UserRequest::$post)) ? UserRequest::$post[$fn . '_r5'] : '';
        $record[$fr . '_R6'] = (array_key_exists($fn . '_r6', UserRequest::$post)) ? UserRequest::$post[$fn . '_r6'] : '';
        $record[$fr . '_R8'] = (array_key_exists($fn . '_r8', UserRequest::$post)) ? UserRequest::$post[$fn . '_r8'] : '';
    }

    function loadField($form_fieldname, &$fieldrecord, &$record, $values) {
        $fn = $form_fieldname;
        $fr = uppercase($fieldrecord['FIELDNAME']);
        $record[$fn . '_r9'] = $values[$fr . '_R9'];
        $record[$fn . '_r0'] = $values[$fr . '_R0'];
        $record[$fn . '_r1'] = $values[$fr . '_R1'];
        $record[$fn . '_r3'] = $values[$fr . '_R3'];
        $record[$fn . '_r2'] = $values[$fr . '_R2'];
        $record[$fn . '_r4'] = $values[$fr . '_R4'];
        $record[$fn . '_r7'] = $values[$fr . '_R7'];
        $record[$fn . '_r5'] = $values[$fr . '_R5'];
        $record[$fn . '_r6'] = $values[$fr . '_R6'];
        $record[$fn . '_r8'] = $values[$fr . '_R8'];
    }

    function defaultField($form_fieldname, $fieldrecord, &$record) {
        $fn = $form_fieldname;
        $record[$fn . '_r9'] = '';
        $record[$fn . '_r0'] = '';
        $record[$fn . '_r1'] = trim($fieldrecord['FIELD_DEFAULT']);
        $record[$fn . '_r3'] = '';
        $record[$fn . '_r2'] = '';
        $record[$fn . '_r4'] = '';
        $record[$fn . '_r7'] = '';
        $record[$fn . '_r5'] = '';
        $record[$fn . '_r6'] = '';
        $record[$fn . '_r8'] = '';
    }

    function formatField($form_fieldname, $fieldrecord, $record) {
             //  comentada consume memoria pero uppercase no trabaja acentos los lookups demasiado en el export  $form_fieldname = uppercase($form_fieldname);
		$form_fieldname = uppercase($form_fieldname);

        $record = array_change_key_case($record, CASE_UPPER);
        $fn = $form_fieldname;
        if (strlen($record[$fn . '_R1']) > CLX_NUM_ZERO) {
		  $c = dict('CLX_DBTF_RECU_' . lowercase($record[$fn . '_R1']));
		  if (empty($record[$fn . '_R1']) || $record[$fn . '_R1'] !== 'NONE' && $record[$fn . '_R1'] !== 'ONE') {
			 if ($record[$fn . '_R0'] === true)
			     $c .= ', ' . dict('CLX_DBTF_RECU_noend');
			 else
			     $c .=', ' . dict('CLX_DBTF_RECU_enddate') . ':' . $record[$fn . '_R9'];
		  }
		  switch ($record[$fn . '_R1']) {
			 case 'DAY':
			     $c .='<br>' . dict('GEN_LBL_every') . ':' . $record[$fn . '_R2'];
			     $c .=' ' . (($record[$fn . '_R3']) ? dict('CLX_DBTF_days') : dict('CLX_DBTF_weekdays'));
			     break;
			 case 'WEEK':
			     $c .='<br>' . dict('GEN_LBL_every') . ': ' . $record[$fn . '_R4'];
			     $c .=' ' . $this->weekDays($record[$fn . '_R7']);
			     if ($record[$fn . '_R7'] > CLX_NUM_ZERO)
			         $c .='(s)';
			     break;
			 case 'MNTH':
			     $c .='<br>' . dict('GEN_LBL_every') . ': ' . $record[$fn . '_R5'];
			     break;
			 case 'YEAR':
			     $c .='<br>' . dict('GEN_LBL_every') . ': ';
			     $c .=$record[$fn . '_R8'] . ' ' . dict('CLX_DBTF_year');
			     break;
			 default:
			     break;
		  }
        }else {
		  $c = '';
        }
        return $c;
    }

    function queryFieldColumn($fieldname, $tablecode = '', $expression = '') {
        $return = array();
        $flag = (strlen($tablecode) > CLX_NUM_ZERO) ? true : false;
        $recu = array();
        $recu[] = $fieldname . '_R9';
        $recu[] = $fieldname . '_R0';
        $recu[] = $fieldname . '_R1';
        $recu[] = $fieldname . '_R3';
        $recu[] = $fieldname . '_R2';
        $recu[] = $fieldname . '_R4';
        $recu[] = $fieldname . '_R7';
        $recu[] = $fieldname . '_R5';
        $recu[] = $fieldname . '_R6';
        $recu[] = $fieldname . '_R8';
        foreach ($recu as $key => $val) {
		  if (!empty($flag))
			 $return[] = $tablecode . '.' . $val . ' ' . $tablecode . '_' . $val;
		  else
			 $return[] = $val;
        }
        return implode(',', $return);
    }

    function queryFieldSorts($fieldname, $tablecode = '', $sort = '', $work_field = false) {
        if (strlen($tablecode) > CLX_NUM_ZERO) {
		  $fieldname = $tablecode . '.' . $fieldname . '_R1';
        }
        return $fieldname . ' ' . $sort;
    }

// End method
    function weekDays($value) {
        $contents = '';
        $weekday[CLX_NUM_ZERO] = 'Weekday';
        $weekday[CLX_NUM_ONE] = 'Monday';
        $weekday[CLX_NUM_TWO] = 'Tuesday';
        $weekday[CLX_NUM_THREE] = 'Wednesday';
        $weekday[CLX_NUM_FOUR] = 'Thursday';
        $weekday[CLX_NUM_FIVE] = 'Friday';
        $weekday[CLX_NUM_SIX] = 'Saturday';
        $weekday[CLX_NUM_7] = 'Sunday';
        if (isset($weekday[$value]))
		  $contents = dict('CLX_DBTF_' . $weekday[$value]);
        return $contents;
    }

// End method
    /**
     * Next Run Date
     */
    function next_run_date($fieldname, $last_date, $record) {
        $fn = $fieldname;
        $next_run_date = '';
        $last_run_date = adodb_getdate(adodb_strtotime($last_date));
        if (trim($record[$fn . '_R9']) == '0000-00-00 00:00:00' || ($record[$fn . '_R9']) == '0000-00-00') {
		  $stop_at = '-1';
        } else {
		  $stop_at = xFormat::date($record[$fn . '_R9'], 'Y-m-d');
        }
        if ($stop_at > - CLX_NUM_ONE && $last_date >= $stop_at) {
            //return CLX_CHAR_EMPTY_STRING;
            return $last_date;
        }
        switch ($record[$fn . '_R1']) {
		  case 'ONE'://One Time only
                //return CLX_CHAR_EMPTY_STRING;
                return $last_date;
			 break;
		  case 'DAY'://Daily
			 /* Every Day dom->lun or Every Week Date meaning lun->vie */
			 if ($record[$fn . '_R3'] === true) {
			     //Significa n Lunes, Martes
			     $next_run_date = adodb_date('Y-m-d H:i:s', adodb_mktime($last_run_date['hours'], $last_run_date['minutes'], $last_run_date['seconds'], $last_run_date['mon'], $last_run_date['mday'] + $record[$fn . '_R2'], $last_run_date['year']));
			 } else {
			     //Cada n Dias
			     $next_run_date = adodb_date('Y-m-d H:i:s', adodb_mktime($last_run_date['hours'], $last_run_date['minutes'], $last_run_date['seconds'], $last_run_date['mon'], $last_run_date['mday'] + $record[$fn . '_R2'], $last_run_date['year']));
			 }
			 break;
		  case 'WEEK':
			 $next_run_date = adodb_date('Y-m-d H:i:s', adodb_mktime($last_run_date['hours'], $last_run_date['minutes'], $last_run_date['seconds'], $last_run_date['mon'], $last_run_date['mday'] + CLX_NUM_7, $last_run_date['year']));
			 break;
		  case 'MNTH':
			 $month_of_last_day = adodb_date('m', adodb_mktime($last_run_date['hours'], $last_run_date['minutes'], $last_run_date['seconds'], $last_run_date['mon'] + CLX_NUM_TWO, CLX_NUM_ZERO, $last_run_date['year']));
			 $month_of_one_month = adodb_date('m', adodb_mktime($last_run_date['hours'], $last_run_date['minutes'], $last_run_date['seconds'], $last_run_date['mon'] + CLX_NUM_ONE, $last_run_date['mday'], $last_run_date['year']));
			 if ($month_of_last_day == $month_of_one_month) {
			     if (adodb_month_last_day($last_date)) {
			         $next_run_date = adodb_date('Y-m-d H:i:s', adodb_mktime($last_run_date['hours'], $last_run_date['minutes'], $last_run_date['seconds'], $last_run_date['mon'] + $record[$fn . '_R5'] + CLX_NUM_ONE, CLX_NUM_ZERO, $last_run_date['year']));
			     } else {
			         $next_run_date = adodb_date('Y-m-d H:i:s', adodb_mktime($last_run_date['hours'], $last_run_date['minutes'], $last_run_date['seconds'], $last_run_date['mon'] + $record[$fn . '_R5'], $last_run_date['mday'], $last_run_date['year']));
			     }
			 } else {
			     $next_run_date = adodb_date('Y-m-d H:i:s', adodb_mktime($last_run_date['hours'], $last_run_date['minutes'], $last_run_date['seconds'], $last_run_date['mon'] + ($record[$fn . '_R5'] + CLX_NUM_ONE), CLX_NUM_ZERO, $last_run_date['year']));
			 }
			 unset($month_of_last_day, $month_of_one_month);
			 break;
		  case 'YEAR':
			 $next_run_date = adodb_date('Y-m-d H:i:s', adodb_mktime($last_run_date['hours'], $last_run_date['minutes'], $last_run_date['seconds'], $last_run_date['mon'], $last_run_date['mday'], $last_run_date['year'] + $record[$fn . '_R8']));
			 break;
		  default:
			 break;
        }//End switch
        return $next_run_date;
    }

}
