<?php

/**
 * Text field object class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: cramirez $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2018-07-30 20:48:09 -0400 (Mon, 30 Jul 2018) $
 * @version    	$Rev: 9869 $
 */
if (!class_exists('dbft') && !defined('clxInstallProcess'))
    include_once kPathToObjects . 'dbft.php';

class dbft_text extends dbft {

    // CONSTRUCTOR //
    function __construct() {
        $this->code = 'TEXT';
        $this->name = 'Text';
        $this->type = 'DBFT';
        $this->version = '0.2.4.2';
        $this->description = 'Can somebody place a nice description here?';
        $this->multifield = false; // Describes it field is a multi field see properties;
        $this->has_lookup = true; // Decribes if field can be set as copy field on lookup
        $this->has_question = true; // Describes if field can be used on queries criteria
        $this->operators = array('===' => 'GEN_LBL_EQUALTO', '!== ' => 'GEN_LBL_NOTEQUALTO',
		  '%=' => 'GEN_LBL_STARTSWITH',
		  '=%' => 'GEN_LBL_ENDWITH',
		  '%%' => 'GEN_LBL_CONTEINS');
    }

    function panel($msg, $fieldrecord, &$form) {
        $c = $this->code;
        if ($fieldrecord['DBFL_fieldtype'] != $c) {
		  $this->defaultPanel($fieldrecord);
        }

        $size = array(dict('GEN_LBL_SMALL'), dict('GEN_LBL_MEDIUM'), dict('GEN_LBL_LARGE'));
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM));
        $table->addCell(dict('GEN_LBL_TEXTFIELDPROP'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $select = new htmlSelect($c . '_size');
        $select->addItem($size[CLX_NUM_ZERO], '0', ($fieldrecord[$c . '_size'] == CLX_NUM_ZERO));
        $select->addItem($size[CLX_NUM_ONE], '1', ($fieldrecord[$c . '_size'] == CLX_NUM_ONE));
        $select->addItem($size[CLX_NUM_TWO], '2', ($fieldrecord[$c . '_size'] == CLX_NUM_TWO));
        $table->addData(dict('GEN_LBL_SIZE') . ':', $select->source());
        $table->addField($msg, dict('GEN_LBL_NUMBEROFLINES') . ':', $this->code . '_extra1', $fieldrecord[$c . '_extra1'], CLX_NUM_FOUR, CLX_NUM_FOUR);
        $table->addField($msg, dict('GEN_LBL_CHARSPERLINE') . ':', $this->code . '_extra2', $fieldrecord[$c . '_extra2'], CLX_NUM_FOUR, CLX_NUM_FOUR);
        $div = new htmlDiv($this->code, 'visibility:hidden; display:none', $table->source());
        return CLX_CHAR_NL . CLX_CHAR_NL . $div->source() . CLX_CHAR_NL . CLX_CHAR_NL;
    }

    function checkPanel(&$msg) {
        $c = $this->code;
        xValidate::integer($msg, $c . '_extra1', UserRequest::$post[$c . '_extra1'], CLX_NUM_ONE, CLX_NUM_255, dict('GEN_LBL_NUMBEROFLINES'));
        xValidate::integer($msg, $c . '_extra2', UserRequest::$post[$c . '_extra2'], CLX_NUM_ONE, CLX_NUM_255, dict('GEN_LBL_CHARSPERLINE'));
    }

    function savePanel(&$fieldrecord) {
        $c = $this->code;
        $fieldrecord['FIELD_SIZE'] = UserRequest::$post[$c . '_size'];
        $fieldrecord['FIELD_REQUIRED'] = CLX_INIT_ZERO;
        $fieldrecord['FIELD_FORMAT'] = '';
        $fieldrecord['FIELD_ALIGN'] = '0';
        $fieldrecord['FIELD_DEFAULT'] = '';
        $fieldrecord['FIELD_MINVALUE'] = '';
        $fieldrecord['FIELD_MAXVALUE'] = '';
        $fieldrecord['FIELD_EXTRA1'] = UserRequest::$post[$c . '_extra1']; // number of lines
        $fieldrecord['FIELD_EXTRA2'] = UserRequest::$post[$c . '_extra2']; // characters per line
        $fieldrecord['FIELD_EXTRA3'] = '';
        $fieldrecord['FIELD_EXTRA4'] = '';
        $fieldrecord['FIELD_EXTRA5'] = '';
    }

    function loadPanel(&$fieldrecord, $values) {
        $c = $this->code;
        if ($values['FIELDTYPE'] == $c) {
		  $fieldrecord[$c . '_size'] = $values['FIELD_SIZE'];
		  $fieldrecord[$c . '_required'] = CLX_INIT_ZERO;
		  $fieldrecord[$c . '_extra1'] = $values['FIELD_EXTRA1'];
		  $fieldrecord[$c . '_extra2'] = $values['FIELD_EXTRA2'];
        } else {
		  $this->defaultPanel($fieldrecord);
        }
    }

    function defaultPanel(&$fieldrecord) {
        $c = $this->code;
        $fieldrecord[$c . '_size'] = CLX_NUM_ONE;
        $fieldrecord[$c . '_required'] = CLX_INIT_ZERO;
        $fieldrecord[$c . '_format'] = '';
        $fieldrecord[$c . '_align'] = CLX_INIT_ZERO;
        $fieldrecord[$c . '_default'] = '';
        $fieldrecord[$c . '_minvalue'] = '';
        $fieldrecord[$c . '_maxvalue'] = '';
        $fieldrecord[$c . '_extra1'] = '4';
        $fieldrecord[$c . '_extra2'] = '48';
        $fieldrecord[$c . '_extra3'] = '';
        $fieldrecord[$c . '_extra4'] = '';
        $fieldrecord[$c . '_extra5'] = '';
    }

    function viewForm(&$table, $record) {

        $size = array(dict('GEN_LBL_SMALL'), dict('GEN_LBL_MEDIUM'), dict('GEN_LBL_LARGE'));
        $table->addData(dict('GEN_LBL_SIZE') . ':', $size[$record['TEXT_size']]);
        $table->addData(dict('GEN_LBL_NUMBEROFLINES') . ':', $record['TEXT_extra1']);
        $table->addData(dict('GEN_LBL_CHARSPERLINE') . ':', $record['TEXT_extra2']);
    }

    function fieldProperties($fieldrecord) {
        // DB implementation
        switch (kDatabaseType) {
		  case 'oci':
			 $size = array('CLOB', 'CLOB', 'CLOB');
			 break;
		  default:
			 $size = array('BLOB', 'MEDIUMBLOB', 'LONGBLOB');
			 break;
        }
        $props = array();
        $props['type'] = $size[$fieldrecord['FIELD_SIZE']];
        $props['default'] = 'null';
        $props['properties'] = '';
        $props['null'] = 'YES';
        return $props;
    }

    function fieldObject(&$form_fieldname, &$fieldrecord, $record) {
        if (!array_key_exists($form_fieldname, $record))
		  $record[$form_fieldname] = '';
        $fieldobj = new htmlTextArea($form_fieldname, $record[$form_fieldname], $fieldrecord['FIELD_EXTRA1'], $fieldrecord['FIELD_EXTRA2']);
        //Added instead of displaying read only files
        if (!empty($fieldrecord['FIELD_READONLY'])) {
		  $fieldobj->setProp('readonly', 'true');
		  $fieldobj->setProp(CLX_PROP_CLASS, 'readonly');
        }
        return $fieldobj;
    }

    function formatField($form_fieldname, $fieldrecord, $record) {
        $form_fieldname = uppercase($form_fieldname);
        $record = array_change_key_case($record, CASE_UPPER);
        if(!empty($fieldrecord['DBQF_FIELD_FORMAT']) &&   $fieldrecord['DBQF_FIELD_FORMAT'] === '1'){
		  if (array_key_exists($form_fieldname, $record)) {
			 $original_value = $record[$form_fieldname];
		  } else {
			 $original_value = '';
		  }
		  return $original_value;
        }
        return $record[$form_fieldname];
    }

    function validateField(&$msg, $form_fieldname, $fieldrecord, &$record, &$table, &$form) {
		
        //Setting form require pref if any
        if (array_key_exists('FIELD_FORM_REQUIRED', $fieldrecord) && $fieldrecord['FIELD_FORM_REQUIRED'] != CLX_NUM_TWO) {
		  $fieldrecord['FIELD_REQUIRED'] = $fieldrecord['FIELD_FORM_REQUIRED'];
        }
        // Note: !array_key_exists was added to enable lookups to validate
        if (!array_key_exists('FIELD_HIDDEN', $fieldrecord))
		  $fieldrecord['FIELD_HIDDEN'] = false;
        //Setting hidden to override required
        if (empty($fieldrecord['FIELD_HIDDEN'])) {
		  if (UserRequest::$request['ac']!='DELETE' &&(strlen($record[$form_fieldname]) == CLX_NUM_ZERO) and $fieldrecord['FIELD_REQUIRED']) {
			 $msg[$form_fieldname] = dict('GEN_LBL_FIELDREQUIRED');
		  }
        }//end if
    }

    function saveField($form_fieldname, $fieldrecord, &$record, $recordid) {
        parent::saveField($form_fieldname, $fieldrecord, $record, $recordid);
        $record[$fieldrecord['FIELDNAME']] = clx_html_spchrs_decode($record[$fieldrecord['FIELDNAME']]);
    }

}
