<?php

/**
 * Object Class
 * For more information contact  support@alenet.com
 *
 * @package		Clientix.Tool.Analytics
 * @abstract		Report Type Object
 * @author       $Author: jclerma $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2017-11-03 15:09:40 -0400 (Fri, 03 Nov 2017) $
 * @version        $Rev: 9615 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  Included label
 */
if (!class_exists('rpft') && !defined('clxInstallProcess')) {
    include_once kPathToObjects . 'rpft.php';
}
if (!class_exists('FPDF') && file_exists(kPathToWebSite.kPathToFpdf . 'fpdf.php')) {
    include_once kPathToWebSite.kPathToFpdf . 'fpdf.php';
}

class rpft_pdf extends FPDF {

    var $table;
    var $width;
    var $needheaders;
    var $linecount;
    var $fpdf = false;
    var $paper_size;
    var $tblw;
    var $tbla;
    var $tblv;
    var $date_format;
    var $boton_margin;

    // CONSTRUCTOR //
    function __construct($orientation = 'P', $unit = 'mm', $size = 'A4') {
        parent::__construct($orientation,$unit,$size);
        $this->code = 'RPPD';
        $this->name = 'PDF';
        $this->type = 'RPFT';
        $this->version = '1.0.0.0';
        $this->description = 'PDF Report Format';
        $this->windowurl = 'pop.php';
        $this->paper_size = 'Letter';
        $this->units = 'mm';
        $this->orientation = 'P';
        $this->hdrs = array(CLX_CLASS_TITLE => '', CLX_PROP_WIDTH => CLX_NUM_ZERO, 'height' => CLX_NUM_ZERO, CLX_PROP_ALIGN => 'C', 'border' => '1', 'ln' => CLX_NUM_ZERO, 'bgcolor' => '', 'logo_img' => '', 'logo_x' => false, 'logo_y' => false, 'logo_width' => false, 'logo_height' => false, 'page_extra' => '', 'pagination' => true, 'date' => true, 'table_headers' => array(), 'table_headerso' => array(), 'table_headers_uaf' => array());
        $this->footers = array('enabled' => false, 'extra1' => '', 'extra2' => '', 'address1' => '', 'address2' => '', 'page_location' => 'C');
        $this->tblw = array();
        $this->tbla = array();
        $this->tblv = array();
        $this->date_format = 'd/m/Y';
        $this->AliasNbPages('{nb}');
        $this->boton_margin = CLX_NUM_30;
	$this->SetAutoPageBreak(true, 100);
        $this->editPanel = array();
        $this->show_prompt_format = false; //untill output is develop
    }

// End method
    function build($reportid) {
		OB_START();
        //check access
        $this->getReportRecord($reportid);
        $DBQY = getTool('DBQY');
        $ANRP = getTool('ANRP');
        $qaccess = $DBQY->getAppAccess($this->reportrec['QUERYCODE']);
        if (!$qaccess->canRead()) {
		  return warning(dict('GEN_ERR_NO_ACCESS') . ' [RPTF_PDF-CAN_READ]' . ' ' . $ANRP->getCode(false) . ' Query:' . $this->reportrec['QUERYCODE']);
        }
        $contents = '<html><head><link href="' . skinPath() . '/styles.css" rel="stylesheet" type="text/css"></head><body>';
        $contents .= note(dict('GEN_LBL_NOTAVAILABLE') . '<br>');
        $contents .= '</body></html>';
        return $contents;
    }

    function setorientation($value) {
        $this->orientation = $value;
    }

// End method
    function setpaper($value) {
        $this->paper_size = $value;
    }

// End method
    function setunits($value) {
        $this->units = $value;
    }

// End method
    /**
     * Set Headers
     * @param type $parameter
     * @param type $value
     */
    function setHeaderAdd($parameter, $value) {
        $this->hdrs[$parameter] = $value;
    }

    /**
     * Set Footers
     * @param type $parameter
     * @param type $value
     */
    function setFooter($parameter, $value) {
        $this->footers[$parameter] = $value;
    }

    //Page header
    function Header() {
        $portrait = ($this->CurOrientation === 'P');
        if ($this->hdrs['logo_img'] <> '') {
		  $this->SetFont('Arial', '', CLX_NUM_7);
		  $this->Image($this->hdrs['logo_img'], $this->hdrs['logo_x'], $this->hdrs['logo_y'], $this->hdrs['logo_width'], $this->hdrs['logo_height']);
		  //$this->Ln(1);
		  if (!empty($this->hdrs['logo_text1'])) {
			 $temp = explode(CLX_CHAR_NL, $this->hdrs['logo_text1']);
			 $i = CLX_NUM_35;
			 foreach ($temp as $key => $val) {
			     $val = trim($val);
			     if (empty($val))
			         continue;
			     $this->Text($this->GetX(), $i, $val);
			     $i += CLX_NUM_THREE;
			 }
		  }
        }
        //Page and Date
        $this->SetFont('Arial', '', CLX_NUM_8);
        if ($this->paper_size === 'legal' && $this->fhPt <= CLX_NUM_1008) {
		  if ($this->hdrs['date'])
			 $this->Text(($portrait) ? CLX_NUM_180 : CLX_NUM_300, CLX_NUM_16, dict('GEN_LBL_DATE') . ':      ' . xFormat::getNow($this->date_format));
		  if ($this->hdrs['pagination'])
			 $this->Text(($portrait) ? CLX_NUM_180 : CLX_NUM_300, CLX_NUM_19, dict('GEN_LBL_PAGE') . ':     ' . $this->PageNo() . '/{nb}');
        }elseif ($this->paper_size === 'letter' && $this->fhPt <= CLX_NUM_1008) {
		  if ($this->hdrs['date'])
			 $this->Text(($portrait) ? CLX_NUM_180 : CLX_NUM_235, CLX_NUM_16, dict('GEN_LBL_DATE') . ':      ' . xFormat::getNow($this->date_format));
		  if ($this->hdrs['pagination'])
			 $this->Text(($portrait) ? CLX_NUM_180 : CLX_NUM_235, CLX_NUM_19, dict('GEN_LBL_PAGE') . ':     ' . $this->PageNo() . '/{nb}');
        }else {
		  if ($this->hdrs['date'])
			 $this->Text(($this->w - $this->rMargin - CLX_NUM_30), ($this->tMargin + CLX_NUM_FIVE), dict('GEN_LBL_DATE') . ':      ' . xFormat::getNow($this->date_format));
		  if ($this->hdrs['pagination'])
			 $this->Text(($portrait) ? CLX_NUM_180 : CLX_NUM_450, CLX_NUM_19, dict('GEN_LBL_PAGE') . ':     ' . $this->PageNo() . '/{nb}');
        }
        if ($this->hdrs['page_extra'] <> '') {
		  if ($this->paper_size === 'legal' && $this->fhPt <= CLX_NUM_1008) {
			 $this->Text(($portrait) ? CLX_NUM_160 : CLX_NUM_300, CLX_NUM_25, $this->hdrs['page_extra']);
		  } elseif ($this->paper_size === 'letter' && $this->fhPt <= CLX_NUM_1008) {
			 $this->Text(($portrait) ? CLX_NUM_160 : CLX_NUM_225, CLX_NUM_25, $this->hdrs['page_extra']);
		  } else {
			 $this->Text(($portrait) ? CLX_NUM_160 : CLX_NUM_450, CLX_NUM_25, $this->hdrs['page_extra']);
		  }
        }
        $this->Ln(CLX_NUM_12);
        $this->SetFont('Arial', 'B', CLX_NUM_11);
        $this->MultiCell($this->hdrs[CLX_PROP_WIDTH], $this->hdrs['height'], $this->hdrs[CLX_CLASS_TITLE], $this->hdrs['border'], $this->hdrs[CLX_PROP_ALIGN]);
        $this->Ln();
        if (!empty($this->hdrs['table_headers'])) {
		  $this->buildTableHeader($this->hdrs['table_headers']);
        } elseif (!empty($this->hdrs['table_headerso'])) {
		  $this->buildTableHeadero($this->hdrs['table_headerso']);
        } elseif (!empty($this->hdrs['table_headers_uaf'])) {
		  $this->buildTableHeaderuaf($this->hdrs['table_headers_uaf']);
        }
        if (!empty($this->hdrs['text'])) {
		  $this->SetFont('Arial', '', CLX_NUM_NINE);
		  $this->SetXY($this->hdrs['text_x'], $this->hdrs['text_y']);
		  foreach ($this->hdrs['text'] as $k => $v) {
			 $this->Cell($this->hdrs['text'][$k]['w'], $this->hdrs['text'][$k]['h'], $this->hdrs['text'][$k]['0'], $this->hdrs['text'][$k]['mrg'], $this->hdrs['text'][$k]['sl'], $this->hdrs['text'][$k]['al']);
		  }
		  if (((array_key_exists('x', $this->hdrs)) && ($this->hdrs['x'] <> CLX_NUM_ZERO)) && ((array_key_exists('y', $this->hdrs) && $this->hdrs['y'] <> CLX_NUM_ZERO))) {
			 $this->SetXY($this->hdrs['x'], $this->hdrs['y']);
		  } else {
			 $this->SetXY(CLX_NUM_TEN, CLX_NUM_78);
		  }
        }
        if (!empty($this->hdrs['text1'])) {
		  $this->SetFont('Arial', '', CLX_NUM_NINE);
		  $this->SetXY($this->hdrs['text1_x'], $this->hdrs['text1_y']);
		  foreach ($this->hdrs['text1'] as $k => $v) {
			 $this->Cell($this->hdrs['text1'][$k]['w'], $this->hdrs['text1'][$k]['h'], $this->hdrs['text1'][$k]['0'], $this->hdrs['text1'][$k]['mrg'], $this->hdrs['text1'][$k]['sl'], $this->hdrs['text1'][$k]['al']);
		  }
		  $this->SetXY($this->hdrs['text1_nx'], $this->hdrs['text1_ny']);
        }
    }

    function subTitle($num, $subtitle) {
        $this->Ln();
        //Arial 12
        $this->SetFont('Arial', '', CLX_NUM_12);
        //Background color
        $this->SetFillColor(CLX_NUM_204, CLX_NUM_204, CLX_NUM_204);
        //Title
        $this->Cell(CLX_NUM_ZERO, CLX_NUM_SIX, $subtitle, CLX_NUM_ZERO, CLX_NUM_ONE, 'L', CLX_NUM_ONE);
    }

    //Page footer
    function Footer() {
        if (empty($this->footers['enabled'])) {
		  return;
        }
        //Position at 15 cm from bottom
        $this->SetY(- CLX_NUM_15);
        //Arial italic 8
        $this->SetFont('Arial', 'I', CLX_NUM_8);
        //Page number
        $this->Cell(CLX_NUM_ZERO, CLX_NUM_TEN, dict('GEN_LBL_PAGE') . ': ' . $this->PageNo() . '/{nb}', CLX_NUM_ZERO, CLX_NUM_ZERO, $this->footers['page_location']);
        if ($this->footers['extra1'] <> '') {
		  $this->SetY(- CLX_NUM_26);
		  $this->SetX(CLX_NUM_ZERO);
		  $this->SetFont('Arial', 'B', CLX_NUM_SIX);
		  $this->Cell(CLX_NUM_ZERO, CLX_NUM_TEN, $this->footers['extra1'], CLX_NUM_ZERO, CLX_NUM_ZERO, 'C');
        }
        if ($this->footers['extra2'] <> '') {
		  $this->SetY(- CLX_NUM_23);
		  $this->SetX(CLX_NUM_ZERO);
		  $this->SetFont('Arial', 'B', CLX_NUM_SIX);
		  $this->Cell(CLX_NUM_ZERO, CLX_NUM_TEN, $this->footers['extra2'], CLX_NUM_ZERO, CLX_NUM_ZERO, 'C');
        }
        if ($this->footers['address1'] <> '') {
		  $this->SetY(- CLX_NUM_19);
		  $this->SetX(CLX_NUM_ZERO);
		  $this->SetFont('Arial', 'I', CLX_NUM_SIX);
		  $this->Cell(CLX_NUM_ZERO, CLX_NUM_TEN, $this->footers['address1'], CLX_NUM_ZERO, CLX_NUM_ZERO, 'C');
        }
        if ($this->footers['address2'] <> '') {
		  $this->SetY(- CLX_NUM_17);
		  $this->SetX(CLX_NUM_ZERO);
		  $this->SetFont('Arial', 'I', CLX_NUM_SIX);
		  $this->Cell(CLX_NUM_ZERO, CLX_NUM_TEN, $this->footers['address2'], CLX_NUM_ZERO, CLX_NUM_ZERO, 'C');
        }
    }

    //TABLES
    //Load data
    function LoadData($file) {
        //Read file lines
        $lines = file($file);
        $data = array();
        foreach ($lines as $line)
		  $data[] = explode(';', chop($line));
        return $data;
    }

    //Simple table
    function buildTableHeader($header) {
        $this->tblw = array();
        $this->tbla = array();
        //Header
        $this->SetFont('Arial', 'B', CLX_NUM_8);
        $this->SetFillColor(CLX_NUM_201, CLX_NUM_201, CLX_NUM_201);
        $this->Ln();
        $total = count($header);
        for ($i = CLX_INIT_ZERO; $i < $total; $i++) {
		  $this->tblw[$i] = $header[$i][CLX_NUM_ONE];
		  $this->tbla[$i] = $header[$i][CLX_NUM_TWO];
		  if (!empty($header[$i]['w']))
			 $w = $header[$i]['w'];
		  else
			 $w = CLX_NUM_TEN;
		  if (strpos($header[$i][CLX_NUM_ZERO], CLX_CHAR_NL) !== false) {
			 //e cho $header[$i][CLX_NUM_ZERO].'<br>';
			 $x = $this->getX();
			 $this->Cell($this->tblw[$i], $w, '', '1', '0', 'C', CLX_NUM_ONE);
			 $parts = explode(CLX_CHAR_NL, $header[$i][CLX_NUM_ZERO]);
			 foreach ($parts as $key => $val) {
			     $this->setX($x);
			     //print_r($parts);
			     $this->Cell($this->tblw[$i], ($w + ($key * CLX_NUM_7)), $val, '0', '0', 'C', CLX_NUM_ZERO);
			 }
		  } else {
			 $this->Cell($this->tblw[$i], $w, $header[$i][CLX_NUM_ZERO], '1', '0', 'C', CLX_NUM_ONE);
		  }
        }
        $this->Ln();
    }

    //Simple table
    function buildTableHeadero($header) {
        $this->tblw = array();
        $this->tbla = array();
        //Header
        $this->SetFont('Arial', 'B', CLX_NUM_8);
        $this->SetFillColor(CLX_NUM_201, CLX_NUM_201, CLX_NUM_201);
        $this->Ln();
        $total = count($header);
        for ($i = CLX_INIT_ZERO; $i < $total; $i++) {

		  $this->tblw[$i] = $header[$i][CLX_NUM_ONE];
		  $this->tbla[$i] = $header[$i][CLX_NUM_TWO];

		  if ($i == CLX_NUM_ZERO) {
			 $this->SetXY(CLX_NUM_TEN, CLX_NUM_54);
			 $this->MultiCell($header[$i]['w'], $header[$i][CLX_NUM_ONE], $header[$i][CLX_NUM_ZERO], CLX_NUM_ONE, $header[$i][CLX_NUM_TWO], true);
		  } elseif ($i == CLX_NUM_ONE) {
			 $this->SetXY((CLX_NUM_TEN + $header[CLX_NUM_ZERO]['w']), CLX_NUM_54);
			 $this->Cell($header[$i]['w'], $header[$i][CLX_NUM_ONE], $header[$i][CLX_NUM_ZERO], CLX_NUM_ONE, $header[$i]['SL'], $header[$i][CLX_NUM_TWO], true);
		  } elseif ($i == CLX_NUM_THREE) {
			 $this->SetXY((CLX_NUM_TEN + $header[CLX_NUM_ZERO]['w']), (CLX_NUM_54 + $header[CLX_NUM_ONE][CLX_NUM_ONE]));
			 $this->Cell($header[$i]['w'], $header[$i][CLX_NUM_ONE], $header[$i][CLX_NUM_ZERO], CLX_NUM_ONE, $header[$i]['SL'], $header[$i][CLX_NUM_TWO], true);
		  } else {
			 $this->Cell($header[$i]['w'], $header[$i][CLX_NUM_ONE], $header[$i][CLX_NUM_ZERO], CLX_NUM_ONE, $header[$i]['SL'], $header[$i][CLX_NUM_TWO], true);
		  }
        }
        $this->Ln();
    }

    //Simple table
    function buildTableHeaderuaf($header) {
        $this->tblw = array();
        $this->tbla = array();
        //Header
        $this->SetFont('Arial', 'B', CLX_NUM_8);
        (!empty($this->hdrs['table_headers_fill_color']) ? $fc = $this->hdrs['table_headers_fill_color'] : $fc = array(CLX_NUM_255, CLX_NUM_255, CLX_NUM_255));
        is_array($fc) ? $this->SetFillColor($fc[CLX_NUM_ZERO], $fc[CLX_NUM_ONE], $fc[CLX_NUM_TWO]) : $this->SetFillColor($fc);
        $this->SetXY($this->hdrs['table_headers_uaf_x'], $this->hdrs['table_headers_uaf_y']);
        $this->Ln();
        $total = count($header);
        for ($i = CLX_INIT_ZERO; $i < $total; $i++) {
		  $this->tblw[$i] = $header[$i][CLX_NUM_ONE];
		  $this->tbla[$i] = $header[$i][CLX_NUM_TWO];
		  $this->Cell($header[$i]['w'], $header[$i][CLX_NUM_ONE], $header[$i][CLX_NUM_ZERO], CLX_NUM_ONE, $header[$i]['SL'], $header[$i][CLX_NUM_TWO], $header[$i]['f']);
        }
        $this->Ln();
    }

    //Simple table
    function buildTableData($data, $variants) {
        $this->SetFont('Arial', '', CLX_NUM_8);
        $this->SetFillColor(CLX_NUM_255, CLX_NUM_255, CLX_NUM_255);
        //Data
        //print_r($this->h);
        foreach ($data as $key => $row) {
		  //Forcing Page Break
		  if ($this->GetY() + $this->boton_margin > $this->h) {
			 $this->addPage();
		  }
		  if (empty($row)) {
			 $total = count($this->tblw);
			 for ($i = CLX_INIT_ZERO; $i < $total; $i++) {
			     $this->Cell($this->tblw[$i], CLX_NUM_FIVE, ' ', CLX_NUM_ONE, '0', $this->tbla[$i]);
			 }
		  } else {
			 $i = CLX_INIT_ZERO;
			 $total_cells = count($this->tblw);
			 foreach ($row as $key2 => $val) {
			     if (array_key_exists($key, $variants)) {
			         if (!empty($variants[$key]['font_face']))
					   $this->SetFont('Arial', $variants[$key]['font_face'], CLX_NUM_8);
			     }
			     if (empty($this->tblw[$i])) {//If more fields than cell then jump and add one cell to display
			         $this->Ln();
			         $this->Cell($this->tblw[CLX_NUM_ZERO], CLX_NUM_FIVE, ' ', CLX_NUM_ONE, '0', $this->tbla[CLX_NUM_ZERO]);
			         $i = CLX_NUM_ONE;
			         $total_cells = count($this->tblw) - CLX_NUM_ONE;
			     }
			     $this->Cell($this->tblw[$i], CLX_NUM_FIVE, $val, CLX_NUM_ONE, '0', $this->tbla[$i]);
			     $this->SetFont('Arial', '', CLX_NUM_8);
			     $total_cells--;
			     $i++;
			 }
			 if ($total_cells > CLX_NUM_ZERO) {
			     for ($j = CLX_INIT_ZERO; $j < $total_cells; $j++) {
			         $this->Cell($this->tblw[$j + $i], CLX_NUM_FIVE, ' ', CLX_NUM_ONE, '0', $this->tbla[$j + $i]);
			     }
			 }
		  }
		  $this->Ln();
        }
    }

    //Simple table
    function buildTable($header, $data, $variants = array()) {
        $w = array();
        //Header
        $this->SetFillColor(CLX_NUM_201, CLX_NUM_201, CLX_NUM_201);
        $total = count($header);
        for ($i = CLX_INIT_ZERO; $i < $total; $i++) {

		  $w[$i] = $header[$i][CLX_NUM_ONE];
		  $a[$i] = $header[$i][CLX_NUM_TWO];
		  $this->Cell($w[$i], CLX_NUM_TEN, $header[$i][CLX_NUM_ZERO], '1', '0', 'C', CLX_NUM_ONE);
        }
        $this->Ln();
        $this->SetFillColor(CLX_NUM_255, CLX_NUM_255, CLX_NUM_255);
        //Data
        foreach ($data as $key => $row) {
		  if (empty($row)) {
			 $total = count($w);
			 for ($i = CLX_INIT_ZERO; $i < $total; $i++) {
			     $this->Cell($w[$i], CLX_NUM_FIVE, ' ', CLX_NUM_ONE, '0', $a[$i]);
			 }
		  } else {
			 $i = CLX_INIT_ZERO;
			 foreach ($row as $key2 => $val) {
			     $this->Cell($w[$i], CLX_NUM_FIVE, $val, CLX_NUM_ONE, '0', $a[$i]);
			     $i++;
			 }
		  }
		  $this->Ln();
        }
    }

}
