<?php

/**
 * Object Class
 * For more information contact  support@alenet.com
 * @package		Clientix.Tool.Analytics
 * @abstract		Report Type Object
 * @author       $Author: fsantana $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2016-05-12 13:42:51 -0400 (Thu, 12 May 2016) $
 * @version        $Rev: 8219 $
 *
 * CHANGE LOG
 *  Object creation
 */
if (!class_exists('rpft') && !defined('clxInstallProcess'))
    include_once kPathToObjects . 'rpft.php';

class rpft_wordx extends rpft {

    /**
     * Name of template
     * @var string
     */
    private $template;

    /**
     * Fields to map on template
     * @var array
     */
    private $fields = array();

    /**
     * Blocks of bookmarks
     * @var string
     */
    private $blocks;
    private $images;
    public $bookmark_srcs;

    /**
     * Constructor
     */
    public function __construct() {
        $this->code = 'RPWX';
        $this->name = 'Microsoft Word (.docx)';
        $this->type = 'RPFT';
        $this->version = '5.5.0.0';
        $this->description = 'Microsoft Word Report Format';
        $this->windowurl = 'window.php';
        $this->show_prompt_format = false; //untill output is develop
        $this->bookmark_srcs = array();
    }

//end
    /**
     * Build Report Id
     * @param int $reportid
     */
    function build($reportid) {
        //loads report information
		ob_start();
        $contents = '<html><head><link href="' . skinPath() . '/styles.css" rel="stylesheet" type="text/css"></head><body>';
        $contents .= note(dict('GEN_LBL_NOTAVAILABLE') . '<br>');
        $contents .= '</body></html>';
        return $contents;
    }

    function getBlocks() {
        return $this->blocks;
    }

    function setBlocks($blocks) {
        $this->blocks = $blocks;
    }

    /**
     * Build Report By Template Id
     * @param int $reportid
     */
    function buildTemplate($reportid) {
        //Get Report Information
        $this->getReportRecord($reportid);
        if (empty($this->reportrec['TEMPLATE'])) {
		  return alert(dict('GEN_LBL_TEMPLATE') . ' ' . dict('GEN_ERR_FILE_NOT_FOUND'));
        }
        //Get Queries and assign information.
        $queries = array();
        $DBQY = &getTool('DBQY');
        $ANQC = &getTool('ANQC');
        $questions = $this->getRequestQuestions();
        //Setting columns, bookmarks and data required for report
        $queries = array();
        $sql = 'SELECT  Q.*, G.BOOKMARKNAME  FROM CLX_AN_REPORT_QUERIES Q LEFT JOIN CLX_AN_REPORT_GROUPS G ON G.REPQRYCODE = Q.IDCODE AND G.ACTIVE = 1 WHERE Q.REPORTCODE = ' . singleQuote($this->reportrec['IDCODE']) . ' ORDER BY Q.SEQNUM ASC, G.SEQNUM ASC';
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {

		  $qaccess = $DBQY->getAppAccess($r['QUERYCODE']);
		  if (!$qaccess->canRead()) {
			 return warning(dict('GEN_ERR_NO_ACCESS') . ' [RPFT_WORDX-CAN_READ]' . ' ' . $this->reportrec['IDCODE'] . ' Query:' . $r['QUERYCODE']);
		  }

		  $queries[$r['IDCODE']] = $DBQY->getquery('ANRP', $r['QUERYCODE']);
		  if (empty($queries[$r['IDCODE']])) {
			 return warning(dict('TOOL_DBQF_ERR_QUERYCODESELNOTFOUND') . ' ' . $this->reportrec['IDCODE'] . ' Query:' . $r['QUERYCODE']);
		  }
		  //set bookmark
		  $queries[$r['IDCODE']]['BOOKMARKNAME'] = $r['BOOKMARKNAME'];
		  //Get User
		  $queries[$r['IDCODE']]['USER_CRITERIA'] = $DBQY->loadAnswersFromRecord($r['QUERYCODE'], $questions, $this->reportrec['SEARCH_FORM']);
		  //Match User Questions with query
		  $queries[$r['IDCODE']]['REP_QUERY_CRITERIA'] = array();
		  //add criterias to child query
		  $sql = 'SELECT * FROM CLX_AN_REP_QRY_CRITS WHERE REPQRYCODE = ' . singleQuote($r['IDCODE']);
		  $rs2 = ClxCommon::$db->executeQuery($sql);
		  while ($r2 = ClxCommon::$db->getNextRecord($rs2, ASSOCIATIVE)) {
			 $queries[$r['IDCODE']]['REP_QUERY_CRITERIA'][] = $r2;
		  }

		  //Add fields to parent query
		  $sql = 'SELECT * FROM CLX_AN_REP_QRY_CRITS WHERE REPQRYCODE2 = ' . singleQuote($r['IDCODE']);
		  $rs2 = ClxCommon::$db->executeQuery($sql);
		  while ($r2 = ClxCommon::$db->getNextRecord($rs2, ASSOCIATIVE)) {
			 //add field to query in case of not existance
			 if (!empty($r2['REPQRYCODE2'])) {
			     if (strpos($queries[$r2['REPQRYCODE2']]['SELECTCLAUSE'], $r2['FIELDNAME2']) === false) {
			         $queries[$r2['REPQRYCODE2']]['SELECTCLAUSE'] .= ', ' . $r2['FIELDNAME2'] . ' ' . $r2['FIELDALIAS2'];
			         //adding fields to report
			         $f = explode('.', $r2['FIELDNAME2']);
			         $queries[$r2['REPQRYCODE2']]['COLUMNS'][$r2['FIELDALIAS2']] = ClxCommon::$db->getRecord('CLX_DB_FIELDS', 'FIELDNAME = ' . singleQuote($f[CLX_NUM_ONE]));
			     }
			 }
		  }
		  $sql = 'SELECT F.*, C.TABLECODE, Q.QUERYCODE, C.FORMAT, Q.SEQNUM,  IFNULL((SELECT 1 FROM CLX_DB_TABLEDFIELDS X WHERE X.TABLECODE = C.TABLECODE AND X.FIELDNAME = F.FIELDNAME LIMIT 1),0) DERIVED FROM CLX_AN_REPORT_QUERIES Q, CLX_AN_REPORT_COLUMNS C, CLX_DB_FIELDS F WHERE Q.IDCODE = C.REPQRYCODE AND F.FIELDNAME = C.FIELDNAME AND C.REPORTCODE = Q.REPORTCODE AND C.ACTIVE = 1 AND Q.IDCODE = ' . singleQuote($r['IDCODE']) . ' UNION SELECT F.*, C.TABLECODE, Q.QUERYCODE, C.FORMAT,Q.SEQNUM, IFNULL((SELECT 1 FROM CLX_DB_TABLEDFIELDS X WHERE X.TABLECODE = C.TABLECODE AND X.FIELDNAME = F.FIELDNAME LIMIT 1),0) DERIVED FROM CLX_AN_REPORT_QUERIES Q, CLX_AN_REPORT_GROUP_FUNC C, CLX_AN_REPORT_GROUPS G, CLX_DB_FIELDS F WHERE Q.IDCODE = G.REPQRYCODE AND G.ID = C.GROUPID AND G.REPORTCODE = Q.REPORTCODE AND F.FIELDNAME = C.FIELDNAME AND G.ACTIVE = 1 AND Q.IDCODE = ' . singleQuote($r['IDCODE']).' ORDER BY SEQNUM ASC';
		  $rs2 = ClxCommon::$db->executeQuery($sql);
		  while ($r2 = ClxCommon::$db->getNextRecord($rs2, ASSOCIATIVE)) {
			 $fieldname = $r2['TABLECODE'] . '.' . $r2['FIELDNAME'];
			 $fieldalias = $r2['TABLECODE'] . '_' . $r2['FIELDNAME'];
			 //add field to query in case of not existance
			 if (strpos($queries[$r['IDCODE']]['SELECTCLAUSE'], $fieldalias) === false) {
			     if (boolval($r2['DERIVED']) === true) {

			     } else {
			         $queries[$r['IDCODE']]['SELECTCLAUSE'] .= ', ' . $fieldname . ' ' . $fieldalias;
			         $queries[$r['IDCODE']]['COLUMNS'][$fieldalias] = $r2;
			     }
			 }
			 if (boolval($r2['DERIVED']) === true) {
			     $queries[$r['IDCODE']]['COLUMNS'][$fieldalias . '_WRK']['FORMAT'] = $r2['FORMAT'];
			 } else {
			     $queries[$r['IDCODE']]['COLUMNS'][$fieldalias]['FORMAT'] = $r2['FORMAT'];
			 }
			 //Init value on template so it will exists when query return empty.
			 //leaveing blank space
			 $this->assign($r['IDCODE'] . '.' . $fieldalias, '');
		  }
        }//end while
        //Performing column query and assigment
        $saved_result = array(); //used for child queries to add criteria
        foreach ($queries as $repquerycode => $qry) {
		  $saved_result[$repquerycode] = array();
		  //Attach search form fields from users if table exists
		  foreach ($qry['USER_CRITERIA'] as $key => $val) {
			 if (strlen($qry['WHERECLAUSE']) > CLX_NUM_ZERO) {
			     $qry['WHERECLAUSE'] .=' AND ';
			 }
			 $qry['WHERECLAUSE'] .= $key . ' = ' . singleQuote($val) . ' ';
		  }
		  //Attach Criteria if exists
		  foreach ($qry['REP_QUERY_CRITERIA'] as $key => $val) {
			 if (empty($val['REPQRYCODE2'])) {
			     $v = $val['VALUE'];
			     $crit = $ANQC->buildOperator($val['OPERATOR'], $val['FIELDNAME1'], $v);
			 } elseif (!empty($val['REPQRYCODE2'])) {
			     if (!array_key_exists($val['REPQRYCODE2'], $saved_result)) {
			         return alert(dict('GEN_LBL_ERROR_REP_QUERY_CHG_SEQ', $repquerycode, $val['REPQRYCODE2']));
			     }
			     if (array_key_exists($val['FIELDALIAS2'], $saved_result[$val['REPQRYCODE2']])) {
			         $v = $saved_result[$val['REPQRYCODE2']][$val['FIELDALIAS2']];
			         $crit = $ANQC->buildOperator($val['OPERATOR'], $val['FIELDNAME1'], $v);
			     } else {
			         $crit = $val['FIELDNAME1'] . '=' . singleQuote(CLX_CHAR_EMPTY_STRING);
			     }
			 }
			 if (strlen($qry['WHERECLAUSE']) > CLX_NUM_ZERO) {
			     $qry['WHERECLAUSE'] .=' AND ';
			 }
			 $qry['WHERECLAUSE'] .= $crit;
		  }

		  $sql = 'SELECT ' . $qry['SELECTCLAUSE'] . ' FROM ' . $qry['FROMCLAUSE'];
		  if (strlen($qry['WHERECLAUSE']) > CLX_NUM_ZERO) {
			 $sql .=' WHERE ' . $qry['WHERECLAUSE'];
		  }
		  if (strlen($qry['GROUPCLAUSE']) > CLX_NUM_ZERO) {
			 $sql .=' GROUP BY ' . $qry['GROUPCLAUSE'];
		  }
		  if (strlen($qry['SORTCLAUSE']) > CLX_NUM_ZERO) {
			 $sql .=' ORDER BY ' . $qry['SORTCLAUSE'];
		  }

		  //Get the fields and assign them
		  $rs = ClxCommon::$db->executeQuery($sql);

		  $i = CLX_INIT_ZERO;
		  while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
			 //run formulas
			 $this->runRecordEvents($reportid, $r);
			 $saved_result[$repquerycode] = $r;
			 foreach ($r as $key => $val) {
			     $tmplflname = $repquerycode . '.' . $key;
			     $fieldrec = $qry['COLUMNS'][$key];
			     if (!empty($fieldrec['WORKFIELD'])) {
			         $tmplflname = $repquerycode . '.' . $fieldrec['TABLECODE'] . '_' . $fieldrec['WORKFIELD_NAME'];
			     } else {
			         $tmplflname = $repquerycode . '.' . $key;
			     }

			     if (!array_key_exists('FORMAT', $fieldrec)) {
			         $fieldrec['FORMAT'] = CLX_INIT_ZERO;
			     }
			     if ($qry['BOOKMARKNAME'] <> '') {
			         if ($fieldrec['FIELDTYPE'] === 'FILE') {
					   $this->assignBookmark($qry['BOOKMARKNAME'], $i, 'image:' . $tmplflname, $this->fieldFormat($key, $r, $fieldrec, false));
			         } else {
					   $this->assignBookmark($qry['BOOKMARKNAME'], $i, $tmplflname, $this->fieldFormat($key, $r, $fieldrec, false));
			         }
			     } else {
			         if ($fieldrec['FIELDTYPE'] === 'FILE') {
					   $this->assign('image:' . $tmplflname, $this->fieldFormat($key, $r, $fieldrec, false));
			         } else {
					   $this->assign($tmplflname, $this->fieldFormat($key, $r, $fieldrec, false));
			         }
			     }
			 }
			 $i++;
		  }
        }

        unset($r, $queries, $sql, $repquerycode, $qry, $saved_result);
        if (!class_exists('clx_zip')) {
		  include_once kPathToWebSite . 'classes/clx_zip.php';
        }

        $filenm = 'tmp_' . batch_id();
        $fileext = 'docx';
        $filename = $filenm . '.' . $fileext;
        $this->template = kPathToTemp . $filename;
        //Creando archivo temporal con reporte desde formato en db
        $file = new clx_file($this->template, 'w');
        $file->open();
        $file->write($this->reportrec['TEMPLATE']);
        $file->close();
        //Guardando information
        $this->save($this->template); //Esto en realidad llama a $this->parseTemplate($filename);
        //Leyendo archivo para descargar todo
        $filedata = file_get_contents($this->template);
        //Descargando archivo
        sendDownloadFile($filenm, $fileext);
        return $filedata;
    }

    /**
     * Formats field according type
     * @param string $fieldname
     * @param array $record
     * @param array $fieldrec
     * @param bool $append_quote=false
     */
    function fieldFormat($fieldname, &$record, &$fieldrec, $append_quote = false) {
        if ($fieldrec['FIELDTYPE'] === 'FILE') {
		  $imgrec = ClxCommon::$db->getRecord('CLX_SD_FILES', 'ID=' . $record[$fieldname]);
		  if (!empty($imgrec['FILENAME'])) {
			 $val = $imgrec['FILENAME'];
		  } elseif (!empty($imgrec['CONTENTS'])) {
			 $val = ''; //set inside
		  } else {
			 $val = '';
		  }
        } else {
		  $val = parent::fieldFormat($fieldname, $record, $fieldrec, $append_quote);
		  $val = html_entity_decode (strip_tags($val));
        }
        return $val;
    }

    // Template Type Methods
    /**
     * Parses xml document
     */
    private function parseTemplate($outputfile) {
        //Copy template to a temporary file
        //check if file exists or throw exception
        clx_file::copyStatic($this->template, $outputfile);
        $zip = new clx_zip();
        if ($zip->open($outputfile) !== true) {
		  throw new Exception('Template file not found');
        }
        //Load images and Parse template
        //headers if exists

        $count = array(CLX_NUM_ONE, CLX_NUM_TWO, CLX_NUM_THREE, CLX_NUM_FOUR, CLX_NUM_FIVE);
        foreach ($count as $val) {
		  if ($zip->locateName('word/header' . $val . '.xml')) {
			 $this->processImages($zip, 'word/_rels/header' . $val . '.xml.rels');
			 $this->loadXml($zip, 'word/header' . $val . '.xml');
		  }
        }

        //Document
        $this->processImages($zip, 'word/_rels/document.xml.rels');
        $this->loadXml($zip, 'word/document.xml');


        //footer if exists
        foreach ($count as $val) {
		  if ($zip->locateName('word/footer' . $val . '.xml')) {
			 $this->processImages($zip, 'word/_rels/footer' . $val . '.xml.rels');
			 $this->loadXml($zip, 'word/footer1.xml');
		  }
        }

        $zip->close();
    }

    private function readBookMarkNode(&$nodes) {
        foreach ($nodes as $node) {
		  if ($node->nodeName !== 'w:bookmarkEnd') {
			 if ($node->nodeName === 'w:bookmarkStart') {
			     $bookmark_name = $node->getAttribute('w:name');

			     if ($bookmark_name === '_GoBack')
			         continue;
			     //Copy section for future reference
			     $this->bookmark_srcs[$bookmark_name] = $node->parentNode->nextSibling->cloneNode(true);
			     return;
			 }
			 //Solo continuar bajando si no se encuentra la seccion
			 if ($node->hasChildNodes()) {
			     $this->readBookMarkNode($node->childNodes);
			 }
		  }
        }
    }

    /**
     * Process file on template output
     * @param $zip
     * @param $zipfile
     */
    private function loadXml(&$zip, $zipfile) {
        //Read the file on zip and parse variables
        $dom = new DOMDocument();
        $xml = $zip->getFromName($zipfile);
        if (empty($xml))
		  return;
        $dom->loadXML($xml);
        if ($dom->documentElement->hasChildNodes()) {
		  $this->readNode($dom->documentElement->childNodes, $this->fields);
        }
        //save back to zip
        $zip->addFromString($zipfile, $dom->saveXML());
    }

    /**
     * Reads xml nodes and replace values
     * @param obj $nodes
     * @param array $recorddata
     */
    private function readNode(&$nodes, $recorddata = array()) {
        if (empty($recorddata)) {
		  $recorddata = array();
        }

        $bookmark = '';
        foreach ($nodes as $node) {

		  if ($node->nodeName == 'w:bookmarkStart') {
			 $bookmark = $node->getAttribute('w:name');
			 if ($bookmark == '_GoBack') {
			     $bookmark = '';
			     continue;
			 }
			 //Get next Item and copy node
			 if (!isset($node->parentNode->nextSibling))
			     continue;
			 $nodesrc = $node->parentNode->nextSibling->cloneNode(true);

			 //Take out original
			 $node->parentNode->parentNode->removeChild($node->parentNode->nextSibling);
			 //Por cada resultado colocar el datos
			 if (!empty($this->fields[$bookmark])) {
			     foreach ($this->fields[$bookmark] as $k => $val) {

			         //copy repeat section and added to the word file
			         $newrow = clone($nodesrc);
			         $this->readNode($newrow->childNodes, $val);
			         $node->parentNode->parentNode->appendChild($newrow);
			     }
			 }

			 continue;
		  }





		  if ($node->hasChildNodes()) {
			 $this->readNode($node->childNodes, $recorddata);
		  } else {
			 $key = trim($node->nodeValue);
			 if (strpos($key, '«') === false) {
			     continue;
			 }

			 $srch_key = str_replace(array(chr(195), '«', '»'), '', $key);
			 if (array_key_exists($srch_key, $recorddata)) {
			     $val = $recorddata[$srch_key];
			     if ($this->isImage($key)) {
			         //build image xml, Import node, and replace the node to display the image.
			         if (empty($val)) {
					   $node->nodeValue = $val; //no image
			         } else {
					   $img = $this->imageElement($val);
					   $imgnode = $node->ownerDocument->importNode($img->firstChild->firstChild->firstChild, true);
					   $node->parentNode->parentNode->parentNode->replaceChild($imgnode, $node->parentNode->parentNode);
			         }
			     } else {
			         $node->nodeValue = $val;
			     }
			 } else {
			     $node->nodeValue = '';
			 }
		  }
        }
    }

    /**
     * Loads images into archive document
     * @param obj $zip
     * @param string $zipfile Xml document
     */
    private function processImages(&$zip, $zipfile) {
		
		if (!is_array($this->images) || count($this->images) == CLX_NUM_ZERO)
		  return; //no field to upload
        if (!$zip->addEmptyDir('word/media')) {

        }
        //If not exists the relation file create it
        $st = $zip->getFromName($zipfile);
        if ($st == '')
		  return;
        $dom = new DOMDocument();
        $dom->loadXML($st);
        $rid = $dom->documentElement->childNodes->length + CLX_NUM_ONE;
        foreach ($this->images as $file => $ref) {
		  if (empty($file))
			 continue;
		  $basefile = basename($file);
		  //set reference id for future use
		  $this->images[$file] = $rid;
		  $localname = 'media/' . $basefile;
		  //add to file to archive
		  $zip->addFile($file, 'word/' . $localname);
		  //add reference
		  $element = $dom->createElement('Relationship');
		  $element->setAttribute('Id', 'rId' . $rid);
		  $element->setAttribute('Target', $localname);
		  $element->setAttribute('Type', 'http://schemas.openxmlformats.org/officeDocument/2006/relationships/image');
		  $dom->documentElement->appendChild($element);
		  $rid++;
        }
        //save back to zip
        $st = $dom->saveXML();
        if (!empty($st))
		  $zip->addFromString($zipfile, $st);
        //Adding content type
        $st = $zip->getFromName('[Content_Types].xml');
        if ($st == '')
		  return;
        $done_types = array();
        $dom = new DOMDocument();
        $dom->loadXML($zip->getFromName('[Content_Types].xml'));
        foreach ($dom->getElementsByTagName('Default') as $node) {
		  $done_types[$node->getAttribute('Extension')] = true;
        }
        //set content type
        $change_flag = false;
        foreach ($this->images as $file => $ref) {
		  if (!empty($file)) {
			 $fl = new clx_file($file, 'r');
			 $file_mime = $fl->getMimeType();
			 $file_ext = $fl->getFileExtension();
			 unset($fl);
			 //check if not done already
			 if ($file_ext === '' || array_key_exists($file_ext, $done_types)) {
			     continue;
			 }
			 //add reference
			 $element = $dom->createElement('Default');
			 $element->setAttribute('ContentType', $file_mime);
			 $element->setAttribute('Extension', $file_ext);
			 $dom->documentElement->insertBefore($element, $dom->documentElement->firstChild);
			 $change_flag = true;
		  }
        }
        //save back to zip
        if (!empty($change_flag)) {
		  $zip->addFromString('[Content_Types].xml', $dom->saveXML());
        }
    }

    /**
     * Builds image element xml
     * @param string $file
     */
    private function imageElement($file) {
        if (empty($file)) {
		  return false;
        }
        $rid = $this->images[$file];
        $imginfo = getimagesize($file);
        $w = $imginfo[CLX_NUM_ZERO] * CLX_NUM_9525; //1px = 9525 emu
        $h = $imginfo[CLX_NUM_ONE] * CLX_NUM_9525; //1px = 9525 emu
        $imagename = 'Picture ' . $rid;
        $xml = '<?xml version="1.0" encoding="utf-8" standalone="yes"?><w:document ' .
			 'xmlns:r="http://schemas.openxmlformats.org/officeDocument/2006/relationships" ' .
			 'xmlns:wp="http://schemas.openxmlformats.org/drawingml/2006/wordprocessingDrawing" ' .
			 'xmlns:w="http://schemas.openxmlformats.org/wordprocessingml/2006/main">' .
			 '<w:p><w:r><w:rPr><w:noProof/></w:rPr><w:drawing><wp:inline>' .
			 '<wp:extent cx="' . $w . '" cy="' . $h . '"/>' .
			 '<wp:docPr id="1" name="' . $imagename . '"/>' .
			 '<wp:cNvGraphicFramePr>' .
			 '<a:graphicFrameLocks xmlns:a="http://schemas.openxmlformats.org/drawingml/2006/main" noChangeAspect="1"/>' .
			 '</wp:cNvGraphicFramePr><a:graphic xmlns:a="http://schemas.openxmlformats.org/drawingml/2006/main">' .
			 '<a:graphicData uri="http://schemas.openxmlformats.org/drawingml/2006/picture">' .
			 '<pic:pic xmlns:pic="http://schemas.openxmlformats.org/drawingml/2006/picture">' .
			 '<pic:nvPicPr><pic:cNvPr id="0" name="' . $imagename . '"/>' .
			 '<pic:cNvPicPr/></pic:nvPicPr><pic:blipFill><a:blip r:embed="rId' . $rid . '">' .
			 '<a:extLst><a:ext uri="{28A0092B-C50C-407E-A947-70E740481C1C}"></a:ext></a:extLst></a:blip>' .
			 '<a:stretch><a:fillRect/></a:stretch>' .
			 '</pic:blipFill><pic:spPr>' .
			 '<a:xfrm><a:off x="0" y="0"/>' .
			 '<a:ext cx="' . $w . '" cy="' . $h . '"/>' .
			 '</a:xfrm>' .
			 '<a:prstGeom prst="rect"><a:avLst/></a:prstGeom>' .
			 '</pic:spPr></pic:pic></a:graphicData></a:graphic></wp:inline></w:drawing></w:r></w:p></w:document>';
        $dom = new DOMDocument();
        $dom->loadXML($xml);
        return $dom;
    }

    /**
     * Returns true if assigned element is an image
     * @param string $field
     * @return bool
     */
    private function isImage($field) {
        return (stripos($field, 'image:') !== false) ? true : false;
    }

    /**
     * Assignes values required on template
     * @param string $field
     * @param mixed $value
     */
    private function assign($field, $value) {
        if ($this->isImage($field))
		  $this->images[$value] = CLX_INIT_ZERO;
        $value = encode_string($value);
        $value = strip_tags($value); //no html
        $value = str_replace(CLX_CHAR_HTML_NBSP, '', $value);
        if (is_array($value)) {
		  $this->fields[$field][] = $value;
        } else {
		  $this->fields[$field] = $value;
        }
    }

    /**
     * Assignes values required on template
     * @param $bookmark
     * @param $index
     * @param $field
     * @param $value
     */
    function assignBookmark($bookmark, $index, $field, $value) {
        if ($this->isImage($field) === true)
		  $this->images[$value] = CLX_INIT_ZERO;
        $value = encode_string($value);
        $value = strip_tags($value); //no html
        $value = str_replace(CLX_CHAR_HTML_NBSP, '', $value);
        $value = str_replace('?', '', $value); //no especial caracters
        $this->fields[$bookmark][$index][$field] = $value;
    }

    /**
     * Saves the processed template
     * @param string $filename
     */
    private function save($filename) {
        //Read the file and parse variables
        $this->parseTemplate($filename);
    }

}
