<?php

/**
 * Object Class
 * For more information contact  support@alenet.com
 *
 * @package		Clientix.Tool.Interface
 * @abstract		System Events Manager - System Alert Message.
 * @author       $Author: jclerma $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2017-11-03 15:09:40 -0400 (Fri, 03 Nov 2017) $
 * @version        $Rev: 9615 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  Included label
 */
if (!class_exists('uimi') && !defined('clxInstallProcess'))
    include_once kPathToObjects . 'uimi.php';

class uimi_tool extends uimi {

    /**
     * Contructor
     */
    function __construct() {
        $this->code = 'TOOL';
        $this->name = 'Tool';
        $this->type = 'UIMI';
        $this->version = '0.1.0.0';
        $this->description = 'This is a Clientix Menu Item that invokes a Clientix Tool';
    }

    /**
     * Build Meny item type on edit menu.
     * @param array $msg
     * @param array $record
     */
    function panel($msg, $record) {
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => '90%'));
        $table->addCell(dict('OBJ_MITM_TOOL_TITLE'), array(CLX_PROP_CLASS => CLX_CLASS_HEADERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $MAGR = &getTool('MAGR');
        $tools = $MAGR->toolPop('TOOL_extra1', (array_key_exists('TOOL_extra1', $record)) ? $record['TOOL_extra1'] : '');
        $table->addData(dict('GEN_LBL_TOOL') . ':', $tools->source());
        $table->addField($msg, dict('GEN_LBL_PARAMETERS') . ':', 'TOOL_extra2', (array_key_exists('TOOL_extra2', $record)) ? $record['TOOL_extra2'] : '', CLX_NUM_48, CLX_NUM_255);
        $div = new htmlDiv($this->code, 'visibility:hidden; display:none', $table->source());
        return CLX_CHAR_NL . CLX_CHAR_NL . $div->source() . CLX_CHAR_NL . CLX_CHAR_NL;
    }

    /**
     * Verifies Menu Item Panel.
     * @param array $msg
     */
    function checkPanel(&$msg) {

    }

    /**
     * Loads form values in record for future save.
     * @param array $record
     */
    function getPanel(&$record) {
        $record['EXTRA1'] = UserRequest::$post['TOOL_extra1'];
        $record['EXTRA2'] = UserRequest::$post['TOOL_extra2'];
    }

    /**
     * Sets form values from menu record
     * @param array $record
     * @param array $values
     */
    function setPanel(&$record, $values) {
        $record['TOOL_extra1'] = $values['EXTRA1'];
        $record['TOOL_extra2'] = $values['EXTRA2'];
    }

    /**
     * Loads Default Panel Values
     * @param array $record
     */
    function defaultPanel(&$record) {
        $record['TOOL_extra1'] = '';
        $record['TOOL_extra2'] = '';
    }

    function viewForm(&$table, $record) {
        $table->addData(dict('GEN_LBL_TOOL') . ':', $record['TOOL_extra1']);
        $table->addData(dict('GEN_LBL_PARAMETERS') . ':', $record['TOOL_extra2']);
    }

    /**
     * Return string with description of menu item.
     * @param array $record
     * @return string
     */
    function describe($record) {
        return 'Execute tool: ' . $record['EXTRA1'] . ' (' . $record['EXTRA2'] . ')';
    }

    /**
     * Returns URL Form Menu Item
     * @param array $record
     * @return string
     */
    function makeURL($record) {
        return sprintf('tool.php?to=%s&%s&menuid=%s', $record['EXTRA1'], $record['EXTRA2'], $record['IDCODE']);
    }

    /**
     * Saves Menu access information
     * @param string $ownercode
     * @param string $ownerref
     */
    function saveAccessWizard($ownercode, $ownerref) {
        $ACTO = getTool('ACTO');
        $user_tools = "'USPR','TASK','USFI','MAIL','MAIL','ANRJ','MAAT','MAAC'";
        $user_tools .=",'PRSC','PRLO','PRSP','PREL','PRSL','UIVW','UIVS','UIVP'";
        $user_tools .=",'UIVC','UIFT','UIFU','UIFC'";
        //Adding All User Interfase at least to read. User will needed or no access
        //due to block may happend
        $sql = 'SELECT DISTINCT MAGR.IDCODE TOOLCODE, 1 CANREAD, IFNULL(MAX(UIMI.CANMODIFY),0) CANMODIFY, IFNULL(MAX(UIMI.CANDELETE),0) CANDELETE, IFNULL(MAX(UIMI.CANAPPEND),0) CANAPPEND, IFNULL(MAX(UIMI.ISADMIN),0) ISADMIN FROM CLX_TOOLS MAGR LEFT JOIN CLX_AC_MENUITEMS UIMI ON MAGR.IDCODE = UIMI.TOOLCODE AND UIMI.OWNERCODE = ' . singleQuote($ownercode) . ' AND UIMI.OWNERREF = ' . singleQuote($ownerref) . ' WHERE MAGR.IDCODE NOT IN (' . $user_tools . ' ) GROUP BY MAGR.IDCODE';
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
		  $ACTO->saveMenuWizard($ownercode, $ownerref, $r['TOOLCODE'], $r['CANREAD'], $r['CANMODIFY'], $r['CANDELETE'], $r['CANAPPEND'], $r['ISADMIN']);
        }
		ClxCommon::$db->freeResult($rs);
        //Setting user default tools
        $sql = 'SELECT DISTINCT MAGR.IDCODE TOOLCODE, 1 CANREAD, 1 CANMODIFY, 1 CANDELETE, 1 CANAPPEND, IFNULL(MAX(UIMI.ISADMIN),0) ISADMIN FROM  CLX_TOOLS MAGR LEFT JOIN CLX_AC_MENUITEMS UIMI ON MAGR.IDCODE = UIMI.TOOLCODE AND UIMI.OWNERCODE = ' . singleQuote($ownercode) . ' AND UIMI.OWNERREF = ' . singleQuote($ownerref) . ' WHERE MAGR.IDCODE IN (' . $user_tools . ') GROUP BY MAGR.IDCODE';
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
		  $ACTO->saveMenuWizard($ownercode, $ownerref, $r['TOOLCODE'], $r['CANREAD'], $r['CANMODIFY'], $r['CANDELETE'], $r['CANAPPEND'], $r['ISADMIN']);
        }
		ClxCommon::$db->freeResult($rs);
    }

}
