<?php

/**
 * Tool Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: alenetdev $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2019-01-22 15:24:38 -0400 (ma., 22 ene. 2019) $
 * @version        $Rev: 10491 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  Included label
 */
class ac_tables extends clx_tool {

    // CONSTRUCTOR //
    function __construct() {
        $this->code = 'ACTA';
        $this->name = 'Table Access Manager';
        $this->system = 'Security';
        $this->version = '0.1.1.2';

        $this->has_access = true;


        $this->tablename = 'CLX_AC_TABLES';
        $this->record_name_singular = 'table access';
        $this->record_name_plural = 'table accesses';
        $this->primary_field = 'OWNERCODE';
        $this->secondary_field = 'OWNERREF';
    }

    function install() {
        $UIMI = getTool('UIMI');
        $UIMI->add($this->code, 'MENU_TOOLS:MENU_SECURITY:MENU_TABLE_ACCESS', 'TOOL', $this->code, 'ac=LIST');
        $UIMI->add($this->code, 'MENU_TOOLS:MENU_SECURITY:MENU_ACCESS_REPORT', 'TOOL', $this->code, 'ac=ACREPORT');
    }

    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['OWNERCODE'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '');
        $fields['OWNERREF'] = array('type' => 'varchar(30)', 'null' => 'NO', 'properties' => '');
        //$fields['TABLECODE'] = array('type' => 'varchar(16)', 'null' => 'NO', 'properties' => '');
        $fields['TABLECODE'] = array('type' => 'varchar(16)', 'null' => 'YES', 'properties' => '');
        $fields['ISADMIN'] = array('type' => 'tinyint(1)', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['CANREAD'] = array('type' => 'tinyint(1)', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['CANMODIFY'] = array('type' => 'tinyint(1)', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['CANDELETE'] = array('type' => 'tinyint(1)', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['CANAPPEND'] = array('type' => 'tinyint(1)', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'UNI', 'fieldname' => 'OWNERCODE,OWNERREF,TABLECODE');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB', 'fields' => $fields, 'indexes' => $indexes);
    }

    function action($ac, $idlist) {
        $contents = '';
        $msg = array();
        if ($this->getAccess()->canRead()) {
		  switch ($ac) {
			 case 'ACREPORT':

			     if (!empty(UserRequest::$post['confirm']) && $this->checkRepForm($msg)) {
			         $contents = $this->repAccess(UserRequest::$post['ACAD_usergroup'], UserRequest::$post['ACAD_username'], UserRequest::$post['ACTA_format']);
			     } else {
			         $contents = $this->repForm(UserRequest::$post, $msg);
			     }

			     break;
			 case 'WIZARD':
			     if (($this->getAccess()->canDisplay()) || $this->getAccess()->canModify()) {
			         $contents = $this->controlWizard();
			     } else {
			         $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [AC_TABLES-CAN_APPEND & CAN_MODIFY 2][' . $ac . '] ' . $this->getCode(false));
			     }
			     break;
			 case 'MODIFY':
			     if ($this->getAccess()->canModify()) {
			         $id = array_pop($idlist);
			         $record = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $id);
			         UserRequest::$post['ACTA_ownercode'] = $record['OWNERCODE'];
			         UserRequest::$post['ACTA_ownerref'] = $record['OWNERREF'];
			         UserRequest::$post['ACTA_toolcode'] = (!empty($record['TOOLCODE'])) ? $record['TOOLCODE'] : ''; //Verify TOOLCODE?
			         UserRequest::$post['step'] = CLX_NUM_THREE;
			         $contents = $this->editForm3(UserRequest::$post, $msg);
			     } else {
			         $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [AC_TABLES-CAN_MODIFY][' . $ac . '] ' . $this->getCode(false));
			     }
			     break;
			 default:
			     $contents = parent::action($ac, $idlist);
			     break;
		  }
        } else {
		  $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [AC_TABLES-CAN_READ][' . $ac . '] ' . $this->getCode(false));
        }
        return $contents;
    }

    function makelist($listingcode = null) {
        //$t = time('U'); genera un warning
        $t = time();
        $to = $this->code;
        $here = 'tool.php?to=' . $to . '&t=' . $t;
        clearHistory();
        pushHistory($this->code, 'LIST', array());
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->setProp(CLX_PROP_ID, 'ACTAList');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'NOT_DEFINED');
        $selectclause = 'ACTA.*,
						DBTB.IDNAME TABLENAME, DBMO.IDNAME MODULENAME, DBMO.IDCODE MODULECODE';
        $fromclause = 'CLX_AC_TABLES ACTA, CLX_DB_TABLES DBTB, CLX_DB_MODULES DBMO';
        $whereclause = 'ACTA.TABLECODE = DBTB.IDCODE AND DBTB.MODULECODE = DBMO.IDCODE';
        $sortclause = '';
        $page = new xPaging();
        $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
        $table = new htmlTable(CLX_WIDTH_FIVE);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_ACTA_TITLE_LIST'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_FIVE));
        $page->header($table);
        $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_NAME'), dict('GEN_LBL_MODULE'), dict('GEN_LBL_TABLE'), dict('GEN_LBL_PRIVILEGES')));
        $i = CLX_INIT_ZERO;
        while ($record = $page->nextRecord()) {
		  $recordid = $record['ID'];
		  $select = new htmlCheckbox('selector_' . $recordid, false);
		  $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
		  $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
		  $access = $this->accessDetail($record);
		  $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($record['OWNERCODE'] . ':' . $record['OWNERREF'], $record['MODULENAME'], $record['TABLENAME'], $access));
		  $i++;
        }
        $buttons = '';
        if ($this->getAccess()->canDelete()) {
		  $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        if ($this->getAccess()->canModify()) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        if ($this->getAccess()->canDisplay()) {
		  $b = new htmlFormSubmit(dict('TOOL_ACTA_BUTTON_CREATEWIZ'), 'WIZARD', $form->getProp(CLX_PROP_ID));
		  $b->setProp(CLX_PROP_ID, 'ON');
		  $buttons .= $b->source();
        }
        $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_FILTER'), array('action' => 'FILTER', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
        $form->addContents($page->footers($table, $buttons));
        return $form->source();
    }

    private function accessDetail($record) {
        $access = '';
        if (!empty($record['ISADMIN'])) {
		  $access = 'Administrator';
        } else {
		  if (!empty($record['CANREAD'])) {
			 if ($access === '') {
			     $access .= 'Can read';
			 } else {
			     $access .= ', read';
			 }
		  }
		  if (!empty($record['CANMODIFY'])) {
			 if ($access === '') {
			     $access .= 'Can modify';
			 } else {
			     $access .= ', modify';
			 }
		  }
		  if (!empty($record['CANDELETE'])) {
			 if ($access === '') {
			     $access .= 'Can delete';
			 } else {
			     $access .= ', delete';
			 }
		  }
		  if (!empty($record['APPEND'])) {
			 if ($access === '') {
			     $access .= 'Can append';
			 } else {
			     $access .= ', append';
			 }
		  }
		  if ($access === '') {
			 $access = 'No access privileges.';
		  } else {
			 $access .= '.';
		  }
        }
        return $access;
    }

    function controlWizard() {
        $msg = array();
        $contents = '';
        $step = (isset(UserRequest::$post['step'])) ? UserRequest::$post['step'] : '0';
        switch ($step) {
		  case '0':
			 UserRequest::$post = $this->defaultRecord();
			 $contents = $this->editForm1(UserRequest::$post, $msg);
			 break;
		  case '1':
			 if ($this->checkForm1($msg)) {
			     $contents = $this->editForm2(UserRequest::$post, $msg);
			 } else {
			     $contents = $this->editForm1(UserRequest::$post, $msg);
			 }
			 break;
		  case '2':
			 if ($this->checkForm2($msg)) {
			     $contents = $this->editForm3(UserRequest::$post, $msg);
			 } else {
			     $contents = $this->editForm2(UserRequest::$post, $msg);
			 }
			 break;
		  case '3':
			 if ($this->checkForm3($msg)) {
			     $this->saveWizard();
			     $info = popHistory();
			     $contents = executeHistory($info);
			 } else {
			     $contents = $this->editForm3(UserRequest::$post, $msg);
			 }
			 break;
		  default:
			 break;
        }
        return $contents;
    }

    function editForm1($record, $msg) {
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'WIZARD');
        $form->addArgument('step', '1');
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_ACTA_TITLE_CREATEWIZ'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addCell(dict('TOOL_ACTA_TITLE_CREATEWIZ1'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $codes = new HTMLSelect('ACTA_ownercode');
        $codes->addItem(dict('GEN_LBL_USER'), 'USER', ($record['ACTA_ownercode'] === 'USER'));
        $codes->addItem(dict('GEN_LBL_USERGROUP'), 'GRUP', ($record['ACTA_ownercode'] === 'GRUP'));
        $table->addData(dict('GEN_LBL_GIVE_ACCESS_TO') . ':', $codes->source());
        $form->addContents($table->source());
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $buttons =htmlButton::getButton('submit', dict('GEN_LBL_CONTINUE'), array('action' => 'WIZARD', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        $c = new htmlButtonCancel();
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->addContents($table->source());
        return $form->source();
    }

    function checkForm1(&$msg) {
        $msg = array();
        return count($msg) == CLX_NUM_ZERO;
    }

    function editForm2($record, $msg) {
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'WIZARD');
        $form->addArgument('step', '2');
        $form->addArgument('ACTA_ownercode', $record['ACTA_ownercode']);
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_ACTA_TITLE_CREATEWIZ'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        if ($record['ACTA_ownercode'] === 'USER') {
		  $table->addCell(dict('TOOL_ACTA_TITLE_CREATEWIZ2U'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
		  $ACUS = &getTool('ACUS');
		  $ACUS->placeChooseUser($msg, dict('GEN_LBL_USERNAME') . ':', 'ACTA_ownerref', (array_key_exists('ACTA_ownerref', $record)) ? $record['ACTA_ownerref'] : '', $table);
        } elseif ($record['ACTA_ownercode'] === 'GRUP') {
		  $table->addCell(dict('TOOL_ACTA_TITLE_CREATEWIZ2G'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
		  $ACGR = &getTool('ACGR');
		  $ACGR->placeChooseGroup($msg, dict('GEN_LBL_USERGROUP') . ':', 'ACTA_ownerref', (array_key_exists('ACTA_ownerref', $record)) ? $record['ACTA_ownerref'] : '', $table);
        }
        $form->addContents($table->source());
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $buttons =htmlButton::getButton('submit', dict('GEN_LBL_CONTINUE'), array('action' => 'WIZARD', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        $c = new htmlButtonCancel();
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->addContents($table->source());
        return $form->source();
    }

    function checkForm2(&$msg) {
        $msg = array();
        if (UserRequest::$post['ACTA_ownercode'] === 'USER') {
		  $ACUS = &getTool('ACUS');
		  if (!$ACUS->userExists(UserRequest::$post['ACTA_ownerref'])) {
			 $msg['ACTA_ownerref'] = 'There is no user with this code.';
		  }
        } elseif (UserRequest::$post['ACTA_ownercode'] === 'GRUP') {
		  $ACGR = &getTool('ACGR');
		  if (!$ACGR->groupExists(UserRequest::$post['ACTA_ownerref'])) {
			 $msg['ACTA_ownerref'] = 'There is no group with this code.';
		  }
        }
        return count($msg) == CLX_NUM_ZERO;
    }

    function editForm3($record, $msg) {
        $variables = array();
        $script = setJavaScriptTag(getJavaScript('ACTO_editform3.js', $variables));
        // the Form --------------------------------------
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'WIZARD');
        $form->addArgument('step', '3');
        $form->addArgument('ACTA_ownercode', $record['ACTA_ownercode']);
        $form->addArgument('ACTA_ownerref', $record['ACTA_ownerref']);
        // header info
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_ACTA_TITLE_CREATEWIZ'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addCell(dict('TOOL_ACTA_TITLE_CREATEWIZ3'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        if ($record['ACTA_ownercode'] === 'USER') {
		  $table->addData(dict('GEN_LBL_USERNAME') . ':', $record['ACTA_ownerref']);
        } elseif ($record['ACTA_ownercode'] === 'GRUP') {
		  $table->addData(dict('GEN_LBL_USERGROUP') . ':', $record['ACTA_ownerref']);
        }
        $form->addContents($table->source());
        // the records --------------------------------------
        $table = new htmlTable(CLX_WIDTH_SIX);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_TABLE'), dict('GEN_LBL_CANREAD'), dict('GEN_LBL_CANMODIFY'), dict('GEN_LBL_CANDELETE'), dict('GEN_LBL_CANAPPEND'), dict('GEN_LBL_ISADMIN')));
        //   - Adds Table and Module codes to improve support activities
        $tablerecs = ClxCommon::$db->executeQuery('SELECT t.*, m.IDNAME MODULENAME, m.IDCODE MODULECODE FROM CLX_DB_TABLES t, CLX_DB_MODULES m WHERE t.MODULECODE=m.IDCODE ORDER BY m.IDNAME');
        $lastmodule = '';
        $i = CLX_INIT_ZERO;
        while (($tablerec = ClxCommon::$db->getNextRecord($tablerecs))) {
		  if ($lastmodule != $tablerec['MODULENAME']) {
			 $lastmodule = $tablerec['MODULENAME'];
			 $table->addCell($lastmodule . ' [' . $tablerec['MODULECODE'] . ']', array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_SIX));
		  }
		  $this->fillAccessForm($record['ACTA_ownercode'], $record['ACTA_ownerref'], $tablerec['IDCODE'], $tablerec['IDNAME'], ($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, $table, $form);
		  $i++;
        }
		ClxCommon::$db->freeResult($tablerecs);
        // buttons
        $buttons = '';
        $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_SAVE'), array('action' => 'WIZARD', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        $c = new htmlButtonCancel();
        $buttons .= $c->source();
        // Masive selectors
        $canread = new htmlCheckBox('Masive_All_canread', false);
        $canread->setProp('onclick', 'checkAll(' . singleQuote($form->getProp(CLX_PROP_ID)) . ', ' . singleQuote('canread') . ');');
        $canmodify = new htmlCheckBox('Masive_All_canmodify', false);
        $canmodify->setProp('onclick', 'checkAll(' . singleQuote($form->getProp(CLX_PROP_ID)) . ', ' . singleQuote('canmodify') . ');');
        $candelete = new htmlCheckBox('Masive_All_candelete', false);
        $candelete->setProp('onclick', 'checkAll(' . singleQuote($form->getProp(CLX_PROP_ID)) . ', ' . singleQuote('candelete') . ');');
        $canappend = new htmlCheckBox('Masive_All_canappend', false);
        $canappend->setProp('onclick', 'checkAll(' . singleQuote($form->getProp(CLX_PROP_ID)) . ', ' . singleQuote('canappend') . ');');
        $isadmin = new htmlCheckBox('Masive_All_isadmin', false);
        $isadmin->setProp('onclick', 'checkAll(' . singleQuote($form->getProp(CLX_PROP_ID)) . ', ' . singleQuote('isadmin') . ');');
        // Footers
        $table->addCells(CLX_CLASS_FOOTERS, array($buttons, $canread->source(), $canmodify->source(), $candelete->source(), $canappend->source(), $isadmin->source()));
        $form->addContents($table->source());
        return $script . $form->source();
    }

    function fillAccessForm($ownercode, $ownerref, $tablecode, $tablename, $class, &$table, &$form) {
        $record = ClxCommon::$db->getRecord($this->tablename, 'OWNERCODE=' . singleQuote($ownercode) . ' AND OWNERREF=' . singleQuote($ownerref) . ' AND TABLECODE=' . singleQuote($tablecode));
        $js = 'checkReadValue(' . singleQuote($form->getProp(CLX_PROP_ID)) . ', ' . singleQuote($tablecode) . ');';
        $canread = new htmlCheckBox($tablecode . '_canread', $record['CANREAD']);
        $canread->setProp('onclick', $js);
        $js = 'checkAccessValues(' . singleQuote($form->getProp(CLX_PROP_ID)) . ',' . singleQuote($tablecode) . ');';
        $canmodify = new htmlCheckBox($tablecode . '_canmodify', $record['CANMODIFY']);
        $canmodify->setProp('onclick', $js);
        $candelete = new htmlCheckBox($tablecode . '_candelete', $record['CANDELETE']);
        $candelete->setProp('onclick', $js);
        $canappend = new htmlCheckBox($tablecode . '_canappend', $record['CANAPPEND']);
        $canappend->setProp('onclick', $js);
        $js = 'checkAdminValue(' . singleQuote($form->getProp(CLX_PROP_ID)) . ', ' . singleQuote($tablecode) . ');';
        $isadmin = new htmlCheckBox($tablecode . '_isadmin', $record['ISADMIN']);
        $isadmin->setProp('onclick', $js);
        //   - Adds Table and Module codes to improve support activities
        $table->addCells($class, array($tablename . ' [' . $tablecode . ']', $canread->source(), $canmodify->source(), $candelete->source(), $canappend->source(), $isadmin->source()));
    }

    function checkForm3(&$msg) {
        $msg = array();
        return count($msg) == CLX_NUM_ZERO;
    }

    function saveWizard() {
        $ownercode = UserRequest::$post['ACTA_ownercode'];
        $ownerref = UserRequest::$post['ACTA_ownerref'];

        $tablerecs = ClxCommon::$db->getRecords('CLX_DB_TABLES');
        while ($tablerec = ClxCommon::$db->getNextRecord($tablerecs)) {
		  $record = array('OWNERCODE' => $ownercode, 'OWNERREF' => $ownerref, 'TABLECODE' => $tablerec['IDCODE'],
			 'CANREAD' => (empty(UserRequest::$post[$tablerec['IDCODE'] . '_canread']) ? CLX_NUM_ZERO : CLX_NUM_ONE),
			 'CANMODIFY' => (empty(UserRequest::$post[$tablerec['IDCODE'] . '_canmodify']) ? CLX_NUM_ZERO : CLX_NUM_ONE),
			 'CANDELETE' => (empty(UserRequest::$post[$tablerec['IDCODE'] . '_candelete']) ? CLX_NUM_ZERO : CLX_NUM_ONE),
			 'CANAPPEND' => (empty(UserRequest::$post[$tablerec['IDCODE'] . '_canappend']) ? CLX_NUM_ZERO : CLX_NUM_ONE),
			 'ISADMIN' => (empty(UserRequest::$post[$tablerec['IDCODE'] . '_isadmin']) ? CLX_NUM_ZERO : CLX_NUM_ONE));
		  $crit = 'OWNERCODE=' . singleQuote($ownercode) . ' AND OWNERREF=' . singleQuote($ownerref) . ' AND TABLECODE = ' . singleQuote($record['TABLECODE']);

		  $id = ClxCommon::$db->loadDbRecord($this->tablename, $record, false, $crit);
		  unset($id);
		}
		ClxCommon::$db->freeResult($tablerecs);
    }

    /**
     * Saves Menu Table Wizard from Menu Items.
     * @param string $ownercode
     * @param string $ownerref
     * @param string $tablecode
     * @param string $canread
     * @param string $canmodify
     * @param string $candelete
     * @param string $canappend
     * @param string $isadmin
     */
    function saveMenuWizard($ownercode, $ownerref, $tablecode, $canread, $canmodify, $candelete, $canappend, $isadmin) {
        $record = array();
        $record['OWNERCODE'] = $ownercode;
        $record['OWNERREF'] = $ownerref;
        $record['TABLECODE'] = $tablecode;
        $record['CANREAD'] = (empty($canread)) ? CLX_NUM_ZERO : CLX_NUM_ONE;
        $record['CANMODIFY'] = (empty($canmodify)) ? CLX_NUM_ZERO : CLX_NUM_ONE;
        $record['CANDELETE'] = (empty($candelete)) ? CLX_NUM_ZERO : CLX_NUM_ONE;
		$record['CANAPPEND'] = (empty($canappend)) ? CLX_NUM_ZERO : CLX_NUM_ONE;
        $record['ISADMIN'] = (empty($isadmin)) ? CLX_NUM_ZERO : CLX_NUM_ONE;
        $crit = 'OWNERCODE=' . singleQuote($ownercode) . ' AND OWNERREF=' . singleQuote($ownerref) . ' AND UPPER(TABLECODE)=' . singleQuote(uppercase($tablecode));
        $id = ClxCommon::$db->loadDbRecord($this->tablename, $record, false, $crit);
        unset($id);
    }

    function confirmDeleteFill(&$table, $record) {
        $table->addData(dict('GEN_LBL_NAME') . ':', $record['ACTA_ownercode'] . ':' . $record['ACTA_ownerref']);
        $table->addData(dict('GEN_LBL_TABLECODE') . ':', $record['ACTA_tablecode']);
    }

    function filterFormFill($msg, &$table) {
        if (!array_key_exists('ACTA_ownerref', UserRequest::$post))
		  UserRequest::$post['ACTA_ownerref'] = '';
        if (!array_key_exists('ACTA_tablecode', UserRequest::$post))
		  UserRequest::$post['ACTA_tablecode'] = '';
        $codes = new HTMLSelect('ACTA_ownercode');
		$codes->addItem('', '', true);
        $codes->addItem(dict('GEN_LBL_USER'), 'USER', false);
        $codes->addItem(dict('GEN_LBL_USERGROUP'), 'GRUP', false);
        $field = new HTMLField('ACTA_ownerref', UserRequest::$post['ACTA_ownerref'], CLX_NUM_30, CLX_NUM_30);
        $table->addData(dict('GEN_LBL_GIVE_ACCESS_TO') . ':', $codes->source() . $field->source());
        $table->addField($msg, dict('GEN_LBL_TABLE') . ':', 'ACTA_tablecode', UserRequest::$post['ACTA_tablecode'], CLX_NUM_FOUR, CLX_NUM_FOUR);
        $sort = new HtmlSelect('ACTA_sort');
        $sort->addItem('', '', true);
        $sort->addItem(dict('GEN_LBL_NAME'), 'ACTA.OWNERCODE, ACTA.OWNERREF', false);
        $sort->addItem(dict('GEN_LBL_TABLE'), 'ACTA.TABLECODE', false);
        $table->addData(dict('GEN_LBL_ORDERBY') . ':', $sort->source());
    }

    function setfilter() {
        $filter = '';
        $boolean = '';
        if (!empty(UserRequest::$post['ACTA_ownercode'])) {
		  $filter .= $boolean . ' ACTA.OWNERCODE = ' . singleQuote(UserRequest::$post['ACTA_ownercode']);
		  $boolean = 'AND';
		  $filter .= $boolean . ' ACTA.OWNERREF LIKE ' . singleQuote(UserRequest::$post['ACTA_ownerref'] . '%');
        }
        if (!empty(UserRequest::$post['ACTA_tablecode'])) {
		  $filter .= $boolean . ' ACTA.TABLECODE LIKE ' . singleQuote(UserRequest::$post['ACTA_tablecode'] . '%');
		  $boolean = 'AND';
        }
        $_SESSION['paging']['ACTAList']['sortby'] = UserRequest::$post['ACTA_sort'];
        $_SESSION['paging']['ACTAList']['criteria'] = $filter;
        $_SESSION['paging']['ACTAList']['startat'] = '0';
    }
	///////////////////////////////////////////////////
	///comentado 22/01/2019 para utilizar la clx_tool
	//////////////////////////////////////////////////
    /*function clearfilter() {
        $_SESSION['paging']['ACTAList']['criteria'] = '';
    }*/

    function loadToolRecord($recordid) {
        $values = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $recordid);
        $record = array();
        $record['ACTA_id'] = $recordid;
        $record['ACTA_ownercode'] = $values['OWNERCODE'];
        $record['ACTA_ownerref'] = $values['OWNERREF'];
        $record['ACTA_tablecode'] = $values['TABLECODE'];
        $record['ACTA_isadmin'] = $values['ISADMIN'];
        $record['ACTA_canread'] = $values['CANREAD'];
        $record['ACTA_canmodify'] = $values['CANMODIFY'];
        $record['ACTA_candelete'] = $values['CANDELETE'];
        $record['ACTA_canappend'] = $values['CANAPPEND'];
        return $record;
    }

    function deleteRecordSk($id) {
        ClxCommon::$db->deleteRecords($this->tablename, 'ID=' . $id);
    }

    function defaultRecord() {
        $record = array();
        $record['ACTA_ownercode'] = 'USER';
        $record['ACTA_ownerref'] = $_SESSION['user_record']['username'];
        $record['ACTA_tablecode'] = '';
        $record['ACTA_isadmin'] = '0';
        $record['ACTA_canread'] = '0';
        $record['ACTA_canmodify'] = '0';
        $record['ACTA_candelete'] = '0';
        $record['ACTA_canappend'] = '0';
        return $record;
    }

    function userGroups($username) {
        $list = array();
        $records = ClxCommon::$db->getRecords('CLX_AC_GROUP_MEMBERS', 'UPPER(USERNAME)=' . singleQuote(uppercase($username)));
        while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
		  $list[] = singlequote($record['GROUPCODE']);
        }
		ClxCommon::$db->freeResult($records);
        return $list;
    }

    function securityQuery() {
        $username = $_SESSION['user_record']['username'];
        $usersql = '(OWNERCODE=' . singleQuote('USER') . ' AND UPPER(OWNERREF)=' . singleQuote(uppercase($username)) . ')';
        $usergroups = $this->userGroups($username);
        if (count($usergroups) > CLX_NUM_ZERO) {
		  $usergroups = toString($usergroups);
		  $sql = '(' . $usersql . ' OR (OWNERCODE=' . singleQuote('GRUP') . ' AND OWNERREF IN (' . $usergroups . ')))';
        } else {
		  $sql = $usersql;
        }
        return $sql;
    }

    /**
     * Build access report form
     *
     * @param array $record
     * @param array $msg
     * @return string
     */
    function repForm($record, $msg) {

        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'ACREPORT');
        $form->addArgument('confirm', '1');
        if (empty($record['ACAD_usergroup'])) {
		  $record['ACAD_usergroup'] = '';
        }
        if (empty($record['ACAD_username'])) {
		  $record['ACAD_username'] = '';
        }
        if (empty($record['ACTA_format'])) {
		  $record['ACTA_format'] = '';
        }
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_ACTA_TITLE_ACREPORT'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addCell(dict('TOOL_ACTA_TITLE_ACREPORT_EXPLAIN'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addLookup($msg, dict('GEN_LBL_USERGROUP') . ':', 'ACAD_usergroup', $record['ACAD_usergroup'], 'ACGR', CLX_NUM_16, CLX_NUM_16);
        $table->addLookup($msg, dict('GEN_LBL_USERNAME') . ':', 'ACAD_username', $record['ACAD_username'], 'ACUS', CLX_NUM_30, CLX_NUM_30);
        $table->addSeparator(CLX_CLASS_DATA);
        $codes = new HTMLSelect('ACTA_format');
        $codes->addItem(dict('GEN_LBL_ONLINE'), 'HTML', ($record['ACTA_format'] === 'HTML') ? true : false);
        $codes->addItem(dict('GEN_LBL_EXCELL'), 'XLS', ($record['ACTA_format'] === 'XLS') ? true : false);
        $table->addData(dict('GEN_LBL_FORMAT'), $codes->source());
        $form->addContents($table->source());
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $buttons =htmlButton::getButton('submit', dict('GEN_LBL_CONTINUE'), array('action' => 'ACREPORT', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        $c = new htmlButtonCancel();
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->addContents($table->source());
        return $form->source();
    }

    /**
     * Validate report form data
     *
     * @param array $msg
     * @return boolean
     */
    function checkRepForm(&$msg) {
        //Check group
        if (!empty(UserRequest::$post['ACAD_usergroup'])) {
            if (ClxCommon::$db->oneExistsRecords('CLX_AC_GROUPS', 'UPPER(USERNAME) = ' . singleQuote(uppercase(UserRequest::$post['ACAD_username']))) == CLX_NUM_ZERO) {
			 $msg['ACAD_usergroup'] = dict('TOOL_ACGR_NOGROUPFOUND');
		  }
        }
        //Check user
        if (!empty(UserRequest::$post['ACAD_username'])) {
            if (ClxCommon::$db->oneExistsRecords('CLX_AC_USERS', 'UPPER(USERNAME) = ' . singleQuote(uppercase(UserRequest::$post['ACAD_username']))) == CLX_NUM_ZERO) {
			 $msg['ACAD_username'] = dict('GEN_LBL_USER_DONT_EXIST');
		  }
        }
        //check if user on group
        if (!empty(UserRequest::$post['ACAD_username']) && !empty(UserRequest::$post['ACAD_usergroup'])) {
            if (ClxCommon::$db->oneExistsRecords('CLX_AC_GROUP_MEMBERS', 'UPPER(USERNAME) = ' . singleQuote(uppercase(UserRequest::$post['ACAD_username'])) . ' AND GROUPCODE = ' . singleQuote(UserRequest::$post['ACAD_usergroup'])) == CLX_NUM_ZERO) {
			 $msg['ACAD_username'] = dict('GEN_LBL_USER_NOTIN_GROUP');
		  }
        }
        return count($msg) == CLX_NUM_ZERO;
    }

    function repAccess($groupcode, $username, $format) {
        $contents = '';
        if (empty($groupcode) && empty($username)) {
		  $sql = 'SELECT IDCODE FROM CLX_AC_GROUPS ORDER BY IDCODE ASC';
		  $rs = ClxCommon::$db->executeQuery($sql);
		  while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
			 $contents .= $this->repAccessBuildGroup($r['IDCODE'], '', $format);
		  }
		  ClxCommon::$db->freeResult($rs);
        } else {
		  $contents .= $this->repAccessBuildGroup($groupcode, $username, $format);
        }

        if ($format === 'XLS') {
			ob_clean();
		  sendDownloadFile('download', 'xls');
        }
        return $contents;
    }

    function repAccessBuildGroup($groupcode, $username, $format) {
        $contents = '';
        $xls = ($format === 'XLS') ? true : false;
        $yesno = array(dict('GEN_LBL_NO'), dict('GEN_LBL_YES'));
        $status = array(dict('GEN_LBL_INACTIVE'), dict('GEN_LBL_ACTIVE'));
        $isadmin = ClxCommon::$db->getField('CLX_AC_USERS', 'ISADMIN', 'UPPER(USERNAME) = ' . singleQuote(uppercase($username)));
        //$sql = 'SELECT DISTINCT (SELECT MI2.NAME FROM CLX_UI_MENUITEMS MI2 WHERE MI.PARENT = MI2.IDCODE) PARENT_MENU_NAME, LI.MODULECODE MODULECODE, MI.IDCODE MENU_ID, MI.NAME  MENU, LI.TABLECODE PRIM_TABLECODE, TB.IDNAME  PRIM_TABLENAME, LI2.TABLECODE SECD_TABLECODE, TB2.IDNAME SECD_TABLENAME, 0 PRIM_ISADMIN, 0 PRIM_CANREAD, 0 PRIM_CANMODIFY, 0 PRIM_CANDELETE, 0 PRIM_CANAPPEND, 0 SECD_ISADMIN, 0 SECD_CANREAD, 0 SECD_CANMODIFY, 0 SECD_CANDELETE, 0 SECD_CANAPPEND, MI.PARENT, MI.SEQNUM, MI.MODULECODE FROM CLX_UI_MENUITEMS MI, CLX_DB_TABLES TB, CLX_UI_LISTINGS LI LEFT JOIN CLX_UI_FORM_LISTINGS FL ON FL.FORMCODE=LI.DISPLAY_FORM LEFT JOIN CLX_UI_LISTINGS LI2 ON LI2.IDCODE=FL.LISTINGCODE LEFT JOIN CLX_DB_TABLES TB2 ON TB2.IDCODE=LI2.TABLECODE WHERE 1=1 AND MI.ACTIVE=1 AND MI.TYPE=' . singleQuote('LSTR') . ' AND LI.IDCODE=MI.EXTRA1 AND TB.IDCODE = LI.TABLECODE ORDER BY MI.PARENT, MI.SEQNUM, MI.MODULECODE';
        $sql = 'SELECT DISTINCT (SELECT MI2.NAME FROM CLX_UI_MENUITEMS MI2 WHERE MI.PARENT = MI2.IDCODE) PARENT_MENU_NAME, LI.MODULECODE MODULECODE, MI.IDCODE MENU_ID, MI.NAME  MENU, LI.TABLECODE PRIM_TABLECODE, TB.IDNAME  PRIM_TABLENAME, LI2.TABLECODE SECD_TABLECODE, TB2.IDNAME SECD_TABLENAME, 0 PRIM_ISADMIN, 0 PRIM_CANREAD, 0 PRIM_CANMODIFY, 0 PRIM_CANDELETE, 0 PRIM_CANAPPEND, 0 SECD_ISADMIN, 0 SECD_CANREAD, 0 SECD_CANMODIFY, 0 SECD_CANDELETE, 0 SECD_CANAPPEND, MI.PARENT, MI.SEQNUM, MI.MODULECODE, MI.ACTIVE MENU_ACCESS FROM CLX_UI_MENUITEMS MI, CLX_DB_TABLES TB, CLX_UI_LISTINGS LI LEFT JOIN CLX_UI_FORM_LISTINGS FL ON FL.FORMCODE=LI.DISPLAY_FORM LEFT JOIN CLX_UI_LISTINGS LI2 ON LI2.IDCODE=FL.LISTINGCODE LEFT JOIN CLX_DB_TABLES TB2 ON TB2.IDCODE=LI2.TABLECODE WHERE 1=1 AND MI.ACTIVE=1 AND MI.TYPE=' . singleQuote('LSTR') . ' AND LI.IDCODE=MI.EXTRA1 AND TB.IDCODE = LI.TABLECODE ORDER BY MI.PARENT, MI.SEQNUM, MI.MODULECODE';
        $rs = ClxCommon::$db->executeQuery($sql);
        if ($xls === true) {
		  $lbreak = CLX_CHAR_NL; //Line Break
		  $fbreak = CLX_CHAR_TAB; //Field Selarator
		  $contents .= dict('TOOL_ACTA_TITLE_ACREPORT') . $lbreak;
		  $contents .= dict('GEN_LBL_DATE') . ': ' . xFormat::getNow('Y-m-d G:i:s') . $lbreak;
		  $contents .= dict('GEN_LBL_CREATEDBY') . ': ' . $_SESSION['user_record']['username'] . $lbreak;
		  if (!empty($groupcode)) {
			 $contents .= dict('GEN_LBL_GROUPCODE') . ': ' . $groupcode . $lbreak;
		  }
		  if (!empty($username)) {
			 $contents .= dict('GEN_LBL_USERNAME') . ': ' . $username . $lbreak;
			 $contents .= dict('GEN_LBL_SYSADMIN') . ': ' . $yesno[$isadmin] . $lbreak;
		  }
		  $contents .= dict('GEN_LBL_MODULE') . $fbreak . dict('GEN_LBL_MENU') . $fbreak . dict('GEN_LBL_STATUS') . $fbreak . dict('GEN_LBL_MAIN_TABLE') . $fbreak . dict('GEN_LBL_CANREAD') . $fbreak . dict('GEN_LBL_CANMODIFY') . $fbreak . dict('GEN_LBL_CANDELETE') . $fbreak . dict('GEN_LBL_CANAPPEND') . $fbreak . dict('GEN_LBL_ISADMIN') . $fbreak . dict('GEN_LBL_SECUNDARY_TABLES') . $fbreak . dict('GEN_LBL_CANREAD') . $fbreak . dict('GEN_LBL_CANMODIFY') . $fbreak . dict('GEN_LBL_CANDELETE') . $fbreak . dict('GEN_LBL_CANAPPEND') . $fbreak . dict('GEN_LBL_ISADMIN') . $lbreak;
		  while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
			 $this->buildAccessRepDetail($groupcode, $username, $r, $isadmin);
			 $contents .= $r['MODULECODE'] . $fbreak;
			 $menu = dict($r['MENU']);
			 if ($r['PARENT_MENU_NAME'] <> '' && $r['PARENT_MENU_NAME'] <> $r['MENU']) {
			     $menu = dict($r['PARENT_MENU_NAME']) . '->' . $menu;
			 }
			 $contents .= $menu . $fbreak;
			 //$contents .= $status[$r['MENU_ACCESS']] . $fbreak;
			 $contents .= $status[CLX_NUM_ZERO+$r['MENU_ACCESS']] . $fbreak;
			 if (!empty($r['PRIM_TABLENAME'])) {
			     $prim_tbl = $r['PRIM_TABLENAME'] . '(' . $r['PRIM_TABLECODE'] . ')';
			     $contents .= $prim_tbl . $fbreak;
			     $contents .= $yesno[$r['PRIM_CANREAD']] . $fbreak;
			     $contents .= $yesno[$r['PRIM_CANMODIFY']] . $fbreak;
			     $contents .= $yesno[$r['PRIM_CANDELETE']] . $fbreak;
			     $contents .= $yesno[$r['PRIM_CANAPPEND']] . $fbreak;
			     $contents .= $yesno[$r['PRIM_ISADMIN']] . $fbreak;
			 } else {
			     $prim_tbl = '';
			     $contents .= $prim_tbl . $fbreak;
			     $contents .= $fbreak . $fbreak . $fbreak . $fbreak;
			 }
			 if (!empty($r['SECD_TABLENAME'])) {
			     $prim_tbl = $r['SECD_TABLENAME'] . '(' . $r['SECD_TABLECODE'] . ')';
			     $contents .= $prim_tbl . $fbreak;
			     $contents .= $yesno[$r['SECD_CANREAD']] . $fbreak;
			     $contents .= $yesno[$r['SECD_CANMODIFY']] . $fbreak;
			     $contents .= $yesno[$r['SECD_CANDELETE']] . $fbreak;
			     $contents .= $yesno[$r['SECD_CANAPPEND']] . $fbreak;
			     $contents .= $yesno[$r['SECD_ISADMIN']];
			 } else {
			     $prim_tbl = '';
			     $contents .= $prim_tbl . $fbreak;
			     $contents .= $fbreak . $fbreak . $fbreak . $fbreak;
			 }
			 $contents .= $lbreak;
		  }
		  ClxCommon::$db->freeResult($s);
		  $contents .= $lbreak;
	    $contents = encode_string($contents, 'ISO-8859-1', 'UTF-8');
        } else {//else xls
		  $table = new htmlTable(CLX_NUM_15);
		  $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
		  $table->addCell(dict('TOOL_ACTA_TITLE_ACREPORT'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_NUM_15));
		  $table->addCell(dict('GEN_LBL_DATE') . ':' . xFormat::getNow('Y-m-d G:i:s'), array(CLX_PROP_CLASS => CLX_CLASS_DATA, CLX_PROP_COLSPAN => CLX_NUM_15));
		  $table->addCell(dict('GEN_LBL_CREATEDBY') . ':' . $_SESSION['user_record']['username'], array(CLX_PROP_CLASS => CLX_CLASS_DATA, CLX_PROP_COLSPAN => CLX_NUM_15));
		  if (!empty($groupcode)) {
			 $table->addCell(dict('GEN_LBL_GROUPCODE') . ':' . $groupcode, array(CLX_PROP_CLASS => CLX_CLASS_DATA, CLX_PROP_COLSPAN => CLX_NUM_15));
		  }
		  if (!empty($username)) {
			 $table->addCell(dict('GEN_LBL_USERNAME') . ':' . $username, array(CLX_PROP_CLASS => CLX_CLASS_DATA, CLX_PROP_COLSPAN => CLX_NUM_15));
			 $table->addCell(dict('GEN_LBL_SYSADMIN') . ':' . $yesno[$isadmin], array(CLX_PROP_CLASS => CLX_CLASS_DATA, CLX_PROP_COLSPAN => CLX_NUM_15));
		  }
		  $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_MODULE'), dict('GEN_LBL_MENU'), dict('GEN_LBL_STATUS'), dict('GEN_LBL_MAIN_TABLE'), dict('GEN_LBL_CANREAD'), dict('GEN_LBL_CANMODIFY'), dict('GEN_LBL_CANDELETE'), dict('GEN_LBL_CANAPPEND'), dict('GEN_LBL_ISADMIN'), dict('GEN_LBL_SECUNDARY_TABLES'), dict('GEN_LBL_CANREAD'), dict('GEN_LBL_CANMODIFY'), dict('GEN_LBL_CANDELETE'), dict('GEN_LBL_CANAPPEND'), dict('GEN_LBL_ISADMIN')));
		  $class = CLX_CLASS_DATA;
		  $i = CLX_INIT_ZERO;
		  while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
			 $class = ($i % CLX_NUM_TWO) ? CLX_CLASS_DATAALT : CLX_CLASS_DATA;
			 $celprop = array(CLX_PROP_CLASS, $class);
			 $i++;
			 $this->buildAccessRepDetail($groupcode, $username, $r, $isadmin);
			 $table->addCell($r['MODULECODE'], $celprop);
			 $menu = dict($r['MENU']);
			 if ($r['PARENT_MENU_NAME'] <> '' && $r['PARENT_MENU_NAME'] <> $r['MENU']) {
			     $menu = dict($r['PARENT_MENU_NAME']) . '->' . $menu;
			 }
			 $table->addCell($menu, $celprop);
			 $table->addCell($status[$r['MENU_ACCESS']], $celprop);
			 if (!empty($r['PRIM_TABLENAME'])) {
			     $table->addCells($class, array($r['PRIM_TABLENAME'] . '(' . $r['PRIM_TABLECODE'] . ')', $yesno[$r['PRIM_CANREAD']], $yesno[$r['PRIM_CANMODIFY']], $yesno[$r['PRIM_CANDELETE']], $yesno[$r['PRIM_CANAPPEND']], $yesno[$r['PRIM_ISADMIN']]));
			 } else {
			     $table->addCells($class, array(CLX_CHAR_HTML_NBSP, CLX_CHAR_HTML_NBSP, CLX_CHAR_HTML_NBSP, CLX_CHAR_HTML_NBSP, CLX_CHAR_HTML_NBSP, CLX_CHAR_HTML_NBSP));
			 }
			 if (!empty($r['SECD_TABLENAME'])) {
			     $table->addCells($class, array($r['SECD_TABLENAME'] . '(' . $r['SECD_TABLECODE'] . ')', $yesno[$r['SECD_CANREAD']], $yesno[$r['SECD_CANMODIFY']], $yesno[$r['SECD_CANDELETE']], $yesno[$r['SECD_CANAPPEND']], $yesno[$r['SECD_ISADMIN']]));
			 } else {
			     $table->addCells($class, array(CLX_CHAR_HTML_NBSP, CLX_CHAR_HTML_NBSP, CLX_CHAR_HTML_NBSP, CLX_CHAR_HTML_NBSP, CLX_CHAR_HTML_NBSP, CLX_CHAR_HTML_NBSP));
			 }
		  }
		  ClxCommon::$db->freeResult($rs);
		  $contents .= $table->source() . '<br>';
        }//end if xls
	$ACRP = getTool('ACRP');
	$contents .=$ACRP->repAccessBuildGroup($groupcode, $username, $format);
        return $contents;
    }

    private function buildAccessRepDetail($groupcode, $username, &$r, $isadmin) {
        if ($isadmin === true) {
		  $r['PRIM_CANREAD'] = $isadmin;
		  $r['PRIM_CANMODIFY'] = $isadmin;
		  $r['PRIM_CANDELETE'] = $isadmin;
		  $r['PRIM_CANAPPEND'] = $isadmin;
		  $r['PRIM_ISADMIN'] = $isadmin;
		  $r['SECD_CANREAD'] = $isadmin;
		  $r['SECD_CANMODIFY'] = $isadmin;
		  $r['SECD_CANDELETE'] = $isadmin;
		  $r['SECD_CANAPPEND'] = $isadmin;
		  $r['SECD_ISADMIN'] = $isadmin;
		  $r['MENU_ACCESS'] = $isadmin;
        } else {
		  //Get primary table
		  $sql = 'SELECT IFNULL(MAX(CANREAD),0) CANREAD, IFNULL(MAX(CANMODIFY),0) CANMODIFY, IFNULL(MAX(CANDELETE),0) CANDELETE, IFNULL(MAX(CANAPPEND),0) CANAPPEND, IFNULL(MAX(ISADMIN),0) ISADMIN FROM CLX_AC_TABLES WHERE 1=1 ';
		  if (!empty($username)) {
			 $sql .=' AND UPPER(OWNERCODE) = ' . singleQuote(uppercase($username)) . ' ';
		  }
		  if (!empty($groupcode)) {
			 $sql .=' AND OWNERREF = ' . singleQuote($groupcode) . ' ';
		  }
		  $sql1 = $sql . ' AND TABLECODE = ' . singleQuote($r['PRIM_TABLECODE']);
		  $prim = ClxCommon::$db->getOneRecord($sql1);
		  $sql1 = $sql . ' AND TABLECODE = ' . singleQuote($r['SECD_TABLECODE']);
		  $secd = ClxCommon::$db->getOneRecord($sql1);
		  unset($sql1);
		  if (!empty($prim)) {
			 $r['PRIM_CANREAD'] = $prim['CANREAD'];
			 $r['PRIM_CANMODIFY'] = $prim['CANMODIFY'];
			 $r['PRIM_CANDELETE'] = $prim['CANDELETE'];
			 $r['PRIM_CANAPPEND'] = $prim['CANAPPEND'];
			 $r['PRIM_ISADMIN'] = $prim['ISADMIN'];
		  }
		  if (!empty($secd)) {
			 $r['SECD_CANREAD'] = $secd['CANREAD'];
			 $r['SECD_CANMODIFY'] = $secd['CANMODIFY'];
			 $r['SECD_CANDELETE'] = $secd['CANDELETE'];
			 $r['SECD_CANAPPEND'] = $secd['CANAPPEND'];
			 $r['SECD_ISADMIN'] = $secd['ISADMIN'];
		  }
		  //adding menu access
		  $menuid = (empty($r['MENU_ID'])) ? CLX_NUM_ZERO : $r['MENU_ID'];
		  $sql = 'SELECT IFNULL(MAX(CANREAD),0) MENU_ACCESS FROM CLX_AC_MENUITEMS WHERE MENUID = ' . singleQuote($menuid);
		  if (!empty($username)) {
			 $sql .=' AND UPPER(OWNERCODE) = ' . singleQuote(uppercase($username));
		  }
		  if (!empty($groupcode)) {
			 $sql .=' AND OWNERREF = ' . singleQuote($groupcode);
		  }
		  $prim = ClxCommon::$db->getOneRecord($sql);
		  unset($sql);
		  if (!empty($prim)) {
			 $r['MENU_ACCESS'] = $prim['MENU_ACCESS'];
		  }
        }
    }

}
