<?php

/**
 * Tool Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: alenetdev $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2019-01-22 15:24:09 -0400 (ma., 22 ene. 2019) $
 * @version        $Rev: 10490 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  Included label
 */
class ac_tools extends clx_tool {

    // CONSTRUCTOR //
    function __construct() {
        $this->code = 'ACTO';
        $this->name = 'Tool Access Manager';
        $this->system = 'Security';
        $this->version = '0.4.1.2';

        $this->has_access = true;


        $this->tablename = 'CLX_AC_TOOLS';
        $this->record_name_singular = 'tool access';
        $this->record_name_plural = 'tool accesses';
        $this->primary_field = 'OWNERCODE';
        $this->secondary_field = 'OWNERREF';
    }

    function install() {
        $UIMI = getTool('UIMI');
        $UIMI->add($this->code, 'MENU_TOOLS:MENU_SECURITY:MENU_TOOL_ACCESS', 'TOOL', $this->code, 'ac=LIST');
    }

    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['OWNERCODE'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '');
        $fields['OWNERREF'] = array('type' => 'varchar(30)', 'null' => 'NO', 'properties' => '');
        $fields['TOOLCODE'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '');
        $fields['ISADMIN'] = array('type' => 'tinyint(1)', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['CANREAD'] = array('type' => 'tinyint(1)', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['CANMODIFY'] = array('type' => 'tinyint(1)', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['CANDELETE'] = array('type' => 'tinyint(1)', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['CANAPPEND'] = array('type' => 'tinyint(1)', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'UNI', 'fieldname' => 'OWNERCODE,OWNERREF,TOOLCODE');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB', 'fields' => $fields, 'indexes' => $indexes);
    }

    function action($ac, $idlist) {
        $contents = '';
        $msg = array();
        if ($this->getAccess()->canRead()) {
		  switch ($ac) {
			 case 'WIZARD':
			     if (($this->getAccess()->canDisplay()) || $this->getAccess()->canModify()) {
			         $contents = $this->controlWizard();
			     } else {
			         $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [AC_TOOLS-CAN_APPEND & CAN_MODIFY][' . $ac . '] ' . $this->getCode(false));
			     }
			     break;
			 case 'MODIFY':
			     if ($this->getAccess()->canModify()) {
			         $id = array_pop($idlist);
			         $record = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $id);
			         UserRequest::$post['ACTO_ownercode'] = $record['OWNERCODE'];
			         UserRequest::$post['ACTO_ownerref'] = $record['OWNERREF'];
			         UserRequest::$post['ACTO_toolcode'] = $record['TOOLCODE'];
			         UserRequest::$post['step'] = CLX_NUM_THREE;
			         $contents = $this->editForm3(UserRequest::$post, $msg);
			     } else {
			         $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [AC_TOOLS-CAN_MODIFY][' . $ac . '] ' . $this->getCode(false));
			     }
			     break;
			 default:
			     $contents = parent::action($ac, $idlist);
			     break;
		  }
        } else {
		  $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [AC_TOOLS-CAN_READ][' . $ac . '] ' . $this->getCode(false));
        }
        return $contents;
    }

    /**
     * Listing for tool
     * @param type $listingcode
     * @return type
     */
    function makelist($listingcode = null) {
         //$t = time('U'); genera un warning
        $t = time();
        $to = $this->code;
        $here = 'tool.php?to=' . $to . '&t=' . $t;
        clearHistory();
        pushHistory($this->code, 'LIST', array());
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->setProp(CLX_PROP_ID, 'ACTOList');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'NOT_DEFINED');
        $selectclause = 'ACTO.*, DBTB.IDNAME TOOLNAME';
        $fromclause = 'CLX_AC_TOOLS ACTO, CLX_TOOLS DBTB';
        $whereclause = 'ACTO.TOOLCODE = DBTB.IDCODE';
        $sortclause = '';
        $page = new xPaging();
        $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
        $table = new htmlTable(CLX_WIDTH_FIVE);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_ACTO_TITLE_LIST'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_FIVE));
        $page->header($table);
        $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_NAME'), dict('GEN_LBL_TOOLCODE'), dict('GEN_LBL_TOOLNAME'), dict('GEN_LBL_PRIVILEGES')));
        $i = CLX_INIT_ZERO;
        while ($record = $page->nextRecord()) {
		  $recordid = $record['ID'];
		  $select = new htmlCheckbox('selector_' . $recordid, false);
		  $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
		  $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
		  $access = $this->accessDetail($record);
		  $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($record['OWNERCODE'] . ':' . $record['OWNERREF'], $record['TOOLCODE'], $record['TOOLNAME'], $access));
		  $i++;
        }
        $buttons = '';
        if ($this->getAccess()->canDelete()) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        if ($this->getAccess()->canModify()) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        if ($this->getAccess()->canDisplay()) {
		  $b = new htmlFormSubmit(dict('TOOL_ACTO_BUTTON_CREATEWIZ'), 'WIZARD', $form->getProp(CLX_PROP_ID));
		  $b->setProp(CLX_PROP_ID, 'ON');
		  $buttons .= $b->source();
        }
        $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_FILTER'), array('action' => 'FILTER', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
        $form->addContents($page->footers($table, $buttons));
        return $form->source();
    }

    function accessDetail($record) {
        $access = '';
        if (!empty($record['ISADMIN'])) {
		  $access = 'Administrator';
        } else {
		  if (!empty($record['CANREAD'])) {
			 if ($access === '') {
			     $access .= 'Can read';
			 } else {
			     $access .= ', read';
			 }
		  }
		  if (!empty($record['CANMODIFY'])) {
			 if ($access === '') {
			     $access .= 'Can modify';
			 } else {
			     $access .= ', modify';
			 }
		  }
		  if (!empty($record['CANDELETE'])) {
			 if ($access === '') {
			     $access .= 'Can delete';
			 } else {
			     $access .= ', delete';
			 }
		  }
		  if (!empty($record['CANAPPEND'])) {
			 if ($access === '') {
			     $access .= 'Can append';
			 } else {
			     $access .= ', append';
			 }
		  }
		  if (empty($access)) {
			 $access = 'No access privileges.';
		  } else {
			 $access .= '.';
		  }
        }
        return $access;
    }

    function controlWizard() {
        $msg = array();
        $contents = '';
        $step = (isset(UserRequest::$post['step'])) ? UserRequest::$post['step'] : '0';
        switch ($step) {
		  case '0':
			 UserRequest::$post = $this->defaultRecord();
			 $contents = $this->editForm1(UserRequest::$post, $msg);
			 break;
		  case '1':
			 if ($this->checkForm1($msg)) {
			     $contents = $this->editForm2(UserRequest::$post, $msg);
			 } else {
			     $contents = $this->editForm1(UserRequest::$post, $msg);
			 }
			 break;
		  case '2':
			 if ($this->checkForm2($msg)) {
			     $contents = $this->editForm3(UserRequest::$post, $msg);
			 } else {
			     $contents = $this->editForm2(UserRequest::$post, $msg);
			 }
			 break;
		  case '3':
			 if ($this->checkForm3($msg)) {
			     $this->saveWizard();
			     $info = popHistory();
			     $contents = executeHistory($info);
			 } else {
			     $contents = $this->editForm3(UserRequest::$post, $msg);
			 }
			 break;
		  default:
			 break;
        }
        return $contents;
    }

    function editForm1($record, $msg) {
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'WIZARD');
        $form->addArgument('step', '1');
		$buttons = '';

        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_ACTO_TITLE_CREATEWIZ'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addCell(dict('TOOL_ACTO_TITLE_CREATEWIZ1'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $codes = new HTMLSelect('ACTO_ownercode');
        $codes->addItem(dict('GEN_LBL_USER'), 'USER', ($record['ACTO_ownercode'] === 'USER'));
        $codes->addItem(dict('GEN_LBL_USERGROUP'), 'GRUP', ($record['ACTO_ownercode'] === 'GRUP'));
        $table->addData(dict('GEN_LBL_GIVE_ACCESS_TO') . ':', $codes->source());
        $form->addContents($table->source());

        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CONTINUE'), array('action' => 'WIZARD', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        $c = new htmlButtonCancel();

        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->addContents($table->source());
        return $form->source();
    }

    function checkForm1(&$msg) {
        $msg = array();
        return count($msg) == CLX_NUM_ZERO;
    }

    function editForm2($record, $msg) {
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'WIZARD');
        $form->addArgument('step', '2');
        $form->addArgument('ACTO_ownercode', $record['ACTO_ownercode']);
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_ACTO_TITLE_CREATEWIZ'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        if ($record['ACTO_ownercode'] === 'USER') {
		  $table->addCell(dict('TOOL_ACTO_TITLE_CREATEWIZ2U'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
		  $ACUS = &getTool('ACUS');
		  $ACUS->placeChooseUser($msg, dict('GEN_LBL_USERNAME') . ':', 'ACTO_ownerref', (array_key_exists('ACTO_ownerref', $record)) ? $record['ACTO_ownerref'] : '', $table);
        } elseif ($record['ACTO_ownercode'] === 'GRUP') {
		  $table->addCell(dict('TOOL_ACTO_TITLE_CREATEWIZ2G'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
		  $ACGR = &getTool('ACGR');
		  $ACGR->placeChooseGroup($msg, dict('GEN_LBL_USERGROUP') . ':', 'ACTO_ownerref', (array_key_exists('ACTO_ownerref', $record)) ? $record['ACTO_ownerref'] : '', $table);
        }
        $form->addContents($table->source());
        $buttons = '';
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CONTINUE'), array('action' => 'WIZARD', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        $c = new htmlButtonCancel();
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->addContents($table->source());
        return $form->source();
    }

    function checkForm2(&$msg) {
        $msg = array();
        if (UserRequest::$post['ACTO_ownercode'] === 'USER') {
		  $ACUS = &getTool('ACUS');
		  if (!$ACUS->userExists(UserRequest::$post['ACTO_ownerref'])) {
			 $msg['ACTO_ownerref'] = 'There is no user with this code.';
		  }
        } elseif (UserRequest::$post['ACTO_ownercode'] === 'GRUP') {
		  $ACGR = &getTool('ACGR');
		  if (!$ACGR->groupExists(UserRequest::$post['ACTO_ownerref'])) {
			 $msg['ACTO_ownerref'] = 'There is no group with this code.';
		  }
        }
        return count($msg) == CLX_NUM_ZERO;
    }

    function editForm3($record, $msg) {
        $variables = array();
        $script = setJavaScriptTag(getJavaScript('ACTO_editform3.js', $variables));
        // the Form --------------------------------------
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'WIZARD');
        $form->addArgument('step', '3');
        $form->addArgument('ACTO_ownercode', $record['ACTO_ownercode']);
        $form->addArgument('ACTO_ownerref', $record['ACTO_ownerref']);
        // header info --------------------------------------
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_ACTO_TITLE_CREATEWIZ'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addCell(dict('TOOL_ACTO_TITLE_CREATEWIZ3'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        if ($record['ACTO_ownercode'] === 'USER') {
		  $table->addData(dict('GEN_LBL_USERNAME') . ':', $record['ACTO_ownerref']);
        } elseif ($record['ACTO_ownercode'] === 'GRUP') {
		  $table->addData(dict('GEN_LBL_USERGROUP') . ':', $record['ACTO_ownerref']);
        }
        $form->addContents($table->source());
        // the records --------------------------------------
        $table = new htmlTable(CLX_WIDTH_SIX);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_TOOLS'), dict('GEN_LBL_CANREAD'), dict('GEN_LBL_CANMODIFY'), dict('GEN_LBL_CANDELETE'), dict('GEN_LBL_CANAPPEND'), dict('GEN_LBL_ISADMIN')));
        //$toolrecs = ClxCommon::$db->getRecords('CLX_TOOLS', 'HAS_ACCESS=1 ORDER BY SYSTEM, IDNAME');
		$toolrecs = ClxCommon::$db->getRecords('CLX_TOOLS A', 'A.HAS_ACCESS=1 ORDER BY A.SYSTEM, A.IDNAME');
        $lastsystem = '';
        $i = CLX_INIT_ZERO;
        while ($toolrec = ClxCommon::$db->getNextRecord($toolrecs)) {
		  if ($lastsystem != $toolrec['SYSTEM']) {
			 $lastsystem = $toolrec['SYSTEM'];
			 $table->addCell($lastsystem, array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_SIX));
		  }
		  $this->fillAccessForm($record['ACTO_ownercode'], $record['ACTO_ownerref'], $toolrec['IDCODE'], $toolrec['IDNAME'] . ' (' . $toolrec['IDCODE'] . ')', ($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, $table, $form);
		  $i++;
        }
		ClxCommon::$db->freeResult( $toolrecs );
        // buttons  --------------------------------------
        $buttons = '';
        $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_SAVE'), array('action' => 'WIZARD', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        $c = new htmlButtonCancel();
        $buttons .= $c->source();
        // Masive selectors  --------------------------------------
        $canread = new htmlCheckBox('Masive_All_canread', false);
        $canread->setProp('onclick', 'checkAll(' . singleQuote($form->getProp(CLX_PROP_ID)) . ', ' . singleQuote('canread') . ');');
        $canmodify = new htmlCheckBox('Masive_All_canmodify', false);
        $canmodify->setProp('onclick', 'checkAll(' . singleQuote($form->getProp(CLX_PROP_ID)) . ', ' . singleQuote('canmodify') . ');');
        $candelete = new htmlCheckBox('Masive_All_candelete', false);
        $candelete->setProp('onclick', 'checkAll(' . singleQuote($form->getProp(CLX_PROP_ID)) . ', ' . singleQuote('candelete') . ');');
        $canappend = new htmlCheckBox('Masive_All_canappend', false);
        $canappend->setProp('onclick', 'checkAll(' . singleQuote($form->getProp(CLX_PROP_ID)) . ', ' . singleQuote('canappend') . ');');
        $isadmin = new htmlCheckBox('Masive_All_isadmin', false);
        $isadmin->setProp('onclick', 'checkAll(' . singleQuote($form->getProp(CLX_PROP_ID)) . ', ' . singleQuote('isadmin') . ');');
        // Footers
        $table->addCells(CLX_CLASS_FOOTERS, array($buttons, $canread->source(), $canmodify->source(), $candelete->source(), $canappend->source(), $isadmin->source()));
        $form->addContents($table->source());
        return $script . $form->source();
    }

    function fillAccessForm($ownercode, $ownerref, $toolcode, $toolname, $class, &$table, &$form) {
        $record = ClxCommon::$db->getRecord($this->tablename, 'OWNERCODE=' . singleQuote($ownercode) . ' AND OWNERREF=' . singleQuote($ownerref) . ' AND TOOLCODE=' . singleQuote($toolcode));
        $js = 'checkReadValue(' . singleQuote($form->getProp(CLX_PROP_ID)) . ', ' . singleQuote($toolcode) . ');';
		if(gettype($record)=='NULL'){
			$record=array('ISADMIN' => 0,'CANREAD' => 0, 'CANMODIFY' => 0, 'CANDELETE' => 0, 'CANAPPEND' => 0);
		}
        $canread = new htmlCheckBox($toolcode . '_canread', $record['CANREAD']);
        $canread->setProp('onclick', $js);
        $js = 'checkAccessValues(' . singleQuote($form->getProp(CLX_PROP_ID)) . ', ' . singleQuote($toolcode) . ');';
        $canmodify = new htmlCheckBox($toolcode . '_canmodify', $record['CANMODIFY']);
        $canmodify->setProp('onclick', $js);
        $candelete = new htmlCheckBox($toolcode . '_candelete', $record['CANDELETE']);
        $candelete->setProp('onclick', $js);
        $canappend = new htmlCheckBox($toolcode . '_canappend', $record['CANAPPEND']);
        $canappend->setProp('onclick', $js);
        $js = 'checkAdminValue(' . singleQuote($form->getProp(CLX_PROP_ID)) . ', ' . singleQuote($toolcode) . ');';
        $isadmin = new htmlCheckBox($toolcode . '_isadmin', $record['ISADMIN']);
        $isadmin->setProp('onclick', $js);
        $table->addCells($class, array($toolname, $canread->source(), $canmodify->source(), $candelete->source(), $canappend->source(), $isadmin->source()));
    }

    function checkForm3(&$msg) {
        $msg = array();
        return count($msg) == CLX_NUM_ZERO;
    }

    function saveWizard() {
        $ownercode = UserRequest::$post['ACTO_ownercode'];
        $ownerref = UserRequest::$post['ACTO_ownerref'];
        //$toolrecs = ClxCommon::$db->getRecords('CLX_TOOLS', 'HAS_ACCESS=1 ORDER BY SYSTEM, IDNAME');
		$toolrecs = ClxCommon::$db->getRecords('CLX_TOOLS A', 'A.HAS_ACCESS=1 ORDER BY A.SYSTEM, A.IDNAME');
        while ($toolrec = ClxCommon::$db->getNextRecord($toolrecs)) {
		  $record = array('OWNERCODE' => $ownercode,
			 'OWNERREF' => $ownerref,
			 'TOOLCODE' => $toolrec['IDCODE'],
			 'CANREAD' => (!empty(UserRequest::$post[$toolrec['IDCODE'] . '_canread'])) ? CLX_NUM_ONE : CLX_NUM_ZERO,
			 'CANMODIFY' => (!empty(UserRequest::$post[$toolrec['IDCODE'] . '_canmodify'])) ? CLX_NUM_ONE : CLX_NUM_ZERO,
			 'CANDELETE' => (!empty(UserRequest::$post[$toolrec['IDCODE'] . '_candelete'])) ? CLX_NUM_ONE : CLX_NUM_ZERO,
			 'CANAPPEND' => (!empty(UserRequest::$post[$toolrec['IDCODE'] . '_canappend'])) ? CLX_NUM_ONE : CLX_NUM_ZERO,
			 'ISADMIN' => (!empty(UserRequest::$post[$toolrec['IDCODE'] . '_isadmin'])) ? CLX_NUM_ONE : CLX_NUM_ZERO);
		  $crit = 'OWNERCODE=' . singleQuote($ownercode) . ' AND OWNERREF=' . singleQuote($ownerref) . ' AND UPPER(TOOLCODE) = ' . singleQuote(uppercase($record['TOOLCODE']));
		  $id = ClxCommon::$db->loadDbRecord($this->tablename, $record, false, $crit);
		  unset($id);
        }
		ClxCommon::$db->freeResult( $toolrecs );
    }

    /**
     * Saves Menu Tool Wizard from Menu Items.
     * @param string $ownercode
     * @param string $ownerref
     * @param string $toolcode
     * @param string $canread
     * @param string $canmodify
     * @param string $candelete
     * @param string $canappend
     * @param string $isadmin
     */
    function saveMenuWizard($ownercode, $ownerref, $toolcode, $canread, $canmodify, $candelete, $canappend, $isadmin) {
        $record = array();
        $record['OWNERCODE'] = $ownercode;
        $record['OWNERREF'] = $ownerref;
        $record['TOOLCODE'] = $toolcode;
        $record['CANREAD'] = (empty($canread)) ? CLX_NUM_ZERO : CLX_NUM_ONE;
        $record['CANMODIFY'] = (empty($canmodify)) ? CLX_NUM_ZERO : CLX_NUM_ONE;
        $record['CANAPPEND'] = (empty($canappend)) ? CLX_NUM_ZERO : CLX_NUM_ONE;
        $record['CANDELETE'] = (empty($candelete)) ? CLX_NUM_ZERO : CLX_NUM_ONE;
        $record['ISADMIN'] = (empty($isadmin)) ? CLX_NUM_ZERO : CLX_NUM_ONE;
        $crit = 'TOOLCODE=' . singleQuote($toolcode) . ' AND OWNERCODE=' . singleQuote($ownercode) . ' AND OWNERREF=' . singleQuote($ownerref);
        $id = ClxCommon::$db->loadDbRecord($this->tablename, $record, false, $crit);
        unset($id);
    }

    function confirmDeleteFill(&$table, $record) {
        $table->addData(dict('GEN_LBL_NAME') . ':', $record['ACTO_ownercode'] . ':' . $record['ACTO_ownerref']);
        $table->addData(dict('GEN_LBL_TOOLCODE') . ':', $record['ACTO_toolcode']);
    }

    function filterFormFill($msg, &$table) {
        $codes = new HTMLSelect('ACTO_ownercode');
        $codes->addItem('', '', true);
        $codes->addItem(dict('GEN_LBL_USER'), 'USER', false);
        $codes->addItem(dict('GEN_LBL_USERGROUP'), 'GRUP', false);
        if (!array_key_exists('ACTO_ownerref', UserRequest::$post))
		  UserRequest::$post['ACTO_ownerref'] = '';
        if (!array_key_exists('ACTO_toolcode', UserRequest::$post))
		  UserRequest::$post['ACTO_toolcode'] = '';
        $field = new HTMLField('ACTO_ownerref', UserRequest::$post['ACTO_ownerref'], CLX_NUM_30, CLX_NUM_30);
        $table->addData(dict('GEN_LBL_GIVE_ACCESS_TO') . ':', $codes->source() . $field->source());
        $table->addField($msg, dict('GEN_LBL_TOOL') . ':', 'ACTO_toolcode', UserRequest::$post['ACTO_toolcode'], CLX_NUM_FOUR, CLX_NUM_FOUR);
        $sort = new HtmlSelect('ACTO_sort');
        $sort->addItem('', '', true);
        $sort->addItem(dict('GEN_LBL_NAME'), 'ACTO.OWNERCODE, ACTO.OWNERREF', false);
        $sort->addItem(dict('GEN_LBL_TOOL'), 'ACTO.TOOLCODE', false);
        $table->addData(dict('GEN_LBL_ORDERBY') . ':', $sort->source());
    }

    function setfilter() {
        $filter = '';
        $boolean = '';
        if (!empty(UserRequest::$post['ACTO_ownercode'])) {
		  $filter .= $boolean . ' ACTO.OWNERCODE = ' . singleQuote(UserRequest::$post['ACTO_ownercode']);
		  $boolean = 'AND';
		  $filter .= $boolean . ' ACTO.OWNERREF LIKE ' . singleQuote(UserRequest::$post['ACTO_ownerref'] . '%');
        }
        if (!empty(UserRequest::$post['ACTO_toolcode'])) {
		  $filter .= $boolean . ' ACTO.TOOLCODE LIKE ' . singleQuote(UserRequest::$post['ACTO_toolcode'] . '%');
		  $boolean = 'AND';
        }
        $_SESSION['paging']['ACTOList']['sortby'] = UserRequest::$post['ACTO_sort'];
        $_SESSION['paging']['ACTOList']['criteria'] = $filter;
        $_SESSION['paging']['ACTOList']['startat'] = '0';
    }
	///////////////////////////////////////////////////
	///comentado 22/01/2019 para utilizar la clx_tool
	//////////////////////////////////////////////////
    /*function clearfilter() {
        $_SESSION['paging']['ACTOList']['criteria'] = '';
    }*/

    function loadToolRecord($recordid) {
        $values = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $recordid);
        $record = array();
        $record['ACTO_id'] = $recordid;
        $record['ACTO_ownercode'] = $values['OWNERCODE'];
        $record['ACTO_ownerref'] = $values['OWNERREF'];
        $record['ACTO_toolcode'] = $values['TOOLCODE'];
        $record['ACTO_isadmin'] = $values['ISADMIN'];
        $record['ACTO_canread'] = $values['CANREAD'];
        $record['ACTO_canmodify'] = $values['CANMODIFY'];
        $record['ACTO_candelete'] = $values['CANDELETE'];
        $record['ACTO_canappend'] = $values['CANAPPEND'];
        return $record;
    }

    function deleteRecordSk($id) {
        ClxCommon::$db->deleteRecords($this->tablename, 'ID=' . $id);
    }

    function defaultRecord() {
        $record = array();
        $record['ACTO_ownercode'] = 'USER';
        $record['ACTO_ownerref'] = $_SESSION['user_record']['username'];
        $record['ACTO_toolcode'] = '';
        $record['ACTO_isadmin'] = CLX_NUM_ZERO;
        $record['ACTO_canread'] = CLX_NUM_ZERO;
        $record['ACTO_canmodify'] = CLX_NUM_ZERO;
        $record['ACTO_candelete'] = CLX_NUM_ZERO;
        $record['ACTO_canappend'] = CLX_NUM_ZERO;
        return $record;
    }

    function userGroups($username) {
        $list = array();
        if (!empty($username)) {
		  $records = ClxCommon::$db->getRecords('CLX_AC_GROUP_MEMBERS', 'UPPER(USERNAME)=' . singleQuote(uppercase($username)));
		  while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
			 $list[] = singlequote($record['GROUPCODE']);
		  }
		  ClxCommon::$db->freeResult( $records );
        }
        return $list;
    }

    function securityQuery() {
        $username = $_SESSION['user_record']['username'];
        $usersql = '(OWNERCODE=' . singleQuote('USER') . ' AND UPPER(OWNERREF)=' . singleQuote(uppercase($username)) . ')';
        $usergroups = $this->userGroups($username);
        if (count($usergroups) > CLX_NUM_ZERO) {
		  $usergroups = toString($usergroups);
		  $sql = '(' . $usersql . ' OR (OWNERCODE=' . singleQuote('GRUP') . ' AND OWNERREF IN (' . $usergroups . ')))';
        } else {
		  $sql = $usersql;
        }
        return $sql;
    }

}
