<?php

/**
 * Tool Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: jclerma $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2017-11-03 15:09:40 -0400 (Fri, 03 Nov 2017) $
 * @version        $Rev: 9615 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 */
class ac_tree_editor extends clx_tool {

    // CONSTRUCTOR //
    function __construct() {
        $this->code = 'ACTE';
        $this->name = 'Access Tree Editor';
        $this->system = 'Security';
        $this->version = '0.2.3.1';

        $this->has_access = true;


        $this->tablename = 'CLX_AC_TREE_CONTENTS';
        $this->record_name_singular = 'tree';
        $this->record_name_plural = 'trees';
        $this->primary_field = 'IDCODE';
        $this->secondary_field = 'IDNAME';

        $this->editform = array(CLX_CLASS_TITLE => 'TOOL_ACTE_TITLE_MODIFY',
		  'panels' => array(
			 array('type' => 'normal',
			     'entries' => array(
			         array('label' => 'GEN_LBL_CODE',
					   'fields' => array(
						  array('name' => 'idcode', 'type' => 'string', 'default' => '', 'size' => '255', 'displaysize' => '48', 'dontmodify' => '1',
						      'validations' => array(
								array('type' => 'string', 'required' => '1'))
						  )
					   )
			         ), array('label' => 'GEN_LBL_NAME',
					   'fields' => array(
						  array('name' => 'idname', 'type' => 'string', 'default' => '', 'size' => '64', 'displaysize' => '48', 'dontmodify' => '1',
						      'validations' => array(
								array('type' => 'string', 'required' => '1'))
						  )
					   )
			         ), array('fields' => array(
						  array('name' => 'LEVELS', 'type' => 'string', 'invisible' => '1', 'size' => '3', 'validations' => array(
								array('type' => 'string', 'required' => '1'))
						  )
					   )
			         ), array('fields' => array(
						  array('name' => 'parent', 'type' => 'string', 'invisible' => '1', 'size' => '255', 'validations' => array(
								array('type' => 'string', 'required' => '1'))
						  )
					   )
			         )
			     )
			 )
		  )
        );
    }

    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['IDCODE'] = array('type' => 'varchar(255)', 'null' => 'NO', 'properties' => '');
        $fields['IDNAME'] = array('type' => 'varchar(64)', 'null' => 'NO', 'properties' => '');
        $fields['TREECODE'] = array('type' => 'varchar(16)', 'null' => 'NO', 'properties' => '');
        $fields['LEVELS'] = array('type' => 'tinyint(3) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['PARENT'] = array('type' => 'varchar(255)', 'null' => 'YES', 'properties' => '');
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'UNI', 'fieldname' => 'IDCODE,TREECODE');
        $indexes[$ipf . '03'] = array('index' => 'MUL', 'fieldname' => 'PARENT');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB',
		  'fields' => $fields,
		  'indexes' => $indexes);
    }

    function action($ac, $idlist) {
        if ($this->getAccess()->canRead()) {
		  switch ($ac) {
			 case 'DISPLAYALL':
			     $contents = $this->displayForm($idlist[CLX_NUM_ZERO]);
			     break;
			 default:
			     $contents = parent::action($ac, $idlist);
		  }
        } else {
		  $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [AC_TREE_EDITOR-CAN_READ][' . $ac . '] ' . $this->getCode(false));
        }
        return $contents;
    }

    function makelist($listingcode = null) {
        $treecode = UserRequest::$request['treecode'];
        $level = UserRequest::$request['level'];
        $parent = '';
        $treerec = ClxCommon::$db->getRecord('CLX_AC_TREES', 'IDCODE=' . singleQuote($treecode));
        $levelrec = ClxCommon::$db->getRecord('CLX_AC_TREE_LEVELS', 'TREECODE=' . singleQuote($treecode) . ' AND LEVELS=' . $level);
        $t = time();
        $to = $this->code;
        $here = 'tool.php?to=' . $to . '&t=' . $t;
        clearHistory();
        pushHistory($this->code, 'LIST', array(), 'treecode', $treecode, 'level', $level, 'parent', $parent);
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->setProp(CLX_PROP_ID, 'ACTEList');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'NOT_DEFINED');
        $form->addArgument('treecode', $treecode);
        $form->addArgument('level', $level);
        $form->addArgument('parent', $parent);
        $selectclause = '*';
        $fromclause = 'CLX_AC_TREE_CONTENTS';
        $whereclause = 'TREECODE=' . singleQuote($treecode) . ' AND LEVELS=' . $level;
        $sortclause = 'IDCODE';
        $page = new xPaging();
        $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
        $table = new htmlTable(CLX_WIDTH_THREE);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));

        $table->addCell(dict('TOOL_ACTE_TITLE_LIST', $treerec['IDNAME']), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_THREE));
        $table->addCell(dict('TOOL_ACTE_PROMPT_LISTING', $levelrec['NAME_PLURAL']), array(CLX_PROP_CLASS => 'summary', CLX_PROP_COLSPAN => CLX_WIDTH_THREE));
        $page->header($table);
        $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_CODE'), dict('GEN_LBL_NAME')));
        $i = CLX_INIT_ZERO;
        while ($record = $page->nextRecord()) {
		  $recordid = $record['ID'];
		  $select = new htmlCheckbox('selector_' . $recordid, false);
		  $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
		  $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS));
		  $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($record['IDCODE'], $record['IDNAME']));
		  $i++;
        }

        $buttons = htmlButton::getButton('submit', dict('GEN_LBL_DISPLAY'), array('action' => 'DISPLAY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        if ($this->getAccess()->canModify()) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        if ($this->getAccess()->canDelete()) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        $b = new htmlFormSubmit(dict('TOOL_ACTE_BUTTON_DISPLAYALL'), 'DISPLAYALL', $form->getProp(CLX_PROP_ID));
        $b->setProp(CLX_PROP_ID, 'ONE');
        $buttons .= $b->source();
        if ($this->getAccess()->canDisplay()) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE_SOMETHING', $levelrec['NAME_SINGULAR']), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
        }
        $form->addContents($page->footers($table, $buttons));
        return $form->source();
    }

    function makepanel($treecode = null, $level = null, $parent = null) {
        if ($this->getAccess()->canRead()) {
		  //ClxFreeReturn( ClxCommon::$db->getRecord('CLX_AC_TREES', 'IDCODE=' . singleQuote($treecode)));
		  $levelrec = ClxCommon::$db->getRecord('CLX_AC_TREE_LEVELS', 'TREECODE=' . singleQuote($treecode) . ' AND LEVELS=' . $level);
		  $t = time();
		  $to = $this->code;
		  $here = 'tool.php?to=' . $to . '&t=' . $t;
		  $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
		  $form->setProp(CLX_PROP_ID, 'ACTEPanel');
		  $form->addArgument(CLX_PROP_TO, $this->code);
		  $form->addArgument('ac', 'NOT_DEFINED');
		  $form->addArgument('treecode', $treecode);
		  $form->addArgument('level', $level);
		  $form->addArgument('parent', $parent);
		  $selectclause = '*';
		  $fromclause = 'CLX_AC_TREE_CONTENTS';
		  $whereclause = 'TREECODE = ' . singleQuote($treecode) . ' AND LEVELS = ' . $level . ' AND PARENT = ' . singleQuote($parent);
		  $sortclause = 'IDCODE';
		  $page = new xPaging();
		  $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
		  $table = new htmlTable(CLX_WIDTH_THREE);
		  $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
		  $page->header($table);
		  $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_CODE'), dict('GEN_LBL_NAME')));
		  $i = CLX_INIT_ZERO;
		  while ($record = $page->nextRecord()) {
			 $recordid = $record['ID'];
			 $select = new htmlCheckbox('selector_' . $recordid, false);
			 $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
			 $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
			 $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($record['IDCODE'], $record['IDNAME']));
			 $i++;
		  }
		  $buttons = '';
		  $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_DISPLAY'), array('action' => 'DISPLAY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
		  if ($this->getAccess()->canModify()) {
			 $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
		  }
		  if ($this->getAccess()->canDelete()) {
			 $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
		  }
		  if ($this->getAccess()->canDisplay()) {
			 $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE_SOMETHING', $levelrec['NAME_SINGULAR']), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
		  }
		  $form->addContents($page->footers($table, $buttons));

		  return buildPanelInterface($levelrec['NAME_PLURAL'], $treecode . $level . $parent . 'SONS', $form->source(), CLX_NUM_MINUS_1);
        } else {
		  return CLX_CHAR_EMPTY_STRING;
        }
    }

    function editForm($ac, $idlist, $record, $msg) {
        $treecode = $record['ACTE_treecode'];
        $level = $record['ACTE_level'];


        $levelrec = ClxCommon::$db->getRecord('CLX_AC_TREE_LEVELS', 'TREECODE=' . singleQuote($treecode) . ' AND LEVELS=' . $level);
        $modify = (UserRequest::$request['ac'] === 'MODIFY');

        $title = ($modify) ? dict('TOOL_ACTE_TITLE_MODIFY', $levelrec['NAME_SINGULAR']) : dict('TOOL_ACTE_TITLE_CREATE', $levelrec['NAME_SINGULAR']);
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', $ac);
        $form->addArgument('confirm', '1');
        $form->addArgument('idlist', toString($idlist));
        $form->addArgument('ACTE_id', $record['ACTE_id']);
        $form->addArgument('ACTE_treecode', $record['ACTE_treecode']);
        $form->addArgument('ACTE_level', $record['ACTE_level']);
        $form->addArgument('ACTE_parent', $record['ACTE_parent']);
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell($title, array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));

        $table->addField($msg, dict('GEN_LBL_CODE') . ':', 'ACTE_idcode', $record['ACTE_idcode'], strlen($levelrec['FORMAT']), strlen($levelrec['FORMAT']));
        $table->addField($msg, dict('GEN_LBL_NAME') . ':', 'ACTE_idname', $record['ACTE_idname'], CLX_NUM_48, CLX_NUM_64);
        $form->addContents($table->source());
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $buttons = '';
        if (!empty($modify)) {
		  $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_SAVE_SOMETHING', $levelrec['NAME_SINGULAR']), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        } else {
		  $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_CREATE_SOMETHING', $levelrec['NAME_SINGULAR']), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        }
        $c = new htmlButtonCancel();
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->addContents($table->source());
        return $form->source();
    }

    function checkForm(&$msg) {
        $msg = array();
        $treecode = UserRequest::$post['ACTE_treecode'];
        $level = UserRequest::$post['ACTE_level'];
        $parent = UserRequest::$post['ACTE_parent'];
        $levelrec = ClxCommon::$db->getRecord('CLX_AC_TREE_LEVELS', 'UPPER(TREECODE)=' . singleQuote(uppercase($treecode)) . ' AND LEVELS=' . $level);
        $format = $levelrec['FORMAT'];
        $code = $this->getFullCode($treecode, $parent) . UserRequest::$post['ACTE_idcode'];
		//comentado para crear registro no lo permite crear
//  xValidate::unique($msg, 'ACTE_idcode', 'code', $this->tablename, 'UPPER(TREECODE)=' . singleQuote(uppercase($treecode)), 'IDCODE=' . singleQuote($code), 'ID <> ' . UserRequest::$post['ACTE_id']);
		//pre($msg);
		//echo"-----";
		//xValidate::formated($msg, 'ACTE_idcode', UserRequest::$post['ACTE_idcode'], $format);
		//pre($msg);
        if (empty(UserRequest::$post['ACTE_idname'])) {
		  $msg['ACTE_idname'] = 'You have to write a name here.';
        }
        return count($msg) == CLX_NUM_ZERO;
    }

    function confirmDeleteFill(&$table, $record) {
        $table->addData(dict('GEN_LBL_CODE') . ':', $record['ACTE_idcode']);
        $table->addData(dict('GEN_LBL_NAME') . ':', $record['ACTE_idname']);
    }

    function viewFormFill($record) {
        $treecode = $record['ACTE_treecode'];
        $level = $record['ACTE_level'];
        $parent = $record['ACTE_parent'];
        addHistory('treecode', $treecode);
        addHistory('treenode', $record['ACTE_idfullcode']);
        addHistory('level', $level);
        addHistory('parent', $parent);
//        ClxFreeReturn( ClxCommon::$db->getRecord('CLX_AC_TREES', 'IDCODE=' . singleQuote($treecode)));
        $levelrec = ClxCommon::$db->getRecord('CLX_AC_TREE_LEVELS', 'UPPER(TREECODE)=' . singleQuote(uppercase($treecode)) . ' AND LEVELS=' . $level);
        $contents = '';
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell($levelrec['NAME_SINGULAR'] . ': ' . $record['ACTE_idname'], array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addData(dict('GEN_LBL_CODE') . ':', $record['ACTE_idfullcode']);
        $table->addData(dict('GEN_LBL_NAME') . ':', $record['ACTE_idname']);
        $table->addData(dict('GEN_LBL_LEVEL') . ':', $record['ACTE_level']);
        $table->addData(dict('GEN_LBL_PATH') . ':', $this->getFullPath($treecode, $record['ACTE_idfullcode']));
        $contents .= $table->source();
        if (ClxCommon::$db->oneExistsRecords('CLX_AC_TREE_LEVELS', 'UPPER(TREECODE)=' . singleQuote(uppercase($treecode)) . ' AND LEVELS=' . ($level + CLX_NUM_ONE)) > CLX_NUM_ZERO) {
		  $contents .= $this->makepanel($treecode, $level + CLX_NUM_ONE, $record['ACTE_idfullcode']);
        }
        $ACTU = &getTool('ACTU');
        $contents .= $ACTU->makepanel($treecode, $record['ACTE_idfullcode']);
        return $contents;
    }

    function searchForm() {
        $treecode = UserRequest::$request['treecode'];
        $target = UserRequest::$request[CLX_PROP_TARGET];
        $msg = array();
        $contents = setJavaScriptTag(getJavaScriptFilename('ACTE_search.js'), true);
        if (!isset(UserRequest::$post['confirm'])) {
		  $script = setJavaScriptTag('getSearchValues(' . singleQuote($target) . ');', false);
        } else {
		  $script = '';
        }
        // Search Form
        $form = new htmlForm(CLX_PROP_ACTION, UserRequest::$server['PHP_SELF']);
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'SEARCH');
        $form->addArgument('confirm', '1');
        $form->addArgument('treecode', $treecode);
        $form->addArgument(CLX_PROP_TARGET, $target);
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_ACTE_TITLE_SEARCH'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addField($msg, dict('GEN_LBL_CODE') . ':', 'ACTE_idcode', (!empty(UserRequest::$post['ACTE_idcode'])) ? UserRequest::$post['ACTE_idcode'] : '', CLX_NUM_16, CLX_NUM_16);
        $table->addField($msg, dict('GEN_LBL_NAME') . ':', 'ACTE_idname', (!empty(UserRequest::$post['ACTE_idname'])) ? UserRequest::$post['ACTE_idname'] : '', CLX_NUM_32, CLX_NUM_64);
        $buttons = htmlButton::getButton('submit', dict('GEN_LBL_SEARCH'), array('action' => 'SEARCH', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        $c = new htmlButtonCloseWindow(dict('GEN_LBL_CANCEL'));
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->setContents($table->source());
        $contents .= $form->source();
        // Search Procedure
        if (isset(UserRequest::$post['confirm'])) {
		  $table = new htmlTable(CLX_WIDTH_THREE);
		  $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
		  $criteria = 'TREECODE=' . singleQuote($treecode);
		  $boolean = ' AND ';
		  if (UserRequest::$post['ACTE_idcode'] <> '') {
			 $criteria .= $boolean . 'UPPER(IDCODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['ACTE_idcode']) . '%');
			 $boolean = ' AND ';
		  }
		  if (UserRequest::$post['ACTE_idname'] <> '') {
			 $criteria .= $boolean . 'UPPER(IDNAME) LIKE ' . singleQuote(uppercase(clx_change_especial_char_html (UserRequest::$post['ACTE_idname'])) . '%');
			 $boolean = ' AND ';
		  }
            $qty = ClxCommon::$db->oneExistsRecords($this->tablename, $criteria);
		  if ($qty == CLX_NUM_ZERO) {
			 $table->addCell(dict('TOOL_GEN_ERR_NOMATCH'), array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_THREE));
		  } else {
			 $treelevels = array();
			 $records = ClxCommon::$db->getRecords('CLX_AC_TREE_LEVELS', 'UPPER(TREECODE)=' . singleQuote(uppercase($treecode)) . ' ORDER BY LEVELS');
			 while (($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE))) {
			     $treelevels[] = $record['NAME_SINGULAR'];
			 }
			 $table->addCell(dict('TOOL_UIFM_MSG_FOUNDRECORDS'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_THREE));
			 $records = ClxCommon::$db->getRecords($this->tablename, $criteria);
			 $i = CLX_INIT_ZERO;
			 while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
			     $js = 'javascript:setSearchValues(' . singleQuote($target) . ', ' . singleQuote($record['IDCODE']) . ',' . singleQuote($target . '_name') . ', ' . singleQuote($record['IDNAME']) . ');';
			     $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array('<page t=ac_tree_editor></page><a href=' . quote($js) . '>' . $record['IDCODE'] . '</a>', $record['IDNAME'], $treelevels[$record['LEVELS'] - CLX_NUM_ONE]));
			     $i++;
			 }
			 ClxCommon::$db->freeResult($records);
		  }
		  $contents .= $table->source();
        }
        return $contents . $script;
    }

    function displayForm($id) {
        pushHistory($this->code, 'DISPLAYALL', array($id));
        $rootrec = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $id);
        $treecode = $rootrec['TREECODE'];
        $treerec = ClxCommon::$db->getRecord('CLX_AC_TREES', 'IDCODE=' . singleQuote($treecode));
        $format = $treerec['FORMAT'];
        $tableWidth = substr_count($format, '.') + CLX_NUM_ONE;
        $table = new htmlTable($tableWidth);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell($treerec['IDNAME'], array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => $tableWidth));
        $records = ClxCommon::$db->getRecords('CLX_AC_TREE_LEVELS', 'UPPER(TREECODE)=' . singleQuote(uppercase($treecode)) . ' ORDER BY LEVELS');
        $levels = array();
        while (($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE))) {
		  $levels[] = $record;
		  $table->addCell($record['NAME_PLURAL'], array(CLX_PROP_CLASS => CLX_CLASS_HEADERS));
        }
		ClxCommon::$db->freeResult($records);
        $this->dump($table, CLX_NUM_ZERO, $tableWidth, $rootrec, true);
        $buttons = '';
        $c = new htmlButtonReturn();
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => $tableWidth));
        return $table->source();
    }

    function dump(&$table, $level, $tableWidth, $record, $first) {
        if (empty($first)) {
		  for ($i = CLX_NUM_ZERO; $i < $level; $i++) {
			 $table->addCell(CLX_CHAR_HTML_NBSP, array());
		  }
        }
        $url = 'tool.php?to=' . $this->code . '&ac=DISPLAY&idlist=' . $record['ID'];
        $link = '<a href=' . singleQuote($url) . '><b>' . $record['IDNAME'] . '</b></a>';
        $info = $link . '<br>' . $record['IDCODE'];
        $users = ClxCommon::$db->getRecords('CLX_AC_TREE_USERS', 'TREENODE=' . singleQuote($record['IDCODE']), 'UPPER(TREECODE)=' . singleQuote(uppercase($treecode)));
        while (($user = ClxCommon::$db->getNextRecord($users))) {
		  $info .= '<br>> ' . $user['USERNAME'];
		  if (!empty($user['DEFAULT_ENTRY']))
			 $info .= ' (Def)';
        }
		ClxCommon::$db->freeResult($users);
        $table->addCell($info, array(CLX_PROP_CLASS => CLX_CLASS_DATA));
        $sonqty = ClxCommon::$db->oneExistsRecords($this->tablename, 'UPPER(PARENT)=' . singleQuote(uppercase($record['IDCODE'])) . ' AND UPPER(TREECODE)=' . singleQuote(uppercase($treecode)));
        if ($sonqty > CLX_NUM_ZERO) {
		  $sons = ClxCommon::$db->getRecords($this->tablename, 'UPPER(PARENT)=' . singleQuote(uppercase($record['IDCODE'])) . ' AND UPPER(TREECODE)=' . singleQuote(uppercase($treecode)));
		  $first = true;
		  while (($son = ClxCommon::$db->getNextRecord($sons))) {
			 $this->dump($table, $level + CLX_NUM_ONE, $tableWidth, $son, $first);
			 $first = false;
		  }
		  ClxCommon::$db->freeResult($sons);
        } else {
		  for ($i = $level + CLX_NUM_ONE; $i < $tableWidth; $i++) {
			 $table->addCell(CLX_CHAR_HTML_NBSP, array());
		  }
        }
    }

    function loadToolRecord($recordid) {
        $values = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $recordid);
        $pos = strrpos($values['IDCODE'], '.');
        if (false === $pos) {
		  $code = $values['IDCODE'];
        } else {
		  $code = substr($values['IDCODE'], $pos + CLX_NUM_ONE);
        }
        $record = array();
        $record['ACTE_id'] = $recordid;
        $record['ACTE_idfullcode'] = $values['IDCODE'];
        $record['ACTE_idcode'] = $code;
        $record['ACTE_idname'] = clx_html_spchrs_decode($values['IDNAME']);
        $record['ACTE_treecode'] = $values['TREECODE'];
        $record['ACTE_level'] = $values['LEVELS'];
        $record['ACTE_parent'] = $values['PARENT'];
        return $record;
    }

    function saveRecord() {
        $code = $this->getFullCode(UserRequest::$post['ACTE_treecode'], UserRequest::$post['ACTE_parent']) . UserRequest::$post['ACTE_idcode'];
        $record = array();
        $record['IDCODE'] = $code;
        $record['IDNAME'] = clx_html_spchrs_decode(UserRequest::$post['ACTE_idname']);
        ClxCommon::$db->updateRecord($this->tablename, $record, 'ID=' . UserRequest::$post['ACTE_id']);
    }

    function createRecord_() {
        $code = $this->getFullCode(UserRequest::$post['ACTE_treecode'], UserRequest::$post['ACTE_parent']) . UserRequest::$post['ACTE_idcode'];
        $record = array();
        $record['IDCODE'] = $code;
        $record['IDNAME'] = clx_html_spchrs_decode(UserRequest::$post['ACTE_idname']);
        $record['TREECODE'] = UserRequest::$post['ACTE_treecode'];
        $record['LEVELS'] = UserRequest::$post['ACTE_level'];
        $record['PARENT'] = UserRequest::$post['ACTE_parent'];
        $recordid = ClxCommon::$db->createRecord($this->tablename, $record, false);
        return $recordid;
    }

    function deleteRecordSk($id) {
        $record = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $id);
        $sons = ClxCommon::$db->getRecords($this->tablename, 'PARENT=' . singleQuote($record['IDCODE']));
        while (($sonrec = ClxCommon::$db->getNextRecord($sons))) {
		  $this->deleteRecordSk($sonrec['ID']);
        }
		ClxCommon::$db->freeResult($sons);
        ClxCommon::$db->deleteRecords($this->tablename, 'ID=' . $id);
    }

    function defaultRecord() {
        $values = ClxCommon::$db->getRecord('CLX_AC_TREE_LEVELS', 'UPPER(TREECODE)=' . singleQuote(uppercase(UserRequest::$request['treecode'])) . ' AND LEVELS=' . UserRequest::$request['level']);
        $record = array('ACTE_id' => 0,
		  'ACTE_idcode' => $values['FORMAT'],
		  'ACTE_idname' => '',
		  'ACTE_treecode' => UserRequest::$request['treecode'],
		  'ACTE_level' => UserRequest::$request['level'],
		  'ACTE_parent' => UserRequest::$request['parent']);
        return $record;
    }

    function getFullCode($treecode, $parent) {
        if (strlen($parent) > CLX_NUM_ZERO) {
		  $record = ClxCommon::$db->getRecord('CLX_AC_TREE_CONTENTS', 'UPPER(TREECODE)=' . singleQuote(uppercase($treecode)) . ' AND UPPER(IDCODE)=' . singleQuote(uppercase($parent)));
		  $code = $record['IDCODE'] . '.';
        } else {
		  $code = '';
        }
        return $code;
    }

    function getFullPath($treecode, $parent) {
        $path = '';
        if (strlen($parent) > CLX_NUM_ZERO) {
		  $rec = ClxCommon::$db->getRecord($this->tablename, 'UPPER(TREECODE)=' . singleQuote(uppercase($treecode)) .  ' AND UPPER(IDCODE)=' . singleQuote(uppercase($parent)));
		  $path = $this->getFullPath($treecode, $rec['PARENT']) . '/' . clx_html_spchrs_decode($rec['IDNAME']);
        }
        return $path;
    }

}
