<?php

/**
 * Tool Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: alenetdev $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2019-01-22 15:23:02 -0400 (ma., 22 ene. 2019) $
 * @version        $Rev: 10488 $
 *
 * CHANGE LOG
 *  Included label
 */
class ac_user_sessions extends clx_tool {

    // CONSTRUCTOR //
    function __construct() {
        $this->code = 'ACUE';
        $this->name = 'User Session Manager';
        $this->system = 'Security';
        $this->version = '0.1.2.0';

        $this->has_access = true;


        $this->tablename = 'CLX_AC_USER_SESSIONS';
        $this->record_name_singular = 'user session';
        $this->record_name_plural = 'user sessions';
        $this->primary_field = 'ID';
        $this->secondary_field = 'SESSION_KEY';
    }

    function install() {
        $UIMI = getTool('UIMI');
        $UIMI->add($this->code, 'MENU_TOOLS:MENU_SECURITY:MENU_SESSIONS', 'TOOL', $this->code, 'ac=LIST');
    }

    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['SESSION_KEY'] = array('type' => 'varchar(255)', 'null' => 'NO', 'properties' => '');
        $fields['SESSION_IP'] = array('type' => 'varchar(32)', 'null' => 'NO', 'properties' => '');
        $fields['SESSION_DATE'] = array('type' => 'datetime', 'null' => 'YES', 'properties' => '');
        //$fields['SESSION_DATA'] = array('type' => 'mediumblob', 'null' => 'NO', 'properties' => '');
        $fields['SESSION_DATA'] = array('type' => 'mediumblob', 'null' => 'YES', 'properties' => '');
        $fields['CDATE'] = array('type' => 'datetime', 'null' => 'YES', 'properties' => '');
        $fields['XDATE'] = array('type' => 'datetime', 'null' => 'YES', 'properties' => '');
        $fields['USERNAME'] = array('type' => 'varchar(30)', 'null' => 'YES', 'properties' => '');
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'UNI', 'fieldname' => 'SESSION_KEY');
        $indexes[$ipf . '03'] = array('index' => 'MUL', 'fieldname' => 'SESSION_IP');
        $indexes[$ipf . '04'] = array('index' => 'MUL', 'fieldname' => 'USERNAME');
        $indexes[$ipf . '05'] = array('index' => 'MUL', 'fieldname' => 'XDATE');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'MyISAM', 'fields' => $fields, 'indexes' => $indexes);
    }

    function makelist($listingcode = null) {
        //$t = time('U'); genera un warning
        $t = time();
        $to = $this->code;
        $here = 'tool.php?to=' . $to . '&t=' . $t;
        clearHistory();
        pushHistory($this->code, 'LIST', array());
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->setProp(CLX_PROP_ID, 'ACUEList');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'NOT_DEFINED');
        $selectclause = '*';
        $fromclause = $this->tablename;
        $whereclause = '';
        $sortclause = 'CDATE';
        $page = new xPaging();
        $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
        $table = new htmlTable(CLX_WIDTH_SIX);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_ACUE_TITLE_LIST'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_SIX));
        $page->header($table);
        $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_USERNAME'), dict('GEN_LBL_CDATE'), dict('GEN_LBL_XDATE'), dict('GEN_LBL_IP'),dict('GEN_LBL_KEY')));
        $i = CLX_INIT_ZERO;
        while ($record = $page->nextRecord()) {
		  $recordid = $record['ID'];
		  $select = new htmlCheckbox('selector_' . $recordid, false);
		  $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
		  $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
		  $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($record['USERNAME'], xFormat::datetime($record['CDATE']), xFormat::datetime($record['XDATE']), $record['SESSION_IP'], $record['SESSION_KEY']));
		  $i++;
        }
        $buttons = '';
        $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_DISPLAY'), array('action' => 'DISPLAY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        if ($this->getAccess()->canDelete()) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_FILTER'), array('action' => 'FILTER', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
        $form->addContents($page->footers($table, $buttons));
        return $form->source();
    }

    function viewFormFill($record) {
        $contents = '';
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_ACUE_TITLE_VIEW'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addData(dict('GEN_LBL_KEY') . ':', $record['ACUE_session_key']);
        $table->addData(dict('GEN_LBL_IP') . ':', $record['ACUE_session_ip']);
        $table->addData(dict('GEN_LBL_CDATE') . ':', xFormat::datetime($record['ACUE_cdate']));
        $table->addData(dict('GEN_LBL_XDATE') . ':', xFormat::datetime($record['ACUE_xdate']));
        $table->addData(dict('GEN_LBL_USERNAME') . ':', $record['ACUE_username']);
        $contents .= $table->source();
        return $contents;
    }

    function filterFormFill($msg, &$table) {
        if (!array_key_exists('ACUE_session_ip', UserRequest::$post))
		  UserRequest::$post['ACUE_session_ip'] = '';
        if (!array_key_exists('ACUE_username', UserRequest::$post))
		  UserRequest::$post['ACUE_username'] = '';
        $table->addField($msg, dict('GEN_LBL_IP'), 'ACUE_session_ip', UserRequest::$post['ACUE_session_ip'], CLX_NUM_24, CLX_NUM_48);
        $table->addField($msg, dict('GEN_LBL_USERNAME'), 'ACUE_username', UserRequest::$post['ACUE_username'], CLX_NUM_30, CLX_NUM_64);
        $sort = new HtmlSelect('ACUE_sort');
        $sort->addItem('', '', true);
        $sort->addItem(dict('GEN_LBL_IP'), 'session_ip', false);
        $sort->addItem(dict('GEN_LBL_USERNAME'), 'session_username', false);
        $sort->addItem(dict('GEN_LBL_CDATE'), 'cdate', false);
        $sort->addItem(dict('GEN_LBL_XDATE'), 'xdate', false);
        $table->addData(dict('GEN_LBL_ORDERBY') . ':', $sort->source());
    }

    function setfilter() {
        $filter = '';
        $boolean = '';
        if (!empty(UserRequest::$post['ACUE_session_ip'])) {
		  $filter .= $boolean . ' SESSION_IP LIKE ' . singleQuote(UserRequest::$post['ACUE_session_ip'] . '%');
		  $boolean = 'AND';
        }
        if (!empty(UserRequest::$post['ACUE_username'])) {
		  $filter .= $boolean . ' UPPER(USERNAME) LIKE ' . singleQuote(uppercase(clx_change_especial_char_html (UserRequest::$post['ACUE_username'])) . '%');
		  $boolean = 'AND';
        }
        $_SESSION['paging']['ACUEList']['sortby'] = UserRequest::$post['ACUE_sort'];
        $_SESSION['paging']['ACUEList']['criteria'] = $filter;
        $_SESSION['paging']['ACUEList']['startat'] = '0';
    }
	///////////////////////////////////////////////////
	///comentado 22/01/2019 para utilizar la clx_tool
	//////////////////////////////////////////////////
    /*function clearfilter() {
        $_SESSION['paging']['ACUEList']['criteria'] = '';
    }*/

    function confirmDeleteFill(&$table, $record) {
        $table->addData(dict('GEN_LBL_IP') . ':', $record['ACUE_session_ip']);
        $table->addData(dict('GEN_LBL_USERNAME') . ':', $record['ACUE_username']);
    }

    function loadToolRecord($recordid) {
        $values = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $recordid);
        $record = array();
        $record['ACUE_id'] = $recordid;
        $record['ACUE_session_key'] = $values['SESSION_KEY'];
        $record['ACUE_session_ip'] = $values['SESSION_IP'];
        $record['ACUE_cdate'] = $values['CDATE'];
        $record['ACUE_xdate'] = $values['XDATE'];
        $record['ACUE_session_data'] = $values['SESSION_DATA'];
        $record['ACUE_username'] = $values['USERNAME'];
        return $record;
    }

    function deleteRecordSk($id) {
        ClxCommon::$db->deleteRecords($this->tablename, 'ID=' . $id);
    }

}
