<?php

/**
 * Tool Class
 * For more information contact  support@alenet.com
 *
 * @package		Clientix.Tool.Interface
 * @abstract		System Chart Manager
 * @author       $Author: cramirez $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2018-10-15 20:57:54 -0400 (Mon, 15 Oct 2018) $
 * @version        $Rev: 10266 $
 *
 * CHANGE LOG
 *  Included label
 */
class an_chart_series extends clx_tool {

    // CONSTRUCTOR //
    function __construct() {
        $this->code = 'ANCS';
        $this->name = 'Chart Series';
        $this->system = 'Analytics';
        $this->version = '0.2.4.1';

        $this->accesskey = 'ANCH';


        $this->tablename = 'CLX_AN_CHART_SERIES';
        $this->record_name_singular = 'chart serie';
        $this->record_name_plural = 'chart series';
        $this->primary_field = 'NAME';
        $this->secondary_field = 'CHARTCODE';
        $this->seqnum_field = 'SEQNUM';
    }

    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI', 'index' => 'PRI');
        $fields['CHARTCODE'] = array('type' => 'varchar(16)', 'null' => 'NO', 'properties' => '');
        $fields['NAME'] = array('type' => 'varchar(64)', 'null' => 'YES', 'properties' => '');
        $fields['TYPE'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '');
        $fields['TABLECODE'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '');
        $fields['FIELDNAME'] = array('type' => 'varchar(64)', 'null' => 'NO', 'properties' => '');
        $fields['FUNCTION'] = array('type' => 'varchar(32)', 'null' => 'YES', 'properties' => '');
        $fields['GROUPBY'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['STATUS'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['SEQNUM'] = array('type' => 'tinyint(4) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['SERIES_PROPERTIES'] = array('type' => 'mediumtext', 'null' => 'YES', 'properties' => '');
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'MUL', 'fieldname' => 'CHARTCODE');
        $indexes[$ipf . '03'] = array('index' => 'MUL', 'fieldname' => 'STATUS');
        $indexes[$ipf . '04'] = array('index' => 'MUL', 'fieldname' => 'SEQNUM');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB',
		  'fields' => $fields,
		  'indexes' => $indexes);
    }

    function makepanel($groupcode = null) {

        if ($this->getAccess()->canRead()) {
		  $t = time();
		  $to = $this->code;
		  $here = 'tool.php?to=' . $to . '&t=' . $t;
		  $yesno = array(dict('GEN_LBL_NO'), dict('GEN_LBL_YES'));
		  $status = array(dict('GEN_LBL_INACTIVE'), dict('GEN_LBL_ACTIVE'));
		  $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
		  $form->setProp(CLX_PROP_ID, 'ANCSPanel');
		  $form->addArgument(CLX_PROP_TO, $this->code);
		  $form->addArgument('CHARTCODE', $groupcode);
		  $form->addArgument('ac', 'NOT_DEFINED');
		  $selectclause = 'CONCAT(T.MODULECODE, CONCAT(' . singleQuote('_') . ', CONCAT( T.IDNAME, CONCAT(' . singleQuote('.') . ', C.FIELDNAME))))  FULLFIELDNAME, C.*';
		  $fromclause = 'CLX_AN_CHART_SERIES C, CLX_DB_TABLES T';
		  $whereclause = 'C.TABLECODE = T.IDCODE AND C.CHARTCODE=' . singleQuote($groupcode);
		  $sortclause = 'SEQNUM';
		  $page = new xPaging();
		  $page->setProp('rpp', CLX_NUM_TEN_THOUSAND);
		  $page->setProp('printheaders', false);
		  $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
		  $table = new htmlTable(CLX_WIDTH_7);
		  $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
		  $page->header($table);
		  $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_NAME'), dict('GEN_LBL_FIELDNAME'), dict('GEN_LBL_FUNCTION'), dict('GEN_LBL_TYPE'), dict('GEN_LBL_GROUP'), dict('GEN_LBL_ACTIVE')));
		  $i = CLX_INIT_ZERO;
		  while ($record = $page->nextRecord()) {
			 $recordid = $record['ID'];
			 $select = new htmlCheckbox('selector_' . $recordid, false);
			 $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
			 $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
			 $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($record['NAME'], $record['FULLFIELDNAME'], $record['FUNCTION'], $record['TYPE'], $yesno[$record['GROUPBY']], $status[$record['STATUS']]));
			 $i++;
		  }

		  $buttons = htmlButton::getButton('submit', dict('GEN_LBL_DISPLAY'), array('action' => 'DISPLAY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
		  if ($this->getAccess()->canModify()) {
			 $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
		  }
		  if ($this->getAccess()->canDelete()) {
			 $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
		  }
		  if ($this->getAccess()->canDisplay()) {
			 $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE_SOMETHING', dict('GEN_LBL_SERIES')), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
		  }
		  if ($page->getProp('qty') > CLX_NUM_ONE) {
			 $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_REORDER'), array('action' => 'REORDER', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
		  }
		  $form->addContents($page->footers($table, $buttons));
		  return buildPanelInterface(dict('TOOL_ANCS_TITLE_LIST'), 'TABLES', $form->source(), CLX_NUM_MINUS_1);
        } else {
		  return CLX_CHAR_EMPTY_STRING;
        }
    }

    function editForm($ac, $idlist, $record, $msg) {
        $chartcode = $record['ANCS_chartcode'];
        $chartrec = ClxCommon::$db->getRecord('CLX_AN_CHARTS', 'IDCODE=' . singleQuote($chartcode));
        // Build/get the different java scripts...

        $OBJS = &getTool('OBJS');
        $chartseries = $OBJS->objectList($chartrec['TYPE']);
        $list = 'var panellist = new Array();' . CLX_CHAR_NL;
        $i = CLX_INIT_ZERO;
        foreach ($chartseries as $chartplot) {
		  $list .= sprintf('panellist[%s] = ' . singleQuote('%s') . ';', $i, $chartplot) . CLX_CHAR_NL;
		  $i++;
        }

        $script = setJavaScriptTag($list, false);
        $topscript = setJavaScriptTag(getJavaScript('dynform_top.js', array('VISIBLEPANEL' => 'ANCS_type')), false);
        $topscript .= setJavaScriptTag(getJavaScriptFilename('ANCS_editform.js'), true);
        $btmscript = setJavaScriptTag(getJavaScriptFilename('dynform_btm.js'), true);


        $btmscript .= setJavaScriptTag('initForm();', false);
        $modify = (UserRequest::$request['ac'] === 'MODIFY');
        $title = ($modify) ? dict('TOOL_ANCS_TITLE_MODIFY') : dict('TOOL_ANCS_TITLE_CREATE');
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', $ac);
        $form->addArgument('confirm', '1');
        $form->addArgument('ANCS_id', $record['ANCS_id']);
        $form->addArgument('ANCS_chartcode', $record['ANCS_chartcode']);
        $form->addArgument('idlist', toString($idlist));
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell($title, array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addField($msg, dict('GEN_LBL_NAME') . ':', 'ANCS_name', $record['ANCS_name'], CLX_NUM_48, CLX_NUM_64);
        // Fieldname and Tablecode
        $sql = 'SELECT CONCAT(T.MODULECODE, CONCAT(' . singleQuote('_') . ', CONCAT(T.IDNAME, CONCAT(' . singleQuote('.') . ',TB.FIELDNAME))))  FIELDNAME, CONCAT(QT.TABLECODE, CONCAT(' . singleQuote('.') . ', TB.FIELDNAME))  FIELDCODE FROM CLX_AN_CHARTS  C, CLX_DB_QUERY_TABLES  QT, CLX_DB_TABLEFIELDS  TB, CLX_DB_TABLES  T WHERE C.IDCODE = ' . singleQuote($record['ANCS_chartcode']) . ' AND QT.QUERYCODE = C.QUERYCODE AND QT.TABLECODE = T.IDCODE AND TB.TABLECODE = T.IDCODE ORDER BY QT.TABLECODE ASC, TB.SEQNUM ASC';
        $fields = ClxCommon::$db->executeQuery($sql);
        $pop = new htmlSelect('ANCS_tablecode_fieldname');
        while (($field = ClxCommon::$db->getNextRecord($fields))) {
		  $pop->addItem($field['FIELDNAME'], $field['FIELDCODE'], ($field['FIELDCODE'] == $record['ANCS_tablecode'] . '.' . $record['ANCS_fieldname']));
        }
        $table->addData(dict('GEN_LBL_FIELDNAME') . ':', $pop->source());
        $pop = new htmlSelect('ANCS_function');
        $pop->addItem('', '', ($record['ANCS_function'] === ''));
        $pop->addItem('Count', 'COUNT', ($record['ANCS_function'] === 'COUNT'));
        $pop->addItem('Average', 'AVG', ($record['ANCS_function'] === 'AVG'));
        $pop->addItem('Minimum Value', 'MIN', ($record['ANCS_function'] === 'MIN'));
        $pop->addItem('Maximum Value', 'MAX', ($record['ANCS_function'] === 'COMAXUNT'));
        $pop->addItem('Sum', 'SUM', ($record['ANCS_function'] === 'SUM'));
        $pop->addItem('Standard Variance ', 'VARIANCE', ($record['ANCS_function'] === 'VARIANCE'));
        $pop->addItem('Standard Deviation ', 'STD', ($record['ANCS_function'] === 'STD'));
        $pop->addItem('Bitwise Or', 'BIT_OR', ($record['ANCS_function'] === 'BIT_OR'));
        $pop->addItem('Bitwise And', 'BIT_AND', ($record['ANCS_function'] === 'BIT_AND'));
        $table->addData(dict('GEN_LBL_FUNCTION') . ':', $pop->source());
        // Group
        $pop = new htmlSelect('ANCS_groupby');
        $pop->addItem(dict('GEN_LBL_YES'), '1', ($record['ANCS_groupby'] === '1'));
        $pop->addItem(dict('GEN_LBL_NO'), '0', ($record['ANCS_groupby'] === '0'));
        $table->addData(dict('GEN_LBL_GROUP') . ':', $pop->source());
        // Status
        $pop = new htmlSelect('ANCS_status');
        $pop->addItem(dict('GEN_LBL_ACTIVE'), '1', ($record['ANCS_status'] === '1'));
        $pop->addItem(dict('GEN_LBL_INACTIVE'), '0', ($record['ANCS_status'] === '0'));
        $table->addData(dict('GEN_LBL_STATUS') . ':', $pop->source());
        // Series type
        $types = $OBJS->objectPop($chartrec['TYPE'], 'ANCS_type', $record['ANCS_type']);
        $types->setProp('onchange', 'updateForm();');
        $table->addData(dict('GEN_LBL_TYPE'), $types->source());
        $form->addContents($table->source());
        foreach ($chartseries as $series) {
		  $o = getToolObject($chartrec['TYPE'], $series);
		  $form->addContents($o->panel($msg, $record));
        }
        $buttons = '';
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        if (!empty($modify)) {
		  $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_SAVE'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        } else {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE'), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        }
        $c = new htmlButtonCancel();
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->addContents($table->source());
        return $script . $topscript . $form->source() . $btmscript;
    }

    function checkForm(&$msg) {
        $msg = array();
        xValidate::required($msg, 'ANCS_name', UserRequest::$post['ANCS_name']);
        $charttype = ClxCommon::$db->getField('CLX_AN_CHARTS', 'type', 'IDCODE=' . singleQuote(UserRequest::$post['ANCS_chartcode']));
        $o = getToolObject($charttype, UserRequest::$post['ANCS_type']);
        $o->checkPanel($msg);
        return count($msg) == CLX_NUM_ZERO;
    }

    function confirmDeleteFill(&$table, $record) {
        $table->addData(dict('GEN_LBL_NAME') . ':', $record['ANCS_name']);
    }

    function reorderRecords() {
        $records = ClxCommon::$db->getRecords($this->tablename, 'CHARTCODE=' . singleQuote(UserRequest::$post['chartcode']) . ' ORDER BY SEQNUM');
        $list = array();
        while (($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE))) {
		  $list[$record['ID']] = $record['NAME'];
        }
        return $list;
    }

    function reorderAction() {
        parent::reorderAction();
        $callingpage = peekHistory();
        $chartcode = $callingpage['chartcode'];
        $ANCH = &getTool('ANCH');
        $ANCH->versionIncrease('idcode', $chartcode);
    }

    function viewFormFill($record) {
        $series_records = array();
        $sql = '';
        $ANCH = &getTool('ANCH');
        $ANCH->chartInformation($record['ANCS_chartcode'], $series_records, $sql);
        $yesno = array(dict('GEN_LBL_NO'), dict('GEN_LBL_YES'));
        $status = array(dict('GEN_LBL_INACTIVE'), dict('GEN_LBL_ACTIVE'));
        $tablerec = ClxCommon::$db->getRecord('CLX_DB_TABLES', 'IDCODE=' . singleQuote($record['ANCS_tablecode']));
        $contents = '';
        // Usual Information
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_ANCS_TITLE_VIEW') . ': ' . $record['ANCS_name'], array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addData(dict('GEN_LBL_NAME') . ':', $record['ANCS_name']);
        $table->addData(dict('GEN_LBL_FIELDNAME') . ':', $tablerec['MODULECODE'] . '_' . $tablerec['IDNAME'] . '.' . $record['ANCS_fieldname']);
        $table->addData(dict('GEN_LBL_FUNCTION') . ':', $record['ANCS_function']);
        $table->addData(dict('GEN_LBL_GROUP') . ':', $yesno[$record['ANCS_groupby']]);
        $table->addData(dict('GEN_LBL_STATUS') . ':', $status[$record['ANCS_status']]);
        $nicesql = str_replace('SELECT', '<b>SELECT</b><br>', $sql);
        $nicesql = str_replace('FROM', '<br><b>FROM</b><br>', $nicesql);
        $nicesql = str_replace('WHERE', '<br><b>WHERE</b><br>', $nicesql);
        $nicesql = str_replace('GROUP BY', '<br><b>GROUP BY</b><br>', $nicesql);
        $table->addData(dict('GEN_LBL_SQL') . ':', $nicesql);
        $contents .= $table->source();
        // Series Information
        $records = ClxCommon::$db->executeQuery($sql);
        $qty = ClxCommon::$db->numFields($records);
        $tableWidth = $qty;
        $table = new htmlTable($tableWidth);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_ANCS_TITLE_SERIESDATA'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => $tableWidth));
        for ($i = CLX_NUM_ZERO; $i < $qty; $i++) {
		  $table->addCell($series_records[$i]['NAME'] . ' (' . $series_records[$i]['AS'] . ')', array(CLX_PROP_CLASS => CLX_CLASS_HEADERS));
        }
        $j = CLX_INIT_ZERO;
        while (($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE))) {
		  for ($i = CLX_NUM_ZERO; $i < $qty; $i++) {
			 $pos = strpos($series_records[$i]['AS'], '_');
			 $fieldname = substr($series_records[$i]['AS'], $pos + CLX_NUM_ONE);
			 $pos = strpos($fieldname, '_');
			 if ($pos !== false) {
			     $fieldname = substr($fieldname, CLX_NUM_ZERO, $pos);
			 }
			 $fieldrec = ClxCommon::$db->getRecord('CLX_DB_FIELDS', 'FIELDNAME=' . singleQuote($fieldname));
			 $fieldinstance = getToolObject('DBFT', $fieldrec['FIELDTYPE']);
			 $data = array();
			 $data[$fieldname] = $record[$series_records[$i]['AS']];
			 $value = $fieldinstance->formatField($fieldname, $fieldrec, $data);
			 $table->addCell($value, array(CLX_PROP_CLASS => ($j % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT));
		  }
		  $j++;
        }
        $contents .= $table->source();
        return $contents;
    }

    function loadToolRecord($recordid) {
        $values = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $recordid);
        $record = array();
        $record['ANCS_id'] = $recordid;
        $record['ANCS_chartcode'] = $values['CHARTCODE'];
        $record['ANCS_name'] = $values['NAME'];
        $record['ANCS_type'] = $values['TYPE'];
        $record['ANCS_tablecode'] = $values['TABLECODE'];
        $record['ANCS_fieldname'] = $values['FIELDNAME'];
        $record['ANCS_function'] = $values['FUNCTION'];
        $record['ANCS_groupby'] = $values['GROUPBY'];
        $record['ANCS_status'] = $values['STATUS'];
        $record['ANCS_seqnum'] = $values['SEQNUM'];
        $charttype = ClxCommon::$db->getField('CLX_AN_CHARTS', 'TYPE', 'IDCODE=' . singleQuote($record['ANCS_chartcode']));
        $o = getToolObject($charttype, $record['ANCS_type']);
        $o->loadPanel($record, $values);
        return $record;
    }

    function saveRecord() {
        $data = UserRequest::$post['ANCS_tablecode_fieldname'];
        $pos = strpos($data, '.');
        $record = array();
        $record['CHARTCODE'] = UserRequest::$post['ANCS_chartcode'];
        $record['NAME'] = UserRequest::$post['ANCS_name'];
        $record['TYPE'] = UserRequest::$post['ANCS_type'];
        $record['TABLECODE'] = substr($data, CLX_NUM_ZERO, $pos);
        $record['FIELDNAME'] = substr($data, $pos + CLX_NUM_ONE);
        $record['FUNCTION'] = UserRequest::$post['ANCS_function'];
        $record['GROUPBY'] = UserRequest::$post['ANCS_groupby'];
        $record['STATUS'] = UserRequest::$post['ANCS_status'];
        $charttype = ClxCommon::$db->getField('CLX_AN_CHARTS', 'TYPE', 'IDCODE=' . singleQuote($record['CHARTCODE']));
        $o = getToolObject($charttype, UserRequest::$post['ANCS_type']);
        $o->savePanel($record);
        ClxCommon::$db->updateRecord($this->tablename, $record, 'ID=' . UserRequest::$post['ANCS_id']);
        $ANCH = &getTool('ANCH');
        $ANCH->versionIncrease('IDCODE', UserRequest::$post['ANCS_chartcode']);
    }

    function createRecordVoid_() {
        ClxFreeReturn($this->createRecord_());
    }

    function createRecord_() {
        $data = UserRequest::$post['ANCS_tablecode_fieldname'];
        $pos = strpos($data, '.');
        $record = array();
        $record['CHARTCODE'] = UserRequest::$post['ANCS_chartcode'];
        $record['NAME'] = UserRequest::$post['ANCS_name'];
        $record['TYPE'] = UserRequest::$post['ANCS_type'];
        $record['TABLECODE'] = substr($data, CLX_NUM_ZERO, $pos);
        $record['FIELDNAME'] = substr($data, $pos + CLX_NUM_ONE);
        $record['FUNCTION'] = UserRequest::$post['ANCS_function'];
        $record['GROUPBY'] = UserRequest::$post['ANCS_groupby'];
        $record['STATUS'] = UserRequest::$post['ANCS_status'];
        $record['SEQNUM'] = ClxCommon::$db->getField($this->tablename, 'SEQNUM', 'CHARTCODE=' . singleQuote($record['CHARTCODE']) . ' ORDER BY SEQNUM DESC') + CLX_NUM_ONE;
        $charttype = ClxCommon::$db->getField('CLX_AN_CHARTS', 'TYPE', 'IDCODE=' . singleQuote($record['CHARTCODE']));
        $o = getToolObject($charttype, UserRequest::$post['ANCS_type']);
        $o->savePanel($record);
        $recordid = ClxCommon::$db->createRecord($this->tablename, $record, false);
        $ANCH = &getTool('ANCH');
        $ANCH->versionIncrease('idcode', UserRequest::$post['ANCS_chartcode']);
        return $recordid;
    }

    function deleteRecordSk($id) {
        $plotrec = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $id);
        ClxCommon::$db->deleteRecords($this->tablename, 'ID=' . $id);
        $ANCH = &getTool('ANCH');
        $ANCH->versionIncrease('IDCODE', $plotrec['CHARTCODE']);
    }

    function defaultRecord() {
        $record = array();
        $record['ANCS_id'] = CLX_INIT_ZERO;
        $record['ANCS_chartcode'] = UserRequest::$request['CHARTCODE'];
        $record['ANCS_name'] = '';
        $record['ANCS_type'] = '?';
        $record['ANCS_tablecode'] = '';
        $record['ANCS_fieldname'] = '';
        $record['ANCS_function'] = '';
        $record['ANCS_groupby'] = '0';
        $record['ANCS_status'] = '1';
        $record['ANCS_seqnum'] = '0';
        $charttype = ClxCommon::$db->getField('CLX_AN_CHARTS', 'TYPE', 'IDCODE=' . singleQuote($record['ANCS_chartcode']));
        $OBJS = &getTool('OBJS');
        $codes = $OBJS->objectList($charttype);
        foreach ($codes as $code) {
		  $o = $OBJS->getObject_($charttype, $code);
		  $o->defaultPanel($record);
        }
        return $record;
    }

}
