<?php

/**
 * Tool Class
 * For more information contact  support@alenet.com
 *
 * @package		Clientix.Tool.Interface
 * @abstract		System Chart Manager
 * @author       $Author: alenetdev $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2019-01-22 15:21:45 -0400 (ma., 22 ene. 2019) $
 * @version        $Rev: 10486 $
 *
 * CHANGE LOG
 *  Included label
 */
class an_charts extends clx_tool {

    var $chart_types;

    // CONSTRUCTOR //
    function __construct() {
        $this->code = 'ANCH';
        $this->name = 'Charts Manager';
        $this->system = 'Analytics';
        $this->version = '0.2.3.0';

        $this->has_access = true;


        $this->tablename = 'CLX_AN_CHARTS';
        $this->record_name_singular = 'chart';
        $this->record_name_plural = 'charts';
        $this->primary_field = 'IDCODE';
        $this->secondary_field = 'IDNAME';

        $this->chart_types = array('CPTY' => 'Bar and Line', 'CPTB' => 'Grouped Bars', 'CPTP' => 'Pie');
    }

    function install() {
        $UIMI = getTool('UIMI');
        $UIMI->add($this->code, 'MENU_TOOLS:MENU_ANALYTICS:MENU_CHARTS', 'TOOL', $this->code, 'ac=LIST');
    }

    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['IDCODE'] = array('type' => 'varchar(16)', 'null' => 'NO', 'properties' => '');
        $fields['IDNAME'] = array('type' => 'varchar(64)', 'null' => 'NO', 'properties' => '');
        $fields['VERSION'] = array('type' => 'varchar(32)', 'null' => 'NO', 'properties' => '');
        $fields['VERSION_COUNTER'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['CDATE'] = array('type' => 'datetime', 'null' => 'YES', 'properties' => '');
        $fields['MDATE'] = array('type' => 'datetime', 'null' => 'YES', 'properties' => '');
        $fields['USERNAME'] = array('type' => 'varchar(30)', 'null' => 'NO', 'properties' => '');
        $fields['DESCRIPTION'] = array('type' => 'mediumtext', 'properties' => '');
        $fields['QUERYCODE'] = array('type' => 'varchar(16)', 'null' => 'YES', 'properties' => '');
        $fields['MODULECODE'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '');
        $fields['TYPE'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '');
        $fields['HSIZE'] = array('type' => 'int(5) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['VSIZE'] = array('type' => 'int(5) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['TITLE'] = array('type' => 'varchar(255)', 'null' => 'YES', 'properties' => '');
        $fields['SUBTITLE'] = array('type' => 'varchar(255)', 'null' => 'YES', 'properties' => '');
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'UNI', 'fieldname' => 'IDCODE');
        $indexes[$ipf . '03'] = array('index' => 'MUL', 'fieldname' => 'IDNAME');
        $indexes[$ipf . '04'] = array('index' => 'MUL', 'fieldname' => 'QUERYCODE');
        $indexes[$ipf . '05'] = array('index' => 'MUL', 'fieldname' => 'MODULECODE');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB', 'fields' => $fields, 'indexes' => $indexes);
    }

    function pack($record) {
        $o = $this->packRecord($record);
        $code = $record['IDCODE'];
        $objs = new XMLObject($this->code . '_OBJECTS');
        $this->packsub($objs, 'ANCP', 'SELECT * FROM CLX_AN_CHART_PROPERTIES WHERE CHARTCODE=' . singleQuote($code));
        $this->packsub($objs, 'ANCS', 'SELECT * FROM CLX_AN_CHART_SERIES WHERE CHARTCODE=' . singleQuote($code));
        $o->addObject($objs);
        return $o;
    }

    function unpack($parameters) {//deprecated
        $parameters = array_values(array_merge(array('package' => null), $parameters));
        list($package) = $parameters;
        $record = $this->unpackRecord($package);
        $chartcode = $record['IDCODE'];
        $exists = (ClxCommon::$db->oneExistsRecords('CLX_AN_CHARTS', 'IDCODE=' . singleQuote($chartcode)) > CLX_NUM_ZERO);
        if ($exists === true) {
		  $chartid = ClxCommon::$db->getField('CLX_AN_CHARTS', 'ID', 'IDCODE=' . singleQuote($chartcode));
		  $this->deleteRecordSk($chartid);
        }
        $chartid = ClxCommon::$db->createRecord($this->tablename, $record, false);
        if ($package->hasSubObject($this->code . '_OBJECTS')) {
		  $objs = $package->getObjectXml($this->code . '_OBJECTS');
		  foreach ($objs->objects as $o) {
			 $toolcode = $o->tag;
			 $tool = &getTool($toolcode);
			 $tool->unpack(array('package' => $o, 'id' => $chartcode));
		  }
        }
    }

    function action($ac, $idlist) {
        if ($this->getAccess()->canRead()) {
		  switch ($ac) {
			 case 'CHART':
			     return $this->chart(UserRequest::$get['cc']);
			     break;
			 default:
			     $contents = parent::action($ac, $idlist);
			     break;
		  }//end switch
        } else {
		  $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [AN_CHARTS-CAN_READ][' . $ac . '] ' . $this->getCode(false));
        }
        return $contents;
    }

    function makelist($listingcode = null) {
        //$t = time('U'); genera un warning
        $t = time();
        $to = $this->code;
        $here = 'tool.php?to=' . $to . '&t=' . $t;
        clearHistory();
        pushHistory($this->code, 'LIST', array());
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->setProp(CLX_PROP_ID, 'ANCHList');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'NOT_DEFINED');
        $selectclause = '*';
        $fromclause = 'CLX_AN_CHARTS';
        $whereclause = '';
        $sortclause = '';
        $page = new xPaging();
        $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');

        $table = new htmlTable(CLX_WIDTH_EIGHT);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_ANCH_TITLE_LIST'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_EIGHT));
        $page->header($table);
        $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_CODE'), dict('GEN_LBL_NAME'), dict('GEN_LBL_TYPE'), dict('GEN_LBL_MODULECODE'), dict('GEN_LBL_QUERYCODE'), dict('GEN_LBL_VERSION'), dict('GEN_LBL_DESCRIPTION')));
        $i = CLX_INIT_ZERO;
        while ($record = $page->nextRecord()) {
		  $recordid = $record['ID'];
		  $select = new htmlCheckbox('selector_' . $recordid, false);
		  $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
		  $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
		  $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($record['IDCODE'], $record['IDNAME'], $this->chart_types[$record['TYPE']], $record['MODULECODE'], $record['QUERYCODE'], $record['VERSION'] . '-' . $record['VERSION_COUNTER'], xFormat::text($record['DESCRIPTION'])));
		  $i++;
        }

        $buttons = htmlButton::getButton('submit', dict('GEN_LBL_DISPLAY'), array('action' => 'DISPLAY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        if ($this->getAccess()->canModify()) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        if ($this->getAccess()->canDelete()) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        if ($this->getAccess()->canDisplay()) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DUPLICATE'), array('action' => 'DUPLICATE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ONE'))->source();
        }
        if ($this->getAccess()->canDisplay()) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE_SOMETHING', dict('GEN_LBL_CHART')), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
        }
        $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_FILTER'), array('action' => 'FILTER', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
        $form->addContents($page->footers($table, $buttons));
        return $form->source();
    }

    function searchForm() {
        $msg = array();


        // Search Form
        $form = new htmlForm(CLX_PROP_ACTION, UserRequest::$server['PHP_SELF']);
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'SEARCH');
        $form->addArgument('confirm', '1');
        $form->addArgument(CLX_PROP_TARGET, UserRequest::$request[CLX_PROP_TARGET]);
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_ANCH_TITLE_SEARCH'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addField($msg, dict('GEN_LBL_CODE') . ':', 'ANCH_idcode', UserRequest::$post['ANCH_idcode'], CLX_NUM_16, CLX_NUM_16);
        $table->addField($msg, dict('GEN_LBL_NAME') . ':', 'ANCH_idname', UserRequest::$post['ANCH_idname'], CLX_NUM_32, CLX_NUM_64);
        $table->addField($msg, dict('GEN_LBL_QUERYCODE') . ':', 'ANCH_querycode', UserRequest::$post['ANCH_querycode'], CLX_NUM_16, CLX_NUM_16);
        $table->addField($msg, dict('GEN_LBL_MODULECODE') . ':', 'ANCH_modulecode', UserRequest::$post['ANCH_modulecode'], CLX_NUM_FOUR, CLX_NUM_FOUR);

        $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_SEARCH'), array('action' => 'SEARCH', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        $c = new htmlButtonCloseWindow(dict('GEN_LBL_CANCEL'));
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->setContents($table->source());
        $contents = $form->source();
        // Search Procedure
        if (isset(UserRequest::$post['confirm'])) {
		  $table = new htmlTable(CLX_WIDTH_TWO);
		  $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
		  $criteria = '';
		  if (UserRequest::$post['ANCH_idcode'] <> '') {
			 $criteria .= 'UPPER(IDCODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['ANCH_idcode']) . '%');
		  }
		  if (UserRequest::$post['ANCH_idname'] <> '') {
			 $criteria .= 'UPPER(IDNAME) LIKE ' . singleQuote(clx_change_especial_char_html (uppercase(UserRequest::$post['ANCH_idname'])) . '%');
		  }
		  if (UserRequest::$post['ANCH_modulecode'] <> '') {
			 $criteria .= 'UPPER(MODULECODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['ANCH_modulecode']) . '%');
		  }
		  if (UserRequest::$post['ANCH_querycode'] <> '') {
			 $criteria .= 'UPPER(QUERYCODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['ANCH_querycode']) . '%');
		  }
            $qty = ClxCommon::$db->oneExistsRecords($this->tablename, $criteria);
		  if ($qty == CLX_NUM_ZERO) {
			 $table->addCell(dict('GEN_LBL_NORECORDFOUND'), array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
		  } else {
			 $table->addCell('Found record(s)', array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
			 $records = ClxCommon::$db->getRecords($this->tablename, $criteria);
			 $i = CLX_INIT_ZERO;
			 while (($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE))) {
			     $js = 'javascript:setLookupValue(' . quote(UserRequest::$request[CLX_PROP_TARGET]) . ', ' . quote($record['IDCODE']) . ');';
			     $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array('<page t=ancharts></page><a href=' . singleQuote($js) . '>' . $record['IDCODE'] . '</a>', $record['IDNAME']));
			     $i++;
			 }
		  }
		  $contents .= $table->source();
        }
        return $contents;
    }

    function editForm($ac, $idlist, $record, $msg) {
        $modify = (UserRequest::$request['ac'] === 'MODIFY');
        $title = ($modify) ? dict('TOOL_ANCH_TITLE_MODIFY') : dict('TOOL_ANCH_TITLE_CREATE');
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', $ac);
        $form->addArgument('confirm', '1');
        $form->addArgument('ANCH_id', $record['ANCH_id']);
        $form->addArgument('idlist', toString($idlist));
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell($title, array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        if (!empty($modify)) {
		  $table->addData(dict('GEN_LBL_CODE') . ':', $record['ANCH_idcode']);
		  $form->addArgument('ANCH_idcode', $record['ANCH_idcode']);
        } else {
		  $table->addField($msg, dict('GEN_LBL_CODE') . ':', 'ANCH_idcode', $record['ANCH_idcode'], CLX_NUM_16, CLX_NUM_16);
        }
        $table->addField($msg, dict('GEN_LBL_NAME') . ':', 'ANCH_idname', $record['ANCH_idname'], CLX_NUM_48, CLX_NUM_64);
        $table->addField($msg, dict('GEN_LBL_VERSION') . ':', 'ANCH_version', $record['ANCH_version'], CLX_NUM_16, CLX_NUM_16);
        $table->addText($msg, dict('GEN_LBL_DESCRIPTION') . ':', 'ANCH_description', $record['ANCH_description'], CLX_NUM_FOUR, CLX_NUM_48);
        if (empty($modify)) {
		  $DBQY = &getTool('DBQY');
		  $DBQY->placeChooseQuery($msg, dict('GEN_LBL_QUERYCODE') . ':', 'ANCH_querycode', $record['ANCH_querycode'], $table);
        }
        $DBMO = &getTool('DBMO');
        $DBMO->placeChooseModule($msg, dict('GEN_LBL_MODULECODE') . ':', 'ANCH_modulecode', $record['ANCH_modulecode'], $table);
        $table->addField($msg, dict('GEN_LBL_HSIZE') . ':', 'ANCH_hsize', $record['ANCH_hsize'], CLX_NUM_FIVE, CLX_NUM_FIVE);
        $table->addField($msg, dict('GEN_LBL_VSIZE') . ':', 'ANCH_vsize', $record['ANCH_vsize'], CLX_NUM_FIVE, CLX_NUM_FIVE);
        if (empty($modify)) {
		  $pop = new HTMLSelect('ANCH_type');
		  foreach ($this->chart_types as $key => $value) {
			 $pop->addItem($value, $key, ($record['ANCH_type'] == $key));
		  }
		  $table->addData(dict('GEN_LBL_TYPE') . ':', $pop->source());
        }
        $form->addContents($table->source());
        $buttons = '';
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        if (!empty($modify)) {
		  $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_SAVE'), array('action' => $ac, 'form' => $form->getProp(CLX_PROP_ID)))->source();
        } else {
		  $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_SAVE'), array('action' => $ac, 'form' => $form->getProp(CLX_PROP_ID)))->source();
        }
        $c = new htmlButtonCancel();
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->addContents($table->source());
        return $form->source();
    }

    function checkForm(&$msg) {
        $msg = array();
        $action = UserRequest::$post['ac'];
        if ($action === 'CREATE') {
		  xValidate::unique($msg, 'ANCH_idcode', 'IDCODE', $this->tablename, 'IDCODE=' . singleQuote(UserRequest::$post['ANCH_idcode']));
		  $DBQY = &getTool('DBQY');
		  if (!$DBQY->queryExists(UserRequest::$post['ANCH_querycode'])) {
			 $msg['ANCH_querycode'] = dict('GEN_LBL_NORECORDFOUND');
		  }
        }
        if (strlen(UserRequest::$post['ANCH_idname']) == CLX_NUM_ZERO) {
		  $msg['ANCH_idname'] = dict('GEN_LBL_NORECORDFOUND');
        }
        $DBMO = &getTool('DBMO');
        if (!$DBMO->moduleExists(UserRequest::$post['ANCH_modulecode'])) {
		  $msg['ANCH_modulecode'] = dict('GEN_LBL_NORECORDFOUND');
        }
        xValidate::integer($msg, 'ANCH_hsize', UserRequest::$post['ANCH_hsize'], CLX_NUM_HUNDRED, '2000', dict('GEN_LBL_HSIZE'));
        xValidate::integer($msg, 'ANCH_vsize', UserRequest::$post['ANCH_vsize'], CLX_NUM_HUNDRED, '2000', dict('GEN_LBL_VSIZE'));
        return count($msg) == CLX_NUM_ZERO;
    }

    function confirmDeleteFill(&$table, $record) {
        $table->addData(dict('GEN_LBL_CODE') . ':', $record['ANCH_idcode']);
        $table->addData(dict('GEN_LBL_NAME') . ':', $record['ANCH_idname']);
    }

    function viewFormFill($record) {

        addHistory('CHARTCODE', $record['ANCH_idcode']);

        $table = new htmlTable(CLX_WIDTH_ONE);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_ANCH_TITLE_VIEW') . ': ' . $record['ANCH_idname'], array(CLX_PROP_CLASS => CLX_CLASS_TITLE));
        $contents = $table->source();

        $infotable = new htmlTable(CLX_WIDTH_TWO);
        $infotable->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $infotable->addData(dict('GEN_LBL_CODE') . ':', $record['ANCH_idcode']);
        $infotable->addData(dict('GEN_LBL_NAME') . ':', $record['ANCH_idname']);
        $infotable->addData(dict('GEN_LBL_VERSION') . ':', $record['ANCH_version'] . '-' . $record['ANCH_version_counter']);
        $infotable->addData(dict('GEN_LBL_DESCRIPTION') . ':', xFormat::text($record['ANCH_description']));
        $infotable->addData(dict('GEN_LBL_MODULECODE') . ':', $record['ANCH_modulecode']);
        $infotable->addData(dict('GEN_LBL_QUERYCODE') . ':', $record['ANCH_querycode']);
        $infotable->addData(dict('GEN_LBL_TYPE') . ':', $this->chart_types[$record['ANCH_type']]);
        $infotable->addData(dict('GEN_LBL_SIZE') . ':', $record['ANCH_hsize'] . ' x ' . $record['ANCH_vsize']);

        $imgtable = new htmlTable(CLX_WIDTH_ONE);
        $imgtable->setProps(array(CLX_PROP_CLASS => 'panel', CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $img = '<img src="window.php?to=ANCH&no_menu=1&no_alerts=1&ac=CHART&cc=' . $record['ANCH_idcode'] . '">';
        $imgtable->addCell($img, array(CLX_PROP_CLASS => CLX_CLASS_DATA));
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => 'panel', CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell($imgtable->source(), array());
        $table->addCell($infotable->source(), array());
        $contents .= $table->source();

        $ANCS = &getTool('ANCS');
        $contents .= $ANCS->makepanel($record['ANCH_idcode']);

        $ANCP = &getTool('ANCP');
        $contents .= $ANCP->makepanel($record['ANCH_idcode']);

        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => 'panel', CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addData(dict('GEN_LBL_CDATE') . ':', $record['ANCH_cdate']);
        $table->addData(dict('GEN_LBL_MDATE') . ':', $record['ANCH_mdate']);
        $table->addData(dict('GEN_LBL_USERNAME') . ':', $record['ANCH_username']);

        $contents .= buildPanelInterface(dict('GEN_TTL_SYSTEMINFO'), 'SYSTEMINFO', $table->source(), CLX_NUM_MINUS_1);
        return $contents;
    }

    function filterFormFill($msg, &$table) {

        $table->addField($msg, dict('GEN_LBL_CODE') . ':', 'ANCH_idcode', (array_key_exists('ANCH_idcode', UserRequest::$post)) ? UserRequest::$post['ANCH_idcode'] : '', CLX_NUM_16, CLX_NUM_16);
        $table->addField($msg, dict('GEN_LBL_NAME') . ':', 'ANCH_idname', (array_key_exists('ANCH_idname', UserRequest::$post)) ? UserRequest::$post['ANCH_idname'] : '', CLX_NUM_32, CLX_NUM_64);
        $table->addField($msg, dict('GEN_LBL_MODULECODE') . ':', 'ANCH_modulecode', (array_key_exists('ANCH_modulecode', UserRequest::$post)) ? UserRequest::$post['ANCH_modulecode'] : '', CLX_NUM_FOUR, CLX_NUM_FOUR);
        $table->addField($msg, dict('GEN_LBL_QUERYCODE') . ':', 'ANCH_querycode', (array_key_exists('ANCH_querycode', UserRequest::$post)) ? UserRequest::$post['ANCH_querycode'] : '', CLX_NUM_16, CLX_NUM_16);
        $sort = new HtmlSelect('ANCH_sort');
        $sort->addItem('', '', true);
        $sort->addItem(dict('GEN_LBL_MODULE'), 'MODULECODE', false);
        $sort->addItem(dict('GEN_LBL_QUERY'), 'QUERYCODE', false);
        $sort->addItem(dict('GEN_LBL_CODE'), 'IDCODE', false);
        $sort->addItem(dict('GEN_LBL_NAME'), 'IDNAME', false);
        $table->addData(dict('GEN_LBL_ORDERBY') . ':', $sort->source());
    }

    function setfilter() {
        $filter = '';
        $boolean = '';
        if (!empty(UserRequest::$post['ANCH_idcode'])) {
		  $filter .= $boolean . ' UPPER(IDCODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['ANCH_idcode']) . '%');
		  $boolean = 'AND';
        }
        if (!empty(UserRequest::$post['ANCH_idname'])) {
		  $filter .= $boolean . ' UPPER(IDNAME) LIKE ' . singleQuote(uppercase(clx_change_especial_char_html (UserRequest::$post['ANCH_idname'])) . '%');
		  $boolean = 'AND';
        }
        if (!empty(UserRequest::$post['ANCH_modulecode'])) {
		  $filter .= $boolean . ' UPPER(MODULECODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['ANCH_modulecode']) . '%');
		  $boolean = 'AND';
        }
        if (!empty(UserRequest::$post['ANCH_querycode'])) {
		  $filter .= $boolean . ' UPPER(QUERYCODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['ANCH_querycode']) . '%');
		  $boolean = 'AND';
        }
        $_SESSION['paging']['ANCHList']['sortby'] = UserRequest::$post['ANCH_sort'];
        $_SESSION['paging']['ANCHList']['criteria'] = $filter;
        $_SESSION['paging']['ANCHList']['startat'] = '0';
    }
	///////////////////////////////////////////////////
	///comentado 22/01/2019 para utilizar la clx_tool
	//////////////////////////////////////////////////
    /*function clearfilter() {
        $_SESSION['paging']['ANCHList']['criteria'] = '';
    }*/

    function loadToolRecord($recordid) {
        $values = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $recordid);
        $record = array();
        $record['ANCH_id'] = $recordid;
        $record['ANCH_idcode'] = $values['IDCODE'];
        $record['ANCH_idname'] = $values['IDNAME'];
        $record['ANCH_version'] = $values['VERSION'];
        $record['ANCH_version_counter'] = $values['VERSION_COUNTER'];
        $record['ANCH_cdate'] = xFormat::date($values['CDATE']);
        $record['ANCH_mdate'] = xFormat::date($values['MDATE']);
        $record['ANCH_username'] = $values['USERNAME'];
        $record['ANCH_description'] = $values['DESCRIPTION'];
        $record['ANCH_modulecode'] = $values['MODULECODE'];
        $record['ANCH_querycode'] = $values['QUERYCODE'];
        $record['ANCH_type'] = $values['TYPE'];
        $record['ANCH_hsize'] = $values['HSIZE'];
        $record['ANCH_vsize'] = $values['VSIZE'];
        return $record;
    }

    function saveRecord() {
        $d = xFormat::getNow('Y-m-d');
        $record = array();
        $record['IDNAME'] = UserRequest::$post['ANCH_idname'];
        $record['VERSION'] = UserRequest::$post['ANCH_version'];
        $record['MDATE'] = $d;
        $record['USERNAME'] = $_SESSION['user_record']['username'];
        $record['DESCRIPTION'] = UserRequest::$post['ANCH_description'];
        $record['MODULECODE'] = UserRequest::$post['ANCH_modulecode'];
        $record['HSIZE'] = UserRequest::$post['ANCH_hsize'];
        $record['VSIZE'] = UserRequest::$post['ANCH_vsize'];
        ClxCommon::$db->updateRecord($this->tablename, $record, 'ID=' . UserRequest::$post['ANCH_id']);
        $this->versionIncrease('ID', UserRequest::$post['ANCH_id']);
    }

    function createRecord_() {

        $d = xFormat::getNow('Y-m-d');
        $record = array();
        $record['IDCODE'] = UserRequest::$post['ANCH_idcode'];
        $record['IDNAME'] = UserRequest::$post['ANCH_idname'];
        $record['VERSION'] = UserRequest::$post['ANCH_version'];
        $record['VERSION_COUNTER'] = '0';
        $record['CDATE'] = $d;
        $record['MDATE'] = $d;
        $record['USERNAME'] = $_SESSION['user_record']['username'];
        $record['DESCRIPTION'] = UserRequest::$post['ANCH_description'];
        $record['QUERYCODE'] = UserRequest::$post['ANCH_querycode'];
        $record['MODULECODE'] = UserRequest::$post['ANCH_modulecode'];
        $record['TYPE'] = UserRequest::$post['ANCH_type'];
        $record['HSIZE'] = UserRequest::$post['ANCH_hsize'];
        $record['VSIZE'] = UserRequest::$post['ANCH_vsize'];
        $recordid = ClxCommon::$db->createRecord($this->tablename, $record, false);
        $DBMO = &getTool('DBMO');
        $DBMO->versionIncrease('IDCODE', $record['MODULECODE']);

        $ANCP = &getTool('ANCP');
        $ANCP->createChartProperties($record['IDCODE']);
        return $recordid;
    }

    function duplicateRecord() {

        $originalchartid = UserRequest::$post['ANCH_id'];
        $originalchartrec = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $originalchartid);
        $originalchartcode = $originalchartrec['IDCODE'];
        $chartid = $this->createRecord_();
        $chartcode = UserRequest::$post['ANCH_idcode'];
        $records = ClxCommon::$db->getRecords('CLX_AN_CHART_SERIES', 'CHARTCODE=' . singleQuote($originalchartcode));
        while (($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE))) {
		  unset($record['ID']);
		  $record['CHARTCODE'] = $chartcode;
		  $rid = ClxCommon::$db->createRecord('CLX_AN_CHART_SERIES', $record, false);
		  unset($rid);
        }
        ClxCommon::$db->deleteRecords('CLX_AN_CHART_PROPERTIES', 'CHARTCODE=' . singleQuote($chartcode));
        $records = ClxCommon::$db->getRecords('CLX_AN_CHART_PROPERTIES', 'CHARTCODE=' . singleQuote($originalchartcode));
        while (($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE))) {
		  unset($record['ID']);
		  $record['CHARTCODE'] = $chartcode;
		  $rid = ClxCommon::$db->createRecord('CLX_AN_CHART_PROPERTIES', $record, false);
		  unset($rid);
        }
        return $chartid;
    }

    function deleteRecordSk($id) {
        $chartrec = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $id);
        $chartcode = $chartrec['IDCODE'];
        ClxCommon::$db->deleteRecords('CLX_AN_CHART_PROPERTIES', 'CHARTCODE=' . singleQuote($chartcode));
        ClxCommon::$db->deleteRecords('CLX_AN_CHART_SERIES', 'CHARTCODE=' . singleQuote($chartcode));
        ClxCommon::$db->deleteRecords($this->tablename, 'ID=' . $id);
        $DBMO = &getTool('DBMO');
        $DBMO->versionIncrease('IDCODE', $chartrec['MODULECODE']);
    }

    function defaultRecord() {
        $record = array();
        $record['ANCH_id'] = CLX_INIT_ZERO;
        $record['ANCH_idcode'] = '';
        $record['ANCH_idname'] = '';
        $record['ANCH_version'] = '0.0.0';
        $record['ANCH_cdate'] = '';
        $record['ANCH_mdate'] = '';
        $record['ANCH_username'] = $_SESSION['user_record']['username'];
        $record['ANCH_description'] = '';
        $record['ANCH_modulecode'] = '';
        $record['ANCH_querycode'] = '';
        $record['ANCH_type'] = 'CPTX'; // Bar line combo
        $record['ANCH_hsize'] = '320';
        $record['ANCH_vsize'] = '240';
        return $record;
    }

    function versionIncrease($fieldname, $fieldvalue) {
        parent::versionIncrease($fieldname, $fieldvalue);
        $modulecode = ClxCommon::$db->getField($this->tablename, 'MODULECODE', $fieldname . '=' . singleQuote($fieldvalue));
        $DBMO = &getTool('DBMO');
        $DBMO->versionIncrease('IDCODE', $modulecode);
    }

    function placeChooseChart($msg, $prompt, $name, $default, &$table) {
        $searchIcon .= getSkinIcon('view', 'javascript:openWindow(' . singleQuote(CLX_CHAR_EMPTY_STRING).', ' . singleQuote('pop.php?to=ANCH&ac=SEARCH&target=' . $name) . ', 500, 300);', 'Search for a chart');
        if ($msg[$name] <> '')
		  $table->addCell($msg[$name], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => $table->width));
        $table->addCell($prompt, array(CLX_PROP_CLASS => CLX_CLASS_PROMPT));
        $c = new htmlField($name, $default, CLX_NUM_16, CLX_NUM_16);
        $table->addCell($c->source() . $searchIcon, array(CLX_PROP_CLASS => CLX_CLASS_DATA));
        return CLX_CHAR_EMPTY_STRING;
    }

    function chartExists(&$idcode) {
        $qty = ClxCommon::$db->oneExistsRecords($this->tablename, 'IDCODE=' . singleQuote($idcode));
        if ($qty > CLX_NUM_ZERO) {
		  $idcode = ClxCommon::$db->getField($this->tablename, 'IDCODE', 'IDCODE=' . singleQuote($idcode));
        }
        return $qty > CLX_NUM_ZERO;
    }

    function chartInformation($chartcode, &$series_records, &$sql) {
        $DBQY = &getTool('DBQY');
        $chartrec = ClxCommon::$db->getRecord('CLX_AN_CHARTS', 'IDCODE=' . singleQuote($chartcode));
        $querycode = $chartrec['QUERYCODE'];
        $queryrec = $DBQY->getQuery($this->code, $querycode);
        //security here
        $chartseries = ClxCommon::$db->getRecords('CLX_AN_CHART_SERIES', 'CHARTCODE=' . singleQuote($chartcode) . ' AND STATUS=1 ORDER BY SEQNUM');
        $selectclause = '';
        $fromclause = $queryrec['FROMCLAUSE'];
        $whereclause = $queryrec['WHERECLAUSE'];
        $groupclause = '';
        $comma1 = '';
        $comma2 = '';
        $series_records = array();
        while (($series_rec = ClxCommon::$db->getNextRecord($chartseries))) {
		  if (empty($series_rec['FUNCTION'])) {
			 $as = $series_rec['TABLECODE'] . '_' . $series_rec['FIELDNAME'];
			 $selectclause .= $comma1 . $series_rec['TABLECODE'] . '.' . $series_rec['FIELDNAME'] . ' ' . $as;
		  } else {
			 $as = $series_rec['TABLECODE'] . '_' . $series_rec['FIELDNAME'] . '_' . $series_rec['ID'];
			 $selectclause .= $comma1 . $series_rec['FUNCTION'] . '(' . $series_rec['TABLECODE'] . '.' . $series_rec['FIELDNAME'] . ')' . ' ' . $as;
		  }
		  $comma1 = ', ';
		  if (!empty($series_rec['GROUPBY'])) {
			 $groupclause .= $comma2 . $series_rec['TABLECODE'] . '.' . $series_rec['FIELDNAME'];
			 $comma2 = ', ';
		  }
		  $series_rec['AS'] = $as;
		  $series_records[] = $series_rec;
        }
        if ($selectclause == '')
		  $selectclause = '1';
        $sql = 'SELECT ' . $selectclause . ' FROM ' . $fromclause;
        if (!empty($whereclause)) {
		  $sql .= ' WHERE ' . $whereclause;
        }
        if (!empty($groupclause)) {
		  $sql .= ' GROUP BY ' . $groupclause;
        }
    }

    /**
     * Builds Pagelet for Tool
     * @param string $code
     * @param string $ac
     * @param integer $objectid
     * @return string
     */
    function pagelet($code, $ac = null, $objectid = null) {
        return '<img src="window.php?to=ANCH&no_menu=1&no_alerts=1&ac=CHART&cc='.$code.'">';
    }

    function chart($chartcode) {
        require_once kPathToWebSite . kPathToJpgraph . 'jpgraph.php';
        require_once kPathToWebSite . kPathToJpgraph . 'jpgraph_pie.php';
        if (empty($chartcode) || !$this->chartExists($chartcode)) {
		  return warning('There is no chart with this code: '.$chartcode);
        }
        // Get the information from the database and build the series
        $sql = '';
        $series_records = array();
        $chartrec = ClxCommon::$db->getRecord($this->tablename, 'IDCODE=' . singleQuote($chartcode));
        $charttype = $chartrec['TYPE'];
        $this->chartInformation($chartcode, $series_records, $sql);
        $series_data = array();
        $records = ClxCommon::$db->executeQuery($sql);
        while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
		  foreach ($series_records as $series_rec) {
			 list(/* $tablecode */, $fieldname) = explode('_', $series_rec['AS']);
			 $fieldrec = ClxCommon::$db->getRecord('CLX_DB_FIELDS', 'FIELDNAME=' . singleQuote($fieldname));
			 $fieldinstance = getToolObject('DBFT', $fieldrec['FIELDTYPE']);
			 $data = array();
			 $data[$fieldrec['FIELDNAME']] = $record[$series_rec['AS']];
			 $value = strip_tags($fieldinstance->formatField($fieldrec['FIELDNAME'], $fieldrec, $data));
			 $series_data[$series_rec['ID']][] = $value;
		  }
        }
        // Create the Graph Class and set some properties
        // Create the graph.
        if ($charttype === 'CPTP') {
		  $graph = new PieGraph($chartrec['HSIZE'], $chartrec['VSIZE']);
        } else {
		  $graph = new Graph($chartrec['HSIZE'], $chartrec['VSIZE']);
		  $graph->SetScale('TEXTLIN');
        }
        // Set the properties
        $prop_records = ClxCommon::$db->getRecords('CLX_AN_CHART_PROPERTIES', 'IDCODE=' . singleQuote($chartcode), 'STATUS=1');
        while (($prop_record = ClxCommon::$db->getNextRecord($prop_records))) {
		  $prop_object = getToolObject('CPTX', $prop_record['IDCODE']);
		  $prop_object->setFunctions($graph, $prop_record['SERIES_PROPERTIES']);
        }

        // Add the series to the graph class
        switch ($charttype) {
		  case 'CPTY': // Bar & line
			 foreach ($series_records as $series_rec) {
			     $series_object = getToolObject($charttype, $series_rec['TYPE']);

			     $series = $series_data[$series_rec['ID']];
			     $plot = $series_object->getPlot($graph, $series_rec['SERIES_PROPERTIES'], $series);
			     if (!empty($plot))
			         $graph->Add($plot);
			 }
			 break;
		  case 'CPTB': // Grouped Bar
			 $plots = array();
			 foreach ($series_records as $series_rec) {
			     $series_object = getToolObject($charttype, $series_rec['TYPE']);
			     $series = $series_data[$series_rec['ID']];
			     $plot = $series_object->getPlot($graph, $series_rec['SERIES_PROPERTIES'], $series);
			     if (!empty($plot))
			         $plots[] = $plot;
			 }
			 $gbplot = new GroupBarPlot($plots);
			 $graph->Add($gbplot);
			 break;
		  case 'CPTP': // Pie
			 $series_object = getToolObject($charttype, $series_records[CLX_NUM_ZERO]['TYPE']);
			 $plot = $series_object->getPlot($graph, $series_records, $series_data);
			 if (!empty($plot))
			     $graph->Add($plot);
			 break;
		  default:
			 break;
        }
        $graph->Stroke();
        salir(CLX_NUM_ZERO);
    }

}
