<?php

/**
 * Reports Query Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: jclerma $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2017-11-03 15:09:40 -0400 (Fri, 03 Nov 2017) $
 * @version    	$Rev: 9615 $
 * @package		Clientix.Tool.Analytics
 * @abstract		Report Columns
 */
class an_report_queries extends clx_tool {

    /**
     * Constructor
     */
    function __construct() {
        $this->code = 'ANRQ';
        $this->name = 'Report Queries';
        $this->system = 'Analytics';
        $this->version = '0.1.2.0';

        $this->accesskey = 'ANRP';


        $this->tablename = 'CLX_AN_REPORT_QUERIES';
        $this->record_name_singular = 'report queries';
        $this->record_name_plural = 'report queries';
        $this->primary_field = 'IDCODE';
        $this->secondary_field = 'IDNAME';
        $this->terciary_field = 'REPORTCODE';
        $this->seqnum_field = 'SEQNUM';
        $this->editform = array(CLX_CLASS_TITLE => 'TOOL_ANQR_TITLE_CREATE',
		  'panels' => array(
			 array('type' => 'normal',
			     'entries' => array(
			         array('label' => 'GEN_LBL_REPORTCODE',
					   'fields' => array(
						  array('name' => 'reportcode',
						      'type' => 'lookup',
						      'default' => '',
						      'readonly' => true,
						      'tool' => 'ANRP',
						      'validations' => array(
								array('type' => 'sqlobject', 'required' => '1'), array('type' => 'lookup',
								    'fieldname' => 'idcode',
								    'tablename' => 'CLX_AN_REPORTS'
								)
						      )
						  )
					   )
			         ), array('label' => 'GEN_LBL_CODE',
					   'fields' => array(
						  array('name' => 'idcode', 'type' => 'string', 'default' => '', 'size' => '16', 'displaysize' => '20', 'dontmodify' => '1',
						      'validations' => array(
								array('type' => 'sqlobject', 'required' => '1'), array('type' => 'unique', 'fieldname' => 'idcode', 'tablename' => $this->tablename))
						  )
					   )
			         ), array('label' => 'GEN_LBL_NAME',
					   'fields' => array(
						  array('name' => 'idname',
						      'type' => 'string',
						      'default' => '',
						      'size' => '64',
						      'displaysize' => '48',
						      'validations' => array(
								array('type' => 'string', 'required' => '1'))
						  )
					   )
			         ), array('label' => 'GEN_LBL_QUERYCODE',
					   'fields' => array(
						  array('name' => 'querycode',
						      'type' => 'lookup',
						      'default' => '',
						      'tool' => 'DBQY',
						      'validations' => array(
								array('type' => 'sqlobject', 'required' => '1'), array('type' => 'lookup',
								    'fieldname' => 'idcode',
								    'tablename' => 'CLX_DB_QUERIES'
								)
						      )
						  )
					   )
			         ), array('label' => 'GEN_LBL_ACTIVE',
					   'fields' => array(
						  array('name' => 'active',
						      'type' => CLX_CLASS_LIST,
						      'default' => '1',
						      'options' => $this->yesno)
					   )
			         )
			     )
			 )
		  )
        );
        $this->viewform = $this->editform;
        $this->viewform[CLX_CLASS_TITLE] = 'TOOL_ANQR_TITLE_DISPLAY';
        $this->toolpanelkeys = array(array('name' => 'REPORTCODE', 'type' => 'string'));
        $this->toolpanel = array('name' => $this->tablename, CLX_CLASS_TITLE => $this->record_name_plural, 'has_reorder' => true,
		  'fields' =>
		  array(array('name' => 'ID', 'type' => 'number'), array('name' => 'IDCODE', 'type' => 'string'), array('name' => 'IDNAME', 'type' => 'string'), array('name' => 'ACTIVE', 'options' => $this->yesno, 'type' => CLX_CLASS_LIST), array('name' => 'QUERYCODE', 'type' => 'lookup', 'tool' => 'DBQY'), array('name' => 'REPORTCODE', 'type' => 'lookup', 'tool' => 'ANRP'),
		  )
        );
    }

    /**
     * Describes the tool table structure.
     * @return array ('type' => 'varchar(255)', 'properties'=>'auto increment',
     * 				'index'=>'', default = '', 'dict'=>true)
     * 				 dict true or false means that field holds dictionary entry
     */
    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['IDCODE'] = array('type' => 'varchar(16)', 'null' => 'NO', 'properties' => '');
        $fields['IDNAME'] = array('type' => 'varchar(64)', 'null' => 'NO', 'properties' => '');
        $fields['REPORTCODE'] = array('type' => 'varchar(16)', 'null' => 'NO', 'properties' => '');
        $fields['QUERYCODE'] = array('type' => 'varchar(16)', 'null' => 'NO', 'properties' => '');
        $fields['ACTIVE'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'default' => '1', 'properties' => '');
        $fields['SEQNUM'] = array('type' => 'int(4) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'MUL', 'fieldname' => 'IDCODE');
        $indexes[$ipf . '03'] = array('index' => 'MUL', 'fieldname' => 'REPORTCODE');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB',
		  'fields' => $fields,
		  'indexes' => $indexes);
    }

    /**
     * Pack tool record on xml Format
     * @param array record
     * @return obj
     */
    function pack($record) {
        $o = $this->packRecord($record);
        $objs = new XMLObject($this->code . '_OBJECTS');
        $this->packsub($objs, 'ANQC', 'SELECT * FROM CLX_AN_REP_QRY_CRITS WHERE REPQRYCODE=' . singleQuote($record['IDCODE']));
        $this->packsub($objs, 'DBQY', 'SELECT * FROM CLX_DB_QUERIES WHERE IDCODE=' . singleQuote($record['QUERYCODE']));
        $o->addObject($objs);
        return $o;
    }

    /**
     * Loads package into system
     * @param string $package
     */
    function unpack($parameters) {
        $parameters = array_values(array_merge(array('package' => null, 'id' => null), $parameters));
        list($package, $reportid) = $parameters;
        $record = $this->unpackRecord($package);
        ClxFreeReturn(ClxCommon::$db->createRecord($this->tablename, $record, false));
        if ($package->hasSubObject($this->code . '_OBJECTS')) {
		  $objs = $package->getObjectXml($this->code . '_OBJECTS');
		  foreach ($objs->objects as $o) {
			 $toolcode = $o->tag;
			 $tool = &getTool($toolcode);
			 $tool->unpack(array('package' => $o, 'id' => $reportid));
		  }
        }
    }

    /**
     * Action is called for every user interaction with the module...
     *
     * @param string $ac
     * @param string $idlist
     * @return string
     */
    function action($ac, $idlist) {
        if ($this->getAccess()->canRead()) {
		  switch ($ac) {
			 case 'QUERY_FIELDS_XML':
			     $repquerycode = (!empty(UserRequest::$request['rqc'])) ? UserRequest::$request['rqc'] : CLX_CHAR_EMPTY_STRING;
			     sendXmlHeader();
			     $contents = $this->getQueryFields($repquerycode)->saveXml();
			     break;
			 default:
			     $contents = parent::action($ac, $idlist);
		  }
        } else {
		  $contents = warning(dict('GEN_ERR_NO_ACCESS') . '[AN_REPORT_QUERIES-CAN_READ][' . $ac . '] ' . $this->getCode(false));
        }
        return $contents;
    }

    /**
     * Build display form contents
     * @param array $record
     * @return string
     */
    function viewFormFill($record) {
        addHistory('repqrycode', $record['ANRQ_idcode']);
        $contents = parent::viewFormFill($record);
        $ANQC = &getTool('ANQC');
        $contents .= $ANQC->makepanel($record['ANRQ_idcode']);
        return $contents;
    }

    /**
     * Creates reorder list
     * @return string
     */
    function reorderRecords() {
        $callingpage = peekHistory();
        $sql = 'SELECT ID, IDCODE, IDNAME  FROM ' . $this->tablename . '  WHERE REPORTCODE=' . singleQuote($callingpage['reportcode']) . '  ORDER BY SEQNUM';
        $list = array();
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
		  $list[$r['ID']] = $r['IDCODE'] . ' ' . $r['IDNAME'];
        }
        return $list;
    }

    /**
     * Performs reorder record action
     */
    function reorderAction() {
        parent::reorderAction();
        $callingpage = peekHistory();
        $ANRP = &getTool('ANRP');
        $ANRP->versionIncrease('IDCODE', $callingpage['reportcode']);
    }

    /**
     * Update records
     *
     * @return integer
     */
    function saveRecord() {
        $record = array();
        $record['IDCODE'] = UserRequest::$post['ANRQ_idcode'];
        $record['IDNAME'] = UserRequest::$post['ANRQ_idname'];
        $record['REPORTCODE'] = UserRequest::$post['ANRQ_reportcode'];
        $record['QUERYCODE'] = UserRequest::$post['ANRQ_querycode'];
        $record['ACTIVE'] = UserRequest::$post['ANRQ_active'];
        ClxCommon::$db->updateRecord($this->tablename, $record, 'ID=' . UserRequest::$post['ANRQ_id']);
        $ANRP = &getTool('ANRP');
        $ANRP->versionIncrease('IDCODE', $record['REPORTCODE']);
        return UserRequest::$post['ANRQ_id'];
    }

    /**
     * Creates record
     *
     * @return integer
     */
    function createRecordVoid_() {
        ClxFreeReturn($this->createRecord_());
    }

    function createRecord_() {
        $record = array();
        $record['IDCODE'] = UserRequest::$post['ANRQ_idcode'];
        $record['IDNAME'] = UserRequest::$post['ANRQ_idname'];
        $record['REPORTCODE'] = UserRequest::$post['ANRQ_reportcode'];
        $record['QUERYCODE'] = UserRequest::$post['ANRQ_querycode'];
        $record['ACTIVE'] = UserRequest::$post['ANRQ_active'];
        $record['SEQNUM'] = ClxCommon::$db->getField($this->tablename, 'MAX(SEQNUM)', 'REPORTCODE = ' . singleQuote(UserRequest::$post['ANRQ_reportcode'])) + CLX_NUM_ONE;
        $ANRP = &getTool('ANRP');
        $ANRP->versionIncrease('IDCODE', $record['REPORTCODE']);
        return ClxCommon::$db->createRecord($this->tablename, $record, false);
    }

    /**
     * Deletes record on selected id
     *
     * @param integer $id
     */
    function deleteRecordSk($id) {
        $qryrec = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $id);
        ClxCommon::$db->deleteRecords('CLX_AN_REP_QRY_CRITS', 'REPQRYCODE=' . singleQuote($qryrec['IDCODE']));
        ClxCommon::$db->deleteRecords($this->tablename, 'ID=' . $id);
        $ANRP = &getTool('ANRP');
        $ANRP->versionIncrease('IDCODE', $qryrec['REPORTCODE']);
    }

    /**
     * Defines default record on initializacion
     *
     * @return array
     */
    function defaultRecord() {
        $callingpage = peekHistory();
        $rpid = ClxCommon::$db->getField('CLX_AN_REPORTS', 'ID', 'IDCODE = ' . singleQuote($callingpage['reportcode']));
        $qty = ClxCommon::$db->countRecords($this->tablename, 'REPORTCODE = ' . singleQuote($callingpage['reportcode'])) + CLX_NUM_ONE;
        $record = array();
        $record['ANRQ_id'] = CLX_INIT_ZERO;
        $record['ANRQ_idcode'] = 'R' . $rpid . '_' . stringAddSpacer($qty, CLX_NUM_THREE);
        $record['ANRQ_idname'] = '';
        $record['ANRQ_reportcode'] = $callingpage['reportcode'];
        $record['ANRQ_querycode'] = '';
        $record['ANRQ_active'] = '1';
        $record['ANRQ_seqnum'] = '0';
        return $record;
    }

    /**
     * Increase tool version
     * @param string $fieldname
     * @param string $fieldvalue
     */
    function versionIncrease($fieldname, $fieldvalue) {
        $reportcode = ClxCommon::$db->getField('CLX_AN_REPORT_QUERIES', 'REPORTCODE', $fieldname . ' = ' . singleQuote($fieldvalue));
        $ANRP = &getTool('ANRP');
        $ANRP->versionIncrease('IDCODE', $reportcode);
    }

    /**
     * Return zml object with posible fields on tables of query
     * @param string $repquerycode
     * @param bool $noderived
     * @return object
     */
    function getQueryFields($repquerycode, $noderived = false) {

        // create a new XML document
        $doc = new DomDocument('1.0');
        if ($repquerycode === ''){
		  return $doc->saveXML();
        }
        $querycode = ClxCommon::$db->getField($this->tablename, 'QUERYCODE', 'IDCODE = ' . singleQuote($repquerycode));
        $DBQT = &getTool('DBQT');
        // create root node
        $node = $doc->createElement('TOOL');
        $node->setAttribute('CODE', $this->code);
        //adding id if not exists
        $numrow = CLX_NUM_ONE;
        $rs = $DBQT->getTableFieldsSql($querycode, $noderived);
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
		  // add node for each row
		  $occ = $doc->createElement($this->code);
		  $occ->setAttribute('NUMROW', $numrow);
		  $numrow++;
		  foreach ($r as $key => $val) {
			 $field = $doc->createElement($key);
			 $value = $doc->createTextNode(utf8_encode($val));
			 $field->appendChild($value);
			 $occ->appendChild($field);
		  }
		  $node->appendChild($occ);
        }
        $doc->appendChild($node);

        return $doc;
    }

}
