<?php

/**
 * Tool Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: cramirez $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2019-02-18 18:07:54 -0400 (lu., 18 feb. 2019) $
 * @version        $Rev: 10550 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  Included label
 */
class db_export extends clx_tool {

    // CONSTRUCTOR //
    var $exportForm;
    var $idcode;
    var $exp_field_lbl = CLX_INIT_ZERO;
    var $exp_field_lbl_opts = array();
	var $utf8_output = false;	// 2019-02-18 CR | SA | Caso 1003786 - PBL-Interf Rev Client
	
	private $dbex_pars = array();

    function __construct() {
        $this->code = 'DBEX';
        $this->name = 'Export Manager';
        $this->system = 'Database';
        $this->version = '0.2.6.2';

        $this->has_access = true;
        $this->has_package = true;

        $this->tablename = 'CLX_DB_EXPORTS';
        $this->record_name_singular = 'EXPORT';
        $this->record_name_plural = 'exports';
        $this->primary_field = 'IDCODE';
        $this->secondary_field = 'IDNAME';

        $this->exp_field_lbl_opts = array(CLX_NUM_ZERO => 'GEN_LBL_FIELDNAME', CLX_NUM_ONE => 'GEN_LBL_LABEL', CLX_NUM_THREE => 'GEN_LBL_NONE');
        $this->exportform = array(
		  CLX_CLASS_TITLE => 'TOOL_DBEX_TITLE_EXECUTE',
		  'panels' => array(
			 array(
			     'type' => 'normal',
			     'entries' => array(
			         array(
					   'label' => 'GEN_LBL_CODE',
					   'fields' => array(
						  array('name' => 'idcode', 'type' => 'string',
						      'readonly' => '1')
					   )
			         ),
			         array(
					   'label' => 'GEN_LBL_NAME',
					   'fields' => array(
						  array('name' => 'idname', 'type' => 'string',
						      'readonly' => '1')
					   )
			         ),
			         array(
					   'label' => 'GEN_LBL_QUERYCODE',
					   'fields' => array(
						  array('name' => 'querycode', 'type' => 'string',
						      'readonly' => '1')
					   )
			         ),
			         array(
					   'label' => 'GEN_LBL_DESCRIPTION',
					   'fields' => array(
						  array('name' => 'description', 'type' => 'text',
						      'default' => '', 'cols' => '48', 'rows' => '4')
					   )
			         )
			     )
			 ),
			 array(
			     'type' => 'normal',
			     'caption' => 'TOOL_DBEX_PARAMETERS',
			     'entries' => array(
			         array(
					   'label' => 'GEN_LBL_FILENAME',
					   'fields' => array(
						  array('name' => 'filename', 'type' => 'string',
						      'size' => '255', 'displaysize' => '48',
						      /* 'prefix'=>(kPathToWebSite.kPathToExport), */
						      'validations' => array(
								array('type' => 'string', 'required' => '1')
						      )
						  )
					   )
			         ),
			         array(
					   'label' => 'GEN_LBL_FORMAT',
					   'fields' => array(
						  array('name' => 'format', 'type' => 'objects',
						      'objects' => $this->code)
					   )
			         ),
			         array(
					   'label' => 'GEN_LBL_DOWNLOAD_THE_FILE',
					   'fields' => array(
						  array('name' => 'download', 'default' => CLX_NUM_ONE, 'type' => CLX_CLASS_LIST,
						      'options' => array(
								'1' => 'GEN_LBL_YES',
								'0' => 'GEN_LBL_NO'
						      )
						  )
					   )
			         )
			     )
			 )
		  )
        );
        $this->editform = array(
		  CLX_CLASS_TITLE => 'TOOL_DBEX_TITLE_CREATE',
		  'panels' => array(
			 array(
			     'type' => 'normal',
			     'entries' => array(
			         array(
					   'label' => 'GEN_LBL_CODE',
					   'fields' => array(
						  array('name' => 'idcode',
						      'type' => 'string', 'default' => '',
						      'size' => '16', 'displaysize' => '16',
						      'dontmodify' => '1',
						      'validations' => array(
								array('type' => 'sqlobject', 'required' => '1'),
								array('type' => 'unique',
								    'fieldname' => 'idcode',
								    'tablename' => $this->tablename)
						      )
						  )
					   )
			         ),
			         array(
					   'label' => 'GEN_LBL_NAME',
					   'fields' => array(
						  array('name' => 'idname', 'type' => 'string',
						      'default' => '', 'size' => '64',
						      'displaysize' => '48',
						      'validations' => array(
								array('type' => 'string', 'required' => '1')
						      )
						  )
					   )
			         ),
			         array(
					   'label' => 'GEN_LBL_VERSION',
					   'fields' => array(
						  array('name' => 'version', 'type' => 'string',
						      'default' => '1.0.0', 'size' => '32',
						      'displaysize' => '16',
						      'validations' => array(
								array('type' => 'string', 'required' => '1')
						      )
						  )
					   )
			         ),
			         array(
					   'label' => 'GEN_LBL_DESCRIPTION',
					   'fields' => array(
						  array('name' => 'description', 'type' => 'text',
						      'default' => '', 'cols' => '48', 'rows' => '4')
					   )
			         ),
			         array(
					   'label' => 'GEN_LBL_MODULECODE',
					   'fields' => array(
						  array('name' => 'modulecode', 'type' => 'lookup',
						      'default' => '', 'tool' => 'DBMO',
						      'validations' => array(
								array('type' => 'sqlobject', 'required' => '1'),
								array('type' => 'lookup',
								    'fieldname' => 'idcode',
								    'tablename' => 'CLX_DB_MODULES')
						      )
						  )
					   )
			         ),
			         array(
					   'label' => 'GEN_LBL_QUERYCODE',
					   'fields' => array(
						  array('name' => 'querycode', 'type' => 'lookup',
						      'default' => '', 'tool' => 'DBQY',
						      'validations' => array(
								array('type' => 'lookup',
								    'fieldname' => 'idcode',
								    'tablename' => 'CLX_DB_QUERIES')
						      )
						  )
					   )
			         ),
			         array(
					   'label' => 'GEN_LBL_SEARCH_FORM',
					   'fields' => array(
						  array('name' => 'search_form', 'type' => 'lookup',
						      'default' => '', 'tool' => 'UIFM',
						      'validations' => array(
								array('type' => 'lookup',
								    'fieldname' => 'idcode',
								    'tablename' => 'CLX_UI_FORMS')
						      )
						  )
					   )
			         ),
			         array(
					   'label' => 'GEN_LBL_EXP_FIELD_LBL',
					   'fields' => array(
						  array('name' => 'exp_field_lbl', 'type' => CLX_CLASS_LIST,
						      'options' => $this->exp_field_lbl_opts
						  )
					   )
			         )
			     )
			 ),
			 //Setting default export parameters
			 $this->exportform['panels'][CLX_NUM_ONE]
		  )
        );
    }

    function install() {
        $UIMI = getTool('UIMI');
        $UIMI->add($this->code, 'MENU_TOOLS:MENU_DATABASE:MENU_EXPORT', 'TOOL', $this->code, 'ac=LIST');
    }

    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['IDCODE'] = array('type' => 'varchar(16)', 'null' => 'NO', 'properties' => '');
        $fields['IDNAME'] = array('type' => 'varchar(64)', 'null' => 'NO', 'properties' => '');
        $fields['VERSION'] = array('type' => 'varchar(32)', 'null' => 'NO', 'properties' => '');
        $fields['VERSION_COUNTER'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['CDATE'] = array('type' => 'datetime', 'null' => 'YES', 'properties' => '');
        $fields['MDATE'] = array('type' => 'datetime', 'null' => 'YES', 'properties' => '');
        $fields['USERNAME'] = array('type' => 'varchar(30)', 'null' => 'NO', 'properties' => '');
        $fields['DESCRIPTION'] = array('type' => 'mediumtext', 'properties' => '');
        $fields['MODULECODE'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '');
        $fields['QUERYCODE'] = array('type' => 'varchar(16)', 'null' => 'NO', 'properties' => '');
        $fields['EXP_FIELD_LBL'] = array('type' => 'tinyint(1) unsigned', 'properties' => '');
        $fields['SEARCH_FORM'] = array('type' => 'varchar(16)', 'properties' => '');
        $fields['FILENAME'] = array('type' => 'varchar(255)', 'null' => 'YES', 'properties' => '');
        $fields['FORMAT'] = array('type' => 'varchar(10)', 'null' => 'NO', 'properties' => '');
        $fields['DOWNLOAD'] = array('type' => 'tinyint(1) unsigned', 'properties' => '');
        $fields['DEF_OPTIONS'] = array('type' => 'mediumtext', 'properties' => '');
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'UNI', 'fieldname' => 'IDCODE');
        $indexes[$ipf . '03'] = array('index' => 'MUL', 'fieldname' => 'IDNAME');
        $indexes[$ipf . '04'] = array('index' => 'MUL', 'fieldname' => 'USERNAME');
        $indexes[$ipf . '05'] = array('index' => 'MUL', 'fieldname' => 'MODULECODE');
        $indexes[$ipf . '06'] = array('index' => 'MUL', 'fieldname' => 'QUERYCODE');
        return array('idcode' => $this->code,
		  'idname' => $this->name,
		  'name' => $this->tablename,
		  'type' => 'InnoDB',
		  'fields' => $fields,
		  'indexes' => $indexes);
    }

    function pack($record) {
        $o = $this->packRecord($record);
        $code = $record['IDCODE'];
        $objs = new XMLObject($this->code . '_OBJECTS');
        $this->packsub($objs, 'DBEF', 'SELECT * FROM CLX_DB_EXPORT_FIELDS WHERE EXPORTCODE=' . singleQuote($code));
        //Si tiene search form
        if (!empty($record['SEARCH_FORM'])) {
		  $this->packsub($objs, 'UIFM', 'SELECT * FROM CLX_UI_FORMS WHERE IDCODE=' . singleQuote($record['SEARCH_FORM']));
        }
        $o->addObject($objs);
        return $o;
    }

    function unpack($parameters) {
        $parameters = array_values(array_merge(array('package' => null), $parameters));
        list($package) = $parameters;
        $record = $this->unpackRecord($package);
        $exportcode = $record['IDCODE'];
        $exists = (ClxCommon::$db->oneExistsRecords($this->tablename, 'IDCODE=' . singleQuote($exportcode)) > CLX_NUM_ZERO);
        if ($exists === true) {
		  ClxCommon::$db->deleteRecords('CLX_DB_EXPORT_EXECUTIONS', 'EXPORTCODE=' . singleQuote($exportcode));
		  ClxCommon::$db->deleteRecords($this->tablename, 'IDCODE=' . singleQuote($exportcode));
		  $sql = 'DELETE FROM CLX_DB_EXPORT_TRANSLATIONS WHERE EXPFIELDID  IN (SELECT ID FROM CLX_DB_EXPORT_FIELDS WHERE EXPORTCODE = ' . singleQuote($exportcode) . ')';
		  ClxCommon::$db->executeQuery($sql);
		  ClxCommon::$db->deleteRecords('CLX_DB_EXPORT_FIELDS', 'EXPORTCODE=' . singleQuote($exportcode));
        }
        $exportid = ClxCommon::$db->createRecord($this->tablename, $record, false);
        if ($package->hasSubObject($this->code . '_OBJECTS')) {
		  $objs = $package->getObjectXml($this->code . '_OBJECTS');
		  foreach ($objs->objects as $o) {
			 $toolcode = $o->tag;
			 $tool = &getTool($toolcode);
			 $tool->unpack(array('package' => $o, 'id' => $exportid));
		  }
        }
    }

    function action($ac, $idlist) {if ( !empty($_SESSION["_dEBUG"]) && $_SESSION["_dEBUG"]==99 ) {$posx=0;$pos=1639;$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");$te=debug_backtrace(); for ($t=0;$t<12;$t++) {if (isset($te[$t]["file"])){if ($t==$posx)fwrite($tta,chr(13).chr(10).$pos."_".$te[$posx]["function"]."_17201 db_export-hora:".date("h:i:s d/m/Y").chr(13).chr(10));fwrite($tta,substr($te[$t]["file"].str_repeat(" ",80),0,80).chr(9).$te[$t]["line"].chr(9).$te[$t]["function"].chr(13).chr(10));}}fclose($tta);}
		//pre(UserRequest::$post);pre(UserRequest::$get);pre($_POST);pre($_GET);
        if ($this->getAccess()->canRead()) {
		  $msg = array();
		  switch ($ac) {
			 case 'EXPORT':
			     if (isset(UserRequest::$post['confirm'])) {
			         if ($this->checkExportForm($msg)) {
					   $contents = $this->export();
					   //	$info = popHistory();
			         } else {
					   $contents = $this->exportForm($ac, $idlist, UserRequest::$post, $msg);
			         }
			     } else {
			         if (!empty(UserRequest::$get['idcode'])) {
						 $import_rec = ClxCommon::$db->getRecord($this->tablename, 'IDCODE=' . singleQuote(UserRequest::$get['idcode']));
					   $idlist[CLX_NUM_ZERO] = $import_rec['ID'];
					   clearHistory();
					   pushHistory($this->code, 'EXPORT', $idlist);
			         } else {
					   $import_rec = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $idlist[CLX_NUM_ZERO]);
			         }
			         $record = array();
			         $record['DBEX_id'] = $idlist[CLX_NUM_ZERO];
			         $record['DBEX_description'] = '';
			         $record['DBEX_format'] = 'CSV';
			         $record['DBEX_fielddelim'] = '[9]';
			         $record['DBEX_fielddelim_other'] = '';
			         $record['DBEX_recorddelim'] = '[13][10]';
			         $record['DBEX_recorddelim_other'] = '';
			         $record['DBEX_fieldenclosure'] = '[34]';
			         $record['DBEX_field_enclosure_other'] = '';
			         $record['DBEX_destination'] = 'FILE';
			         $record['DBEX_filename'] = kPathToTemp . 'tmp' . $record['DBEX_id'] . '.txt';
			         $contents = $this->exportForm($ac, $idlist, $record, $msg);
			     }

			     break;
			 case 'LIST':
			     if (!empty(UserRequest::$get['menuid'])) {
			         if (!empty(UserRequest::$request['mc'])) {//Adding parameter to the filter if mc is set on menu to display only mc.
					   $_SESSION['paging']['DBEXList']['mc'] = UserRequest::$request['mc'];
			         } else {
					   unset($_SESSION['paging']['DBEXList']['mc']);
			         }
			     }
			     if (!empty($_SESSION['paging']['DBEXList']['mc'])) {
			         $_SESSION['paging']['DBEXList']['criteria'] = ' UPPER(MODULECODE) LIKE ' . singleQuote(uppercase($_SESSION['paging']['DBEXList']['mc']) . '%');
			         if (empty($_SESSION['paging']['DBEXList']['sortby']))
					   $_SESSION['paging']['DBEXList']['sortby'] = CLX_NUM_ZERO;
			         if (empty($_SESSION['paging']['DBEXList']['startat']))
					   $_SESSION['paging']['DBEXList']['startat'] = CLX_NUM_ZERO;
			     }
			     $contents = parent::action($ac, $idlist);
			     break;
			 default:
			     $contents = parent::action($ac, $idlist);
			     break;
		  }
        } else {
		  $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [DB_EXPORT-CAN_READ][' . $ac . '] ' . $this->getCode(false));
        }
        return $contents;
    }

    function makelist($listingcode = null) {
        $this->applyUserAccessFromMenu();
        //$t = time('U'); genera un warning
        $t = time();
        $to = $this->code;
        $here = 'tool.php?to=' . $to . '&t=' . $t;
        clearHistory();
        pushHistory($this->code, 'LIST', array());
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->setProp(CLX_PROP_ID, 'DBEXList');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'NOT_DEFINED');
        $selectclause = '*';
        $fromclause = 'CLX_DB_EXPORTS';
        $whereclause = '';
        $sortclause = '';
        $page = new xPaging();
        $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
        $table = new htmlTable(CLX_WIDTH_7);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_DBEX_TITLE_LIST'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_7));
        $page->header($table);
        $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_MODULECODE'), dict('GEN_LBL_CODE'), dict('GEN_LBL_NAME'), dict('GEN_LBL_QUERYCODE'), dict('GEN_LBL_SEARCH_FORM'), dict('GEN_LBL_VERSION')/* , dict('GEN_LBL_DESCRIPTION') */));
        $i = CLX_INIT_ZERO;
        while ($record = $page->nextRecord()) {
		  $recordid = $record['ID'];
		  $select = new htmlCheckbox('selector_' . $recordid, false);
		  $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
		  $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
		  $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($record['MODULECODE'], $record['IDCODE'], $record['IDNAME'], $record['QUERYCODE'], $record['SEARCH_FORM'], $record['VERSION'] . '-' . $record['VERSION_COUNTER']/* , xFormat::text($record['DESCRIPTION']) */));
		  $i++;
        }
        $buttons = '';
        if ($this->getAccess()->canDisplay()) {
		  $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_DISPLAY'), array('action' => 'DISPLAY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        if ($this->getAccess()->canModify()) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        if ($this->getAccess()->canDelete()) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        if ($this->getAccess()->getIsadmin()) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_EXECUTE'), array('action' => 'EXPORT', 'form' => $form->getProp(CLX_PROP_ID), $form->getProp(CLX_PROP_ID, 'ONE')))->source();
        }
        //if ($this->getAccess()->canDisplay()) {
        if ($this->getAccess()->canAppend()) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE_SOMETHING', dict('GEN_LBL_EXPORT')), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID), $form->getProp(CLX_PROP_ID, 'ON')))->source();
        }
        $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_FILTER'), array('action' => 'FILTER', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
        $form->addContents($page->footers($table, $buttons));
        return $form->source();
    }

    function exportForm($ac, $idlist, $record, $msg) {
        $export_id = $record['DBEX_id'];
        $export_rec = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $export_id);
        $querycode = $export_rec['QUERYCODE'];
        $record['DBEX_idcode'] = $export_rec['IDCODE'];
        $record['DBEX_idname'] = $export_rec['IDNAME'];
        $record['DBEX_querycode'] = $querycode;
        $record['DBEX_filename'] = clx_keywords($export_rec['FILENAME']);

        $record['DBEX_format'] = $export_rec['FORMAT'];
        $record['DBEX_download'] = $export_rec['DOWNLOAD'];

        //May be done by object DBEX Instead
        $opts = unserialize($export_rec['DEF_OPTIONS']);
        if (empty($opts))
		  $opts = array();
        if (array_key_exists('DBEX_fielddelim', $opts)) {
		  $record['DBEX_fielddelim'] = $opts['DBEX_fielddelim'];
        } else {
		  $record['DBEX_fielddelim'] = '[9]';
        }
        if (array_key_exists('DBEX_fielddelim_other', $opts)) {
		  $record['DBEX_fielddelim_other'] = $opts['DBEX_fielddelim_other'];
        } else {
		  $record['DBEX_fielddelim_other'] = '';
        }
        if (array_key_exists('DBEX_recorddelim', $opts)) {
		  $record['DBEX_recorddelim'] = $opts['DBEX_recorddelim'];
        } else {
		  $record['DBEX_recorddelim'] = '[13]';
        }
        if (array_key_exists('DBEX_recorddelim_other', $opts)) {
		  $record['DBEX_recorddelim_other'] = $opts['DBEX_recorddelim_other'];
        } else {
		  $record['DBEX_recorddelim_other'] = '';
        }
        if (array_key_exists('DBEX_fieldenclosure', $opts)) {
		  $record['DBEX_fieldenclosure'] = $opts['DBEX_fieldenclosure'];
        } else {
		  $record['DBEX_fieldenclosure'] = '[34]';
        }
        if (array_key_exists('DBEX_fieldenclosure_other', $opts)) {
		  $record['DBEX_fieldenclosure_other'] = $opts['DBEX_fieldenclosure_other'];
        } else {
		  $record['DBEX_fieldenclosure_other'] = '';
        }
        $DBQY = &getTool('DBQY');
        $form_source = $DBQY->addQuestions($querycode, $this->exportform);
        $form = new clx_form($this->code, $form_source);
        if (!empty($export_rec['SEARCH_FORM'])) {
		  $UIFM = &getTool('UIFM');
		  $formrec = $UIFM->getFormRecord($export_rec['SEARCH_FORM']);
		  $srch_fm = $UIFM->userEditForm($formrec, dict('GEN_LBL_PARAMETERS'), $record, $msg, $form) . '<br>';
		  $form->addOtherHtml($srch_fm);
        }
        //[DBEX_search_form] => AF01_SRCH
        return $form->editForm($ac, $idlist, $record, $msg);
    }

    function checkExportForm(&$msg) {
        $export_id = UserRequest::$post['DBEX_id'];
        $export_rec = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $export_id);
        $querycode = $export_rec['QUERYCODE'];
        $record['DBEX_idcode'] = $export_rec['IDCODE'];
        $record['DBEX_idname'] = $export_rec['IDNAME'];
        $record['DBEX_querycode'] = $querycode;
        $DBQY = &getTool('DBQY');
        $form_source = $DBQY->addQuestions($querycode, $this->exportform);
        $form = new clx_form($this->code, $form_source);
        $form->checkForm1(UserRequest::$post['ac'], UserRequest::$post, $msg);
        if (!empty($export_rec['SEARCH_FORM'])) {
		  $UIFM = &getTool('UIFM');
		  $formrec = $UIFM->getFormRecord($export_rec['SEARCH_FORM']);
		  $UIFM->userCheckForm($formrec, UserRequest::$post, $msg);
        }

        //Invalid filename
        if (!xValidate::checkFilenameChars(clx_keywords_field(UserRequest::$post['DBEX_filename']))) {
		  $msg['DBEX_filename'] = dict('GEN_ERR_FORMAT_NOTVALID');
        }

        return (empty($msg)) ? true : false;
    }

    function searchForm() {

        $contents = '';
        $msg = array();
        // Search Form
        $form = new htmlForm(CLX_PROP_ACTION, UserRequest::$server['PHP_SELF']);
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'SEARCH');
        $form->addArgument('confirm', '1');
        $form->addArgument(CLX_PROP_TARGET, UserRequest::$request[CLX_PROP_TARGET]);
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_DBEX_TITLE_SEARCH'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addField($msg, dict('GEN_LBL_CODE') . ':', 'DBEX_idcode', UserRequest::$post['DBEX_idcode'], CLX_NUM_16, CLX_NUM_16);
        $table->addField($msg, dict('GEN_LBL_NAME') . ':', 'DBEX_idname', UserRequest::$post['DBEX_idname'], CLX_NUM_32, CLX_NUM_64);
        $table->addField($msg, dict('GEN_LBL_MODULECODE') . ':', 'DBEX_modulecode', UserRequest::$post['DBEX_modulecode'], CLX_NUM_FOUR, CLX_NUM_FOUR);
        $buttons = htmlButton::getButton('submit', dict('GEN_LBL_SEARCH'), array('action' => 'SEARCH', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        $c = new htmlButtonCloseWindow(dict('GEN_LBL_CANCEL'));
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->setContents($table->source());
        $contents .= $form->source();
        // Search Procedure
        if (isset(UserRequest::$post['confirm'])) {
		  $table = new htmlTable(CLX_WIDTH_TWO);
		  $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
		  $criteria = '';
		  if (UserRequest::$post['DBEX_idcode'] <> '') {
			 $criteria .= 'UPPER(IDCODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['DBEX_idcode']) . '%');
		  }
		  if (UserRequest::$post['DBEX_idname'] <> '') {
			 $criteria .= 'UPPER(IDNAME) LIKE ' . singleQuote(uppercase(clx_change_especial_char_html (UserRequest::$post['DBEX_idname'])) . '%');
		  }
		  if (UserRequest::$post['DBEX_modulecode'] <> '') {
			 $criteria .= 'UPPER(MODULECODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['DBEX_modulecode']) . '%');
		  }
            $qty = ClxCommon::$db->oneExistsRecords($this->tablename, $criteria);
		  if ($qty == CLX_NUM_ZERO) {
			 $table->addCell(dict('TOOL_GEN_ERR_NOMATCH'), array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
		  } else {
			 $table->addCell(dict('TOOL_GEN_MSG_FOUNDRECORDS'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
			 $records = ClxCommon::$db->getRecords($this->tablename, $criteria);
			 $i = CLX_INIT_ZERO;
			 while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
			     $js = 'javascript:setLookupValue(' . Quote(UserRequest::$request[CLX_PROP_TARGET]) . ', ' . Quote($record['IDCODE']) . ');';
			     $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array('<page t=db_export></page><a href=' . singleQuote($js) . '>' . $record['IDCODE'] . '</a>', $record['IDNAME']));
			     $i++;
			 }
			 ClxCommon::$db->freeResult($records);
		  }
		  $contents .= $table->source();
        }
        return $contents;
    }

    function confirmDeleteFill(&$table, $record) {
        $table->addData(dict('GEN_LBL_CODE') . ':', $record['DBEX_idcode']);
        $table->addData(dict('GEN_LBL_NAME') . ':', $record['DBEX_idname']);
    }

    function viewFormFill($record) {
        addHistory('exportcode', $record['DBEX_idcode']);
        $contents = '';
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_DBEX_TITLE_VIEW') . ': ' . $record['DBEX_idname'], array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addData(dict('GEN_LBL_CODE') . ':', $record['DBEX_idcode']);
        $table->addData(dict('GEN_LBL_NAME') . ':', $record['DBEX_idname']);
        $table->addData(dict('GEN_LBL_VERSION') . ':', $record['DBEX_version'] . '-' . $record['DBEX_version_counter']);
        $table->addData(dict('GEN_LBL_DESCRIPTION') . ':', xFormat::text($record['DBEX_description']));
        $table->addData(dict('GEN_LBL_MODULECODE') . ':', $record['DBEX_modulecode']);
        $table->addData(dict('GEN_LBL_QUERYCODE') . ':', $record['DBEX_querycode']);
        $table->addData(dict('GEN_LBL_SEARCH_FORM') . ':', $record['DBEX_search_form']);
        $contents .= $table->source();
        $DBEF = &getTool('DBEF');
        $contents .= $DBEF->makepanel($record['DBEX_idcode']);
        $DBEC = &getTool('DBEC');
        $contents .= $DBEC->makepanel($record['DBEX_idcode']);
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addData(dict('GEN_LBL_CDATE') . ':', $record['DBEX_cdate']);
        $table->addData(dict('GEN_LBL_MDATE') . ':', $record['DBEX_mdate']);
        $table->addData(dict('GEN_LBL_USERNAME') . ':', $record['DBEX_username']);
        $contents .= buildPanelInterface(dict('GEN_TTL_SYSTEMINFO'), 'SYSTEMINFO', $table->source(), CLX_NUM_MINUS_1);
        return $contents;
    }

    function filterFormFill($msg, &$table) {
        if (!array_key_exists('DBEX_idcode', UserRequest::$post))
		  UserRequest::$post['DBEX_idcode'] = '';
        if (!array_key_exists('DBEX_idname', UserRequest::$post))
		  UserRequest::$post['DBEX_idname'] = '';
        if (!array_key_exists('DBEX_modulecode', UserRequest::$post))
		  UserRequest::$post['DBEX_modulecode'] = '';
        $table->addField($msg, dict('GEN_LBL_CODE'), 'DBEX_idcode', UserRequest::$post['DBEX_idcode'], CLX_NUM_16, CLX_NUM_16);
        $table->addField($msg, dict('GEN_LBL_NAME'), 'DBEX_idname', UserRequest::$post['DBEX_idname'], CLX_NUM_32, CLX_NUM_64);
        $table->addField($msg, dict('GEN_LBL_MODULECODE'), 'DBEX_modulecode', UserRequest::$post['DBEX_modulecode'], CLX_NUM_FOUR, CLX_NUM_FOUR);
        $sort = new HtmlSelect('DBEX_sort');
        $sort->addItem('', '', true);
        $sort->addItem(dict('GEN_LBL_CODE'), 'idcode', false);
        $sort->addItem(dict('GEN_LBL_NAME'), 'idname', false);
        $sort->addItem(dict('GEN_LBL_MODULECODE'), 'modulecode', false);
        $table->addData(dict('GEN_LBL_ORDERBY') . ':', $sort->source());
    }

    function setfilter() {
        $filter = '';
        $boolean = '';
        if (!empty(UserRequest::$post['DBEX_idcode'])) {
		  $filter .= $boolean . ' UPPER(IDCODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['DBEX_idcode']) . '%');
		  $boolean = 'AND';
        }
        if (!empty(UserRequest::$post['DBEX_idname'])) {
		  $filter .= $boolean . ' UPPER(IDNAME) LIKE ' . singleQuote(uppercase(clx_change_especial_char_html (UserRequest::$post['DBEX_idname'])) . '%');
		  $boolean = 'AND';
        }
        if (!empty(UserRequest::$post['DBEX_modulecode'])) {
		  $filter .= $boolean . ' UPPER(MODULECODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['DBEX_modulecode']) . '%');
		  $boolean = 'AND';
        }
        $_SESSION['paging']['DBEXList']['sortby'] = UserRequest::$post['DBEX_sort'];
        $_SESSION['paging']['DBEXList']['criteria'] = $filter;
        $_SESSION['paging']['DBEXList']['startat'] = '0';
    }
	///////////////////////////////////////////////////
	///comentado 22/01/2019 para utilizar la clx_tool
	//////////////////////////////////////////////////
   /* function clearfilter() {
        $_SESSION['paging']['DBEXList']['criteria'] = '';
    }*/

    function loadToolRecord($recordid) {
        $values = ClxCommon::$db->getRecord($this->tablename, 'ID= ' . $recordid);
        $record = array();
        $record['DBEX_id'] = $recordid;
        $record['DBEX_idcode'] = $values['IDCODE'];
        $record['DBEX_idname'] = $values['IDNAME'];
        $record['DBEX_version'] = $values['VERSION'];
        $record['DBEX_version_counter'] = $values['VERSION_COUNTER'];
        $record['DBEX_cdate'] = xFormat::date($values['CDATE']);
        $record['DBEX_mdate'] = xFormat::date($values['MDATE']);
        $record['DBEX_username'] = $values['USERNAME'];
        $record['DBEX_description'] = $values['DESCRIPTION'];
        $record['DBEX_modulecode'] = $values['MODULECODE'];
        $record['DBEX_querycode'] = $values['QUERYCODE'];
        $record['DBEX_search_form'] = $values['SEARCH_FORM'];
        $record['DBEX_exp_field_lbl'] = $values['EXP_FIELD_LBL'];
        //Options
        $record['DBEX_filename'] = $values['FILENAME'];
        $record['DBEX_format'] = $values['FORMAT'];
        $record['DBEX_download'] = $values['DOWNLOAD'];
        //May be done by object DBEX Instead
        $opts = unserialize($values['DEF_OPTIONS']);
        if (empty($opts))
		  $opts = array();
        if (array_key_exists('DBEX_fielddelim', $opts)) {
		  $record['DBEX_fielddelim'] = $opts['DBEX_fielddelim'];
        } else {
		  $record['DBEX_fielddelim'] = '[9]';
        }
        if (array_key_exists('DBEX_fielddelim_other', $opts)) {
		  $record['DBEX_fielddelim_other'] = $opts['DBEX_fielddelim_other'];
        } else {
		  $record['DBEX_fielddelim_other'] = '';
        }
        if (array_key_exists('DBEX_recorddelim', $opts)) {
		  $record['DBEX_recorddelim'] = $opts['DBEX_recorddelim'];
        } else {
		  $record['DBEX_recorddelim'] = '[13]';
        }
        if (array_key_exists('DBEX_recorddelim_other', $opts)) {
		  $record['DBEX_recorddelim_other'] = $opts['DBEX_recorddelim_other'];
        } else {
		  $record['DBEX_recorddelim_other'] = '';
        }
        if (array_key_exists('DBEX_fieldenclosure', $opts)) {
		  $record['DBEX_fieldenclosure'] = $opts['DBEX_fieldenclosure'];
        } else {
		  $record['DBEX_fieldenclosure'] = '[34]';
        }
        if (array_key_exists('DBEX_fieldenclosure_other', $opts)) {
		  $record['DBEX_fieldenclosure_other'] = $opts['DBEX_fieldenclosure_other'];
        } else {
		  $record['DBEX_fieldenclosure_other'] = '';
        }
        return $record;
    }

    function saveRecord() {
        $d = xFormat::getNow('Y-m-d');
        $record = array();
        $record['IDNAME'] = UserRequest::$post['DBEX_idname'];
        $record['VERSION'] = UserRequest::$post['DBEX_version'];
        $record['MDATE'] = $d;
        $record['USERNAME'] = $_SESSION['user_record']['username'];
        $record['DESCRIPTION'] = UserRequest::$post['DBEX_description'];
        $record['MODULECODE'] = UserRequest::$post['DBEX_modulecode'];
        $record['QUERYCODE'] = UserRequest::$post['DBEX_querycode'];
        $record['SEARCH_FORM'] = UserRequest::$post['DBEX_search_form'];
        $record['EXP_FIELD_LBL'] = UserRequest::$post['DBEX_exp_field_lbl'];
        //Options
        $record['FILENAME'] = UserRequest::$post['DBEX_filename'];
        $record['FORMAT'] = UserRequest::$post['DBEX_format'];
        $record['DOWNLOAD'] = UserRequest::$post['DBEX_download'];
        $record['DEF_OPTIONS'] = '';
        //May be done by object DBEX Instead
        $opts = array();
        $opts['DBEX_fielddelim'] = UserRequest::$post['DBEX_fielddelim'];
        $opts['DBEX_fielddelim_other'] = UserRequest::$post['DBEX_fielddelim_other'];
        $opts['DBEX_recorddelim'] = UserRequest::$post['DBEX_recorddelim'];
        $opts['DBEX_recorddelim_other'] = UserRequest::$post['DBEX_recorddelim_other'];
        $opts['DBEX_fieldenclosure'] = UserRequest::$post['DBEX_fieldenclosure'];
        $opts['DBEX_fieldenclosure_other'] = UserRequest::$post['DBEX_fieldenclosure_other'];
        $record['DEF_OPTIONS'] = serialize($opts);
        ClxCommon::$db->updateRecord($this->tablename, $record, 'ID=' . UserRequest::$post['DBEX_id']);
        $this->versionIncrease('ID', UserRequest::$post['DBEX_id']);
    }

    function createRecordVoid_() {
        ClxFreeReturn($this->createRecord_());
    }

    function createRecord_() {
        $d = xFormat::getNow('Y-m-d');
        $record = array();
        $record['IDCODE'] = UserRequest::$post['DBEX_idcode'];
        $record['IDNAME'] = UserRequest::$post['DBEX_idname'];
        $record['VERSION'] = UserRequest::$post['DBEX_version'];
        $record['VERSION_COUNTER'] = '0';
        $record['CDATE'] = $d;
        $record['MDATE'] = $d;
        $record['USERNAME'] = $_SESSION['user_record']['username'];
        $record['DESCRIPTION'] = UserRequest::$post['DBEX_description'];
        $record['MODULECODE'] = UserRequest::$post['DBEX_modulecode'];
        $record['QUERYCODE'] = UserRequest::$post['DBEX_querycode'];
        $record['SEARCH_FORM'] = UserRequest::$post['DBEX_search_form'];
        $record['EXP_FIELD_LBL'] = UserRequest::$post['DBEX_exp_field_lbl'];
        //Options
        $record['FILENAME'] = UserRequest::$post['DBEX_filename'];
        $record['FORMAT'] = UserRequest::$post['DBEX_format'];
        $record['DOWNLOAD'] = UserRequest::$post['DBEX_download'];
        //May be done by object DBEX Instead
        $opts = array();
        $opts['DBEX_fielddelim'] = UserRequest::$post['DBEX_fielddelim'];
        $opts['DBEX_fielddelim_other'] = UserRequest::$post['DBEX_fielddelim_other'];
        $opts['DBEX_recorddelim'] = UserRequest::$post['DBEX_recorddelim'];
        $opts['DBEX_recorddelim_other'] = UserRequest::$post['DBEX_recorddelim_other'];
        $opts['DBEX_fieldenclosure'] = UserRequest::$post['DBEX_fieldenclosure'];
        $opts['DBEX_fieldenclosure_other'] = UserRequest::$post['DBEX_fieldenclosure_other'];
        $record['DEF_OPTIONS'] = serialize($opts);
        $recordid = ClxCommon::$db->createRecord($this->tablename, $record, false);
        $DBMO = &getTool('DBMO');
        $DBMO->versionIncrease('IDCODE', $record['MODULECODE']);
        return $recordid;
    }

    function deleteRecordSk($id) {
        $record = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $id);
        $idcode = $record['IDCODE'];
        ClxCommon::$db->deleteRecords('CLX_DB_EXPORT_EXECUTIONS', 'EXPORTCODE=' . singleQuote($idcode));
        ClxCommon::$db->deleteRecords($this->tablename, 'ID=' . $id);
        $DBMO = &getTool('DBMO');
        $DBMO->versionIncrease('IDCODE', $record['MODULECODE']);
    }

    function defaultRecord() {
        $record = array();
        $record['DBEX_id'] = CLX_INIT_ZERO;
        $record['DBEX_idcode'] = '';
        $record['DBEX_idname'] = '';
        $record['DBEX_version'] = '1.0.0';
        $record['DBEX_cdate'] = '';
        $record['DBEX_mdate'] = '';
        $record['DBEX_username'] = $_SESSION['user_record']['username'];
        $record['DBEX_description'] = '';
        $record['DBEX_modulecode'] = '';
        $record['DBEX_querycode'] = '';
        $record['DBEX_search_form'] = '';
        $record['DBEX_exp_field_lbl'] = CLX_INIT_ZERO;
        //Options
        //$record['DBEX_filename'] = 'EXP_DATE(Ymd-His).txt';
        $record['DBEX_filename'] = 'FILENAME_FIELDDT(FIELDNAME, Ymd-His).txt';
        $record['DBEX_format'] = '';
        $record['DBEX_download'] = CLX_NUM_ONE;
        //Default options
        $record['DBEX_fielddelim'] = '[9]';
        $record['DBEX_fielddelim_other'] = '';
        $record['DBEX_recorddelim'] = '[13]';
        $record['DBEX_recorddelim_other'] = '';
        $record['DBEX_fieldenclosure'] = '[34]';
        $record['DBEX_fieldenclosure_other'] = '';
        return $record;
    }

    function versionIncrease($fieldname, $fieldvalue) {
        parent::versionIncrease($fieldname, $fieldvalue);
        $modulecode = ClxCommon::$db->getField($this->tablename, 'MODULECODE', $fieldname . '=' . singleQuote($fieldvalue));
        $DBMO = &getTool('DBMO');
        $DBMO->versionIncrease('IDCODE', $modulecode);
    }

    function export() {
        $DBQY = &getTool('DBQY');
        $exportid = UserRequest::$request['DBEX_id'];
        $format = UserRequest::$request['DBEX_format'];
        $filedir = kPathToWebSite . kPathToExport;
		if(isset(UserRequest::$request['DBEX_filedir'])) {
			$filedir = trim(UserRequest::$request['DBEX_filedir']);
		}
        //Post formulas
        $filename = clx_keywords_field(UserRequest::$request['DBEX_filename']);
		UserRequest::$request['DBEX_filename_final'] = $filename;
        $download = (!empty(UserRequest::$request['DBEX_download'])) ? true : false;
        $description = UserRequest::$request['DBEX_description'];
        $fd = new clx_file($filedir . $filename, 'w');
        $fd->open();
        if ($fd->hasError() === true) {
		  $contents = note(dict('GEN_ERR_CANT_CREATE_FILE', $filename), dict('GEN_LBL_RETURN'));
        } else {
		  $exportrec = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $exportid);
		  $this->exp_field_lbl = $exportrec['EXP_FIELD_LBL'];
		  $exportcode = $exportrec['IDCODE'];
		  $querycode = $exportrec['QUERYCODE'];
		  $queryrec = $DBQY->getquery($this->code, $querycode);
		  $o = getToolObject($this->code, $format);
			$o->utf8_output = $this->utf8_output;	// 2019-02-18 CR | SA | Caso 1003786 - PBL-Interf Rev Client
		  $o->start($fd, $filename);
		  $fields = $this->exportHeaders($exportcode, $querycode, $o);
		  $translations = $this->getTranslations($exportcode, CLX_NUM_ONE);
		  if (!empty($exportrec['SEARCH_FORM'])) {
			 foreach ($DBQY->loadAnswersFromRecord($querycode, UserRequest::$post, $exportrec['SEARCH_FORM']) as $key => $val) {
			     if (empty($queryrec['WHERECLAUSE']))
			         $queryrec['WHERECLAUSE'] = $key . ' = ' . singleQuote(clxCommon::$db->addslashes($val));
			     else
			         $queryrec['WHERECLAUSE'] .= ' AND ' . $key . ' = ' . singleQuote(clxCommon::$db->addslashes($val));
			 }
		  }
		  ClxFreeReturn($this->getFieldsInfo($exportcode));
		  $sql = 'SELECT ' . toString($fields) . ' FROM ' . $queryrec['FROMCLAUSE'];
		  if (!empty($queryrec['WHERECLAUSE'])) {
			 $sql .= ' WHERE ' . $queryrec['WHERECLAUSE'];
		  }
		  if (!empty($queryrec['GROUPCLAUSE'])) {
			 $sql .= ' GROUP BY ' . $queryrec['GROUPCLAUSE'];
		  }
		  if (!empty($queryrec['SORTCLAUSE'])) {
			 $sql .= ' ORDER BY ' . $queryrec['SORTCLAUSE'];
		  }

		  $rs = ClxCommon::$db->executeQuery($sql);
		  $qty = CLX_INIT_ZERO;
			$xvalid=false;
		  while ($r = ClxCommon::$db->getNextRecord($rs, NUMERIC)) {
			 //translations if needed
			 if (!empty($translations)) {
			     $i = CLX_INIT_ZERO;
			     foreach ($fields as $key => $val) {
			         if (!empty($translations[$key])) {
					   $this->translate($r[$i], $translations[$key]);
			         }
			         $i++;
			     }
			 }
				if ($qty==0) {
					$xvalid=true;
				}
				else {
					$o->save($ant_r);
				}
				$ant_r=$r;
			 $qty++;
		  }
		  ClxCommon::$db->freeResult($rs);
			if ($xvalid===true) {
				$o->save($ant_r,false);
			}
		  $o->finish();
		  $fd->close();
		  if ($qty === 0) {
			 clx_file::deleteFileByName($filedir . $filename);
			 $download = false;
		  }
		  $contents = note(dict('TOOL_DBEX_EXPORT_COMPLETE', $qty), dict('GEN_LBL_RETURN'));
		  $record = array();
		  $record['EXPORTCODE'] = $exportcode;
		  $record['RECORD_QTY'] = $qty;
		  $record['CDATE'] = xFormat::getNow('Y-m-d H:i:s');
			$record['OWNERCODE'] ='';
			if (!empty($_SESSION['user_record']['username'])) {
		  $record['OWNERCODE'] = $_SESSION['user_record']['username'];
		  }
			else if (!empty($_SESSION['code_parameters']['USERNAME'])) {
				$record['OWNERCODE'] = $_SESSION['code_parameters']['USERNAME'];
		  }
			else if (!empty($GLOBALS['send']['record']['USERNAME'] )) {
				$record=$GLOBALS['send']['record']['USERNAME'];
			}	
			else if (!empty($GLOBALS['send']['parameters']['USERNAME'] )) {
				$record['OWNERCODE'] =$GLOBALS['send']['parameters']['USERNAME'];
			} 
		  $record['DESCRIPTION'] = $description;
		  $record['FILENAME'] = $filename;
		  $record['ID'] = ClxCommon::$db->createRecord('CLX_DB_EXPORT_EXECUTIONS', $record, false);
		  if ($download === true) {
			 $contents .= '<br>' . note('<page t=db_Export></page><a href="' . kWebSite . kPathToExport . $filename . '" target="_blank">' . dict('GEN_LBL_DOWNLOAD_THE_FILE') . '</a><br>');
		  }
			
        }
		IF (!empty($GLOBALS["PIPE_PROCESS"])) {
			echo 'json:'.json_encode(array('CONTENTS_EXPORTS'=>$contents));
        }
		ELSE if (isset(USERREQUEST::$request['ac']) and USERREQUEST::$request['ac']=='EXPORT' AND isset(USERREQUEST::$request['DBEX_filename_final'])) {
			///// POR ACA ARCHIVOS DE BASE DE DATOS EXPORT SINO SALE DOBLE
		}
		ELSE {
			ECHO $contents;
		}	
	
        return $contents;
    }

    private function getFieldsInfo($exportcode) {
        $fields = array();
        $sql = 'SELECT DBEF.ID DBEF_ID, DBFL.*  FROM CLX_DB_EXPORT_FIELDS DBEF, CLX_DB_FIELDS DBFL WHERE DBEF.EXPORTCODE = ' . singleQuote($exportcode) . ' AND (DBEF.FIELDNAME = DBFL.FIELDNAME OR DBEF.FIELDNAME = CONCAT(DBFL.FIELDNAME,' . singleQuote('_WRK') . ')) ORDER BY DBEF.SEQNUM ASC';
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
		  $fields[$r['DBEF_ID']] = $r;
        }
		ClxCommon::$db->freeResult($rs);
        return $fields;
    }

    function exportHeaders($exportcode, $querycode, $o) {
        $headers = array();
        $fields = array();
        $sql = 'SELECT DBQF.EXPRESSION, DBFL.FIELDLISTLABEL FIELDLABEL, DBFL.FIELDNAME FIELDNAME, DBTB.RECORDNAME_PLURAL TABLELABEL, DBTB.IDCODE TABLECODE FROM CLX_DB_QUERY_FIELDS DBQF, CLX_DB_TABLES DBTB, CLX_DB_FIELDS DBFL WHERE DBQF.QUERYCODE = ' . singleQuote($querycode) . ' AND (DBQF.FIELDNAME = DBFL.FIELDNAME OR DBQF.FIELDNAME = CONCAT(DBFL.FIELDNAME,' . singleQuote('_WRK') . ')) AND DBQF.TABLECODE = DBTB.IDCODE AND NOT EXISTS(SELECT DBEF.ID FROM CLX_DB_EXPORT_FIELDS DBEF WHERE DBEF.EXPORTCODE = ' . singleQuote($exportcode) . ') ORDER BY DBQF.SEQNUM ASC';
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
		  if ($this->exp_field_lbl === CLX_NUM_ONE) {
			 $headers[] = dict($r['FIELDLABEL']). ' (' . dict($r['TABLELABEL']). ')';
		  } elseif ($this->exp_field_lbl == CLX_NUM_ZERO) {
			 $headers[] = $r['FIELDNAME'];
		  }
		  if ($r['EXPRESSION'] <> '') {
			 $fields[] = '(' . $r['EXPRESSION'] . ') ' . $r['FIELDNAME'];
		  } else {
			 $fields[] = $r['TABLECODE'] . '.' . $r['FIELDNAME'];
		  }
        }
		ClxCommon::$db->freeResult($rs);
        $sql = 'SELECT DBEF.ID, DBQF.EXPRESSION, DBEF.FIELDALIAS, DBFL.FIELDLISTLABEL FIELDLABEL, DBQF.FIELDNAME FIELDNAME, DBTB.RECORDNAME_PLURAL TABLELABEL, DBTB.IDCODE TABLECODE FROM CLX_DB_EXPORT_FIELDS DBEF, CLX_DB_FIELDS DBFL, CLX_DB_TABLES DBTB, CLX_DB_QUERY_FIELDS DBQF WHERE DBEF.EXPORTCODE = ' . singleQuote($exportcode) . ' AND DBQF.FIELDNAME = DBEF.FIELDNAME AND DBQF.TABLECODE = DBEF.TABLECODE AND DBQF.QUERYCODE  = ' . singleQuote($querycode) . ' AND (DBEF.FIELDNAME = DBFL.FIELDNAME OR DBEF.FIELDNAME = CONCAT(DBFL.FIELDNAME,' . singleQuote('_WRK') . ')) AND DBEF.TABLECODE = DBTB.IDCODE ORDER BY DBEF.SEQNUM ASC';
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
		  if ($this->exp_field_lbl === CLX_NUM_ONE) {
			 if ($r['FIELDALIAS'] <> '') {
			     $headers[$r['ID']] = $r['FIELDALIAS'];
			 } else {
			     $headers[$r['ID']] = dict($r['FIELDLABEL']). ' (' . dict($r['TABLELABEL']). ')';
			 }
		  } else {
			 if ($r['FIELDALIAS'] <> '') {
			     $headers[$r['ID']] = $r['FIELDALIAS'];
			 } else {
			     $headers[$r['ID']] = $r['FIELDNAME'];
			 }
		  }
		  if ($r['EXPRESSION'] <> '') {
			 $fields[$r['ID']] = '(' . $r['EXPRESSION'] . ') ' . $r['FIELDNAME'];
		  } else {
			 $fields[$r['ID']] = $r['TABLECODE'] . '.' . $r['FIELDNAME'];
		  }
        }
		ClxCommon::$db->freeResult($rs);
        if ($this->exp_field_lbl != CLX_NUM_THREE) {//excluir encabezados si no son requeridos en import
		  $o->columnHeaders($headers);
        }
        return $fields;
    }

    function getTranslations($exportcode, $direction = CLX_NUM_ZERO) {
        $translations = array();
        $sql = 'SELECT DBEF.ID, DBET.ORIGINAL_VALUE, DBET.REPLACE_VALUE, DBET.CASE_SENSITIVE, DBET.SEQNUM FROM CLX_DB_EXPORT_TRANSLATIONS DBET, CLX_DB_EXPORT_FIELDS DBEF WHERE DBEF.ID = DBET.EXPFIELDID AND (DBET.TRANSL_CODE IS NULL OR DBET.TRANSL_CODE = ' . singleQuote(CLX_CHAR_EMPTY_STRING) . ') AND DBEF.EXPORTCODE = ' . singleQuote($exportcode) . ' UNION ALL SELECT DBEF.ID, Z.ORIGINAL_VALUE, Z.REPLACE_VALUE, Z.CASE_SENSITIVE, Z.SEQNUM FROM CLX_DB_EXPORT_TRANSLATIONS DBET, CLX_DB_EXPORT_FIELDS DBEF, CLX_SYS_TRANSL_SETS Y, CLX_SYS_TRANSL_ITEMS Z WHERE Y.IDCODE = Z.TRANSL_CODE AND DBET.TRANSL_CODE = Y.IDCODE AND DBET.EXPFIELDID = DBEF.ID AND DBEF.EXPORTCODE = ' . singleQuote($exportcode) . ' AND Z.DIRECTION = ' . $direction . ' ';
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
		  $translations[$r['ID']][$r['ORIGINAL_VALUE']] = $r;
        }
		ClxCommon::$db->freeResult($rs);
        return $translations;
    }

    function translate(&$field_value, &$translations) {
        if (array_key_exists($field_value, $translations)) {
		  $info = $translations[$field_value];
		  if ($info['CASE_SENSITIVE'] === true) {
			 if (strcmp($field_value, $info['ORIGINAL_VALUE']) == CLX_NUM_ZERO) {
			     $field_value = $info['REPLACE_VALUE'];
			 }
		  } else {
			 $field_value = $info['REPLACE_VALUE'];
		  }
        }
    }

    function export_from_list($list_file = null) {if ( !empty($_SESSION["_dEBUG"]) && $_SESSION["_dEBUG"]==99 ) {$posx=0;$pos=1613;$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");$te=debug_backtrace(); for ($t=0;$t<12;$t++) {if (isset($te[$t]["file"])){if ($t==$posx)fwrite($tta,chr(13).chr(10).$pos."_".$te[$posx]["function"]."_db_import-hora:".date("h:i:s d/m/Y").chr(13).chr(10));fwrite($tta,substr($te[$t]["file"].str_repeat(" ",80),0,80).chr(9).$te[$t]["line"].chr(9).$te[$t]["function"].chr(13).chr(10));}}fclose($tta);}
		$prev_envir_val = array();
        $prev_envir_val['init_time_out'] = ini_get('max_execution_time');
		$prev_envir_val['post'] = UserRequest::$post;
		$prev_envir_val['request'] = UserRequest::$request;
		$this->dbex_pars['export_from_list'] = array();
		$this->dbex_pars['export_from_list']['response'] = array();
		//$prev_envir_val['get'] = UserRequest::$get;
		//
		$cli_envir = (isset($argv)); // true = invoked from command line
		$lbreak = ($cli_envir === true ? CLX_CHAR_NL : '<BR/>'); // Line Break on CLI or Web environments
		$export_folder = CLX_CHAR_EMPTY_STRING;
		if(is_null($export_folder)) {
			$list_file = 'export_list.txt';
	        $export_folder = kPathToWebSite . kPathToImport;
		}
        echo $lbreak, '********************************', $lbreak;
        echo uppercase(dict('BASE_PROC_LBL_START')), $lbreak;
        echo 'DATE:', xFormat::getNow('Y-m-d H:i:s'), $lbreak;
        echo '********************************', $lbreak;
        $f = new clx_file($export_folder . $list_file, 'r');
        $f->open();
        if (!empty($f->error)) {
		  echo uppercase(dict('GEN_ERR_FILE_NOT_FOUND'), ': ', $list_file), $lbreak;
		  return;
        }
		if($cli_envir === true && !isset($_SESSION['user_record'])) {
			if (!defined('DBEX_DEF_LIST_IMP_USER')) {
				if (!defined('DBIM_DEF_LIST_IMP_USER')) {
					echo 'Missing system parameters DBEX_DEF_LIST_IMP_USER / DBIM_DEF_LIST_IMP_USER', $lbreak;
					return;
				} else {
					$oth_par = DBIM_DEF_LIST_IMP_USER;
				}
			} else {
				$oth_par = DBEX_DEF_LIST_IMP_USER;
			}
	        //Login in for system to set user information
			$ACUS = &getTool('ACUS');
			if (!$ACUS->process_list_import_login($oth_par)) {
				echo $ACUS->get_login_msg(), $lbreak;
				return;
			}
		}
        $i = CLX_INIT_ZERO;
        while (empty($f->eof)) {
		  UserResponse::setRunTimeLimit(CLX_NUM_24 * CLX_NUM_60 * CLX_NUM_60);
		  $ln = $f->read_line();
		  $i++;
		  ob_flush();
		  if ($i == CLX_NUM_ONE || strlen(trim($ln)) == CLX_NUM_ZERO)
			 continue; //Skeep first line
		  $ln = explode(',', $ln);
		  //Validating Options
		  if (count($ln) < CLX_NUM_TWO) {
			 echo 'Invalid line. Instruction must be at least one and up to four element separated by comma on line. Example: EXPORT_CODE, FILE_NAME, FILE_DIR, OTHER_PARAMETERS'.$i, $lbreak;
			 continue;
		  }
		  echo $lbreak;
		  $ln[CLX_NUM_ZERO] = trim($ln[CLX_NUM_ZERO]); // EXPORT CODE
		  $ln[CLX_NUM_ONE] = trim($ln[CLX_NUM_ONE]); // EXPORT FILENAME
		  $ln[CLX_NUM_TWO] = trim($ln[CLX_NUM_TWO]); // output directory
			if(!isset($ln[CLX_NUM_THREE])) {
				$ln[CLX_NUM_THREE] = '';
			}
		  $ln[CLX_NUM_THREE] = trim($ln[CLX_NUM_THREE]); // OTHER PARAMETERS, IN FORMAT PAR_NAME1=PAR_VALUE1&PAR_NAME2=PAR_VALUE2&PAR_NAMEn=PAR_VALUEn
			//Test export existance
		  $dbex = ClxCommon::$db->getFields($this->tablename, 'ID, IDCODE, FORMAT, DESCRIPTION, FILENAME, DOWNLOAD, EXP_FIELD_LBL, DEF_OPTIONS', 'IDCODE = ' . singleQuote($ln[CLX_NUM_ZERO]));
			if (empty($dbex)) {
			 echo uppercase('Export ', $ln[CLX_NUM_ZERO], ' do not exists on line '.$i), $lbreak;
			 continue;
		  }
			//Prepare information
		  //Set POST and FILES for Import to work
	        $opts = unserialize($dbex['DEF_OPTIONS']);
			UserRequest::$post = array('confirm' => CLX_NUM_ONE);
		  UserRequest::$request = array(CLX_PROP_TO => 'DBEX', 'ac' => 'EXPORT', 'idlist' => $dbex['ID'],
			 'DBEX_id' => $dbex['ID'], 
				'DBEX_description' => trim($dbex['DESCRIPTION']),
			 'DBEX_filename' => ($ln[CLX_NUM_ONE] !== CLX_CHAR_EMPTY_STRING ? $ln[CLX_NUM_ONE] : trim($dbex['FILENAME'])), // Id specified, must be filespec, meaning: dir/filename
			 'DBEX_format' => $dbex['FORMAT'], 
			 'DBEX_download' => CLX_NUM_ZERO, 
				'DBEX_exp_field_lbl' => $dbex['EXP_FIELD_LBL'],
				'DBEX_fielddelim' => $opts['DBEX_fielddelim'], 
				'DBEX_fielddelim_other' => $opts['DBEX_fielddelim_other'],
				'DBEX_recorddelim' => $opts['DBEX_recorddelim'],
		        'DBEX_recorddelim_other' => $opts['DBEX_recorddelim_other'],
				'DBEX_fieldenclosure' => $opts['DBEX_fieldenclosure'],
		        'DBEX_fieldenclosure_other' => $opts['DBEX_fieldenclosure_other'],
				'DBEX_filedir' => ($ln[CLX_NUM_TWO] !== CLX_CHAR_EMPTY_STRING ? $ln[CLX_NUM_TWO] . DIRECTORY_SEPARATOR : kPathToWebSite . kPathToExport) // To leave directory especification in the filename
			);
			// Populate other parameters
		  $oth_par = explode('&', $ln[CLX_NUM_THREE]);
			foreach($oth_par as $par_pair) {
			  $name_value = explode('=', $par_pair);
				UserRequest::$post[$name_value[CLX_NUM_ZERO]] = (isset($name_value[CLX_NUM_ONE]) ? $name_value[CLX_NUM_ONE] : CLX_CHAR_EMPTY_STRING);
			}
			$oth_par = null;
			$par_pair = null;
			$name_value = null;
			// Execute export process
		  $result = $this->export();
		  ClxCommon::$db->commit();
//		  $result = str_replace(dict('GEN_LBL_DISPLAY') . ' ' . dict('GEN_LBL_ERRORS'), '', $result);
		  echo 'EXPORT ', $ln[CLX_NUM_ZERO], ' COMPLETED FOR LINE: ', $i, $lbreak, '- FILE: ', UserRequest::$request['DBEX_filedir'] . UserRequest::$request['DBEX_filename_final'], ' TIME: ', xFormat::getNow('Y-m-d H:i:s'), $lbreak;
		  echo '-', uppercase(str_replace($lbreak, ' ', strip_tags($result))), $lbreak;
			// In order to called be able to retrieve files generated...
			$this->dbex_pars['export_from_list']['response'][] = UserRequest::$request['DBEX_filedir'] . UserRequest::$request['DBEX_filename_final'];
		  $dbex = null;
		  $result = null;
        }
        $f->close();
        // Adds date/time at the end
        echo $lbreak;
        echo '********************************', $lbreak;
        echo uppercase(dict('GEN_LBL_PROCESS_COMPLETE')), $lbreak;
        echo 'TIME: ', xFormat::getNow('Y-m-d H:i:s'), $lbreak;
        echo '********************************', $lbreak;
        //Restore original value
        UserResponse::setRunTimeLimit($prev_envir_val['init_time_out']);
		UserRequest::$post = $prev_envir_val['post'];
		UserRequest::$request = $prev_envir_val['request'];
		$prev_envir_val = null;
        ob_flush();
    }
	
	function get_dbex_pars($entry = null) {
		if(is_null($entry)) {
			return $this->dbex_pars;
		}
		return $this->dbex_pars[$entry];
	}
}
