<?php

/**
 * Tool Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: cramirez $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2018-10-04 10:54:28 -0400 (Thu, 04 Oct 2018) $
 * @version        $Rev: 10207 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  Included label
 */
class dictionary extends clx_tool {

    var $exportform;
    var $importform;
    var $debug;

    function __construct() {
        $this->code = 'DICT';
        $this->name = 'Dictionary Manager';
        $this->system = 'System';
        $this->version = '0.6.10.3';
        $this->debug = false;
        $this->has_access = true;

        $this->tablename = 'CLX_DICTIONARY';
        $this->record_name_singular = 'dictionary entry';
        $this->record_name_plural = 'dictionary entries';
        $this->primary_field = 'ENTRY';
        $this->secondary_field = 'LANGUAGE';

        $this->exportform = array(
		  CLX_CLASS_TITLE => 'TOOL_DICT_TITLE_EXPORT',
		  'panels' => array(
			 array(
			     'type' => 'normal',
			     'initial_state' => '1',
			     'entries' => array(
			         array(
					   'label' => 'GEN_LBL_FILE',
					   'fields' => array(
						  array('name' => 'filename', 'type' => 'string', 'default' => '', 'size' => '255', 'displaysize' => '48',
						      'validations' => array(
								array('type' => 'string', 'required' => '1')
						      )
						  )
					   )
			         )
			     )
			 )
		  )
        );
        $this->importform = array(
		  CLX_CLASS_TITLE => 'TOOL_DICT_TITLE_IMPORT',
		  'panels' => array(
			 array(
			     'type' => 'normal',
			     'initial_state' => '1',
			     'entries' => array(
			         array(
					   'label' => 'TOOL_PACK_PROMPT_FILEONSERVER',
					   'fields' => array(
						  array('name' => 'localfile', 'type' => 'string', 'default' => '', 'size' => '255', 'displaysize' => '48',
						      'validations' => array(
								array('type' => 'string', 'required' => '0')
						      )
						  )
					   )
			         ),
			         array(
					   'label' => 'GEN_LBL_FILE',
					   'fields' => array(
						  array('name' => 'filename', 'type' => 'file', 'default' => '',
						      'validations' => array(
								array('type' => 'string', 'required' => '0')
						      )
						  )
					   )
			         ),
			         array(
					   'label' => 'GEN_LBL_OPTIONS',
					   'fields' => array(
						  array('name' => 'options', 'type' => CLX_CLASS_LIST, 'default' => '',
						      'options' => array(
								'NEW' => 'TOOL_DICT_IMPORT_NEW',
								'UPD' => 'TOOL_DICT_UPDATE',
								'REP' => 'TOOL_DICT_REPLACE'
						      )
						  )
					   )
			         )
			     )
			 )
		  )
        );
        $this->quickfilterform = array(
		  CLX_CLASS_TITLE => 'TOOL_PRLO_TITLE_CREATE',
		  'panels' => array(
			 array(
			     'type' => 'normal',
			     'entries' => array(
			         array(
					   'label' => 'GEN_LBL_ENTRY',
					   'fields' => array(
						  array(
						      'name' => 'entry',
						      'type' => 'string',
						      'size' => '',
						      'displaysize' => '35',
						      'default' => ''
						  )
					   )
			         ),
			         array(
					   'label' => 'GEN_LBL_VALUE',
					   'fields' => array(
						  array(
						      'name' => 'value',
						      'type' => 'string',
						      'size' => '',
						      'displaysize' => '35',
						      'default' => ''
						  )
					   )
			         ),
			         array(
					   'label' => 'GEN_LBL_LANGUAGE',
					   'fields' => array(
						  array(
						      'name' => 'language',
						      'type' => 'string',
						      'size' => '10',
						      'displaysize' => '10',
						      'default' => ''
						  )
					   )
			         ),
			         array(
					   'label' => 'GEN_LBL_MDATE',
					   'fields' => array(
						  array(
						      'name' => 'mdate',
						      'type' => 'date',
						      'displaysize' => '35'
						  )
					   )
			         )
			     )
			 )
		  )
        );
    }

    function install() {
        $UIMI = getTool('UIMI');
        $UIMI->add($this->code, 'MENU_TOOLS:MENU_SYSTEM:MENU_DICTIONARY', 'TOOL', $this->code, 'ac=LIST');
        $DBLT = getTool('DBLT');
        $items = array('en' => 'GEN_LBL_ENGLISH', 'es' => 'GEN_LBL_SPANISH');
        $DBLT->createList('LANG', 'Languages', 'These are the languages supported by the dictionary', CLX_NUM_ONE, $items);
    }

    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['ENTRY'] = array('type' => 'varchar(64)', 'null' => 'YES', 'properties' => '');
        $fields['VALUE'] = array('type' => 'varchar(255)', 'null' => 'YES', 'properties' => '');
        $fields['LANGUAGE'] = array('type' => 'varchar(10)', 'null' => 'NO', 'properties' => '');
        $fields['MDATE'] = array('type' => 'datetime', 'null' => 'YES', 'properties' => '');
        $fields['ENTRY_VERSION'] = array('type' => 'int(3)', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['SOURCE_LANGUAGE'] = array('type' => 'varchar(10)', 'null' => 'YES', 'properties' => '');
        $fields['TOOL_ENTRY'] = array('type' => 'int(1)', 'null' => 'YES', 'properties' => '');
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'UNI', 'fieldname' => 'ENTRY,LANGUAGE');
		$indexes[$ipf . '03'] = array('index' => 'MUL', 'fieldname' => 'ENTRY');
		$indexes[$ipf . '04'] = array('index' => 'MUL', 'fieldname' => 'LANGUAGE');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'MyISAM', 'fields' => $fields, 'indexes' => $indexes);
    }

    function unpack($parameters) {
		$this->call_free_reg_dupli();
        $parameters = array_values(array_merge(array('package' => null), $parameters));
        list($package) = $parameters;
        $record = $this->unpackRecord($package);
        if (empty($record['ENTRY_VERSION'])) {
		  $record['ENTRY_VERSION'] = CLX_INIT_ZERO;
        }
        if (empty($record['TOOL_ENTRY'])) {
		  $record['TOOL_ENTRY'] = CLX_INIT_ZERO;
        }
        if(is_null($record['VALUE']) || is_null($record['ENTRY']) || trim($record['ENTRY']) === CLX_CHAR_EMPTY_STRING) {
		  return;
        }
		$condition=false;
		$record['LANGUAGE']=clx_language_lower_upper($record['LANGUAGE']);
		if (ClxCommon::$db->oneExistsRecords($this->tablename, 'UPPER(ENTRY)=' . singleQuote(uppercase($record['ENTRY'])) . ' AND UPPER(LANGUAGE)=' . singleQuote(uppercase($record['LANGUAGE']))) == CLX_NUM_ZERO) {
		  ClxCommon::$db->createRecord($this->tablename, $record, false);
        } else {
		  ClxCommon::$db->updateRecord($this->tablename, $record, 'ENTRY=' . singleQuote($record['ENTRY']) . ' AND UPPER(LANGUAGE)=' . singleQuote(uppercase($record['LANGUAGE'])));
        }
		ClxCommon::$db->executequery('commit');
    }

    /**
     * Action is called for every user interaction with the module.
     * @param type $ac
     * @param type $idlist
     * @return type
     */
    function action($ac, $idlist) {
        $contents = '';
        if ($this->getAccess()->canRead()) {
		  $msg = array();
		  $record = array();
		  switch ($ac) {
			 case 'EXPORT':
			     if (isset(UserRequest::$post['confirm'])) {
			         if ($this->checkAnyForm($this->exportform, $msg)) {
					   $this->exportDictRecords();
					   $info = popHistory();
					   $contents .= executeHistory($info);
			         } else {
					   $contents = $this->useAnyForm($this->exportform, $ac, array(), $record, $msg);
			         }
			     } else {
			         $record['DICT_filename'] = 'packages/DICT_' . clx_language_lower_upper($_SESSION['user_record']['username']) . date('-Ymd-Hi') . '.xml';
			         $contents = $this->useAnyForm($this->exportform, $ac, array(), $record, $msg);
			     }
			     break;
			 case 'IMPORT':
			     if (isset(UserRequest::$post['confirm'])) {
			         if ($this->checkImportForm($msg)) {
					   $this->importDictRecords();
					   $info = popHistory();
					   $contents .= executeHistory($info);
			         } else {
					   $contents = $this->useAnyForm($this->importform, $ac, array(), $record, $msg);
			         }
			     } else {
			         $contents = $this->useAnyForm($this->importform, $ac, array(), $record, $msg);
			     }
			     break;
			 case 'CREATE_FAST':
			     if ($this->getAccess()->canDisplay()) {
			         if (isset(UserRequest::$post['confirm'])) {
					   if ($this->checkForm($msg)) {
						  $this->createRecord_();
						  $contents = setJavaScriptTag('self.close();', false);
					   } else {
						  $contents = $this->editForm($ac, array(), UserRequest::$post, $msg);
					   }
			         } else {
					   $msg = array();
					   $record = $this->loadToolRecord($this->findEntryId(UserRequest::$request['entry']));
					   if (empty($record['DICT_entry'])) {
						  $record = $this->defaultRecord();
						  $record['DICT_entry'] = UserRequest::$request['entry'];
					   }
					   $contents = $this->editForm($ac, array(), $record, $msg);

			         }
			     } else {
			         $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [DICTIONARY-CAN_APPEND][' . $ac . '] ' . $this->getCode(false));
			     }
		    $contents.=	'<script>'.
					'$( "form" ).submit(function( event ) {'.
					'  closeWindow();'.
					'})'.
					'</script>';
			     break;
			 case 'MODIFY_FAST': //Review there if no updateRecord
			     if ($this->getAccess()->canModify()) {
			         if (isset(UserRequest::$post['confirm'])) {
					   if ($this->checkForm($msg)) {
						$this->createRecord_();//  $this->updateRecord();
					    //  echo  $contents=jQueryIncludes(). '<script>$("#windowpop0").dialog("close");</script>'; //
							 
							 //$contents=jQueryIncludes().setJavaScriptTag( '$("#windowpop0").hide()');
						  $contents = setJavaScriptTag('self.close();', false);
							  
					   } else {
						  $contents = $this->editForm($ac, array(), UserRequest::$post, $msg);
					   }
			         } else {
					   $msg = array();
					   $record = $this->loadToolRecord($this->findEntryId(UserRequest::$request['entry']));
					   $contents = $this->editForm($ac, array(), $record, $msg);
			         }
			     } else {
			         $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [DICTIONARY-CAN_MODIFY][' . $ac . '] ' . $this->getCode(false));
			     }
			     break;
			 default:
			     $contents = parent::action($ac, $idlist);
		  }
        } else {
		  $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [DICTIONARY-CAN_READ][' . $ac . '] ' . $this->getCode(false));
        }
        return $contents;
    }
	function call_free_reg_dupli() { 
		 if (kDatabaseType === 'oci') {
			ClxCommon::$db->executequery('
			DELETE FROM  CLX_DICTIONARY WHERE ROWID IN (
				SELECT ROWID FROM  CLX_DICTIONARY WHERE ENTRY||'.singlequote('_').' ||LANGUAGE IN(
					SELECT ENTRY||'.singlequote('_').' ||MIN(LANGUAGE) FROM CLX_DICTIONARY WHERE ENTRY||'.singlequote('_').' || UPPER( LANGUAGE) IN (
						SELECT ENTRY||'.singlequote('_').' || UPPER( LANGUAGE) FROM CLX_DICTIONARY GROUP BY ENTRY,UPPER(LANGUAGE) HAVING COUNT(*)>1
					) GROUP BY ENTRY, UPPER( LANGUAGE)
				)
			)');
		} else {
    //        $r = ClxCommon::$db->getRecord($this->tablename, 'UPPER(ENTRY)=' . singleQuote(uppercase($entry)) . '  AND UPPER(LANGUAGE)=' . singleQuote(uppercase($language)));
        }

	}		
    function makelist($listingcode = null) {
        $t = time();
        $to = $this->code;
        $here = 'tool.php?to=' . $to . '&t=' . $t;
        clearHistory();
        pushHistory($this->code, 'LIST', array());
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->setProp(CLX_PROP_ID, $this->code . 'List');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'NOT_DEFINED');
        $page = new xPaging();
       
		$_SESSION['paging'][$form->getProp(CLX_PROP_ID)]['criteria']=str_replace('&gt;','>',$_SESSION['paging'][$formname]['criteria']);
		$_SESSION['paging'][$form->getProp(CLX_PROP_ID)]['criteria']=str_replace('&lt;','<',$_SESSION['paging'][$formname]['criteria']);
        $page->pageInit($form->getProp(CLX_PROP_ID), $here, '*', $this->tablename, '', '', '');
        $table = new htmlTable(CLX_WIDTH_FIVE);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_DICT_TITLE_LIST'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_FIVE));
        $page->header($table);
        $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_ENTRY'), dict('GEN_LBL_LANGUAGE'), dict('GEN_LBL_VALUE'), dict('GEN_LBL_MDATE')));
        $i = CLX_INIT_ZERO;
        while ($record = $page->nextRecord()) {
		  $recordid = $record['ID'];
		  $select = new htmlCheckbox('selector_' . $recordid, false);
		  $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ')');
		  $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
		  $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array(uppercase($record['ENTRY']), clx_language_lower_upper($record['LANGUAGE']), $record['VALUE'], xFormat::datetime($record['MDATE'])));
		  $i++;
        }
        $buttons = htmlButton::getButton('submit', dict('GEN_LBL_DISPLAY'), array('action' => 'DISPLAY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        if ($this->getAccess()->canModify()) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        if ($this->getAccess()->canDelete()) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        if ($this->getAccess()->canDisplay()) {
		  $b = new htmlFormSubmit(dict('GEN_LBL_CREATE_SOMETHING', dict('GEN_LBL_DICTENTRY')), 'CREATE', $form->getProp(CLX_PROP_ID));
		  $b->setProp(CLX_PROP_ID, 'ON');
		  $buttons .= $b->source();
        }
        $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_EXPORT'), array('action' => 'EXPORT', 'form' => $form->getProp(CLX_PROP_ID), $b->setProp(CLX_PROP_ID, 'ON')))->source();
        if ($this->getAccess()->canDisplay()) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_IMPORT'), array('action' => 'IMPORT', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
        }
        $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_FILTER'), array('action' => 'FILTER', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
        $form->addContents($page->footers($table, $buttons));
        return $form->source();
    }

    function searchForm() {
	$msg = array();
        $contents = '';
        // Search Form
        $form = new htmlForm(CLX_PROP_ACTION, UserRequest::$server['PHP_SELF']);
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'SEARCH');
        $form->addArgument('confirm', '1');
        $form->addArgument(CLX_PROP_TARGET, UserRequest::$request[CLX_PROP_TARGET]);
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_DICT_TITLE_SEARCH'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addField($msg, dict('GEN_LBL_ENTRY') . ':', 'DCT_entry', UserRequest::$post['DCT_entry'], CLX_NUM_40, CLX_NUM_40);
        $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_SEARCH'), array('action' => 'SEARCH', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        $c = new htmlButtonCloseWindow(dict('GEN_LBL_CANCEL'));
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->setContents($table->source());
        $contents .= $form->source();
        if (isset(UserRequest::$post['confirm'])) {
		  $table = new htmlTable(CLX_WIDTH_TWO);
		  $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
		  $criteria = '';
		  if (!empty(UserRequest::$post['DCT_entry'])) {
			 $criteria .= 'UPPER(ENTRY) LIKE ' . singleQuote(uppercase(UserRequest::$post['DCT_entry']) . '%');
		  }
		  if ($criteria === '') {
			 $table->addCell(dict('GEN_LBL_REQUIRED'), array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
		  } else {
             if (ClxCommon::$db->oneExistsRecords($this->tablename, $criteria) === CLX_NUM_ZERO) {
			     $table->addCell(dict('TOOL_GEN_ERR_NOMATCH'), array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
			 } else {
			     $table->addCell(dict('TOOL_GEN_MSG_FOUNDRECORDS'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
			     $records = ClxCommon::$db->getRecords($this->tablename, $criteria);
			     $i = CLX_INIT_ZERO;
			     while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
			         $js = 'javascript:setLookupValue(' . quote(UserRequest::$request[CLX_PROP_TARGET]) . ', ' . quote($record['ENTRY']) . ');';
			         $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array('<page t=dictionary></page><a href="' . $js . '">' . $record['ENTRY'] . '</a>', $record['VALUE']));
			         $i++;
			     }
				 ClxCommon::$db->freeResult($records);
			 }
		  }
		  $contents .= $table->source();
        }
        return $contents;
    }

    function editForm($ac, $idlist, $record, $msg) {
        $modify = ((UserRequest::$request['ac'] === 'MODIFY') or ( UserRequest::$request['ac'] === 'MODIFY_FAST'));
        $title = ($modify) ? dict('TOOL_DICT_TITLE_MODIFY') : dict('TOOL_DICT_TITLE_CREATE');
        $js = setJavaScriptTag(getJavaScriptFilename('DICT_form.js'), true);
        $file = (UserRequest::$request['ac'] === 'CREATE_FAST') ? 'pop.php' : 'tool.php';
        $form = new htmlForm(CLX_PROP_ACTION, $file);
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', $ac);
        $form->addArgument('confirm', '1');
        $form->addArgument('idlist', toString($idlist));
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell($title, array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        if (!empty($modify)) {
		  $table->addData(dict('GEN_LBL_ENTRY') . ':', $record['DICT_entry']);
		  $form->addArgument('DICT_entry', $record['DICT_entry']);
        } else {
		  $table->addField($msg, dict('GEN_LBL_ENTRY') . ':', 'DICT_entry', $record['DICT_entry'], CLX_NUM_40, CLX_NUM_40);
        }
        $istool = new htmlCheckbox('DICT_tool_entry', (!empty($record['DICT_tool_entry'])));
        $table->addData(dict('GEN_LBL_ISTOOLENTRY') . ':', $istool->source());
        $lbl_null = dict('GEN_LBL_null') . '?';
        $DBLT = &getTool('DBLT');
        $languages = $DBLT->getListAsArray('LANG');
        foreach ($languages as $langcode => $langname) {
			$langcode=clx_language_lower_upper($langcode);
		  $table->addCell($langname . ':', array(CLX_PROP_CLASS => CLX_CLASS_PROMPT));
		  $value = (!array_key_exists('DICT_' . $langcode, $record)) ? '' : $record['DICT_' . $langcode];
		  $field = new htmlField('DICT_' . $langcode, $value, CLX_NUM_48, CLX_NUM_255);
		  $field->setProp('onChange', 'javascript:DICT_form(' . singleQuote($langcode) . ')');
		  $cell = $field->source();
	    $checkbox = new htmlCheckbox("DICT_" . $langcode . '_null', ((array_key_exists('DICT_' . $langcode, $record) && strlen($record["DICT_" . $langcode]) > 0) ? false : true), 1);
		  //$checkbox = new htmlCheckbox('DICT_' . $langcode . '_null', ((array_key_exists('DICT_' . $langcode, $record)) ? false : true), CLX_NUM_ONE);
		  $cell .= ' ' . $lbl_null . $checkbox->source();
		  $table->addCell($cell, array(CLX_PROP_CLASS => CLX_CLASS_DATA));
        }
        $form->addContents($table->source());
        $buttons = '';
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        if (!empty($modify)) {
		  $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_SAVE'), array('action' => $ac, 'form' => $form->getProp(CLX_PROP_ID)))->source();
        } else {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE'), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        }
        if (($ac !== 'CREATE_FAST') and ( $ac !== 'MODIFY_FAST')) {
		  $c = new htmlButtonCancel();
		  $buttons .= $c->source();
        }
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->addContents($table->source());
        return $js . $form->source();
    }

    function checkForm(&$msg) {
        if (empty($msg)) {
		  $msg = array();
        }
        $modify = ((UserRequest::$post['ac'] === 'MODIFY') or ( UserRequest::$post['ac'] === 'MODIFY_FAST'));
        if (empty($modify)) {
		  xValidate::required($msg, 'DICT_entry', UserRequest::$post['DICT_entry']);
        }
        return count($msg) == CLX_NUM_ZERO;
    }

    function useAnyForm($aform, $ac, $idlist, $record, $msg) {
        $form = new clx_form($this->code, $aform);
        return $form->editForm($ac, $idlist, $record, $msg);
    }

    function checkAnyForm($aform, &$msg) {
        $form = new clx_form($this->code, $aform);
        $result = $form->checkForm1(UserRequest::$post['ac'], UserRequest::$post, $msg);
        return $result;
    }

    function checkImportForm(&$msg) {
        $msg = array();
        if (empty(UserRequest::$post['DICT_localfile'])) {
		  if (UserRequest::$files['DICT_filename']['error']) {
			 $msg['DICT_filename'] = dict('ERR_PACK_FILETOOBIG');
		  }
        } elseif (!file_exists(UserRequest::$post['DICT_localfile'])) {
		  if (strpos(UserRequest::$post['DICT_localfile'], 'http') === false) {
			 $msg['DICT_localfile'] = 'There is no file with this name.';
		  } else {
			 $fd = fopen(UserRequest::$post['DICT_localfile'], 'r');
			 if (empty($fd)) {
			     $msg['DICT_localfile'] = 'There is no file with this name.';
			 }
		  }
        }
        return count($msg) == CLX_NUM_ZERO;
    }

    function confirmDeleteFill(&$table, $record) {
        $table->addData(dict('GEN_LBL_ENTRY') . ':', $record['DICT_entry']);
        $table->addData(dict('GEN_LBL_VALUE') . ':', dict($record['DICT_entry']));
    }

    function viewFormFill($record) {
        $DBLT = &getTool('DBLT');
        $languages = $DBLT->getListAsArray('LANG');
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_DICT_TITLE_VIEW'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addData('Entry:', $record['DICT_entry']);
        foreach ($languages as $langcode => $langname) {
			$langcode=clx_language_lower_upper($langcode);
		  $value = (!array_key_exists('DICT_' . $langcode, $record)) ? '' : $record['DICT_' . $langcode];
		  $table->addData($langname . ':', $value);
        }
        return $table->source();
    }

    function setfilter() {
        $filter = '';
        $boolean = '';
        if (!empty(UserRequest::$post['DICT_entry'])) {
			 $booleanX=' AND ';
			if (strpos('||',UserRequest::$post['DICT_entry'])>0 ){
				$textos=explode('||',UserRequest::$post['DICT_entry']);
				$booleanX=' OR  ';
			}
			else {
				
				UserRequest::$post['DICT_entry']=str_replace("&amp;","&",UserRequest::$post['DICT_entry']);
				
				$textos=explode('&&',UserRequest::$post['DICT_entry']);
				
			}
			foreach ($textos as $key=>$value) {
				$extra='';
				if (substr(trim($value),0,1)=='!') {
					$extra=' NOT ';
					$value=substr(trim($value),1);
				}	
				$filter .= $boolean . ' UPPER(ENTRY) '.$extra.' LIKE ' . singleQuote('%'.uppercase($value) . '%');
		
				$boolean = $booleanX;
			}
			$filter='('.$filter.')';
        }
		if (!empty(UserRequest::$post['DICT_value']) && UPPERCASE(UserRequest::$post['DICT_value'])=="!(NULL)") {
			$filter .= $boolean . ' VALUE IS NOT NULL ';
			$boolean = 'AND';
		}
		else if (!empty(UserRequest::$post['DICT_value']) && UPPERCASE(UserRequest::$post['DICT_value'])=="(NULL)") {
			$filter .= $boolean . ' VALUE IS NULL ';
		  $boolean = 'AND';
        }
		else if (!empty(UserRequest::$post['DICT_value'])) {
//			UserRequest::$post['DICT_value']=uppercase(clx_change_especial_char_html (UserRequest::$post['DICT_value']));
		  $filter .= $boolean . ' UPPER(VALUE) LIKE ' . singleQuote(uppercase(clx_change_especial_char_html (UserRequest::$post['DICT_value'])) . '%');
		  $boolean = 'AND';
        }
        if (!empty(UserRequest::$post['DICT_language'])) {
		  $filter .= $boolean . ' UPPER(LANGUAGE) LIKE ' . singleQuote(uppercase(UserRequest::$post['DICT_language']) . '%');
		  $boolean = 'AND';
        }
        if (!empty($_POST["DICT_mdate"])) {
			$spliter=explode(",",$_POST["DICT_mdate"]);
			if (strlen($_POST["DICT_mdate"])>10 and count($spliter)>1) {
				$filter .= $boolean . " MDATE BETWEEN '" . xFormat::date($spliter[0], "Y-m-d H:i:s") . "' AND '" . xFormat::date($spliter[1], "Y-m-d H:i:s") . "'" ;
				$boolean = "AND";
			}
			else {
				$filter .= $boolean . " MDATE >= '" . xFormat::date($_POST["DICT_mdate"], "Y-m-d H:i:s") . "'";
				$boolean = "AND";
			}
		}

        $sortby = (array_key_exists('DICT_sort', UserRequest::$post)) ? UserRequest::$post['DICT_sort'] : CLX_CHAR_EMPTY_STRING;
        $this->setFilterKeys($filter, $sortby, CLX_NUM_ZERO);
    }

    function findEntryId($entry) {
        $id = CLX_INIT_ZERO;
        $id = ClxCommon::$db->getField($this->tablename, 'ID', 'UPPER(ENTRY)=' . singleQuote(uppercase($entry)));
        return $id;
    }

    function loadToolRecord($recordinfo) {
        $DBLT = &getTool('DBLT');
        $languages = $DBLT->getListAsArray('LANG');
        if (is_numeric($recordinfo)) {
		  $entry = ClxCommon::$db->getField($this->tablename, 'ENTRY', 'ID=' . $recordinfo);
        } else {
		  $entry = $recordinfo;
        }
        $record = array();
        $record['DICT_entry'] = $entry;
        foreach ($languages as $language => $name) {
			$language=clx_language_lower_upper($language);
		  $value = ClxCommon::$db->getRecord($this->tablename, 'UPPER(ENTRY)=' . singleQuote(uppercase($entry)) . ' AND UPPER(LANGUAGE)=' . singleQuote(uppercase($language)));
		  if (empty($value)) {
			 continue;
		  }
		  $record['DICT_' . $language] = $value['VALUE'];
		  $record['DICT_tool_entry'] = $value['TOOL_ENTRY'];
        }
        return $record;
    }

    function saveRecord() {
        $DBLT = &getTool('DBLT');
        $languages = $DBLT->getListAsArray('LANG');
        //Addition to cache dictionary
        $DICA = getTool('DICA');
        $entry = UserRequest::$post['DICT_entry'];
        $this->clearDictHeap($entry);
		$clean_entry = uppercase(trim($entry));
		foreach ($languages as $key=>$value) {
			apc_deleteFct(kSpaceName . '.dict.' . $clean_entry . $key);
		}
        $record = array();
        $record['ENTRY'] = $entry;
        $record['MDATE'] = xFormat::getNow('Y-m-d- H:i:s');
        $record['TOOL_ENTRY'] = (!empty(UserRequest::$post['DICT_tool_entry'])) ? CLX_NUM_ONE : CLX_NUM_ZERO;
        $record['ENTRY_VERSION'] = '0';
		$sql = 'DELETE FROM ' . $this->tablename . ' WHERE UPPER(ENTRY) = ' . singleQuote(uppercase($entry)) ;
        ClxCommon::$db->executeQuery($sql);
        $sql = 'DELETE FROM CLX_DICTIONARY_HEAP WHERE UPPER(ENTRY) = ' . singleQuote(uppercase($entry)) ;
        ClxCommon::$db->executeQuery($sql);

        foreach ($languages as $language => $name) {
			$language=clx_language_lower_upper($language);
		  $record['LANGUAGE'] = clx_language_lower_upper($language);
		  $record['VALUE'] = clx_html_spchrs_decode(UserRequest::$post['DICT_' . $language]);
			if ( kApcTtl_9>0 && !empty($record['VALUE']) ) {
				apc_addFct(kSpaceName . '.dict.' . $clean_entry . $record['LANGUAGE'], $record['VALUE'], kApcTtl_9);
			}
		  if (!empty(UserRequest::$post['DICT_' . $language . '_null']) && strlen($record['VALUE']) == CLX_NUM_ZERO) {
			 $sql = 'DELETE FROM ' . $this->tablename . ' WHERE UPPER(ENTRY) = ' . singleQuote(uppercase($entry)) . ' AND UPPER(LANGUAGE) = ' . singleQuote(uppercase($language));
			 ClxCommon::$db->executeQuery($sql);
			 $sql = 'DELETE FROM CLX_DICTIONARY_HEAP WHERE UPPER(ENTRY) = ' . singleQuote(uppercase($entry)) . ' AND UPPER(LANGUAGE) = ' . singleQuote(uppercase($language));
			 ClxCommon::$db->executeQuery($sql);
			 continue;
		  }

		  ClxCommon::$db->loadDbRecord($this->tablename, $record, false, 'UPPER(ENTRY) = ' . singleQuote(uppercase($entry)) . ' AND UPPER(LANGUAGE) = ' . singleQuote(uppercase($language)));

		  //Addition to cache dictionary, cleaning to allow update later
		  $DICA->removeCache($record['ENTRY']);
			$clean_entry = uppercase(trim($entry));
			foreach ($languages as $key=>$value) {
				apc_deleteFct(kSpaceName . '.dict.' . $clean_entry . $key);
		
			}
        }
    }

    function createRecord_() {
        $this->saveRecord();
    }

    function deleteRecordSk($id) {
        $record = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $id);
        $this->clearDictHeap($record['ENTRY']);
        ClxCommon::$db->deleteRecords($this->tablename, 'ID=' . $id);
    }

    /**
     * Genrates default record
     * @return string
     */
    function defaultRecord() {
        $DBLT = &getTool('DBLT');
        $languages = $DBLT->getListAsArray('LANG');
        $record = array();
        $record['DICT_entry'] = '';
        $record['DICT_tool_entry'] = CLX_INIT_ZERO;
        $entry = '';
        foreach ($languages as $language => $name) {
			$language=clx_language_lower_upper($language);
		  $value = ClxCommon::$db->getRecord($this->tablename, 'UPPER(ENTRY)=' . singleQuote(uppercase($entry)) . ' AND UPPER(LANGUAGE)=' . singleQuote(uppercase($language)));
		  unset($value);
		  $record['DICT_' . $language] = '';
        }
        return $record;
    }

    /**
     * See clxtools
     * @param type $name
     * @param type $default
     * @param type $data
     * @param type $display_size
     * @param type $maxlength
     * @return type
     */
    function chooseRecord($name, $default, $data = array(), $display_size = CLX_NUM_32, $maxlength = CLX_NUM_64) {
        $code = $this->code;
        $alt = 'TOOL_' . $code . '_ALT_SEARCH';
		$searchIcon = getSkinIcon('view', 'javascript:openWindow("", "pop.php?to=$code&ac=SEARCH&target=' . $name . '", 500, 300,null,null,'.singlequote( __FILE__ .'_'. __LINE__ ) .');', dict($alt));
        $c = new htmlField($name, $default, null, CLX_NUM_48, CLX_NUM_128);
        return $c->source() . $searchIcon;
    }

    function recordExists($field_label, &$field_value, &$msg) {
        return true;
    }

    function dictFastEdit($entry) {
        if (!empty($this->debug)) {
		  echoLog('Entering DICT->viewEntry<br>'); //Review modify fast -> create fast
        }
        $url = 'javascript:openWindow(' . singleQuote(CLX_CHAR_EMPTY_STRING) . ', ' . singleQuote('pop.php?to=DICT&ac=CREATE_FAST&entry=' . $entry) . ', 500, 300,null,null,'.singlequote( __FILE__ .'_'. __LINE__ ) .')';
        $searchIcon = getSkinIcon('viewdict', $url, 'Display Dictionary Entry');
        $value = dict($entry);
        return $value . $searchIcon;
    }

    function viewEntry($prompt, $entry, &$table) {
        $table->addData($prompt, $this->dictFastEdit($entry), CLX_PROP_CLASS, CLX_CLASS_DATA);
    }

    /**
     * Defines form field for edition
     *
     * @param array $msg
     * @param string $prompt
     * @param string $name
     * @param array $record
     * @param obj $table
     * @param obj $form
     */
    function editEntry($msg, $prompt, $name, $record, &$table, &$form) {
        if (!empty($this->debug)) {
		  echoLog('Entering DICT->editEntry<br>');
        }
        $form->addArgument($name, $record[$name]);
        $table->addText($msg, $prompt, $name . '_DICT', (array_key_exists($name . '_DICT', $record)) ? $record[$name . '_DICT'] : '', CLX_NUM_FIVE, CLX_NUM_60);
    }

    /**
     * Validate Entry
     */
    function validateEntry($entry, $record, $required, &$msg) {
        if (!empty($this->debug)) {
		  echoLog('Entering DICT->validateEntry, Validating Entry Input if something or if Required<br>');
        }
        $something = (strlen($record[$entry . '_DICT']) > CLX_NUM_ZERO);
        if (empty($something) && !empty($required)) {
		  $msg[$entry . '_DICT'] = dict('GEN_ERR_REQUIRED_FIELD');
		  $msg[$entry] = dict('GEN_ERR_REQUIRED_FIELD');
        }
    }

    /**
     * Creates a new entry into the dictionary
     */
    function saveEntry($form_fieldname, $record) {

        $value = clx_html_spchrs_decode($record[$form_fieldname . '_DICT']);
        $language = clx_language_lower_upper($_SESSION ['prefs'] ['LANGUAGE']);
        if (strlen($record[$form_fieldname]) > CLX_NUM_ZERO) {
		  $entry = clx_language_lower_upper($record[$form_fieldname]);
        } else {
		  $entry = clx_language_lower_upper($this->getNextEntryId($value));
        }
        $update_entry = (strpos($entry, '-') !== CLX_NUM_ZERO) ? true : false;
        $r = array();
        $r['ENTRY'] = $entry;
        $r['VALUE'] = $value;
        $r['MDATE'] = xFormat::getNow('Y-m-d- H:i:s');
        $r['LANGUAGE'] = $language;
        $r['TOOL_ENTRY'] = CLX_NUM_ZERO;
        $r['ENTRY_VERSION'] = CLX_NUM_ZERO;
        $r['SOURCE_LANGUAGE'] = getCurrentLanguage();
        $qty = ClxCommon::$db->oneExistsRecords($this->tablename, 'UPPER(ENTRY)=' . singleQuote(uppercase($entry)));
        if ($qty === CLX_NUM_ZERO) {
		  if (!empty($this->debug)) {
			 echoLog('DICT->saveEntry Creating new item for entry: ' . $entry . '<br>');
		  }
		  $rid = ClxCommon::$db->createRecord($this->tablename, $r, false);
		  unset($rid);
        } else {
		  $old_entry = ClxCommon::$db->getRecord($this->tablename, 'UPPER(ENTRY)=' . singleQuote(uppercase($entry)) . ' AND UPPER(LANGUAGE)=' . singleQuote(uppercase($language)));
		  if ($old_entry['VALUE'] != $r['VALUE']) {
			 if (!empty($update_entry)) {

			     $new_entry_version = (int) ClxCommon::$db->getField($this->tablename, 'MAX(ENTRY_VERSION)+1', 'UPPER(ENTRY) LIKE ' . singleQuote(uppercase($entry) . '%'));
			     $i = CLX_INIT_ZERO;
			     while ($i < CLX_NUM_30) {
                     if (ClxCommon::$db->oneExistsRecords($this->tablename, 'UPPER(ENTRY) = ' . singleQuote((uppercase($entry) . '-' . $new_entry_version))) == CLX_NUM_ZERO) {
					   break;
			         } else {
					   $new_entry_version++;
			         }
			         $i++;
			     }
			     $r['ENTRY_VERSION'] = $new_entry_version;
					$old_entries = (kDatabaseType === 'oci') ? ClxCommon::$db->getRecords($this->tablename, 'UPPER(ENTRY)=' . singleQuote(uppercase($entry)) ): ClxCommon::$db->getRecords($this->tablename, 'UPPER(ENTRY)=' . singleQuote(uppercase($entry)));
			     while ($old_entry = ClxCommon::$db->getNextRecord($old_entries)) {
			         $rn = array();
			         $rn['ENTRY'] = $r['ENTRY'] . '-' . $new_entry_version;
			         $rn['MDATE'] = xFormat::getNow('Y-m-d- H:i:s');
			         $rn['LANGUAGE'] = clx_language_lower_upper($old_entry['LANGUAGE']);
			         $rn['ENTRY_VERSION'] = $new_entry_version;
			         $rn['SOURCE_LANGUAGE'] = clx_language_lower_upper($old_entry['SOURCE_LANGUAGE']);
			         $rn['VALUE'] = $old_entry['VALUE'];
			         if ($old_entry['LANGUAGE'] == clx_language_lower_upper($r['LANGUAGE'])) {
					   $rn['VALUE'] = $r['VALUE'];
			         }
			         $rid = ClxCommon::$db->createRecord($this->tablename, $rn, false);
			         unset($rid);
			         $entry = $rn['ENTRY'];
			     }
				 ClxCommon::$db->freeResult($old_entries);
			 } else {
			     ClxCommon::$db->loadDbRecord($this->tablename, $r, false, 'UPPER(ENTRY)=' . singleQuote(uppercase($entry)) . ' AND UPPER(LANGUAGE)=' . singleQuote(uppercase($language)));
			 }
		  }
        }
        $this->clearDictHeap($entry);
        return $entry;
    }

    /**
     * Creates a new entry into the dictionary
     */
    function createEntry() {
        if (!empty($this->debug))
		  echoLog('Entering DICT->createEntry');
        $r = array();
        if (func_num_args() == CLX_NUM_TWO) {
		  $form_fieldname = func_get_arg(CLX_NUM_ZERO);
		  $record = func_get_arg(CLX_NUM_ONE);
		  $r['VALUE'] = clx_html_spchrs_decode($record[$form_fieldname . '_DICT']);
        } else {
		  $value = clx_html_spchrs_decode(func_get_arg(CLX_NUM_ZERO));
		  $r['VALUE'] = $value;
        }
        $r['ENTRY'] = clx_language_lower_upper($this->getNextEntryId($r['VALUE']));
        $qty = ClxCommon::$db->oneExistsRecords($this->tablename, 'ENTRY=' . singleQuote($r['ENTRY']));
        if ($qty == CLX_NUM_ZERO) {
		  $r['MDATE'] = xFormat::getNow('Y-m-d- H:i:s');
			if (empty($_SESSION ['prefs'] ['LANGUAGE']))
				$_SESSION ['prefs'] ['LANGUAGE'] = getCurrentLanguage();
		
		  $r['LANGUAGE'] = clx_language_lower_upper($_SESSION ['prefs'] ['LANGUAGE']);
		  $r['ENTRY_VERSION'] = CLX_NUM_ZERO;
		  $r['TOOL_ENTRY'] = CLX_NUM_ZERO;
		  $r['SOURCE_LANGUAGE'] = clx_language_lower_upper(getCurrentLanguage());
		  ClxCommon::$db->createRecord($this->tablename, $r, false);
        }
        if (!empty($this->debug)) {
		  echoLog('DICT->createEntry, Searching entry. Found ' . $qty . ' if $qty>0 entry not created');
        }
        return $r['ENTRY'];
    }

    /**
     * Creates dictionary entry by value
     *
     * @param type $value
     * @return type
     */
    function createEntryByValue($value) {
        $r = array();
        $entry = $this->getNextEntryId($value);
        $qty = ClxCommon::$db->oneExistsRecords($this->tablename, 'UPPER(ENTRY)=' . singleQuote(uppercase($entry)));
        if ($qty == CLX_NUM_ZERO) {
		  $r['ENTRY'] = $entry;
		  $r['VALUE'] = clx_html_spchrs_decode($value);
		  $r['MDATE'] = xFormat::getNow('Y-m-d- H:i:s');
		  $r['LANGUAGE'] = clx_language_lower_upper($_SESSION ['prefs'] ['LANGUAGE']);
		  $r['ENTRY_VERSION'] = CLX_NUM_ZERO;
		  $r['TOOL_ENTRY'] = CLX_NUM_ZERO;
		  $r['SOURCE_LANGUAGE'] = clx_language_lower_upper(getCurrentLanguage());
		  ClxCommon::$db->createRecord($this->tablename, $r, false);
		  if (kApcEnabled) {
			 $DBLT = &getTool('DBLT');
			 $languages = clx_language_lower_upper($DBLT->getListAsArray('LANG'));
			 foreach ($languages as $lang => $langname) {
					$lang=clx_language_lower_upper($lang);
                    apc_deleteFct(kSpaceName . '.dict.' . $entry . $lang);
			 }
		  }
        }
        if (!empty($this->debug))
		  echoLog('DICT->createEntry, Searching entry. Found ' . $qty . ' if $qty>0 entry not created');
        return $entry;
    }

    /**
     * Loads the dictionary entry value into the recordset
     */
    function loadEntry($fieldname, $entry, &$record) {
        if (!empty($this->debug)) {
		  echoLog('Entering DICT->loadEntry<br>');
        }
        $language = clx_language_lower_upper($_SESSION ['prefs'] ['LANGUAGE']);
        if (kDatabaseType === 'oci') {
		  $r = ClxCommon::$db->getRecord($this->tablename, 'UPPER(ENTRY)=' . singleQuote(uppercase($entry)) . '  AND UPPER(LANGUAGE)=' . singleQuote(uppercase($language)));
        } else {
		  $r = ClxCommon::$db->getRecord($this->tablename, 'UPPER(ENTRY)=' . singleQuote(uppercase($entry)) . '  AND UPPER(LANGUAGE)=' . singleQuote(uppercase($language)));
        }
        $record[$fieldname] = $entry;
		if (is_null($r))
			$r=array('VALUE'=>'');
        $record[$fieldname . '_DICT'] = $r['VALUE'];
    }

    function deleteEntry($entry) {
        $this->clearDictHeap($entry);
    }

    function duplicateEntry($original_entry) {
        $language = clx_language_lower_upper($_SESSION ['prefs'] ['LANGUAGE']);
        if (kDatabaseType === 'oci') {
            $qty = ClxCommon::$db->oneExistsRecords($this->tablename, 'UPPER(ENTRY)=UPPER(' . singleQuote($original_entry) . ' ) AND UPPER(LANGUAGE)=' . singleQuote(uppercase($language)));
        } else {
            $qty = ClxCommon::$db->oneExistsRecords($this->tablename, 'ENTRY=' . singleQuote($original_entry) . ' AND UPPER(LANGUAGE)=' . singleQuote(uppercase($language)));
        }
        if ($qty > CLX_NUM_ZERO) {
		  if (kDatabaseType === 'oci') {
			 $r = ClxCommon::$db->getRecord($this->tablename, 'UPPER(ENTRY)=UPPER(' . singleQuote($original_entry) . ') AND UPPER(LANGUAGE)=' . singleQuote(uppercase($language)));
		  } else {
			 $r = ClxCommon::$db->getRecord($this->tablename, 'ENTRY=' . singleQuote($original_entry) . ' AND UPPER(LANGUAGE)=' . singleQuote(uppercase($language)));
		  }
		  unset($r['ID']);
		  $r['ENTRY'] = $this->getNextEntryId();
		  $r['VALUE'] = clx_html_spchrs_decode($ClxCommon::$db->addslashes($r['VALUE']));
		  $r['MDATE'] = xFormat::getNow('Y-m-d- H:i:s');
		  ClxCommon::$db->createRecord($this->tablename, $r, false);
		  return $r['ENTRY'];
        } else {
		  return '';
        }
    }

    protected function clearDictHeap($entry) {
        if (kApcEnabled) {
		  $DBLT = &getTool('DBLT');
		  $languages = $DBLT->getListAsArray('LANG');
		  foreach ($languages as $lang => $langname) {
				$lang=clx_language_lower_upper($lang);
			 unset($langname);
                apc_deleteFct(kSpaceName . '.dict.' . $entry . $lang);
		  }
        }
        $DICA = &getTool('DICA');
        $DICA->removeCache($entry);
        ClxCommon::$db->deleteRecords('CLX_DICTIONARY_HEAP', 'UPPER(ENTRY)=' . singleQuote(uppercase($entry)));
    }

    function getNextEntryId($value) {
        $lang = clx_language_lower_upper(getCurrentLanguage());
        $sql = 'SELECT ENTRY FROM CLX_DICTIONARY WHERE VALUE = ' . singlequote($value) . ' AND TOOL_ENTRY = 0 AND ENTRY_VERSION = 0 AND UPPER(SOURCE_LANGUAGE) = ' . singlequote(uppercase($lang)) ;
        $entry = ClxCommon::$db->getOne($sql);
		$u = (!empty($entry)) ? $entry : clx_language_lower_upper(microtime(true));
        return $u;
    }

}
