<?php

/**
 * Tool Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: jclerma $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2017-11-03 15:09:40 -0400 (Fri, 03 Nov 2017) $
 * @version        $Rev: 9615 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  Included label
 */
class dictionary_heap extends clx_tool {

    var $working;
    var $debug;

    // CONSTRUCTOR //
    function __construct() {
        $this->code = 'DICH';
        $this->name = 'Dictionary Heap';
        $this->system = 'System';
        $this->version = '0.4.1.4';
        $this->has_preferences = true;


        $this->accesskey = 'DICT';
        $this->tablename = 'CLX_DICTIONARY_HEAP';
        $this->record_name_singular = 'dictionary entry';
        $this->record_name_plural = 'dictionary entries';
        $this->primary_field = 'ENTRY';
        $this->secondary_field = 'LANGUAGE';

        $this->working = true;
        $this->debug = false;
    }

    function toolTableStructure() {
        $fields = array();
        $fields['ENTRY'] = array('type' => 'varchar(64)', 'null' => 'YES', 'properties' => '');
        $fields['VALUE'] = array('type' => 'varchar(255)', 'null' => 'YES', 'properties' => '');
        $fields['LANGUAGE'] = array('type' => 'varchar(10)', 'null' => 'NO', 'properties' => '');
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'UNI', 'fieldname' => 'ENTRY,LANGUAGE');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => ((kMysqlVersion >= '5.0') ? 'MEMORY' : 'HEAP'),
		  'fields' => $fields,
		  'indexes' => $indexes);
    }

    function preferencesForm($record, &$form, &$table) {
        $table->addCell(dict('TOOL_DICT_PREF_LANGUAGE'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => $table->width));
        $DBLT = getTool('DBLT');
        $languages = $DBLT->getList('LANG', 'LANGUAGE', uppercase($record['LANGUAGE']));
        $table->addData(dict('GEN_LBL_LANGUAGE') . ':', $languages->source());
    }

    function preferencesCheckForm(&$msg) {

    }

    function preferencesLoad(&$record) {
        $record['LANGUAGE'] = clx_language_lower_upper(getCurrentLanguage());
    }

    function preferencesSave() {
        $USPR = &getTool('USPR');
        $USPR->prfSavePref($_SESSION['user_record']['username'], 'LANGUAGE', clx_language_lower_upper(UserRequest::$post['LANGUAGE']));
    }

    function preferencesDefault(&$record) {
        $values = array();
        $values['LANGUAGE'] = clx_language_lower_upper(kSystemLanguage);
        return $values;
    }

    function entry($entry, $language, $create_fast_link = true) {
        $value = false;
		$language=clx_language_lower_upper($language);
		$search_bd=false;
        if ($this->working) { // master switch
		  if (trim($entry) === '') {
			 return $entry;
		  }
		
		  //Addition to cache dictionary
		  $DICA = getTool('DICA');
		  $entry = uppercase(addslashes($entry));
		  // If not language set it so it will work on base language
		  if (empty($language)) {
			 $language = clx_language_lower_upper(kSystemLanguage);
		  }
			$fetched = false;
			$verf=0;
			if (kApcTtl_1>0) { 
				$value=apc_fetchFct($findVar='dict_'.$entry.'_'.$language,$fetched);
				
				$verf=apc_fetchFct($findVar.'_2',$fetched2);
			}	//$fetched=false;
			$usecache=1;
		//	$verf==0;
		//	$value = false;
			if ($verf==0) {
				if ( empty($fetched)||$value==$entry) {//echo $entry;
					$value = $DICA->searchCache($entry, $language);
				if ($value === false ) {
					if (kDatabaseType === 'oci') {
						$sql = 'SELECT VALUE FROM CLX_DICTIONARY_HEAP WHERE ENTRY = :ENTRY AND UPPER(LANGUAGE) = :LANGUAGE';
						$binds = array('ENTRY' => $entry, 'LANGUAGE' => uppercase($language));
					///	pre("====== value ========");;
						$value = ClxCommon::$db->getOne($sql, $binds);
					///	pre("====== $value ========");;
						unset($binds);
					} else {
						$value = ClxCommon::$db->getField('CLX_DICTIONARY_HEAP', 'VALUE', 'ENTRY=' . singleQuote($entry), 'UPPER(LANGUAGE)=' . singleQuote(uppercase($language)));
					}
					
				}
				 
				if (empty($value) && $value !== '' ) { // no entry in the heap for the selected language
					//echo $entry;
					 if (kDatabaseType === 'oci') {
						 $sql = 'SELECT VALUE FROM CLX_DICTIONARY WHERE ENTRY = :ENTRY AND UPPER(LANGUAGE) = :LANGUAGE';
						 $binds = array('ENTRY' => $entry, 'LANGUAGE' => uppercase($language));
						 $value = ClxCommon::$db->getOne($sql, $binds);
						 $seach_bd=true;
					 } else {
						 $value = ClxCommon::$db->getField('CLX_DICTIONARY', 'VALUE', 'ENTRY=' . singleQuote($entry), 'UPPER(LANGUAGE)=' . singleQuote(uppercase($language)));
					 }
					 if ($value === false) {
						 // if this is the system language, give up
						 if (clx_language_lower_upper($language) == clx_language_lower_upper(kSystemLanguage)) {
						   if ($create_fast_link === true) {
							  $url = 'javascript:openWindow(' . singleQuote(CLX_CHAR_EMPTY_STRING) . ', ' . singleQuote('pop.php?to=DICT&ac=CREATE_FAST&entry=' . $entry) . ', ' . singleQuote('500') . ', ' . singleQuote('300') . ');';
							  $value = '<a href=' . quote($url) . ' style="color: red;">' . $entry . '</a>';
							  $usecache=false;
						   } else {
							  $value = $entry;
							  $usecache=false;
						   }
						 } else { // if not, try the system language
						   $value = $this->entry($entry, clx_language_lower_upper(kSystemLanguage));
						 }
					 } else if (!is_null($value))  {
						 $value = $this->copyToHeap($entry, clx_language_lower_upper($language), $value);
						 $DICA->addCache($entry, clx_language_lower_upper($language), $value);
					 }
				 } else {//If in heap
					 // Not in cache adding it
					 $DICA->addCache($entry, clx_language_lower_upper($language), $value);
				 }
			//	 if ($_SERVER["REMOTE_ADDR"]=='186.91.7.193')
			//		ECHO '----->entry:', $entry,'   fast:',$create_fast_link, ' usercache: ',$usercache,' value:'.$value.'<br>';
				
					if (kApcTtl_1>0  ) {
						if (empty($value)) {
							if ($search_bd===false)
								$value=$entry;
							
						}
						apc_addFct($findVar,$value, kApcTtl_1);
						apc_addFct($findVar.'_2',1, kApcTtl_1);
					}
				
				}
				///$value=str_replace('9E7469CBB0B20C2EC8B3EED0F7D42AC2','',$value);
			}
			//if($value==$entry)
			//	$value='';
			
        } else {
		
		  $value = $entry;
        }
	return stripslashes($value);
    }

    function copyToHeap($entry, $language, $value) {
       	$r = array('ENTRY' => uppercase($entry), 'VALUE' => addslashes($value), 'LANGUAGE' => clx_language_lower_upper($language));
		ClxCommon::$db->loadDbRecord('CLX_DICTIONARY_HEAP', $r, true, 'ENTRY = ' . singleQuote($r['ENTRY']) . ' AND UPPER(LANGUAGE) = ' . singleQuote(uppercase($r['LANGUAGE'])));
        
		return $value;
    }

}
