<?php

/**
 * Tool Class
 * For more information contact  support@alenet.com
 *
 * @package		Clientix.Tool.IbsSockets
 * @abstract		IBS Socket Messages Manager
 * @author       $Author: jclerma $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2017-11-03 15:09:40 -0400 (Fri, 03 Nov 2017) $
 * @version        $Rev: 9615 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  Included label
 */
class eb_socket_errors extends clx_tool {

    // CONSTRUCTOR //
    function __construct() {
        $this->code = 'EBIE';
        $this->name = 'IBS Socket Manager Import Errors';
        $this->system = 'Sockets';
        $this->version = '1.0.0.0';


        $this->accesskey = 'EBSK';

        $this->tablename = 'CLX_EB_SOCKET_ERRORS';
        $this->record_name_singular = 'eb socket error';
        $this->record_name_plural = 'eb socket errors';
        $this->primary_field = 'ID';
        $this->secondary_field = 'EBSOCKETCODE';
    }

    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['EBSOCKETCODE'] = array('type' => 'varchar(16)', 'null' => 'NO', 'properties' => '');
        $fields['FIELDNAME'] = array('type' => 'varchar(255)', 'null' => 'NO', 'properties' => '');
        $fields['LINENUM'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['ERRORNUM'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['ERRORMSG'] = array('type' => 'varchar(255)', 'null' => 'NO', 'properties' => '');
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'MUL', 'fieldname' => 'EBSOCKETCODE');
        $indexes[$ipf . '03'] = array('index' => 'MUL', 'fieldname' => 'LINENUM');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB',
		  'fields' => $fields,
		  'indexes' => $indexes);
    }

    function action($ac, $idlist) {
        if ($this->getAccess()->canRead()) {
		  $contents = parent::action($ac, $idlist);
        } else {
		  $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [EB_SOCKET_ERRORS-CAN_READ][' . $ac . '] ' . $this->getCode(false));
        }
        return $contents;
    }

    function makepanel($groupcode = null) {
        if ($this->getAccess()->canRead()) {
		  $t = time();
		  $to = $this->code;
		  $here = 'tool.php?to=' . $to . '&t=' . $t;
		  $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
		  $form->setProp(CLX_PROP_ID, 'EBIEPanel');
		  $form->addArgument(CLX_PROP_TO, $this->code);
		  $form->addArgument('ebsocketcode', $groupcode);
		  $form->addArgument('ac', 'NOT_DEFINED');
		  $selectclause = '*';
		  $fromclause = $this->tablename;
		  $whereclause = 'EBSOCKETCODE=' . singleQuote($groupcode);
		  $sortclause = 'LINENUM';
		  $page = new xPaging();
		  $page->setProp('rpp', CLX_NUM_TEN_THOUSAND);
		  $page->setProp('printheaders', false);
		  $page->setProp('selector', false);
		  $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
		  $table = new htmlTable(CLX_WIDTH_THREE);
		  $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
		  $page->header($table);
		  $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_LINE_NUM'), dict('GEN_LBL_TABLE_FIELD'), dict('GEN_LBL_ERROR_MSG')));
		  $i = CLX_INIT_ZERO;
		  while ($record = $page->nextRecord()) {
			 $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($record['LINENUM'], $record['FIELDNAME'], $record['ERRORMSG']));
			 $i++;
		  }
		  $form->addContents($page->footers($table, ''));
		  return buildPanelInterface(dict('TOOL_EBIE_TITLE_LIST'), 'IMPORT_ERRORS_' . $groupcode, $form->source(), CLX_NUM_MINUS_1);
        } else {
		  return CLX_CHAR_EMPTY_STRING;
        }
    }

}
