<?php

/**
 * Tool Class
 * For more information contact  support@alenet.com
 *
 * @package		Clientix.Tool.IbsSockets
 * @abstract		IBS Socket Messages Manager
 * @author       $Author: cramirez $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2018-02-26 09:26:43 -0500 (Mon, 26 Feb 2018) $
 * @version        $Rev: 9747 $
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  package import bug fixed: wrong table name for CLX_EB_SOCKET_MAPPINGS, CLX_EB_SOCKET_TRANSLATIONS
 *  readMessage() function added to read files y socket streams from IBS
 *  bug fix: in the returning array $this->msgrecord[] the element 'DECIMALS' was wrong assigned to $field['DECIMALS'], and it should be assigned to $field['FIELD_DECIMALS']. Fixed at createMessage() and readMessage() methods
 *  warning fix: send() method, line 1089, $result = false if it cannot write to the socket to avoid the 'undefined $return' notice. Additionally it was assigniing $return variable instead $result one witch is the one in the return statement
 *  		send(): local var. $sck converted to property $sck. See EIBS_utilitities.php
 */
class eb_sockets extends clx_tool {

    var $import_types = array('CSV' => 'GEN_LBL_DELIMITED_FIELDS', 'FRL' => 'GEN_LBL_FRL');
    var $importForm;
    var $msgcode;  //Message Code
    var $errors;  //Field Message Errors array
    var $msgrecord;  //Holds message record array
    var $priormsgcode; //Prior Message Code
    var $priormsgrecord; //Prior Message Code
    var $msglength = CLX_INIT_ZERO; //lenght of message
    var $priormsglength = CLX_INIT_ZERO; //lenght of header
    var $skco_code;  //Socket Connector code
    var $tfields_only = false; //Set to true to return only Tablafields values on record
    var $remove_decimal_point = true;
    var $rename_file = false;
    var $rename_file_extension = '.OLD';
    var $message_record = array();
    var $verbouse = false;
    //Message structure caching properties.
    var $mappings = array(); //stores message mappings
    var $translations = array(); //stores message translations
    var $search_keys = array(); //stores search keys
    var $read_onlys = array(); //stores readonlys
    var $formulas = array(); //stores readonlys
    var $table_info = array(); //stores table information for mappings
    var $copy_fields = array();
    var $debug = false;
    var $direction = CLX_INIT_ZERO; //Direction of interfase 0=in o 1=Out
    var $sck;
    var $timeout = CLX_NUM_ONE; //Seconds Use connector intead of changing this.
    var $record_proces_counter = array('create' => CLX_NUM_ZERO, 'modify' => CLX_NUM_ZERO, 'delete' => CLX_NUM_ZERO, 'replace' => CLX_NUM_ZERO);
    var $record_commit_counter = CLX_INIT_ZERO; // [CLX_NUM_ZERO] = only one commit at the end of batch processing, [N] = (any positive number) do a commit after process N records
    //Message types
    var $msg_type = false; //Type of message CSV, FIX
    var $record_delim = false; //Line separator for type CSV Usually \r, '.CLX_CHAR_NL.'
    var $field_delim = false; //Field Separator for type CSV Usually \t
    var $action_types = array(
        'NONE' => 'GEN_LBL_NOACTION',
        'SELECT' => 'GEN_LBL_SELECT',
        'CREATE' => 'GEN_LBL_CREATE',
        'REPLACE' => 'GEN_SKMS_REPLACE',
        'MODIFY' => 'GEN_LBL_MODIFY',
        'LOAD' => 'GEN_SKMS_LOAD',
        'DELETE' => 'GEN_LBL_DELETE',
        'PROC' => 'GEN_LBL_PROCESS_ENGINE',
        'CODE' => 'GEN_LBL_EXECUTE_CODE',
        'CREATE_REMOTE' => 'GEN_LBL_CREATE_REMOTE', //see 0001
        'MODIFY_REMOTE' => 'GEN_LBL_MODIFY_REMOTE', //see 0005
        'SELECT_FOR_UPDATE_REMOTE' => 'GEN_LBL_SEL_FOR_UP_REMOTE'
    );
    var $convert_skt_output_to_ascii = false; //convert soket output to ascii in tostring method
    // Caso 1002664 - BEGIN
    var $create_log = false; // Create a .log file in batch operations
    var $log_file_handle; // Log file handle
    var $use_record_delimiter_reading_fixlen_files = false; // flag to use record delimiter instead of record length reading fixed length files
    var $invalid_field_values_report = true; // flag to include invalid field values in log report summary
    var $invalid_field_values_ar = array(); // invalid field values array property
    var $invalid_field_values_max_cnt = 40; // invalid field values maximun entry counter for each individual field
    var $valid_field_values_ar = NULL; // values considered as valid regardless field validation rules
    // Caso 1002664 - END
    var $generate_column_headers = false; /* 2018-02-23 - Caso 1003630 - PBL-Interfaz contable: Genera la primera linea con los nombres de las columnas */
    var $column_headers = array(); /* 2018-02-23 - Caso 1003630 - PBL-Interfaz contable: Genera la primera linea con los nombres de las columnas */

    /**
     * List of IBS Operation Codes
     * The list includes:
     * 		Create 0001,
     * 		Query: 0002
     * 		Manteinance(2 steps): 0004->0002+0005
     * 		Modify: 0005
     * 		Dump File record: blank
     * @var string
     */
    var $ibs_operation_codes = array('' => 'FILEDUMP',
        '0001' => 'CREATE',
        '0002' => 'SELECT',
        '0003' => 'UNDEFINED',
        '0004' => 'MANTEINANCE',
        '0005' => 'SEND_MODIFIED_DATA');

    /**
     * Constructor
     */
    function __construct() {
        $this->code = 'EBSK';
        $this->name = 'IBS Socket Manager';
        $this->system = 'Sockets';
        $this->version = '1.0.0.0';

        $this->has_access = true;
        $this->has_package = true;

        $this->tablename = 'CLX_EB_SOCKETS';
        $this->record_name_singular = 'eb socket import';
        $this->record_name_plural = 'eb socket imports';
        $this->primary_key = array('IDCODE');
        $this->primary_field = 'IDCODE';
        $this->secondary_field = 'IDNAME';

        //Ini message Record
        $this->msgrecord = array();
        $this->msgcode = '';
        $this->msglength = CLX_INIT_ZERO;
        $this->priormsgrecord = array();
        $this->priormsgcode = '';
        $this->priormsglength = CLX_INIT_ZERO;
        $this->skco_code = '';
        $this->errors = array();
        $this->editform = array(
		  CLX_CLASS_TITLE => 'TOOL_EBSK_TITLE_CREATE',
		  'panels' => array(
			 array(
			     'type' => 'normal',
			     'entries' => array(
			         array(
					   'label' => 'GEN_LBL_CODE',
					   'fields' => array(
						  array('name' => 'idcode', 'type' => 'string',
						      'default' => '', 'size' => '16',
						      'displaysize' => '16', 'dontmodify' => '1',
						      'validations' => array(
								array('type' => 'sqlobject', 'required' => '1'),
								array('type' => 'unique', 'fieldname' => 'idcode', 'tablename' => $this->tablename)
						      )
						  )
					   )
			         ),
			         array(
					   'label' => 'GEN_LBL_NAME',
					   'fields' => array(
						  array('name' => 'idname', 'type' => 'string', 'default' => '', 'size' => '64', 'displaysize' => '48',
						      'validations' => array(
								array('type' => 'string', 'required' => '1')
						      )
						  )
					   )
			         ),
			         array(
					   'label' => 'GEN_LBL_DESCRIPTION',
					   'fields' => array(
						  array('name' => 'description', 'type' => 'text', 'default' => '', 'cols' => '48', 'rows' => '4')
					   )
			         ),
			         array(
					   'label' => 'GEN_LBL_VERSION',
					   'fields' => array(
						  array('name' => 'version', 'type' => 'string', 'default' => '1.0.0', 'size' => '32', 'displaysize' => '16',
						      'validations' => array(
								array('type' => 'string', 'required' => '1')
						      )
						  )
					   )
			         ),
			         array(
					   'label' => 'GEN_LBL_MODULECODE',
					   'fields' => array(
						  array('name' => 'modulecode', 'type' => 'lookup', 'default' => '', 'tool' => 'DBMO',
						      'validations' => array(
								array('type' => 'sqlobject', 'required' => '1'),
								array('type' => 'lookup', 'fieldname' => 'idcode', 'tablename' => 'CLX_DB_MODULES')
						      )
						  )
					   )
			         )
			     )
			 ),
			 array(
			     CLX_CLASS_TITLE => 'GEN_LBL_CONNECTION_INFO',
			     'type' => 'normal',
			     'entries' => array(
			         array(
					   'label' => 'GEN_LBL_SKCONNECTOR_CODE',
					   'fields' => array(
						  array('name' => 'skconnector_code', 'type' => 'lookup', 'default' => '', 'tool' => 'SKCO',
						      'validations' => array(
								array('type' => 'sqlobject', 'required' => '0'),
								array('type' => 'lookup', 'fieldname' => 'idcode', 'tablename' => 'CLX_SK_CONNECTORS')
						      )
						  )
					   )
			         )
			     )
			 ),
			 array(
			     CLX_CLASS_TITLE => 'GEN_LBL_FILETYPE_INFO',
			     'type' => 'normal',
			     'entries' => array(
			         array(
					   'label' => 'GEN_LBL_TYPE',
					   'fields' => array(
						  array('name' => 'type', 'type' => 'dynamic',
						      'panels' => $this->import_types
						  )
					   )
			         )
			     )
			 ),
			 array(
			     'type' => 'dynamic',
			     'code' => 'CSV',
			     'entries' => array(
			         array(
					   'label' => 'GEN_LBL_FIELD_DELIM',
					   'fields' => array(
						  array('name' => 'field_delim', 'type' => 'radiobutton', 'value' => '[9]', CLX_CLASS_PROMPT => 'GEN_LBL_TAB'),
						  array('name' => 'field_delim', 'type' => 'radiobutton', 'value' => ',', CLX_CLASS_PROMPT => 'GEN_LBL_COMMA'),
						  array('name' => 'field_delim', 'type' => 'radiobutton', 'value' => ';', CLX_CLASS_PROMPT => 'GEN_LBL_SEMICOLON'),
						  array('name' => 'field_delim', 'type' => 'radiobutton', 'value' => '[32]', CLX_CLASS_PROMPT => 'GEN_LBL_SPACE'),
						  array('name' => 'field_delim', 'type' => 'radiobutton', 'value' => 'other', CLX_CLASS_PROMPT => 'GEN_LBL_OTHER'),
						  array('name' => 'field_delim_other', 'type' => 'string', 'default' => '', 'size' => '24', 'displaysize' => '24')
					   )
			         ),
			         array(
					   'label' => 'GEN_LBL_RECORD_DELIM',
					   'fields' => array(
						  array('name' => 'record_delim', 'type' => 'radiobutton', 'value' => '[13]', CLX_CLASS_PROMPT => 'GEN_LBL_CARRIAGE_RETURN'),
						  array('name' => 'record_delim', 'type' => 'radiobutton', 'value' => '[10]', CLX_CLASS_PROMPT => 'GEN_LBL_LINE_FEED'),
						  array('name' => 'record_delim', 'type' => 'radiobutton', 'value' => '[13][10]', CLX_CLASS_PROMPT => 'GEN_LBL_CRLF'),
						  array('name' => 'record_delim', 'type' => 'radiobutton', 'value' => 'other', CLX_CLASS_PROMPT => 'GEN_LBL_OTHER'),
						  array('name' => 'record_delim_other', 'type' => 'string', 'default' => '', 'size' => '24', 'displaysize' => '24')
					   )
			         ),
			         array(
					   'label' => 'GEN_LBL_IGNORE_LINES',
					   'fields' => array(
						  array('name' => 'ignore_lines', 'type' => 'string', 'default' => '0', 'size' => '4', 'displaysize' => '4',
						      'validations' => array(
								array('type' => 'integer', 'min' => CLX_NUM_ZERO)
						      )
						  )
					   )
			         )
			     )
			 ),
			 array(
			     'type' => 'dynamic',
			     'code' => 'FRL',
			     'entries' => array(
			         array(
					   'label' => 'GEN_LBL_HEADERMSG_CODE',
					   'fields' => array(
						  array('name' => 'headermsg_code', 'type' => 'lookup', 'default' => '', 'tool' => $this->code,
						      'validations' => array(
								array('type' => 'lookup', 'fieldname' => 'idcode', 'tablename' => $this->tablename)
						      )
						  )
					   )
			         ),
			         array(
					   'label' => 'GEN_LBL_HEADER_SIZE',
					   'fields' => array(
						  array('name' => 'header_size', 'type' => 'string', 'default' => '', 'size' => '64', 'displaysize' => '48',
						      'validations' => array(
								array('type' => 'string', 'required' => '1')
						      )
						  )
					   )
			         ),
			         array(
					   'label' => 'GEN_LBL_RECORD_DELIM',
					   'fields' => array(
						  array('name' => 'record_delim', 'type' => 'radiobutton', 'value' => '[13]', CLX_CLASS_PROMPT => 'GEN_LBL_CARRIAGE_RETURN'),
						  array('name' => 'record_delim', 'type' => 'radiobutton', 'value' => '[10]', CLX_CLASS_PROMPT => 'GEN_LBL_LINE_FEED'),
						  array('name' => 'record_delim', 'type' => 'radiobutton', 'value' => '[13][10]', CLX_CLASS_PROMPT => 'GEN_LBL_CRLF'),
						  array('name' => 'record_delim', 'type' => 'radiobutton', 'value' => 'other', CLX_CLASS_PROMPT => 'GEN_LBL_OTHER'),
						  array('name' => 'record_delim_other', 'type' => 'string', 'default' => '', 'size' => '24', 'displaysize' => '24')
					   )
			         ),
			         array(
					   'label' => 'GEN_LBL_IGNORE_LINES',
					   'fields' => array(
						  array('name' => 'ignore_lines', 'type' => 'string', 'default' => '0', 'size' => '4', 'displaysize' => '4',
						      'validations' => array(
								array('type' => 'integer', 'min' => CLX_NUM_ZERO)
						      )
						  )
					   )
			         ),
			         array(
					   'label' => 'GEN_LBL_RECORD_LENGTH',
					   'fields' => array(
						  array('name' => 'record_length', 'type' => 'string', 'default' => '', 'size' => '32', 'displaysize' => '16', 'sufix' => 'GEN_LBL_CHARACTERS', 'readonly' => true,
						      'validations' => array(
								array('type' => 'integer', 'required' => '1')
						      )
						  )
					   )
			         )
			     )
			 )
		  )
        );
        $this->viewform = $this->editform;
        $this->quickfilterform = array(
		  'panels' => array(
			 array(
			     'type' => 'normal',
			     'entries' => array(
			         array(
					   'label' => 'GEN_LBL_CODE',
					   'fields' => array(
						  array(
						      'name' => 'idcode',
						      'type' => 'string',
						      'default' => '',
						      'size' => '16',
						      'displaysize' => '16'
						  )
					   )
			         ),
			         array(
					   'label' => 'GEN_LBL_NAME',
					   'fields' => array(
						  array(
						      'name' => 'idname',
						      'type' => 'string',
						      'default' => '',
						      'size' => '64',
						      'displaysize' => '48'
						  )
					   )
			         ),
			         array(
					   'label' => 'GEN_LBL_MODULECODE',
					   'fields' => array(
						  array(
						      'name' => 'modulecode',
						      'type' => 'lookup',
						      'default' => '',
						      'tool' => 'DBMO'
						  )
					   )
			         )
			     )
			 )
		  )
        );
        $this->toollist = array(
		  'name' => $this->tablename,
		  CLX_CLASS_TITLE => $this->record_name_plural,
		  'fields' => array(
			 array('name' => 'ID', 'type' => 'number', 'label' => 'GEN_LBL_ID'),
			 array('name' => 'IDCODE', 'type' => 'string'),
			 array('name' => 'IDNAME', 'type' => 'number'),
			 array('name' => 'MODULECODE', 'type' => 'lookup', 'tool' => 'DBMO'),
			 array('name' => 'TYPE', 'type' => CLX_CLASS_LIST, 'options' => $this->import_types),
			 array('name' => 'VERSION', 'type' => 'string'),
			 array('name' => 'DESCRIPTION', 'type' => 'text'),
			 array('name' => 'SKCONNECTOR_CODE', 'type' => 'lookup', 'tool' => 'SKCO', 'label' => 'GEN_LBL_SKCONNECTOR_CODE')
		  ),
		  'buttons' =>
		  array(
			 array('access' => 'canread', 'tool' => 'has_display', 'library' => 'GEN_LBL_CHECK', 'name' => 'CHECK', 'prop' => 'ONE'),
			 array('access' => 'canread', 'tool' => 'has_display', 'library' => 'GEN_LBL_FILE_UTIL', 'name' => 'FILE_UTIL', 'prop' => 'ONE')
		  )
        );
        if (defined('EBSK_IBS_SOCKET_ASCCI_TRANSLATE')) {
		  $this->convert_skt_output_to_ascii = (EBSK_IBS_SOCKET_ASCCI_TRANSLATE === '1' ? true : false);
        }
        if (defined('EBSK_IBS_DEBUG') && EBSK_IBS_DEBUG === '1') {
		  $this->debug = true;
        }
    }

    /**
     * Install procedure
     *
     */
    function install() {
        $UIMI = &getTool('UIMI');
        $UIMI->add($this->code, 'MENU_TOOLS:MENU_SERVCONNECTOR:MENU_EBSOCKETS', 'TOOL', $this->code, 'ac=LIST');
    }

    /**
     * Describes the tool table structure.
     * @return array ('type' => 'varchar(255)', 'properties'=>'auto increment',
     * 				'index'=>'', default = '', 'dict'=>true)
     * 				 dict true or false means that field holds dictionary entry
     */
    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['IDCODE'] = array('type' => 'varchar(16)', 'null' => 'NO', 'properties' => '');
        $fields['IDNAME'] = array('type' => 'varchar(64)', 'null' => 'NO', 'properties' => '');
        $fields['VERSION'] = array('type' => 'varchar(32)', 'null' => 'NO', 'properties' => '');
        $fields['VERSION_COUNTER'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['CDATE'] = array('type' => 'datetime', 'null' => 'YES', 'properties' => '');
        $fields['MDATE'] = array('type' => 'datetime', 'null' => 'YES', 'properties' => '');
        $fields['USERNAME'] = array('type' => 'varchar(30)', 'null' => 'NO', 'properties' => '');
        $fields['DESCRIPTION'] = array('type' => 'mediumtext', 'properties' => '');
        $fields['MODULECODE'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '');
        $fields['HEADER_SIZE'] = array('type' => 'varchar(16)', 'null' => 'NO', 'properties' => '');
        $fields['RECORD_LENGTH'] = array('type' => 'varchar(16)', 'null' => 'NO', 'properties' => '');
        $fields['SKCONNECTOR_CODE'] = array('type' => 'varchar(16)', 'null' => 'NO', 'properties' => '');
        $fields['HEADERMSG_CODE'] = array('type' => 'varchar(16)', 'null' => 'YES', 'properties' => '');
        $fields['TYPE'] = array('type' => 'varchar(4)', 'null' => 'NO', 'default' => '' . singleQuote('FR') . '', 'default_raw' => 'FRL');
        $fields['FIELD_DELIM'] = array('type' => 'varchar(16)', 'null' => 'YES', 'properties' => '');
        $fields['RECORD_DELIM'] = array('type' => 'varchar(16)', 'null' => 'YES', 'properties' => '');
        $fields['IGNORE_LINES'] = array('type' => 'int(3) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'UNI', 'fieldname' => 'IDCODE');
        $indexes[$ipf . '03'] = array('index' => 'MUL', 'fieldname' => 'IDNAME');
        $indexes[$ipf . '04'] = array('index' => 'MUL', 'fieldname' => 'USERNAME');
        $indexes[$ipf . '05'] = array('index' => 'MUL', 'fieldname' => 'MODULECODE');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB', 'fields' => $fields, 'indexes' => $indexes);
    }

    /**
     * Pack tool record on xml Format
     * @param array record
     * @return obj
     */
    function pack($record) {
        $o = $this->packRecord($record);
        $code = $record['IDCODE'];
        $objs = new XMLObject($this->code . '_OBJECTS');
        $this->packsub($objs, 'EBIF', 'SELECT * FROM CLX_EB_SOCKET_FIELDS WHERE EBSOCKETCODE=' . singleQuote($code));
        $this->packsub($objs, 'EBIC', 'SELECT * FROM CLX_EB_SOCKET_MAPPINGS WHERE EBSOCKETCODE=' . singleQuote($code));
        $o->addObject($objs);
        return $o;
    }

    /**
     * Loads package into system
     * @param string $package
     */
    function unpack($parameters) {
        $parameters = array_values(array_merge(array('package' => null), $parameters));
        list($package) = $parameters;
        $record = $this->unpackRecord($package);
        $ebsocketcode = $record['IDCODE'];
        $exists = (ClxCommon::$db->oneExistsRecords($this->tablename, 'IDCODE=' . singleQuote($ebsocketcode)) > CLX_NUM_ZERO);
        if ($exists === true) {
		  $mappings = ClxCommon::$db->getRecords('CLX_EB_SOCKET_MAPPINGS', 'EBSOCKETCODE=' . singleQuote($ebsocketcode));
		  while (($mapping = ClxCommon::$db->getNextRecord($mappings))) {
			 $mappingid = $mapping['ID'];
			 ClxCommon::$db->deleteRecords('CLX_EB_SOCKET_TRANSLATIONS', 'MAPID=' . $mappingid);
		  }
		  ClxCommon::$db->deleteRecords('CLX_EB_SOCKET_FIELDS', 'EBSOCKETCODE=' . singleQuote($ebsocketcode));
		  ClxCommon::$db->deleteRecords('CLX_EB_SOCKET_MAPPINGS', 'EBSOCKETCODE=' . singleQuote($ebsocketcode));
		  ClxCommon::$db->deleteRecords('CLX_EB_SOCKET_EXCEPTIONS', 'EBSOCKETCODE=' . singleQuote($ebsocketcode));
		  ClxCommon::$db->deleteRecords('CLX_EB_SOCKET_EXECUTIONS', 'EBSOCKETCODE=' . singleQuote($ebsocketcode));
		  ClxCommon::$db->deleteRecords('CLX_EB_SOCKET_ERRORS', 'EBSOCKETCODE=' . singleQuote($ebsocketcode));
		  ClxCommon::$db->deleteRecords($this->tablename, 'IDCODE=' . singleQuote($ebsocketcode));
        }
        $importid = ClxCommon::$db->createRecord($this->tablename, $record, false);
        if ($package->hasSubObject($this->code . '_OBJECTS')) {
		  $objs = $package->getObjectXml($this->code . '_OBJECTS');
		  foreach ($objs->objects as $o) {
			 $toolcode = $o->tag;
			 $tool = &getTool($toolcode);
			 $tool->unpack(array('package' => $o, 'id' => $importid));
		  }
        }
    }

    /**
     * Action is called for every user interaction with the module...
     *
     * @param unknown_type $ac
     * @param unknown_type $idlist
     * @return unknown
     */
    function action($ac, $idlist) {
        if ($this->getAccess()->canRead()) {
		  switch ($ac) {
			 case 'FILE_UTIL':
			     $contents = $this->file_utility();
			     break;
			 case 'CHECK':
			     if (!empty(UserRequest::$post['confirm_check'])) {
			         $contents = $this->check();
			     } else {
			         $contents = $this->askCheck();
			     }
			     break;
			 default:
			     $contents = parent::action($ac, $idlist);
			     break;
		  }
        } else {
		  $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [EB_SOCKETS-CAN_READ][' . $ac . '] ' . $this->getCode(false));
        }
        return $contents;
    }

    /**
     * Builds Tool Search form
     * @return string
     */
    function searchForm() {
        $msg = array();
        $contents = '';
        // Search Form
        $form = new htmlForm(CLX_PROP_ACTION, UserRequest::$server['PHP_SELF']);
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'SEARCH');
        $form->addArgument('confirm', '1');
        $form->addArgument(CLX_PROP_TARGET, UserRequest::$request[CLX_PROP_TARGET]);
        if (!array_key_exists('EBSK_idcode', UserRequest::$post))
		  UserRequest::$post['EBSK_idcode'] = '';
        if (!array_key_exists('EBSK_idname', UserRequest::$post))
		  UserRequest::$post['EBSK_idname'] = '';
        if (!array_key_exists('EBSK_modulecode', UserRequest::$post))
		  UserRequest::$post['EBSK_modulecode'] = '';
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_EBSK_TITLE_SEARCH'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addField($msg, dict('GEN_LBL_CODE') . ':', 'EBSK_idcode', UserRequest::$post['EBSK_idcode'], CLX_NUM_16, CLX_NUM_16);
        $table->addField($msg, dict('GEN_LBL_NAME') . ':', 'EBSK_idname', UserRequest::$post['EBSK_idname'], CLX_NUM_32, CLX_NUM_64);
        $table->addField($msg, dict('GEN_LBL_MODULECODE') . ':', 'EBSK_modulecode', UserRequest::$post['EBSK_modulecode'], CLX_NUM_FOUR, CLX_NUM_FOUR);
        $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_SEARCH'), array('action' => 'SEARCH', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        $c = new htmlButtonCloseWindow(dict('GEN_LBL_CANCEL'));
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->setContents($table->source());
        $contents .= $form->source();
        // Search Procedure
        if (isset(UserRequest::$post['confirm'])) {
		  $table = new htmlTable(CLX_WIDTH_TWO);
		  $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
		  $criteria = '';
		  if (UserRequest::$post['EBSK_idcode'] <> '') {
			 $criteria .= 'UPPER(IDCODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['EBSK_idcode']) . '%');
		  }
		  if (UserRequest::$post['EBSK_idname'] <> '') {
			 $criteria .= 'UPPER(IDNAME) LIKE ' . singleQuote(uppercase(clx_change_especial_char_html (UserRequest::$post['EBSK_idname'])) . '%');
		  }
		  if (UserRequest::$post['EBSK_modulecode'] <> '') {
			 $criteria .= 'UPPER(MODULECODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['EBSK_modulecode']) . '%');
		  }
            $qty = ClxCommon::$db->oneExistsRecords($this->tablename, $criteria);
		  if ($qty == CLX_NUM_ZERO) {
			 $table->addCell(dict('TOOL_GEN_ERR_NOMATCH'), array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
		  } else {
			 $table->addCell(dict('TOOL_GEN_MSG_FOUNDRECORDS'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
			 $records = ClxCommon::$db->getRecords($this->tablename, $criteria);
			 $i = CLX_INIT_ZERO;
			 while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
			     $js = 'javascript:setLookupValue(' . quote(UserRequest::$request[CLX_PROP_TARGET]) . ', ' . quote($record['IDCODE']) . ');';
			     $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array('<page t=eb_sockets></page><a href=' . singleQuote($js) . '>' . $record['IDCODE'] . '</a>', $record['IDNAME']));
			     $i++;
			 }
		  }
		  $contents .= $table->source();
        }
        return $contents;
    }

    /**
     * Builds confirm delete form for tool
     * @param obj $table
     * @param array $record
     */
    function confirmDeleteFill(&$table, $record) {
        $table->addData(dict('GEN_LBL_CODE') . ':', $record['EBSK_idcode']);
        $table->addData(dict('GEN_LBL_NAME') . ':', $record['EBSK_idname']);
    }

    /**
     * Build display form contents
     * @param array $record
     * @return string
     */
    function viewFormFill($record) {
        addHistory('ebsocketcode', $record['EBSK_idcode']);
        addHistory('ebsocket_type', $record['EBSK_type']);
        $contents = parent::viewFormFill($record);
        // Mappings
        $EBIC = &getTool('EBIC');
        $contents .= $EBIC->makepanel($record['EBSK_idcode']);
        // Exceptions
        if (!empty($record['EBIM_type']) && $record['EBIM_type'] === 'FRL') {
		  $EBIP = &getTool('EBIP');
		  $contents .= $EBIP->makepanel($record['EBSK_idcode']);
        }
        // Executions
        $EBIX = &getTool('EBIX');
        $contents .= $EBIX->makepanel($record['EBSK_idcode']);
        // Errors
        $EBIE = &getTool('EBIE');
        $contents .= $EBIE->makepanel($record['EBSK_idcode']);
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addData(dict('GEN_LBL_CDATE') . ':', $record['EBSK_cdate']);
        $table->addData(dict('GEN_LBL_MDATE') . ':', $record['EBSK_mdate']);
        $table->addData(dict('GEN_LBL_USERNAME') . ':', $record['EBSK_username']);
        $contents .= buildPanelInterface(dict('GEN_TTL_SYSTEMINFO'), 'SYSTEMINFO', $table->source(), CLX_NUM_MINUS_1);
        return $contents;
    }

    /**
     * Adds Filter Form contents
     * @param array $msg
     * @param obj $table
     */
    function filterFormFill($msg, &$table) {
        if (!array_key_exists('EBSK_idcode', UserRequest::$post))
		  UserRequest::$post['EBSK_idcode'] = '';
        if (!array_key_exists('EBSK_idname', UserRequest::$post))
		  UserRequest::$post['EBSK_idname'] = '';
        if (!array_key_exists('EBSK_modulecode', UserRequest::$post))
		  UserRequest::$post['EBSK_modulecode'] = '';
        $table->addField($msg, dict('GEN_LBL_CODE'), 'EBSK_idcode', UserRequest::$post['EBSK_idcode'], CLX_NUM_16, CLX_NUM_16);
        $table->addField($msg, dict('GEN_LBL_NAME'), 'EBSK_idname', UserRequest::$post['EBSK_idname'], CLX_NUM_32, CLX_NUM_64);
        $table->addField($msg, dict('GEN_LBL_MODULECODE'), 'EBSK_modulecode', UserRequest::$post['EBSK_modulecode'], CLX_NUM_FOUR, CLX_NUM_FOUR);
        $sort = new HtmlSelect('EBSK_sort');
        $sort->addItem('', '', true);
        $sort->addItem(dict('GEN_LBL_CODE'), 'idcode', false);
        $sort->addItem(dict('GEN_LBL_NAME'), 'idname', false);
        $sort->addItem(dict('GEN_LBL_MODULECODE'), 'modulecode', false);
        $table->addData(dict('GEN_LBL_ORDERBY') . ':', $sort->source());
    }

    /**
     * Register filter information on user session
     */
    function setfilter() {
        $filter = '';
        $boolean = '';
        if (!empty(UserRequest::$post['EBSK_idcode'])) {
		  $filter .= $boolean . ' UPPER(IDCODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['EBSK_idcode']) . '%');
		  $boolean = 'AND';
        }
        if (!empty(UserRequest::$post['EBSK_idname'])) {
		  $filter .= $boolean . ' UPPER(IDNAME) LIKE ' . singleQuote(uppercase(clx_change_especial_char_html (UserRequest::$post['EBSK_idname'])) . '%');
		  $boolean = 'AND';
        }
        if (!empty(UserRequest::$post['EBSK_modulecode'])) {
		  $filter .= $boolean . ' UPPER(MODULECODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['EBSK_modulecode']) . '%');
		  $boolean = 'AND';
        }

        $sortby = (array_key_exists('EBSK_sort', UserRequest::$post)) ? UserRequest::$post['EBSK_sort'] : CLX_CHAR_EMPTY_STRING;
        $this->setFilterKeys($filter, $sortby, CLX_NUM_ZERO);
    }

    /**
     * Loads current record on selected id
     *
     * @param integer $id
     * @return array
     */
    function loadToolRecord($recordid) {
        $values = ClxCommon::$db->getRecord($this->tablename, 'ID= ' . $recordid);
        $record = array();
        $record['EBSK_id'] = $recordid;
        $record['EBSK_idcode'] = $values['IDCODE'];
        $record['EBSK_idname'] = $values['IDNAME'];
        $record['EBSK_version'] = $values['VERSION'];
        $record['EBSK_version_counter'] = $values['VERSION_COUNTER'];
        $record['EBSK_cdate'] = xFormat::date($values['CDATE']);
        $record['EBSK_mdate'] = xFormat::date($values['MDATE']);
        $record['EBSK_username'] = $values['USERNAME'];
        $record['EBSK_description'] = $values['DESCRIPTION'];
        $record['EBSK_headermsg_code'] = $values['HEADERMSG_CODE'];
        $record['EBSK_modulecode'] = $values['MODULECODE'];
        $record['EBSK_header_size'] = $values['HEADER_SIZE'];
        $record['EBSK_record_length'] = $values['RECORD_LENGTH'];
        $record['EBSK_type'] = $values['TYPE'];
        $record['EBSK_skconnector_code'] = $values['SKCONNECTOR_CODE'];
        $record['EBSK_ignore_lines'] = $values['IGNORE_LINES'];
        $record['EBSK_field_delim'] = $values['FIELD_DELIM'];
        $record['EBSK_record_delim'] = $values['RECORD_DELIM'];
        $record['EBSK_field_delim_other'] = $values['FIELD_DELIM'];
        $record['EBSK_record_delim_other'] = $values['RECORD_DELIM'];
        return $record;
    }

    /**
     * Update records
     *
     * @return integer
     */
    function saveRecord() {
        $d = xFormat::getNow('Y-m-d');
        $record = array();
        $record['IDNAME'] = UserRequest::$post['EBSK_idname'];
        $record['VERSION'] = UserRequest::$post['EBSK_version'];
        $record['MDATE'] = $d;
        $record['USERNAME'] = $_SESSION['user_record']['username'];
        $record['DESCRIPTION'] = UserRequest::$post['EBSK_description'];
        $record['MODULECODE'] = UserRequest::$post['EBSK_modulecode'];
        $record['HEADER_SIZE'] = UserRequest::$post['EBSK_header_size'];
        $record['RECORD_LENGTH'] = UserRequest::$post['EBSK_record_length'];
        $record['SKCONNECTOR_CODE'] = UserRequest::$post['EBSK_skconnector_code'];
        $record['HEADERMSG_CODE'] = UserRequest::$post['EBSK_headermsg_code'];
        $record['IGNORE_LINES'] = (empty(UserRequest::$post['EBSK_ignore_lines'])) ? CLX_NUM_ZERO : UserRequest::$post['EBSK_ignore_lines'];
        $record['TYPE'] = UserRequest::$post['EBSK_type'];
        if (!empty(UserRequest::$post['EBSK_record_delim'])) {
		  if ($record['TYPE'] === 'CSV') {
			 if (UserRequest::$post['EBSK_field_delim'] === 'other') {
			     $record['FIELD_DELIM'] = UserRequest::$post['EBSK_field_delim_other'];
			 } else {
			     $record['FIELD_DELIM'] = UserRequest::$post['EBSK_field_delim'];
			 }
		  }
		  if (UserRequest::$post['EBSK_record_delim'] === 'other') {
			 $record['RECORD_DELIM'] = UserRequest::$post['EBSK_record_delim_other'];
		  } else {
			 $record['RECORD_DELIM'] = UserRequest::$post['EBSK_record_delim'];
		  }
        }
        ClxCommon::$db->updateRecord($this->tablename, $record, 'ID=' . UserRequest::$post['EBSK_id']);
        $this->versionIncrease('ID', UserRequest::$post['EBSK_id']);
    }

    /**
     * Creates record
     *
     * @return integer
     */
    function createRecord_() {
        $d = xFormat::getNow('Y-m-d');
        $record = array();
        $record['IDCODE'] = UserRequest::$post['EBSK_idcode'];
        $record['IDNAME'] = UserRequest::$post['EBSK_idname'];
        $record['VERSION'] = UserRequest::$post['EBSK_version'];
        $record['VERSION_COUNTER'] = '0';
        $record['CDATE'] = $d;
        $record['MDATE'] = $d;
        $record['USERNAME'] = $_SESSION['user_record']['username'];
        $record['DESCRIPTION'] = UserRequest::$post['EBSK_description'];
        $record['MODULECODE'] = UserRequest::$post['EBSK_modulecode'];
        $record['HEADER_SIZE'] = UserRequest::$post['EBSK_header_size'];
        $record['SKCONNECTOR_CODE'] = UserRequest::$post['EBSK_skconnector_code'];
        $record['HEADERMSG_CODE'] = UserRequest::$post['EBSK_headermsg_code'];
        $record['RECORD_LENGTH'] = UserRequest::$post['EBSK_record_length'] + $record['HEADER_SIZE'];
        $record['IGNORE_LINES'] = (empty(UserRequest::$post['EBSK_ignore_lines'])) ? CLX_NUM_ZERO : UserRequest::$post['EBSK_ignore_lines'];
        $record['TYPE'] = UserRequest::$post['EBSK_type'];
        if (!empty(UserRequest::$post['EBSK_record_delim'])) {
		  if (UserRequest::$post['EBSK_record_delim'] === 'other') {
			 $record['RECORD_DELIM'] = UserRequest::$post['EBSK_record_delim_other'];
		  } else {
			 $record['RECORD_DELIM'] = UserRequest::$post['EBSK_record_delim'];
		  }
        }
        if ($record['TYPE'] === 'CSV') {
		  if (UserRequest::$post['EBSK_field_delim'] === 'other') {
			 $record['FIELD_DELIM'] = UserRequest::$post['EBSK_field_delim_other'];
		  } else {
			 $record['FIELD_DELIM'] = UserRequest::$post['EBSK_field_delim'];
		  }
        } else {
		  $record['FIELD_DELIM'] = '';
        }
        $recordid = ClxCommon::$db->createRecord($this->tablename, $record, false);
        $DBMO = &getTool('DBMO');
        $DBMO->versionIncrease('IDCODE', $record['MODULECODE']);
        //Creating Auto first Map for Header
        if (UserRequest::$request['ac'] <> 'DUPLICATE' && $record['TYPE'] === 'FRL' && $record['RECORD_LENGTH'] > CLX_NUM_ZERO) {
		  $rec = array();
		  $rec['FILE_FIELDNAME'] = 'HEADER';
		  $rec['type'] = 'CONS';
		  $rec['DATA_TYPE'] = 'CHAR';
		  $rec['VAL_CONSTANT'] = addslashes(singleQuote(UserRequest::$post['EBSK_idcode']));
		  $rec['START_POSITION'] = CLX_INIT_ZERO;
		  $rec['FIELD_LENGTH'] = $record['RECORD_LENGTH'];
		  $rec['EBSOCKETCODE'] = UserRequest::$post['EBSK_idcode'];
		  $rec['SEQNUM'] = CLX_INIT_ZERO;
		  $rec['description'] = 'General Header must be first 10 positions of every message.';
		  ClxCommon::$db->createRecord('CLX_EB_SOCKET_MAPPINGS', $rec, false);
        }
        return $recordid;
    }

    /**
     * Duplicate Records
     */
    function duplicateRecord() {
        $msg_id = $this->createRecord_();
        $old_record = ClxCommon::$db->getRecord($this->tablename, 'ID=' . UserRequest::$post['EBSK_id']);
        $rs = ClxCommon::$db->getRecords('CLX_EB_SOCKET_MAPPINGS', 'EBSOCKETCODE=' . singleQuote($old_record['IDCODE']) . ' ORDER BY SEQNUM ASC');
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
		  $old_map_id = $r['ID'];
		  unset($r['ID']);
		  $r['EBSOCKETCODE'] = UserRequest::$post['EBSK_idcode'];
		  $new_map_id = ClxCommon::$db->createRecord('CLX_EB_SOCKET_MAPPINGS', $r, false);
		  $rs2 = ClxCommon::$db->getRecords('CLX_EB_SOCKET_TRANSLATIONS', 'MAPID=' . singleQuote($old_map_id));
		  while ($r2 = ClxCommon::$db->getNextRecord($rs2, ASSOCIATIVE)) {
			 unset($r2['ID']);
			 $r2['MAPID'] = $new_map_id;
			 ClxCommon::$db->createRecord('CLX_EB_SOCKET_TRANSLATIONS', $r2, false);
		  }
        }
        $rs = ClxCommon::$db->getRecords('CLX_EB_SOCKET_FIELDS', 'EBSOCKETCODE=' . singleQuote($old_record['IDCODE']) . ' ORDER BY SEQNUM ASC');
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
		  $old_map_id = $r['ID'];
		  unset($r['ID']);
		  $r['EBSOCKETCODE'] = UserRequest::$post['EBSK_idcode'];
		  ClxCommon::$db->createRecord('CLX_EB_SOCKET_FIELDS', $r, false);
        }
        return $msg_id;
    }

    /**
     * Deletes record on selected id
     *
     * @param integer $id
     */
    function deleteRecordSk($id) {
        $record = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $id);
        $idcode = $record['IDCODE'];
        $maps = ClxCommon::$db->getRecords('CLX_EB_SOCKET_MAPPINGS', 'EBSOCKETCODE=' . singleQuote($idcode));
        while (($map = ClxCommon::$db->getNextRecord($maps))) {
		  ClxCommon::$db->deleteRecords('CLX_EB_SOCKET_TRANSLATIONS', 'MAPID=' . $map['ID']);
        }
        ClxCommon::$db->deleteRecords('CLX_EB_SOCKET_FIELDS', 'EBSOCKETCODE=' . singleQuote($idcode));
        ClxCommon::$db->deleteRecords('CLX_EB_SOCKET_EXECUTIONS', 'EBSOCKETCODE=' . singleQuote($idcode));
        ClxCommon::$db->deleteRecords('CLX_EB_SOCKET_MAPPINGS', 'EBSOCKETCODE=' . singleQuote($idcode));
        ClxCommon::$db->deleteRecords('CLX_EB_SOCKET_ERRORS', 'EBSOCKETCODE=' . singleQuote($idcode));
        ClxCommon::$db->deleteRecords($this->tablename, 'ID=' . $id);
        $DBMO = &getTool('DBMO');
        $DBMO->versionIncrease('IDCODE', $record['MODULECODE']);
    }

    /**
     * Defines default record on initializacion
     *
     * @return array
     */
    function defaultRecord() {
        $record = array();
        $record['EBSK_id'] = CLX_INIT_ZERO;
        $record['EBSK_idcode'] = '';
        $record['EBSK_idname'] = '';
        $record['EBSK_version'] = '1.0.0';
        $record['EBSK_cdate'] = '';
        $record['EBSK_mdate'] = '';
        $record['EBSK_username'] = $_SESSION['user_record']['username'];
        $record['EBSK_description'] = '';
        $record['EBSK_modulecode'] = '';
        $record['EBSK_header_size'] = CLX_NUM_TEN;
        $record['EBSK_record_length'] = CLX_INIT_ZERO;
        $record['EBSK_skconnector_code'] = '';
        $record['EBSK_headermsg_code'] = '';
        $record['EBSK_type'] = 'FRL';
        $record['EBSK_field_delim'] = '[9]';
        $record['EBSK_record_delim'] = '[13][10]';
        $record['EBSK_ignore_lines'] = CLX_INIT_ZERO;
        return $record;
    }

    /**
     * Increase tool version
     * @param string $fieldname
     * @param string $fieldvalue
     */
    function versionIncrease($fieldname, $fieldvalue) {
        parent::versionIncrease($fieldname, $fieldvalue);
        $modulecode = ClxCommon::$db->getField($this->tablename, 'MODULECODE', $fieldname . '=' . singleQuote($fieldvalue));
        $DBMO = &getTool('DBMO');
        $DBMO->versionIncrease('IDCODE', $modulecode);
    }

    /*
     * Save errors in socket erorrs table
     *
     * @param string $ebsocketcode, int $linenum, string $msg, int $saved_errors
     * @return void
     */

    function saveErrors($ebsocketcode, $linenum, $msg, &$saved_errors) {
        $record = array();
        $record['EBSOCKETCODE'] = $ebsocketcode;
        $record['LINENUM'] = $linenum;
        $record['ERRORNUM'] = '0';
        foreach ($msg as $fieldname => $error) {
		  $record['FIELDNAME'] = $fieldname;
		  $record['ERRORMSG'] = addslashes($error);
		  ClxCommon::$db->createRecord('CLX_EB_SOCKET_ERRORS', $record, false);
		  $saved_errors++;
        }
    }

    /**
     * Sets mapping property.
     * @param string $ebsocketcode
     */
    function setMappings($ebsocketcode) {
        if (array_key_exists($ebsocketcode, $this->mappings))
		  return;
        $this->copy_fields[$ebsocketcode] = array();
        $map = array();
        $ro = array();
        $sk = array();
        $rs = ClxCommon::$db->getRecords('CLX_EB_SOCKET_MAPPINGS', 'EBSOCKETCODE = ' . singleQuote($ebsocketcode) . ' ORDER BY SEQNUM');
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
		  $map[$r['ID']] = $r;
		  if (intval($r['READ_ONLY']) > CLX_NUM_ZERO)
			 $ro[$r['ID']] = $r['READ_ONLY'];
		  if (intval($r['SEARCH_KEY']) === CLX_NUM_ONE)
			 $sk[$r['ID']] = CLX_NUM_ONE;
		  //Search Table Information
		  if (mb_strlen($r['TABLEFULLNAME'], kDefaultEncoding) > CLX_NUM_ZERO && !array_key_exists($r['TABLEFULLNAME'], $this->table_info)) {
			 $this->table_info[$r['TABLEFULLNAME']] = ClxCommon::$db->getRecord('CLX_DB_TABLES', 'CONCAT(MODULECODE,\'_\',IDNAME) = ' . singleQuote($r['TABLEFULLNAME']));
		  }
		  if (mb_strlen($r['FORMULA'], kDefaultEncoding) > CLX_NUM_ZERO) {
			 $this->formulas[$ebsocketcode][$r['ID']] = CLX_NUM_ONE;
		  }
		  if (mb_strlen($r['COPY_FILEFIELD'], kDefaultEncoding) > CLX_NUM_ZERO) {
			 $this->copy_fields[$ebsocketcode][$r['ID']] = CLX_NUM_ONE;
		  }
        }
        $this->mappings[$ebsocketcode] = $map;
        $this->search_keys[$ebsocketcode] = $sk;
        $this->read_onlys[$ebsocketcode] = $ro;
        foreach ($this->copy_fields[$ebsocketcode] as $key => $val) {
		  foreach ($this->mappings[$ebsocketcode] as $key2 => $val2) {
			 if ($val2['FILE_FIELDNAME'] == $this->mappings[$ebsocketcode][$key]['COPY_FILEFIELD']) {
			     $this->copy_fields[$ebsocketcode][$key] = $val2['SEQNUM'];
			     break;
			 }
		  }
        }
    }

    /**
     * Get socket definitions (mappings)
     * @param string $ebsocketcode
     * @return array
     */
    function getMappings($ebsocketcode) {
        $this->setMappings($ebsocketcode);
        return $this->mappings[$ebsocketcode];
    }

    /**
     * Returns search key fields
     * @param string $ebsocketcode
     * @return array
     */
    function getSearchKeys($ebsocketcode) {
        return $this->search_keys[$ebsocketcode];
    }

    /**
     * Returns read only fields
     * @param string $ebsocketcode
     * @return array
     */
    function getReadOnlys($ebsocketcode) {
        return $this->read_onlyes[$ebsocketcode];
    }

    /**
     * Returns socket exceptions
     * @param string $ebsocketcode
     * @return array
     */
    function getExceptions($ebsocketcode) {
        $exceptions = array();
        $records = ClxCommon::$db->getRecords('CLX_EB_SOCKET_EXCEPTIONS', 'EBSOCKETCODE = ' . singleQuote($ebsocketcode) . '  ORDER BY SEQNUM');
        while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
		  $exceptions[$record['TABLE_FIELDNAME']] = $record;
        }
        return $exceptions;
    }

    /**
     * Get translations for each mapping
     *
     * @param string $ebsocketcode, array $mappings
     * @return array
     */
    function getTranslations($ebsocketcode, $direction = CLX_NUM_ZERO) {
//pre('getTranslations('. singleQuote($ebsocketcode', '. singleQuote($direction')');
        $translations = array();
        $sql = 'SELECT
			Z.MAPID,
			Z.ORIGINAL_VALUE,
			Z.REPLACE_VALUE,
			Z.CASE_SENSITIVE,
			-1 DIRECTION,
			Z.SEQNUM
		  FROM
		    CLX_EB_SOCKET_TRANSLATIONS Z,
		    CLX_EB_SOCKET_MAPPINGS M
		  WHERE
			M.ID = Z.MAPID
			AND M.EBSOCKETCODE = ' . singleQuote($ebsocketcode) . '
			AND (Z.TRANSL_CODE IS NULL OR Z.TRANSL_CODE = ' . singleQuote(CLX_CHAR_EMPTY_STRING) . ')
		  UNION ALL
		  SELECT
			X.MAPID,
			Z.ORIGINAL_VALUE,
			Z.REPLACE_VALUE,
			Z.CASE_SENSITIVE,
			Z.DIRECTION,
			Z.SEQNUM
		  FROM
		    CLX_EB_SOCKET_MAPPINGS M,
		    CLX_EB_SOCKET_TRANSLATIONS X,
		    CLX_SYS_TRANSL_SETS Y,
		    CLX_SYS_TRANSL_ITEMS Z
		  WHERE
		    M.ID = X.MAPID
		    AND M.EBSOCKETCODE = ' . singleQuote($ebsocketcode) . '
		    AND Y.IDCODE = Z.TRANSL_CODE
		    AND X.TRANSL_CODE = Y.IDCODE
		    ';
        if ($direction <> -CLX_NUM_ONE)
		  $sql .= ' AND Z.DIRECTION = ' . $direction;
        $records = ClxCommon::$db->executeQuery($sql);
//pre($sql);
        while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
		  if ($record['DIRECTION'] == -CLX_NUM_ONE) {
			 // Because CLX_EB_SOCKET_TRANSLATIONS doesn´t have DIRECTION column, must register both directions in array...
			 $translations[$record['MAPID']][$record['ORIGINAL_VALUE'] . '1'] = $record;
			 $translations[$record['MAPID']][$record['ORIGINAL_VALUE'] . '0'] = $record;
		  } else {
			 $translations[$record['MAPID']][$record['ORIGINAL_VALUE'] . $record['DIRECTION']] = $record;
		  }
        }
//e cho debug_dump('translations: ebsocketcode = $ebsocketcode, direction = $direction', $translations);
        return $translations;
    }

    /**
     * Do translation for a given field
     *
     * @param string $field_value, array $translations
     * @return void
     */
    function translate(&$field_value, &$translations, $direction = CLX_NUM_ZERO) {
//e cho debug_dump('translate(field_value = $field_value) === ', $translations);
        $t_field_value = trim($field_value) . $direction;
        if (array_key_exists($t_field_value, $translations)) {
		  if (!empty($translations[$t_field_value]['case_sensitive'])) {
			 if (strcmp(trim($field_value), $translations[$t_field_value]['ORIGINAL_VALUE']) == CLX_NUM_ZERO) {
			     $field_value = $translations[$t_field_value]['REPLACE_VALUE'];
			 }
		  } else {
			 $field_value = $translations[$t_field_value]['REPLACE_VALUE'];
		  }
        }
    }

    /**
     * Return true if it's an exception
     *
     * @param array $exceptions, string $buffer
     * @return bool
     */
    function isException($exceptions, $buffer) {
        $is = false;
        foreach ($exceptions as $exception) {
		  switch ($exception['OPERATOR']) {
			 case 'EQ':
			     $is = ($is or ( substr($buffer, $exception['POSITION'], mb_strlen($exception['value'], kDefaultEncoding)) == $exception['value']));
			     break;
			 case 'NQ':
			     $is = ($is or ( substr($buffer, $exception['POSITION'], mb_strlen($exception['value'], kDefaultEncoding)) != $exception['value']));
			     break;
			 case 'LS':
			     $is = ($is or ( substr($buffer, $exception['POSITION'], mb_strlen($exception['value'], kDefaultEncoding)) < $exception['value']));
			     break;
			 case 'LQ':
			     $is = ($is or ( substr($buffer, $exception['POSITION'], mb_strlen($exception['value'], kDefaultEncoding)) <= $exception['value']));
			     break;
			 case 'GQ':
			     $is = ($is or ( substr($buffer, $exception['POSITION'], mb_strlen($exception['value'], kDefaultEncoding)) >= $exception['value']));
			     break;
			 case 'GT':
			     $is = ($is or ( substr($buffer, $exception['POSITION'], mb_strlen($exception['value'], kDefaultEncoding)) > $exception['value']));
			     break;
			 default:
			     break;
		  }
        }
        return $is;
    }

    /**
     * Get total lenght of a mapping record
     *
     * @param string $ebsocketcode
     * @return void
     */
    function getLength($ebsocketcode) {
        $sql = 'SELECT SUM(FIELD_LENGTH) LENGTH FROM CLX_EB_SOCKET_MAPPINGS WHERE EBSOCKETCODE = ' . singleQuote($ebsocketcode);
        return ClxCommon::$db->getOne($sql);
    }

    /**
     * Save total lenght of a mapping record
     *
     * @param string $ebsocketcode
     * @return void
     */
    function saveLength($ebsocketcode) {
        $a = array();
        $a['RECORD_LENGTH'] = $this->getLength($ebsocketcode);
        ClxCommon::$db->updateRecord($this->tablename, $a, 'IDCODE = ' . singleQuote($ebsocketcode));
    }

    /**
     * Returns field label to be displayed on validation
     * @param array $mapinfo
     */
    function getMapFieldLabel(&$mapinfo) {
        if (mb_strlen($mapinfo['TABLE_FIELDNAME'], kDefaultEncoding) > CLX_NUM_ZERO) {
		  return dict(ClxCommon::$db->getField('CLX_DB_FIELDS', 'FIELDFORMLABEL', 'FIELDNAME = ' . singleQuote($mapinfo['TABLE_FIELDNAME'])));
        } else {
		  return $mapinfo['FILE_FIELDNAME'];
        }
    }

    /**
     * Maping
     * @param &$mapinfo
     * @param &$value
     */
    function validateMapping(&$mapinfo, &$value) {
        $contenidoHTML = '';
        switch ($mapinfo['DATA_TYPE']) {
		  case 'DECN':
			 if ($mapinfo['FIELD_LENGTH'] <> CLX_NUM_ZERO && mb_strlen($value, kDefaultEncoding) > $mapinfo['FIELD_LENGTH']) {
			     //TOOL_EBSK_ERR_DATATOLONG / Field ^1 smaller than data length. value: ^2, Required Size: ^3 String size: ^4';/ Campo ^1 mas pequeño que el tamaño del campo. Valor: ^2, Tamaño Requirido: ^3 Tamaño de Valor: ^4';
			     $this->setMassageError($mapinfo['FILE_FIELDNAME'], dict('TOOL_EBSK_ERR_DATATOLONG', $this->getMapFieldLabel($mapinfo), $value, $mapinfo['FIELD_LENGTH'], mb_strlen($value, kDefaultEncoding)));
			     if ($this->debug === true)
			         $contenidoHTML .= $this->errors[$mapinfo['FILE_FIELDNAME']];
			 } else {
			     //Validate number of decimals on string
			     if (!empty($mapinfo['FIELD_DECIMALS']) && mb_strlen($value, kDefaultEncoding) > CLX_NUM_ZERO && !$this->remove_decimal_point) {
			         if (strpos($value, '.') === false) {
					   //TOOL_EBSK_ERR_NODECIMAL Decimal value required on ^1/ Valor decimal requerido on ^1
					   $this->setMassageError($mapinfo['FILE_FIELDNAME'], dict('TOOL_EBSK_ERR_NODECIMAL', $this->getMapFieldLabel($mapinfo)));
			         } elseif ($mapinfo['FIELD_DECIMALS'] < mb_strlen(substr($value, strpos($value, '.') + CLX_NUM_ONE, mb_strlen($value) - strpos($value, '.')), kDefaultEncoding)) {
					   //TOOL_EBSK_ERR_DECIMALTOLONG / Decimal positions grater than maximun allowed for field for message./ Número de decimales es mayor que el requerido por el campo en la trama.
					   $this->setMassageError($mapinfo['FILE_FIELDNAME'], dict('TOOL_EBSK_ERR_DECIMALTOLONG', $this->getMapFieldLabel($mapinfo)));
			         }
			         if ($this->debug === true)
					   $contenidoHTML .= $this->errors[$mapinfo['FILE_FIELDNAME']];
			     }
			 }
			 break;
		  default:
			 if ($mapinfo['FIELD_LENGTH'] <> CLX_NUM_ZERO && mb_strlen($value, kDefaultEncoding) > $mapinfo['FIELD_LENGTH']) {
			     //TOOL_EBSK_ERR_DATATOLONG / Field ^1 smaller than data length. value: ^2, Required Size: ^3 String size: ^4'; /Campo ^1 mas pequeño que el tamaño del campo. Valor: ^2, Tamaño Requirido: ^3 Tamaño de Valor: ^4';
			     $this->setMassageError($mapinfo['FILE_FIELDNAME'], dict('TOOL_EBSK_ERR_DATATOLONG', $this->getMapFieldLabel($mapinfo), $value, $mapinfo['FIELD_LENGTH'], mb_strlen($value, kDefaultEncoding)));
			     if ($this->debug === true)
			         $contenidoHTML .= $this->errors[$mapinfo['FILE_FIELDNAME']];
			 }
			 break;
        }//end switch
        return $contenidoHTML;
    }

    /**
     * Load Message
     * Fill message record with all values from record.
     *
     * @param string $message_code
     * @param array $record
     * @return array
     */
    function createMessageVoid($message_code, $record, $action = 'NONE') {
        ClxFreeReturn($this->createMessage($message_code, $record, $action));
    }

    function createMessage($message_code, $record, $action = 'NONE') {
        $this->msgrecord = array();
        $this->translations = array();
        $this->msgcode = $message_code;
        $this->compileCodeFormulas($message_code);
        $msgrec = ClxCommon::$db->getRecord($this->tablename, 'IDCODE =' . singleQuote($message_code));
        $this->msg_type = $msgrec['TYPE'];
        $this->record_delim = '';
        $this->field_delim = '';
        if($this->msg_type === 'CSV') {
		  $this->record_delim = replace_ascii_chars($msgrec['RECORD_DELIM']);
		  $this->field_delim = replace_ascii_chars($msgrec['FIELD_DELIM']);
        }
        $this->skco_code = $msgrec['SKCONNECTOR_CODE']; //Set socket connector information
        $this->skco_code = (defined('EBSK_IBS_DEF_CONNECTOR') && EBSK_IBS_DEF_CONNECTOR <> CLX_NUM_ZERO) ? EBSK_IBS_DEF_CONNECTOR : $msgrec['SKCONNECTOR_CODE'];
        $SKCO = &getTool('SKCO');
        $connector = $SKCO->getConnector($this->skco_code);
        // Check headers generarion
        if($this->generate_column_headers === true) {
		  $a = new eb_sockets();
		  $a->createMessage($message_code, array(), 'HEADERS');
		  $this->column_headers = $a->msgrecord;
		  unset($a);
		  $this->generate_column_headers = false;
        }
        //Check if exists prior message
        if ($msgrec['HEADERMSG_CODE'] <> '') {
		  $a = new eb_sockets();
		  $a->createMessage($msgrec['HEADERMSG_CODE'], $record, $action);
		  $this->priormsgcode = $a->msgcode;
		  $this->priormsgrecord = $a->msgrecord;
		  $this->priormsglength = $a->msglength;
		  unset($a);
        } else {
		  $this->priormsglength = CLX_INIT_ZERO;
		  $this->priormsgcode = '';
		  $this->priormsgrecord = array();
        }
        $this->msglength = $msgrec['RECORD_LENGTH'];
        $this->setMappings($message_code);
        if (!array_key_exists($message_code, $this->translations)) {
		  $this->translations[$message_code] = $this->getTranslations($message_code, CLX_NUM_MINUS_1);
        }
        foreach ($this->mappings[$message_code] as $key => $field) {
		  if($action === 'HEADERS') {
			 $this->msgrecord[$key] = array('FILEFIELD' => $field['FILE_FIELDNAME'], 'FIELD' => $field['TABLE_FIELDNAME'], 'TYPE' => $field['DATA_TYPE'], 'SIZE' => $field['FIELD_LENGTH'], 'START' => $field['START_POSITION'], 'DECIMALS' => $field['FIELD_DECIMALS'], 'FIELD_DTFORMAT' => $field['FIELD_DTFORMAT'], 'VAL' => $field['FILE_FIELDNAME']);
			 continue;
		  }
		  if ($field['FIELD_LENGTH'] == CLX_NUM_ZERO && $this->msg_type === 'FRL')
			 continue; //just spacer
		  $val = '';
		  if (array_key_exists($field['FILE_FIELDNAME'], $record)) {
			 $val = $record[$field['FILE_FIELDNAME']];
		  }
		  if (trim($field['TABLE_FIELDNAME']) !== '' && array_key_exists($field['TABLE_FIELDNAME'], $record)) {
			 //execute formulas if exists

			 $val = '';
			 $this->includeFormulaConstants($field, $record, $val);
			 $val = $record[$field['TABLE_FIELDNAME']];
		  }

		  //Check is formula
		  if ($field['TYPE'] === 'CONS') {
			 $this->includeFormulaConstants($field, $record, $val);
		  }
//if($val<>'') echo 'Valor 2:'.$val.'<br>';
		  if (intval($field['OPERATION_CODE']) === CLX_NUM_ONE) {
			 switch ($action) {
			     case 'CREATE_REMOTE':
			         $val = '0001';
			         break;
			     case 'SELECT_FOR_UPDATE_REMOTE':
			         //see must query first and then send
			         $val = '0002';
			         break;
			     case 'UPDATE_REMOTE':
			         //after select for update
			         $val = '0005';
			         break;
			     default:
			         if (empty($val))
					   $val = '0004';
			         break;
			 }
		  }
		  //Handle Transalations
		  if (array_key_exists($key, $this->translations[$message_code])) {
			 $this->translate($val, $this->translations[$message_code][$key], CLX_NUM_ONE);
		  }

		  //Setting ibs user if file has type user
		  if ($field['IS_IBS_USER'] === '1') {
			 if (!empty($connector['OTHER_USERNAME'])) {
			     $val = $connector['OTHER_USERNAME'];
			 } elseif ($val === '') {
			     $val = $_SESSION['user_record']['username'];
			 }
		  }
		  //completing spaces or ceros depending of type
		  switch ($field['DATA_TYPE']) {
			 case 'DECN':
			     if ($field['FIELD_DECIMALS'] > CLX_NUM_ZERO) {
			         $val = rounder($val, $field['FIELD_DECIMALS']);
			     }
			     break;
			 case 'DATE':
			     $val = fromIsoDate($val, $field['FIELD_DTFORMAT']);
			     break;
			 case 'DATI':
			     $val = fromIsoDate($val, $field['FIELD_DTFORMAT']);
			     break;
			 default: //characters
			     //do nothing
			     break;
		  }//end switch
		  //Decimals must be an integer and should go without comma or dots or simbols
		  //Check with fernando about - sign
		  if ($field['DATA_TYPE'] === 'DECN' && $this->remove_decimal_point) {
			 $val = str_replace(array('.', '-', ','), '', rounder($val, $field['FIELD_DECIMALS']));
		  }
		  //Completing dimensions
		  if ($field['FIELD_LENGTH'] <> CLX_NUM_ZERO && $this->msg_type === 'FRL') {
			 $this->addSpaceHolders($field['DATA_TYPE'], $field['FIELD_LENGTH'], $val);
		  }
		  //perform validations
		  ClxFreeReturn($this->validateMapping($field, $val));
		  //Store mapping
		  $this->msgrecord[$key] = array('FILEFIELD' => $field['FILE_FIELDNAME'], 'FIELD' => $field['TABLE_FIELDNAME'], 'TYPE' => $field['DATA_TYPE'], 'SIZE' => $field['FIELD_LENGTH'], 'START' => $field['START_POSITION'], 'DECIMALS' => $field['FIELD_DECIMALS'], 'FIELD_DTFORMAT' => $field['FIELD_DTFORMAT'], 'VAL' => $val);
        }

        return $this->msgrecord;
    }

    /**
     * Parses the input into message definition.
     * Fill message record with all values from flat file line.
     *
     * @param string $message_code
     * @param string $s_ffile_line
     * @param boolean $sw_ignore_headermsg_code
     * @return array
     */
    function readMessage($message_code, $s_ffile_line, $sw_ignore_headermsg_code = false) {
        $contenidoHTML = '';
        $i_record_rd_pointer = CLX_INIT_ZERO;
        $msgrecord = array();
        $copyfields = array(); //used to temporary store values for copy fields
        //  Processing optimization...         //$this->translations = array();
        if (!is_array($this->translations))
		  $this->translations = array();
        //Priparing Header Message, basically message recieve prior to actual message
        if (!array_key_exists($message_code, $this->message_record)) {
		  $this->message_record[$message_code] = ClxCommon::$db->getRecord($this->tablename, 'IDCODE =' . singleQuote($message_code));
        }
        //Set socket connector information
        $this->skco_code = $this->message_record[$message_code]['SKCONNECTOR_CODE'];
        //Check if exists prior message
        if ($sw_ignore_headermsg_code == false) {
		  if ($this->message_record[$message_code]['HEADERMSG_CODE'] <> '') {
			 $msgrecord[CLX_NUM_ZERO] = $this->readMessage($this->message_record[$message_code]['HEADERMSG_CODE'], $s_ffile_line);
			 $i_record_rd_pointer = $this->priormsglength + CLX_NUM_TWO; //moving the pointer to exclude prior message
		  }
        }
        $this->setMappings($message_code);
        if (!array_key_exists($message_code, $this->translations)) {
		  $this->translations[$message_code] = $this->getTranslations($message_code, CLX_NUM_MINUS_1);
        }
        if ($this->msg_type === 'CSV') {
		  $s_ffile_line = explode($this->field_delim, $s_ffile_line);
        }
        foreach ($this->mappings[$message_code] as $key => $field) {
		  // Read field from pointer up to the specified lenght
		  if ($this->msg_type === 'CSV') {
			 if (array_key_exists($field['SEQNUM'], $s_ffile_line)) {
			     $val = $s_ffile_line[$field['SEQNUM']];
			 } elseif ($field['TYPE'] === 'COPY') {
			     $val = $s_ffile_line[$this->copy_fields[$message_code][$key]];
			 } else {
			     $val = '';
			 }
		  } else {
			 if ($field['DATA_TYPE'] === 'DECN') {
			     $val = substr($s_ffile_line, $i_record_rd_pointer, $field['FIELD_LENGTH']);
			 } elseif ($field['DATA_TYPE'] <> 'DATE' && $field['DATA_TYPE'] <> 'DATI') {
			     $val = trim(substr($s_ffile_line, $i_record_rd_pointer, $field['FIELD_LENGTH']));
			 } else {//dates must keep space separators
			     $val = substr($s_ffile_line, $i_record_rd_pointer, $field['FIELD_LENGTH']);
			 }
		  }
		  // Check if is it a formula
		  if ($field['TYPE'] === 'CONS') {
			 //$val = $this->executeConstant($field, CLX_NUM_ZERO, $val);
			 $this->includeFormulaConstants($field, $record, $val);
		  }
		  switch ($field['DATA_TYPE']) {
			 case 'DECN':
			     //Validate number of decimals on string
			     if (!empty($field['FIELD_DECIMALS']) > CLX_NUM_ZERO) {
			         //has decimal?
			         if (strpos($val, '.') === false) {
					   // No decimal point, so add to at the specified number of decimal positions
					   $i_integert_part_len = $field['FIELD_LENGTH'] - $field['FIELD_DECIMALS'];
					   $val = trim(substr($val, CLX_NUM_ZERO, $i_integert_part_len)) . '.' . substr($val, $i_integert_part_len, $field['FIELD_DECIMALS']);
			         } elseif ($field['FIELD_DECIMALS'] < mb_strlen(substr($val, strpos($val, '.') + CLX_NUM_ONE, mb_strlen($val, kDefaultEncoding) - strpos($val, '.')), kDefaultEncoding)) {
					   //TOOL_EBSK_ERR_DECIMALTOLONG Decimal positions grater than maximun allowed for field for message./ Número de decimales es mayor que el requerido por el campo en la trama.
					   $this->errors[$field['FILE_FIELDNAME']] = dict('TOOL_EBSK_ERR_DECIMALTOLONG');
					   if ($this->debug === true)
						  $contenidoHTML .= $this->errors[$field['FILE_FIELDNAME']];
			         }
			     }
					$val = trim($val);
			     break;
			 case 'DATE':
			     $val = toIsoDate($val, $field['FIELD_DTFORMAT']);
			     break;
			 default:
			     break;
		  }//end swich
		  //Handling Encoding
		  $val = encode_string($val, kDefaultEncoding);
		  //save COPY_FILEFIELD
		  $copyfields[$field['FILE_FIELDNAME']] = $val;
		  $copyfields[$field['TABLE_FIELDNAME']] = $val;
		  if ($field['COPY_FILEFIELD'] <> '' && array_key_exists($field['COPY_FILEFIELD'], $copyfields)) {
			 $val = $copyfields[$field['COPY_FILEFIELD']];
		  }
		  //Handle Transalations
		  if (array_key_exists($key, $this->translations[$message_code])) {
			 $this->translate($val, $this->translations[$message_code][$key], CLX_NUM_ZERO);
		  }
		  $msgrecord[] = array('FILEFIELD' => $field['FILE_FIELDNAME'],
			 'TABLE' => $field['TABLEFULLNAME'],
			 'FIELD' => $field['TABLE_FIELDNAME'],
			 'TYPE' => $field['DATA_TYPE'],
			 'SIZE' => $field['FIELD_LENGTH'],
			 'DECIMALS' => $field['FIELD_DECIMALS'],
			 'START' => $field['START_POSITION'],
			 'POINTER' => $i_record_rd_pointer,
			 'VAL' => $val);
		  // Pointer increment
		  $i_record_rd_pointer += $field['FIELD_LENGTH'];
        }
        if ($this->debug === true) {
		  $contenidoHTML .= $this->debug_table_from_ar($msgrecord);
        }
        echo $contenidoHTML;
        return $msgrecord;
    }

    /**
     * Display array in html table. If column count different from one row to another, creates a new table
     * @param array $ar
     */
    function debug_table_from_ar($ar) {
        $contenidoHTML = '';
        $last_count = null;
        $sw_first = false;
        foreach ($ar as $rowkey => $col) {
		  if (count($col) != $last_count) {
			 $sw_first = true;
			 if (!is_null($last_count))
			     $contenidoHTML .= '</table><br>';
			 $contenidoHTML .= '<table border=1>';
			 $last_count = count($col);
		  }
		  $contenidoHTML .= '<tr>';
		  if ($sw_first === true) {
			 $contenidoHTML .= '<td>###</td>';
			 foreach ($col as $key => $val) {
			     $contenidoHTML .= '<td>' . $key . '</td>';
			 }
			 $contenidoHTML .='</tr><tr>';
			 $sw_first = false;
		  }
		  $contenidoHTML .='<td>' . $rowkey . '</td>';
		  foreach ($col as $key => $val) {
			 if (is_array($val)) {
			     $contenidoHTML .= '<td>' . implode(', ', $val) . '</td>';
			 } else {
			     $contenidoHTML .= '<td>' . (is_null($val) || $val === '' ? CLX_CHAR_HTML_NBSP : $val) . '</td>';
			 }
		  }
		  $contenidoHTML .='</tr>';
        }
        $contenidoHTML .='</table><br><br>';
        return $contenidoHTML;
    }

    /**
     * Sets the characters required for the string to be sent to ibs
     * numeric must be completed with ceros and string with spaces
     * @param strng $type
     * @param string $fieldlength
     * @param string $val
     */
    function addSpaceHolders($type, $fieldlength, &$val) {
        if ($type === 'CHAR' || $type === 'DATE' || $type === 'DATI') {
		  $val = stringAddSpacer($val, $fieldlength, false, ' ', false);
        } else {
		  $val = stringAddSpacer($val, $fieldlength, true, CLX_NUM_ZERO, false);
        }
    }

    /**
     * Converts Array Message into string
     *
     * @return string
     */
    function toStringPrior() {
        /* 2018-02-23 Caso 1003630 - PBL-Interfaz contable: Genera la primera linea con los nombres de las columnas */
        $return = (string) '';
        if(count($this->column_headers) > 0) {
		  $prev_record = $this->msgrecord;
		  $this->msgrecord = $this->column_headers;
		  $return = $this->toString_();
		  $return .= CLX_CHAR_CR . CLX_CHAR_NL;
		  $this->column_headers = array();
		  $this->msgrecord = $prev_record;
        }
        return $return . $this->toString_();
    }
    
    /**
     * Converts Array Message into string
     *
     * @return string
     */
    function toString_() {
        if ($this->debug === true) {
		  echoLog('<br>Preparing to Send...<br>');
		  echoLog($this->debug_table_from_ar($this->msgrecord));
        }
        $return = (string) '';
        foreach ($this->msgrecord as $key => $val) {
		  if($this->msg_type === 'FRL') {
			 $return .= strval($val['VAL']);
		  } else {
			 $return .= ($return === '' ? '' : $this->field_delim) . strval($val['VAL']);
		  }
        }
        if ($this->convert_skt_output_to_ascii === true)
		  $return = decode_string($return);
        return $return;
    }

    /**
     * bool Return true if NO error has being found on message
     *
     * @param void
     * @return void
     */
    function hasErrors() {
        return (empty($this->errors)) ? false : true;
    }

    /**
     * Sends Message to socket
     *
     * @param void
     * @return array
     */
    function send() {
        $contenidoHTML = '';
        if ($this->hasErrors()) {//dont do anything already on error
		  return false;
        }
        //Connector
        $SKCO = &getTool('SKCO');
        $skcorec = $SKCO->getConnector($this->skco_code);
        if (empty($skcorec)) {
		  //$result = false;
		  $this->setMassageError('SCKDEFNOTFND_ERR', dict('GEN_LBL_SOCK_INVALID', $this->skco_code));
		  return false;
        } else {

		  if (!empty($skcorec['SOCKET_TIMEOUT'])) {
			 $this->timeout = $skcorec['SOCKET_TIMEOUT'];
		  }
		  if ($this->debug === true) {
			 $contenidoHTML .= '<br>' . var_dump($skcorec['IP_ADDRESS']) . '<br>' . var_dump($skcorec['PORT_NUMBER']) . '<br>';
		  }
		  $this->sck = new clx_socket($skcorec['IP_ADDRESS'], $skcorec['PORT_NUMBER'], true, $this->debug);
		  if ($this->timeout != null) {
			 $this->sck->timeout = $this->timeout;
			 $this->timeout = null;
		  }
		  if ($this->sck->connect()) {
			 $in = $this->toString_();
			 //$in = substr($in, 0, 2248);
			 $this->sck->write($this->sck->stringByteLength($in));
			 if ($this->sck->write($in)) {
			     $out = $this->sck->read();
			     if (mb_strlen($out, kDefaultEncoding) > CLX_NUM_TWO) {
			         $tmpmstr = substr($out, CLX_NUM_ZERO, CLX_NUM_TWO);
			         if (empty($tmpmstr)) {
					   $tmpmstr = '';
			         }
			         $read_length = unpack('n', $tmpmstr);
			         if (array_key_exists(CLX_NUM_ONE, $read_length)) {
					   $read_length = $read_length[CLX_NUM_ONE];
					   $out = substr($out, CLX_NUM_TWO, mb_strlen($out, kDefaultEncoding));
			         } else {
					   $read_length = mb_strlen($out, kDefaultEncoding);
			         }
			     } else {
			         $read_length = mb_strlen($out, kDefaultEncoding);
			     }
			     $result = $this->readMessage($this->msgcode, $out);
			     if (mb_strlen($out, kDefaultEncoding) == CLX_NUM_ZERO) {
			         // Transmision error
			         $this->setMassageError($this->sck->a_last_error_msg['errorNum'], dict('GEN_LBL_SOCK_READ_FAIL', $this->msgcode) . ' ' . $this->sck->a_last_error_msg['errorMsg']);
			         echo $contenidoHTML;
			         return false;
			     }
			 } else {
			     //write error
			     $this->setMassageError('SCKWERR', dict('GEN_LBL_SOCK_WIRTE_FAIL', $this->msgcode));
			     echo $contenidoHTML;
			     return false;
			 }
			 $this->sck->close();
		  } else {
			 // Connection error
			 //$this->setMassageError('SCKERR',dict('GEN_LBL_SOCK_CNX_FAIL', $this->msgcode));
			 $errmsg = dict('GEN_LBL_SOCK_CNX_FAIL', $this->msgcode);
			 if ($this->sck->a_last_error_msg['errorMsg'] !== '') {
			     $errmsg .= '<br>' . CLX_CHAR_NL . encode_string($this->sck->a_last_error_msg['errorMsg']) . ' [' . $this->sck->a_last_error_msg['errorNum'] . ']';
			 }
			 $this->setMassageError('SCKERR', $errmsg);
			 echo $contenidoHTML;
			 return false;
		  }
        }
        echo $contenidoHTML;
        return $result;
    }

    /**
     * Set message error
     * @param $code
     * @param $msg
     */
    function setMassageError($code, $msg) {
        $this->errors[$code] = $msg;
    }

    /**
     * Builds ibs standard error array from a referential array
     *
     * @param array $a_err
     * @return array
     */
    function build_ibs_style_error_array($a_err) {
        reset($a_err);
        $a_response['HEADERMSG'] = array('ERRNUM' => count($a_err));
        $i = CLX_NUM_ZERO;
        foreach ($a_err as $name => $val) {
		  $i++;
		  $s_i = $i;
		  if ($i < CLX_NUM_TEN)
			 $s_i = '0' . $s_i;
		  $a_response['HEADERMSG']['ERNU' . $s_i] = $name;
		  $a_response['HEADERMSG']['ERDS' . $s_i] = $val;
        }
        return array($a_response);
    }

    /**
     * Returns a string with error messages from ibs standard error array
     *
     * @param array $ibsuser_data
     * @return string
     */
    function string_error_ibs_message($ibsuser_data) {
        $s_errmsg = '';
        //e cho debug_dump('ibsuser_data', $ibsuser_data);
        $msg = array();
        $errmsg = $ibsuser_data[CLX_NUM_ZERO];
        if (array_key_exists('HEADERMSG', $errmsg)) {
		  $msg = $errmsg['HEADERMSG']; //if message comes from send message format is diferent
        } else {
		  foreach ($errmsg as $key => $val) {
			 $msg[$val['FILEFIELD']] = $val['VAL'];
		  }
		  if (empty($msg['ERRNUM']))
			 $msg['ERRNUM'] = CLX_NUM_ZERO;
        }
        for ($i = CLX_NUM_ONE; $i <= CLX_NUM_TEN; $i++) {
		  $s_i = $i;
		  if ($i < CLX_NUM_TEN)
			 $s_i = '0' . $s_i;
		  if (array_key_exists('ERNU' . $s_i, $msg) && $msg['ERNU' . $s_i] !== '') {
			 $s_errmsg .= '<li>' . $msg['ERDS' . $s_i] . ' [' . $msg['ERNU' . $s_i] . ']</li>';
		  }
        }
        return $s_errmsg;
    }

    function askCheck() {

        $idlist = getIdlist();
        $action = (!empty(UserRequest::$server['PHP_SELF'])) ? UserRequest::$server['PHP_SELF'] : 'tool.php';
        $form = new htmlForm(CLX_PROP_ACTION, $action);
        $form->setProp(CLX_PROP_ID, 'theForm');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'CHECK');
        $form->addArgument('idlist', toString($idlist));
        $form->addArgument('confirm', CLX_NUM_ONE);
        $form->addArgument('confirm_check', CLX_NUM_ONE);
        $form->toolcode = $this->code;
        $msg_rec = ClxCommon::$db->getRecord($this->tablename, 'ID=' . toString($idlist));
        $msg_record = ClxCommon::$db->getRecord($this->tablename, 'ID = ' . singleQuote($idlist[CLX_NUM_ZERO]));

        //Load formula code
        $this->compileCodeFormulas($msg_rec['IDCODE']);

        //Search Connector Info
        $this->skco_code = (defined('EBSK_IBS_DEF_CONNECTOR') && EBSK_IBS_DEF_CONNECTOR <> CLX_NUM_ZERO) ? EBSK_IBS_DEF_CONNECTOR : $msg_record['SKCONNECTOR_CODE'];
        $SKCO = &getTool('SKCO');
        $connector = $SKCO->getConnector($this->skco_code);
        $title = 'GEN_LBL_CHECK_IBS_MSG';
        //Setting form
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict($title), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addCell(dict('GEN_LBL_SET_PARAMETERS'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addData(dict('GEN_LBL_SOCKET_MESSAGE'), $msg_rec['IDNAME'] . ' (' . $msg_rec['IDCODE'] . ')');
        $table->addData(dict('GEN_LBL_DESCRIPTION'), $msg_rec['DESCRIPTION']);
        $table->addData(dict('GEN_LBL_CONNECTOR'), $connector['IDNAME'] . '(' . $connector['IDCODE'] . ')');
        $table->addSeparator(CLX_CLASS_DATA);
        $table->addCell(dict('GEN_LBL_PARAMETERS'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        //Setting required fields
        $rs = ClxCommon::$db->getRecords('CLX_EB_SOCKET_MAPPINGS', 'EBSOCKETCODE = ' . singleQuote($msg_rec['IDCODE']) . ' AND FIELD_LENGTH>0 ORDER BY SEQNUM ASC');
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
		  $form_fieldname = (mb_strlen($r['TABLE_FIELDNAME'], kDefaultEncoding) > CLX_NUM_ZERO) ? $r['TABLE_FIELDNAME'] : $r['FILE_FIELDNAME'];
		  $record[$form_fieldname] = '';
		  $default = '';
		  $this->includeFormulaConstants($r, $record, $default);
		  if (strval($default) === '' && $r['IS_IBS_USER'] === '1') {
			 $default = (!empty($connector['OTHER_USERNAME'])) ? $connector['OTHER_USERNAME'] : $_SESSION['user_record']['username'];
		  }
		  if (mb_strlen($r['TABLE_FIELDNAME'], kDefaultEncoding) > CLX_NUM_ZERO) {
			 $r['FILE_FIELDNAME'] .= '->' . $r['TABLE_FIELDNAME'];
		  }
		  $fl = new htmlField($form_fieldname, $default, CLX_NUM_40, $r['FIELD_LENGTH']);
		  $table->addData($r['DESCRIPTION'], $fl->source() . ' ' . $r['FILE_FIELDNAME']);
        }
        $form->addContents($table->source());
        //Buttons
        $b = new htmlFormSubmit(dict('GEN_LBL_SUBMIT'), 'CHECK', $form->getProp(CLX_PROP_ID));
        $b2 = new htmlButtonReturn(null, null, null);
        $btn = $b->source() . $b2->source();
        $table = new htmlTable(CLX_WIDTH_ONE);
        $table->addCell($btn, array(CLX_PROP_CLASS => 'buttons', CLX_PROP_COLSPAN => CLX_NUM_ONE));
        $form->addContents($table->source());
        return $form->source();
    }

    /**
     * Returns a communication check screen with debug data
     *
     * @return string
     */
    function check() {
        $idlist = getIdlist();
        $record = UserRequest::$post;
        $msg_record = ClxCommon::$db->getRecord($this->tablename, 'ID = ' . singleQuote($idlist[CLX_NUM_ZERO]));
        $this->skco_code = (defined('EBSK_IBS_DEF_CONNECTOR') && EBSK_IBS_DEF_CONNECTOR <> CLX_NUM_ZERO) ? EBSK_IBS_DEF_CONNECTOR : $msg_record['SKCONNECTOR_CODE'];
        //Some Message Info
        $contents = 'Message Code:' . $msg_record['IDCODE'] . '<br>';
        $contents .= 'Message Definition Length:' . $this->getLength($msg_record['IDCODE']) . '<br>';
        $this->createMessageVoid($msg_record['IDCODE'], $record, 'SELECT');
        $strmsg = $this->toString_();
        $contents .= 'Prepared Message Length:' . mb_strlen($strmsg, kDefaultEncoding) . '<br>';
        $txt = new htmlTextArea('', 'Start->' . $strmsg . '<-End', CLX_NUM_TEN, CLX_NUM_110);
        $contents .= 'Message:<br>' . $txt->source();
        $result = $this->send();
        if (!$this->hasErrors()) {
		  $result = $this->record($result, $msg_record['IDCODE']);
        }
        $txt = new htmlTextArea('', var_export($result, true), CLX_NUM_TEN, CLX_NUM_110);
        $contents .= '<br>Response:<br>' . $txt->source();
        $i = CLX_NUM_ZERO;
        $table = new htmlTable(CLX_WIDTH_ONE);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell('E-IBS Exchange', array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_ONE));
        $table->addCell($contents, array(CLX_PROP_CLASS => ($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT));
        $c = new htmlButtonGoto(dict('GEN_LBL_CLOSE'), 'tool.php?to=EBSK&ac=LIST');
        $table->addCell($c->source(), array(CLX_PROP_CLASS => 'footer'));
        return $table->source();
    }

    /**
     * Returns record array
     *
     * @param array $row
     * @param string $message_code
     * @return array
     */
    function record($row, $message_code, $blank_as_null = true) {

        $record = array();
        //Check if exists prior message
        if ($this->message_record[$message_code]['HEADERMSG_CODE'] <> '') {
		  foreach ($row[CLX_NUM_ZERO] as $key => $val) {
			 $record[CLX_NUM_ZERO]['HEADERMSG'][$val['FILEFIELD']] = ($val['VAL'] === '') ? '' : $val['VAL'];
		  }
		  unset($row[CLX_NUM_ZERO]);
        }

        //adding fields
        foreach ($row as $key => $val) {
		  $record[CLX_NUM_ZERO][$val['FILEFIELD']] = $val['VAL'];
		  if (mb_strlen($val['FIELD'], kDefaultEncoding) > CLX_NUM_ZERO)
			 $record[CLX_NUM_ZERO][$val['FIELD']] = $val['VAL'];
		  if ($val['TABLE'] <> '' && $val['FIELD'] <> '') {
			 if ($blank_as_null === false) {
			     $record[$val['TABLE']][$val['FIELD']] = ($val['VAL'] === '') ? '' : $val['VAL'];
			 } else {
			     $record[$val['TABLE']][$val['FIELD']] = ($val['VAL'] === '') ? 'null' : $val['VAL'];
			 }
		  }
        }

        //building tables
        if ($message_code <> '') {
		  foreach ($record as $table => $rec) {
			 if (empty($table))
			     continue; //No need to execute formulas on error msg
			 $this->executeFormula($message_code, $record[$table]);
		  }
        }

        return $record;
    }

    /**
     * Build test file on file builder
     * @return string
     */
    function file_utility() {
        $idlist = toString(getIdList());
        $msg_code = ClxCommon::$db->getField($this->tablename, 'IDCODE', 'ID=' . $idlist);
        $table = new htmlTable(CLX_WIDTH_ONE);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM));
        $table->addCell(dict('GEN_LBL_FILE_UTIL'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE));
        $table->addCell($this->fix_length($msg_code), array(CLX_CLASS_DATA => CLX_CLASS_DATA));
        return $table->source();
    }

    /**
     * builds file utility
     * @param $message_code
     * @return string
     */
    function fix_length($message_code) {
        $layout = array();
        $sql = 'SELECT FILE_FIELDNAME, DATA_TYPE, START_POSITION, FIELD_LENGTH, FIELD_DECIMALS FROM CLX_EB_SOCKET_MAPPINGS WHERE EBSOCKETCODE = ' . singleQuote($message_code) . ' ORDER BY SEQNUM ASC';
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, NUMERIC)) {
		  $layout[] = array($r[CLX_NUM_ZERO], $r[CLX_NUM_ONE], $r[CLX_NUM_TWO], $r[CLX_NUM_THREE], $r[CLX_NUM_FOUR]);
        }
        $form = '<script language="javascript">
    	//FIELDNAME, TYPE, START, LENGTH, DECIMALS
    	var layout=new Array();';
        foreach ($layout as $key => $val) {
		  $form .='layout[' . $key . ']=new Array(' . singleQuote($val[CLX_NUM_ZERO]) . ',' . singleQuote($val[CLX_NUM_ONE]) . ',' . singleQuote($val[CLX_NUM_TWO]) . ',' . singleQuote($val[CLX_NUM_THREE]) . ',' . singleQuote($val[CLX_NUM_FOUR]) . ');' . CLX_CHAR_NL;
        }
        $form .='
    	function format_fix(){
    	 var tab = document.getElementById("tab");
    	 var fix = document.getElementById("fix");
    	// var tabl = tab.value.length;
    	// var rowl = 1*layout[layout.length-1][2]+1*layout[layout.length-1][3];
    	 var nline = String.fromCharCode(13);
    	 var ntab = String.fromCharCode(9);
    	 fix.value = ' . singleQuote(CLX_CHAR_EMPTY_STRING) . ';
    	 var src = new Array();
    	 src = tab.value.split(nline);//by new line
    	 for(var i=0;i<src.length; i++){
    		if(src[i]==' . singleQuote(CLX_CHAR_EMPTY_STRING) . ') continue;
    		src[i] = src[i].replace(String.fromCharCode(13),' . singleQuote(CLX_CHAR_EMPTY_STRING) . ');
    		src[i] = src[i].replace(String.fromCharCode(10),' . singleQuote(CLX_CHAR_EMPTY_STRING) . ');
    		var ln = new Array();
    		ln = src[i].split(ntab);//by tab
    		if(ln.length != layout.length){
    			fix.value = ' . singleQuote('Invalid columns count: Tab') . '+ln.length+' . singleQuote(' vs Def') . '+layout.length;
    			break;
    		}
    		for(var j=0;j<ln.length; j++){
        		if(layout[j][1]==' . singleQuote('DEC') . ') {
    				ln[j] = ln[j].replace(' . singleQuote(CLX_CHAR_EMPTY_STRING) . ',' . singleQuote(CLX_CHAR_EMPTY_STRING) . ');
    				ln[j] = ln[j].replace(' . singleQuote(CLX_CHAR_EMPTY_STRING) . ',' . singleQuote(CLX_CHAR_EMPTY_STRING) . ');
    				ln[j] = ln[j].replace(' . singleQuote(CLX_CHAR_EMPTY_STRING) . ',' . singleQuote(CLX_CHAR_EMPTY_STRING) . ');
    			}
    			var dif = 1*layout[j][3]-1*ln[j].length;
    			if(dif<0){
    				fix.value = ' . singleQuote('Field overflow on line:') . '+(i*1+1)+' . singleQuote(' Column') . '+(j*1+1)+' . singleQuote(' Value') . '+ln[j];
    				return;
    			}else if(dif>0){
    				if(layout[j][1]==' . singleQuote('CHAR') . '){
    					for(var d = CLX_INIT_ZERO ; d<dif; d++){
    						fix.value += ' . singleQuote(CLX_CHAR_EMPTY_STRING) . ';
    					}
    					fix.value += ln[j];
    				}else{
    					for(var d = CLX_INIT_ZERO ; d<dif; d++){
    						fix.value += ' . singleQuote(CLX_CHAR_EMPTY_STRING) . ';
    					}
    					fix.value += ln[j];
    				}
    			}else{
    				fix.value += ln[j];
    			}
    		}//end for j
    		fix.value += nline;
    	 }//end for i
    	}
    	</script>
    	INPUT: Copy Tab Delimited Data:<br>
    	<textarea name=' . singleQuote('tab') . ' id=' . singleQuote('tab') . ' rows="10" cols="80" onChange="javascript:format_fix();"></textarea><br>
    	<br>
    	OUTPUT: Fixed Length Data: <br>
    	<textarea name=' . singleQuote('fix') . ' id=' . singleQuote('fix') . ' rows="10" cols="80"></textarea><br>';
        return $form;
    }

    /**
     * Returs message Record
     * @param $message_code
     */
    function getMessage($message_code) {
        return ClxCommon::$db->getRecord($this->tablename, 'IDCODE = ' . singleQuote($message_code));
    }

    /**
     * Parse read only properties and unset value if read only
     *
     * @param string $record_action
     * @param array $record
     */
    function executeReadOnly($message_code, $record_action, &$record) {
        foreach ($this->read_onlys[$message_code] as $key => $val) {
		  $fieldname = $this->mappings[$message_code][$key]['TABLE_FIELDNAME'];
		  switch (intval($this->mappings[$message_code][$key]['READ_ONLY'])) {
			 case CLX_NUM_ONE://always
			     unset($record[$fieldname]); //always
			     break;
			 case CLX_NUM_TWO:
			     if ($record_action === 'CREATE') {
			         unset($record[$fieldname]); //always
			     }
			     break;
			 case CLX_NUM_THREE://on modify
			     if ($record_action === 'MODIFY') {
			         unset($record[$fieldname]); //always
			     }
			     break;
			 default:
			     break;
		  }//end switch
        }
    }

    /**
     * Builds search Criteria
     * @param array $record
     * @return string
     */
    function executeSearchKeys($message_code, &$record) {
        $criterias = '';
        $comma = '';
        foreach ($this->search_keys[$message_code] as $key => $val) {
		  $fieldname = $this->mappings[$message_code][$key]['TABLE_FIELDNAME'];
		  $criterias .= $comma . $fieldname . '=' . singleQuote($record[$fieldname]);
		  $comma = ' AND ';
        }
        return $criterias;
    }

    /**
     * Excecute formula in a formula on records
     *
     * @param array $maprecord, int $line, string $buffer
     * @return array
     */
    function executeFormula($message_code, &$record) {
        if (empty($this->formulas[$message_code]))
		  return;
        foreach ($this->formulas[$message_code] as $key => $val) {
		  //$this->executeFieldFormula($this->mappings[$message_code][$key]['FORMULA'], $record);
		  $val = '';
		  $this->includeFormulaConstants($this->mappings[$message_code][$key], $record, $val);
        }
    }

    /**
     * Includes formula
     * @param type $maprecord
     * @param type $record
     * @param type $val
     */
    private function includeFormulaConstants(&$maprecord, &$record, &$val) {
        $val = '';
        include $this->formula_file;
    }

    //DATA IMPORT METHODS

    /**
     * Insert Map Record
     * @param string $tablename
     * @param string $record
     */
    function processCreateRecord($message_code, $tablename, $record) {
        $ac = UserRequest::$request['ac'];
        $app_tbl = false;
        if (!empty($this->table_info[$tablename])) {
		  $tablerec = $this->table_info[$tablename];
		  $app_tbl = true;
        }
        $msg = array();
        $EVNT = &getTool('EVNT');
        $ac = 'CREATE';
        if ($app_tbl === true) {
		  $EVNT->execute5('DBTB', $tablerec['ID'], 'CREP', $record, $msg); //  Create Record Pre
        }
        $this->executeReadOnly($message_code, $ac, $record);
        $record['ID'] = ClxCommon::$db->createRecord($tablename, $record, false);
        $this->record_proces_counter['create'] ++;
        if ($app_tbl === true) {
		  $EVNT->execute5('DBTB', $tablerec['ID'], 'CREC', $record, $msg); //  Create Record Post
        }
        return $record['ID'];
    }

    /**
     * Updates update maped record.
     * @param string $tablename
     * @param array $record
     */
    function processModifyRecord($message_code, $tablename, $record) {
        $ac = UserRequest::$request['ac'];
        $app_tbl = false;
        if (!empty($this->table_info[$tablename])) {
		  $tablerec = $this->table_info[$tablename];
		  $app_tbl = true;
        }
        $msg = array();
        $EVNT = &getTool('EVNT');
        $ac = 'MODIFY';
        $criteria = $this->executeSearchKeys($message_code, $record);
        $this->executeReadOnly($message_code, $ac, $record);
        $original_record = ClxCommon::$db->getRecord($tablename, $criteria);
        $record = array_merge($original_record, $record);
        if ($app_tbl === true) {
		  $EVNT->execute5('DBTB', $tablerec['ID'], 'UREP', $record, $msg); //  Update Record (pre database commit)
        }
        $record['ID'] = ClxCommon::$db->updateRecord($tablename, $record, $criteria);
        $this->record_proces_counter['modify'] ++;
        if ($app_tbl === true) {
		  $EVNT->execute5('DBTB', $tablerec['ID'], 'UREC', $record, $msg); //  Update Record (post database commit)
        }
        return $record['ID'];
    }

    /**
     * Loads record if not exists
     * @param string $tablename
     * @param array $record
     */
    function processLoadRecord($message_code, $tablename, $record) {
        $criteria = $this->executeSearchKeys($message_code, $record);
        if (mb_strlen($criteria, kDefaultEncoding) > CLX_NUM_ZERO) {
            $qty = ClxCommon::$db->oneExistsRecords($tablename, $criteria);
        } else {
		  $qty = CLX_INIT_ZERO;
        }
        if ($qty > CLX_NUM_ZERO) {
		  $return = $this->processModifyRecord($message_code, $tablename, $record);
        } else {
		  $return = $this->processCreateRecord($message_code, $tablename, $record);
        }

        // Caso 1002664 - BEGIN
        if ($this->create_log && isset($this->record_proces_counter['eff_lin_count'])) { // Log created/updated line
		  $log_file_wrk_str = $this->log_file_operations('str_line', $this->record_proces_counter['eff_lin_count']) . dict(($qty > CLX_NUM_ZERO ? 'GEN_LBL_UPDATE' : 'GEN_LBL_CREATED')) . (mb_strlen($criteria, kDefaultEncoding) > 0 ? (' | ' . $criteria) : '');
		  $this->log_file_operations('write', $log_file_wrk_str);
        }
        // Caso 1002664 - END
        return $return;
    }

    /**
     * Replace Record
     * @param string $tablename
     * @param array $record
     */
    function processReplaceRecord($message_code, $tablename, $record) {
        $this->record_proces_counter['replace'] ++;
        ClxFreeReturn($this->processDeleteRecord($message_code, $tablename, $record));
        ClxFreeReturn($this->processCreateRecord($message_code, $tablename, $record));
    }

    /**
     * Delete Records
     * @param string $tablename
     * @param array $record
     */
    function processDeleteRecordVoid($message_code, $tablename, $record) {
        ClxFreeReturn($this->processDeleteRecord($message_code, $tablename, $record));
    }

    function processDeleteRecord($message_code, $tablename, $record) {
        $ac = UserRequest::$request['ac'];
        $status = true;
        $ac_store = $ac;
        //1. Find if record already exists based on search keys
        $this->record_proces_counter['delete'] ++;
        $criteria = $this->executeSearchKeys($message_code, $record);
        if (!empty($criteria)) {
		  ClxCommon::$db->deleteRecord($tablename, $criteria);
        }
        $ac = $ac_store;
        return $status;
    }

    // Implementation methods to be called from

    /**
     * Sends socket message
     * @param string $message_code
     * @param string action QUERY, CRE
     * @param string $parameters
     */
    function sendMessage($message_code, $parameters, $action) {
        $this->errors = array();
        $msg_record = ClxCommon::$db->getRecord($this->tablename, 'IDCODE = ' . singleQuote($message_code));
        $this->skco_code = (defined('EBSK_IBS_DEF_CONNECTOR') && EBSK_IBS_DEF_CONNECTOR <> CLX_NUM_ZERO) ? EBSK_IBS_DEF_CONNECTOR : $msg_record['SKCONNECTOR_CODE'];
        switch ($action) {
		  case 'SELECT':
		  case 'CREATE_REMOTE':
		  case 'SELECT_FOR_UPDATE_REMOTE':
		  case 'REPLACE':
		  case 'CREATE':
		  case 'MODIFY':
		  case 'LOAD':
		  case 'DELETE':
		  case 'PROC':
		  case 'CODE':
			 $this->createMessageVoid($msg_record['IDCODE'], $parameters, $action);
			 $blank_as_null = true;
			 break;
		  case 'UPDATE_REMOTE':
			 ClxFreeReturn($this->createMessage($msg_record['IDCODE'], $parameters, 'SELECT_FOR_UPDATE_REMOTE'));
			 $blank_as_null = false;
			 break;
		  default:
			 break;
        }
        $result = $this->send();
        if ($this->hasErrors())
		  return $this->build_ibs_style_error_array($this->errors);   //returning error on ibs style
        $result = $this->record($result, $message_code, $blank_as_null);
        switch ($action) {
		  case 'SELECT':
		  case 'CREATE_REMOTE':
		  case 'SELECT_FOR_UPDATE_REMOTE':
		  case 'PROC':
		  case 'CODE':
			 return $result;
			 break;
		  case 'UPDATE_REMOTE':   //mergin client info with user parameters 		//sending new data
			 $this->createMessageVoid($msg_record['IDCODE'], array_merge($result[CLX_NUM_ZERO], $parameters), 'UPDATE_REMOTE');
			 $result = $this->send();
			 if ($this->hasErrors())
			     return $this->build_ibs_style_error_array($this->errors);   //returning error on ibs style
			 return $this->record($result, $message_code);
			 break;
		  case 'CREATE':
			 foreach ($result as $table => $record) {
			     if (strval($table) === '0')
			         continue;
			     ClxFreeReturn($this->processCreateRecord($message_code, $table, $record));
			 }
			 return $result;
			 break;
		  case 'REPLACE':
			 foreach ($result as $table => $record) {
			     if (strval($table) === '0')
			         continue;
			     $this->processReplaceRecord($message_code, $table, $record);
			 }
			 return $result;
			 break;
		  case 'MODIFY':
			 foreach ($result as $table => $record) {
			     if (strval($table) === '0')
			         continue;
			     ClxFreeReturn($this->processModifyRecord($message_code, $table, $record));
			 }
			 return $result;
			 break;
		  case 'LOAD':
			 foreach ($result as $table => $record) {
			     if (strval($table) === '0')
			         continue;
			     ClxFreeReturn($this->processLoadRecord($message_code, $table, $record));
			 }
			 return $result;
			 break;
		  case 'DELETE':
			 foreach ($result as $table => $record) {
			     if (strval($table) === '0')
			         continue;
			     ClxFreeReturn($this->processDeleteRecord($message_code, $table, $record));
			 }
			 return $result;
			 break;
		  default:
			 return false;
			 break;
        }//end switch
    }

    /**
     * Runs interfase inserting records into database
     *
     * @param string $message_code
     */
    public function read($message_code) {
        $contenidoHTML = '';
        // Caso 1002664 - BEGIN
        $this->log_file_operations('counter_init'); //$error_count = CLX_INIT_ZERO;
        // Caso 1002664 - END
        $msg_rec = $this->getMessage($message_code);
        if (empty($msg_rec)) {
		  //send error message not found
		  return false;
        }
        $SKCO = &getTool('SKCO');
        $cnx_rec = $SKCO->getConnector($msg_rec['SKCONNECTOR_CODE']);
        if (empty($cnx_rec)) {
		  //drop exception
		  return false;
        }
        if ($this->debug === true) {
		  echoLog('<br>Connector: ' . $msg_rec['SKCONNECTOR_CODE'] . '<BR>');
		  echoLog('<br>Message: ' . $message_code . '<BR>');
        }
        $this->msg_type = $msg_rec['TYPE'];
        switch ($cnx_rec['CNX_TYPE']) {
		  case 'FILE':
			 //Rename file to lock it
			 if ($msg_rec['TYPE'] === 'CSV') {
			     $this->record_delim = replace_ascii_chars($msg_rec['RECORD_DELIM']);
			     $this->field_delim = replace_ascii_chars($msg_rec['FIELD_DELIM']);
			     if (!class_exists('clx_file_delimited'))
			         require_once kPathToWebSite . 'classes/clx_file_delimited.php';
			     $file = new clx_file_delimited($cnx_rec['IP_ADDRESS'], $this->record_delim, $this->field_delim);
			 }else {
			     if (!class_exists('clx_file_fixed_record_length')) {
			         require_once kPathToWebSite . 'classes/clx_file_fixed_record_length.php';
			     }
			     $record_delimiter = replace_ascii_chars($msg_rec['RECORD_DELIM']);
			     $record_length = intval($msg_rec['RECORD_LENGTH']) + mb_strlen($record_delimiter, kDefaultEncoding);
			     $file = new clx_file_fixed_record_length($cnx_rec['IP_ADDRESS'], $record_length);
			     // Caso 1002664 - BEGIN
			     // It's fixed lenght, but reads only until EOL character, to avoid cases of not so fixed lenght files like IBS's
			     if ($this->use_record_delimiter_reading_fixlen_files == true) {
			         $file->line_delim = $record_delimiter;
			     } // Caso 1002664 - END
			 }
			 // Caso 1002664 - BEGIN
			 if ($this->create_log) {
			     if (!$this->log_file_operations('open', $cnx_rec['IP_ADDRESS'])) {
			         return false; // Couldn't open log file
			     }
			 } // Caso 1002664 - END
			 //Check if file exists
			 if ($file->error === true) {
			     $contenidoHTML .= '<li>' . dict('GEN_ERR_FILE_NOT_FOUND') . ' ' . $cnx_rec['IP_ADDRESS'] . '</li>'; //drop exception
			     echo $contenidoHTML;
			     // Caso 1002664 - BEGIN
			     if ($this->create_log) {
			         $this->log_file_operations('write', $log_file_wrk_str);
			         $this->log_file_operations('close'); // Close log file
			     } // Caso 1002664 - END
			     return false;
			 }
			 $i = CLX_INIT_ZERO;
			 $commit_count = CLX_INIT_ZERO;
			 //read line by line and parse record message
                while (($buffer = $file->getNextRecord()) === true) {
			     // Caso 1002664 - BEGIN
			     if ($this->create_log) {
			         $this->record_proces_counter['eff_lin_count'] ++; // Effective line counter increment
			     } // Caso 1002664 - END
			     //skiping lines
			     if ($msg_rec['IGNORE_LINES'] <> CLX_NUM_ZERO && $msg_rec['IGNORE_LINES'] > $i) {
			         // Caso 1002664 - BEGIN
			         if ($this->create_log) { // Log ignored line
					   $log_file_wrk_str = $this->log_file_operations('str_line', $this->record_proces_counter['eff_lin_count']) . dict('GEN_LBL_IGNORED') . ' [0]';
					   $this->log_file_operations('write', $log_file_wrk_str);
					   $this->record_proces_counter['ignored'] ++;
			         } // Caso 1002664 - END
			         $i++;
			         continue;
			     }
			     //no Blank Lines
			     if (trim($buffer) === '') {
			         // Caso 1002664 - BEGIN
			         if ($this->create_log) {
					   // Log ignored line
					   $log_file_wrk_str = $this->log_file_operations('str_line', $this->record_proces_counter['eff_lin_count']) . dict('GEN_LBL_IGNORED') . ' [1]';
					   $this->log_file_operations('write', $log_file_wrk_str);
					   $this->record_proces_counter['ignored'] ++;
			         } // Caso 1002664 - END
			         continue;
			     }
			     // Caso 1002664 - BEGIN
			     if ($this->create_log) {
			         if ($this->use_record_delimiter_reading_fixlen_files == true) {
					   // Calculates buffer length adding the record delimiter (because it's removed in record reading method)
					   $log_file_wrk_str = mb_strlen($buffer, kDefaultEncoding) + mb_strlen($record_delimiter, kDefaultEncoding);
					   if ($log_file_wrk_str != $record_length) { // Log WARNING : Line doesn't have spected length
						  /*
						   * GEN_LBL_WRNG_LINE_LENGTH
						   * Line length [^1] doesn't have spected length [^2]
						   * El tamaño de la línea [^1] no tiene el tamaño esperado [^2]
						   */
						  $log_file_wrk_str = $this->log_file_operations('str_line', $this->record_proces_counter['eff_lin_count']) . uppercase(dict('GEN_LBL_WARNING')) . ": " . dict('GEN_LBL_WRNG_LINE_LENGTH', $log_file_wrk_str, $record_length);
						  $this->log_file_operations('write', $log_file_wrk_str);
						  $this->record_proces_counter['warning'] ++;
					   }
			         }
			     } // Caso 1002664 - END
			     $row = $this->record($this->readMessage($message_code, $buffer), $message_code);
			     if ($this->debug === true)
			         pre($row);
			     foreach ($row as $table => $record) {
			         if (strval($table) === '0')
					   continue;
			         //Process load
			         $app_tbl = false;
			         if (!empty($this->table_info[$table])) {
					   $tablerec = $this->table_info[$table];
					   $app_tbl = true;
			         }
			         $msg = array();
			         if (empty($app_tbl)) {//validar solo si existe tabla con codigo definido
					   ClxFreeReturn($this->processLoadRecord($message_code, $table, $record));
			         } elseif ($app_tbl && $this->validateRecordSocket($tablerec, $record, $msg)) {
					   ClxFreeReturn($this->processLoadRecord($message_code, $table, $record));
			         } else {
					   if (!empty($msg)) {
						  // Caso 1002664 - BEGIN
						  $this->record_proces_counter['error_count'] ++;
						  if ($this->record_proces_counter['error_count'] < CLX_NUM_HUNDRED && $this->debug) {
						      pre($msg);
						  }
						  if ($this->create_log) {
						      // Log line with error msgs
						      $log_file_wrk_str = '';
						      foreach ($msg as $keyq => $valq) {
								$log_file_wrk_str .= ($log_file_wrk_str != '' ? ', ' : '') . $valq;
						      }
						      $log_file_wrk_str = $this->log_file_operations('str_line', $this->record_proces_counter['eff_lin_count']) . '[' . $table . '] ' . dict('GEN_LBL_ERROR') . ': ' . $log_file_wrk_str;
						      $this->log_file_operations('write', $log_file_wrk_str);
						  }
					   } else {
						  if ($this->create_log) {
						      $this->record_proces_counter['error_count'] ++;
						      // Log line without error msgs
						      $log_file_wrk_str .= dict('GEN_LBL_ERROR') . ' [???]';
						      $this->log_file_operations('write', $log_file_wrk_str);
						  }
					   } // Caso 1002664 - END
					   //Adding process exception
					   continue;
			         }
			     }
			     if ($this->record_commit_counter > CLX_NUM_ZERO && $commit_count > $this->record_commit_counter) {
			         $commit_count = CLX_INIT_ZERO;
			         ClxCommon::$db->commit();
			     }
			     $commit_count += CLX_NUM_ONE; //   - this was missing from previous revision, so commit never happens...
			 }
			 $file->close();
			 ClxCommon::$db->commit(); //last commit for pending cases
			 $operation_response = true;
			 if ($this->rename_file === true) {
                    if ($this->debug === true || $this->verbouse === true)
                        echoLog('<br>RENAME [1] "' . $file->file_name . '" TO "' . $filename . '<br>');
                    $d = xFormat::getNow('_Ymd_his');
                    $filename = str_replace('.TXT', $d . $this->rename_file_extension, $file->file_name);
			     $rename_response = $file->renameVerify($file->file_name, $filename);
			     if ($rename_response['error'] === true) {
			         $operation_response = false;
			         $contenidoHTML .= '<li>' . $rename_response['errormsg'] . '</li>';
			     }
			 }
			 if ($this->debug === true || $this->verbouse === true) {
			     echoLog('<br>' . dict('GEN_LBL_SUMMARY') . ':<br>' . $msg_rec['IDNAME'] . '<br> ' . $cnx_rec['IP_ADDRESS'] . '<br>' . dict('GEN_LBL_CREATED') . ': ' . $this->record_proces_counter['create'] . ' ' . dict('GEN_LBL_MODIFIED') . ': ' . $this->record_proces_counter['modify'] . ' ' . dict('GEN_LBL_REPLACED') . ' Replaced: ' . $this->record_proces_counter['replace'] . ' ' . dict('GEN_LBL_DELETED') . ': ' . $this->record_proces_counter['delete'] . ' ' . dict('GEN_LBL_ERRORS') . ': ' . $error_count . '<br>');
			 }
			 echo $contenidoHTML;
			 // Caso 1002664 - BEGIN
			 $log_file_wrk_str = $this->log_file_operations('summary', $msg_rec['IDNAME'], $cnx_rec['IP_ADDRESS']);
			 if ($this->create_log) {
			     $this->log_file_operations('write', str_replace('<br>', "\n", $log_file_wrk_str));
			     $this->log_file_operations('close'); // Close log file
			 }
			 if ($this->debug || $this->verbouse) {
			     echo $log_file_wrk_str;
			 } // Caso 1002664 - END
			 return $operation_response;
			 break;
		  default:
			 return false;
			 break;
        }//end switch
    }

    private function validateRecordSocket(&$tablerec, &$record, &$msg) {
        //Ejecutando
        //Run table Validate events
        //constructing record tablecode_fieldname forma. must be done after formulas to pass values to validate events on table.
        $DBFL = &getTool('DBFL');
        $EVNT = &getTool('EVNT');
        $savepost = UserRequest::$post;
        $validate_record = array();
        foreach ($record as $table_fieldname => $val) {
		  //Validate field first
		  $fieldrec = $DBFL->getField($table_fieldname);
		  $fieldinstance = getToolObject('DBFT', $fieldrec['FIELDTYPE']);
		  $fieldinstance->saveFieldImport($table_fieldname, $fieldrec, $record);
		  if ($val === '') {
			 if ($fieldrec['FIELDTYPE'] === 'BOOL') {
			     $val = ($fieldrec['FIELD_DEFAULT']) ? CLX_NUM_ONE : CLX_NUM_ZERO;
			 }
		  }
		  // Caso 1002664 - BEGIN
		  if ($fieldrec["FIELDTYPE"] == 'DATE' && $val == '0000-00-00') {
			 $val = null; // Para que no de error "No se puede interpretar el valor suministrado" para las fechas que vienen en blanco como 0000-00-00
		  }
		  if ($fieldrec["FIELDTYPE"] == 'DECN' || $fieldrec["FIELDTYPE"] == 'INTN') {
			 $val = 0 + $val;
			 $val = "" . $val;
		  } elseif ($val == 'NULL') {
			 $val = null; // Para que no de error "No se puede interpretar el valor suministrado" para los nulos
		  } // Caso 1002664 - END
		  $fieldinstance->validateImportField($msg, $fieldrec, $val);
		  // Caso 1002664 - BEGIN
		  /* if(!empty($msg)){
		    return false;   //return if error is found
		    } */
		  $tbl_fld_name = $tablerec['IDCODE'] . '.' . $table_fieldname; // table_code.field_name
		  if (!is_null($this->valid_field_values_ar) && isset($msg[$table_fieldname])) { // Field has error, so verify if the value is in valid field values array
			 if (isset($this->valid_field_values_ar[$tbl_fld_name])) {
			     if (in_array($val, $this->valid_field_values_ar[$tbl_fld_name])) {
			         unset($msg[$table_fieldname]); // Value is in valid field values array, so obviate the error and allows it
			     }
			 }
		  }
		  if (isset($msg[$table_fieldname])) { // There's error(s) for field $table_fieldname...
			 if ($this->create_log) { // We need all fields with error message on each line when log is enabled
			     $msg[$table_fieldname] = '{FLD: ' . $table_fieldname . ' = ' . $val . '} ' . $msg[$table_fieldname];
			     if ($this->invalid_field_values_report === true) { // If invalid field values summary report is enabled...
			         if (!isset($this->invalid_field_values_ar[$tbl_fld_name])) {
					   $this->invalid_field_values_ar[$tbl_fld_name] = array(); // Creates field entry for first time
			         }
			         if (!in_array($val, $this->invalid_field_values_ar[$tbl_fld_name])) { // If invalid value is not registered in array, add entry to report at the end
					   if (count($this->invalid_field_values_ar[$tbl_fld_name]) < $this->invalid_field_values_max_cnt) {
						  $this->invalid_field_values_ar[$tbl_fld_name][] = $val;
					   } elseif (count($this->invalid_field_values_ar[$tbl_fld_name]) == $this->invalid_field_values_max_cnt) {
						  $this->invalid_field_values_ar[$tbl_fld_name][] = '... [' . dict('GEN_LBL_ERROR_LIMIT') . ']'; // Value error limit reached for this field
					   }
			         }
			     }
			 } else {
			     return false; //return if error is found (and not log creation)
			 }
		  }
		  // Caso 1002664 - END
		  //addit to the event record
		  $validate_record[$tablerec['IDCODE'] . '_' . $table_fieldname] = $val;
        }
        // Caso 1002664 - BEGIN
        if (!empty($msg)) {
		  return false; //return if any field error is found
        } // Caso 1002664 - END
        UserRequest::$post = $validate_record;
        $EVNT->execute5('DBTB', $tablerec['ID'], 'VREC', $validate_record, $msg); //  Validate Record
        unset($validate_record);
        UserRequest::$post = $savepost;
        // Caso 1002664 - BEGIN
        if ($this->create_log) {
		  foreach ($msg as $keyq => $valq) {
			 $msg[$keyq] = '{EVE} ' . $valq;
		  }
        } // Caso 1002664 - END
        return (empty($msg)) ? true : false;
    }

    /**
     * Reads file carrinying multiples rows to multiple messages
     * @param string $connector_code
     * @param boolean $flg_production
     */
    public function readMultiMessageFile($connector_code, $flg_production = true, $message_code = '') {
        $message_code = (!isset($message_code) ? '' : (is_null($message_code) ? '' : trim($message_code)));
        // Caso 1002664 - BEGIN
        //$this->record_proces_counter = array('create' => CLX_NUM_ZERO, 'modify' => CLX_NUM_ZERO, 'delete' => CLX_NUM_ZERO, 'replace' => CLX_NUM_ZERO);
        $this->log_file_operations('counter_init');
        // Caso 1002664 - END
        $SKCO = &getTool('SKCO');
        $cnx_rec = $SKCO->getConnector($connector_code);
        if (empty($cnx_rec)) {
		  echoLog('<br>Connector Not Found: ' . $connector_code . '<BR>');
		  return false; //drop exception here
        }
        if ($this->debug === true) {
		  echoLog('<br>Connector: ' . $connector_code . '<BR>');
        }
        $sw_ignore_headermsg_code = false;
        switch ($cnx_rec['CNX_TYPE']) {
		  case 'FILE':
			 $sw_ignore_headermsg_code = true;
			 $commit_count = CLX_INIT_ZERO;
			 // Caso 1002664 - BEGIN
			 if ($this->create_log) {
			     if (!$this->log_file_operations('open', $cnx_rec['IP_ADDRESS'])) {
			         return false; // Couldn't open log file
			     }
			 } // Caso 1002664 - END
			 $contenidoHTML = '<br>' . (isset($this->message_record[$message_code]) ? $this->message_record[$message_code]['IDNAME'] : '') . '<br>' . dict('GEN_LBL_FILE') . ': ' . $cnx_rec['IP_ADDRESS'] . '<br><br>';
			 $file = new clx_file($cnx_rec['IP_ADDRESS'], 'r');
			 if (!$file->fileExists_()) { //Check if file exists
			     // Caso 1002664 - BEGIN
			     $log_file_wrk_str = dict('GEN_ERR_FILE_NOT_FOUND') . ': ' . $cnx_rec['IP_ADDRESS'];
			     $contenidoHTML .= '<li>' . $log_file_wrk_str . '</li>';
			     echo $contenidoHTML; //drop exception
			     if ($this->create_log) {
			         $this->log_file_operations('write', $log_file_wrk_str);
			         $this->log_file_operations('close'); // Close log file
			     } // Caso 1002664 - END
			     return false; //drop exception here
			 }
			 $file->open();
			 if ($file->error) {
			     $log_file_wrk_str = dict('GEN_LBL_ERROR') . ': ' . $file->errorMsg;
			     $contenidoHTML .= '<li>' . $log_file_wrk_str . '</li>'; //can't open file error
			     echo $contenidoHTML; //drop exception
			     if ($this->create_log) {
			         $this->log_file_operations('write', $log_file_wrk_str);
			         $this->log_file_operations('close'); // Close log file
			     }
			     return false;
			 }
			 $this->compiledMessages = array();
			 while (($buffer = $file->read_line())) {
			     // Caso 1002664 - BEGIN
			     if ($this->create_log) {
			         $this->record_proces_counter['eff_lin_count'] ++; // Effective line counter increment
			     } // Caso 1002664 - END
			     if (trim($buffer) === '') {
			         continue; //nothing on line
			     }
			     if ($message_code === '') {
			         $message_code = trim(substr($buffer, CLX_NUM_ZERO, CLX_NUM_NINE)); //toma el codigo de la trama.
			     }
			     // Generate event_xxx.php file (once per message) to effectivelly get formulas and constants
			     if (!isset($this->compiledMessages[$message_code])) {
			         $this->compileCodeFormulas($message_code);
			         $this->compiledMessages[$message_code] = CLX_NUM_ONE;
			     }
			     //Verificar si es necesario colocar tipo csv
			     $row = $this->record($this->readMessage($message_code, $buffer, $sw_ignore_headermsg_code), $message_code);
			     if ($flg_production === true) {
			         foreach ($row as $table => $record) {
					   if (strval($table) === '0') {
						  continue;
					   }
					   ClxFreeReturn($this->processLoadRecord($message_code, $table, $record));
					   // Verify commit condition
					   if ($this->record_commit_counter > CLX_NUM_ZERO && $commit_count > $this->record_commit_counter) {
						  $commit_count = CLX_INIT_ZERO;
						  ClxCommon::$db->commit();
					   }
			         }
			         $commit_count += CLX_NUM_ONE; //   - this was missing from previous revision, so commit never happens...
			     } else {
			         echoLog($this->debug_table_from_ar($row)); // Debug only
			     }
			 }
			 $file->close();
			 // Caso 1002664 - BEGIN
			 /* if ($this->debug === true) {
			   echoLog('<br>Record Operation Summary,' . ' Created: ' . $this->record_proces_counter['create'] . ' Modified: ' . $this->record_proces_counter['modify'] . ' Replaced: ' . $this->record_proces_counter['replace'] . ' Deleted: ' . $this->record_proces_counter['delete'] . '<br>');
			   } */
			 $log_file_wrk_str = (isset($this->message_record[$message_code]) ? $this->message_record[$message_code]['IDNAME'] : '');
			 $log_file_wrk_str = $this->log_file_operations('summary', $log_file_wrk_str, $cnx_rec['IP_ADDRESS']);
			 if ($this->create_log) {
			     $this->log_file_operations('write', str_replace('<br>', "\n", $log_file_wrk_str));
			     $this->log_file_operations('close'); // Close log file
			 }
			 if ($this->debug || $this->verbouse) {
			     echo $log_file_wrk_str;
			 } // Caso 1002664 - END
			 if ($flg_production === true) {
			     if ($this->rename_file === true) {
			         $d = xFormat::getNow('_Ymd_his');
			         $new_file = str_replace('.', $d . '.', $cnx_rec['IP_ADDRESS']);
			         if ($this->debug === true || $this->verbouse === true) {
					   echoLog('<br>RENAME [2] "' . $cnx_rec['IP_ADDRESS'] . '" TO "' . $new_file . '<br>');
			         }
			         $rename_response = $file->renameVerify($cnx_rec['IP_ADDRESS'], $new_file);
			         if ($rename_response['error'] === true) {
					   echoLog('<br>' . $rename_response['errormsg'] . '<br>');
			         }
			     }
			 }
			 return true;
			 break;
		  default:
			 return false;
			 break;
        }//end switch
    }

    /**
     * Returns  multidimensional array with all of its elements with element's ket name as 'FILE_FIELDname).' OR given definition element value.
     * SEE function eibs_mapping_array_all_pars($ar_mappings, $s_fieldname = 'FILE_FIELDNAME')  @ data/code/EIBS_utilities.php -
     * @param array
     * @return array
     */
    function get_mappings_by_fieldname($s_msg_code, $s_fieldname = 'FILE_FIELDNAME') {
        $ar_mappings = $this->getMappings($s_msg_code);
        $ar_records = array();
        foreach ($ar_mappings as $key => $field) {
		  $ar_records[$field[$s_fieldname]] = $field;
        }
        return $ar_records;
    }

    // Caso 1002664 - BEGIN
    /**
     * Batch Log file operations. Return value depends on $mode.
     * @param string, string, string, string
     * @return variant
     */
    function log_file_operations($mode, $par1 = '', $par2 = '') {
        $ret = NULL;
        switch ($mode) {
		  case 'open':
			 if (!class_exists('clx_file'))
			     require_once kPathToWebSite . "classes/clx_file.php";
			 $this->log_file_handle = new clx_file((trim($par1) . '.log'), 'w'); // Create a .log file
			 $this->log_file_handle->open();
			 if ($this->log_file_handle->error) {
			     $contenidoHTML = '<li>' . dict('GEN_LBL_ERROR') . ': ' . $this->log_file_handle->errorMsg . '</li>';
			     echo $contenidoHTML; //drop exception
			     $ret = false;
			 } else {
			     $ret = true;
			 }
		  // no break in order to do the counter init
		  //break;
		  case 'counter_init':
			 $this->record_proces_counter = array('warning' => 0, 'ignored' => 0, 'create' => 0, 'modify' => 0, 'delete' => 0, 'replace' => 0, 'error_count' => 0, 'eff_lin_count' => 0, 'start_date_time' => xFormat::getNow("Y-m-d H:i:s"));
			 break;
		  case 'close':
			 $ret = $this->log_file_handle->close();
			 break;
		  case 'write':
			 $ret = $this->log_file_handle->write($par1 . "\n");
			 break;
		  case 'str_line':
			 $ret = dict('GEN_LBL_LINENUM') . ' ' . $par1 . ': ';
			 break;
		  case 'counter_add':
			 $this->record_proces_counter[$par1] ++;
			 break;
		  case 'summary':
			 $ret = '<br>' . $par1 . '<br>' . dict('GEN_LBL_FILE') . ': ' . $par2 . '<br><br>' . dict('GEN_LBL_SUMMARY') . ':<br><br>' . dict('GEN_LBL_CREATED') . ': ' . $this->record_proces_counter['create'] . ', ' . dict('GEN_LBL_MODIFIED') . ': ' . $this->record_proces_counter['modify'] . ', ' . dict('GEN_LBL_REPLACED') . ': ' . $this->record_proces_counter['replace'] . ', ' . dict('GEN_LBL_DELETED') . ': ' . $this->record_proces_counter['delete'] . ', ' . dict('GEN_LBL_ERRORS') . ': ' . $this->record_proces_counter['error_count'] . '<br>';
			 if ($this->create_log) {
			     // Log process results stats
			     $ret .= dict('GEN_LBL_IGNORED') . ': ' . $this->record_proces_counter['ignored'] . ", " . dict('GEN_LBL_WARNING') . ': ' . $this->record_proces_counter['warning'] . "<br>" . dict('GEN_LBL_TOTAL') . ': ' . dict('GEN_LBL_LINESPROCESSED', $this->record_proces_counter['eff_lin_count']) . "<br>";
			     if ($this->invalid_field_values_report === true && count($this->invalid_field_values_ar) > 0) {
			         $ret .= '<br>' . dict('GEN_ERR_INVALID_ENTRY') . ' - ' . dict('GEN_LBL_SUMMARY') . ':<br><br>';
			         // If invalid field values summary report is enabled, include the report
			         foreach ($this->invalid_field_values_ar as $key => $val_ar) {
					   $ret .= $key . " => " . implode(", ", $val_ar) . '<br>'; // Log table.field with invalid value list
			         }
			     }
			     $ret .= '<br>' . dict('GEN_LBL_TIME') . ': ' . $this->record_proces_counter['start_date_time'] . ' >> ' . xFormat::getNow("Y-m-d H:i:s") . '<br>';
			 }
			 break;
        }
        return $ret;
    }

    /////////////////////////////////////////////////////////////////////

    function GenFileFromQry($parameters, $group_sep = ',', $pair_sep = '|', $noderived = false) {
//$this->debug = true;
//$this->verbouse = true;
        $response = array(
		  'error' => true,
		  'msg' => '',
		  'data' => array()
        );
        // Get and validate parameters
        if (!isset($parameters['CONNECTORCODE'])) {
		  // Parameter CONNECTORCODE not specified / (drop exception here)
		  $response['msg'] = 'Parameter CONNECTORCODE not specified';
		  return $response;
        }
        $intermediary_directory = ( isset($parameters['WORK_DIR']) ? trim($parameters['WORK_DIR']) : kPathToTemp );
        if (empty($intermediary_directory))
		  $intermediary_directory = kPathToTemp;
        if (mb_substr($intermediary_directory, mb_strlen($intermediary_directory) - 1, 1) != "\\" && mb_substr($intermediary_directory, mb_strlen($intermediary_directory) - 1, 1) != '/') {
		  $intermediary_directory .= DIRECTORY_SEPARATOR;
        }
        // CONNECTORCODE format | EBSK_MSG_CODE_1|CLX_QUERY_CODE_1,EBSK_MSG_CODE_2|CLX_QUERY_CODE_2,...
        $ebsk_msgs_to_process = explode($group_sep, $parameters['CONNECTORCODE']);
        $count = count($ebsk_msgs_to_process);
        for ($i = 0; $i < $count; $i++) {
		  $tmp_var = explode($pair_sep, $ebsk_msgs_to_process[$i]);
		  $ebsk_msgs_to_process[$i] = array('msg_code' => trim($tmp_var[0]), 'query_code' => trim($tmp_var[1]));
        }
        if (!class_exists('clx_file'))
		  require_once kPathToWebSite . "classes/clx_file.php";
        $files_to_copy_queue = array();
        if ($this->debug || $this->verbouse) {
		  $contenidoHTML = '<br>';
		  echo $contenidoHTML;
        }
        // Process each pair EBSK Msg / Query
        foreach ($ebsk_msgs_to_process as $ebsk_qry_pair) {
		  // Get EBSK message definition to get connector code
		  $msg_rec = $this->getMessage($ebsk_qry_pair['msg_code']);
		  if (empty($msg_rec)) {
			 // Error : message not found / Drop exception here
			 $response['msg'] = 'EBSK Message not found: ' . $ebsk_qry_pair['msg_code'];
			 return $response;
		  }
		  // Get conector definition to get output file name
		  $SKCO = &getTool('SKCO');
		  $cnx_rec = $SKCO->getConnector($msg_rec['SKCONNECTOR_CODE']);
		  if (empty($cnx_rec)) {
			 // Connector not found / Drop exception here
			 $response['msg'] = 'EBSK Connector not found: ' . $msg_rec['SKCONNECTOR_CODE'];
			 return $response;
		  }
		  // Build final output filespec
		  $output_filespec = trim($cnx_rec['IP_ADDRESS']);
		  $output_filespec = str_replace('%yyyy-mm-dd%', xFormat::getNow("Y-m-d"), $output_filespec);
		  $output_filespec = str_replace('%yyyymmdd%', xFormat::getNow("Ymd"), $output_filespec);
		  $output_filespec = str_replace('%hh-mm-ss%', xFormat::getNow("H-i-s"), $output_filespec);
		  $output_filespec = str_replace('%hhmmss%', xFormat::getNow("His"), $output_filespec);
		  // Build intermediate output filespec
		  // It is created to avoid inteference with listener (the process which get the file) while file creation
		  // Once created, it will be copied to final filespec, where the listener can get it completelly
		  $tmp_var = pathinfo($output_filespec);
		  $intermediate_filespec = $intermediary_directory . $tmp_var['basename'];
		  if ($this->debug || $this->verbouse) {
			 $contenidoHTML = '<br><br>EBSK Message: ' . $ebsk_qry_pair['msg_code'] . ' : ' . $msg_rec['IDNAME'] . '<br>Query: ' . $ebsk_qry_pair['query_code'] . '<br>Intermediate Filespec: ' . $intermediate_filespec . '<br>Final Filespec: ' . $output_filespec . '<br>';
			 echo $contenidoHTML;
		  }
		  // Add detailed data to be reported to caller
		  $response['data'][] = array(
			 'msg_code' => $ebsk_qry_pair['msg_code'],
			 'query_code' => $ebsk_qry_pair['query_code'],
			 'wrk_file' => $intermediate_filespec,
			 'out_file' => $output_filespec
		  );
		  // Create intermediate output file
		  $file = new clx_file($intermediate_filespec, 'w');
		  //Open file or throw exception
		  $file->open();
		  if ($file->error) {
			 // Can't open output file error / Drop exception here
			 $response['msg'] = 'Can\'t open output file: ' . $intermediate_filespec;
			 return $response;
		  }
		  // Prepares query
		  $DBQY = getTool("DBQY");
		  if (!$DBQY->queryExists($ebsk_qry_pair['query_code'])) {
			 // Query doesn't exist / Drop exception here
			 $response['msg'] = 'Query doesn\'t exist: ' . $ebsk_qry_pair['query_code'];
			 return $response;
		  }
		  if (empty($DBQY->getAccess()->getCanRead())) {
			 // Access denied to any query table / Drop exception here
			 $response['msg'] = dict("GEN_ERR_NO_ACCESS") . ' [EBSK-CAN_READ][GenFileFromQry] ' . $ebsk_qry_pair['msg_code'] . ' - Query:' . $ebsk_qry_pair['query_code'];
			 return $response;
		  }
		  // Process each record of result set from query execution
		  $sql = $DBQY->getSql($this->code, $ebsk_qry_pair['query_code']);
		  $rs = ClxCommon::$db->executeQuery($sql);
		  $first_line = true;
		  while (($record = ClxCommon::$db->getNextRecord($rs)) == true) {
			 // Convert all values to ASCII to avoid problems with fixed line size and special charaters like áéíóú
			 foreach ($record as $key => $val) {
			     if (is_string($val))
			         $record[$key] = decode_string($val);
			 }
			 // Create formatted output line
			 $this->msgrecord = $this->createMessage($ebsk_qry_pair['msg_code'], $record);
			 $s_socket_line = $this->toString();
			 if ($first_line == true) {
			     $first_line = false; // To avoid last blank line...
			 } else {
			     $s_socket_line = "\r\n" . $s_socket_line; // Put CR+LF at the beguinning of each line, except the first one
			 }
			 $file->write($s_socket_line); // Write formatted output line
		  }
		  // Close intermediate output file
		  $file->close();
		  $files_to_copy_queue[] = array(
			 'from' => $intermediate_filespec,
			 'to' => $output_filespec
		  );
		  // Next EBSK / Query pair...
        }

        // Copy all generated files to its final destination
        if ($this->debug || $this->verbouse) {
		  $contenidoHTML = '<br>';
		  echo $contenidoHTML;
        }
        foreach ($files_to_copy_queue as $val) {
		  if ($this->debug || $this->verbouse) {
			 $contenidoHTML = '<br>copy [' . $val['from'] . '] to [' . $val['to'] . ']';
			 echo $contenidoHTML;
		  }
		  // Copy intermediate output file to final file
		  if (!$file->copy($val['from'], $val['to'])) {
			 // Drop exception here
			 $response['msg'] = 'Copy error, copying "' . $intermediate_filespec . '" to "' . $output_filespec . '"';
			 return $response;
		  }
        }
        $response['error'] = false;
        $response['msg'] = 'OK';
        return $response;
    }

    // Caso 1002664 - END

    /**
     * Builds events phisical library
     */
    private function compileCodeFormulas($messagecode) {
        $rs = ClxCommon::$db->getRecords('CLX_EB_SOCKET_MAPPINGS', 'EBSOCKETCODE = ' . singleQuote($messagecode) . ' AND (FORMULA <> ' . singleQuote('') . ' OR VAL_CONSTANT<>' . singleQuote('') . ')');
        $src = '$val = null; ' . CLX_CHAR_NL . 'switch (intval($maprecord[' . singleQuote('ID') . '])) {';
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
		  if (!empty($r['VAL_CONSTANT'])) {
			 $r['FORMULA'] = '$val = ' . $r['VAL_CONSTANT'];
		  }
		  $fm = str_replace(';;', ';', trim($r['FORMULA']) . ';');
		  $src .= CLX_CHAR_NL . CLX_CHAR_TAB . 'case ' . $r['ID'] . ':' . CLX_CHAR_NL . str_repeat(CLX_CHAR_TAB, 2) . $fm . CLX_CHAR_NL . str_repeat(CLX_CHAR_TAB, 2) . ' break;';
        }
        $src .= CLX_CHAR_NL . CLX_CHAR_TAB . 'default:' . CLX_CHAR_NL . str_repeat(CLX_CHAR_TAB, 2) . CLX_CHAR_NL . str_repeat(CLX_CHAR_TAB, 2) . ' break;' . CLX_CHAR_NL . ' }';
        $this->formula_file = saveAndUpdatePhpCodeEvents('ebsk_' . $messagecode, $src);
    }

}
