<?php

/**
 * Tool Class
 * For more information contact  support@alenet.com
 * @package		Clientix.Tool.Event
 * @abstract		System Events Manager
 * @author       $Author: alenetdev $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2019-01-04 12:12:29 -0400 (vi., 04 ene. 2019) $
 * @version        $Rev: 10441 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  Included label
 */
class events extends clx_tool {

    /**
     * List of Events allowed
     * @var array
     */
    var $events = array();
    var $cache_events = true;

    /**
     * Constructor
     */
    function __construct() {
        $this->code = 'EVNT';
        $this->name = 'Event Manager';
        $this->system = 'System';
        $this->version = '7.0.0.0';

        $this->has_access = true;


        $this->tablename = 'CLX_EVENTS';
        $this->record_name_singular = 'event';
        $this->record_name_plural = 'GEN_LBL_EVENTS';
        $this->primary_field = 'ID';
        $this->secondary_field = 'OWNERCODE';
        $this->seqnum_field = 'SEQNUM';
        $this->primary_key = array('ID');
        $this->event = array('LINI' => 'GEN_LBL_INIT_LISTING', 'LQSR' => 'GEN_LBL_QUICK_SEARCH');
        $this->events = array();
        // Record Events
        $this->events['IREC'] = array('name' => 'Init Record', 'object' => 'DBTB', 'language' => 'PHP');
        $this->events['LREC'] = array('name' => 'Load Record', 'object' => 'DBTB', 'language' => 'PHP');
        $this->events['VREC'] = array('name' => 'Validate Record', 'object' => 'DBTB', 'language' => 'PHP');
        $this->events['CREC'] = array('name' => 'Create Record -Post Database Commit', 'object' => 'DBTB', 'language' => 'PHP');
        $this->events['CREP'] = array('name' => 'Create Record -Pre Database Commit', 'object' => 'DBTB', 'language' => 'PHP');
        $this->events['UREC'] = array('name' => 'Update Record -Post Database Commit', 'object' => 'DBTB', 'language' => 'PHP');
        $this->events['UREP'] = array('name' => 'Update Record -Pre Database Commit', 'object' => 'DBTB', 'language' => 'PHP');
        $this->events['DREP'] = array('name' => 'Delete Record -Post Database Commit', 'object' => 'DBTB', 'language' => 'PHP');
        $this->events['DREC'] = array('name' => 'Delete Record -Pre Database Commit', 'object' => 'DBTB', 'language' => 'PHP');
        $this->events['AREP'] = array('name' => 'Autorization Approve Record -Post Database Commit', 'object' => 'DBTB', 'language' => 'PHP');
        $this->events['AREC'] = array('name' => 'Autorization Approve Record -Pre Database Commit', 'object' => 'DBTB', 'language' => 'PHP');
        $this->events['ARRP'] = array('name' => 'Autorization Reject Record -Post Database Commit', 'object' => 'DBTB', 'language' => 'PHP');
        //Import Events
        $this->events['INPR'] = array('name' => 'Pre-Import Execution', 'object' => 'DBIM', 'language' => 'PHP');
        $this->events['INPO'] = array('name' => 'Post-Import Execution', 'object' => 'DBIM', 'language' => 'PHP');
        // Form Events
        $this->events['INFO'] = array('name' => 'Init Form', 'object' => 'UIFM', 'language' => 'PHP');
        $this->events['VAFO'] = array('name' => 'Validate Form', 'object' => 'UIFM', 'language' => 'PHP');
        $this->events['SAFO'] = array('name' => 'Save Record', 'object' => 'UIFM', 'language' => 'PHP');
        $this->events['CRFO'] = array('name' => 'Create Record', 'object' => 'UIFM', 'language' => 'PHP');
        //form event js
        $this->events['FRLO'] = array('name' => 'Form Load', 'object' => 'UIFM', 'language' => 'JS');
        // Panel Events
        $this->events['INPA'] = array('name' => 'Init Panel', 'object' => 'UIFP', 'language' => 'PHP');
        $this->events['VAPA'] = array('name' => 'Validate Panel', 'object' => 'UIFP', 'language' => 'PHP');
        //Report Events
        $this->events['RPBS'] = array('name' => 'Before Start', 'object' => 'ANRP', 'language' => 'PHP');
        $this->events['RPBR'] = array('name' => 'Load Record', 'object' => 'ANRP', 'language' => 'PHP');
        // Listing Events
        $this->events['LINI'] = array('name' => 'Init Listing', 'object' => 'UILG', 'language' => 'PHP');
        $this->events['LQSR'] = array('name' => 'Quick Search', 'object' => 'UILG', 'language' => 'PHP');
        $this->events['LIRL'] = array('name' => 'Listing Load Record', 'object' => 'UILG', 'language' => 'PHP');
        $this->events['LIDI'] = array('name' => 'Listing Display Record', 'object' => 'UILG', 'language' => 'PHP');
        $this->events['LISE'] = array('name' => 'Listing Summary Format Option', 'object' => 'UILG', 'language' => 'PHP');
        //System Alert Events
        $this->events['SALI'] = array('name' => 'Before Start', 'object' => 'SYAL', 'language' => 'PHP');
        $this->events['SALO'] = array('name' => 'Load Record', 'object' => 'SYAL', 'language' => 'PHP');
        // Field Events
        $this->events['INFI'] = array('name' => 'Init Field', 'object' => 'UIFF', 'language' => 'PHP');
        $this->events['VAFI'] = array('name' => 'Validate Field', 'object' => 'UIFF', 'language' => 'PHP');
        $this->events['LOAD'] = array('name' => 'onload', 'object' => 'UIFF', 'language' => 'JS');
        $this->events['UNLD'] = array('name' => 'onunload', 'object' => 'UIFF', 'language' => 'JS');
        $this->events['CLIK'] = array('name' => 'onclick', 'object' => 'UIFF', 'language' => 'JS');
        $this->events['DCLK'] = array('name' => 'ondblclick', 'object' => 'UIFF', 'language' => 'JS');
        $this->events['MDWN'] = array('name' => 'onmousedown', 'object' => 'UIFF', 'language' => 'JS');
        $this->events['MOUP'] = array('name' => 'onmouseup', 'object' => 'UIFF', 'language' => 'JS');
        $this->events['MOVR'] = array('name' => 'onmouseover', 'object' => 'UIFF', 'language' => 'JS');
        $this->events['MMVE'] = array('name' => 'onmousemove', 'object' => 'UIFF', 'language' => 'JS');
        $this->events['MOUT'] = array('name' => 'onmouseout', 'object' => 'UIFF', 'language' => 'JS');
        $this->events['MFOC'] = array('name' => 'onfocus', 'object' => 'UIFF', 'language' => 'JS');
        $this->events['MBLR'] = array('name' => 'onblur', 'object' => 'UIFF', 'language' => 'JS');
        $this->events['KPRE'] = array('name' => 'onkeypress', 'object' => 'UIFF', 'language' => 'JS');
        $this->events['KDWN'] = array('name' => 'onkeydown', 'object' => 'UIFF', 'language' => 'JS');
        $this->events['KEUP'] = array('name' => 'onkeyup', 'object' => 'UIFF', 'language' => 'JS');
        $this->events['SBMT'] = array('name' => 'onsubmit', 'object' => 'UIFF', 'language' => 'JS');
        $this->events['RSET'] = array('name' => 'onreset', 'object' => 'UIFF', 'language' => 'JS');
        $this->events['SLCT'] = array('name' => 'onselect', 'object' => 'UIFF', 'language' => 'JS');
        $this->events['CHGE'] = array('name' => 'onchange', 'object' => 'UIFF', 'language' => 'JS');
        //Process Steps
        $this->events['PRPH'] = array('name' => 'Process Step', 'object' => 'PRSE', 'language' => 'COMP');
        //Sk Services
        $this->events['SKSV'] = array('name' => 'Execute Service', 'object' => 'SKSV', 'language' => 'PHP');
        //BP Tool
        $this->events['BCNO'] = array('name' => 'GEN_LBL_PROCESS_FLOW', 'object' => 'BPPR', 'language' => 'PHP', 'wait_user_action' => false);
        $this->events['BPUE'] = array('name' => 'GEN_LBL_USER_EVENT', 'object' => 'BPPR', 'language' => 'PHP', 'wait_user_action' => true);
        $this->events_list = array();
    }

    /**
     * Describes the tool table structure.
     * @return array ('type' => 'varchar(255)', 'properties'=>'auto increment',
     * 				'index'=>'', default = '', 'dict'=>true)
     * 				 dict true or false means that field holds dictionary entry
     */
    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['OWNERCODE'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '');
        $fields['OWNERID'] = array('type' => 'varchar(32)', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO, 'default_raw' => CLX_NUM_ZERO);
        $fields['EVENT'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '');
        $fields['ACTION'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '');
        $fields['STATUS'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['SEQNUM'] = array('type' => 'int(11) unsigned', 'null' => 'YES', 'properties' => '', 'index' => '', 'default' => CLX_NUM_ZERO);
        $fields['DO_SECTION'] = array('type' => 'varchar(20)', 'null' => 'YES', 'properties' => '');
        $fields['DO_SECTION_TYPE'] = array('type' => 'varchar(10)', 'null' => 'YES', 'properties' => '');
        $fields['EXTRA1'] = array('type' => 'int(11) unsigned', 'null' => 'YES', 'properties' => '');
        $fields['EXTRA2'] = array('type' => 'text', 'null' => 'YES', 'properties' => '');
        $fields['EXTRA3'] = array('type' => 'varchar(255)', 'null' => 'YES', 'properties' => '');
        $fields['EXTRA4'] = array('type' => 'varchar(255)', 'null' => 'YES', 'properties' => '');
        $fields['EXTRA5'] = array('type' => 'varchar(255)', 'null' => 'YES', 'properties' => '');
        $fields['EXTRA6'] = array('type' => 'text', 'null' => 'YES', 'properties' => '');
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'MUL', 'fieldname' => 'OWNERCODE');
        $indexes[$ipf . '03'] = array('index' => 'MUL', 'fieldname' => 'OWNERID');
        $indexes[$ipf . '04'] = array('index' => 'MUL', 'fieldname' => 'EVENT');
        $indexes[$ipf . '05'] = array('index' => 'MUL', 'fieldname' => 'ACTION');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB', 'fields' => $fields, 'indexes' => $indexes);
    }

    /**
     * Loads package into system
     * @param string $package
     */
    function unpack($parameters) {
        $parameters = array_values(array_merge(array('package' => null, 'id' => ''), $parameters));
        list($package, $ownerid) = $parameters;
        $record = $this->unpackRecord($package);
        foreach ($record as $key => $val) {
		  $record[$key] = stripslashes($val);
        }
        $record['OWNERID'] = uppercase((string) $ownerid);
        ClxCommon::$db->createRecordVoid($this->tablename, $record, false);
    }

    /**
     * Action is called for every user interaction with the module...
     *
     * @param unknown_type $ac
     * @param unknown_type $idlist
     * @return unknown
     */
    function action($ac, $idlist) {
        $contents = '';
        if ($this->getAccess()->isAdmin()) {
		  switch ($ac) {
			 case 'EVNT_COMPILE_CODE':
			     //$contents = note(dict('GEN_LBL_COMPLETED'));
			     $CODE = &getTool('CODE');
			     $contents = $CODE->action($ac, $idlist);
			     $this->compileCodeEvents();
			     break;
			 default:
			     $contents = parent::action($ac, $idlist);
			     break;
		  }
        } else {
		  $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [EVENTS-CAN_READ][' . $ac . '] ' . $this->getCode(false));
        }
        return $contents;
    }

    /**
     * Builds Panel listing on tool
     *
     * @param string $groupcode
     * @return string
     */
    function makepanel($groupcode = null, $ownerid = '') {
        //Compatibilidad con clx_tools
        $ownercode = $groupcode;
        if ($this->getAccess()->canRead()) {
		  $t = time();
		  $to = $this->code;
		  $here = 'tool.php?to=' . $to . '&t=' . $t;
		  $modulecode = '';
		  $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
		  $form->setProp(CLX_PROP_ID, 'EVNTList');
		  $form->addArgument(CLX_PROP_TO, $this->code);
		  $form->addArgument('ownercode', $ownercode);
		  $form->addArgument('ownerid', uppercase($ownerid));
		  $form->addArgument('ac', 'NOT_DEFINED');
		  $selectclause = '*';
		  $fromclause = 'CLX_EVENTS';
		  $whereclause = 'OWNERCODE=' . singleQuote($ownercode) . ' AND OWNERID=' . singleQuote(uppercase($ownerid));
		  $sortclause = 'EVENT, SEQNUM';
		  $page = new xPaging();
		  $page->setProp('rpp', CLX_NUM_TEN_THOUSAND);
		  $page->setProp('printheaders', false);
		  $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
		  $table = new htmlTable(CLX_WIDTH_FIVE);
		  $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
		  $page->header($table);
		  $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_EVENT'), dict('GEN_LBL_SEQNUM'), dict('GEN_LBL_STATUS'), dict('GEN_LBL_ACTION')));
		  $i = CLX_INIT_ZERO;
		  $status = array(dict('GEN_LBL_INACTIVE'), dict('GEN_LBL_ACTIVE'));
		  while ($record = $page->nextRecord()) {
			 $recordid = $record['ID'];
			 $select = new htmlCheckbox('selector_' . $recordid, false);
			 $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
			 $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
			 $action = getToolObject('ACTN', $record['ACTION']);
			 $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($this->events[$record['EVENT']]['name'], $record['SEQNUM'], $status[$record['STATUS']], nl2br($action->describe($record))));
			 $i++;
		  }
		  $buttons = '';
		  $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_DISPLAY'), array('action' => 'DISPLAY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
		  if ($this->getAccess()->canModify()) {
			 $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
		  }
		  if ($this->getAccess()->canDelete()) {
			 $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
		  }
		  if ($this->getAccess()->canDisplay()) {
			 $b = new htmlFormSubmit(dict('GEN_LBL_CREATE_SOMETHING', dict('GEN_LBL_EVENT')), 'CREATE', $form->getProp(CLX_PROP_ID));
			 $b->setProp(CLX_PROP_ID, 'ON');
			 $buttons .= $b->source();
		  }
		  if (($page->getProp('qty') > CLX_NUM_ONE) && $this->getAccess()->canModify()) {
			 $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_REORDER'), array('action' => 'REORDER', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
		  }
		  $form->addContents($page->footers($table, $buttons));
		  return buildPanelInterface(dict('TOOL_EVNT_TITLE_LIST'), $modulecode . '_EVENTS', $form->source(), CLX_NUM_MINUS_1);
        } else {
		  return CLX_CHAR_EMPTY_STRING;
        }
    }

    /**
     * Tool Edit Form
     * @param string $ac
     * @param string $idlist
     * @param array $record
     * @param array $msg
     * @return string
     */
    function editForm($ac, $idlist, $record, $msg) {
        // Build/get the different java scripts...
        $OBJS = &getTool('OBJS');
        $actioncodes = $OBJS->objectList('ACTN');
        $list = CLX_CHAR_TAB . 'var panellist = new Array();' . CLX_CHAR_NL;
        $i = CLX_INIT_ZERO;
        foreach ($actioncodes as $actioncode) {
		  $list .= sprintf(CLX_CHAR_TAB . 'panellist[%s] = ' . singleQuote('%s') . ';' . CLX_CHAR_NL, $i, $actioncode);
		  $i++;
        }
        $variables = array('CONTENTS' => $list);
        $script = setJavaScriptTag(getJavaScript('EVNT_editform.js', $variables), false);
        unset($variables['CONTENTS']);
        $variables['VISIBLEPANEL'] = 'EVNT_action';
        $topscript = setJavaScriptTag(getJavaScript('dynform_top.js', $variables), false);
        $btmscript = setJavaScriptTag(getJavaScript('dynform_btm.js', $variables), false);
        //Build the form
        $modify = (UserRequest::$request['ac'] === 'MODIFY') ? true : false;
        $title = ($modify) ? dict('TOOL_EVNT_TITLE_MODIFY') : dict('TOOL_EVNT_TITLE_CREATE');
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', $ac);
        $form->addArgument('confirm', CLX_NUM_ONE);
        $form->addArgument('EVNT_id', (!empty($record['EVNT_id'])) ? $record['EVNT_id'] : '');
        $form->addArgument('EVNT_ownercode', $record['EVNT_ownercode']);
        $form->addArgument('EVNT_ownerid', uppercase($record['EVNT_ownerid']));
        $form->addArgument('EVNT_language', $record['EVNT_language']);
        $form->addArgument('EVNT_seqnum', $record['EVNT_seqnum']);
        $form->addArgument('idlist', toString($idlist));
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell($title, array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $event = new htmlSelect('EVNT_event');
        foreach ($this->events as $code => $info) {
		  if ($record['EVNT_ownercode'] <> 'UIFF' && $code === 'DBLT') {
			 continue;
		  }
		  if ($info['object'] === $record['EVNT_ownercode']) {
			 $event->addItem($info['name'] . ' (' . $info['language'] . ')', $code, ($code == $record['EVNT_event']));
		  }
        }
        $event->setProp('onchange', 'updateEventLanguage();');
        $table->addData(dict('GEN_LBL_EVENT') . ':', $event->source());
        $table->addYesNo(dict('GEN_LBL_ACTIVE'), 'EVNT_status', $record['EVNT_status'], CLX_NUM_ZERO, true);
        if (isset($msg['EVNT_action'])) {
		  $table->addCell($msg['EVNT_action'], array(CLX_PROP_COLSPAN => CLX_WIDTH_FIVE, CLX_PROP_CLASS => CLX_CLASS_ALERT));
        }
        $actions = $OBJS->objectPop('ACTN', 'EVNT_action', $record['EVNT_action'], true, $record['EVNT_ownercode']);
        $actions->setProp('onchange', 'updateForm();');
        $table->addData(dict('GEN_LBL_ACTION'), $actions->source());
        $form->addContents($table->source());
        //adding event type
        foreach ($actioncodes as $actioncode) {
		  $object = getToolObject('ACTN', $actioncode);
		  $form->addContents($object->panel($msg, $record));
        }
        // Footers
        $table2 = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $buttons = '';
        if (!empty($modify)) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        } else {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE'), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        }
        $c = new htmlButtonCancel();
        $buttons .= $c->source();
        $table2->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->addContents($table2->source());
        return $script . $topscript . $form->source() . $btmscript;
    }

    /**
     * Verify Tool Edit Forms
     * @param array $msg
     * @return bool
     */
    function checkForm(&$msg) {
        $msg = array();
		IF (!isset(UserRequest::$post['EVNT_event']))
			UserRequest::$post['EVNT_event']='';
        if (UserRequest::$post['EVNT_event'] === 'DBLT' && UserRequest::$post['EVNT_action'] <> 'DBLT') {
		  //validate list tipe
		  $msg['EVNT_action'] = dict('GEN_LBL_ACTION_DBLT_REQ');
        }
        $actionobject = getToolObject('ACTN', UserRequest::$post['EVNT_action']);
        $actionobject->checkPanel($msg);
        return count($msg) == CLX_NUM_ZERO;
    }

    /**
     * Builds confirm delete form for tool
     * @param obj $table
     * @param array $record
     */
    function confirmDeleteFill(&$table, $record) {
        $table->addCell(dict('TOOL_EVNT_TITLE_VIEW'), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_NUM_TWO));
        $table->addData(dict('GEN_LBL_OWNER') . ':', $record['EVNT_ownercode'] . ':' . uppercase($record['EVNT_ownerid']));
        $table->addData(dict('GEN_LBL_EVENT') . ':', $this->events[$record['EVNT_event']]['name']);
        $table->addData(dict('GEN_LBL_ACTION') . ':', $record['EVNT_action']);
    }

    /**
     * Creates reorder list
     * @return string
     */
    function reorderRecords() {
        $rs = ClxCommon::$db->getRecords($this->tablename, 'OWNERCODE=' . singleQuote(UserRequest::$post['ownercode']), 'OWNERID=' .singleQuote(uppercase(UserRequest::$post['ownerid'])) . ' ORDER BY SEQNUM');
        $list = array();
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
		  $list[$r['ID']] = $this->events[$r['EVENT']]['name'] . ' - ' . $r['SEQNUM'] . ' - ' . addslashes(substr(htmlentities(str_replace(CLX_CHAR_SINGLE_QUOTE, '', $r['EXTRA2'])), CLX_NUM_ZERO, CLX_NUM_TEN)) . '...';
        }
		ClxCommon::$db->freeResult( $rs );
        return $list;
    }

    /**
     * Build display form contents
     * @param array $record
     * @return string
     */
    function viewFormFill($record) {
        $status = array(dict('GEN_LBL_INACTIVE'), dict('GEN_LBL_ACTIVE'));
        // Headers
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_EVNT_TITLE_VIEW'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addData(dict('GEN_LBL_OWNER') . ':', $record['EVNT_ownercode'] . ':' . uppercase($record['EVNT_ownerid']));
        $table->addData(dict('GEN_LBL_EVENT') . ':', $this->events[$record['EVNT_event']]['name']);
        $table->addData(dict('GEN_LBL_STATUS') . ':', $status[$record['EVNT_status']]);
        $contents = $table->source();
        // Action Details
        $table2 = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $actioninstance = getToolObject('ACTN', $record['EVNT_action']);
        $actioninstance->viewForm_2($table2, $record);
        $contents .= $table2->source();
        return $contents;
    }

    /**
     * Loads current record on selected id
     *
     * @param integer $id
     * @return array
     */
    function loadToolRecord($recordid) {
        $values = ClxCommon::$db->getRecord('CLX_EVENTS', 'ID= ' . $recordid);
        $record = array();
        $record['EVNT_id'] = $recordid;
        $record['EVNT_ownercode'] = $values['OWNERCODE'];
        $record['EVNT_ownerid'] = uppercase($values['OWNERID']);
        $record['EVNT_event'] = $values['EVENT'];
        $record['EVNT_action'] = $values['ACTION'];
        $record['EVNT_status'] = $values['STATUS'];
        $record['EVNT_seqnum'] = '0';
        $record['EVNT_language'] = $this->events[$values['EVENT']]['language'];
        $actionobject = getToolObject('ACTN', $values['ACTION']);
        $actionobject->setPanel($record, $values);
        return $record;
    }

    /**
     * Update records
     *
     * @return integer
     */
    function saveRecord() {
        $record = array();
        $record['EVENT'] = UserRequest::$post['EVNT_event'];
        $record['ACTION'] = UserRequest::$post['EVNT_action'];
        $actionobject = getToolObject('ACTN', UserRequest::$post['EVNT_action']);
        $actionobject->getPanel($record, array());
        $record['STATUS'] = UserRequest::$post['EVNT_status'];
        ClxCommon::$db->updateRecord('CLX_EVENTS', $record, 'ID=' . UserRequest::$post['EVNT_id']);
		if ($record['ACTION']=='CODE') {
			$archivo=kPathToWebSite . kPathToCodeEvents . 'evtn_' . UserRequest::$post['EVNT_id'] . '.php';
			$xa=fopen($archivo,'w+');
			fwrite($xa,"<?PHP".chr(13).chr(10).$record['EXTRA2'].chr(13).chr(10)."?>");
			fclose($xa);
		}
        $owner = &getTool(UserRequest::$post['EVNT_ownercode']);
		if (UserRequest::$post['EVNT_ownercode'] === 'PRSE' ||UserRequest::$post['EVNT_ownercode'] === 'UIFP') {
		  $owner->versionIncrease('IDCODE', uppercase(UserRequest::$post['EVNT_ownerid']));
        } else {
		  $owner->versionIncrease('ID', uppercase(UserRequest::$post['EVNT_ownerid']));
        }
        $this->clearHeap();
    }

    /**
     * Creates record
     *
     * @return integer
     */
    function createRecord_() {
        $record = array();
		IF (!isset(UserRequest::$post['EVNT_event']))
			UserRequest::$post['EVNT_event']='';
        $record['OWNERCODE'] = UserRequest::$post['EVNT_ownercode'];
        $record['OWNERID'] = uppercase(UserRequest::$post['EVNT_ownerid']);
        $record['EVENT'] = UserRequest::$post['EVNT_event'];
        $record['ACTION'] = UserRequest::$post['EVNT_action'];
        $record['STATUS'] = UserRequest::$post['EVNT_status'];
        $sql = 'SELECT IFNULL(MAX(SEQNUM)+1,0) FROM ' . $this->tablename . ' WHERE OWNERCODE = ' . singleQuote(UserRequest::$post['EVNT_ownercode']) . ' AND OWNERID = ' . singleQuote(uppercase(UserRequest::$post['EVNT_ownerid']));
        $record['SEQNUM'] = ClxCommon::$db->getOne($sql);
        $actionobject = getToolObject('ACTN', UserRequest::$post['EVNT_action']);
        $actionobject->getPanel($record, array());
        $recordid = ClxCommon::$db->createRecord('CLX_EVENTS', $record, false);
        $owner = &getTool(UserRequest::$post['EVNT_ownercode']);
        if (UserRequest::$post['EVNT_ownercode'] === 'PRSE' || UserRequest::$post['EVNT_ownercode'] === 'UIFP'	 ) {
		  $owner->versionIncrease('IDCODE', uppercase(UserRequest::$post['EVNT_ownerid']));
        } else {
		  $owner->versionIncrease('ID', uppercase(UserRequest::$post['EVNT_ownerid']));
        }
        return $recordid;
    }

    /**
     * Deletes record on selected id
     *
     * @param integer $id
     */
    function deleteRecordSk($id) {
        $eventrec = ClxCommon::$db->getRecord('CLX_EVENTS', 'ID=' . $id);
        $owner = &getTool($eventrec['OWNERCODE']);
        //$owner->versionIncrease('ID', $eventrec['OWNERID']);
		$id_field = 'ID';
        if ($eventrec['OWNERCODE'] === 'PRSE' || $eventrec['OWNERCODE'] === 'UIFP') {
			$id_field = 'IDCODE';
		}
        $owner->versionIncrease($id_field, uppercase($eventrec['OWNERID']));
        ClxCommon::$db->deleteRecords('CLX_EVENTS', 'ID=' . $id);
        $this->clearHeap();
    }

    /**
     * Defines default record on initializacion
     *
     * @return array
     */
    function defaultRecord() {
        $callingpage = peekHistory();
        $record['EVNT_ownercode'] = $callingpage['ownercode'];
        $record['EVNT_ownerid'] = uppercase($callingpage['ownerid']);
        $record['EVNT_event'] = 'CREC';
        $record['EVNT_action'] = 'MAIL';
        $record['EVNT_status'] = '1';
        $record['EVNT_language'] = 'COMP';
        $record['EVNT_do_section'] = '';
        $record['EVNT_seqnum'] = '0';
        $OBJS = &getTool('OBJS');
        $actioncodes = $OBJS->objectList('ACTN');
        foreach ($actioncodes as $actioncode) {
		  $actionobject = getToolObject('ACTN', $actioncode);
		  $actionobject->defaultPanel($record);
        }
        return $record;
    }

    function getOwnerEvents($ownercode, $ownerid, $event) {

		$fetched = false;
		$findVar=kSpaceName.'event_OWNERCODE_' .$ownercode.'_OWNERID_' . $ownerid. '_EVENT_' .$event. 'STATUS=1 ORDER BY SEQNUM ASC';
			
		if(kApcTtl_9>0)$evntrecs = apc_fetchFct($findVar , $fetched);
			
		if ($fetched===false) {
			//just as normal
			$evntrecs = array();
			$events = ClxCommon::$db->getRecords('CLX_EVENTS', 'OWNERCODE=' . singleQuote($ownercode), 'OWNERID=' . singleQuote(uppercase($ownerid)), 'EVENT=' . singleQuote($event), 'STATUS=1 ORDER BY SEQNUM ASC');
			while (($eventrec = ClxCommon::$db->getNextRecord($events))) {
				$evntrecs[] = $eventrec;
				//pre($evntrecs);
			}
			ClxCommon::$db->freeResult( $events );
			if (count($evntrecs)==0)
				 $eventrecs='vacio';
			if(kApcTtl_9>0){
				apc_addFct($findVar,$evntrecs,kApcTtl_9);
			}	
		}
		else {
			if (!is_array($evntrecs))
				$evntrecs=array();  
		}
  
		
        return $evntrecs;
    }

    /**
     * Retrieve event
     * @param string $ownercode
     * @param string $ownerid
     * @param string $action
     * @return mixed <mixed, multitype:unknown >
     */
    function getOwnerEventsByAction($ownercode, $ownerid, $action) {
		$fetched = false;
		if(kApcTtl_9>0)$evntrecs = apc_fetchFct($findVar=kSpaceName . '.evnt.' . $ownercode . uppercase($ownerid) . $action, $fetched);
		if (empty($fetched)) {
			$evntrecs = array();
			$events = ClxCommon::$db->getRecords('CLX_EVENTS', 'OWNERCODE=' . singleQuote($ownercode), 'OWNERID=' . singleQuote(uppercase($ownerid)), 'ACTION=' . singleQuote($action), 'STATUS=1 ORDER BY SEQNUM ASC');
			while (($eventrec = ClxCommon::$db->getNextRecord($events))) {
				$evntrecs[] = $eventrec;
			}
			ClxCommon::$db->freeResult( $events );
			if (count($evntrecs)==0)
				$eventrecs='vacio';
			if(kApcTtl_9>0 ) {
				apc_addFct($findVar, $evntrecs, kApcTtl_9);
			}
			else {
				if (!is_array($evntrecs))
					$evntrecs=array();  
			}
		}
		
        return $evntrecs;
    }

    /**
     * Returns true event exists for owner
     * @param unknown_type $ownercode
     * @param unknown_type $ownerid
     * @param unknown_type $event
     * @return boolean
     */
    function OwnerEventExists($ownercode, $ownerid, $event) {
        return (ClxCommon::$db->oneExistsRecords('CLX_EVENTS', 'OWNERCODE=' . singleQuote($ownercode) . ' AND OWNERID=' . singleQuote(uppercase($ownerid)) . ' AND EVENT=' . singleQuote($event) . ' AND STATUS=1') == CLX_NUM_ZERO) ? false : true;
    }

    /**
     * Executes events from objects
     * @param string $ownercode
     * @param integer $ownerid
     * @param string $event
     * @param array $record
     * @param array $msg
     */
    function execute5($ownercode, $ownerid, $event, &$record, &$msg) {
        IF (ISSET($GLOBALS['ADDRDEBUGEVENTS'])) { 
		  $GLOBALS['ADDRDEBUGEVENTS']=1;
		  echo __METHOD__.' '. __FILE__ .':'.__LINE__;
		  echo 'ownerid:'; 
		  pre($ownerid); echo 'event:';pre($event); 
        }
        if (!empty($ownerid)) {
			$evntrecs = $this->getOwnerEvents($ownercode, uppercase($ownerid), $event);
		  foreach ($evntrecs as $key => $eventrec) {
			 IF (ISSET($GLOBALS['ADDRDEBUGEVENTS'])) { 
                    echo __METHOD__.' '. __FILE__ .':'.__LINE__; 
			     pre($eventrec); 
				}
			 $action = getToolObject('ACTN', $eventrec['ACTION']);
			 $action->execute3($eventrec, $record, $msg);
		  }
        }
    }

    /**
     * Executes events from objects
     * @param integer $eventid
     * @param array $record
     * @param array $msg
     * @return mixed
     */
    function execute_by_id($eventid, &$record, &$msg) {
        if (empty($eventid))
		  return false;
        $eventrec = ClxCommon::$db->getRecord('CLX_EVENTS', 'ID=' . $eventid);
        if (!empty($eventrec)) {
		  $action = getToolObject('ACTN', $eventrec['ACTION']);
		  return $action->execute3($eventrec, $record, $msg);
        } else {
		  return false;
        }
    }

    /**
     * Executes events from objects
     * @param integer $eventid
     * @param array $record
     * @param array $msg
     * @param array $parameters
     * @param bool $debug
     * @return mixed
     */
    function execute_process_event($eventid, &$record, &$msg, $parameters = array(), $debug = false) {
        if (empty($eventid))
		  return false;
        $eventrec = ClxCommon::$db->getRecord('CLX_EVENTS', 'ID=' . $eventid);
//pre($GLOBALS); PRE($parameters);
	if (!empty($eventrec)) {
	    $action = getToolObject('ACTN', $eventrec['ACTION']);
		  return $action->execute_process_event($eventrec, $record, $parameters, $msg, $debug);

        } else {
			
		  return false;
        }
    }

    /**
     * Loads the action list from actn objects
     * @return $list
     */
    function set_action_list() {
        $list = array();
        $sql = 'SELECT IDCODE FROM CLX_OBJECTS WHERE TYPE = ' . singleQuote('ACTN');
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, NUMERIC)) {
		  $list[$r[CLX_NUM_ZERO]] = 'GEN_LBL_' . $r[CLX_NUM_ZERO];
        }
		ClxCommon::$db->freeResult( $rs );
        return $list;
    }

    /**
     * Clear all events in folder to rebuild
     * @param type $path
     */
    private function deleteCodeDirectory($path) {
        if (is_dir($path)) {
		  $dir = opendir($path);
		  while ($file = readdir($dir)) {
			 if (!is_dir($path . $file) && $file !== 'index.php' && substr($file, CLX_NUM_ZERO, CLX_NUM_FOUR) === 'evtn') {
			     unlink($path . $file);
			 }
		  }
        }
    }

    /**
     * Builds events phisical library
     */
    private function compileCodeEvents() {
        $path = kPathToWebSite . kPathToCodeEvents;
        //Reset directory
        $this->deleteCodeDirectory($path);
        $rs = ClxCommon::$db->getRecords($this->tablename, 'ACTION = ' . singleQuote('CODE') . ' AND EXTRA2 IS NOT NULL');
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
		  if ($r['EXTRA3']=='GEN_REC_UTIL'){
			}
			else if ($this->events[$r['EVENT']]['language'] !== 'PHP' && $this->events[$r['EVENT']]['language'] !== 'COMP') {
				continue;
			}
			$filename = savePhpCodeEvents('evtn_' . $r['ID'], $r['EXTRA2']);
		  unset($filename);
        }
		ClxCommon::$db->freeResult( $rs );
    }

    /**
     * Clear field form heap
     * @param string $fieldname
     */
    private function clearHeap() {
        apc_clear_cacheFct('user');
    }

}
