<?php

/**
 * Tool Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: alenetdev $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2019-01-22 15:45:22 -0400 (ma., 22 ene. 2019) $
 * @version        $Rev: 10496 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  Included label
 */
class ma_mail_attachments extends clx_tool {

    var $debug = false;

    function __construct() {
        $this->code = 'MAAT';
        $this->name = 'The Clientix Webmail - Attachments';
        $this->system = 'Mail';
        $this->version = '1.0.0.0';




        $this->tablename = 'CLX_MA_MAILATTACHMENTS';
        $this->record_name_singular = 'ma_mail_mail';
        $this->record_name_plural = 'ma_mail_mails';
        $this->primary_field = 'IDNAME';
        $this->secondary_field = 'EMAIL';
    }

    function action($ac, $idlist) {
        if ($this->getAccess()->canRead()) {
            switch ($ac) {
                case 'SHOW_FILE':
                    $contents = '';
                    $this->showFile(UserRequest::$request['ff']);
                    break;
                default:
                    $contents = parent::action($ac, $idlist);
                    break;
            }
        } else {
            $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [MA_MAIL_ATTACHMENTS-CAN_READ][' . $ac . '] ' . $this->getCode(false));
        }
        return $contents;
    }

    function install() {

    }

    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['MAILID'] = array('type' => 'int(11)', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['NAME'] = array('type' => 'varchar(255)', 'null' => 'NO', 'properties' => '');
        $fields['FILENAME'] = array('type' => 'varchar(255)', 'properties' => '');
        $fields['TYPE'] = array('type' => 'varchar(64)', 'null' => 'NO', 'properties' => '');
        $fields['SIZES'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['CDATE'] = array('type' => 'datetime', 'null' => 'YES', 'properties' => '');
        $fields['USERNAME'] = array('type' => 'varchar(30)', 'null' => 'NO', 'properties' => '');
        $fields['CONTENTS'] = array('type' => 'longblob', 'properties' => '');
        $fields['DESCRIPTION'] = array('type' => 'varchar(255)', 'properties' => '');
        $fields['ENCODING'] = array('type' => 'varchar(255)', 'properties' => '');
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'MUL', 'fieldname' => 'NAME');
        $indexes[$ipf . '03'] = array('index' => 'MUL', 'fieldname' => 'USERNAME');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB',
            'fields' => $fields,
            'indexes' => $indexes);
    }

//-- M A I L
    function makelist($listingcode = null) {
        $t = time();
        $to = $this->code;
        $here = 'tool.php?to=' . $to . '&t=' . $t;
        clearHistory();
        pushHistory($this->code, 'LIST', array());
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->setProp(CLX_PROP_ID, 'MAATList');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'NOT_DEFINED');
        $selectclause = '*';
        $fromclause = $this->tablename;
        $whereclause = '';
        $sortclause = 'NAME';
        $page = new xPaging();
        $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
        $table = new htmlTable(CLX_WIDTH_SIX);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_MAAT_TITLE_LIST'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_SIX));
        $page->header($table);
        $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_NAME'), dict('GEN_LBL_TYPE'), dict('GEN_LBL_SIZE'), dict('GEN_LBL_CDATE'), dict('GEN_LBL_FILENAME')));
        $i = CLX_INIT_ZERO;
        while ($record = $page->nextRecord()) {
            $recordid = $record['ID'];
            $select = new htmlCheckbox('selector_' . $recordid, false);
            $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
            $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
            $link = new htmlAnchor($record['name'], 'javascript: openWindow("fi' . $record['ID'] . '", "window.php?to=MAAT&ac=SHOW_FILE&ff=' . $record['ID'] . '", 400, 400);');
            $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($record['name'], $record['TYPE'], xFormat::filesize($record['SIZES']), xFormat::date($record['CDATE']), $link->source()));
            $i++;
        }
        $buttons = '';
        $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_DISPLAY'), array('action' => 'DISPLAY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        if ($this->getAccess()->canDelete()) {
            $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_FILTER'), array('action' => 'FILTER', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
        $form->addContents($page->footers($table, $buttons));
        return $form->source();
    }

    function viewFormFill($record) {
        $contents = '';
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_MAAT_TITLE_VIEW') . ': ' . $record['MAAT_name'], array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addData(dict('GEN_LBL_MAILID') . ':', $record['MAAT_mailid']);
        $table->addData(dict('GEN_LBL_NAME') . ':', $record['MAAT_name']);
        $table->addData(dict('GEN_LBL_SIZE') . ':', xFormat::filesize($record['MAAT_size']));
        $table->addData(dict('GEN_LBL_TYPE') . ':', $record['MAAT_type']);
        $table->addData(dict('GEN_LBL_FILENAME') . ':', $record['MAAT_filename']);
        $table->addData(dict('GEN_LBL_DESCRIPTION') . ':', $record['MAAT_description']);
        $table->addData(dict('GEN_LBL_ENCODING') . ':', $record['MAAT_encoding']);
        //Disply File
        $table->addData(dict('GEN_LBL_FILE') . ':', $this->formatAttachment($record['MAAT_id'], $record['MAAT_name'], $record['MAAT_type']));
        $contents .= $table->source();
        return $contents;
    }

    function confirmDeleteFill(&$table, $record) {
        $table->addData(dict('GEN_LBL_NAME') . ':', $record['MAAT_name']);
        $table->addData(dict('GEN_LBL_TYPE') . ':', $record['MAAT_type']);
    }

    function formatAttachment($id, $name, $type) {
        $contents = '';
        if (!empty($id)) {
            $mimerec = ClxCommon::$db->getRecord('CLX_SD_MIMETYPES', 'IDCODE=' . singleQuote($type));
            if (empty($mimerec['HTML_TAG'])) {
                $windowname = 'window' . $id;
                $url = 'javaScript:openWindow("' . singleQuote($windowname) . '", "window.php?to=MAAT&ac=SHOW_FILE&ff=' . $id . '", 800, 600);';
                $contents = '<a href=' . singleQuote($url) . '>' . $name . '</a>';
            } else {
                $contents = $mimerec['HTML_TAG'];
                $contents = str_replace('[PATHNAME]', 'window.php?to=MAAT&ac=SHOW_FILE&ff=' . $id, $contents);
            }
        }
        return $contents;
    }

    function filterFormFill($msg, &$table) {
        if (!array_key_exists('MAAT_filename', UserRequest::$post))
            UserRequest::$post['MAAT_filename'] = '';
        $table->addField($msg, dict('GEN_LBL_FILENAME'), 'MAAT_filename', UserRequest::$post['MAAT_filename'], CLX_NUM_32, CLX_NUM_64);
    }

    function setfilter() {
        $filter = '';
        $boolean = '';
        if (!empty(UserRequest::$post['MAAT_filename'])) {
            $filter .= $boolean . ' FILENAME LIKE ' . singleQuote(UserRequest::$post['MAAT_filename'] . '%');
            $boolean = 'AND';
        }
        $_SESSION['paging']['MAATList']['criteria'] = $filter;
        $_SESSION['paging']['MAATList']['startat'] = '0';
    }
	///////////////////////////////////////////////////
	///comentado 22/01/2019 para utilizar la clx_tool
	//////////////////////////////////////////////////
    /*function clearfilter() {
        $_SESSION['paging']['MAATList']['criteria'] = '';
    }*/

    function loadToolRecord($recordid) {
        $values = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $recordid);
        $record = array();
        $record['MAAT_id'] = $values['ID'];
        $record['MAAT_mailid'] = $values['MAILID'];
        $record['MAAT_name'] = $values['name'];
        $record['MAAT_size'] = $values['SIZES'];
        $record['MAAT_type'] = $values['TYPE'];
        $record['MAAT_filename'] = $values['FILENAME'];
        $record['MAAT_description'] = $values['DESCRIPTION'];
        $record['MAAT_encoding'] = $values['ENCODING'];
        return $record;
    }

    function deleteRecordSk($id) {
        $result = ClxCommon::$db->getRecord($this->tablename, 'ID = ' . $id);
        if (strlen($result['FILENAME']) > CLX_NUM_ZERO) {
            $path = kPathToWebSite . kPathToData;
            unlink($path . $result['FILENAME']);
        }
        ClxCommon::$db->deleteRecords($this->tablename, 'ID=' . $id);
    }

    function deleteAttachments($mailid) {
        if (!empty($mailid)) {
            $results = ClxCommon::$db->getRecords($this->tablename, 'MAILID = ' . $mailid);
            while ($result = ClxCommon::$db->getNextRecord($results, ASSOCIATIVE)) {
                if (strlen($result['FILENAME']) > CLX_NUM_ZERO) {
                    $path = kPathToWebSite . kPathToData;
                    unlink($path . $result['FILENAME']);
                }
            }
			ClxCommon::$db->freeResult($results);
            ClxCommon::$db->deleteRecords($this->tablename, 'MAILID=' . $mailid);
        }
    }

    function saveFileByName($mailid, $name, $saveindatabase = true) {
        $file = new clx_file($name, 'r');
        $file->open();
        //Adding attachments here see latter
        $type = $file->getMimeType();
        $contents = $file->read();
        $path = $file->filePathInfo();
        $encoding = '';
        $description = $name;
        $file->close();
        ClxFreeReturn($this->saveFile($mailid, $name, $type, $contents, $saveindatabase, $path, $encoding, $description));
    }

    function saveFile($mailid, $name, $type, $contents, $saveindatabase = true, $path = '', $encoding = '', $description = '') {
        $cdate = xFormat::getNow('Y-m-d H:i:s');
        $username = $_SESSION['user_record']['username'];
        $mimeexists = (ClxCommon::$db->oneExistsRecords('CLX_SD_MIMETYPES', 'IDCODE=' . singleQuote($type)) > CLX_NUM_ZERO);
        if ($mimeexists === true) {
            $mimerec = ClxCommon::$db->getRecord('CLX_SD_MIMETYPES', 'IDCODE=' . singleQuote($type));
        } else {
            $mimerec = array();
            $mimerec['EXTENSION'] = '';
        }
        $filerecord = array();
        $filerecord['ID'] = '';
        $filerecord['MAILID'] = $mailid;
        $filerecord['name'] = $name;
        $filerecord['type'] = $type;
        $filerecord['SIZES'] = strlen($contents);
        $filerecord['CDATE'] = $cdate;
        $filerecord['username'] = $username;
        $filerecord['ENCODING'] = $encoding;
        $filerecord['description'] = $description;
        if ($saveindatabase === true) {
            $filerecord['FILENAME'] = 'null';
            $filerecord['CONTENTS'] = addslashes($contents);
        } else {
            $u = str_replace('.', '', str_replace(' ', '', microtime()));
            $fail = false;
            $path = kPathToWebSite . kPathToData;
            $filename = 'MAAT_' . $mailid . '_' . $u . '.' . $mimerec['EXTENSION'];
            $fd = fopen($path . $filename, 'w');
            if (empty($fd)) {
                addMessage('Cannot open file $filename', 'NOTE');
                //if Fail go to db
                $filerecord['FILENAME'] = 'null';
                $filerecord['CONTENTS'] = $contents;
                $filename = '';
                $fail = true;
            }
            // Write $somecontent to our opened file.
            if (empty($fail) && !fwrite($fd, $contents)) {
                die('Cannot write to file $filename');
                //if Fail go to db
                $filerecord['FILENAME'] = 'null';
                $filerecord['CONTENTS'] = addslashes($contents);
                $filename = '';
                $fail = true;
            }
            if (!empty($fd))
                fclose($fd);
            if (strlen($filename) > CLX_NUM_ZERO) {
                $filerecord['FILENAME'] = $filename;
                $filerecord['CONTENTS'] = 'null';
            }
        }
        $fileid = ClxCommon::$db->createRecord($this->tablename, $filerecord, false);
        return $fileid;
    }

    /**
     * Dump to file to screen
     */
    function showFile($fileid) {
        if (empty($fileid) || !is_numeric($fileid)) {
            salir(CLX_NUM_ZERO);
        }
        $filerec = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $fileid);
        if (empty($filerec)) {
            header('HTTP/1.0 404 Not found');
            salir(CLX_NUM_ZERO);
        }
        $mimerec = ClxCommon::$db->getRecord('CLX_SD_MIMETYPES', 'IDCODE=' . singleQuote($filerec['TYPE']));
        header('Content-type: ' . $mimerec['IDCODE']);
        header('Content-Disposition: inline; filename=' . $filerec['NAME']);
        header('Pragma: no-cache');
        header('Expires: 0');
        header('Cache-control: private');
        if (empty($filerec['FILENAME'])) {
            echo stripslashes($filerec['CONTENTS']);
        } elseif (strpos(lowercase($filerec['NAME']), 'http') !== false) {
            echo file_get_contents($filerec['FILENAME']);
        } else {
            echo file_get_contents(kPathToWebSite, $filerec['FILENAME']);
        }
        salir(CLX_NUM_ZERO);
    }

}
