<?php

/**
 * Tool Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: jclerma $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2017-11-03 15:09:40 -0400 (Fri, 03 Nov 2017) $
 * @version        $Rev: 9615 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  Included label
 */
class notes extends clx_tool {

    // CONSTRUCTOR //
    function __construct() {
        $this->code = 'NOTE';
        $this->name = 'Notes';
        $this->system = 'Utilities';
        $this->version = '0.2.3.0';

        $this->has_access = true;


        $this->tablename = 'CLX_NOTES';
        $this->record_name_singular = 'note';
        $this->record_name_plural = 'notes';
        $this->primary_field = 'OWNERCODE';
        $this->secondary_field = 'OWNERID';
    }

    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['OWNERCODE'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '');
        $fields['OWNERID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['CDATE'] = array('type' => 'datetime', 'null' => 'YES', 'properties' => '');
        $fields['NOTE'] = array('type' => 'mediumtext', 'properties' => '');
        $fields['USERNAME'] = array('type' => 'varchar(30)', 'null' => 'NO', 'properties' => '');
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'MUL', 'fieldname' => 'OWNERCODE');
        $indexes[$ipf . '03'] = array('index' => 'MUL', 'fieldname' => 'OWNERID');
        $indexes[$ipf . '04'] = array('index' => 'MUL', 'fieldname' => 'CDATE');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB',
		  'fields' => $fields,
		  'indexes' => $indexes);
    }

//-- N O T E S
    function action($ac, $idlist) {
        if ($this->getAccess()->canRead()) {
		  $contents = parent::action($ac, $idlist);
        } else {
		  $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [NOTES-CAN_READ][' . $ac . '] ' . $this->getCode(false));
        }
        return $contents;
    }

    function makepanel($groupcode = null, $ownerid = '') {
        $ownercode = $groupcode;
        if ($this->getAccess()->canRead()) {
		  $t = time();
		  $to = $this->code;
		  $here = 'tool.php?to=' . $to . '&t=' . $t;
		  $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
		  $form->setProp(CLX_PROP_ID, 'notelist');
		  $form->addArgument(CLX_PROP_TO, $this->code);
		  $form->addArgument('ac', 'NOT_DEFINED');
		  $form->addArgument('NTE_ownercode', $ownercode);
		  $form->addArgument('NTE_ownerid', $ownerid);
		  $selectclause = '*';
		  $fromclause = 'CLX_NOTES';
		  $whereclause = 'OWNERCODE=' . singleQuote($ownercode) . ' AND OWNERID=' . $ownerid;
		  $sortclause = 'CDATE DESC';
		  $page = new xPaging();
		  $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
		  $page->rpp = CLX_NUM_TEN_THOUSAND;

		  $table = new htmlTable(CLX_WIDTH_FOUR);
		  $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
		  $page->header($table);
		  $table->addCells(CLX_CLASS_HEADERS, array('Select', 'Date', 'User', 'Note'));
		  $i = CLX_INIT_ZERO;
		  while ($record = $page->nextRecord()) {
			 $recordid = $record['ID'];
			 $select = new htmlCheckbox('selector_' . $recordid, false);
			 $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
			 $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
			 $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array(xFormat::datetime($record['CDATE'], 'M, d h:i:s'), $record['USERNAME'], $record['NOTE']));
		  }
		  $buttons = '';
		  if ($this->getAccess()->canDelete()) {
			 $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
		  }
		  if ($this->getAccess()->canDisplay()) {
			 $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE'), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
		  }
		  $form->addContents($page->footers($table, $buttons));
		  return buildPanelInterface(dict('TOOL_NOTE_TITLE_LIST'), $ownercode . '_NOTES', $form->source(), CLX_NUM_MINUS_1);
        } else {
		  return CLX_CHAR_EMPTY_STRING;
        }
    }

    function editForm($ac, $idlist, $record, $msg) {
        $title = dict('TOOL_NOTE_TITLE_CREATE');
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', $ac);
        $form->addArgument('confirm', '1');
        $form->addArgument('NTE_id', $record['NTE_id']);
        $form->addArgument('NTE_ownercode', $record['NTE_ownercode']);
        $form->addArgument('NTE_ownerid', $record['NTE_ownerid']);
        $form->addArgument('idlist', toString($idlist));
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell($title, array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addText($msg, 'Note:', 'NTE_note', $record['NTE_note'], CLX_NUM_FOUR, CLX_NUM_48);
        $form->addContents($table->source() . '<br>');
        $buttons = '';
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE'), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        $c = new htmlButtonCancel();
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->addContents($table->source());
        return $form->source();
    }

    function checkForm(&$msg) {
        $msg = array();
        if (UserRequest::$post['NTE_note'] === '') {
		  $msg['NTE_note'] = 'You have to write a note.';
        }
        return count($msg) == CLX_NUM_ZERO;
    }

    function confirmDeleteFill(&$table, $record) {
        $table->addData('Note:', $record['NTE_note']);
    }

    function loadToolRecord($recordid) {
        $values = ClxCommon::$db->getRecord('CLX_NOTES', 'ID= ' . $recordid);
        $record = array();
        $record['NTE_id'] = $recordid;
        $record['NTE_ownercode'] = $values['OWNERCODE'];
        $record['NTE_ownerid'] = $values['OWNERID'];
        $record['NTE_cdate'] = $values['CDATE'];
        $record['NTE_note'] = $values['NOTE'];
        $record['NTE_username'] = $values['USERNAME'];
        return $record;
    }

    function createRecord_() {
        $d = xFormat::getNow('Y-m-d H:i:s');
        $record = array();
        $record['OWNERCODE'] = UserRequest::$post['NTE_ownercode'];
        $record['OWNERID'] = UserRequest::$post['NTE_ownerid'];
        $record['CDATE'] = $d;
        $record['NOTE'] = UserRequest::$post['NTE_note'];
        $record['USERNAME'] = $_SESSION['user_record']['username'];
        $recordid = ClxCommon::$db->createRecord('CLX_NOTES', $record, false);
        return $recordid;
    }

    function defaultRecord() {
        $d = xFormat::getNow('Y-m-d H:i:s');
        $record = array();
        $record['NTE_id'] = CLX_INIT_ZERO;
        $record['NTE_ownercode'] = UserRequest::$post['NTE_ownercode'];
        $record['NTE_ownerid'] = UserRequest::$post['NTE_ownerid'];
        $record['NTE_cdate'] = $d;
        $record['NTE_note'] = '';
        $record['NTE_username'] = $_SESSION['user_record']['username'];
        return $record;
    }

    function deleteRecordSk($id) {
        ClxCommon::$db->deleteRecords('CLX_NOTES', 'ID=' . $id);
    }

}
