<?php

/**
 * Process Engine Manager
 *
 * @package		Clientix.Tool.Process
 * @abstract		Process Engine Manager
 * @author       $Author: jclerma $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2017-11-03 15:09:40 -0400 (Fri, 03 Nov 2017) $
 * @version    	$Rev: 9615 $
 *
 */
class pr_processes extends clx_tool {

    var $schedulecode = '';       //Schedule ID Code
    var $processcode = '';       //Process ID Code
    var $startStamp = '';       //Start Process Timestamp
    var $batch_pid = CLX_INIT_ZERO; //Batch Process ID
    var $runid = CLX_INIT_ZERO; //Run Id
    private $stop = false;    //set to true to stop process
    var $executedby = 'system'; //system
    var $is_debug = false;
    var $is_display = false;    //Set true to have Graphical output during Process
    var $is_html = false;    //Html Format on output
    var $openHtml;
    var $log_file = false;
    var $url = '';
    var $message = '';
    var $parameters = array(); //Parameters Records

    /**
     * Constructor
     */

    function __construct() {//$_SESSION["maxdebug"]=4000000;$_SESSION['_dEBUG']=99;
        $this->code = 'PRPR';
        $this->name = 'Process Manager';
        $this->system = 'Process';
        $this->version = '5.5.0.1';

        $this->has_access = true;
        $this->has_package = true;

        $this->tablename = 'CLX_PR_PROCESS';
        $this->record_name_singular = 'GEN_LBL_PROCESS';
        $this->record_name_plural = 'GEN_LBL_PROCESSES';
        $this->primary_key = array('IDCODE');
        $this->primary_field = 'IDCODE';
        $this->secondary_field = 'IDNAME';

        $this->editform = array(CLX_CLASS_TITLE => 'TOOL_PRPR_TITLE_CREATE',
		  'panels' => array(
			 array('type' => 'normal',
			     'entries' => array(
			         array('label' => 'GEN_LBL_CODE',
					   'fields' => array(
						  array('name' => 'idcode',
						      'type' => 'string',
						      'default' => '',
						      'size' => '16',
						      'displaysize' => '16',
						      'dontmodify' => '1',
						      'validations' => array(
								array('type' => 'sqlobject',
								    'required' => '1'
								), array('type' => 'unique',
								    'fieldname' => 'idcode',
								    'tablename' => $this->tablename)
						      )
						  )
					   )
			         ), array('label' => 'GEN_LBL_NAME',
					   'fields' => array(
						  array('name' => 'idname',
						      'type' => 'string',
						      'default' => '',
						      'size' => '64',
						      'displaysize' => '48',
						      'validations' => array(
								array('type' => 'string',
								    'required' => '1'
								)
						      )
						  )
					   )
			         ), array('label' => 'GEN_LBL_STATUS',
					   'fields' => array(
						  array('name' => 'status',
						      'type' => CLX_CLASS_LIST,
						      'default' => '1',
						      'options' => $this->status)
					   )
			         ), array('label' => 'GEN_LBL_DESCRIPTION',
					   'fields' => array(
						  array('name' => 'description', 'type' => 'text', 'default' => '', 'cols' => '48', 'rows' => '4'))
			         ), array('label' => 'GEN_LBL_PRIORITY_SEQNUM',
					   'fields' => array(
						  array('name' => 'priority_seqnum',
						      'type' => 'string', 'default' => CLX_NUM_HUNDRED, 'size' => '16', 'displaysize' => '16', 'validations' => array(
								array('type' => 'integer',
								    'required' => '1'
								)
						      )
						  )
					   )
			         ), array('label' => 'GEN_LBL_MODULECODE',
					   'fields' => array(
						  array('name' => 'modulecode', 'type' => 'lookup', 'default' => '', 'tool' => 'DBMO', 'validations' => array(
								array('type' => 'sqlobject', 'required' => '1'), array('type' => 'lookup',
								    'fieldname' => 'idcode',
								    'tablename' => 'CLX_DB_MODULES')
						      )
						  )
					   )
			         )
			     )
			 ), array('type' => 'normal',
			     CLX_CLASS_TITLE => 'GEN_LBL_RECORD_INFO',
			     'entries' => array(
			         array('label' => 'GEN_LBL_VERSION',
					   'fields' => array(
						  array('name' => 'version', 'type' => 'string', 'default' => '1.0.0', 'size' => '32', 'displaysize' => '16',
						      'validations' => array(
								array('type' => 'string', 'required' => '1'))
						  )
					   )
			         ), array('label' => 'GEN_LBL_VERSION_COUNTER',
					   'fields' => array(
						  array('name' => 'version_counter', 'type' => 'string', 'default' => '1.0.0', 'readonly' => true, 'size' => '32', 'displaysize' => '16',
						      'validations' => array()
						  )
					   )
			         )
			     )
			 )
		  )
        );
        $this->viewform = $this->editform;
        $this->toollist = array('name' => $this->tablename,
		  CLX_CLASS_TITLE => $this->record_name_plural,
		  'fields' => array(array('name' => 'ID', 'type' => 'number'),
			 array('name' => 'IDCODE', 'type' => 'string'),
			 array('name' => 'IDNAME', 'type' => 'string'),
			 array('name' => 'DESCRIPTION', 'type' => 'string'),
			 array('name' => 'MODULECODE', 'type' => 'string'),
			 array('name' => 'STATUS', 'type' => CLX_CLASS_LIST, 'label' => 'GEN_LBL_STATUS', 'options' => $this->status, 'default' => CLX_NUM_ONE),
			 array('name' => 'PRIORITY_SEQNUM', 'type' => 'number'),
			 array('name' => 'VERSION', 'type' => 'fieldgroup', 'fields' => array(array('name' => 'VERSION', 'type' => 'string', 'sufix' => '.'), array('name' => 'VERSION_COUNTER', 'type' => 'number')))
		  )
        );

        $this->quickfilterform = array(CLX_CLASS_TITLE => 'TOOL_PRPR_TITLE_CREATE',
		  'panels' => array(
			 array('type' => 'normal',
			     'entries' => array(
			         array('label' => 'GEN_LBL_CODE',
					   'fields' => array(
						  array('name' => 'idcode',
						      'type' => 'string',
						      'default' => '',
						      'size' => '16',
						      'displaysize' => '16'
						  )
					   )
			         ), array('label' => 'GEN_LBL_NAME',
					   'fields' => array(
						  array('name' => 'idname',
						      'type' => 'string',
						      'default' => '',
						      'size' => '64'
						  )
					   )
			         ), array('label' => 'GEN_LBL_MODULECODE',
					   'fields' => array(
						  array('name' => 'modulecode',
						      'type' => 'lookup',
						      'default' => '',
						      'tool' => 'DBMO'
						  )
					   )
			         )
			     )
			 )
		  )
        );

        if (empty($_SESSION)) {
			if (!isset($_SESSION['user_record']))
				$_SESSION['user_record']=array();
		  $_SESSION['user_record']['username'] = 'system';
        }
        $this->executedby = $_SESSION['user_record']['username'];
        $this->batch_pid = getmypid() . '.' . getmyinode();
		IF (ISSET($GLOBALS['ADDRDEBUGPRPROCESS']) && (empty($_SERVER['REMOTE_ADDR']) || $GLOBALS['ADDRDEBUGPRPROCESS']==$_SERVER['REMOTE_ADDR'])) {
			$this->debug=true;
		}
    }

    /**
     * Tool Install Method
     */
    function install() {
        $UIMI = getTool('UIMI');
        $UIMI->add($this->code, 'MENU_TOOLS:MENU_PROCESS:MENU_' . $this->code, 'TOOL', $this->code, 'ac=LIST');
    }

    /**
     * Tool Table Structure
     * @return array
     */
    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['IDCODE'] = array('type' => 'varchar(32)', 'null' => 'NO', 'properties' => '');
        $fields['IDNAME'] = array('type' => 'varchar(64)', 'null' => 'NO', 'properties' => '');
        $fields['DESCRIPTION'] = array('type' => 'mediumtext', 'properties' => '');
        $fields['MODULECODE'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '');
        $fields['STATUS'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'default' => '1', 'properties' => '');
        $fields['PRIORITY_SEQNUM'] = array('type' => 'tinyint(4) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['PARAMETERS_TABLE'] = array('type' => 'varchar(4)', 'null' => 'YES', 'properties' => '');
        $fields['VERSION'] = array('type' => 'varchar(32)', 'null' => 'NO', 'properties' => '', 'default' => '1.0');
        $fields['VERSION_COUNTER'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['CDATE'] = array('type' => 'datetime', 'null' => 'YES', 'properties' => '');
        $fields['MDATE'] = array('type' => 'datetime', 'null' => 'YES', 'properties' => '');
        $fields['CREATEDBY'] = array('type' => 'varchar(30)', 'null' => 'NO', 'properties' => '');
        $fields['MODIFIEDBY'] = array('type' => 'varchar(30)', 'null' => 'NO', 'properties' => '');
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'UNI', 'fieldname' => 'IDCODE');
        $indexes[$ipf . '04'] = array('index' => 'MUL', 'fieldname' => 'MODULECODE');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB',
		  'fields' => $fields,
		  'indexes' => $indexes);
    }

    //-- M E S A G E S
    /**
     * Creates an xml object from record
     * @param array $record
     * @return obj
     */
    function pack($record) {
        $o = $this->packRecord($record);
        $objs = new XMLObject($this->code . '_OBJECTS');
        $this->packsub($objs, 'PRSE', 'SELECT * FROM CLX_PR_SECTIONS WHERE PROCESSCODE =' . singleQuote($record['IDCODE']));
        $this->packsub($objs, 'PRPM', 'SELECT * FROM CLX_PR_PARAMETERS WHERE PROCESSCODE =' . singleQuote($record['IDCODE']));
        $o->addObject($objs);
        return $o;
    }

    /**
     * Unpack Record into Metadata
     * @param obj $o
     * @return array
     */
    function unpack($parameters) {
        $parameters = array_values(array_merge(array('package' => null), $parameters));
        list($package) = $parameters;
        $record = $this->unpackRecord($package);
        $exists = (ClxCommon::$db->oneExistsRecords($this->tablename, 'UPPER(IDCODE)=' . singleQuote(uppercase($record['IDCODE']))));
        if (!empty($exists)) {
		  unset($record['ID']);
		  $id = ClxCommon::$db->updateRecord($this->tablename, $record, 'UPPER(IDCODE)=' . singleQuote(uppercase($record['IDCODE'])));
		  $record['ID'] = $id;
        } else {
		  $id = ClxCommon::$db->createRecord($this->tablename, $record, false);
        }
        ClxCommon::$db->deleteRecords('CLX_PR_PARAMETERS', 'PROCESSCODE=' . singleQuote($record['IDCODE']));
        if ($package->hasSubObject($this->code . '_OBJECTS')) {
		  $objs = $package->getObjectXml($this->code . '_OBJECTS');
		  foreach ($objs->objects as $o) {
			 $toolcode = $o->tag;
			 $tool = getTool($toolcode);
			 $tool->unpack(array('package' => $o, 'id' => $id));
		  }
        }
    }

    /**
     * Tool Action
     * @param string $ac Action code
     * @param string $idlist Record(s) affected by action
     * @return string
     */
    function action($ac, $idlist) {
        if ($this->getAccess()->canRead()) {
		  switch ($ac) {
			 case 'LIST':
			     if (!empty(UserRequest::$get['menuid'])) {
			         if (!empty(UserRequest::$request['mc'])) {//Adding parameter to the filter if mc is set on menu to display only mc.
					   $_SESSION['paging']['PRPRList']['mc'] = UserRequest::$request['mc'];
			         } else {
					   unset($_SESSION['paging']['PRPRList']['mc']);
			         }
			     }
			     if (!empty($_SESSION['paging']['PRPRList']['mc'])) {
			         $_SESSION['paging']['PRPRList']['criteria'] = ' UPPER(MODULECODE) LIKE ' . singleQuote(uppercase($_SESSION['paging']['PRPRList']['mc']) . '%');
			         if (empty($_SESSION['paging']['PRPRList']['sortby']))
					   $_SESSION['paging']['PRPRList']['sortby'] = CLX_INIT_ZERO;
			         if (empty($_SESSION['paging']['PRPRList']['startat']))
					   $_SESSION['paging']['PRPRList']['startat'] = CLX_INIT_ZERO;
			     }
			     $contents = parent::action($ac, $idlist);
			     break;
			 default:
			     $contents = parent::action($ac, $idlist);
			     break;
		  }
        } else {
		  $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [PR_PROCESSES-CAN_READ][' . $ac . '] ' . $this->getCode(false));
        }
        return $contents;
    }

    /**
     * Builds Tool Search form
     * @return string
     */
    function searchForm() {
        $contents = '';
        $msg = array();
        // Search Form
        $form = new htmlForm(CLX_PROP_ACTION, UserRequest::$server['PHP_SELF']);
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'SEARCH');
        $form->addArgument('confirm', '1');
        $form->addArgument(CLX_PROP_TARGET, UserRequest::$request[CLX_PROP_TARGET]);
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_PRPR_TITLE_SEARCH'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addField($msg, dict('GEN_LBL_CODE') . ':', 'PRPR_idcode', (array_key_exists('PRPR_idcode', UserRequest::$post)) ? UserRequest::$post['PRPR_idcode'] : '', CLX_NUM_16, CLX_NUM_16);
        $table->addField($msg, dict('GEN_LBL_NAME') . ':', 'PRPR_idname', (array_key_exists('PRPR_idname', UserRequest::$post)) ? UserRequest::$post['PRPR_idname'] : '', CLX_NUM_32, CLX_NUM_64);
        $buttons = htmlButton::getButton('submit', dict('GEN_LBL_SEARCH'), array('action' => 'SEARCH', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        $c = new htmlButtonCloseWindow(dict('GEN_LBL_CANCEL'));
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->setContents($table->source());
        $contents .= $form->source();
        // Search Procedure
        if (isset(UserRequest::$post['confirm'])) {
		  $table = new htmlTable(CLX_WIDTH_TWO);
		  $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
		  $criteria = '';
		  if (UserRequest::$post['PRPR_idcode'] <> '') {
			 $criteria .= 'UPPER(IDCODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['PRPR_idcode']) . '%');
		  }
		  if (UserRequest::$post['PRPR_idname'] <> '') {
			 $criteria .= 'UPPER(IDNAME) LIKE ' . singleQuote(uppercase(clx_change_especial_char_html (UserRequest::$post['PRPR_idname'])) . '%');
		  }
            $qty = ClxCommon::$db->oneExistsRecords($this->tablename, $criteria);
		  if ($qty == CLX_NUM_ZERO) {
			 $table->addCell(dict('TOOL_GEN_ERR_NOMATCH'), array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
		  } else {
			 $table->addCell(dict('TOOL_GEN_MSG_FOUNDRECORDS'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
			 $records = ClxCommon::$db->getRecords($this->tablename, $criteria);
			 $i = CLX_INIT_ZERO;
			 while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
			     $js = 'javascript:setLookupValue(' . quote(UserRequest::$request[CLX_PROP_TARGET]) . ', ' . quote($record['IDCODE']) . ');';
			     $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array('<page t=pr_processes></page><a href=' . singleQuote($js) . '>' . $record['IDCODE'] . '</a>', $record['IDNAME']));
			     $i++;
			 }
			 ClxCommon::$db->freeResult( $records );
		  }
		  $contents .= $table->source();
        }
        return $contents;
    }

    /**
     * Builds confirm delete form for tool
     * @param obj $table
     * @param array $record
     */
    function confirmDeleteFill(&$table, $record) {
        $table->addData(dict('GEN_LBL_CODE') . ':', $record['PRPR_idcode']);
        $table->addData(dict('GEN_LBL_NAME') . ':', $record['PRPR_idname']);
    }

    /**
     * Build display form contents
     * @param array $record
     * @return string
     */
    function viewFormFill($record) {
        addHistory('processcode', $record['PRPR_idcode']);
        // Elements
        $PRSE = &getTool('PRSE');
        $contents = $PRSE->makepanel($record['PRPR_idcode']);
        $PRPM = &getTool('PRPM');
        $contents .= $PRPM->makepanel($record['PRPR_idcode']);
        return $contents;
    }

    function checkForm(&$msg) {
        $msg = array();
        parent::checkForm($msg);
        return count($msg) == CLX_NUM_ZERO;
    }

    /**
     * Deletes record on selected id
     *
     * @param integer $id
     */
    function deleteRecordSk($id) {
        $idcode = ClxCommon::$db->getField($this->tablename, 'IDCODE', 'ID=' . $id);
        //Process
        $PRSE = getTool('PRSE');
        $rs = ClxCommon::$db->getRecords($PRSE->tablename, 'PROCESSCODE=' . singleQuote($idcode));
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
		  $PRSE->deleteRecordSk($r['ID']);
        }
		ClxCommon::$db->freeResult( $rs );
        //Paramters
        $PRPM = getTool('PRPM');
        $rs = ClxCommon::$db->getRecords($PRPM->tablename, 'PROCESSCODE=' . singleQuote($idcode));
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
		  $PRPM->deleteRecordSk($r['ID']);
        }
		ClxCommon::$db->freeResult( $rs );
        //Schedules
        $PRSC = getTool('PRSC');
        $rs = ClxCommon::$db->getRecords($PRPM->tablename, 'PROCESSCODE=' . singleQuote($idcode));
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
		  $PRSC->deleteRecordSk($r['ID']);
        }
		ClxCommon::$db->freeResult( $rs );
        ClxCommon::$db->deleteRecords($this->tablename, 'ID=' . $id);
    }

	function duplicateRecord() {
		$originalid = UserRequest::$post ['PRPR_id'];
		$originalrec = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $originalid);
		$originalcode = $originalrec ['IDCODE'];
		//PRE(UserRequest::$post);
		$recordnew=array( 
			'VERSION_COUNTER'	=>	 UserRequest::$post['PRPR_version_counter'],	
			'IDCODE'			=>	 UserRequest::$post['PRPR_idcode'] ,	
			'IDNAME'			=>	 UserRequest::$post['PRPR_idname'] ,	
			'STATUS'			=>	 UserRequest::$post['PRPR_status'] ,	
			'DESCRIPTION'		=>	 UserRequest::$post['PRPR_description'] ,	
			'PRIORITY_SEQNUM'	=>	 UserRequest::$post['PRPR_priority_seqnum'], 	
			'MODULECODE'		=>	 UserRequest::$post['PRPR_modulecode'],
			'CREATEDBY'			=>   $_SESSION['user_record']['username'],
			'MODIFIEDBY'		=> 	 $_SESSION['user_record']['username']

			
		);
		//pre($recordnew);

		$id = ClxCommon::$db->createRecord($this->tablename,$recordnew,false);
        $code = UserRequest::$post ['PRPR_idcode'];
		
		$records = ClxCommon::$db->getRecords('CLX_PR_PARAMETERS', 'PROCESSCODE=' . singleQuote($originalcode));
        while ($record = ClxCommon::$db->getNextRecord($records)) {
		  unset($record ['ID']);
		  $record ['PROCESSCODE'] = $code;
		  ClxCommon::$db->createRecord('CLX_PR_PARAMETERS', $record, false);
        }
		ClxCommon::$db->freeResult( $records );
		$records = ClxCommon::$db->getRecords('CLX_PR_SECTIONS', 'PROCESSCODE=' . singleQuote($originalcode));
        while ($record = ClxCommon::$db->getNextRecord($records)) {
		  $originalid2=$record ['ID'];
			unset($record ['ID']);
		  $record ['PROCESSCODE'] = $code;
			$x=rand(1,10000);
			$oldidcode=$record['IDCODE'];
			$record['IDCODE']=$record['IDCODE'].'_'.$x;
			$record['IDNAME']=$record['IDNAME']. '_'.$x;
		  $newid=ClxCommon::$db->createRecord('CLX_PR_SECTIONS', $record, false);
		//	PRE(' AND OWNERID=' . singleQuote($originalid2));
			$records2 = ClxCommon::$db->getRecords('CLX_EVENTS', 'OWNERCODE=' . singleQuote('PRSE') . ' AND OWNERID=' . singleQuote($oldidcode));
		  while ($record2 = ClxCommon::$db->getNextRecord($records2)) {
					unset($record2 ['ID']);
			     $record2 ['OWNERID'] = $record['IDCODE'];
			     ClxCommon::$db->createRecord('CLX_EVENTS', $record2, false);
		  }
		  ClxCommon::$db->freeResult( $records2 );
			
        }
		ClxCommon::$db->freeResult( $records );
        
    }


    /**
     * Prepares tool default record
     * @return array
     */
    function defaultRecord() {
        $record = parent::defaultRecord();
        $record['PRPR_version'] = '1.0.0';
        $record['PRPR_status'] = '1';
        $record['PRPR_priority_seqnum'] = CLX_NUM_HUNDRED;
        return $record;
    }

    // PROCESS EXECUTION METHODS

    /**
     * Set btach process
     */
    function setPid($pid) {
        $this->batch_pid = $pid;
    }

    /**
     * Execute Process
     * @param string $processcode
     * @param string $schedulecode
     * @param bool $display
     * @param bool $debug
     * @param bool $html
     */
    function execute_process($processcode, $schedulecode, $display = false, $debug = false, $html = false) {
        $this->openHtml = CLX_INIT_ZERO;
        $this->is_html = $html;
        $this->is_debug = $debug;
        $this->is_display = $display;
        $this->processcode = $processcode;
        $this->schedulecode = $schedulecode;
        $this->now = getNow('Y-m-d H:i:s');
        $this->status = 'RUN';
        $this->runid = CLX_INIT_ZERO;
        $this->parameters['PROCESSCODE'] = $this->processcode;

        // Verify if ob_start() has been already called, otherwise call it now to activate process log messages in step log message output
        if (!ob_get_level()) 
		  ob_start();
        
        //create log
        $this->logProcess($processcode, $this->status, dict('BASE_PROC_TTL_RUN'), CLX_CLASS_TITLE,'1');
        $this->logProcess($processcode, $this->status, dict('GEN_LBL_STATUS') . ' ' . $schedulecode, CLX_CLASS_SUBTITLE);
        $this->logProcess($processcode, $this->status, dict('BASE_PROC_LBL_START', $processcode) . ':' . $this->now, 'ok');

        //Get Process
        $procRec = ClxCommon::$db->getRecord($this->tablename, 'IDCODE=' . singleQuote($processcode));

        //Eval Process Status
        if (empty($procRec['STATUS'])) {

		  $this->logProcess($processcode, 'ERRO', dict('BASE_PROC_ERR_NOTACTIVE', $processcode), 'fail');
		  $this->endProcess();
        } else {

		  //Get Process Sections
            $qty = ClxCommon::$db->oneExistsRecords('CLX_PR_SECTIONS', 'PROCESSCODE=' . singleQuote($procRec['IDCODE']) . ' AND IS_MAINSECTION = 1 AND STATUS = 1');
		  if ($qty === CLX_NUM_ZERO) {

			 $this->logProcess($processcode, 'ERRO', dict('BASE_PROC_ERR_NOMAIN', $processcode), 'fail');
			 $this->endProcess();
		  } else {

			 //Load Parameters
				if (!empty($this->parameters_agenda)) 
					$this->parameters =$this->parameters_agenda;
				else	
			 $this->parameters = $this->loadParameters($schedulecode);
			 $rs = ClxCommon::$db->getRecords('CLX_PR_SECTIONS', 'STATUS = 1 AND PROCESSCODE=' . singleQuote($procRec['IDCODE']) . ' AND IS_MAINSECTION = 1');
			 while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
			     if ($this->getStop())
			         break;
					$r2=$r;
					
					if (isset($_SESSION['user_record'])) {
						foreach ($_SESSION['user_record'] as $key=>$value ) {
							if (!isset($r2[uppercase($key)]))
								$r2[uppercase($key)]=$value;
						}
					}
					ClxFreeReturn($this->execSection($r['IDCODE'],$r2));
			 }
			 ClxCommon::$db->freeResult( $rs );
		  }
        }
		if ( !empty($_SESSION["_dEBUG"]) && $_SESSION["_dEBUG"]==99 ) {$posx=0;$pos=1610;$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");fwrite($tta,chr(13).chr(10)."END PROCESS STATUS: ".$this->status.chr(13).chr(10));$te=debug_backtrace(); for ($t=0;$t<12;$t++) {if (isset($te[$t]["file"])){if ($t==$posx)fwrite($tta,chr(13).chr(10).$pos."_".$te[$posx]["function"]."_db_import-hora:".date("h:i:s d/m/Y").chr(13).chr(10));fwrite($tta,substr($te[$t]["file"].str_repeat(" ",80),0,80).chr(9).$te[$t]["line"].chr(9).$te[$t]["function"].chr(13).chr(10));}}fclose($tta);}

        if ($this->status <> 'ERRO') {
		  if ($this->hasExceptions()) {
			 $this->status = 'ENDR';
		  } else {
			 $this->status = 'END';
		  }
        }

        $this->logProcess($processcode, $this->status, dict('BASE_PROC_LBL_END', $processcode) . ': ' . getNow('Y-m-d H:i:s'), 'ok');

        return ($this->status <> 'ERRO') ? true : false;
    }

    /**
     * Executes Section
     * @param string $secCode
     * @param array $record
     */
    function execSection($secCode, $record = array()) {
       //End if stop
	   	$inittime=microtime(true);
		if($this->debug==true)
			echo 'ejecutando seccion';
		
        if ($this->getStop()) {
		  return false;
        }
		$original_record=$record;
		If ( isset($_SESSION["_dEBUGX"]) and $_SESSION["_dEBUGX"]==99) {
			$tta=fopen(kPathToWebSite."/".kPathToTemp."/process_".(!EMPTY($original_record['PROCESSCODE'])?$original_record['PROCESSCODE']:'')."_executions.txt","a+");
			fwrite($tta,chr(13).chr(10).'1105- hora:'.date( 'h:i:s d/m/Y').chr(13).chr(10));	
			fwrite($tta,chr(13).chr(10).'ExecSection init '.$secCode.chr(13).chr(10));
			fwrite($tta,chr(13).chr(10)."record: ".print_r($record,true).chr(13).chr(10));
			fclose($tta);
		}
        //Init return parameters
        if (empty($record)) {
		  $record = array();
        }

        //Get Section Record
        $sectionRec = ClxCommon::$db->getRecord('CLX_PR_SECTIONS', 'IDCODE=' . singleQuote($secCode));
        if ($sectionRec['STATUS'] <> CLX_NUM_ONE) {
		  $this->logProcess($this->processcode, 'RUN', dict('BASE_PROC_ERR_SECNOTACTIVE') . ':' . $sectionRec['IDCODE'], 'fail');
		  $this->endProcess();
		  return false;
        } else {
		  $this->logProcess($this->processcode, 'RUN', dict('BASE_PROC_LBL_SECSTART', $sectionRec['IDCODE']), 'ok');
        }

        $eventrecs = ClxCommon::$db->getRecords('CLX_EVENTS', 'STATUS = 1 AND OWNERID=' . singleQuote($sectionRec['IDCODE']) . ' AND OWNERCODE = ' . singleQuote('PRSE') . ' ORDER BY SEQNUM');
        while ($eventrec = ClxCommon::$db->getNextRecord($eventrecs)) {

		  ob_flush();

		  if ($this->getStop()) {
			 break;
		  }

		  $stepid = $this->logStep($eventrec, 'RUN', $this->message, null, $secCode);
		  $this->logProcess($this->processcode, 'RUN', dict('BASE_PROC_LBL_STEPSTART') . '. ' . dict('GEN_LBL_SEQNUM') . ': ' . $eventrec['SEQNUM'] . ', ' . dict('GEN_LBL_TYPE') . ': ' . $eventrec['ACTION'] . ',' . dict('GEN_LBL_DO_SECTION_TYPE') . ':' . $eventrec['DO_SECTION_TYPE'], 'ok');

		  //Build Code and variables
		  $EVNT = &getTool('EVNT');
		  if ($eventrec['ACTION'] === 'SQL') {//try sql first since only with flow control
			 //check iteration
				IF ($this->debug==true){ echo '---------->EVENTO SQL'; }
			         
			 switch ($eventrec['DO_SECTION_TYPE']) {
			     case 'DS'://Do Select
						IF ($this->debug==true){ echo '---------->EJECUTANDO DS'; }
			         $rs = $EVNT->execute_process_event($eventrec['ID'], $record, $this->message, $this->parameters, $this->is_debug);
			         if (!empty($this->is_debug)) {
					   $this->logProcess($this->processcode, 'RUN', dict('GEN_LBL_SOURCECODE'), CLX_CLASS_DATA);
			         }

			         if (ClxCommon::$db->errorNum() == CLX_NUM_ZERO) {
					   if (ClxCommon::$db->isSelect($rs)) {
						  while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
								If ( isset($_SESSION["_dEBUGX"]) and $_SESSION["_dEBUGX"]==99) {
									$tta=fopen(kPathToWebSite."/".kPathToTemp."/process_".(!EMPTY($original_record['PROCESSCODE'])?$original_record['PROCESSCODE']:'')."_executions.txt","a+");
									fwrite($tta,chr(13).chr(10).'Internal ExecSection init 1, id event: '.print_r($eventrec,true).chr(13).chr(10));
									fwrite($tta,chr(13).chr(10).'Tiempo section:'. (microtime(true)-$inittime).chr(13).chr(10));
									fclose($tta);
								}
						      $record = $this->execSection($eventrec['DO_SECTION'], $r);
								If ( isset($_SESSION["_dEBUGX"]) and $_SESSION["_dEBUGX"]==99) {
									$tta=fopen(kPathToWebSite."/".kPathToTemp."/process_".(!EMPTY($original_record['PROCESSCODE'])?$original_record['PROCESSCODE']:'')."_executions.txt","a+");
									fwrite($tta,chr(13).chr(10).'Internal ExecSection end 1, id event: '.print_r($eventrec,true).chr(13).chr(10));
									fwrite($tta,chr(13).chr(10).'Tiempo section:'. (microtime(true)-$inittime).chr(13).chr(10));
									fclose($tta);
								}	
						  }
						  ClxCommon::$db->freeResult( $rs );
					   }
			         } else {
					   //updateStepLog status errro
					   $this->logProcess($this->processcode, 'ERRO', ClxCommon::$db->errorMsg(), 'fail');
					   $this->endProcess();
			         }

			         break;
			     case 'DW':
						IF ($this->debug==true){ echo '---------->EJECUTANDO DW'; }
			         $flag = true;
			         while ($flag) {
							If ( isset($_SESSION["_dEBUGX"]) and $_SESSION["_dEBUGX"]==99) {
								IF (ISSET($original_record['PROCESSCODE'])){
									$tta=fopen(kPathToWebSite."/".kPathToTemp."/process_".$original_record['PROCESSCODE']."_executions.txt","a+");
								}
								else {
									$tta=fopen(kPathToWebSite."/".kPathToTemp."/process_"."_executions.txt","a+");
				
								}
								fwrite($tta,chr(13).chr(10).'Internal ExecSection start 2, id event:  '.print_r($eventrec,true).chr(13).chr(10));
								fwrite($tta,chr(13).chr(10).'Tiempo section:'. (microtime(true)-$inittime).chr(13).chr(10));
								fclose($tta);
							}	
					   $rs = $EVNT->execute_process_event($eventrec['ID'], $record, $this->message, $this->parameters, $this->is_debug);
							If ( isset($_SESSION["_dEBUGX"]) and $_SESSION["_dEBUGX"]==99) {
								$tta=fopen(kPathToWebSite."/".kPathToTemp."/process_".(!EMPTY($original_record['PROCESSCODE'])?$original_record['PROCESSCODE']:'')."_executions.txt","a+");
								fwrite($tta,chr(13).chr(10).'Internal ExecSection end 2, id event:  '.print_r($eventrec,true).chr(13).chr(10));
								fwrite($tta,chr(13).chr(10).'Tiempo section:'. (microtime(true)-$inittime).chr(13).chr(10));
								fclose($tta);
							}			
					   if (!empty($this->is_debug)) {
						  $this->logProcess($this->processcode, 'RUN', dict('GEN_LBL_SOURCECODE'), CLX_CLASS_DATA);
					   }
					   if (!ClxCommon::$db->hasError()) {
						  $this->logProcess($this->processcode, 'ERRO', ClxCommon::$db->errorMsg(), 'fail');
						  $this->endProcess();
						  $flag = false;
					   } else {
						  if (is_resource($rs) > CLX_NUM_ZERO) {
						      $r = ClxCommon::$db->getNextRecord($rs);
						      if (!empty($r)) {
								$record = $this->execSection($eventrec['DO_SECTION'], $r);
						      }
						  } else {
						      $flag = false;
						  }
					   }
			         }
			         break;
			     case 'DWH': //Do When
						IF ($this->debug==true){ echo '---------->EJECUTANDO DWH'; }
						If ( isset($_SESSION["_dEBUGX"]) and $_SESSION["_dEBUGX"]==99) {
							$tta=fopen(kPathToWebSite."/".kPathToTemp."/process_".(!EMPTY($original_record['PROCESSCODE'])?$original_record['PROCESSCODE']:'')."_executions.txt","a+");
							fwrite($tta,chr(13).chr(10).'Internal ExecSection start 3, id event: '.print_r($eventrec,true).chr(13).chr(10));
							fwrite($tta,chr(13).chr(10).'Tiempo section:'. (microtime(true)-$inittime).chr(13).chr(10));
							fclose($tta);
						}	
			         $rs = $EVNT->execute_process_event($eventrec['ID'], $record, $this->message, $this->parameters, $this->is_debug);
						If ( isset($_SESSION["_dEBUGX"]) and $_SESSION["_dEBUGX"]==99) {
							$tta=fopen(kPathToWebSite."/".kPathToT.kPathToTemp."/process_".(!EMPTY($original_record['PROCESSCODE'])?$original_record['PROCESSCODE']:'')."_executions.txt","a+");
							fwrite($tta,chr(13).chr(10).'Internal ExecSection end 3, id event: '.print_r($eventrec,true).chr(13).chr(10));
							fwrite($tta,chr(13).chr(10).'Tiempo section:'. (microtime(true)-$inittime).chr(13).chr(10));
							fclose($tta);
						}
			         if (!empty($this->is_debug)) {
					   $this->logProcess($this->processcode, 'RUN', dict('GEN_LBL_SOURCECODE'), CLX_CLASS_DATA);
			         }
			         if (!ClxCommon::$db->hasError()) {
					   $this->logProcess($this->processcode, 'ERRO', ClxCommon::$db->errorMsg(), 'fail');
					   $this->endProcess();
			         } else {
					   if (ClxCommon::$db->isSelect($rs)) {
						  $r = ClxCommon::$db->getNextRecord($rs);
						  $record = $this->execSection($eventrec['DO_SECTION'], $r);
					   }
			         }
			         break;
			     case 'DU'://Need Revision, occurs until condition is met
			         IF ($this->debug==true){ echo '---------->EJECUTANDO DU'; }
			         $flag = true;
			         while ($flag) {
						 If ( isset($_SESSION["_dEBUGX"]) and $_SESSION["_dEBUGX"]==99) {
							$tta=fopen(kPathToWebSite."/".kPathToTemp."/process_".(!EMPTY($original_record['PROCESSCODE'])?$original_record['PROCESSCODE']:'')."_executions.txt","a+");
							fwrite($tta,chr(13).chr(10).'Internal ExecSection start 4,id event: '.print_r($eventrec,true).chr(13).chr(10));
							fwrite($tta,chr(13).chr(10).'Tiempo section:'. (microtime(true)-$inittime).chr(13).chr(10));
							fclose($tta);
						 }	
					   $rs = $EVNT->execute_process_event($eventrec['ID'], $record, $this->message, $this->parameters, $this->is_debug);
						If ( isset($_SESSION["_dEBUGX"]) and $_SESSION["_dEBUGX"]==99) {
							$tta=fopen(kPathToWebSite."/".kPathToTemp."/process_".(!EMPTY($original_record['PROCESSCODE'])?$original_record['PROCESSCODE']:'')."_executions.txt","a+");
							fwrite($tta,chr(13).chr(10).'Internal ExecSection end 4 ,id event:'.print_r($eventrec,true).chr(13).chr(10));
							fwrite($tta,chr(13).chr(10).'Tiempo section:'. (microtime(true)-$inittime).chr(13).chr(10));
							fclose($tta);
						}	
					   if (!empty($this->is_debug)) {
						  $this->logProcess($this->processcode, 'RUN', dict('GEN_LBL_SOURCECODE'), CLX_CLASS_DATA);
					   }
					   if (!ClxCommon::$db->hasError()) {
						  $this->logProcess($this->processcode, 'ERRO', ClxCommon::$db->errorMsg(), 'fail');
						  $this->endProcess();
						  $flag = false;
					   } else {
						  if (ClxCommon::$db->isSelect($rs)) {
						      $r = ClxCommon::$db->getNextRecord($rs);
						      if (!empty($r)) {
								$record = $this->execSection($eventrec['DO_SECTION'], $r);
						      }
						      $flag = false;
						  }
					   }
			         }
			         break;
			     default: 
						IF ($this->debug==true){ echo '---------->EJECUTANDO DEFAULT'; }
						If ( isset($_SESSION["_dEBUGX"]) and $_SESSION["_dEBUGX"]==99) {
							$tta=fopen(kPathToWebSite."/".kPathToTemp."/process_".(!EMPTY($original_record['PROCESSCODE'])?$original_record['PROCESSCODE']:'')."_executions.txt","a+");
							fwrite($tta,chr(13).chr(10).'Internal ExecSection start 5, id event: '.print_r($eventrec,true).chr(13).chr(10));
							fwrite($tta,chr(13).chr(10).'Tiempo section:'. (microtime(true)-$inittime).chr(13).chr(10));
							fclose($tta);	
						}	
						$rs = $EVNT->execute_process_event($eventrec['ID'], $record, $this->message, $this->parameters, $this->is_debug);
						If ( isset($_SESSION["_dEBUGX"]) and $_SESSION["_dEBUGX"]==99) {
							$tta=fopen(kPathToWebSite."/".kPathToTemp."/process_".(!EMPTY($original_record['PROCESSCODE'])?$original_record['PROCESSCODE']:'')."_executions.txt","a+");
							fwrite($tta,chr(13).chr(10).'Internal ExecSection end 5, id event:'.print_r($eventrec,true).chr(13).chr(10));
							fwrite($tta,chr(13).chr(10).'Tiempo section:'. (microtime(true)-$inittime).chr(13).chr(10));
							fclose($tta);
						}	
			         if (!empty($this->is_debug)) {
					   $this->logProcess($this->processcode, 'RUN', dict('GEN_LBL_SOURCECODE'), CLX_CLASS_DATA);
			         }

			         if (!ClxCommon::$db->hasError()) {
					   if (ClxCommon::$db->isSelect($rs)) {
						  $record = ClxCommon::$db->getNextRecord($rs);
					   }
			         } else {
					   $this->logProcess($this->processcode, 'ERRO', ClxCommon::$db->errorMsg(), 'fail');
					   $this->endProcess();
			         }


			         break;
			 }//end switch
		  } else { 	IF ($this->debug==true){ echo '---------->EVENTO  ELSE<BR>'; }
			     
			 if (!empty($this->is_debug)) {
			     echoLog($eventrec['EXTRA2']);
			     $this->logProcess($this->processcode, 'RUN', dict('GEN_LBL_SOURCECODE'), CLX_CLASS_DATA);
			 }
			 If ( isset($_SESSION["_dEBUGX"]) and $_SESSION["_dEBUGX"]==99) {
				$tta=fopen(kPathToWebSite."/".kPathToTemp."/process_".(!EMPTY($original_record['PROCESSCODE'])?$original_record['PROCESSCODE']:'')."_executions.txt","a+");
				fwrite($tta,chr(13).chr(10)."PROCESS ID: ".$eventrec['ID'].chr(13).chr(10));
				fclose($tta);
			 }	
				//if ( PHP_OS=='WINNT') {
				//	$EVNT->execute_process_event($eventrec['ID'], $record, $this->message, $this->parameters, $this->is_debug);
				//	$this->logProcess($this->processcode, 'RUN', dict('BASE_PROC_LBL_DISPLAY'), CLX_CLASS_DATA);
				//}
			//	else {		
			IF ($this->debug==true){ echo '---------->CALL PIPE PREV<BR>'; }
					$this->call_pipe($eventrec['ID'], $record, $this->message, $this->parameters, $this->is_debug);
			IF ($this->debug==true){ echo '---------->CALL PIPE POST<BR>'; }
		  ##      $EVNT->execute_process_event($eventrec['ID'], $record, $this->message, $this->parameters, $this->is_debug);
					$this->logProcess($this->processcode, 'RUN', dict('BASE_PROC_LBL_DISPLAY'), CLX_CLASS_DATA);
							
			//	}
		    }

		  $step_status = $this->status;
		  if ($this->status === 'RUN') {
			 $step_status = 'END';
		  }

		  ClxFreeReturn($this->logStep($eventrec, $step_status, $this->message, $stepid));
			
        }//end while
		ClxCommon::$db->freeResult( $eventrecs );
		If ( isset($_SESSION["_dEBUGX"]) and $_SESSION["_dEBUGX"]==99) {
			$tta=fopen(kPathToWebSite."/".kPathToTemp."/process_".(!EMPTY($original_record['PROCESSCODE'])?$original_record['PROCESSCODE']:'')."_executions.txt","a+");
			fwrite($tta,chr(13).chr(10).'1105- hora:'.date( 'h:i:s d/m/Y').chr(13).chr(10));	
			fwrite($tta,chr(13).chr(10).'ExecSection FIN '.$secCode.chr(13).chr(10));
			fwrite($tta,chr(13).chr(10).'Tiempo section:'. (microtime(true)-$inittime).chr(13).chr(10));
			
			fclose($tta);
		}
        return $record;
    }
	function call_pipe($ID, &$record, $message, $parameters, $is_debug) {
		$returns=array();
		If ( isset($_SESSION["_dEBUGX"]) and $_SESSION["_dEBUGX"]==99) {
			$tta=fopen(kPathToWebSite."/".kPathToTemp."/process_executions.txt","a+");
			fwrite($tta,chr(13).chr(10)."call_pipe".chr(13).chr(10));
			fwrite($tta,chr(13).chr(10)."record".print_r($record,true).chr(13).chr(10));
			fwrite($tta,chr(13).chr(10)."parameters".print_r($parameters,true).chr(13).chr(10));

			fclose($tta);
		}
		if (isset(UserRequest::$request['wind']))
			$parameters["request_wind"]=UserRequest::$request['wind'];	
		else
			$parameters["request_wind"]='';
		$keys_addr=array();
		foreach ($_SESSION as $keyX=>$valueX) {
			if (substr($keyX,0,4)==="ADDR"){
				$keys_addr[$keyX]=$valueX;
			}
		}
		$varServer=$_SERVER;		
		unset ($varServer['COMPUTERNAME'],    $varServer['OS'],    $varServer['Path'],    $varServer['PATHEXT'],    $varServer['SystemDrive'],    $varServer['SystemRoot'],$varServer['TEMP'],    $varServer['TMP'],    $varServer['USERDOMAIN'],     $varServer['USERNAME'],     $varServer['windir'],$varServer['PROCESSOR_ARCHITECTURE'],$varServer['PROCESSOR_IDENTIFIER'],$varServer['PROCESSOR_LEVEL'],$varServer['PROCESSOR_REVISION'],		$varServer['PROCESSOR_ARCHITEW6432'],    $varServer['FCGI_X_PIPE_'],$varServer['PHPRC'],$varServer['PHP_FCGI_MAX_REQUESTS'],		$varServer['ALLUSERSPROFILE'],$varServer['APPDATA'],$varServer['APP_POOL_CONFIG'],$varServer['APP_POOL_ID'],   		$varServer['CommonProgramFiles'],    $varServer['CommonProgramFiles(x86)'],    $varServer['CommonProgramW6432'],$varServer['LD_LIBRARY_PATH'],		$varServer['LOCALAPPDATA'],$varServer['NUMBER_OF_PROCESSORS'],$varServer['ProgramData'],		$varServer['ProgramFiles'],$varServer['ProgramFiles(x86)'],$varServer['ProgramW6432'],$varServer['PSModulePath'],$varServer['PUBLIC'],		$varServer['TNS_ADMIN'],$varServer['_FCGI_X_PIPE_'],		$varServer['ComSpec'],$varServer['FP_NO_HOST_CHECK'],$varServer['USERPROFILE'],		$varServer['VBOX_MSI_INSTALL_PATH'],$varServer['INSTANCE_META_PATH'],$varServer['INSTANCE_NAME'],$varServer['INSTANCE_ID'],$varServer['GATEWAY_INTERFACE'],$varServer['CONTENT_TYPE'],$varServer['CONTENT_LENGTH'],$varServer['CERT_SUBJECT'],$varServer['CERT_SERIALNUMBER'],$varServer['CERT_ISSUER'],$varServer['CERT_FLAGS'],$varServer['CERT_COOKIE'],$varServer['AUTH_USER'],$varServer['AUTH_PASSWORD'],$varServer['AUTH_TYPE'],$varServer['APPL_PHYSICAL_PATH'],$varServer['APPL_MD_PATH'],$varServer['IIS_UrlRewriteModule'],$varServer['FCGI_ROLE'],$varServer['REQUEST_TIME_FLOAT'],$varServer['REQUEST_TIME']);	
	
		$varServer=base64_encode(gzcompress(serialize($varServer),9));//ECHO STRLEN($varServer);
		$serverBDname=explode(':',kDatabaseURL)[0];
		$serverBDip=gethostbyname($serverBDname);
				$send=json_encode(array('ID'=>$ID,
						'record'=>base64_encode(serialize($record)),
						'message'=>base64_encode(serialize($message)),
						'parameters'=>base64_encode(serialize($parameters)),
				'server'=>$varServer,
		//		'parameter'=>$varServer,
						'user_record'=>base64_encode(serialize($_SESSION['user_record'])),
				'prefs'=> base64_encode(serialize($_SESSION['prefs'])),
						'keys_addr'=> base64_encode(serialize($keys_addr)),
				'serverBDnameIp'=>base64_encode(serialize(array($serverBDname,$serverBDip))),
						'is_debug'=>$is_debug));
		//	pre($send);		
		##	session_write_close (  );
		##	session_register_shutdown ( );
			$descriptorspec = array(
									0 => array("pipe", "r"),  // stdin es una tubera usada por el hijo para lectura
									1 => array("pipe", "w"),  // stdout es una tubera usada por el hijo para escritura
									2 => array("file", kPathToWebSite."/".kPathToTemp."/error-output.txt", "a") // stderr es un fichero para escritura
								);
			$traza='';
			$cwd = kPathToWebSite."/".kPathToTemp;
			
			if(uppercase(PHP_OS)=='LINUX' && empty($_SERVER["REMOTE_ADDR"] )){
				//pre(PRPR_USER_DEFAULT_LINUX_FILE);
				if (file_exists(kPathToWebSite."/".kPathToTemp."/error-output.txt")) {
					if(PRPR_USER_DEFAULT_LINUX_FILE!=posix_getpwuid(fileowner(kPathToWebSite."/".kPathToTemp."/error-output.txt"))['name']){
						chown(kPathToWebSite."/".kPathToTemp."/error-output.txt", PRPR_USER_DEFAULT_LINUX_FILE);				
					}
				}
			}
		//	restore_error_handler();
		//	error_reporting(E_ALL);
			IF ($this->debug==true){ echo '---------->PROC_OPEN PIPE<BR>'; }
			if (PHP_OS=='WIN' or PHP_OS=='WINNT' ) {
				$env = array('bypass_shell'=>true);
				$process = proc_open(kPhpCLI.' -c '. QUOTE(php_ini_loaded_file()) ,$descriptorspec, $pipes, $cwd, null,  array('bypass_shell'=>true));
			}
			ELSE {
				$env = null;				
				$process = proc_open(kPhpCLI.' -c '. QUOTE(php_ini_loaded_file()), $descriptorspec, $pipes, $cwd, $env);
			}
			IF ($this->debug==true){
				echo 'PHP: '.$cwd,'<br>';
				echo 'descriptors: ';
				pre($descriptorspec,true);
				echo 'pipes: ';
				pre($pipes,true);
				echo 'cwd: '.kPhpCLI,'<br>';
				echo 'id from process:'.$process,'<br>';
			}
	//ECHO 'XXXXXXXXXXXXXXXXX';
			if (is_resource($process)) {
				// $pipes ahora ser algo como:
				// 0 => gestor de escritura conectado al stdin hijo
				// 1 => gestor de lectura conectado al stdout hijo
				// Cualquier error de salida ser anexado a /tmp/error-output.txt
				IF ($this->debug==true){ echo '---------->RESOURCE VALID<BR>'; }
				fwrite($pipes[0], '<?php '.
						
					'  	$GLOBALS[\'PIPE_PROCESS\']=1;
					chdir ( "'.kPathToWebSite.'");
					if (!defined(\'__DIR__\')) {
						define(\'__DIR__\', dirname(__FILE__));
					}
					set_include_path(str_replace(\'\\\\\', DIRECTORY_SEPARATOR, __DIR__) . DIRECTORY_SEPARATOR . PATH_SEPARATOR . get_include_path());
					require "'.kPathToWebSite.'includes/constants_xpengine.php";	
					require "'.kPathToWebSite.'includes/includes.php";
					$send=json_decode(\''.$send.'\',true);
					IF (!isset($_SESSION))  
					  session_start();
					$XSERVER=unserialize(gzuncompress(base64_decode($send["server"])));
					foreach($XSERVER as $key=>$value) {
						$_SERVER[$key]=$XSERVER[$key];
					}	
					$keys_addr=unserialize(base64_decode($send["keys_addr"]));
					foreach ($keys_addr as $keyX=>$valueX) {
						$_SESSION[$keyX]=$valueX;
						
					}		
				//	$GLOBALS["ADDRDEBUGIMPORT"]=1;
					$GLOBALS["PIPE_PROCESSBD"]=unserialize(base64_decode($send["serverBDnameIp"]));
					
					$_SESSION["prefs"]= unserialize(base64_decode($send["prefs"]));
					$_SESSION["user_record"]=unserialize(base64_decode($send["user_record"]));
				//	$_SESSION["_dEBUG"]=99;
				//	pre($_SESSION);
					PrepareConstants();
				//		pre(get_defined_constants());
					//pre($_SESSION["user_record"]);
					ConnectToDatabase();
					initToolsCron();
				
					$send["record"]=unserialize(base64_decode($send["record"]));
					$send["parameters"]=unserialize(base64_decode($send["parameters"]));
					$send["message"]=unserialize(base64_decode($send["message"]));
					$EVNT = &getTool("EVNT");
					$GLOBALS[\'PIPE_PROCESS\']=0;
					$EVNT->execute_process_event($send["ID"],$send["record"],$send["message"], $send["parameters"], $send["is_debug"]);
					
					?>');
					
				fclose($pipes[0]);
				$status=print_r(proc_get_status($process),true);
				IF ($this->debug==true) {
					///pre($textoProbar);
					pre("status:");
					pre($status);
				
				}
				$mensaje=stream_get_contents($pipes[1],63250000);
				fclose($pipes[1]);
				// Es importante que se cierren todas las tubera antes de llamar a
				// proc_close para evitar as un punto muerto
				$return_value = proc_close($process);
				$sibyzerodivide=strpos($mensaje,"Division by zero");
				if ($return_value>0 || $sibyzerodivide>0) {
					$BUGS = getTool('BUGS');
					if (!empty($BUGS)) {
						if (!$sibyzerodivide) {
										
								$pos=strpos($mensaje,"on line");
								$linea=0;
								if ($pos>0) {
									$linea=substr($mensaje,$pos+7);
									$mensaje=substr($mensaje,0,$pos-1);
								}
							
								$php="";
								$pos=strpos($mensaje," in ");
								if ($pos>0) {
									$php=substr($mensaje,$pos+4);
									$mensaje=substr($mensaje,0,$pos-1);
								}
								
						}
						else {
							##$mensaje=str_replace("Division by zero","",$mensaje);
							$linea=0;
							$php=0;
							
						}	
						$request='';
						echo $mensaje;
						echo dict("command returned ").$return_value;
						$te=debug_backtrace(); 
						$pos=$posx=0;
						$back=' parameters:'.chr(13).chr(10).
						print_r($parameters,true).
						chr(13).chr(10).chr(13).chr(10).
						' record: '.chr(13).chr(10).
						print_r($record,true).
						chr(13).chr(10).chr(13).chr(10);
						
						for ($t=0;$t<12;$t++) {
							if (isset($te[$t]["file"])){
								if ($t==$posx)
									$back.= chr(13).chr(10). $pos."_".$te[$posx]["function"]."_util(pre)-hora:".date("h:i:s d/m/Y")."";
								$back.= chr(13).chr(10). substr($te[$t]["file"].str_repeat(" ",80),0,80).chr(9).$te[$t]["line"].chr(9).$te[$t]["function"]."";
							}
							
						}
						$request.=$back;
						$BUGS->report(E_USER_ERROR, $mensaje . ' (' . $return_value . ' )',$php ,$linea, $request, CLX_NUM_ONE, 'Process','index.php',true);
						
					}
					else {
						echo $mensaje;
						echo dict("command returned ").$return_value;
					}
				}
				else {
						if (substr($mensaje,0,5)=='json:') {
							$textoretornos=substr($mensaje,5);
							$returns=json_decode($textoretornos,true);
							foreach($returns as $key=>$value) {
								$GLOBALS[$key]=$value;
							}

						}
						else {
							echo $mensaje;
						}	
				}
			}
			return ;
	}
    /**
     * Initialize run parameters for process
     * @param: string schedulecode
     */
    function loadParameters($schedulecode,$silog=true) {
        $this->parameters['RUNID'] = $this->runid;
        $this->parameters['START_TIME'] = $this->now;
        $this->parameters['PROCESSCODE'] = $this->processcode;
        $this->parameters['SCHEDULECODE'] = $schedulecode;
        $this->parameters['CURRENT_TIME'] = xFormat::date('now', 'Y-m-d h:i:s');
        $parameters = array();
        reset($this->parameters);
        foreach ($this->parameters as $key => $val) {
		  $parameters[$key] = $val;
        }
        $rs = ClxCommon::$db->getRecords('CLX_PR_SCHED_PRMTS', 'SCHEDULECODE = ' . singleQuote($schedulecode));
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
		  $parameters[$r['FIELDNAME']] = TRIM($r['PARAMETER_VALUE']);     //se agrego TRIM ficosah coloco un salto de linea en el textarea (invisible) y por tanto una fecha no hacia match
        }
		ClxCommon::$db->freeResult( $rs );
        $pmlbl = '<ul>';
        foreach ($parameters as $k => $v) {
		  $pmlbl .= '<li>' . $k . ': ' . $v . '</li>';
        }
        $pmlbl .= '</ul>';
		if ($silog==true)
        $this->logProcess($this->processcode, 'RUN', dict('GEN_LBL_PARAMETERS') . '<br>' . $pmlbl, 'ok');
        return $parameters;
    }

    /**
     * Send process to end
     */
    function endProcess() {
        $this->setStop(true);
    }

    function logStep($eventrec, $status, $msg, $stepid, $secCode = '') {
        if (empty($stepid)) {
		  $record = array();
		  $record['LOG_RECID'] = $this->runid;
		  $record['SEQNUM'] = $eventrec['SEQNUM'];
		  $record['CODE_TYPE'] = $eventrec['ACTION'];
		  $record['CODE_TEXT'] = $eventrec['EXTRA2'];
		  $record['DO_SECTION'] = $eventrec['DO_SECTION'];
		  $record['DO_SECTION_TYPE'] = $eventrec['DO_SECTION_TYPE'];
		  $record['STEP_STATUS'] = $status;
		  $record['STEP_MSG'] = ClxCommon::$db->addslashes($msg);
		  $record['SECTIONCODE'] = $secCode;
		  $record['SDATE'] = getNow('Y-m-d H:i:s');
		  $stepid = ClxCommon::$db->createRecord('CLX_PR_STEP_LOGS', $record, false);
		  ClxCommon::$db->commit();
        } else {
		  $a = $this->arrayToString($this->parameters);
		  $binds = array('EDATE' => getNow('Y-m-d H:i:s'), 'PARAMETERS_OUT' => ClxCommon::$db->addslashes($a), 'STEP_STATUS' => $status, 'STEP_MSG' => $msg);
		  $stepid = ClxCommon::$db->updateRecord('CLX_PR_STEP_LOGS', $binds, 'ID=' . $stepid);
		  $this->stepid = false;
        }
        $this->message = '';
        return $stepid;
    }

    function logProcess($processcode, $status, $msg, $class = '',$initial=0) {
        ob_start();
        if ($this->is_display) {
		  $msg_ob = ' ' . ob_get_contents();
		  ob_clean();

		  if ($this->is_html) {
			 if (strlen($msg_ob) > 0)
			     $msg .= '<br>' . $msg_ob;
			 echo '<table width="90%" align="center"><tr><td class="' . $class . '">' . wordwrap($msg, 90) . '</td></tr></table>';
		  }else {
			 if (strlen($msg_ob) > 0)
			     $msg .= "\n" . $msg_ob;
			 echo $msg . "\n";
		  }
        }
        ob_flush();

        $this->message .= $msg . '<br>';
        //$msg = strip_tags($msg);
        $this->status = $status;
        if (!$this->is_display)
		  $msg = '';
		if (empty($this->executedby))
			$this->executedby = $_SESSION['user_record']['username'];
		if (empty($this->executedby))
			$this->executedby ='system';
        if (empty($this->runid)) {
		  $record = array();
		  $sql = 'SELECT PRLO.*,LOG_METHOD FROM CLX_PR_SCHEDULES PRSC, CLX_PR_PROC_LOGS PRLO '.
					' WHERE PRSC.LOG_METHOD = 1 AND PRSC.IDCODE = SCHEDULECODE AND PRLO.SCHEDULECODE = ' . 
						singleQuote($this->schedulecode) . 
					' AND PRLO.PROC_STATUS = ' . singleQuote('END') .''. 
					' AND PRLO.ID = (SELECT MAX(PRLO2.ID) FROM CLX_PR_PROC_LOGS PRLO2 WHERE '.
					' PRLO2.SCHEDULECODE = PRLO.SCHEDULECODE)';
		  $record = ClxCommon::$db->getOneRecord($sql);
			if ( !empty($_SESSION["_dEBUG"]) and $_SESSION["_dEBUG"]==99) { $tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");	fwrite($tta,'-=-->record:'.print_r($record,true).chr(13).chr(10));fclose($tta);}
			
			$LOG_METHOD=(isset($record['LOG_METHOD'])) ?$record['LOG_METHOD']: 0;
			unset($record['LOG_METHOD']);

		  if (empty($record) or ($initial==1 && $LOG_METHOD==0))  {
			 $record = array();
			 $record['PROCESSCODE'] = $processcode;
			 $record['EXECUTEDBY'] = $this->executedby;
			 $record['SDATE'] = $this->now;
			 $record['EDATE'] = getNow('Y-m-d H:i:s');
			 //$record['PROCESS_MSG'] = $db->addslashes($msg);
			 $record['PROC_STATUS'] = $status;
			 $record['SCHEDULECODE'] = $this->schedulecode;
			 $record['BATCH_PID'] = $this->batch_pid;
			 $this->runid = ClxCommon::$db->createRecord('CLX_PR_PROC_LOGS', $record, false);
			 ClxCommon::$db->commit();
		  } else {
			 $this->runid = $record['ID'];
			 $record['EXECUTEDBY'] = $this->executedby;
			 $record['SDATE'] = $this->now;
			 $record['EDATE'] = getNow('Y-m-d H:i:s');
			 $record['PROC_STATUS'] = $status;
			 $record['BATCH_PID'] = $this->batch_pid;
			 ClxCommon::$db->updateRecord('CLX_PR_PROC_LOGS', $record, "ID = " . $record['ID']);
		  }
		  $this->parameters['RUNID'] = $this->runid;
		  $_SESSION['user_record']['username'] = $this->executedby;
		  $this->parameters['USERNAME'] = $this->executedby;
        } else {
		  if (kDatabaseType == "oci") {
			 $msg = '<table width="90%" align="center"><tr><td class="data">' . $msg . '</td></tr></table>';
			 $binds = array();
			 $sql = 'UPDATE CLX_PR_PROC_LOGS SET SDATE = ' . singleQuote($this->now) . ', EDATE = ' . singleQUote(getNow('Y-m-d H:i:s')) . ', PROC_STATUS = ' . singleQuote($status) . ', schedulecode = ' . singleQuote($this->schedulecode) . ' WHERE ID=' . singleQuote($this->runid);
			 //unset($binds['PROCESS_MSG']);
			 $this->notifySchedule($status);
			 ClxCommon::$db->executeQuery($sql, $binds);
		  } else {
			 $msg = CLX_CHAR_NL . $msg;
			 ClxCommon::$db->addslashes($msg);
			 $sql = 'UPDATE CLX_PR_PROC_LOGS SET SDATE = ' . singleQUote($this->now) . ', EDATE = ' . singleQUote(getNow('Y-m-d H:i:s')) . ', PROC_STATUS = ' . singleQUote($status) . ', schedulecode = ' . singleQUote($this->schedulecode) . ' WHERE ID=' . singleQuote($this->runid);
			 $this->notifySchedule($status);
			 ClxCommon::$db->executeQuery($sql);
		  }
        }
		ClxCommon::$db->executeQuery('COMMIT');
    }

    function notifyScheduleVoid($status) {
        ClxFreeReturn($this->notifySchedule($status));
    }

    function notifySchedule($status) {
        if (defined('PRSC' . $this->schedulecode)) {
		  return true;
        }
        $excludeemails = array();
        $excludetask = array();
        $excludealert = array();
        $DBLT = getTool('DBLT');
        $status_name = $DBLT->getListItem('PROS', $status);
        $header = '<font size=2>' . uppercase(dict('GEN_LBL_SYSTEM')) . ': <a href="REPLACE_URL">' . kWebSite . '</a><br>';
        $header .= uppercase(dict('GEN_LBL_SCHEDULE')) . ': ' . $this->schedulecode . '<br>';
        $header .= uppercase(dict('GEN_LBL_STATUS')) . ': ' . $status_name . ' <br>';
        $header .= uppercase(dict('GEN_LBL_MESSAGE')) . ':</font> <br><br><br>';
        $sql = 'SELECT PRSC.NOTIFY_STATUS, PRSC.TOUSER USERNAME, PRSC.SUBJECT, PRSC.MESSAGE, PRSC.CREATE_TASK, PRSC.CREATE_ALERT, PRSC.SEND_EMAIL, ACUS.EMAIL, ACUS.FIRSTNAME, ACUS.LASTNAME FROM CLX_PR_SCHEDULES PRSC, CLX_AC_USERS ACUS WHERE PRSC.IDCODE = ' . singleQuote($this->schedulecode) . ' AND PRSC.TOUSER = ACUS.USERNAME AND PRSC.NOTIFY_STATUS =' . singleQuote($status);
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
		  if (!defined('PRSC' . $this->schedulecode)) {
			 define('PRSC' . $this->schedulecode, CLX_NUM_ONE);
		  }
		  //Sending emails
		  if ($r['SEND_EMAIL'] == CLX_NUM_ONE && empty($excludeemails[$r['EMAIL']])) {
			 $url = kWebSite . 'pop.php?to=EDIT&ac=DISPLAY&li=PRLO&qy=PRLO&idlist=' . $this->runid;
			 $msg = str_replace('REPLACE_URL', $url, $header) . $r['MESSAGE'];
			 clx_mail($r['EMAIL'], $r['EMAIL'], $msg, $r['SUBJECT']);
			 $excludeemails[$r['EMAIL']] = CLX_NUM_ONE;
		  }
		  //sending alert
		  if ($r['CREATE_ALERT'] == CLX_NUM_ONE && empty($excludealert[$r['USERNAME']])) {
			 $url = 'pop.php?to=EDIT&ac=DISPLAY&li=PRLO&qy=PRLO&idlist=' . $this->runid;
			 $msg = str_replace('REPLACE_URL', $url, $header) . $r['MESSAGE'];
			 alrt_send($msg, 'now', $r['USERNAME']);
			 $excludealert[$r['USERNAME']] = CLX_NUM_ONE;
		  }
		  //creating tasks
		  if ($r['CREATE_TASK'] == CLX_NUM_ONE && empty($excludetask[$r['USERNAME']])) {
			 $url = 'pop.php?to=EDIT&ac=DISPLAY&li=PRLO&qy=PRLO&idlist=' . $this->runid;
			 $msg = str_replace('REPLACE_URL', $url, $header) . $r['MESSAGE'];
			 //Create Tasks
			 $task = array();
			 $task['STATUS'] = 'IDLE';
			 $task['TYPE'] = 'FIEK';
			 $task['SUBJECT'] = $r['SUBJECT'];
			 $task['description'] = strip_tags(str_replace('<br>', CLX_CHAR_NL, $msg));
			 $task['PERCENT_COMPLETE'] = '0';
			 $task['COMPLETE_DATE'] = 'null';
			 $task['PRIORITY'] = 'N';
			 $task['OWNERCODE'] = $r['USERNAME'];
			 $task['ASSIGNEDTO'] = $r['USERNAME'];
			 $task['REMINDER_DATETIME'] = 'null';
			 $task['RECURRING'] = 'NONE';
			 $task['DEPENDS'] = '0';
			 $task['START_DATE'] = getNow('Y-m-d H:i:s');
			 $task['DUE_DATE'] = '';
			 $excludetask[$r['USERNAME']] = CLX_NUM_ONE;
			 ClxCommon::$db->createRecord('CLX_US_TASKS', $task, false);
		  }//end if task
        }
		ClxCommon::$db->freeResult( $rs );
    }

    /**
     * Set the next Schedule Code
     */
    function setNextRun($schedule_rec) {
        $next_run_date = '';
        if (empty($schedule_rec['PR_RECURRENCE'])) {
		  $schedule_rec['PR_RECURRENCE'] = CLX_INIT_ZERO;
        }
        if (empty($schedule_rec['PR_VALUE'])) {
		  $schedule_rec['PR_VALUE'] = CLX_INIT_ZERO;
        }
        $last_run_date = adodb_getdate(adodb_strtotime($schedule_rec['NEXT_RUN_DATI']));
        $interval_minutes = $schedule_rec['PR_RECURRENCE'] * $schedule_rec['PR_VALUE'];
        if ($interval_minutes == CLX_NUM_ZERO) {
		  $sql = 'UPDATE CLX_PR_SCHEDULES SET NEXT_RUN_DATI = null WHERE IDCODE = ' . singleQuote($schedule_rec['IDCODE']);
        } else {
		  $last_run_date['minutes'] = $last_run_date['minutes'] + $interval_minutes;
		  $next_run_date = adodb_date('Y-m-d H:i:s', adodb_mktime($last_run_date['hours'], $last_run_date['minutes'], $last_run_date['seconds'], $last_run_date['mon'], $last_run_date['mday'], $last_run_date['year']));
		  $sql = 'UPDATE CLX_PR_SCHEDULES SET NEXT_RUN_DATI = ' . singleQuote($next_run_date) . ' WHERE IDCODE = ' . singleQuote($schedule_rec['IDCODE']);
        }
        ClxCommon::$db->executeQuery($sql);
    }

    /**
     * Set process exceptions
     */
    function report_exception($schedulecode, $start_time, $runid, $msg, $tablecode = '', $status = 'ERRO') {
        ClxCommon::$db->createRecord('CLX_PR_EXCEPT_LOGS', array('SCHEDULECODE' => $schedulecode, 'EFFTDATE' => $start_time, 'TABLECODE' => $tablecode, 'LOG_RECID' => $runid, 'PROC_STATUS' => $status, 'DESCRIPTION' => $msg), false);
    }

    /**
     * Reports exception from parameters
     *
     * @param array $parameters
     * @param string $msg
     * @param string $tablecode
     * @param string $status
     */
    function report_exception_by_parameters($parameters, $msg, $tablecode = '', $status = 'ERRO') {
        $runid = '';
        $this->report_exception($parameters['SCHEDULECODE'], $parameters['START_TIME'], $runid, $msg, $tablecode, $status);
    }

    function arrayToString($array) {
        $result = '<ul>';
        foreach ($array as $key => $val) {
		  $result .= '<li>' . $key . ': ' . $val . '</li>';
        }
        return $result . '</ul>';
    }

    /**
     * Verify if execution has exceptions
     */
    private function hasExceptions() {
        return ClxCommon::$db->oneExistsRecords('CLX_PR_EXCEPT_LOGS', 'SCHEDULECODE = ' . singleQuote($this->schedulecode) . ' AND LOG_RECID = ' . singleQuote($this->runid)) <> CLX_NUM_ZERO;
    }

    /**
     * Stop setter
     * @return bool
     */
    public function getStop() {
        return $this->stop;
    }

    /**
     * Stop getter
     * @param bool $stop
     */
    public function setStop($stop) {
        $this->stop = (empty($stop)) ? false : true;
    }

}
