<?php

/**
 * Application Form Panel Fields Manager
 * For more information contact  support@alenet.com
 *
 * @package		Clientix.Tool.Interface
 * @abstract		User Interface - Forms Panel Fields
 * @package		Clientix.Tool.Interface
 * @abstract		User Interface - Forms
 * @author       $Author: jclerma $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2017-11-03 15:09:40 -0400 (Fri, 03 Nov 2017) $
 * @version    	$Rev: 9615 $
 *
 * CHANGE LOG
 *  - Hidden field added property
 */
class ui_form_panel_fields extends clx_tool {

    var $events; //List of events
    var $pf_types = array('FIELD' => 'GEN_LBL_FIELD', 'SPACE' => 'GEN_LBL_SPACER');

    /**
     * Constructor
     */
    function __construct() {
        $this->code = 'UIFF';
        $this->name = 'Form Panel Fields';
        $this->system = 'Interface';
        $this->version = '5.6.1.0';
        $this->has_preferences = true;

        $this->accesskey = 'UIFM';

        $this->tablename = 'CLX_UI_FORM_PANEL_FIELDS';
        $this->record_name_singular = 'panel field';
        $this->record_name_plural = 'panel fields';
        $this->primary_field = 'PANELID';
        $this->secondary_field = 'FIELDNAME';
        $this->seqnum_field = 'FIELDSEQNUM';
        $this->primary_key = array('ID');
        $this->edit_modes = array('0' => 'GEN_LBL_FIELD_DEFAULT', 'WYSIWYG' => 'GEN_LBL_WYSIWYG');
        $this->events = array('0' => dict('GEN_LBL_NEVER'), '1' => dict('GEN_LBL_ALWAYS'), '2' => dict('GEN_LBL_ONCREATE'), '3' => dict('GEN_LBL_ONMODIFY'));
    }

    /**
     * Describes the tool table structure.
     * @return array
     */
    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['PANELID'] = array('type' => 'varchar(32)', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['PF_TYPE'] = array('type' => 'varchar(10)', 'null' => 'NO', 'properties' => '', 'default' => '' . singleQuote('FIELD') . '', 'default_raw' => 'FIELD');
        $fields['CAPTION'] = array('type' => 'varchar(255)', 'properties' => '', 'dict' => true);
        $fields['CAPTION_CLASS'] = array('type' => 'varchar(255)', 'properties' => '');
        $fields['FIELDNAME'] = array('type' => 'varchar(64)', 'null' => 'YES', 'properties' => '');
        $fields['FIELDSEQNUM'] = array('type' => 'int(11) unsigned', 'null' => 'YES', 'properties' => '');
        $fields['FIELDFORMLABEL'] = array('type' => 'varchar(64)', 'properties' => '', 'dict' => true);
        $fields['FIELDPREFIX'] = array('type' => 'varchar(64)', 'properties' => '', 'dict' => true);
        $fields['FIELDSUFIX'] = array('type' => 'varchar(64)', 'properties' => '', 'dict' => true);
        $fields['FIELD_DISPLAYSIZE'] = array('type' => 'int(11) unsigned', 'properties' => '', 'null' => 'NO', 'default' => CLX_NUM_ZERO);
        $fields['FIELD_SIZE'] = array('type' => 'int(11) unsigned', 'properties' => '', 'null' => 'NO', 'default' => CLX_NUM_ZERO);
        $fields['FIELD_READONLY'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'default' => '0', 'properties' => '');
        $fields['FIELD_PASSWORD'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'default' => '0', 'properties' => '');
        $fields['FIELD_HIDDEN'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'default' => '0', 'properties' => '');
        $fields['FIELD_REQUIRED'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'default' => '0', 'properties' => '');
        $fields['FIELD_RELATED'] = array('type' => 'varchar(64)', 'null' => 'YES', 'properties' => '');
        $fields['FIELD_DEFAULT'] = array('type' => 'varchar(255)', 'properties' => '');
        $fields['FIELD_PARENT'] = array('type' => 'varchar(64)', 'null' => 'YES', 'properties' => '');
        $fields['PRELOAD_LIST'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'default' => '0', 'properties' => '');
        $fields['EDIT_MODE'] = array('type' => 'varchar(10)', 'null' => 'NO', 'properties' => '', 'default' => '0');
        $fields['TABLECODE'] = array('type' => 'varchar(4)', 'null' => 'YES', 'properties' => '');
        $fields['MASK_CODE'] = array('type' => 'varchar(16)', 'null' => 'YES', 'properties' => '');
        $fields['SRCH_LISTING'] = array('type' => 'varchar(16)', 'null' => 'YES', 'properties' => '');
        $fields['ACTIVE'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'default' => '1', 'properties' => '');
        $fields['USE_USR_DEFAULT'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'default' => CLX_NUM_ZERO);
        $fields['ADD_LIST_ENTRY'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'default' => CLX_NUM_ZERO);
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'MUL', 'fieldname' => 'PANELID,ACTIVE');
        $indexes[$ipf . '03'] = array('index' => 'MUL', 'fieldname' => 'FIELDNAME');
        $indexes[$ipf . '04'] = array('index' => 'MUL', 'fieldname' => 'FIELDSEQNUM');
        $indexes[$ipf . '05'] = array('index' => 'MUL', 'fieldname' => 'USE_USR_DEFAULT');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB', 'fields' => $fields, 'indexes' => $indexes);
    }

    /**
     * Pack tool record on xml Format
     * @param array record
     * @return obj
     */
    function pack($record) {
        $o = $this->packRecord($record);
        $objs = new XMLObject($this->code . '_OBJECTS');
        $this->packsub($objs, 'UIFR', 'SELECT UIFR.* FROM CLX_UI_FORM_FLD_PROP UIFR, CLX_UI_FORM_PANEL_FIELDS UIPF, CLX_UI_FORM_PANELS UIFP WHERE UIFP.IDCODE = UIPF.PANELID AND UIFR.FORMCODE = UIFP.FORMCODE AND UIFR.FIELDNAME = UIPF.FIELDNAME AND UIPF.ID = ' . $record['ID']);
        $this->packsub($objs, 'UISC', 'SELECT * FROM CLX_UI_SRCH_LIST_CRITS WHERE PFID=' . $record['ID']);
        $this->packsub($objs, 'EVNT', 'SELECT * FROM CLX_EVENTS WHERE OWNERCODE=' . singleQuote($this->code) . ' AND OWNERID=' . singlequote($record['ID']));
        $this->packsub($objs, 'DICT', 'SELECT DICT.* FROM CLX_DICTIONARY DICT, CLX_UI_FORM_PANEL_FIELDS X WHERE (DICT.ENTRY = X.FIELDFORMLABEL OR DICT.ENTRY = X.FIELDPREFIX OR DICT.ENTRY = X.FIELDSUFIX) AND X.ID = ' . singleQuote($record['ID']));
        $o->addObject($objs);
        return $o;
    }

    /**
     * Loads package into system
     * @param string $package
     */
    function unpack($parameters) {

        $record = $this->unpackRecord($parameters['package']);
        $record['PANELID'] = $parameters['id'];
        $record['FIELDNAME'] = uppercase($record['FIELDNAME']);
        $fieldid = ClxCommon::$db->createRecord($this->tablename, $record, false);
        if (!empty($parameters['package']->hasSubObject($this->code . '_OBJECTS'))) {
		  $objs = $parameters['package']->getObjectXml($this->code . '_OBJECTS');
		  foreach ($objs->objects as $o) {
			 $toolcode = $o->tag;
			 $tool = &getTool($toolcode);
			 $tool->unpack(array('package' => $o, 'id' => $fieldid));
		  }
        }
    }
    
    /**
     * Add this tool preferences to the preferences form,
     * $record are the saved preferences.
     *
     * @param array $record
     * @param object $form
     * @param object $table
     */
    function preferencesForm($record, &$form, &$table) {
        $msg = array();
        $jsevents = array();
        $table->addCell(dict('GEN_LBL_DEFAULTS'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => $table->width));
        $sql = $this->getPreferenceFields();
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
		  //remove de requirements
		  $r['FIELD_HIDDEN'] = CLX_INIT_ZERO;
		  $r['FIELD_REQUIRED'] = CLX_INIT_ZERO;
		  $r['ADD_LIST_ENTRY'] = CLX_INIT_ZERO;
		  $r['FIELD_DEFAULT'] = '';
		  $fieldinstance = getToolObject('DBFT', $r['FIELDTYPE']);
		  $form_fieldname = $this->getPreferenceFormFieldName($r);
		  $cell = $fieldinstance->placeEditField($msg, $form_fieldname, $r, $record, $form, $jsevents);
		  $table->addCell($cell->source(), array(CLX_PROP_COLSPAN => $table->width));
        }
		ClxCommon::$db->freeResult( $rs );
    }

    /**
     * Load preferences from the database.
     * $record will be used as first argument int preferencesForm
     *
     * @param array $record
     */
    function preferencesLoad(&$record) {
        $username = $_SESSION['user_record']['username'];
        $sql = 'SELECT PREFKEY, PREFVALUE FROM CLX_US_PREFERENCES WHERE UPPER( USERNAME)=' . singleQuote(uppercase($username)) . ' AND PREFKEY LIKE ' . singleQuote($this->code . '_%');
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
		  $record[$r['PREFKEY']] = $r['PREFVALUE'];
        }
		ClxCommon::$db->freeResult( $rs);
    }

    /**
     * Save user input.  Use UserRequest::$post to check user input.
     *
     * @param array $record
     * @return boolean
     */
    function preferencesSave() {
        $USPR = &getTool('USPR');
        ClxCommon::$db->deleteRecords('CLX_US_PREFERENCES', 'UPPER(USERNAME) = ' . singleQuote($_SESSION['user_record']['username']) . ' AND PREFKEY LIKE ' . singleQuote($this->code . '_%'));
        $sql = $this->getPreferenceFields();
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
		  $form_fieldname = $this->getPreferenceFormFieldName($r);
		  $field_value = (array_key_exists($form_fieldname, UserRequest::$post)) ? UserRequest::$post[$form_fieldname] : '';
		  $USPR->prfSavePref($_SESSION['user_record']['username'], $form_fieldname, $field_value);
        }
		ClxCommon::$db->freeResult( $rs );
    }

    /**
     * Create preferences for new users.
     *
     * @param array $record
     * @return boolean
     */
    function preferencesDefault(&$record) {
        $values = array();
        return $values;
    }

    /**
     * Select Field to collect preference
     * @return string
     */
    function getPreferenceFields() {
        $sql = 'SELECT DISTINCT DBFL.ID, DBFL.FIELDNAME, DBFL.FIELDFORMLABEL, DBFL.FIELDLISTLABEL, DBFL.FIELDPREFIX, DBFL.FIELDSUFIX, DBFL.FIELDTYPE, DBFL.FIELD_ANCHOR, DBFL.FIELD_SIZE, DBFL.FIELD_REQUIRED, DBFL.FIELD_FORMAT, DBFL.FIELD_ALIGN, DBFL.FIELD_DEFAULT, DBFL.FIELD_MINVALUE, DBFL.FIELD_MAXVALUE, DBFL.FIELD_DISPLAYSIZE, DBFL.FIELD_EXTRA1, DBFL.FIELD_EXTRA2, DBFL.FIELD_EXTRA3, DBFL.FIELD_EXTRA4, DBFL.FIELD_EXTRA5, DBFL.FIELD_EXTRA6, DBFL.FIELD_EXTRA7, DBFL.FIELD_EXTRA8, DBFL.FIELD_EXTRA9, ' . singleQuote(CLX_CHAR_EMPTY_STRING) . ' TABLECODE, ' . CLX_NUM_ZERO . ' FIELD_FORM_REQUIRED, ' . CLX_NUM_ZERO . ' FIELD_HIDDEN, ' . singleQuote(CLX_CHAR_EMPTY_STRING) . ' CAPTION, ' . CLX_NUM_ZERO . ' FIELD_READONLY, ' . CLX_NUM_ZERO . ' PFID, ' . singleQuote(CLX_CHAR_EMPTY_STRING) . ' FIELD_PARENT, UIFF.ADD_LIST_ENTRY FROM CLX_UI_FORMS UIFM, CLX_UI_FORM_PANEL_FIELDS UIFF, CLX_UI_FORM_PANELS UIFP, CLX_DB_FIELDS DBFL WHERE UIFM.IDCODE = UIFP.FORMCODE AND DBFL.FIELDNAME = UIFF.FIELDNAME AND UIFF.PANELID = UIFP.IDCODE AND UIFP.ACTIVE = ' . CLX_NUM_ONE . ' AND UIFF.ACTIVE = ' . CLX_NUM_ONE . ' AND UIFF.USE_USR_DEFAULT = ' . CLX_NUM_ONE . ' ORDER BY DBFL.FIELDNAME ASC';
        return $sql;
    }

    /**
     * Return the form fieldname for preference edit form.
     * @param array $record
     * return string
     */
    function getPreferenceFormFieldName(&$record) {
        return $this->code . '_' . $record['FIELDNAME'];
    }

    /**
     * Loads de current default value
     * @param $form_fieldname
     * @param $record
     * @see ui_forms::editForm()
     * @see edit:defaultRecord()
     */
    function setUserPrefFieldDefault($form_fieldname, &$fieldrecord) {
        if (empty($fieldrecord['USE_USR_DEFAULT'])) {
		  return;
        }
        if (!array_key_exists('prefs', $_SESSION)) {
		  return;
        }
        $ff = 'UIFF_' . $fieldrecord['FIELDNAME'];
        if (array_key_exists($ff, $_SESSION['prefs'])) {
		  if (strlen($_SESSION['prefs'][$ff]) > CLX_NUM_ZERO) {
			 $fieldrecord['FIELD_DEFAULT'] = $_SESSION['prefs'][$ff];
		  }
        }
    }

    /**
     * Action is called for every user interaction with the module...
     *
     * @param unknown_type $ac
     * @param unknown_type $idlist
     * @return unknown
     */
    function action($ac, $idlist) {
        $contents = '';
        $msg = array();
        if ($this->getAccess()->canRead()) {
		  switch ($ac) {
			 case 'SPACE':
			     if ($this->getAccess()->canDisplay()) {
			         $this->createSpacer();
			         $info = popHistory();
			         $to = &getTool($info[CLX_PROP_TO]);
			         $contents = $to->action($info['ac'], toArray($info['idlist']));
			     } else {
			         $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [UI_FORM_PANEL_FIELDS-CAN_APPEND_1][' . $ac . '] ' . $this->getCode(false));
			     }
			     break;
			 case 'INCLUDE':
			     if ($this->getAccess()->canDisplay()) {
			         if (!empty(UserRequest::$post['confirm']) && UserRequest::$post['confirm'] == CLX_NUM_TWO) {
					   if ($this->checkForm($msg)) {
						  ClxFreeReturn($this->createRecord_());
						  $info = popHistory();
						  $to = &getTool($info[CLX_PROP_TO]);
						  $contents = $to->action($info['ac'], toArray($info['idlist']));
					   } else {
						  $contents = $this->editForm($ac, array(), UserRequest::$post, $msg);
					   }
			         } elseif (!empty(UserRequest::$post['confirm']) && UserRequest::$post['confirm'] == CLX_NUM_ONE) {
					   $values = ClxCommon::$db->getRecord('CLX_DB_FIELDS', 'FIELDNAME=' . singleQuote(UserRequest::$post['UIFF_fieldname']));
					   //get values from derived fields
					   $callingpage = peekHistory();
					   $panelid = $callingpage['panelid'];
					   if (empty($panelid))
						  $panelid = CLX_INIT_ZERO;
					   $sql = 'SELECT TF.FIELD_DEFAULT, TF.FIELD_REQUIRED FROM CLX_DB_TABLEFIELDS TF, CLX_UI_FORM_PANELS FP, CLX_UI_FORMS F WHERE F.IDCODE = FP.FORMCODE AND TF.TABLECODE = F.TABLECODE AND TF.FIELDNAME = ' . singleQuote(UserRequest::$post['UIFF_fieldname']) . ' AND FP.IDCODE = ' . singleQuote($panelid);
					   $table_field = ClxCommon::$db->getOneRecord($sql);
					   if (!empty($table_field)) {
						  $values['FIELD_DEFAULT'] = $table_field['FIELD_DEFAULT'];
						  $values['FIELD_REQUIRED'] = $table_field['FIELD_REQUIRED'];
					   }
					   $record = $this->defaultRecord();
					   $record['UIFF_caption'] = '';
					   $record['UIFF_caption_DICT'] = '';
					   $record['UIFF_caption_class'] = CLX_CLASS_DATA;
					   $record['UIFF_fieldname'] = $values['FIELDNAME'];
					   $record['UIFF_fieldformlabel'] = uppercase(!empty($values['FIELDFORMLABEL'])) ? $values['FIELDFORMLABEL'] : '';
					   $record['UIFF_fieldprefix'] = (!empty($values['FIELDPREFIX'])) ? $values['FIELDPREFIX'] : '';
					   $record['UIFF_fieldsufix'] = (!empty($values['FIELDSUFIX'])) ? $values['FIELDSUFIX'] : '';
					   $record['UIFF_field_default'] = (!empty($values['FIELD_DEFAULT'])) ? $values['FIELD_DEFAULT'] : '';
					   $record['UIFF_field_required'] = (!empty($values['FIELD_REQUIRED'])) ? CLX_NUM_ONE : CLX_NUM_ZERO;
					   $record['UIFF_field_parent'] = (!empty($values['FIELD_PARENT'])) ? $values['FIELD_PARENT'] : '';
					   $record['UIFF_fieldformlabel_DICT'] = uppercase(!empty($values['FIELDFORMLABEL'])) ? dict($values['FIELDFORMLABEL']) : '';
					   $record['UIFF_fieldprefix_DICT'] = (!empty($values['FIELDPREFIX'])) ? dict($values['FIELDPREFIX']) : '';
					   $record['UIFF_fieldsufix_DICT'] = (!empty($values['FIELDSUFIX'])) ? dict($values['FIELDSUFIX']) : '';
					   $record['UIFF_field_displaysize'] = CLX_INIT_ZERO;
					   $record['UIFF_field_size'] = CLX_INIT_ZERO;
					   $record['UIFF_add_list_entry'] = CLX_INIT_ZERO;
					   $record['UIFF_srch_listing'] = '';
					   $contents = $this->editForm($ac, array(), $record, $msg);
			         } else {
					   $contents = $this->chooseFieldForm();
			         }
			     } else {
			         $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [UI_FORM_PANEL_FIELDS-CAN_APPEND_2][' . $ac . '] ' . $this->getCode(false));
			     }
			     break;
			 default:
			     $contents = parent::action($ac, $idlist);
			     break;
		  }
        } else {
		  $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [UI_FORM_PANEL_FIELDS-CAN_READ][' . $ac . '] ' . $this->getCode(false));
        }
        return $contents;
    }

    /**
     * Builds Panel listing on tool
     *
     * @param string $groupcode
     * @return string
     */
    function makepanel($groupcode = null) {
        $panelid = $groupcode;
        if ($this->getAccess()->canRead()) {
		  $t = time();
		  $to = $this->code;
		  $here = 'tool.php?to=' . $to . '&t=' . $t;
		  $yesno = array(dict('GEN_LBL_NO'), dict('GEN_LBL_YES'), dict('GEN_LBL_FIELD_DEFAULT'));
		  $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
		  $form->setProp(CLX_PROP_ID, 'UIFFPanel');
		  $form->addArgument(CLX_PROP_TO, $this->code);
		  $form->addArgument('panelid', $panelid);
		  $form->addArgument('ac', 'NOT_DEFINED');
		  $selectclause = '*';
		  $fromclause = $this->tablename;
		  $whereclause = 'PANELID=' . singleQuote($panelid);
		  $sortclause = 'FIELDSEQNUM ASC';
		  $page = new xPaging();
		  $page->setProp('rpp', CLX_NUM_TEN_THOUSAND);
		  $page->setProp('printheaders', false);
		  $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
		  $DICT = getTool('DICT');
		  $table = new htmlTable(CLX_WIDTH_22);
		  $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
		  $page->header($table);
		  $field_default_lbl = dict('GEN_LBL_FIELD_DEFAULT');
		  $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'),
			 dict('GEN_LBL_NAME'), dict('GEN_LBL_ACTIVE'), dict('GEN_LBL_FORMLABEL'), dict('GEN_LBL_READONLY'),
			 dict('GEN_LBL_PASSWORD'), dict('GEN_LBL_HIDDEN'), dict('GEN_LBL_REQUIRED'), dict('GEN_LBL_DISPLAY_SIZE'),
			 dict('GEN_LBL_FIELD_SIZE'), dict('GEN_LBL_FIELD_DEFAULT'), dict('GEN_LBL_USE_USR_DEFAULT'), dict('GEN_LBL_RELATEDDISPLAY'),
			 dict('GEN_LBL_FIELD_PARENT'), dict('GEN_LBL_PREFIX'), dict('GEN_LBL_SUFIX'), dict('GEN_LBL_TABLE'), dict('GEN_LBL_MASKING'),
			 dict('GEN_LBL_EDIT_MODE'), dict('GEN_LBL_PRELOAD_LIST'),
			 dict('GEN_ADD_LIST_ENTRY'),
			 dict('GEN_LBL_SRCH_LISTING')));
		  $i = CLX_INIT_ZERO;
		  while ($record = $page->nextRecord()) {
				$record['FIELDFORMLABEL']=uppercase($record['FIELDFORMLABEL']);
			 $recordid = $record['ID'];
			 $select = new htmlCheckbox('selector_' . $recordid, false);
			 $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
			 $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
			 $class = ($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT;
			 if ($record['PF_TYPE'] === 'SPACE') {
			     //Just add plase Holder and continue
			     $table->addCell(dict('GEN_LBL_SPACER'), array(CLX_PROP_CLASS => $class, CLX_PROP_COLSPAN => '1'));
			     $table->addCell($yesno[$record['ACTIVE']], array(CLX_PROP_CLASS => $class, CLX_PROP_COLSPAN => '1'));
			     $table->addCell($DICT->dictFastEdit($record['FIELDFORMLABEL']), array(CLX_PROP_CLASS => $class, CLX_PROP_COLSPAN => '1'));
			     $table->addCell(CLX_CHAR_HTML_NBSP, array(CLX_PROP_CLASS => $class, CLX_PROP_COLSPAN => '18'));
			     $i++;
			     continue;
			 }
			 if (empty($record['FIELD_READONLY'])) {
			     $record['FIELD_READONLY'] = CLX_INIT_ZERO;
			 }
			 if (empty($record['FIELD_PASSWORD'])) {
			     $record['FIELD_PASSWORD'] = CLX_INIT_ZERO;
			 }
			 if (empty($record['FIELD_HIDDEN'])) {
			     $record['FIELD_HIDDEN'] = CLX_INIT_ZERO;
			 }
			 if (!array_key_exists('FIELD_REQUIRED', $record) || $record['FIELD_REQUIRED'] === '' || $record['FIELD_REQUIRED'] == null) {
			     $record['FIELD_REQUIRED'] = CLX_NUM_TWO;
			 }
			 if (empty($record['FIELD_DISPLAYSIZE'])) {
			     $record['FIELD_DISPLAYSIZE'] = '0';
			 }
			 if (empty($record['ADD_LIST_ENTRY'])) {
			     $record['ADD_LIST_ENTRY'] = '0';
			 }

			 $table->addCells($class, array($record['FIELDNAME'],
			     $yesno[$record['ACTIVE']],
			     $DICT->dictFastEdit($record['FIELDFORMLABEL']),
			     $this->events[$record['FIELD_READONLY']],
			     $yesno[$record['FIELD_PASSWORD']],
			     $yesno[$record['FIELD_HIDDEN']],
			     $yesno[$record['FIELD_REQUIRED']],
			     (($record['FIELD_DISPLAYSIZE'] == CLX_NUM_ZERO) ? $field_default_lbl : $record['FIELD_DISPLAYSIZE']),
			     (($record['FIELD_SIZE'] == CLX_NUM_ZERO) ? $field_default_lbl : $record['FIELD_SIZE']),
			     $record['FIELD_DEFAULT'], dict($this->yesno[$record['USE_USR_DEFAULT']]),
			     $record['FIELD_RELATED'], $record['FIELD_PARENT'],
			     $DICT->dictFastEdit($record['FIELDPREFIX']),
			     $DICT->dictFastEdit($record['FIELDSUFIX']),
			     $record['TABLECODE'], $record['MASK_CODE'],
			     dict($this->edit_modes[$record['EDIT_MODE']]),
			     dict($this->yesno[$record['PRELOAD_LIST']]),
			     dict($this->yesno[$record['ADD_LIST_ENTRY']]),
			     $record['SRCH_LISTING']));
			 $i++;
		  }
		  $buttons = '';
		  $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_DISPLAY'), array('action' => 'DISPLAY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
		  if ($this->getAccess()->canModify()) {
			 $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
		  }
		  if ($this->getAccess()->canDelete()) {
			 $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_EXCLUDE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
		  }
		  if ($this->getAccess()->canDisplay()) {
			 $b = new htmlFormSubmit(dict('GEN_LBL_INCLUDE_SOMETHING', dict('GEN_LBL_FIELD')), 'INCLUDE', $form->getProp(CLX_PROP_ID));
			 $b->setProp(CLX_PROP_ID, 'ON');
			 $buttons .= $b->source();
		  }
		  if ($this->getAccess()->canDisplay()) {
			 $b = new htmlFormSubmit(dict('GEN_LBL_ADD_SPACE'), 'SPACE', $form->getProp(CLX_PROP_ID));
			 $b->setProp(CLX_PROP_ID, 'ON');
			 $buttons .= $b->source();
		  }
		  if (($page->getProp('qty') > CLX_NUM_ONE) && $this->getAccess()->canModify()) {
			 $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_REORDER'), array('action' => 'REORDER', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
		  }
		  $form->addContents($page->footers($table, $buttons));
		  $modulecode = '';
		  return buildPanelInterface(dict('TOOL_UIFF_TITLE_LIST'), $modulecode . 'TABLES', $form->source(), CLX_NUM_MINUS_1);
        } else {
		  return '';
        }
    }

    /**
     * Builds form field lookup
     * @return string
     */
    function chooseFieldForm() {
        $callingpage = peekHistory();
        $panelid = $callingpage['panelid'];
        $formcode = ClxCommon::$db->getField('CLX_UI_FORM_PANELS', 'FORMCODE', 'IDCODE=' . singleQuote($panelid));
        $tablecode = ClxCommon::$db->getField('CLX_UI_FORMS', 'TABLECODE', 'IDCODE=' . singleQuote($formcode));
        $items_flag = false;
        $sql = 'SELECT
					DBTF.FIELDNAME
				FROM
					CLX_DB_TABLEFIELDS DBTF
				WHERE
					DBTF.TABLECODE=' . singleQuote($tablecode) . ' 					AND NOT EXISTS (SELECT UIPF.ID     					FROM CLX_UI_FORM_PANELS UIFP, CLX_UI_FORM_PANEL_FIELDS UIPF     					WHERE UIFP.IDCODE = UIPF.PANELID     					AND UIFP.FORMCODE=' . singleQuote($formcode) . '     					AND UIPF.FIELDNAME=DBTF.FIELDNAME) 				UNION 				SELECT DBTF.FIELDNAME 				FROM CLX_DB_TABLEDFIELDS DBTF 				WHERE 				DBTF.TABLECODE=' . singleQuote($tablecode) . ' 				AND NOT EXISTS (SELECT UIPF.ID 					FROM CLX_UI_FORM_PANELS UIFP, 						CLX_UI_FORM_PANEL_FIELDS UIPF 					WHERE UIFP.IDCODE = UIPF.PANELID 					AND UIFP.FORMCODE=' . singleQuote($formcode) . ' 					AND UIPF.FIELDNAME=DBTF.FIELDNAME) 				AND NOT EXISTS (SELECT DBTF2.FIELDNAME 					FROM CLX_DB_TABLEFIELDS DBTF2     				WHERE DBTF2.TABLECODE=' . singleQuote($tablecode) . '     					AND DBTF2.FIELDNAME=DBTF.FIELDNAME)';
        $fields = new htmlSelect('UIFF_fieldname');
        $records = ClxCommon::$db->executeQuery($sql);
        while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
		  $fields->addItem($record['FIELDNAME'], $record['FIELDNAME'], false);
		  if (empty($items_flag)) {
			 $items_flag = true;
		  }
        }
		
        $title = dict('TOOL_UIFF_TITLE_CHOOSEFIELD');
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'INCLUDE');
        $form->addArgument('confirm', '1');
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM));
        $table->addCell($title, array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $buttons = '';
        if ($items_flag === true) {
		  $table->addData(dict('GEN_LBL_FIELDNAME') . ':', $fields->source());
		  $c = new htmlFormSubmit(dict('GEN_LBL_INCLUDE'), 'INCLUDE', $form->getProp(CLX_PROP_ID));
		  $buttons .= $c->source();
        } else {
		  $table->addData(dict('GEN_LBL_FIELDNAME') . ':', dict('TOOL_UIFF_ERR_NOMOREFIELDS'));
        }
        $c = new htmlButtonCancel();
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->setContents($table->source());
        return $form->source();
    }

    /**
     * Tool Edit Form
     * @param string $ac
     * @param string $idlist
     * @param array $record
     * @param array $msg
     * @return string
     */
    function editForm($ac, $idlist, $record, $msg) {
        if ($record['UIFF_pf_type'] === 'SPACE') {
		  return $this->editFormSpacer($ac, $idlist, $record, $msg);
        }
        $modify = (UserRequest::$request['ac'] === 'MODIFY');
        $title = ($modify) ? dict('TOOL_UIFF_TITLE_MODIFY') : dict('TOOL_UIFF_TITLE_INCLUDE');
        $buttons = '';
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', $ac);
        $form->addArgument('confirm', '2');
        $form->addArgument('UIFF_id', (!empty($record['UIFF_id'])) ? $record['UIFF_id'] : '');
        $form->addArgument('UIFF_fieldname', $record['UIFF_fieldname']);
        $form->addArgument('UIFF_pf_type', $record['UIFF_pf_type']);
        $form->addArgument('idlist', toString($idlist));
        $form->addArgument('pattern_lang', 'PCRE');
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell($title, array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addData(dict('GEN_LBL_FIELDNAME') . ':', $record['UIFF_fieldname']);
        $table->addYesNo(dict('GEN_LBL_ACTIVE'), 'UIFF_active', $record['UIFF_active'], CLX_NUM_20);
        $DICT = &getTool('DICT');
        $DICT->editEntry($msg, dict('GEN_LBL_FORMLABEL') . ':', 'UIFF_fieldformlabel', $record, $table, $form);
        $DICT->editEntry($msg, dict('GEN_LBL_PREFIX') . ':', 'UIFF_fieldprefix', $record, $table, $form);
        $DICT->editEntry($msg, dict('GEN_LBL_SUFIX') . ':', 'UIFF_fieldsufix', $record, $table, $form);
        $table->addField($msg, dict('GEN_LBL_DISPLAY_SIZE') . ':', 'UIFF_field_displaysize', (!empty($record['UIFF_field_displaysize'])) ? $record['UIFF_field_displaysize'] : '0', CLX_NUM_FOUR, CLX_NUM_FOUR);
        $table->addField($msg, dict('GEN_LBL_FIELD_SIZE') . ':', 'UIFF_field_size', (!empty($record['UIFF_field_size'])) ? $record['UIFF_field_size'] : '0', CLX_NUM_FOUR, CLX_NUM_FOUR);
        $table->addField($msg, dict('GEN_LBL_FIELD_DEFAULT') . ':', 'UIFF_field_default', $record['UIFF_field_default'], CLX_NUM_32, CLX_NUM_255);
        $table->addYesNo(dict('GEN_LBL_USE_USR_DEFAULT'), 'UIFF_use_usr_default', $record['UIFF_use_usr_default'], CLX_NUM_ZERO, false);
        $DICT->editEntry($msg, dict('GEN_LBL_CAPTION') . ':', 'UIFF_caption', $record, $table, $form);
        $table->addField($msg, dict('GEN_LBL_CSSCLASS') . ':', 'UIFF_caption_class', $record['UIFF_caption_class'], CLX_NUM_48, CLX_NUM_64);
        $form->addContents($table->source() . '<br>');
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('GEN_LBL_FIELD_PROPERTIES'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $pop = new HTMLSelect('UIFF_readonly');
        if (empty($record['UIFF_readonly'])) {
		  $record['UIFF_readonly'] = CLX_INIT_ZERO;
        }
        foreach ($this->events as $key => $val) {
		  $pop->addItem($val, $key, ($record['UIFF_readonly'] == $key));
        }
        $table->addData(dict('GEN_LBL_READONLY'), $pop->source());
        $pop = new HTMLSelect('UIFF_password');
        if (empty($record['UIFF_password'])) {
		  $record['UIFF_password'] = CLX_INIT_ZERO;
        }
        $pop->addItem(dict('GEN_LBL_YES'), '1', ($record['UIFF_password'] == CLX_NUM_ONE));
        $pop->addItem(dict('GEN_LBL_NO'), '0', ($record['UIFF_password'] == CLX_NUM_ZERO));
        $table->addData(dict('GEN_LBL_PASSWORD'), $pop->source());
        $table->addYesNo(dict('GEN_LBL_HIDDEN'), 'UIFF_hidden', $record['UIFF_hidden'], CLX_NUM_ZERO, false);
        $pop = new HTMLSelect('UIFF_required');
        if (!array_key_exists('UIFF_required', $record)) {
		  $record['UIFF_required'] = '';
        }
        if ($record['UIFF_required'] === '' || $record['UIFF_required'] == null) {
		  $record['UIFF_required'] = CLX_NUM_TWO;
        }
        $pop->addItem(dict('GEN_LBL_FIELD_DEFAULT'), '2', ($record['UIFF_required'] == CLX_NUM_TWO));
        $pop->addItem(dict('GEN_LBL_YES'), '1', ($record['UIFF_required'] == CLX_NUM_ONE));
        $pop->addItem(dict('GEN_LBL_NO'), '0', ($record['UIFF_required'] == CLX_NUM_ZERO));
        $table->addData(dict('GEN_LBL_REQUIRED'), $pop->source());
        $pop = new HTMLSelect('UIFF_password');
        $table->addYesNo(dict('GEN_LBL_PRELOAD_LIST'), 'UIFF_preload_list', $record['UIFF_preload_list'], CLX_NUM_ZERO, false);
        $table->addYesNo(dict('GEN_LBL_ADD_LIST_ENTRY'), 'UIFF_add_list_entry', $record['UIFF_add_list_entry'], CLX_NUM_ZERO, false);
        $pop = new HTMLSelect('UIFF_edit_mode');
        if (!array_key_exists('UIFF_edit_mode', $record)) {
		  $record['UIFF_edit_mode'] = '0';
        }
        if ($record['UIFF_edit_mode'] === '' || $record['UIFF_edit_mode'] == null) {
		  $record['UIFF_edit_mode'] = '0';
        }
        foreach ($this->edit_modes as $key => $val) {
		  $pop->addItem(dict($val), $key, ($record['UIFF_edit_mode'] == $key));
        }
        $table->addData(dict('GEN_LBL_EDIT_MODE'), $pop->source());
        //Related Field section
        $callingpage = peekHistory();
        $formcode = ClxCommon::$db->getField('CLX_UI_FORM_PANELS', 'FORMCODE', 'IDCODE=' . singleQuote($callingpage['panelid']));
        $tablecode = ClxCommon::$db->getField('CLX_UI_FORMS', 'TABLECODE', 'IDCODE=' . singleQuote($formcode));

        $sql = 'SELECT TF.TABLECODE, F.FIELDNAME, F.FIELDTYPE, F.FIELD_EXTRA1,F.FIELD_EXTRA2, F.FIELD_EXTRA3, F.FIELD_EXTRA4 , F.FIELDFORMLABEL FROM CLX_DB_TABLEFIELDS TF, CLX_DB_FIELDS F WHERE TF.FIELDNAME = F.FIELDNAME AND TF.TABLECODE=' . singleQuote($tablecode) . ' ORDER BY SEQNUM';
        $fieldslist = ClxCommon::$db->executeQuery($sql);

        $fields = new htmlSelect('UIFF_related');
        $fields->addItem('', '', ($record['UIFF_related'] === ''));
        $lookups = array();
        while ($fieldlist = ClxCommon::$db->getNextRecord($fieldslist)) {
		  //Store posible fields for grant parent relations
		  $fieldrec = ClxCommon::$db->getFields('CLX_DB_FIELDS', 'FIELDTYPE, FIELD_EXTRA1', 'FIELDNAME =' . singleQuote($fieldlist['FIELDNAME']));
		  if ($fieldrec['FIELDTYPE'] === 'LOOK') {
			 $lookups[$fieldrec['FIELD_EXTRA1']] = $fieldrec['FIELD_EXTRA1'];
		  }
		  unset($fieldrec);
		  $fields->addItem(dict($fieldlist['FIELDFORMLABEL']) . ' (' . $fieldlist['FIELDNAME'] . ')', $fieldlist['FIELDNAME'], ($fieldlist['FIELDNAME'] == $record['UIFF_related']));
        }
		ClxCommon::$db->freeResult($fieldslist);

        //Adding Fields form Grant parent tables
        $sql = 'SELECT TF.TABLECODE, F.FIELDNAME, F.FIELDTYPE, F.FIELD_EXTRA1,F.FIELD_EXTRA2, F.FIELD_EXTRA3, F.FIELD_EXTRA4, F.FIELDFORMLABEL FROM CLX_DB_TABLEFIELDS TF, CLX_DB_FIELDS F WHERE TF.FIELDNAME = F.FIELDNAME AND F.FIELDTYPE = ' . singleQuote('LOOK') . ' AND TF.TABLECODE IN (' . singleQuote(implode(CLX_CHAR_EMPTY_STRING, $lookups)) . ')';
        $results = ClxCommon::$db->executeQuery($sql);

        //Adding Parent Records
        $i = CLX_INIT_ZERO;
        while ($result = ClxCommon::$db->getNextRecord($results, ASSOCIATIVE)) {
		  if ($i == CLX_NUM_ZERO) {
			 $fields->addItem('', '', false);
			 $i++;
		  }
		  $fields->addItem(dict($result['FIELDFORMLABEL']) . ' (' . $result['TABLECODE'] . '.' . $result['FIELDNAME'] . ')', $result['TABLECODE'] . '.' . $result['FIELDNAME'], ($result['TABLECODE'] . '.' . $result['FIELDNAME'] == $record['UIFF_related']));
        }
		ClxCommon::$db->freeResult($results);
        $table->addData(dict('GEN_LBL_RELATEDDISPLAY') . ':', $fields->source());

        //Parent Field
        $parent = new htmlSelect('UIFF_field_parent');
        $parent->addItem('', '', CLX_NUM_ZERO);
        $sql = 'SELECT UIFF.FIELDNAME, UIFF.FIELDFORMLABEL FROM CLX_UI_FORMS UIFM, CLX_UI_FORM_PANEL_FIELDS UIFF, CLX_UI_FORM_PANELS UIFP, CLX_DB_FIELDS DBFL WHERE UIFP.IDCODE =  UIFF.PANELID AND UIFM.IDCODE = UIFP.FORMCODE AND DBFL.FIELDNAME = UIFF.FIELDNAME AND UIFF.ID <> ' . $record['UIFF_id'] . ' AND UIFP.FORMCODE = ' . singleQuote($formcode) . ' ORDER BY UIFF.FIELDNAME ASC';
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
		  $parent->addItem(dict($r['FIELDFORMLABEL']) . ' (' . $r['FIELDNAME'] . ')', $r['FIELDNAME'], ($record['UIFF_field_parent'] == $r['FIELDNAME']));
        }
		ClxCommon::$db->freeResult($rs);
        if (!empty($msg['DBLT_list_pfld'])) {
		  $table->addCell($msg['UIFF_parent_field'], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => $table->width));
        }
        $table->addData(dict('GEN_LBL_PARENT_FIELD'), $parent->source());
        //Optional Tablecode
        $DBTB = &getTool('DBTB');
        $DBTB->placeChooseTable($msg, dict('TOOL_UIFF_PROMPT_TABLECODE') . ':', 'UIFF_tablecode', $record['UIFF_tablecode'], $table);
        $CODE = &getTool('CODE');
        $scripts = $CODE->placeChooseCodeLibrary($msg, 'pattern_lang', dict('GEN_LBL_MASKLIB') . ':', 'UIFF_mask_code', $record['UIFF_mask_code'], $table);
        $UILG = &getTool('UILG');
        $table->addCell(dict('GEN_LBL_SRCH_LISTING_CAPTION'), array(CLX_PROP_CLASS => 'caption', CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $UILG->placeChooseListing($msg, dict('GEN_LBL_LISTING') . ':', 'UIFF_srch_listing', $record['UIFF_srch_listing'], $table);
        $form->addContents($scripts . $table->source());
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        if (!empty($modify)) {
		  $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_SAVE'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        } else {
		  $c = new htmlFormSubmit(dict('GEN_LBL_INCLUDE'), 'INCLUDE', $form->getProp(CLX_PROP_ID));
		  $buttons .= $c->source();
        }
        $c = new htmlButtonCancel();
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->addContents($table->source());
        return $form->source();
    }

    /**
     * Tool Edit Form Witdh spacer
     * @param string $ac
     * @param string $idlist
     * @param array $record
     * @param array $msg
     * @return string
     */
    function editFormSpacer($ac, $idlist, $record, $msg) {
        $modify = (UserRequest::$request['ac'] === 'MODIFY');
        $title = ($modify) ? dict('TOOL_UIFF_TITLE_MODIFY') : dict('TOOL_UIFF_TITLE_INCLUDE');
        $buttons = '';
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', $ac);
        $form->addArgument('confirm', '2');
        $form->addArgument('UIFF_id', (!empty($record['UIFF_id'])) ? $record['UIFF_id'] : '');
        $form->addArgument('UIFF_fieldname', $record['UIFF_fieldname']);
        $form->addArgument('UIFF_pf_type', $record['UIFF_pf_type']);
        $form->addArgument('idlist', toString($idlist));
        $form->addArgument('pattern_lang', 'PCRE');
        $form->addArgument('UIFF_fieldname', $record['UIFF_fieldname']);
        $form->addArgument('UIFF_fieldprefix', $record['UIFF_fieldprefix']);
        $form->addArgument('UIFF_field_displaysize', $record['UIFF_field_displaysize']);
        $form->addArgument('UIFF_field_size', $record['UIFF_field_size']);
        $form->addArgument('UIFF_field_default', $record['UIFF_field_default']);
        $form->addArgument('UIFF_caption', $record['UIFF_caption']);
        $form->addArgument('UIFF_fieldprefix_DICT', $record['UIFF_fieldprefix_DICT']);
        $form->addArgument('UIFF_fieldsufix_DICT', $record['UIFF_fieldsufix_DICT']);
        $form->addArgument('UIFF_caption_DICT', $record['UIFF_caption_DICT']);
        $form->addArgument('UIFF_readonly', $record['UIFF_readonly']);
        $form->addArgument('UIFF_password', $record['UIFF_password']);
        $form->addArgument('UIFF_hidden', $record['UIFF_hidden']);
        $form->addArgument('UIFF_preload_list', $record['UIFF_preload_list']);
        $form->addArgument('UIFF_required', $record['UIFF_required']);
        $form->addArgument('UIFF_edit_mode', $record['UIFF_edit_mode']);
        $form->addArgument('UIFF_related', $record['UIFF_related']);
        $form->addArgument('UIFF_field_parent', $record['UIFF_field_parent']);
        $form->addArgument('UIFF_tablecode', $record['UIFF_tablecode']);
        $form->addArgument('UIFF_mask_code', $record['UIFF_mask_code']);
        $form->addArgument('UIFF_srch_listing', $record['UIFF_srch_listing']);
        $form->addArgument('UIFF_use_usr_default', CLX_NUM_ZERO);
        $form->addArgument('UIFF_add_list_entry', CLX_NUM_ZERO);
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell($title, array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addData(dict('GEN_LBL_PF_TYPE'), dict($this->pf_types['SPACE']));
        $table->addYesNo(dict('GEN_LBL_ACTIVE'), 'UIFF_active', $record['UIFF_active'], CLX_NUM_20);
        $table->addField($msg, dict('GEN_LBL_CSSCLASS') . ':', 'UIFF_caption_class', $record['UIFF_caption_class'], CLX_NUM_48, CLX_NUM_64);
        $DICT = &getTool('DICT');
        $DICT->editEntry($msg, dict('GEN_LBL_FORMLABEL') . ':', 'UIFF_fieldformlabel', $record, $table, $form);
        $form->addContents($table->source());
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        if (!empty($modify)) {
		  $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_SAVE'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        }
        $c = new htmlButtonCancel();
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->addContents($table->source());
        return $form->source();
    }

    /**
     * Verify Tool Edit Forms
     * @param array $msg
     * @return bool
     */
    function checkForm(&$msg) {
        $msg = array();
        xValidate::integer($msg, 'UIFF_field_displaysize', UserRequest::$post['UIFF_field_displaysize'], CLX_NUM_ZERO, CLX_NUM_255, dict('GEN_LBL_DISPLAY_SIZE'));
        xValidate::integer($msg, 'UIFF_field_size', UserRequest::$post['UIFF_field_size'], CLX_NUM_ZERO, CLX_NUM_255, dict('GEN_LBL_FIELD_SIZE'));
        $sql = 'SELECT FIELD_SIZE FROM CLX_DB_FIELDS WHERE FIELDNAME = ' . singleQuote(UserRequest::$post['UIFF_fieldname']);
        $physical_field_size = ClxCommon::$db->getOne($sql);
        if (intval($physical_field_size) < intval(UserRequest::$post['UIFF_field_size'])) {
		  $msg['UIFF_field_size'] = dict('GEN_LBL_MAX_FIELD_ALLOWED') . ': ' . $physical_field_size;
        }
        if (strlen(UserRequest::$post['UIFF_tablecode']) > CLX_NUM_ZERO) {
		  $DBTB = &getTool('DBTB');
		  if (!$DBTB->tableExists(UserRequest::$post['UIFF_tablecode'])) {
			 $msg['UIFF_tablecode'] = dict('GEN_ERR_TABLE_DOES_NOT_EXISTS');
		  }
        }
        if (strlen(UserRequest::$post['UIFF_srch_listing']) > CLX_NUM_ZERO) {
		  $UILG = &getTool('UILG');
		  if (!$UILG->listingExists(UserRequest::$post['UIFF_srch_listing'])) {
			 $msg['UIFF_srch_listing'] = dict('GEN_ERR_LISTING_DOES_NOT_EXISTS');
		  }
        }
        if (!empty(UserRequest::$post['UIFF_mask_code'])) {
		  //check that mask exists and is
		  $CODE = &getTool('CODE');
		  $c = UserRequest::$post['UIFF_mask_code'];
		  if (!$CODE->codeLibraryExists($c, 'PCRE')) {
			 $msg['UIFF_mask_code'] = dict('EVNT_LBL_LIB_EXISTS');
		  }
        }
        $DICT = &getTool('DICT');
        if (UserRequest::$post['UIFF_pf_type'] === 'FIELD') {
		  $DICT->validateEntry('UIFF_fieldformlabel', UserRequest::$post, true, $msg);
		  $DICT->validateEntry('UIFF_fieldprefix', UserRequest::$post, false, $msg);
		  $DICT->validateEntry('UIFF_fieldsufix', UserRequest::$post, false, $msg);
		  $DICT->validateEntry('UIFF_caption', UserRequest::$post, false, $msg);
        }
        return count($msg) == CLX_NUM_ZERO;
    }

    /**
     * Builds confirm delete form for tool
     * @param obj $table
     * @param array $record
     */
    function confirmDeleteFill(&$table, $record) {
        $table->addData(dict('GEN_LBL_FIELDNAME') . ':', $record['UIFF_fieldname']);
    }

    /**
     * Build tool display form
     * @param $idlist
     * @return string
     */
    function viewFormFill($record) {
        addHistory('fieldname', $record['UIFF_fieldname']);
        addHistory('ownercode', $this->code);
        addHistory('ownerid', $record['UIFF_id']);
        $sql = 'SELECT UIFP.FORMCODE, UIPF.FIELDNAME FROM CLX_UI_FORM_PANEL_FIELDS UIPF, CLX_UI_FORM_PANELS UIFP WHERE UIFP.IDCODE = UIPF.PANELID AND UIPF.ID = ' . $record['UIFF_id'];
        $r = ClxCommon::$db->getOneRecord($sql);
        addHistory('formcode', $r ['FORMCODE']);
        $contents = '';
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_UIFF_TITLE_VIEW') . ': ' . $record['UIFF_fieldname'], array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addData(dict('GEN_LBL_FIELDNAME') . ':', $record['UIFF_fieldname']);
        $table->addData(dict('GEN_LBL_ACTIVE') . ':', dict($this->yesno[$record['UIFF_active']]));
        $table->addData(dict('GEN_LBL_FORMLABEL') . ':', dict($record['UIFF_fieldformlabel']));
        $table->addData(dict('GEN_LBL_PREFIX') . ':', dict($record['UIFF_fieldprefix']));
        $table->addData(dict('GEN_LBL_SUFIX') . ':', dict($record['UIFF_fieldsufix']));
        $table->addData(dict('GEN_LBL_DISPLAY_SIZE') . ':', $record['UIFF_field_displaysize']);
        $table->addData(dict('GEN_LBL_FIELD_SIZE') . ':', $record['UIFF_field_size']);
        $table->addData(dict('GEN_LBL_FIELD_DEFAULT') . ':', $record['UIFF_field_default']);
        $table->addData(dict('GEN_LBL_USE_USR_DEFAULT') . ':', dict($this->yesno[$record['UIFF_use_usr_default']]));
        $table->addData(dict('GEN_LBL_PRELOAD_LIST') . ':', dict($this->yesno[$record['UIFF_preload_list']]));
        $table->addData(dict('GEN_LBL_ADD_LIST_ENTRY') . ':', dict($this->yesno[$record['UIFF_add_list_entry']]));
        $table->addData(dict('GEN_LBL_RELATEDDISPLAY') . ':', $record['UIFF_related']);
        $table->addData(dict('GEN_LBL_FIELD_PARENT') . ':', $record['UIFF_field_parent']);
        $table->addData(dict('GEN_LBL_CAPTION') . ':', dict($record['UIFF_caption']));
        $table->addData(dict('GEN_LBL_CSSCLASS') . ':', $record['UIFF_caption_class']);
        $table->addData(dict('GEN_LBL_MASKING') . ':', $record['UIFF_mask_code']);
        $table->addData(dict('GEN_LBL_EDIT_MODE') . ':', dict($this->edit_modes[$record['UIFF_edit_mode']]));
        $table->addData(dict('GEN_LBL_SRCH_LISTING') . ':', $record['UIFF_srch_listing']);
        $contents .= $table->source();
        $UISC = &getTool('UISC');
        $contents .= $UISC->makepanel($this->code, $record['UIFF_id']);
        $UIFR = &getTool('UIFR');
        $contents .= $UIFR->makepanel($r ['FORMCODE'], $r ['FIELDNAME']);
        $EVNT = &getTool('EVNT');
        $contents .= $EVNT->makepanel($this->code, $record['UIFF_id']);
        return $contents;
    }

    /**
     * Creates reorder list
     * @return string
     */
    function reorderRecords() {
        $rs = ClxCommon::$db->getRecords($this->tablename, 'PANELID=' . singleQuote(UserRequest::$post['panelid']) . ' ORDER BY FIELDSEQNUM');
        $list = array();
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
		  if ($r['PF_TYPE'] === 'SPACE') {
			 $list[$r['ID']] = dict('GEN_LBL_SPACER') . '-' . dict($r['FIELDFORMLABEL']);
		  } else {
			 $list[$r['ID']] = $r['FIELDNAME'];
		  }
        }
		ClxCommon::$db->freeResult($rs);
        return $list;
    }

    /**
     * Performs reorder record action
     */
    function reorderAction() {
        parent::reorderAction();
        $callingpage = peekHistory();
        $panelid = $callingpage['panelid'];
        $UIFP = &getTool('UIFP');
        $UIFP->versionIncrease('IDCODE', $panelid);
    }

    /**
     * Add an spacer to metadata
     */
    function createSpacer() {
        $savepost = UserRequest::$post;
        UserRequest::$post = $this->defaultRecord();
        UserRequest::$post['UIFF_pf_type'] = 'SPACE';
        $id = $this->createRecord_();
        unset($id);
        UserRequest::$post = $savepost;
    }

    /**
     * Loads current record on selected id
     *
     * @param integer $id
     * @return array
     */
    function loadToolRecord($recordid) {

        $DICT = &getTool('DICT');
        $values = ClxCommon::$db->getRecord($this->tablename, 'ID= ' . $recordid);
        $record = array();
        $record['UIFF_id'] = $recordid;
        $record['UIFF_pf_type'] = $values['PF_TYPE'];
        $DICT->loadEntry('UIFF_caption', $values['CAPTION'], $record);
        $record['UIFF_caption_class'] = $values['CAPTION_CLASS'];
        $record['UIFF_fieldname'] = $values['FIELDNAME'];
        $record['UIFF_active'] = $values['ACTIVE'];
        $DICT->loadEntry('UIFF_fieldformlabel', $values['FIELDFORMLABEL'], $record);
        $DICT->loadEntry('UIFF_fieldprefix', $values['FIELDPREFIX'], $record);
        $DICT->loadEntry('UIFF_fieldsufix', $values['FIELDSUFIX'], $record);
		IF ($record['UIFF_fieldprefix_DICT']=='<b></b>' or $record['UIFF_fieldprefix_DICT']=='<!---->' )
			$record['UIFF_fieldprefix_DICT']='';
		IF ($record['UIFF_fieldsufix_DICT']=='<b></b>' or $record['UIFF_fieldsufix_DICT']=='<!---->' )
			$record['UIFF_fieldsufix_DICT']='';
		
        $record['UIFF_field_displaysize'] = $values['FIELD_DISPLAYSIZE'];
        $record['UIFF_field_size'] = $values['FIELD_SIZE'];
        $record['UIFF_field_default'] = $values['FIELD_DEFAULT'];
        $record['UIFF_related'] = $values['FIELD_RELATED'];
        $record['UIFF_fieldseqnum'] = $values['FIELDSEQNUM'];
        $record['UIFF_readonly'] = $values['FIELD_READONLY'];
        $record['UIFF_password'] = $values['FIELD_PASSWORD'];
        $record['UIFF_hidden'] = $values['FIELD_HIDDEN'];
        $record['UIFF_required'] = $values['FIELD_REQUIRED'];
        $record['UIFF_preload_list'] = $values['PRELOAD_LIST'];
        $record['UIFF_field_parent'] = $values['FIELD_PARENT'];
        $record['UIFF_tablecode'] = $values['TABLECODE'];
        $record['UIFF_mask_code'] = $values['MASK_CODE'];
        $record['UIFF_edit_mode'] = $values['EDIT_MODE'];
        $record['UIFF_srch_listing'] = $values['SRCH_LISTING'];
        $record['UIFF_use_usr_default'] = $values['USE_USR_DEFAULT'];
        $record['UIFF_add_list_entry'] = $values['ADD_LIST_ENTRY'];
        if (!array_key_exists('UIFF_tablecode', $record))
		  $record['UIFF_tablecode'] = '';
        if (!array_key_exists('UIFF_mask_code', $record))
		  $record['UIFF_mask_code'] = '';
        return $record;
    }

    /**
     * Update records
     *
     * @return integer
     */
    function saveRecord() {
        ClxFreeReturn(peekHistory());
        $DICT = &getTool('DICT');
        $record = array();
        $record['PF_TYPE'] = UserRequest::$post['UIFF_pf_type'];
        $record['CAPTION'] = $DICT->saveEntry('UIFF_caption', UserRequest::$post);
        $record['CAPTION_CLASS'] = UserRequest::$post['UIFF_caption_class'];
        $record['FIELDFORMLABEL'] = uppercase($DICT->saveEntry('UIFF_fieldformlabel', UserRequest::$post));
		//pre( UserRequest::$post);
		IF (empty( UserRequest::$post['UIFF_fieldprefix_DICT']))
			 UserRequest::$post['UIFF_fieldprefix_DICT']='<!---->';
			                                
		IF (empty( UserRequest::$post['UIFF_fieldsufix_DICT']))
			 UserRequest::$post['UIFF_fieldsufix_DICT']='<!---->';
	    $record['FIELDPREFIX'] = $DICT->saveEntry('UIFF_fieldprefix', UserRequest::$post);
        $record['FIELDSUFIX'] = $DICT->saveEntry('UIFF_fieldsufix', UserRequest::$post);
        $record['ACTIVE'] = UserRequest::$post['UIFF_active'];
        $record['FIELD_DISPLAYSIZE'] = UserRequest::$post['UIFF_field_displaysize'];
        $record['FIELD_SIZE'] = UserRequest::$post['UIFF_field_size'];
        $record['FIELD_SIZE'] = UserRequest::$post['UIFF_field_size'];
        $record['FIELD_DEFAULT'] = UserRequest::$post['UIFF_field_default'];
        $record['FIELD_RELATED'] = UserRequest::$post['UIFF_related'];
        $record['FIELD_READONLY'] = UserRequest::$post['UIFF_readonly'];
        $record['FIELD_PASSWORD'] = UserRequest::$post['UIFF_password'];
        $record['FIELD_HIDDEN'] = UserRequest::$post['UIFF_hidden'];
        $record['FIELD_REQUIRED'] = UserRequest::$post['UIFF_required'];
        $record['PRELOAD_LIST'] = UserRequest::$post['UIFF_preload_list'];
        $record['FIELD_PARENT'] = UserRequest::$post['UIFF_field_parent'];
        $record['TABLECODE'] = (UserRequest::$post['UIFF_tablecode'] === '') ? 'null' : UserRequest::$post['UIFF_tablecode'];
        $record['MASK_CODE'] = UserRequest::$post['UIFF_mask_code'];
        $record['EDIT_MODE'] = UserRequest::$post['UIFF_edit_mode'];
        $record['SRCH_LISTING'] = UserRequest::$post['UIFF_srch_listing'];
        $record['USE_USR_DEFAULT'] = (!empty(UserRequest::$post['UIFF_use_usr_default'])) ? CLX_NUM_ONE : CLX_NUM_ZERO;
        $record['ADD_LIST_ENTRY'] = (!empty(UserRequest::$post['UIFF_add_list_entry'])) ? CLX_NUM_ONE : CLX_NUM_ZERO;
        ClxCommon::$db->updateRecord($this->tablename, $record, 'ID=' . UserRequest::$post['UIFF_id']);
        $this->versionIncrease('ID', UserRequest::$post['UIFF_id']);
    }

    /**
     * Creates record
     *
     * @return integer
     */
    function createRecord_() {
        $callingpage = peekHistory();
        $DICT = &getTool('DICT');
        $record = array();
        $record['PANELID'] = $callingpage['panelid'];
        $record['PF_TYPE'] = UserRequest::$post['UIFF_pf_type'];
        $record['CAPTION'] = $DICT->createEntry('UIFF_caption', UserRequest::$post);
        $record['CAPTION_CLASS'] = UserRequest::$post['UIFF_caption_class'];
        $record['FIELDNAME'] = UserRequest::$post['UIFF_fieldname'];
        $record['ACTIVE'] = UserRequest::$post['UIFF_active'];
        $record['FIELDSEQNUM'] = ClxCommon::$db->getField($this->tablename, 'FIELDSEQNUM', 'PANELID=' . singleQuote($callingpage['panelid']) . ' ORDER BY FIELDSEQNUM DESC') + CLX_NUM_ONE;
        $record['FIELDFORMLABEL'] = $DICT->createEntry('UIFF_fieldformlabel', UserRequest::$post);
        IF (empty( UserRequest::$post['UIFF_fieldprefix_DICT']))
			 UserRequest::$post['UIFF_fieldprefix_DICT']='<!---->';
			                                
		IF (empty( UserRequest::$post['UIFF_fieldsufix_DICT']))
			 UserRequest::$post['UIFF_fieldsufix_DICT']='<!---->';
		$record['FIELDPREFIX'] = $DICT->createEntry('UIFF_fieldprefix', UserRequest::$post);
        $record['FIELDSUFIX'] = $DICT->createEntry('UIFF_fieldsufix', UserRequest::$post);
        $record['FIELD_DISPLAYSIZE'] = UserRequest::$post['UIFF_field_displaysize'];
        $record['FIELD_SIZE'] = UserRequest::$post['UIFF_field_size'];
        $record['FIELD_DEFAULT'] = UserRequest::$post['UIFF_field_default'];
        $record['FIELD_RELATED'] = UserRequest::$post['UIFF_related'];
        $record['FIELD_READONLY'] = UserRequest::$post['UIFF_readonly'];
        $record['FIELD_PASSWORD'] = UserRequest::$post['UIFF_password'];
        $record['FIELD_HIDDEN'] = UserRequest::$post['UIFF_hidden'];
        $record['FIELD_REQUIRED'] = UserRequest::$post['UIFF_required'];
        $record['PRELOAD_LIST'] = UserRequest::$post['UIFF_preload_list'];
        $record['TABLECODE'] = (UserRequest::$post['UIFF_tablecode'] === '') ? 'null' : UserRequest::$post['UIFF_tablecode'];
        $record['FIELD_PARENT'] = UserRequest::$post['UIFF_field_parent'];
        $record['MASK_CODE'] = UserRequest::$post['UIFF_mask_code'];
        $record['EDIT_MODE'] = UserRequest::$post['UIFF_edit_mode'];
        $record['SRCH_LISTING'] = UserRequest::$post['UIFF_srch_listing'];
        $record['USE_USR_DEFAULT'] = UserRequest::$post['UIFF_use_usr_default'];
        $record['ADD_LIST_ENTRY'] = (!empty(UserRequest::$post['UIFF_add_list_entry'])) ? CLX_NUM_ONE : CLX_NUM_ZERO;
        $recordid = ClxCommon::$db->createRecord($this->tablename, $record, false);
        $this->versionIncrease('ID', $recordid);
        return $recordid;
    }

    /**
     * Deletes record on selected id
     *
     * @param integer $id
     */
    function deleteRecordSk($id) {
        $UISC = &getTool('UISC');
        $sql = 'SELECT ID FROM ' . $UISC->tablename . ' WHERE PFID = ' . $id;
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
		  $UISC->deleteRecordSk($r['ID']);
        }
		ClxCommon::$db->freeResult($rs);
        $UIFR = &getTool('UIFR');
        $sql = 'SELECT UIFR.ID FROM CLX_UI_FORM_PANEL_FIELDS UIPF, CLX_UI_FORM_PANELS UIFP, CLX_UI_FORM_FLD_PROP UIFR WHERE UIFP.IDCODE = UIPF.PANELID AND UIFR.FORMCODE = UIFP.FORMCODE AND UIFR.FIELDNAME = UIPF.FIELDNAME AND UIPF.ID = ' . $id;
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
		  $UIFR->deleteRecordSk($r['ID']);
        }
		ClxCommon::$db->freeResult($rs);


        $this->versionIncrease('ID', $id);
        ClxCommon::$db->deleteRecords($this->tablename, 'ID=' . $id);
        ClxCommon::$db->deleteRecords('CLX_EVENTS', 'OWNERCODE=' . singleQuote($this->code) . ' AND OWNERID=' . singlequote($id));
    }

    /**
     * Defines default record on initializacion
     *
     * @return array
     */
    function defaultRecord() {
        $record = array('UIFF_id' => '0', 'UIFF_idname' => '',
		  'UIFF_formcode' => '', 'UIFF_fieldname' => '',
		  'UIFF_active' => '1', 'UIFF_pf_type' => 'FIELD',
		  'UIFF_fieldformlabel' => false, 'UIFF_fieldformlabel_DICT' => '',
		  'UIFF_fieldprefix' => false, 'UIFF_fieldprefix_DICT' => '',
		  'UIFF_fieldsufix' => false, 'UIFF_description' => '',
		  'UIFF_field_displaysize' => '0', 'UIFF_field_size' => '0',
		  'UIFF_field_default' => '', 'UIFF_related' => '',
		  'UIFF_field_parent' => '', 'UIFF_caption' => '',
		  'UIFF_caption_DICT' => '', 'UIFF_caption_class' => CLX_CLASS_DATA,
		  'UIFF_fieldsufix_DICT' => '', 'UIFF_type' => CLX_INIT_ZERO,
		  'UIFF_seqnum' => CLX_INIT_ZERO, 'UIFF_initial_state' => CLX_NUM_ONE,
		  'UIFF_readonly' => CLX_INIT_ZERO, 'UIFF_password' => CLX_INIT_ZERO,
		  'UIFF_hidden' => CLX_INIT_ZERO, 'UIFF_preload_list' => CLX_INIT_ZERO,
		  'UIFF_required' => CLX_NUM_TWO, 'UIFF_tablecode' => '',
		  'UIFF_mask_code' => '', 'UIFF_edit_mode' => CLX_INIT_ZERO,
		  'UIFF_srch_listing' => '', 'UIFF_use_usr_default' => CLX_INIT_ZERO,
		  'UIFF_add_list_entry' => CLX_INIT_ZERO);
        return $record;
    }

    /**
     * Increase tool version
     * @param string $fieldname
     * @param string $fieldvalue
     */
    function versionIncrease($fieldname, $fieldvalue) {
        $panelid = ClxCommon::$db->getField($this->tablename, 'PANELID', $fieldname . '=' . singleQuote($fieldvalue));
        $formcode = ClxCommon::$db->getField('CLX_UI_FORM_PANELS', 'FORMCODE', 'IDCODE=' . singleQuote($panelid));
        $UIFM = &getTool('UIFM');
        $UIFM->versionIncrease('IDCODE', $formcode);
    }

}
