<?php

/**
 * Pages Row Objects Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: jclerma $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2017-11-03 15:09:40 -0400 (Fri, 03 Nov 2017) $
 * @version      $Rev: 9615 $
 *
 * CHANGE LOG
 *  Included label
 */
class ui_page_row_objects extends clx_tool {

    // CONSTRUCTOR //
    function __construct() {
        $this->code = 'UIPO';
        $this->name = 'Page Row Objects';
        $this->system = 'Interface';
        $this->version = '0.2.3.0';

        $this->accesskey = 'UIPG';


        $this->tablename = 'CLX_UI_PAGE_OBJECTS';
        $this->record_name_singular = 'row object';
        $this->record_name_plural = 'row objects';
        $this->primary_field = 'ID';
        $this->secondary_field = 'OBJECTCODE';
        $this->seqnum_field = 'SEQNUM';
        $this->yesno = array('GEN_LBL_NO', 'GEN_LBL_YES');
        $this->form_actions = array('QFILTER' => 'GEN_LBL_QFILTER', 'SEARCH' => 'GEN_LBL_SEARCH',
		  'DISPLAY' => 'GEN_LBL_DISPLAY',
		  'CREATE' => 'GEN_LBL_CREATE',
		  'UPDATE' => 'GEN_LBL_UPDATE');
    }

    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['IDROW'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['TITLE'] = array('type' => 'varchar(64)', 'null' => 'YES', 'properties' => '', 'dict' => true);
        $fields['SEQNUM'] = array('type' => 'tinyint(4) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['TOOLCODE'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '');
        $fields['OBJECTCODE'] = array('type' => 'varchar(32)', 'null' => 'YES', 'properties' => '');
        $fields['FORM_ACTION'] = array('type' => 'varchar(16)', 'null' => 'YES', 'properties' => '');
        $fields['ACTIVE'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'default' => '1', 'properties' => '');
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'MUL', 'fieldname' => 'IDROW');
        $indexes[$ipf . '03'] = array('index' => 'MUL', 'fieldname' => 'TITLE');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB',
		  'fields' => $fields,
		  'indexes' => $indexes);
    }

    function pack($record) {
        $o = $this->packRecord($record);
        $objs = new XMLObject($this->code . '_OBJECTS');
        $this->packsub($objs, 'UIPC', 'SELECT * FROM CLX_UI_PAGE_OBJ_CRITS WHERE OBJECTID=' . $record['ID']);
        $o->addObject($objs);
        return $o;
    }

    function unpack($parameters) {
        $parameters = array_values(array_merge(array('package' => null, 'id' => ''), $parameters));
        list($package, $formid) = $parameters;
        $record = $this->unpackRecord($package);
        $record['IDROW'] = $formid;
        $objectid = ClxCommon::$db->createRecord($this->tablename, $record, false);
        if ($package->hasSubObject($this->code . '_OBJECTS')) {
		  $objs = $package->getObjectXml($this->code . '_OBJECTS');
		  foreach ($objs->objects as $o) {
			 $toolcode = $o->tag;
			 $tool = &getTool($toolcode);
			 $tool->unpack(array('package' => $o, 'id' => $objectid));
		  }
        }
    }

    function action($ac, $idlist) {
        $msg = array();
        if ($this->getAccess()->canRead()) {
		  switch ($ac) {
			 case 'CTOOL':
			     if ($this->getAccess()->canDisplay()) {
			         if (isset(UserRequest::$post['confirm'])) {
					   $record = $this->defaultRecord();
					   $toolcode = UserRequest::$post['UIPO_toolcode'];
					   $tool = &getTool($toolcode);
					   $test = $tool->pageletform();
					   if (empty($test)) {
						  $contents = $this->editForm('CREATE', array(), $record, $msg);
					   } else {
						  UserRequest::$post['UIPO_objectcode'] = '';
						  ClxFreeReturn($this->createRecord_());
						  $info = popHistory();
						  $contents = executeHistory($info);
					   }
			         } else {
					   $record = array();
					   $contents = $this->chooseToolForm($record, $msg);
			         }
			     } else {
			         $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [UI_PAGE_ROW_OBJECTS-CAN_APPEND][' . $ac . '] ' . $this->getCode(false));
			     }
			     break;
			 default:
			     $contents = parent::action($ac, $idlist);
			     break;
		  }
        } else {
		  $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [UI_PAGE_ROW_OBJECTS-CAN_READ][' . $ac . '] ' . $this->getCode(false));
        }
        return $contents;
    }

    function makepanel($groupcode = null) {
        if ($this->getAccess()->canRead()) {
		  $t = time();
		  $to = $this->code;
		  $here = 'tool.php?to=' . $to . '&t=' . $t;
		  $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
		  $form->setProp(CLX_PROP_ID, 'UIPOPanel');
		  $form->addArgument(CLX_PROP_TO, $this->code);
		  $form->addArgument('idrow', $groupcode);
		  $form->addArgument('ac', 'NOT_DEFINED');
		  $selectclause = '*';
		  $fromclause = 'CLX_UI_PAGE_OBJECTS';
		  $whereclause = 'IDROW=' . $groupcode;
		  $sortclause = 'SEQNUM ASC';
		  $page = new xPaging();
		  $page->setProp('rpp', CLX_NUM_TEN_THOUSAND);
		  $page->setProp('printheaders', false);
		  $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
		  $table = new htmlTable(CLX_WIDTH_FIVE);
		  $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
		  $page->header($table);
		  $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_TOOL'), dict('GEN_LBL_OBJECT'), dict('GEN_LBL_ACTIVE'), dict('GEN_LBL_TITLE')));
		  $i = CLX_INIT_ZERO;
		  while ($record = $page->nextRecord()) {
			 $recordid = $record['ID'];
			 $select = new htmlCheckbox('selector_' . $recordid, false);
			 $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
			 $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
			 $tool = &getTool($record['TOOLCODE']);
			 if (strlen($record['OBJECTCODE']) > CLX_NUM_ZERO) {
			     $objectrec = ClxCommon::$db->getRecord($tool->tablename, 'IDCODE=' . singleQuote($record['OBJECTCODE']));
			     $objectdesc = $objectrec['IDNAME'] . ' (' . $objectrec['IDCODE'] . ') ';
			     if (!empty($record['FORM_ACTION'])) {
			         $objectdesc .= dict('GEN_LBL_ACTION') . ': ' .
						  dict($this->form_actions[$record['FORM_ACTION']]);
			     }
			 } else {
			     $objectdesc = $tool->name;
			 }
			 $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array(
			     $tool->name . ' (' . $tool->code . ')', $objectdesc, dict($this->yesno[$record['ACTIVE']]), dict($record['TITLE'])));
			 $i++;
		  }
		  $buttons = '';
		  $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_DISPLAY'), array('action' => 'DISPLAY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
		  if ($this->getAccess()->canModify()) {
			 $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
		  }
		  if ($this->getAccess()->canDelete()) {
			 $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_EXCLUDE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
		  }
		  if ($this->getAccess()->canDisplay()) {
			 $b = new htmlFormSubmit(dict('GEN_LBL_INCLUDE_SOMETHING', dict('GEN_LBL_OBJECT')), 'CTOOL', $form->getProp(CLX_PROP_ID));
			 $b->setProp(CLX_PROP_ID, 'ON');
			 $buttons .= $b->source();
		  }
		  if (($page->getProp('qty') > CLX_NUM_ONE) && $this->getAccess()->canModify()) {
			 $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_REORDER'), array('action' => 'REORDER', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
		  }
		  $form->addContents($page->footers($table, $buttons));
		  return buildPanelInterface(dict('TOOL_UIPO_TITLE_LIST'), 'TABLES', $form->source(), -1 * CLX_NUM_ONE);
        } else {
		  return CLX_CHAR_EMPTY_STRING;
        }
    }

    function chooseToolForm($record, $msg) {
        ClxFreeReturn($record);
        ClxFreeReturn($msg);
        $callingpage = peekHistory();
        $idrow = $callingpage['idrow'];
        $title = dict('TOOL_UIPO_TITLE_CHOOSETOOL');
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'NOT_DEFINED');
        $form->addArgument('confirm', '1');
        $form->addArgument('UIPO_idrow', $idrow);
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell($title, array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $tools = ClxCommon::$db->getRecords('CLX_TOOLS', 'HAS_PAGELET=1 ORDER BY IDNAME');
        $pop = new htmlSelect('UIPO_toolcode');
        while ($toolrec = ClxCommon::$db->getNextRecord($tools)) {
		  $pop->addItem($toolrec['IDNAME'], $toolrec['IDCODE'], false);
        }
		ClxCommon::$db->freeResult($tools);
        $table->addData(dict('GEN_LBL_TOOL') . ':', $pop->source());
        $form->addContents($table->source());
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $c = new htmlFormSubmit(dict('GEN_LBL_SELECT'), 'CTOOL', $form->getProp(CLX_PROP_ID));
        $buttons = $c->source();
        $c = new htmlButtonCancel();
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->addContents($table->source());
        return $form->source();
    }

    function editForm($ac, $idlist, $record, $msg) {
        $modify = (UserRequest::$request['ac'] === 'MODIFY');
        $title = ($modify) ? dict('TOOL_UIPO_TITLE_MODIFY') : dict('TOOL_UIPO_TITLE_CREATE');
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', $ac);
        $form->addArgument('confirm', '1');
        $form->addArgument('idlist', toString($idlist));
        $form->addArgument('UIPO_id', $record['UIPO_id']);
        $form->addArgument('UIPO_idrow', $record['UIPO_idrow']);
        $form->addArgument('UIPO_toolcode', $record['UIPO_toolcode']);
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell($title, array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $tool = &getTool($record['UIPO_toolcode']);
        $table->addData(dict('GEN_LBL_TOOL') . ':', $tool->name . ' (' . $tool->code . ')');
        $DICT = &getTool('DICT');
        $DICT->editEntry($msg, dict('GEN_LBL_TITLE') . ':', 'UIPO_title', $record, $table, $form);
        if (isset($msg['UIPO_objectcode'])) {
		  $table->addCell($msg['UIPO_objectcode'], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        }

        if ($record['UIPO_toolcode'] === 'USTK') {
		  $form->addArgument('UIPO_objectcode', CLX_CHAR_EMPTY_STRING);
        } else {
		  $source = $tool->chooseRecord('UIPO_objectcode', $record['UIPO_objectcode']);
		  $table->addData(dict('GEN_LBL_OBJECT') . ':', $source);
        }


        if ($record['UIPO_toolcode'] === 'UIFM') {
		  $form_action = new htmlSelect('UIPO_form_action');
		  foreach ($this->form_actions as $key => $val) {
			 $form_action->addItem(dict($val), $key, ($record['UIPO_form_action'] == $key));
		  }
		  $table->addData(dict('GEN_LBL_FORM_ACTION') . ':', $form_action->source());
		  unset($form_action);
        }
        $table->addYesNo(dict('GEN_LBL_ACTIVE'), 'UIPO_active', $record['UIPO_active'], CLX_WIDTH_TWO, true);
        $form->addContents($table->source());
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $buttons = '';
        if (!empty($modify)) {
		  $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_SAVE'), array('action' => $ac, 'form' => $form->getProp(CLX_PROP_ID)))->source();
        } else {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE'), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        }
        $c = new htmlButtonCancel();
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->addContents($table->source());
        return $form->source();
    }

    function viewFormFill($record) {
        addHistory('idrow', $record['UIPO_id']);
        $contents = '';
        $title = dict('TOOL_UIPO_TITLE_DISPLAY');
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell($title, array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addData(dict('GEN_LBL_ID') . ':', $record['UIPO_id']);
        $table->addData(dict('GEN_LBL_OBJECT') . ':', $record['UIPO_objectcode']);
        $table->addData(dict('GEN_LBL_TOOL') . ':', $record['UIPO_toolcode']);
        $table->addData(dict('GEN_LBL_ACTIVE') . ':', dict($this->yesno[$record['UIPO_active']]));
        $contents .= $table->source();
        if ($record['UIPO_toolcode'] === 'UILG' || $record['UIPO_toolcode'] === 'PLOT' || $record['UIPO_toolcode'] === 'ANPL' || ($record['UIPO_toolcode'] === 'UIFM' && $record['UIPO_form_action'] <> 'SEARCH' && $record['UIPO_form_action'] <> 'QFILTER')) {


		  //Criterias
		  $UIPC = &getTool('UIPC');
		  $contents .= $UIPC->makepanel($record['UIPO_id']);
        }
        return $contents;
    }

    function checkForm(&$msg) {
        $msg = array();
        //$action = UserRequest::$post['ac'];
        if (UserRequest::$post['UIPO_toolcode'] !== 'USTK') {
		  $tool = &getTool(UserRequest::$post['UIPO_toolcode']);
		  $tool->recordExists('UIPO_objectcode', UserRequest::$post['UIPO_objectcode'], $msg);
        }
        return count($msg) == CLX_NUM_ZERO;
    }

    function confirmDeleteFill(&$table, $record) {
        $table->addData(dict('GEN_LBL_NAME') . ':', $record['UIPO_toolcode']);
    }

    function reorderRecords() {
        $records = ClxCommon::$db->getRecords($this->tablename, 'IDROW=' . singleQuote(UserRequest::$post['idrow']) . ' ORDER BY SEQNUM');
        $list = array();
        while (($record = ClxCommon::$db->getNextRecord($records))) {
		  $tool = &getTool($record['TOOLCODE']);
		  $objectrec = ClxCommon::$db->getRecord($tool->tablename, 'IDCODE=' . singleQuote($record['OBJECTCODE']));
		  $list[$record['ID']] = $objectrec['IDNAME'] . ' (' . $objectrec['IDCODE'] . ')';
        }
		ClxCommon::$db->freeResult($records);
			
        return $list;
    }

    function reorderAction() {
        parent::reorderAction();
        $callingpage = peekHistory();
        $idrow = $callingpage['idrow'];
        $UIPR = &getTool('UIPR');
        $UIPR->versionIncrease(CLX_PROP_ID, $idrow);
    }

    function loadToolRecord($recordid) {
        $DICT = &getTool('DICT');
        $values = ClxCommon::$db->getRecord($this->tablename, 'ID= ' . $recordid);
        $record = array();
        $record['UIPO_id'] = $recordid;
        $DICT->loadEntry('UIPO_title', $values['TITLE'], $record);
        $record['UIPO_seqnum'] = $values['SEQNUM'];
        $record['UIPO_idrow'] = $values['IDROW'];
        $record['UIPO_toolcode'] = $values['TOOLCODE'];
        $record['UIPO_objectcode'] = $values['OBJECTCODE'];
        $record['UIPO_form_action'] = $values['FORM_ACTION'];
        $record['UIPO_active'] = $values['ACTIVE'];
        return $record;
    }

    function saveRecord() {


        $record = array();
        if (array_key_exists('UIPO_title_DICT', UserRequest::$post)) {
		  $DICT = &getTool('DICT');
		  $record['TITLE'] = $DICT->createEntry('UIPO_title', UserRequest::$post);
        }
        $record['OBJECTCODE'] = UserRequest::$post['UIPO_objectcode'];
        $record['ACTIVE'] = UserRequest::$post['UIPO_active'];
        if (array_key_exists('UIPO_form_action', UserRequest::$post)) {
		  $record['FORM_ACTION'] = UserRequest::$post['UIPO_form_action'];
        } else {
		  $record['FORM_ACTION'] = 'null';
        }
        ClxCommon::$db->updateRecord($this->tablename, $record, 'ID=' . UserRequest::$post['UIPO_id']);
        $UIPR = &getTool('UIPR');
        $UIPR->versionIncrease('ID', UserRequest::$post['UIPO_idrow']);
    }

    function createRecord_() {

        $record = array();
        $record['IDROW'] = UserRequest::$post['UIPO_idrow'];
        if (array_key_exists('UIPO_title_DICT', UserRequest::$post)) {
		  $DICT = &getTool('DICT');
		  $record['TITLE'] = $DICT->createEntry('UIPO_title', UserRequest::$post);
        }
        $record['TOOLCODE'] = UserRequest::$post['UIPO_toolcode'];
        $record['OBJECTCODE'] = UserRequest::$post['UIPO_objectcode'];
        $record['ACTIVE'] = (empty(UserRequest::$post['UIPO_active'])) ? 0 : 1;
        $record['SEQNUM'] = ClxCommon::$db->getField('CLX_UI_PAGE_OBJECTS', 'SEQNUM', 'IDROW=' . UserRequest::$post['UIPO_idrow'] .
			         ' ORDER BY SEQNUM DESC') + CLX_NUM_ONE;
        if (array_key_exists('UIPO_form_action', UserRequest::$post)) {
		  $record['FORM_ACTION'] = UserRequest::$post['UIPO_form_action'];
        } else {
		  $record['FORM_ACTION'] = 'null';
        }
        $recordid = ClxCommon::$db->createRecord($this->tablename, $record, false);
        $UIPR = &getTool('UIPR');
        $UIPR->versionIncrease('ID', UserRequest::$post['UIPO_idrow']);
        return $recordid;
    }

    function deleteRecordSk($id) {
        $objrec = ClxCommon::$db->getRecord($this->tablename, 'ID=' . singleQuote($id));
        ClxCommon::$db->deleteRecords($this->tablename, 'ID=' . $id);
        $UIPR = &getTool('UIPR');
        $UIPR->versionIncrease('ID', $objrec['IDROW']);
    }

    function defaultRecord() {
        ClxFreeReturn(peekHistory());
        $record = array();
        $record['UIPO_id'] = CLX_INIT_ZERO;
        $record['UIPO_seqnum'] = CLX_INIT_ZERO;
        $record['UIPO_title'] = '';
        $record['UIPO_active'] = CLX_NUM_ONE;
        $record['UIPO_title_DICT'] = '';
        $record['UIPO_idrow'] = UserRequest::$post['UIPO_idrow'];
        $record['UIPO_toolcode'] = UserRequest::$post['UIPO_toolcode'];
        $record['UIPO_objectcode'] = '';
        $record['UIPO_form_action'] = '';
        return $record;
    }

}
