<?php

/**
 * ui_views
 * For more information contact  support@alenet.com
 *
 * @author       $Author: alenetdev $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2019-01-22 15:45:22 -0400 (ma., 22 ene. 2019) $
 * @version    	$Rev: 10496 $
 *
 * Change log
 *  Added setProp:'MULT' on shortlist buttons
 *  Changed access to accessed on selects from
 *  Oracle uppercase record changed
 *  Dictionary Changes
 */
class ui_views extends clx_tool {

    var $access_types;
    var $directions;
    var $ownership;

    // CONSTRUCTOR //
    function __construct() {
        $this->code = 'UIVW';
        $this->name = 'View Manager';
        $this->system = 'Interface';
        $this->version = '0.2.5.5';

        $this->has_access = true;


        $this->tablename = 'CLX_UI_VIEWS';
        $this->record_name_singular = 'view';
        $this->record_name_plural = 'views';
        $this->primary_field = 'QUERYCODE';
        $this->secondary_field = 'IDNAME';

        $this->access_types = array(dict('GEN_LBL_PRIVATE'), dict('GEN_LBL_PUBLIC'));
        $this->directions = array('DESC' => 'DESC', 'ASC' => 'ASC');
        $this->ownership = array('GEN_LBL_NO', 'GEN_LBL_YES');
    }

    function install() {
        $UIMI = getTool('UIMI');
        $UIMI->add($this->code, 'MENU_TOOLS:MENU_INTERFACE:MENU_VIEWS', 'TOOL', $this->code, 'ac=LIST');
    }

    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['IDNAME'] = array('type' => 'varchar(64)', 'null' => 'NO', 'properties' => '');
        $fields['DESCRIPTION'] = array('type' => 'mediumtext', 'null' => 'YES', 'properties' => '');
        $fields['QUERYCODE'] = array('type' => 'varchar(16)', 'null' => 'NO', 'properties' => '');
        $fields['USERNAME'] = array('type' => 'varchar(30)', 'null' => 'NO', 'properties' => '');
        $fields['ACCESSES'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'MUL', 'fieldname' => 'IDNAME');
        $indexes[$ipf . '03'] = array('index' => 'MUL', 'fieldname' => 'QUERYCODE');
        $indexes[$ipf . '04'] = array('index' => 'MUL', 'fieldname' => 'USERNAME');
        return array('idcode' => $this->code,
		  'idname' => $this->name,
		  'name' => $this->tablename,
		  'type' => 'InnoDB',
		  'fields' => $fields,
		  'indexes' => $indexes);
    }

    function action($ac, $idlist) {
        $msg = array();
        if ($this->getAccess()->canRead()) {
		  switch ($ac) {
			 case 'CQUERY':
			     if (isset(UserRequest::$post['confirm'])) {
			         if ($this->checkchooseQueryForm($msg)) {
					   $record = $this->defaultRecord();
					   $contents = $this->editForm('CREATE', array(), $record, $msg);
			         } else {
					   $contents = $this->chooseQueryForm(UserRequest::$post, $msg);
			         }
			     } else {
			         $info = peekHistory();
			         if (isset($info['qy'])) {
					   UserRequest::$post['UIVW_querycode'] = $info['qy'];
					   $record = $this->defaultRecord();
					   $contents = $this->editForm('CREATE', array(), $record, $msg);
			         } else {
					   $record = array();
					   $contents = $this->chooseQueryForm($record, $msg);
			         }
			     }
			     break;
			 case 'CREATE':
			     if ($this->getAccess()->canDisplay()) {
			         if (isset(UserRequest::$post['confirm'])) {
					   $this->createRecordVoid_();
					   $info = popHistory();
					   $contents = executeHistory($info);
			         } else {
					   $record = $this->defaultRecord();
					   $contents = $this->editForm($ac, array(), $record, $msg);
			         }
			     } else {
			         $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [UI_VIEWS-CAN_APPEND][' . $ac . '] ' . $this->getCode(false));
			     }
			     break;
			 case 'MODIFY':
			     if ($this->getAccess()->canModify()) {
			         if (isset(UserRequest::$post['confirm'])) {
					   $this->saveRecord();
					   if (count($idlist) > CLX_NUM_ZERO) {
						  $id = array_pop($idlist);
						  $record = $this->loadToolRecord($id);
						  $contents = $this->editForm($ac, $idlist, $record, $msg);
					   } else {
						  $info = popHistory();
						  $contents = executeHistory($info);
					   }
			         } else {
					   $id = array_pop($idlist);
					   $record = $this->loadToolRecord($id);
					   // Cannot modify if not the owner or administrator
					   if ($record['UIVW_username'] != $_SESSION['user_record']['username'] && empty($_SESSION['user_record']['isadmin'])) {
						  $contents = note(dict('TOOL_UIVW_ERR_CANNOTEDIT'), dict('GEN_LBL_RETURN'));
					   } else {
						  $contents = $this->editForm($ac, $idlist, $record, $msg);
					   }
			         }
			     } else {
			         $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [UI_VIEWS-CAN_MODIFY][' . $ac . '] ' . $this->getCode(false));
			     }
			     break;
			 case 'DELETE':
			     if ($this->getAccess()->canDelete()) {
			         $id = array_pop($idlist);
			         $record = $this->loadToolRecord($id);
			         // Cannot modify if not the owner or administrator
			         if ($record['UIVW_username'] != $_SESSION['user_record']['username'] && empty($_SESSION['user_record']['isadmin'])) {
					   $contents = note(dict('TOOL_UIVW_ERR_CANNOTDELETE'), dict('GEN_LBL_RETURN'));
			         } else {
					   array_push($idlist, $id);
					   $contents = parent::action($ac, $idlist);
			         }
			     } else {
			         $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [UI_VIEWS-CAN_DELETE][' . $ac . '] ' . $this->getCode(false));
			     }
			     break;
			 case 'PREF':
			     $id = array_pop($idlist);
			     $this->setPref($id);
			     $info = popHistory();
			     $contents = executeHistory($info);
			     break;
			 case 'NOPREF':
			     $id = array_pop($idlist);
			     $this->unsetPref($id);
			     $info = popHistory();
			     $contents = executeHistory($info);
			     break;
			 case 'SHORTLIST':
			     $querycode = UserRequest::$request['qy'];
			     $contents = $this->makeshortlist($querycode);
			     break;
			 default:
			     $contents = parent::action($ac, $idlist);
			     break;
		  }
        } else {
		  $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [UI_VIEWS-CAN_READ][' . $ac . '] ' . $this->getCode(false));
        }
        return $contents;
    }

    function makelist($listingcode = null) {
       ClxFreeReturn($listingcode);
         //$t = time('U'); genera un warning
        $t = time();
        $to = $this->code;
        $action = (!empty(UserRequest::$server['PHP_SELF'])) ? UserRequest::$server['PHP_SELF'] : 'tool.php';
        $here = $action . '?to=' . $to.'&t=' . $t;
        $username = $_SESSION['user_record']['username'];
        clearHistory();
        pushHistory($this->code, 'LIST', array());
        $form = new htmlForm(CLX_PROP_ACTION, $action);
        $form->setProp(CLX_PROP_ID, 'UIVWList');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'NOT_DEFINED');
        $selectclause = '*';
        $fromclause = 'CLX_UI_VIEWS';
        $whereclause = ($this->getAccess()->isAdmin()) ? CLX_CHAR_EMPTY_STRING : '(USERNAME=' . singleQuote(uppercase($username)) . ' OR ACCESSES=1)';

        $sortclause = '';
        $page = new xPaging();
        $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
        $table = new htmlTable(CLX_WIDTH_SIX);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST));

        $table->addCell(dict('TOOL_UIVW_TITLE_LIST'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_SIX));
        $page->header($table);
        $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_NAME'), dict('GEN_LBL_QUERYCODE'), dict('GEN_LBL_DESCRIPTION'), dict('GEN_LBL_ACCESS'), dict('GEN_LBL_OWNERNAME')));
        $i = CLX_INIT_ZERO;
        while ($record = $page->nextRecord()) {
		  $recordid = $record['ID'];
		  $select = new htmlCheckbox('selector_' . $recordid, false);
		  $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
		  $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
		  $querycode = $record['QUERYCODE'];
            if (ClxCommon::$db->oneExistsRecords('CLX_UI_VIEW_PREFS', 'UPPER(QUERYCODE)=' . singleQuote(UPPERCASE($querycode)) . ' AND UPPER( USERNAME)=' . singleQuote(uppercase($username)) . ' AND VIEWID=' . $recordid) > CLX_NUM_ZERO) {
			 $name = '<b>' . $record['IDNAME'] . '</b>';
		  } else {
			 $name = $record['IDNAME'];
		  }
		  $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($name,
			 $querycode,
			 xFormat::text($record['DESCRIPTION']),
			 $this->access_types[$record['ACCESSES']],
			 $record['USERNAME']));
		  $i++;
        }
        $buttons = '';
        if ($this->getAccess()->canModify()) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        if ($this->getAccess()->canDelete()) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        $b = new htmlFormSubmit(dict('GEN_LBL_SETPREFERED'), 'PREF', $form->getProp(CLX_PROP_ID));
        $b->setProp(CLX_PROP_ID, 'ONE');
        $buttons .= $b->source();
        $b = new htmlFormSubmit(dict('GEN_LBL_UNSETPREFERED'), 'NOPREF', $form->getProp(CLX_PROP_ID));
        $b->setProp(CLX_PROP_ID, 'ONE');
        $buttons .= $b->source();
        if ($this->getAccess()->canDisplay()) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE_SOMETHING', dict('GEN_LBL_VIEW')), array('action' => 'CQUERY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
        }
        $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_FILTER'), array('action' => 'FILTER', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
        $form->addContents($page->footers($table, $buttons));
        return $form->source();
    }

    function makeshortlist($querycode) {
        $t = time();
        $to = $this->code;
        $action = (!empty(UserRequest::$server['PHP_SELF'])) ? UserRequest::$server['PHP_SELF'] : 'tool.php';
        $here = $action . '?to=' . $to . '&t=' . $t;
        $username = $_SESSION['user_record']['username'];
        pushHistory($this->code, 'SHORTLIST', array(), 'qy', $querycode);
        $form = new htmlForm(CLX_PROP_ACTION, $action);
        $form->setProp(CLX_PROP_ID, 'UIVWShortList');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'NOT_DEFINED');
        $selectclause = '*';
        $fromclause = 'CLX_UI_VIEWS';
        $whereclause = '(USERNAME=' . singleQuote(uppercase($username)) . ' OR ACCESSES=1) AND UPPER(QUERYCODE)=' . singleQuote(UPPERCASE($querycode)) . ' ORDER BY USERNAME';
        $sortclause = '';
        $page = new xPaging();
        $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
        $table = new htmlTable(CLX_WIDTH_SIX);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_UIVW_TITLE_LIST'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_SIX));
        $page->header($table);
        $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_NAME'), dict('GEN_LBL_QUERYCODE'), dict('GEN_LBL_DESCRIPTION'), dict('GEN_LBL_ACCESS'), dict('GEN_LBL_OWNERNAME')));
        $i = CLX_INIT_ZERO;
        while ($record = $page->nextRecord()) {
		  $recordid = $record['ID'];
		  $select = new htmlCheckbox('selector_' . $recordid, false);
		  $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
		  $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
		  $querycode = $record['QUERYCODE'];
            if (ClxCommon::$db->oneExistsRecords('CLX_UI_VIEW_PREFS', 'UPPER(QUERYCODE)=' . singleQuote(UPPERCASE($querycode)) . ' AND UPPER( USERNAME)=' . singleQuote(uppercase($username)) . ' AND VIEWID=' . $recordid) > CLX_NUM_ZERO) {
			 $name = '<b>' . $record['IDNAME'] . '</b>';
		  } else {
			 $name = $record['IDNAME'];
		  }
		  $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($name, $querycode,
			 xFormat::text($record['DESCRIPTION']),
			 $this->access_types[$record['ACCESSES']],
			 $record['USERNAME']));
		  $i++;
        }
        $buttons = '';
        $selectButtons = '';
        $b = new htmlButtonReturn('', '', $action);
        $buttons .= $b->source();
        if ($this->getAccess()->canModify()) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        if ($this->getAccess()->canDelete()) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        $selectGroup = new htmlSpan('SELECTorbuttons', '', $selectButtons);
        $singleSelectorButtons = '';
        $b = new htmlFormSubmit(dict('GEN_LBL_SETPREFERED'), 'PREF', $form->getProp(CLX_PROP_ID));
        $b->setProp(CLX_PROP_ID, 'MULT');
        $singleSelectorButtons .= $b->source();
        $b = new htmlFormSubmit(dict('GEN_LBL_UNSETPREFERED'), 'NOPREF', $form->getProp(CLX_PROP_ID));
        $b->setProp(CLX_PROP_ID, 'ONE');
        $singleSelectorButtons .= $b->source();
        $singleSelectGroup = new htmlSpan('singleselectorbuttons', '', $singleSelectorButtons);
        if ($this->getAccess()->canDisplay()) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE_SOMETHING', dict('GEN_LBL_VIEW')), array('action' => 'CQUERY', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        }
        $form->addContents($page->footers($table, $selectGroup->source() . $singleSelectGroup->source() . $buttons));
        return $form->source();
    }

    function chooseQueryForm($record, $msg) {
        $title = dict('TOOL_UIVW_TITLE_CHOOSEQUERY');
        $action = (!empty(UserRequest::$server['PHP_SELF'])) ? UserRequest::$server['PHP_SELF'] : 'tool.php';
        $form = new htmlForm(CLX_PROP_ACTION, $action);
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'CQUERY');
        $form->addArgument('confirm', '1');
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell($title, array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $DBQY = &getTool('DBQY');
        $DBQY->placeChooseQuery($msg, dict('GEN_LBL_QUERYCODE') . ':', 'UIVW_querycode', (array_key_exists('UIVW_querycode', $record)) ? $record['UIVW_querycode'] : '', $table);
        $form->addContents($table->source());
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $c = new htmlFormSubmit(dict('GEN_LBL_SEL'), 'CQUERY', $form->getProp(CLX_PROP_ID));
        $buttons = $c->source();
        $c = new htmlButtonCancel('', '', $action);
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->addContents($table->source());
        return $form->source();
    }

    function checkchooseQueryForm(&$msg) {
        $msg = array();
        $DBQY = &getTool('DBQY');
        if (!$DBQY->queryExists(UserRequest::$post['UIVW_querycode'])) {
		  $msg['UIVW_querycode'] = dict('TOOL_UIVW_NOQUERYCODE');
		  /*
		    TOOL_UIVW_NOQUERYCODE
		    There is no query with this code
		    No se encontraron consultas con c�digo proporcionado.
		   */
        }
        return count($msg) == CLX_NUM_ZERO;
    }

    function editForm($ac, $idlist, $record, $msg) {

        $action = (!empty(UserRequest::$server['PHP_SELF'])) ? UserRequest::$server['PHP_SELF'] : 'tool.php';
        $modify = (UserRequest::$request['ac'] === 'MODIFY');
        $title = ($modify) ? dict('TOOL_UIVW_TITLE_MODIFY') : dict('TOOL_UIVW_TITLE_CREATE');
        $viewid = $record['UIVW_id'];
        $querycode = $record['UIVW_querycode'];
        $queryfields = $this->queryFields($querycode);

        $contents = setJavaScriptTag(getJavaScriptFilename('UIVW_editform.js'), true);


        $form = new htmlForm(CLX_PROP_ACTION, $action);
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', $ac);
        $form->addArgument('confirm', '1');
        $form->addArgument('idlist', toString($idlist));
        $form->addArgument('UIVW_id', $viewid);
        $form->addArgument('UIVW_querycode', $querycode);
        $form->addArgument('viewfields', '');
        $form->addArgument('sortfields', '');
        $form->setProp('onsubmit', 'return processForm()');

        // H E A D E R S ----------------------------------------------------------
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell($title, array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addData(dict('GEN_LBL_QUERYCODE') . ':', $record['UIVW_querycode']);
        $table->addField($msg, dict('GEN_LBL_NAME') . ':', 'UIVW_idname', $record['UIVW_idname'], CLX_NUM_48, CLX_NUM_64);
        $table->addText($msg, dict('GEN_LBL_DESCRIPTION') . ':', 'UIVW_description', $record['UIVW_description'], CLX_NUM_FOUR, CLX_NUM_48);
        $pop = new HTMLSelect('UIVW_accesses');
        foreach ($this->access_types as $key => $value) {
		  $pop->addItem($value, $key, (array_key_exists('UIVW_accesses', $record) && $record['UIVW_accesses'] == $key));
        }
        $table->addData(dict('GEN_LBL_ACCESS') . ':', $pop->source());
        $form->addContents($table->source());

        // C O L U M N S ----------------------------------------------------------
        $fields = array();
        if($viewid == CLX_NUM_ZERO){// new view
		  $records = false;
        }else{
		  $records = ClxCommon::$db->getRecords('CLX_UI_VIEW_COLUMNS', 'VIEWID=' . $viewid . ' ORDER BY SEQNUM');
		  while($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)){
			 $record['ID'] = $record['TABLECODE'] . '.' . $record['FIELDNAME'];
			 if(empty($queryfields[$record['ID']])){
			     continue;
			 }
			 $fields[$record['ID']] = $queryfields[$record['ID']];
		  }
        }
		 ClxCommon::$db->freeResult($records);



        /*
		GEN_LBL_VIEWCOLUMNS
		View Columns
		Campos Mostrados
         */
        $table = new htmlTable(CLX_WIDTH_FOUR);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('GEN_LBL_VIEWCOLUMNS'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_FOUR));
        $table->addCell(dict('GEN_LBL_AVA_COL'), array(CLX_PROP_CLASS => CLX_CLASS_PROMPT, CLX_PROP_COLSPAN => '2'));
        $table->addCell(dict('GEN_LBL_SEL_COL'), array(CLX_PROP_CLASS => CLX_CLASS_PROMPT, CLX_PROP_COLSPAN => '2'));

        // Columns
        $masterList = new htmlSelect('masterFieldList');
        $masterList->setProp('size', '10');
        foreach ($queryfields as $id => $prompt) {
		  if (!key_exists($id, $fields)) {
			 $masterList->addItem($prompt, $id, false);
		  }
        }

        $list = new htmlSelect('fieldList');
        $list->setProp('size', '10');
        foreach ($fields as $id => $prompt) {
		  $list->addItem($prompt, $id, false);
        }


        $table->addCell($masterList->source(), array(CLX_PROP_CLASS => CLX_CLASS_DATA));
        $buttons = '<input type="button" value="' . dict('GEN_LBL_MOVE') . ' >>" onclick="changeList(' . singleQuote('masterFieldList') . ',' . singleQuote('fieldList') . ')"><br>';
        $buttons .= '<input type="button" value="<< ' . dict('GEN_LBL_MOVE') . '" onclick="changeList(' . singleQuote('fieldList') . ', ' . singleQuote('masterFieldList') . ')"><br>';
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_DATA));
        $table->addCell($list->source(), array(CLX_PROP_CLASS => CLX_CLASS_DATA));
        $buttons = '';
        $buttons .= '<input type="button" value="' . dict('GEN_LBL_MOVEUP') . '" onclick="moveItem(' . singleQuote('fieldList') . ', -1)"><br>';
        $buttons .= '<input type="button" value="' . dict('GEN_LBL_MOVEDOWN') . '" onclick="moveItem(' . singleQuote('fieldList') . ', 1)"><br>';
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_DATA));
        $form->addContents($table->source());
        // S O R T S  ----------------------------------------------------------
        $records = ClxCommon::$db->getRecords('CLX_UI_VIEW_SORTS', 'VIEWID=' . $viewid . ' ORDER BY SEQNUM');
        $fields = array();
        while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
		  $record['ID'] = $record['TABLECODE'] . '.' . $record['FIELDNAME'];
		  if (empty($queryfields[$record['ID']])) {
			 continue;
		  }
		  $fields[$record['ID']] = $queryfields[$record['ID']] . ' [' . $record['DIRECTION'] . ']';
        }
		 ClxCommon::$db->freeResult($records);
        /*
		GEN_LBL_SORTBY
		Sort By
		Ordenar Por
         */
        $table = new htmlTable(CLX_WIDTH_FOUR);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('GEN_LBL_ORDERBY'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_FOUR));
        $table->addCell(dict('GEN_LBL_AVA_COL'), array(CLX_PROP_CLASS => CLX_CLASS_PROMPT, CLX_PROP_COLSPAN => '2'));
        $table->addCell(dict('GEN_LBL_SEL_COL'), array(CLX_PROP_CLASS => CLX_CLASS_PROMPT, CLX_PROP_COLSPAN => '2'));
        // Master Field List
        $masterSortList = new htmlSelect('masterSortList');
        $masterSortList->setProp('size', '7');
        foreach ($queryfields as $id => $prompt) {
		  if (!key_exists($id, $fields)) {
			 $masterSortList->addItem($prompt . ' [ASC]', $id, false);
		  }
        }

        $sortList = new htmlSelect('sortList');
        $sortList->setProp('size', '7');
        foreach ($fields as $id => $prompt) {
		  $sortList->addItem($prompt, $id, false);
        }
        $table->addCell($masterSortList->source(), array(CLX_PROP_CLASS => CLX_CLASS_DATA));
        $buttons = '<input type="button" value="' . dict('GEN_LBL_MOVE') . ' >>" onclick="changeList(' . singleQuote('masterSortList') . ', ' . singleQuote('sortList') . ')"><br>';
        $buttons .= '<input type="button" value="<< ' . dict('GEN_LBL_MOVE') . '" onclick="changeList(' . singleQuote('sortList') . ', ' . singleQuote('masterSortList') . ')"><br>';
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_DATA));
        $table->addCell($sortList->source(), array(CLX_PROP_CLASS => CLX_CLASS_DATA));

        $buttons = '<input type="button" value="' . dict('GEN_LBL_BTN_CHGDIR') . '" onclick="changeDirection();"><br>';
        $buttons .= '<input type="button" value="' . dict('GEN_LBL_MOVEUP') . '" onclick="moveItem(' . singleQuote('sortList') . ', -1)"><br>';
        $buttons .= '<input type="button" value="' . dict('GEN_LBL_MOVEDOWN') . '" onclick="moveItem(' . singleQuote('sortList') . ', 1)"><br>';
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_DATA));
        $form->addContents($table->source());

        // F O O T E R S ----------------------------------------------------------
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $buttons = '';
        if (!empty($modify)) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_SAVE'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        } else {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE'), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        }
        $c = new htmlButtonCancel(dict('GEN_LBL_CANCEL'), '', $action);
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->addContents($table->source());
        $contents .= $form->source();
        return $contents;
    }

    function queryFields($querycode) { // Check if this can not be changed to DBQY
        $sql = "SELECT
			     TF.TABLECODE,
			     TF.FIELDNAME,
			     F.FIELDLISTLABEL,
			     T.MODULECODE,
			     T.IDNAME,
			     TF.SEQNUM,
			     'F' TIPO
			 FROM
			     CLX_DB_QUERIES Q,
			     CLX_DB_QUERY_TABLES QT,
			     CLX_DB_TABLEFIELDS TF,
			     CLX_DB_TABLES T,
			     CLX_DB_FIELDS F
			 WHERE
			     Q.IDCODE = " . singleQuote($querycode) . "
			     AND Q.IDCODE = QT.QUERYCODE
			     AND QT.TABLECODE = TF.TABLECODE
			     AND TF.TABLECODE = T.IDCODE
			     AND F.FIELDNAME = TF.FIELDNAME
			 UNION ALL
			 SELECT
			     TF.TABLECODE,
			     F.FIELDNAME,
			     F.FIELDLISTLABEL,
			     T.MODULECODE,
			     T.IDNAME,
			     TF.SEQNUM,
			     'D' TIPO
			 FROM
			     CLX_DB_QUERIES Q,
			     CLX_DB_QUERY_TABLES QT,
			     CLX_DB_TABLEDFIELDS TF,
			     CLX_DB_TABLES T,
			     CLX_DB_FIELDS F,
			     CLX_DB_QUERY_FIELDS QF
			 WHERE
			     Q.IDCODE = " . singleQuote($querycode) . "
			     AND Q.IDCODE = QT.QUERYCODE
			     AND QT.TABLECODE = TF.TABLECODE
			     AND TF.TABLECODE = T.IDCODE
			     AND F.FIELDNAME = TF.FIELDNAME
			     AND Q.IDCODE = QF.QUERYCODE
			     AND QF.FIELDNAME = CONCAT(F.FIELDNAME, '_WRK')
			     AND QF.EXPRESSION IS NOT NULL
			     AND TF.TABLECODE = QF.TABLECODE
			 ORDER BY
			     TIPO DESC,
			     TABLECODE ASC,
			     SEQNUM ASC
			 ";
        
        $fields = ClxCommon::$db->executeQuery($sql);
        $list = array();
        while ($field = ClxCommon::$db->getNextRecord($fields)) {
		  $field['LONGFIELDNAME'] = $field['MODULECODE'] . '_' . $field['IDNAME'];
		  $field['ID'] = $field['TABLECODE'] . '.' . $field['FIELDNAME'];
		  $list[$field['ID']] = dict($field['FIELDLISTLABEL'], false) . ' (' . $field['TABLECODE'] . '_' . $field['FIELDNAME'] . ')';
        }
		 ClxCommon::$db->freeResult($fields);
        return $list;
    }

    function confirmDeleteFill(&$table, $record) {
        $table->addData(dict('GEN_LBL_QUERYCODE') . ':', $record['UIVW_querycode']);
        $table->addData(dict('GEN_LBL_NAME') . ':', $record['UIVW_idname']);
    }

    function filterFormFill($msg, &$table) {
        if (!array_key_exists('UIVW_querycode', UserRequest::$post)) {
		  UserRequest::$post['UIVW_querycode'] = '';
        }
        if (!array_key_exists('UIVW_idname', UserRequest::$post)) {
		  UserRequest::$post['UIVW_idname'] = '';
        }
        $table->addField($msg, dict('GEN_LBL_QUERYCODE') . ':', 'UIVW_querycode', UserRequest::$post['UIVW_querycode'], CLX_NUM_16, CLX_NUM_16);
        $table->addField($msg, dict('GEN_LBL_NAME') . ':', 'UIVW_idname', UserRequest::$post['UIVW_idname'], CLX_NUM_32, CLX_NUM_64);
        $pop = new HTMLSelect('UIVW_accesses');
        $pop->addItem('', '', true);
        foreach ($this->access_types as $key => $value) {
		  $pop->addItem($value, $key, false);
        }
        $table->addData(dict('GEN_LBL_ACCESS') . ':', $pop->source());
        $sort = new HtmlSelect('UIVW_sort');
        $sort->addItem('', '', true);
        $sort->addItem(dict('GEN_LBL_QUERYCODE'), 'querycode', false);
        $sort->addItem(dict('GEN_LBL_NAME'), 'idname', false);
        $table->addData(dict('GEN_LBL_ORDERBY') . ':', $sort->source());
    }

    function setfilter() {
        $filter = '';
        $boolean = '';
        if (!empty(UserRequest::$post['UIVW_querycode'])) {
		  $filter .= $boolean . ' UPPER(QUERYCODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['UIVW_querycode']) . '%');
		  $boolean = 'AND';
        }
        if (!empty(UserRequest::$post['UIVW_idname'])) {
		  $filter .= $boolean . ' UPPER(IDNAME) LIKE ' . singleQuote(uppercase(clx_change_especial_char_html (UserRequest::$post['UIVW_idname'])) . '%');
		  $boolean = 'AND';
        }
        if (!empty(UserRequest::$post['UIVW_accesses'])) {
		  $filter .= $boolean . ' ACCESSES = ' . UserRequest::$post['UIVW_accesses'];
		  $boolean = 'AND';
        }
        $_SESSION['paging']['UIVWList']['sortby'] = UserRequest::$post['UIVW_sort'];
        $_SESSION['paging']['UIVWList']['criteria'] = $filter;
        $_SESSION['paging']['UIVWList']['startat'] = '0';
    }
	///////////////////////////////////////////////////
	///comentado 22/01/2019 para utilizar la clx_tool
	//////////////////////////////////////////////////
    /*function clearfilter() {
        $_SESSION['paging']['UIVWList']['criteria'] = '';
    }*/

    function loadToolRecord($recordid) {
        $values = ClxCommon::$db->getRecord($this->tablename, 'ID= ' . $recordid);
        $record = array();
        $record['UIVW_id'] = $recordid;
        $record['UIVW_idname'] = $values['IDNAME'];
        $record['UIVW_username'] = $values['USERNAME'];
        $record['UIVW_description'] = $values['DESCRIPTION'];
        $record['UIVW_querycode'] = $values['QUERYCODE'];
        $record['UIVW_accesses'] = $values['ACCESSES'];
        return $record;
    }

    function saveRecord() {
        $record = array();
        $record['IDNAME'] = UserRequest::$post['UIVW_idname'];
        $record['DESCRIPTION'] = UserRequest::$post['UIVW_description'];
        $record['ACCESSES'] = UserRequest::$post['UIVW_accesses'];
        ClxCommon::$db->updateRecord($this->tablename, $record, 'ID=' . UserRequest::$post['UIVW_id']);
        $this->saveDetails(UserRequest::$post['UIVW_id']);
    }

    function createRecordVoid_() {
        ClxFreeReturn($this->createRecord_());
    }

    function createRecord_() {
        $record = array();
        $record['IDNAME'] = UserRequest::$post['UIVW_idname'];
        $record['DESCRIPTION'] = UserRequest::$post['UIVW_description'];
        $record['QUERYCODE'] = UserRequest::$post['UIVW_querycode'];
        $record['ACCESSES'] = UserRequest::$post['UIVW_accesses'];
        $record['USERNAME'] = $_SESSION['user_record']['username'];
        $recordid = ClxCommon::$db->createRecord($this->tablename, $record, false);
        $this->saveDetails($recordid);
        return $recordid;
    }

    function deleteRecordSk($id) {
        ClxCommon::$db->deleteRecords('CLX_UI_VIEW_COLUMNS', 'VIEWID=' . $id);
        ClxCommon::$db->deleteRecords('CLX_UI_VIEW_SORTS', 'VIEWID=' . $id);
        ClxCommon::$db->deleteRecords('CLX_UI_VIEW_PREFS', 'VIEWID=' . $id);
        ClxCommon::$db->deleteRecords($this->tablename, 'ID=' . $id);
    }

    function defaultRecord() {
        $record = array();
        $record['UIVW_id'] = CLX_INIT_ZERO;
        $record['UIVW_idname'] = '';
        $record['UIVW_description'] = '';
        $record['UIVW_querycode'] = UserRequest::$post['UIVW_querycode'];
        $record['UIVW_username'] = $_SESSION['user_record']['username'];
        $record['UIVW_accesses'] = '1';
        return $record;
    }

    function saveDetails($viewid) {
        $record = array();
        $record['VIEWID'] = $viewid;
        ClxCommon::$db->deleteRecords('CLX_UI_VIEW_COLUMNS', 'VIEWID=' . $viewid);
        if (strlen(UserRequest::$post['viewfields']) > CLX_NUM_ZERO) {
		  $viewfields = toArray(UserRequest::$post['viewfields']);
		  $i = CLX_INIT_ZERO;
		  foreach ($viewfields as $fieldid) {
			 $field = str_replace('+', '', $fieldid);
			 $field = str_replace('-', '', $field);
			 $field = explode('.', $field);
			 $record['FIELDID'] = CLX_INIT_ZERO;
			 $record['TABLECODE'] = $field[CLX_NUM_ZERO];
			 $record['FIELDNAME'] = $field[CLX_NUM_ONE];
			 $record['SEQNUM'] = $i;
			 ClxCommon::$db->createRecord('CLX_UI_VIEW_COLUMNS', $record, false);
			 $i++;
		  }
        }
        ClxCommon::$db->deleteRecords('CLX_UI_VIEW_SORTS', 'VIEWID=' . $viewid);
        if (strlen(UserRequest::$post['sortfields']) > CLX_NUM_ZERO) {
		  $sortfields = toArray(UserRequest::$post['sortfields']);
		  $i = CLX_INIT_ZERO;
		  foreach ($sortfields as $fieldid) {
			 $field = str_replace('+', '', $fieldid);
			 $field = str_replace('-', '', $field);
			 $field = explode('.', $field);
			 $record['FIELDID'] = CLX_INIT_ZERO;
			 $record['TABLECODE'] = $field[CLX_NUM_ZERO];
			 $record['FIELDNAME'] = $field[CLX_NUM_ONE];
			 $record['DIRECTION'] = (strpos($fieldid, '+') !== CLX_NUM_ZERO) ? 'ASC' : 'DESC';
			 $record['SEQNUM'] = $i;
			 ClxCommon::$db->createRecord('CLX_UI_VIEW_SORTS', $record, false);
			 $i++;
		  }
        }
    }

    function setPref($viewid) {
        $viewrec = ClxCommon::$db->getRecord('CLX_UI_VIEWS', 'ID=' . $viewid);
        $querycode = $viewrec['QUERYCODE'];
        $username = $_SESSION['user_record']['username'];
        ClxCommon::$db->deleteRecords('CLX_UI_VIEW_PREFS', 'UPPER(QUERYCODE)=' . singleQuote(UPPERCASE($querycode)) . ' AND  UPPER(USERNAME)=' . singleQuote(uppercase($username)));
        $record = array();
        $record['VIEWID'] = $viewid;
        $record['QUERYCODE'] = $querycode;
        $record['USERNAME'] = $username;
        ClxCommon::$db->createRecord('CLX_UI_VIEW_PREFS', $record, false);
    }

    function unsetPref($viewid) {
        $viewrec = ClxCommon::$db->getRecord('CLX_UI_VIEWS', 'ID=' . $viewid);
        $querycode = $viewrec['QUERYCODE'];
        $username = $_SESSION['user_record']['username'];
        ClxCommon::$db->deleteRecords('CLX_UI_VIEW_PREFS', 'UPPER(QUERYCODE)=' . singleQuote(UPPERCASE($querycode)) . ' AND  UPPER(USERNAME)=' . singleQuote(uppercase($username)));
    }

    function loadView($querycode) {
        $view = array();
        $username = $_SESSION['user_record']['username'];
        $DBQY = &getTool('DBQY');
        $queryrec = $DBQY->getquery($this->code, $querycode);

		if (ClxCommon::$db->oneExistsRecords('CLX_UI_VIEW_PREFS', 'UPPER(QUERYCODE)=' . singleQuote(UPPERCASE($querycode)) . ' AND  UPPER(USERNAME)=' . singleQuote(uppercase($username))) > CLX_NUM_ZERO) {
		  $prefrec = ClxCommon::$db->getRecord('CLX_UI_VIEW_PREFS', 'UPPER(QUERYCODE)=' . singleQuote(UPPERCASE($querycode)) . ' AND  UPPER(USERNAME)=' . singleQuote(uppercase($username)));
		  $view = $this->applyView($queryrec, $prefrec['VIEWID']);
	} else {
	    $view = $this->createViewFromQuery($queryrec);
	}
		
				$querycode=$queryrec[1];
				
				IF (($querycode=='REQUESTS_0111' || $querycode=='MY_REQUESTS11')) {
					$ACCO=$this->loadViewColumnsTablecode('ACCO');
				
					$view['COLUMNS']['ACCO_IDNAME']=$ACCO['ACCO_IDNAME'];
					$view['COLUMNS']['ACCO_LEGAL_NAME']=$ACCO['ACCO_LEGAL_NAME'];
					$view['COLUMNS']['SRRQ_IBS_CUSCUN']=$ACCO['ACCO_IBS_CUSCUN'];
				//	$f=$this->loadViewColumn('NUM_CASE')['_NUM_CASE'];
				//	$f['TABLECODE']='SQQR';
					//$view['COLUMNS']['SRRQ_NUM_CASE_WRK']=$f;
					
				//	$SRRQ['SRRQ_ACTI_PEND'];
					//PRE($queryrec);
					
				//	DIE;
			}
		
	return $view;
    }

    function applyView($queryrec, $viewid) {
        $columns = array();
        $view = array();
        $viewrec = ClxCommon::$db->getRecord('CLX_UI_VIEWS', 'ID=' . $viewid);
        $selectclause = '';
        $sortclause = '';
        $this->loadViewColumns($viewid, $columns, $selectclause);
        $this->loadViewSorts($viewid, $sortclause);
        $view['SUBTITLE'] = $viewrec['IDNAME'];
        $view['COLUMNS'] = $columns;
        $view['EXPRESSIONS'] = (!empty($queryrec['EXPRESSIONS'])) ? $queryrec['EXPRESSIONS'] : '';
        $view['SELECTCLAUSE'] = $selectclause;
        $view['FROMCLAUSE'] = $queryrec['FROMCLAUSE'];
        $view['WHERECLAUSE'] = $queryrec['WHERECLAUSE'];
        $view['GROUPCLAUSE'] = $queryrec['GROUPCLAUSE'];

        //$view["SORTCLAUSE"] = (strlen($sortclause)==0) ? $queryrec["SORTCLAUSE"] : $sortclause;
        $view["SORTCLAUSE"] = (strlen($sortclause)===CLX_NUM_ZERO) ? '' : $sortclause;
        
        return $view;
    }

    function loadViewColumns($viewid, &$columns, &$selectclause){
        $DBFL = getTool('DBFL');
        $sql = "SELECT TF.TABLECODE, F.*, C.SEQNUM, 'F' TIPO FROM CLX_UI_VIEWS V, CLX_UI_VIEW_COLUMNS C, CLX_DB_TABLEFIELDS TF, CLX_DB_FIELDS F WHERE V.ID = " . $viewid . " AND C.VIEWID = V.ID AND TF.TABLECODE = C.TABLECODE AND TF.FIELDNAME = C.FIELDNAME AND F.FIELDNAME = TF.FIELDNAME UNION ALL SELECT TF.TABLECODE, F.*, C.SEQNUM, 'D' TIPO FROM CLX_UI_VIEWS V, CLX_UI_VIEW_COLUMNS C, CLX_DB_TABLEDFIELDS TF, CLX_DB_FIELDS F WHERE V.ID = " . $viewid . " AND C.VIEWID = V.ID AND TF.TABLECODE = C.TABLECODE AND TF.FIELDNAME = C.FIELDNAME AND F.FIELDNAME = TF.FIELDNAME ORDER BY SEQNUM ASC";
        $columns = array();
        $selectclause = array();
        $records = ClxCommon::$db->executeQuery($sql);
        while($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)){
		  $record['FIELDPREFIX_VAL'] = dict($record['FIELDPREFIX']);
		  $record['FIELDSUFIX_VAL'] = dict($record['FIELDSUFIX']);
		  $columns[$record['TABLECODE'] . '_' . $record['FIELDNAME']] = $record;
		  $fieldinstance = getToolObject('DBFT', $record['FIELDTYPE']);
		  
		  if($record["TIPO"] == "F")
			 $selectclause[] = $fieldinstance->queryFieldColumn($record['FIELDNAME'], $record["TABLECODE"]);
		  else{
			 //Campos Derivados
			 $col_name = $record["TABLECODE"] . '_' . $record['FIELDNAME'];
			 $sqlExpresion = "
						  SELECT
						      DBQF.EXPRESSION
						  FROM
						      CLX_DB_QUERY_FIELDS DBQF
						  WHERE
						      DBQF.FIELDNAME = CONCAT('" . $record['FIELDNAME'] . "', '_WRK')
						      AND DBQF.EXPRESSION <> ''
						      AND DBQF.EXPRESSION IS NOT NULL
						      AND DBQF.QUERYCODE = (SELECT
												UIVW.QUERYCODE
											   FROM
												CLX_UI_VIEWS UIVW
											   WHERE
												UIVW.ID = $viewid)
			         ";
			 $expresion = ClxCommon::$db->getOne($sqlExpresion);
			 if(empty($expresion))
			     $expresion = "NULL";
			 
			 $selectclause[] = $fieldinstance->queryFieldColumn($record['FIELDNAME'], $record["TABLECODE"], '(' . $expresion . ') ' . $col_name);
		  }
        }
		 ClxCommon::$db->freeResult($records);
        if(count($selectclause) > CLX_NUM_ZERO){
		  $selectclause = implode(', ', $selectclause);
        }else{
		  $selectclause = '';
        }
    }
	function loadViewColumnsTablecode($tablecode){
        $DBFL = getTool('DBFL');
        $sql = 'SELECT TF.TABLECODE, F.*, "F" TIPO FROM  CLX_DB_TABLEFIELDS TF, CLX_DB_FIELDS F WHERE TF.TABLECODE = ' . SINGLEQUOTE($tablecode) . ' AND F.FIELDNAME = TF.FIELDNAME UNION ALL SELECT TF.TABLECODE, F.*,  "D" TIPO FROM CLX_DB_TABLEDFIELDS TF, CLX_DB_FIELDS F WHERE TF.TABLECODE ='. singlequote($tablecode).'  AND F.FIELDNAME = TF.FIELDNAME ';
        $columns = array();
        $selectclause = array();
        $records = ClxCommon::$db->executeQuery($sql);
        while($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)){
            $record['FIELDPREFIX_VAL'] = dict($record['FIELDPREFIX']);
            $record['FIELDSUFIX_VAL'] = dict($record['FIELDSUFIX']);
            $columns[$record['TABLECODE'] . '_' . $record['FIELDNAME']] = $record;
        }
		return $columns;
    }
	function loadViewColumn($FieldName){
        $DBFL = getTool('DBFL');
        $sql = 'SELECT "" TABLECODE, F.*, "F" TIPO FROM CLX_DB_FIELDS F WHERE  F.FIELDNAME =  '.singlequote($FieldName);
        $columns = array();
        $selectclause = array();
        $records = ClxCommon::$db->executeQuery($sql);
        while($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)){
            $record['FIELDPREFIX_VAL'] = dict($record['FIELDPREFIX']);
            $record['FIELDSUFIX_VAL'] = dict($record['FIELDSUFIX']);
            $columns[$record['TABLECODE'] . '_' . $record['FIELDNAME']] = $record;
        }
		return $columns;
    }

    function loadViewSorts($viewid, &$sortclause){
        $DBFL = &getTool('DBFL');
        $sql = 'SELECT S.TABLECODE TABLECODE, S.FIELDNAME FIELDNAME, TF.ID PREFIX, S.DIRECTION SORT FROM CLX_UI_VIEWS V, CLX_UI_VIEW_SORTS S  LEFT JOIN CLX_DB_TABLEFIELDS TF ON TF.FIELDNAME = S.FIELDNAME AND TF.TABLECODE = S.TABLECODE WHERE V.ID = ' . $viewid . ' AND S.VIEWID = V.ID ORDER BY S.SEQNUM ASC';
        $sortclause = array();
        $records = ClxCommon::$db->executeQuery($sql);
        while($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)){
		  $fieldrec = $DBFL->getField($record['FIELDNAME']);
		  $fieldinstance = getToolObject('DBFT', $fieldrec['FIELDTYPE']);
		  if(!empty($record["PREFIX"]))
			 $sortclause[] = $fieldinstance->queryFieldSorts($record['FIELDNAME'], $record["TABLECODE"], $record['SORT'], false);
		  else
			 $sortclause[] = $fieldinstance->queryFieldSorts($record['FIELDNAME'], $record["TABLECODE"], $record['SORT'], true);
        }
		 ClxCommon::$db->freeResult($records);
		
        if(count($sortclause) > CLX_NUM_ZERO){
		  $sortclause = implode(', ', $sortclause);
        }else{
		  $sortclause = '';
        }
    }

    function createViewFromQuery(&$queryrec) {
        $view = array();
        $view['SUBTITLE'] = '';
        $view['COLUMNS'] = $queryrec['COLUMNS'];
        $view['EXPRESSIONS'] = (empty($queryrec['EXPRESSIONS'])) ? '' : $queryrec['EXPRESSIONS'];
        $view['SELECTCLAUSE'] = $queryrec['SELECTCLAUSE'];
        $view['FROMCLAUSE'] = $queryrec['FROMCLAUSE'];
        $view['WHERECLAUSE'] = (empty($queryrec['WHERECLAUSE'])) ? '' : $queryrec['WHERECLAUSE'];
        $view['GROUPCLAUSE'] = (empty($queryrec['GROUPCLAUSE'])) ? '' : $queryrec['GROUPCLAUSE'];
        $view['SORTCLAUSE'] = (empty($queryrec['SORTCLAUSE'])) ? '' : $queryrec['SORTCLAUSE'];
        return $view;
    }

}
