<?php

/**
 * Tool Class
 * For more information contact  support@alenet.com
 *
 * @package		Clientix.Tool.User
 * @abstract		User Preferences
 * @author       $Author: cramirez $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2018-09-22 05:22:40 -0400 (Sat, 22 Sep 2018) $
 * @version        $Rev: 10119 $
 *
 * CHANGE LOG
 *  Included label
 */
class us_preferences extends clx_tool {

    /**
     * Constructor
     */
    function __construct() {
        $this->code = 'USPR';
        $this->name = 'Preferences';
        $this->system = 'User';
        $this->version = '0.3.1.2';

        $this->has_access = true;


        $this->tablename = 'CLX_US_PREFERENCES';
        $this->record_name_singular = 'user preference';
        $this->record_name_plural = 'user preferences';
        $this->primary_field = 'username';
        $this->secondary_field = 'PREFKEY';
    }

    /**
     * Install procedure
     *
     */
    function install() {
        $UIMI = getTool('UIMI');
        $UIMI->add($this->code, 'MENU_PERSONAL_INFORMATION:MENU_PREFERENCES', 'TOOL', $this->code, 'ac=MODIFY');
    }

    /**
     * Describes the tool table structure.
     * @return array
     */
    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['USERNAME'] = array('type' => 'varchar(30)', 'null' => 'NO', 'properties' => '');
        $fields['PREFKEY'] = array('type' => 'varchar(32)', 'null' => 'NO', 'properties' => '');
        $fields['PREFVALUE'] = array('type' => 'varchar(255)', 'null' => 'YES', 'properties' => '');
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'UNI', 'fieldname' => 'USERNAME,PREFKEY');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB', 'fields' => $fields, 'indexes' => $indexes);
    }

    /**
     * Action is called for every user interaction with the module...
     *
     * @param unknown_type $ac
     * @param unknown_type $idlist
     * @return unknown
     */
    function action($ac, $idlist) {
        $contents = '';
        $msg = array();
        if ($this->getAccess()->canRead()) {
		  switch ($ac) {
			 case 'MODIFY':
			     if ($this->getAccess()->canModify()) {
			         if (isset(UserRequest::$post['confirm'])) {
					   if ($this->prfCheckForm($msg)) {
						  $this->prfSaveRecord();
						  $contents = note(dict('GEN_LBL_PREF_SAVE'), dict('GEN_LBL_CONTINUE'));
					   } else {
						  $contents = $this->prfForm(UserRequest::$post, $msg);
					   }
			         } else {
					   $msg = array();
					   $record = $this->prfLoadRecord();
					   $contents = $this->prfForm($record, $msg);
			         }
			     } else {
			         $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [US_PREFERENCES-CAN_MODIFY][' . $ac . '] ' . $this->getCode(false));
			     }
			     break;
			 default:
			     $contents = parent::action($ac, $idlist);
			     break;
		  }
        } else {
		  $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [US_PREFERENCES-CAN_READ][' . $ac . '] ' . $this->getCode(false));
        }
        return $contents;
    }

    /**
     * Builds Preference Form
     * @param array $record
     * @param array $msg
     * @return array
     */
    function prfForm($record, $msg) {
        $title = dict('TOOL_PREF_TITLE_MODIFY');
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'MODIFY');
        $form->addArgument('confirm', '1');
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM));
        $table->addCell($title, array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        // Global preferences
        $table->addCell(dict('TOOL_PREF_TITLE_GLOBALPREFS'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        // Record per page
        $table->addField($msg, dict('GEN_LBL_RECPERPAGE'), 'PAGING_RPP', $record['PAGING_RPP'], CLX_NUM_FOUR, CLX_NUM_FOUR);
        // Panel state
        $yesno = new htmlSelect('PANEL_STATE');
        $yesno->addItem(dict('GEN_LBL_EXPANDED'), '1', ($record['PANEL_STATE'] === '1'));
        $yesno->addItem(dict('GEN_LBL_COLAPSED'), '0', ($record['PANEL_STATE'] === '0'));
        $table->addData(dict('GEN_LBL_PANELDEFSTATE'), $yesno->source());
        // Case insensitive
        $yesno = new htmlSelect('LSRCH_CI');
        $yesno->addItem(dict('GEN_LBL_YES'), '1', ($record['LSRCH_CI'] === '1'));
        $yesno->addItem(dict('GEN_LBL_NO'), '0', ($record['LSRCH_CI'] === '0'));
        $table->addData(dict('GEN_LBL_LSRCH_CI'), $yesno->source());
        // ClientTools specifics
        $tools = ClxCommon::$db->getRecords('CLX_TOOLS', 'HAS_PREFERENCES = 1');
        while ($tool = ClxCommon::$db->getNextRecord($tools)) {
		  $t = &getTool($tool['IDCODE']);
		  $t->preferencesForm($record, $form, $table);
        }
		ClxCommon::$db->freeResult($tools);
		
        $buttons = htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        $c1 = new htmlButtonGoto(dict('GEN_LBL_CANCEL'), 'index.php', null);
        $buttons .= $c1->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->setContents($table->source());
        $contents = $form->source();
        return $contents;
    }

    /**
     * Verify preference form
     * @param array $msg
     * @return bool
     */
    function prfCheckForm(&$msg) {
        $msg = array();
		UserRequest::$post['PAGING_RPP'] = (defined('UTIL_MAX_REC_PPP')&& UserRequest::$post['PAGING_RPP']>UTIL_MAX_REC_PPP ? UTIL_MAX_REC_PPP:UserRequest::$post['PAGING_RPP']);

        xValidate::integer($msg, 'PAGING_RPP', UserRequest::$post['PAGING_RPP'], CLX_NUM_FIVE, CLX_NUM_HUNDRED, dict('GEN_LBL_RECPERPAGE'));
        $tools = ClxCommon::$db->getRecords('CLX_TOOLS', 'HAS_PREFERENCES=1');
        while ($tool = ClxCommon::$db->getNextRecord($tools)) {
		  $t = getTool($tool['IDCODE']);
		  $t->preferencesCheckForm($msg);
        }
		ClxCommon::$db->freeResult($tools);
        return count($msg) == CLX_NUM_ZERO;
    }

    /**
     * Loads preference information required by form
     * @return array
     */
    function prfLoadRecord() {
        $username = $_SESSION['user_record']['username'];
        $record = array();
		$max=ClxCommon::$db->getField('CLX_US_PREFERENCES', 'PREFVALUE', 'UPPER(USERNAME)=' . singleQuote(uppercase($username)) . ' AND PREFKEY=' . singleQuote('PAGING_RPP'));
        $record['PAGING_RPP'] = (defined('UTIL_MAX_REC_PPP')&& $max>UTIL_MAX_REC_PPP ? UTIL_MAX_REC_PPP:$max);
        $record['PANEL_STATE'] = ClxCommon::$db->getField('CLX_US_PREFERENCES', 'PREFVALUE', 'UPPER(USERNAME)=' . singleQuote(uppercase($username)) . ' AND PREFKEY=' . singleQuote('PANEL_STATE'));
        $record['LSRCH_CI'] = ClxCommon::$db->getField('CLX_US_PREFERENCES', 'PREFVALUE', 'UPPER(USERNAME)=' . singleQuote(uppercase($username)) . ' AND PREFKEY=' . singleQuote('LSRCH_CI'));
        if(is_null($record['LSRCH_CI']) || $record['LSRCH_CI'] === CLX_CHAR_EMPTY_STRING) {
		  $record['LSRCH_CI'] = CLX_CHAR_EMPTY_STRING . CLX_NUM_ONE;
        }
        $tools = ClxCommon::$db->getRecords('CLX_TOOLS', 'HAS_PREFERENCES=1');
        while ($tool = ClxCommon::$db->getNextRecord($tools)) {
		  $t = &getTool($tool['IDCODE']);
		  $t->preferencesLoad($record);
        }
		
		$recod['PAGING_RPP'] = (defined('UTIL_MAX_REC_PPP')&& UserRequest::$post['PAGING_RPP']>UTIL_MAX_REC_PPP ? UTIL_MAX_REC_PPP:UserRequest::$post['PAGING_RPP']);

        return $record;
    }

    /**
     * Saves key
     * @param type $username
     * @param type $prefkey
     * @param type $prefval
     */
    function prfSavePref($username, $prefkey, $prefval) {
        $id = ClxCommon::$db->loadDbRecord($this->tablename, array('USERNAME' => $username, 'PREFKEY' => $prefkey, 'PREFVALUE' => $prefval), false, 'UPPER(USERNAME) = ' . singleQuote(uppercase($username)) . ' AND PREFKEY = ' . singleQuote($prefkey));
        unset($id);
    }

    /**
     * Stores Form Preferences
     */
    function prfSaveRecord() {
        $_SESSION['prefs']['PAGING_RPP'] = (defined('UTIL_MAX_REC_PPP')&& UserRequest::$post['PAGING_RPP']>UTIL_MAX_REC_PPP ? UTIL_MAX_REC_PPP:UserRequest::$post['PAGING_RPP']);

        $_SESSION['prefs']['PANEL_STATE'] = UserRequest::$post['PANEL_STATE'];
        $_SESSION['prefs']['LSRCH_CI'] = UserRequest::$post['LSRCH_CI'];
        $this->prfSavePref($_SESSION['user_record']['username'], 'PAGING_RPP', $_SESSION['prefs']['PAGING_RPP']);
        $this->prfSavePref($_SESSION['user_record']['username'], 'PANEL_STATE', UserRequest::$post['PANEL_STATE']);
        $this->prfSavePref($_SESSION['user_record']['username'], 'LSRCH_CI', UserRequest::$post['LSRCH_CI']);
        $tools = ClxCommon::$db->executeQuery('SELECT IDCODE FROM CLX_TOOLS WHERE HAS_PREFERENCES = 1');
        while ($tool = ClxCommon::$db->getNextRecord($tools)) {
		  $t = &getTool($tool['IDCODE']);
		  $t->preferencesSave();
        }
		ClxCommon::$db->freeResult($tools);
        $this->loadPreferences();
		
    }

    /**
     * Set preference values into session
     */
    function loadPreferences() {
        $_SESSION['prefs'] = array();
		if (!iSSET($_SESSION['user_record']))
			$_SESSION['user_record']=array();
		if (!iSSET($_SESSION['user_record']['username']))
			$_SESSION['user_record']['username']='system';
        $prefs = ClxCommon::$db->getRecords('CLX_US_PREFERENCES', 'USERNAME=' . singleQuote($_SESSION['user_record']['username']));
        while ($pref = ClxCommon::$db->getNextRecord($prefs)) {
		  $_SESSION['prefs'][$pref['PREFKEY']] = $pref['PREFVALUE'];
        }
		ClxCommon::$db->freeResult($prefs);
        if (empty($_SESSION['prefs']['EDIT_BUTTON_LOCATION'])) {
		  $_SESSION['prefs']['EDIT_BUTTON_LOCATION'] = CLX_NUM_ZERO;
        }
        //setting pagination default if not exists
        if (!isset($_SESSION['prefs']['NEXT_ACTION'])) {
		  $_SESSION['prefs']['NEXT_ACTION'] = CLX_NUM_ONE;
        }
        if (empty($_SESSION['prefs']['DATE_FORMAT'])) {
		  $_SESSION['prefs']['DATE_FORMAT'] = 'Y-m-d';
        }

	if (empty($_SESSION['prefs']['PAGING_RPP'])) {
		  $_SESSION['prefs']['PAGING_RPP'] = CLX_NUM_15;
        }
        if (empty($_SESSION['prefs']['PANEL_STATE'])) {
		  $_SESSION['prefs']['PANEL_STATE'] = CLX_INIT_ZERO;
        }
        if (empty($_SESSION['prefs']['ALRT_CHECK_INTVAL'])) {
		  $_SESSION['prefs']['ALRT_CHECK_INTVAL'] = CLX_NUM_60 * CLX_NUM_TWO * CLX_NUM_THOUSAND; //two minutes
        }
        if (!isset($_SESSION['prefs']['LSRCH_CI'])) {
		  $_SESSION['prefs']['LSRCH_CI'] = CLX_CHAR_EMPTY_STRING . CLX_NUM_ONE; // case insensitive search panel enabled
        }
		
		getcompanydefault($_SESSION['user_record']['username']);
    }

    /**
     * Sets the default preference value
     * @param string $username
     */
    function defaultPreferences($username) {
        $records = array(array('USERNAME' => $username, 'PREFKEY' => 'PAGING_RPP', 'PREFVALUE' => '10'),
			         array('USERNAME' => $username, 'PREFKEY' => 'PANEL_STATE', 'PREFVALUE' => '1'),
			         array('USERNAME' => $username, 'PREFKEY' => 'DATE_FORMAT', 'PREFVALUE' => 'Y-m-d'),
			         array('USERNAME' => $username, 'PREFKEY' => 'LSRCH_CI', 'PREFVALUE' => '1')
		  );
        foreach($records as $record){
		  ClxCommon::$db->loadDbRecord($this->tablename, $record, false, 'PREFKEY=' . singleQuote($record['PREFKEY']) . ' AND UPPER(USERNAME) = ' . singleQuote(uppercase($username)));
        }

        //tool preferences
        $tools = ClxCommon::$db->getRecords('CLX_TOOLS', 'HAS_PREFERENCES=1');
        $record = array();
        $record['USERNAME'] = $username;
        while ($tool = ClxCommon::$db->getNextRecord($tools)) {
		  $t = &getTool($tool['IDCODE']);
		  $values = $t->preferencesDefault($record);
		  if (!empty($values)) {
			 foreach ($values as $key => $value) {
			     $record['PREFKEY'] = $key;
			     $record['PREFVALUE'] = $value;
			     ClxCommon::$db->loadDbRecord($this->tablename, $record, false, 'PREFKEY=' . singleQuote($key) . ' AND UPPER(USERNAME) = ' . singleQuote(uppercase($username)));
			 }
		  }
        }
		ClxCommon::$db->freeResult($tools);
    }

}
